@@ -2,8 +2,10 @@
 # License, v. 2.0. If a copy of the MPL was not distributed with this
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
-// gSyncUI handles updating the tools menu
+// gSyncUI handles updating the tools menu and displaying notifications.
 let gSyncUI = {
+  DEFAULT_EOL_URL: "https://www.mozilla.org/firefox/?utm_source=synceol",
+
   _obs: ["weave:service:sync:start",
          "weave:service:sync:delayed",
          "weave:service:quota:remaining",
@@ -12,17 +14,25 @@ let gSyncUI = {
          "weave:service:login:finish",
          "weave:service:logout:finish",
          "weave:service:start-over",
+         "weave:service:start-over:finish",
          "weave:ui:login:error",
          "weave:ui:sync:error",
          "weave:ui:sync:finish",
-         "weave:ui:clear-error"],
+         "weave:ui:clear-error",
+         "weave:eol",
+  ],
 
   _unloaded: false,
 
-  init: function SUI_init() {
+  init: function () {
+    Cu.import("resource://services-common/stringbundle.js");
+
     // Proceed to set up the UI if Sync has already started up.
     // Otherwise we'll do it when Sync is firing up.
-    if (Weave.Status.ready) {
+    let xps = Components.classes["@mozilla.org/weave/service;1"]
+                                .getService(Components.interfaces.nsISupports)
+                                .wrappedJSObject;
+    if (xps.ready) {
       this.initUI();
       return;
     }
@@ -79,30 +89,65 @@ let gSyncUI = {
   _wasDelayed: false,
 
   _needsSetup: function SUI__needsSetup() {
+    // We want to treat "account needs verification" as "needs setup". So
+    // "reach in" to Weave.Status._authManager to check whether we the signed-in
+    // user is verified.
+    // Referencing Weave.Status spins a nested event loop to initialize the
+    // authManager, so this should always return a value directly.
+    // This only applies to fxAccounts-based Sync.
+    if (Weave.Status._authManager._signedInUser) {
+      // If we have a signed in user already, and that user is not verified,
+      // revert to the "needs setup" state.
+      if (!Weave.Status._authManager._signedInUser.verified) {
+        return true;
+      }
+    }
+
     let firstSync = "";
     try {
       firstSync = Services.prefs.getCharPref("services.sync.firstSync");
     } catch (e) { }
+
     return Weave.Status.checkSetup() == Weave.CLIENT_NOT_CONFIGURED ||
            firstSync == "notReady";
   },
 
+  _loginFailed: function () {
+    return Weave.Status.login == Weave.LOGIN_FAILED_LOGIN_REJECTED;
+  },
+
   updateUI: function SUI_updateUI() {
     let needsSetup = this._needsSetup();
-    document.getElementById("sync-setup-state").hidden = !needsSetup;
-    document.getElementById("sync-syncnow-state").hidden = needsSetup;
+    let loginFailed = this._loginFailed();
+
+    // Start off with a clean slate
+    document.getElementById("sync-reauth-state").hidden = true;
+    document.getElementById("sync-setup-state").hidden = true;
+    document.getElementById("sync-syncnow-state").hidden = true;
+
+    if (loginFailed) {
+      document.getElementById("sync-reauth-state").hidden = false;
+    } else if (needsSetup) {
+      document.getElementById("sync-setup-state").hidden = false;
+    } else {
+      document.getElementById("sync-syncnow-state").hidden = false;
+    }
 
     if (!gBrowser)
       return;
 
-    let button = document.getElementById("sync-button");
-    if (!button)
-      return;
+    let syncButton = document.getElementById("sync-button");
+    let panelHorizontalButton = document.getElementById("PanelUI-fxa-status");
+    [syncButton, panelHorizontalButton].forEach(function(button) {
+      if (!button)
+        return;
+      button.removeAttribute("status");
+    });
+
+    if (needsSetup && syncButton)
+      syncButton.removeAttribute("tooltiptext");
 
-    button.removeAttribute("status");
     this._updateLastSyncTime();
-    if (needsSetup)
-      button.removeAttribute("tooltiptext");
   },
 
 
@@ -111,11 +156,12 @@ let gSyncUI = {
     if (!gBrowser)
       return;
 
-    let button = document.getElementById("sync-button");
-    if (!button)
-      return;
-
-    button.setAttribute("status", "active");
+    ["sync-button", "PanelUI-fxa-status"].forEach(function(id) {
+      let button = document.getElementById(id);
+      if (!button)
+        return;
+      button.setAttribute("status", "active");
+    });
   },
 
   onSyncDelay: function SUI_onSyncDelay() {
@@ -140,7 +186,7 @@ let gSyncUI = {
   },
 
   onSetupComplete: function SUI_onSetupComplete() {
-    onLoginFinish();
+    this.onLoginFinish();
   },
 
   onLoginError: function SUI_onLoginError() {
@@ -152,6 +198,11 @@ let gSyncUI = {
       this.updateUI();
       return;
     }
+    // if we are still waiting for the identity manager to initialize, don't show errors
+    if (Weave.Status.login == Weave.LOGIN_FAILED_NOT_READY) {
+      this.updateUI();
+      return;
+    }
 
     let title = this._stringBundle.GetStringFromName("error.login.title");
 
@@ -204,6 +255,41 @@ let gSyncUI = {
     Weave.Notifications.replaceTitle(notification);
   },
 
+  _getAppName: function () {
+    try {
+      let syncStrings = new StringBundle("chrome://browser/locale/sync.properties");
+      return syncStrings.getFormattedString("sync.defaultAccountApplication", [brandName]);
+    } catch (ex) {}
+    let brand = new StringBundle("chrome://branding/locale/brand.properties");
+    return brand.get("brandShortName");
+  },
+
+  onEOLNotice: function (data) {
+    let code = data.code;
+    let kind = (code == "hard-eol") ? "error" : "warning";
+    let url = data.url || gSyncUI.DEFAULT_EOL_URL;
+
+    let title = this._stringBundle.GetStringFromName(kind + ".sync.eol.label");
+    let description = this._stringBundle.formatStringFromName(kind + ".sync.eol.description",
+                                                              [this._getAppName()],
+                                                              1);
+
+    let buttons = [];
+    buttons.push(new Weave.NotificationButton(
+      this._stringBundle.GetStringFromName("sync.eol.learnMore.label"),
+      this._stringBundle.GetStringFromName("sync.eol.learnMore.accesskey"),
+      function() {
+        window.openUILinkIn(url, "tab");
+        return true;
+      }
+    ));
+
+    let priority = (kind == "error") ? Weave.Notifications.PRIORITY_WARNING :
+                                       Weave.Notifications.PRIORITY_INFO;
+    let notification = new Weave.Notification(title, description, null, priority, buttons);
+    Weave.Notifications.replaceTitle(notification);
+  },
+
   openServerStatus: function () {
     let statusURL = Services.prefs.getCharPref("services.sync.statusURL");
     window.openUILinkIn(statusURL, "tab");
@@ -211,7 +297,7 @@ let gSyncUI = {
 
   // Commands
   doSync: function SUI_doSync() {
-    setTimeout(function() Weave.ErrorHandler.syncAndReportErrors(), 0);
+    setTimeout(function() Weave.Service.errorHandler.syncAndReportErrors(), 0);
   },
 
   handleToolbarButton: function SUI_handleStatusbarButton() {
@@ -235,13 +321,26 @@ let gSyncUI = {
    */
 
   openSetup: function SUI_openSetup(wizardType) {
-    let win = Services.wm.getMostRecentWindow("Weave:AccountSetup");
-    if (win)
-      win.focus();
-    else {
-      window.openDialog("chrome://browser/content/sync/setup.xul",
-                        "weaveSetup", "centerscreen,chrome,resizable=no",
-                        wizardType);
+    let xps = Components.classes["@mozilla.org/weave/service;1"]
+                                .getService(Components.interfaces.nsISupports)
+                                .wrappedJSObject;
+    if (xps.fxAccountsEnabled) {
+      fxAccounts.getSignedInUser().then(userData => {
+        if (userData) {
+          this.openPrefs();
+        } else {
+          switchToTabHavingURI("about:accounts", true);
+        }
+      });
+    } else {
+      let win = Services.wm.getMostRecentWindow("Weave:AccountSetup");
+      if (win)
+        win.focus();
+      else {
+        window.openDialog("chrome://browser/content/sync/setup.xul",
+                          "weaveSetup", "centerscreen,chrome,resizable=no",
+                          wizardType);
+      }
     }
   },
 
@@ -271,6 +370,9 @@ let gSyncUI = {
     openPreferences("paneSync");
   },
 
+  openSignInAgainPage: function () {
+    switchToTabHavingURI("about:accounts?action=reauth", true);
+  },
 
   // Helpers
   _updateLastSyncTime: function SUI__updateLastSyncTime() {
@@ -401,6 +503,13 @@ let gSyncUI = {
       return;
     }
 
+    // Unwrap, just like Svc.Obs, but without pulling in that dependency.
+    if (subject && typeof subject == "object" &&
+        ("wrappedJSObject" in subject) &&
+        ("observersModuleSubjectWrapper" in subject.wrappedJSObject)) {
+      subject = subject.wrappedJSObject.object;
+    }
+
     switch (topic) {
       case "weave:service:sync:start":
         this.onActivityStart();
@@ -435,6 +544,9 @@ let gSyncUI = {
       case "weave:service:start-over":
         this.onStartOver();
         break;
+      case "weave:service:start-over:finish":
+        this.updateUI();
+        break;
       case "weave:service:ready":
         this.initUI();
         break;
@@ -444,6 +556,9 @@ let gSyncUI = {
       case "weave:ui:clear-error":
         this.clearError();
         break;
+      case "weave:eol":
+        this.onEOLNotice(subject);
+        break;
     }
   },
 
