@@ -29,6 +29,9 @@ const EVENTS = {
   SOURCE_SHOWN: "Debugger:EditorSourceShown",
   SOURCE_ERROR_SHOWN: "Debugger:EditorSourceErrorShown",
 
+  // When the editor has shown a source and set the line / column position
+  EDITOR_LOCATION_SET: "Debugger:EditorLocationSet",
+
   // When scopes, variables, properties and watch expressions are fetched and
   // displayed in the variables view.
   FETCHED_SCOPES: "Debugger:FetchedScopes",
@@ -86,8 +89,7 @@ const FRAME_TYPE = {
 
 Cu.import("resource://gre/modules/Services.jsm");
 Cu.import("resource://gre/modules/XPCOMUtils.jsm");
-Cu.import("resource://gre/modules/devtools/dbg-client.jsm");
-Cu.import("resource:///modules/devtools/shared/event-emitter.js");
+Cu.import("resource://gre/modules/devtools/event-emitter.js");
 Cu.import("resource:///modules/devtools/SimpleListWidget.jsm");
 Cu.import("resource:///modules/devtools/BreadcrumbsWidget.jsm");
 Cu.import("resource:///modules/devtools/SideMenuWidget.jsm");
@@ -136,13 +138,6 @@ let DebuggerController = {
     this.shutdownDebugger = this.shutdownDebugger.bind(this);
     this._onTabNavigated = this._onTabNavigated.bind(this);
     this._onTabDetached = this._onTabDetached.bind(this);
-
-    // Chrome debugging lives in a different process and needs to handle
-    // debugger startup and shutdown by itself.
-    if (window._isChromeDebugger) {
-      window.addEventListener("DOMContentLoaded", this.startupDebugger, true);
-      window.addEventListener("unload", this.shutdownDebugger, true);
-    }
   },
 
   /**
@@ -156,20 +151,7 @@ let DebuggerController = {
       return this._startup;
     }
 
-    // Chrome debugging lives in a different process and needs to handle
-    // debugger startup by itself.
-    if (window._isChromeDebugger) {
-      window.removeEventListener("DOMContentLoaded", this.startupDebugger, true);
-    }
-
-    return this._startup = DebuggerView.initialize().then(() => {
-      // Chrome debugging needs to initiate the connection by itself.
-      if (window._isChromeDebugger) {
-        return this.connect();
-      } else {
-        return promise.resolve(null); // Done.
-      }
-    });
+    return this._startup = DebuggerView.initialize();
   },
 
   /**
@@ -183,12 +165,6 @@ let DebuggerController = {
       return this._shutdown;
     }
 
-    // Chrome debugging lives in a different process and needs to handle
-    // debugger shutdown by itself.
-    if (window._isChromeDebugger) {
-      window.removeEventListener("unload", this.shutdownDebugger, true);
-    }
-
     return this._shutdown = DebuggerView.destroy().then(() => {
       DebuggerView.destroy();
       this.SourceScripts.disconnect();
@@ -196,22 +172,12 @@ let DebuggerController = {
       this.ThreadState.disconnect();
       this.Tracer.disconnect();
       this.disconnect();
-
-      // Chrome debugging needs to close its parent process on shutdown.
-      if (window._isChromeDebugger) {
-        return this._quitApp();
-      } else {
-        return promise.resolve(null); // Done.
-      }
     });
   },
 
   /**
-   * Initiates remote or chrome debugging based on the current target,
-   * wiring event handlers as necessary.
-   *
-   * In case of a chrome debugger living in a different process, a socket
-   * connection pipe is opened as well.
+   * Initiates remote debugging based on the current target, wiring event
+   * handlers as necessary.
    *
    * @return object
    *         A promise that is resolved when the debugger finishes connecting.
@@ -224,40 +190,27 @@ let DebuggerController = {
     let startedDebugging = promise.defer();
     this._connection = startedDebugging.promise;
 
-    if (!window._isChromeDebugger) {
-      let target = this._target;
-      let { client, form: { chromeDebugger, traceActor }, threadActor } = target;
-      target.on("close", this._onTabDetached);
-      target.on("navigate", this._onTabNavigated);
-      target.on("will-navigate", this._onTabNavigated);
-      this.client = client;
+    let target = this._target;
+    let { client, form: { chromeDebugger, traceActor } } = target;
+    target.on("close", this._onTabDetached);
+    target.on("navigate", this._onTabNavigated);
+    target.on("will-navigate", this._onTabNavigated);
+    this.client = client;
 
-      if (target.chrome) {
-        this._startChromeDebugging(chromeDebugger, startedDebugging.resolve);
-      } else {
-        this._startDebuggingTab(startedDebugging.resolve);
-        const startedTracing = promise.defer();
+    if (target.chrome) {
+      this._startChromeDebugging(chromeDebugger, startedDebugging.resolve);
+    } else {
+      this._startDebuggingTab(startedDebugging.resolve);
+      const startedTracing = promise.defer();
+      if (Prefs.tracerEnabled && traceActor) {
         this._startTracingTab(traceActor, startedTracing.resolve);
-
-        return promise.all([startedDebugging.promise, startedTracing.promise]);
+      } else {
+        startedTracing.resolve();
       }
 
-      return startedDebugging.promise;
+      return promise.all([startedDebugging.promise, startedTracing.promise]);
     }
 
-    // Chrome debugging needs to make its own connection to the debuggee.
-    let transport = debuggerSocketConnect(
-      Prefs.chromeDebuggingHost, Prefs.chromeDebuggingPort);
-
-    let client = this.client = new DebuggerClient(transport);
-    client.addListener("tabNavigated", this._onTabNavigated);
-    client.addListener("tabDetached", this._onTabDetached);
-    client.connect(() => {
-      client.listTabs(aResponse => {
-        this._startChromeDebugging(aResponse.chromeDebugger, startedDebugging.resolve);
-      });
-    });
-
     return startedDebugging.promise;
   },
 
@@ -270,15 +223,6 @@ let DebuggerController = {
       return;
     }
 
-    // When debugging local or a remote instance, the connection is closed by
-    // the RemoteTarget. Chrome debugging needs to specifically close its own
-    // connection to the debuggee.
-    if (window._isChromeDebugger) {
-      this.client.removeListener("tabNavigated", this._onTabNavigated);
-      this.client.removeListener("tabDetached", this._onTabDetached);
-      this.client.close();
-    }
-
     this._connection = null;
     this.client = null;
     this.activeThread = null;
@@ -443,33 +387,6 @@ let DebuggerController = {
     });
   },
 
-  /**
-   * Attempts to quit the current process if allowed.
-   *
-   * @return object
-   *         A promise that is resolved if the app will quit successfully.
-   */
-  _quitApp: function() {
-    let deferred = promise.defer();
-
-    // Quitting the app is synchronous. Give the returned promise consumers
-    // a chance to do their thing before killing the process.
-    Services.tm.currentThread.dispatch({ run: () => {
-      let quit = Cc["@mozilla.org/supports-PRBool;1"].createInstance(Ci.nsISupportsPRBool);
-      Services.obs.notifyObservers(quit, "quit-application-requested", null);
-
-      // Somebody canceled our quit request.
-      if (quit.data) {
-        deferred.reject(quit.data);
-      } else {
-        deferred.resolve(quit.data);
-        Services.startup.quit(Ci.nsIAppStartup.eForceQuit);
-      }
-    }}, 0);
-
-    return deferred.promise;
-  },
-
   _startup: null,
   _shutdown: null,
   _connection: null,
@@ -790,8 +707,13 @@ StackFrames.prototype = {
    * Handler for the debugger's prettyprintchange notification.
    */
   _onPrettyPrintChange: function() {
+    // Makes sure the selected source remains selected
+    // after the fillFrames is called.
+    const source = DebuggerView.Sources.selectedValue;
     if (this.activeThread.state == "paused") {
-      this.activeThread.fillFrames(CALL_STACK_PAGE_SIZE);
+      this.activeThread.fillFrames(
+         CALL_STACK_PAGE_SIZE,
+         () => DebuggerView.Sources.selectedValue = source);
     }
   },
 
@@ -1188,6 +1110,12 @@ SourceScripts.prototype = {
       return;
     }
 
+    if (aResponse.sources.length === 0) {
+      DebuggerView.Sources.emptyText = L10N.getStr("noSourcesText");
+      window.emit(EVENTS.SOURCES_ADDED);
+      return;
+    }
+
     // Add all the sources in the debugger view sources container.
     for (let source of aResponse.sources) {
       // Ignore bogus scripts, e.g. generated from 'clientEvaluate' packets.
@@ -1294,7 +1222,7 @@ SourceScripts.prototype = {
     deferred.promise.pretty = wantPretty;
     this._cache.set(aSource.url, deferred.promise);
 
-    const afterToggle = ({ error, message, source: text }) => {
+    const afterToggle = ({ error, message, source: text, contentType }) => {
       if (error) {
         // Revert the rejected promise from the cache, so that the original
         // source's text may be shown when the source is selected.
@@ -1302,7 +1230,7 @@ SourceScripts.prototype = {
         deferred.reject([aSource, message || error]);
         return;
       }
-      deferred.resolve([aSource, text]);
+      deferred.resolve([aSource, text, contentType]);
     };
 
     if (wantPretty) {
@@ -1354,14 +1282,15 @@ SourceScripts.prototype = {
     }
 
     // Get the source text from the active thread.
-    this.activeThread.source(aSource).source(({ error, message, source: text }) => {
+    this.activeThread.source(aSource)
+        .source(({ error, message, source: text, contentType }) => {
       if (aOnTimeout) {
         window.clearTimeout(fetchTimeout);
       }
       if (error) {
         deferred.reject([aSource, message || error]);
       } else {
-        deferred.resolve([aSource, text]);
+        deferred.resolve([aSource, text, contentType]);
       }
     });
 
@@ -1400,13 +1329,13 @@ SourceScripts.prototype = {
     }
 
     /* Called if fetching a source finishes successfully. */
-    function onFetch([aSource, aText]) {
+    function onFetch([aSource, aText, aContentType]) {
       // If fetching the source has previously timed out, discard it this time.
       if (!pending.has(aSource.url)) {
         return;
       }
       pending.delete(aSource.url);
-      fetched.push([aSource.url, aText]);
+      fetched.push([aSource.url, aText, aContentType]);
       maybeFinish();
     }
 
@@ -1489,7 +1418,7 @@ Tracer.prototype = {
     ], this._trace, (aResponse) => {
       const { error } = aResponse;
       if (error) {
-        DevToolsUtils.reportException(error);
+        DevToolsUtils.reportException("Tracer.prototype.startTracing", error);
         this._trace = null;
       }
 
@@ -1507,7 +1436,7 @@ Tracer.prototype = {
     this.traceClient.stopTrace(this._trace, aResponse => {
       const { error } = aResponse;
       if (error) {
-        DevToolsUtils.reportException(error);
+        DevToolsUtils.reportException("Tracer.prototype.stopTracing", error);
       }
 
       this._trace = null;
@@ -1690,11 +1619,10 @@ EventListeners.prototype = {
           if (aResponse.error) {
             const msg = "Error getting function definition site: " + aResponse.message;
             DevToolsUtils.reportException("scheduleEventListenersFetch", msg);
-            deferred.reject(msg);
-            return;
+          } else {
+            aListener.function.url = aResponse.url;
           }
 
-          aListener.function.url = aResponse.url;
           deferred.resolve(aListener);
         });
 
@@ -2115,16 +2043,6 @@ Breakpoints.prototype = {
     }
   },
 
-  /**
-   * Gets all Promises for the BreakpointActor client objects that are
-   * either enabled (added to the server) or disabled (removed from the server,
-   * but for which some details are preserved).
-   */
-  get _addedOrDisabled() {
-    for (let [, value] of this._added) yield value;
-    for (let [, value] of this._disabled) yield value;
-  },
-
   /**
    * Get a Promise for the BreakpointActor client object which is already added
    * or currently being added at the given location.
@@ -2167,6 +2085,18 @@ Breakpoints.prototype = {
   }
 };
 
+/**
+ * Gets all Promises for the BreakpointActor client objects that are
+ * either enabled (added to the server) or disabled (removed from the server,
+ * but for which some details are preserved).
+ */
+Object.defineProperty(Breakpoints.prototype, "_addedOrDisabled", {
+  get: function* () {
+    yield* this._added.values();
+    yield* this._disabled.values();
+  }
+});
+
 /**
  * Localization convenience methods.
  */
@@ -2176,8 +2106,6 @@ let L10N = new ViewHelpers.L10N(DBG_STRINGS_URI);
  * Shortcuts for accessing various debugger preferences.
  */
 let Prefs = new ViewHelpers.Prefs("devtools", {
-  chromeDebuggingHost: ["Char", "debugger.chrome-debugging-host"],
-  chromeDebuggingPort: ["Int", "debugger.chrome-debugging-port"],
   sourcesWidth: ["Int", "debugger.ui.panes-sources-width"],
   instrumentsWidth: ["Int", "debugger.ui.panes-instruments-width"],
   panesVisibleOnStartup: ["Bool", "debugger.ui.panes-visible-on-startup"],
@@ -2193,15 +2121,6 @@ let Prefs = new ViewHelpers.Prefs("devtools", {
   editorTabSize: ["Int", "editor.tabsize"]
 });
 
-/**
- * Returns true if this is a chrome debugger instance.
- * @return boolean
- */
-XPCOMUtils.defineLazyGetter(window, "_isChromeDebugger", function() {
-  // We're inside a single top level XUL window in a different process.
-  return !(window.frameElement instanceof XULElement);
-});
-
 /**
  * Convenient way of emitting events from the panel window.
  */
