@@ -10,8 +10,8 @@ const MIN_ZOOM = 0.5;
 const MAX_ZOOM = 2;
 
 let {Cc, Ci, Cu} = require("chrome");
-let promise = require("sdk/core/promise");
-let EventEmitter = require("devtools/shared/event-emitter");
+let {Promise: promise} = require("resource://gre/modules/Promise.jsm");
+let EventEmitter = require("devtools/toolkit/event-emitter");
 let Telemetry = require("devtools/shared/telemetry");
 let HUDService = require("devtools/webconsole/hudservice");
 
@@ -42,8 +42,6 @@ loader.lazyGetter(this, "toolboxStrings", () => {
 });
 
 loader.lazyGetter(this, "Requisition", () => {
-  let {require} = Cu.import("resource://gre/modules/devtools/Require.jsm", {});
-  Cu.import("resource://gre/modules/devtools/gcli.jsm", {});
   return require("gcli/cli").Requisition;
 });
 
@@ -75,6 +73,8 @@ function Toolbox(target, selectedTool, hostType, hostOptions) {
   this._splitConsoleOnKeypress = this._splitConsoleOnKeypress.bind(this)
   this.destroy = this.destroy.bind(this);
   this.highlighterUtils = new ToolboxHighlighterUtils(this);
+  this._highlighterReady = this._highlighterReady.bind(this);
+  this._highlighterHidden = this._highlighterHidden.bind(this);
 
   this._target.on("close", this.destroy);
 
@@ -255,21 +255,20 @@ Toolbox.prototype = {
         this._addZoomKeys();
         this._loadInitialZoom();
 
-        // Load the toolbox-level actor fronts and utilities now
-        this._target.makeRemote().then(() => {
-          this._telemetry.toolOpened("toolbox");
+        this._telemetry.toolOpened("toolbox");
 
-          this.selectTool(this._defaultToolId).then(panel => {
-            this.emit("ready");
-            deferred.resolve();
-          });
+        this.selectTool(this._defaultToolId).then(panel => {
+          this.emit("ready");
+          deferred.resolve();
         });
       };
 
-      iframe.setAttribute("src", this._URL);
-
-      let domHelper = new DOMHelpers(iframe.contentWindow);
-      domHelper.onceDOMReady(domReady);
+      // Load the toolbox-level actor fronts and utilities now
+      this._target.makeRemote().then(() => {
+        iframe.setAttribute("src", this._URL);
+        let domHelper = new DOMHelpers(iframe.contentWindow);
+        domHelper.onceDOMReady(domReady);
+      });
 
       return deferred.promise;
     });
@@ -467,8 +466,10 @@ Toolbox.prototype = {
   fireCustomKey: function(toolId) {
     let toolDefinition = gDevTools.getToolDefinition(toolId);
 
-    if (toolDefinition.onkey && this.currentToolId === toolId) {
-      toolDefinition.onkey(this.getCurrentPanel());
+    if (toolDefinition.onkey &&
+        ((this.currentToolId === toolId) ||
+          (toolId == "webconsole" && this.splitConsole))) {
+      toolDefinition.onkey(this.getCurrentPanel(), this);
     }
   },
 
@@ -537,12 +538,13 @@ Toolbox.prototype = {
     }
 
     let spec = CommandUtils.getCommandbarSpec("devtools.toolbox.toolbarSpec");
-    let env = CommandUtils.createEnvironment(this.target.tab.ownerDocument,
-                                             this.target.window.document);
-    let req = new Requisition(env);
-    let buttons = CommandUtils.createButtons(spec, this._target, this.doc, req);
+    let environment = CommandUtils.createEnvironment(this, '_target');
+    this._requisition = new Requisition({ environment: environment });
+    let buttons = CommandUtils.createButtons(spec, this._target,
+                                             this.doc, this._requisition);
     let container = this.doc.getElementById("toolbox-buttons");
     buttons.forEach(container.appendChild.bind(container));
+    this.setToolboxButtonsVisibility();
   },
 
   /**
@@ -562,6 +564,57 @@ Toolbox.prototype = {
     this._pickerButton.addEventListener("command", this._togglePicker, false);
   },
 
+  /**
+   * Return all toolbox buttons (command buttons, plus any others that were
+   * added manually).
+   */
+  get toolboxButtons() {
+    // White-list buttons that can be toggled to prevent adding prefs for
+    // addons that have manually inserted toolbarbuttons into DOM.
+    return [
+      "command-button-pick",
+      "command-button-splitconsole",
+      "command-button-responsive",
+      "command-button-paintflashing",
+      "command-button-tilt",
+      "command-button-scratchpad"
+    ].map(id => {
+      let button = this.doc.getElementById(id);
+      // Some buttons may not exist inside of Browser Toolbox
+      if (!button) {
+        return false;
+      }
+      return {
+        id: id,
+        button: button,
+        label: button.getAttribute("tooltiptext"),
+        visibilityswitch: "devtools." + id + ".enabled"
+      }
+    }).filter(button=>button);
+  },
+
+  /**
+   * Ensure the visibility of each toolbox button matches the
+   * preference value.  Simply hide buttons that are preffed off.
+   */
+  setToolboxButtonsVisibility: function() {
+    this.toolboxButtons.forEach(buttonSpec => {
+      let {visibilityswitch, id, button}=buttonSpec;
+      let on = true;
+      try {
+        on = Services.prefs.getBoolPref(visibilityswitch);
+      } catch (ex) { }
+
+      if (button) {
+        if (on) {
+          button.removeAttribute("hidden");
+        } else {
+          button.setAttribute("hidden", "true");
+        }
+      }
+    });
+  },
+
   /**
    * Build a tab for one tool definition and add to the toolbox
    *
@@ -713,7 +766,7 @@ Toolbox.prototype = {
         this.emit(id + "-ready", panel);
         gDevTools.emit(id + "-ready", this, panel);
         deferred.resolve(panel);
-      });
+      }, console.error);
     };
 
     iframe.setAttribute("src", definition.url);
@@ -820,6 +873,14 @@ Toolbox.prototype = {
     iframe.focus();
   },
 
+  /**
+   * Focus split console's input line
+   */
+  focusConsoleInput: function() {
+    let hud = this.getPanel("webconsole").hud;
+    hud.jsterm.inputNode.focus();
+  },
+
   /**
    * Toggles the split state of the webconsole.  If the webconsole panel
    * is already selected, then this command is ignored.
@@ -835,7 +896,7 @@ Toolbox.prototype = {
 
       if (this._splitConsole) {
         this.loadTool("webconsole").then(() => {
-          this.focusTool("webconsole");
+          this.focusConsoleInput();
         });
       }
     }
@@ -1033,27 +1094,23 @@ Toolbox.prototype = {
    * Returns a promise that resolves when the fronts are initialized
    */
   initInspector: function() {
-    let deferred = promise.defer();
-
-    if (!this._inspector) {
-      this._inspector = InspectorFront(this._target.client, this._target.form);
-      this._inspector.getWalker().then(walker => {
-        this._walker = walker;
+    if (!this._initInspector) {
+      this._initInspector = Task.spawn(function*() {
+        this._inspector = InspectorFront(this._target.client, this._target.form);
+        this._walker = yield this._inspector.getWalker();
         this._selection = new Selection(this._walker);
-        if (this.highlighterUtils.isRemoteHighlightable) {
-          this._inspector.getHighlighter().then(highlighter => {
-            this._highlighter = highlighter;
-            deferred.resolve();
-          });
-        } else {
-          deferred.resolve();
-        }
-      });
-    } else {
-      deferred.resolve();
-    }
 
-    return deferred.promise;
+        if (this.highlighterUtils.isRemoteHighlightable) {
+          let autohide = !gDevTools.testing;
+
+          this.walker.on("highlighter-ready", this._highlighterReady);
+          this.walker.on("highlighter-hide", this._highlighterHidden);
+
+          this._highlighter = yield this._inspector.getHighlighter(autohide);
+        }
+      }.bind(this));
+    }
+    return this._initInspector;
   },
 
   /**
@@ -1061,6 +1118,10 @@ Toolbox.prototype = {
    * Returns a promise that resolves when the fronts are destroyed
    */
   destroyInspector: function() {
+    if (this._destroying) {
+      return this._destroying;
+    }
+
     if (!this._inspector) {
       return promise.resolve();
     }
@@ -1076,6 +1137,11 @@ Toolbox.prototype = {
           this._selection.destroy();
         }
 
+        if (this.walker) {
+          this.walker.off("highlighter-ready", this._highlighterReady);
+          this.walker.off("highlighter-hide", this._highlighterHidden);
+        }
+
         this._inspector = null;
         this._highlighter = null;
         this._selection = null;
@@ -1086,7 +1152,9 @@ Toolbox.prototype = {
     // Releasing the walker (if it has been created)
     // This can fail, but in any case, we want to continue destroying the
     // inspector/highlighter/selection
-    let walker = this._walker ? this._walker.release() : promise.resolve();
+    let walker = (this._destroying = this._walker) ?
+                 this._walker.release() :
+                 promise.resolve();
     return walker.then(outstanding, outstanding);
   },
 
@@ -1151,6 +1219,9 @@ Toolbox.prototype = {
     // Remove the host UI
     outstanding.push(this.destroyHost());
 
+    if (this.target.isLocalTab) {
+      this._requisition.destroy();
+    }
     this._telemetry.destroy();
 
     return this._destroyer = promise.all(outstanding).then(() => {
@@ -1172,7 +1243,15 @@ Toolbox.prototype = {
       this._host = null;
       this._toolPanels.clear();
     }).then(null, console.error);
-  }
+  },
+
+  _highlighterReady: function() {
+    this.emit("highlighter-ready");
+  },
+
+  _highlighterHidden: function() {
+    this.emit("highlighter-hide");
+  },
 };
 
 /**
@@ -1232,6 +1311,7 @@ ToolboxHighlighterUtils.prototype = {
     let deferred = promise.defer();
 
     let done = () => {
+      this._isPicking = true;
       this.toolbox.emit("picker-started");
       this.toolbox.on("select", this.stopPicker);
       deferred.resolve();
@@ -1241,20 +1321,20 @@ ToolboxHighlighterUtils.prototype = {
       this.toolbox.initInspector(),
       this.toolbox.selectTool("inspector")
     ]).then(() => {
-      this._isPicking = true;
       this.toolbox._pickerButton.setAttribute("checked", "true");
 
       if (this.isRemoteHighlightable) {
-        this.toolbox.highlighter.pick().then(done);
-
         this.toolbox.walker.on("picker-node-hovered", this._onPickerNodeHovered);
         this.toolbox.walker.on("picker-node-picked", this._onPickerNodePicked);
+
+        this.toolbox.highlighter.pick().then(done);
       } else {
-        this.toolbox.walker.pick().then(node => {
-          this.toolbox.selection.setNodeFront(node, "picker-node-picked");
-          this.stopPicker();
+        return this.toolbox.walker.pick().then(node => {
+          this.toolbox.selection.setNodeFront(node, "picker-node-picked").then(() => {
+            this.stopPicker();
+            done();
+          });
         });
-        done();
       }
     });
 
@@ -1352,16 +1432,21 @@ ToolboxHighlighterUtils.prototype = {
    * Hide the highlighter.
    * @return a promise that resolves when the highlighter is hidden
    */
-  unhighlight: function() {
-    if (this.isRemoteHighlightable) {
+  unhighlight: function(forceHide=false) {
+    let unhighlightPromise;
+    forceHide = forceHide || !gDevTools.testing;
+
+    if (forceHide && this.isRemoteHighlightable && this.toolbox.highlighter) {
       // If the remote highlighter exists on the target, use it
-      return this.toolbox.initInspector().then(() => {
-        return this.toolbox.highlighter.hideBoxModel();
-      });
+      unhighlightPromise = this.toolbox.highlighter.hideBoxModel();
     } else {
       // If not, no need to unhighlight as the older highlight method uses a
       // setTimeout to hide itself
-      return promise.resolve();
+      unhighlightPromise = promise.resolve();
     }
+
+    return unhighlightPromise.then(() => {
+      this.toolbox.emit("node-unhighlight");
+    });
   }
 };
