@@ -1,4 +1,4 @@
-/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
+/* -*- Mode: Javascript; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /* vim: set ft=javascript ts=2 et sw=2 tw=80: */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -8,14 +8,12 @@ const {Cc, Ci, Cu, Cr} = require("chrome");
 
 Cu.import("resource://gre/modules/Services.jsm");
 
-let Promise = require("sdk/core/promise");
-let EventEmitter = require("devtools/shared/event-emitter");
+let promise = require("sdk/core/promise");
+let EventEmitter = require("devtools/toolkit/event-emitter");
 let {CssLogic} = require("devtools/styleinspector/css-logic");
 
 loader.lazyGetter(this, "MarkupView", () => require("devtools/markupview/markup-view").MarkupView);
-loader.lazyGetter(this, "Selection", () => require ("devtools/inspector/selection").Selection);
 loader.lazyGetter(this, "HTMLBreadcrumbs", () => require("devtools/inspector/breadcrumbs").HTMLBreadcrumbs);
-loader.lazyGetter(this, "Highlighter", () => require("devtools/inspector/highlighter").Highlighter);
 loader.lazyGetter(this, "ToolSidebar", () => require("devtools/framework/sidebar").ToolSidebar);
 loader.lazyGetter(this, "SelectorSearch", () => require("devtools/inspector/selector-search").SelectorSearch);
 
@@ -23,9 +21,35 @@ const LAYOUT_CHANGE_TIMER = 250;
 
 /**
  * Represents an open instance of the Inspector for a tab.
- * The inspector controls the highlighter, the breadcrumbs,
- * the markup view, and the sidebar (computed view, rule view
- * and layout view).
+ * The inspector controls the breadcrumbs, the markup view, and the sidebar
+ * (computed view, rule view, font view and layout view).
+ *
+ * Events:
+ * - ready
+ *      Fired when the inspector panel is opened for the first time and ready to
+ *      use
+ * - new-root
+ *      Fired after a new root (navigation to a new page) event was fired by
+ *      the walker, and taken into account by the inspector (after the markup
+ *      view has been reloaded)
+ * - markuploaded
+ *      Fired when the markup-view frame has loaded
+ * - layout-change
+ *      Fired when the layout of the inspector changes
+ * - breadcrumbs-updated
+ *      Fired when the breadcrumb widget updates to a new node
+ * - layoutview-updated
+ *      Fired when the layoutview (box model) updates to a new node
+ * - markupmutation
+ *      Fired after markup mutations have been processed by the markup-view
+ * - computed-view-refreshed
+ *      Fired when the computed rules view updates to a new node
+ * - computed-view-property-expanded
+ *      Fired when a property is expanded in the computed rules view
+ * - computed-view-property-collapsed
+ *      Fired when a property is collapsed in the computed rules view
+ * - rule-view-refreshed
+ *      Fired when the rule view updates to a new node
  */
 function InspectorPanel(iframeWindow, toolbox) {
   this._toolbox = toolbox;
@@ -33,6 +57,10 @@ function InspectorPanel(iframeWindow, toolbox) {
   this.panelDoc = iframeWindow.document;
   this.panelWin = iframeWindow;
   this.panelWin.inspector = this;
+  this._inspector = null;
+
+  this._onBeforeNavigate = this._onBeforeNavigate.bind(this);
+  this._target.on("will-navigate", this._onBeforeNavigate);
 
   EventEmitter.decorate(this);
 }
@@ -44,10 +72,44 @@ InspectorPanel.prototype = {
    * open is effectively an asynchronous constructor
    */
   open: function InspectorPanel_open() {
-    let deferred = Promise.defer();
+    return this.target.makeRemote().then(() => {
+      return this._getPageStyle();
+    }).then(() => {
+      return this._getDefaultNodeForSelection();
+    }).then(defaultSelection => {
+      return this._deferredOpen(defaultSelection);
+    }).then(null, console.error);
+  },
 
-    this.onNavigatedAway = this.onNavigatedAway.bind(this);
-    this.target.on("navigate", this.onNavigatedAway);
+  get toolbox() {
+    return this._toolbox;
+  },
+
+  get inspector() {
+    return this._toolbox.inspector;
+  },
+
+  get walker() {
+    return this._toolbox.walker;
+  },
+
+  get selection() {
+    return this._toolbox.selection;
+  },
+
+  get isOuterHTMLEditable() {
+    return this._target.client.traits.editOuterHTML;
+  },
+
+  get hasUrlToImageDataResolver() {
+    return this._target.client.traits.urlToImageDataResolver;
+  },
+
+  _deferredOpen: function(defaultSelection) {
+    let deferred = promise.defer();
+
+    this.onNewRoot = this.onNewRoot.bind(this);
+    this.walker.on("new-root", this.onNewRoot);
 
     this.nodemenu = this.panelDoc.getElementById("inspector-node-popup");
     this.lastNodemenuItem = this.nodemenu.lastChild;
@@ -56,14 +118,12 @@ InspectorPanel.prototype = {
     this.nodemenu.addEventListener("popupshowing", this._setupNodeMenu, true);
     this.nodemenu.addEventListener("popuphiding", this._resetNodeMenu, true);
 
-    // Create an empty selection
-    this._selection = new Selection();
     this.onNewSelection = this.onNewSelection.bind(this);
-    this.selection.on("new-node", this.onNewSelection);
+    this.selection.on("new-node-front", this.onNewSelection);
     this.onBeforeNewSelection = this.onBeforeNewSelection.bind(this);
-    this.selection.on("before-new-node", this.onBeforeNewSelection);
+    this.selection.on("before-new-node-front", this.onBeforeNewSelection);
     this.onDetached = this.onDetached.bind(this);
-    this.selection.on("detached", this.onDetached);
+    this.selection.on("detached-front", this.onDetached);
 
     this.breadcrumbs = new HTMLBreadcrumbs(this);
 
@@ -72,20 +132,6 @@ InspectorPanel.prototype = {
       this.scheduleLayoutChange = this.scheduleLayoutChange.bind(this);
       this.browser.addEventListener("resize", this.scheduleLayoutChange, true);
 
-      this.highlighter = new Highlighter(this.target, this, this._toolbox);
-      let button = this.panelDoc.getElementById("inspector-inspect-toolbutton");
-      button.hidden = false;
-      this.onLockStateChanged = function() {
-        if (this.highlighter.locked) {
-          button.removeAttribute("checked");
-          this._toolbox.raise();
-        } else {
-          button.setAttribute("checked", "true");
-        }
-      }.bind(this);
-      this.highlighter.on("locked", this.onLockStateChanged);
-      this.highlighter.on("unlocked", this.onLockStateChanged);
-
       // Show a warning when the debugger is paused.
       // We show the warning only when the inspector
       // is selected.
@@ -121,17 +167,9 @@ InspectorPanel.prototype = {
       this.isReady = true;
 
       // All the components are initialized. Let's select a node.
-      if (this.target.isLocalTab) {
-        let root = this.browser.contentDocument.documentElement;
-        this._selection.setNode(root);
-      } else if (this.target.window) {
-        let root = this.target.window.document.documentElement;
-        this._selection.setNode(root);
-      }
+      this.selection.setNodeFront(defaultSelection, "inspector-open");
 
-      if (this.highlighter) {
-        this.highlighter.unlock();
-      }
+      this.markup.expandNode(this.selection.nodeFront);
 
       this.emit("ready");
       deferred.resolve(this);
@@ -143,11 +181,51 @@ InspectorPanel.prototype = {
     return deferred.promise;
   },
 
+  _onBeforeNavigate: function() {
+    this._defaultNode = null;
+    this.selection.setNodeFront(null);
+    this._destroyMarkup();
+    this.isDirty = false;
+  },
+
+  _getPageStyle: function() {
+    return this._toolbox.inspector.getPageStyle().then(pageStyle => {
+      this.pageStyle = pageStyle;
+    });
+  },
+
   /**
-   * Selection object (read only)
+   * Return a promise that will resolve to the default node for selection.
    */
-  get selection() {
-    return this._selection;
+  _getDefaultNodeForSelection: function() {
+    if (this._defaultNode) {
+      return this._defaultNode;
+    }
+    let walker = this.walker;
+    let rootNode = null;
+
+    // If available, set either the previously selected node or the body
+    // as default selected, else set documentElement
+    return walker.getRootNode().then(aRootNode => {
+      rootNode = aRootNode;
+      return walker.querySelector(rootNode, this.selectionCssSelector);
+    }).then(front => {
+      if (front) {
+        return front;
+      }
+      return walker.querySelector(rootNode, "body");
+    }).then(front => {
+      if (front) {
+        return front;
+      }
+      return this.walker.documentElement(this.walker.rootNode);
+    }).then(node => {
+      if (walker !== this.walker) {
+        promise.reject(null);
+      }
+      this._defaultNode = node;
+      return node;
+    });
   },
 
   /**
@@ -184,24 +262,13 @@ InspectorPanel.prototype = {
    * Hooks the searchbar to show result and auto completion suggestions.
    */
   setupSearchBox: function InspectorPanel_setupSearchBox() {
-    let searchDoc;
-    if (this.target.isLocalTab) {
-      searchDoc = this.browser.contentDocument;
-    } else if (this.target.window) {
-      searchDoc = this.target.window.document;
-    } else {
-      return;
-    }
     // Initiate the selectors search object.
-    let setNodeFunction = function(node) {
-      this.selection.setNode(node, "selectorsearch");
-    }.bind(this);
     if (this.searchSuggestions) {
       this.searchSuggestions.destroy();
       this.searchSuggestions = null;
     }
     this.searchBox = this.panelDoc.getElementById("inspector-searchbox");
-    this.searchSuggestions = new SelectorSearch(searchDoc, this.searchBox, setNodeFunction);
+    this.searchSuggestions = new SelectorSearch(this, this.searchBox);
   },
 
   /**
@@ -209,7 +276,7 @@ InspectorPanel.prototype = {
    */
   setupSidebar: function InspectorPanel_setupSidebar() {
     let tabbox = this.panelDoc.querySelector("#inspector-sidebar");
-    this.sidebar = new ToolSidebar(tabbox, this);
+    this.sidebar = new ToolSidebar(tabbox, this, "inspector");
 
     let defaultTab = Services.prefs.getCharPref("devtools.inspector.activeSidebar");
 
@@ -218,7 +285,6 @@ InspectorPanel.prototype = {
     }.bind(this);
 
     this.sidebar.on("select", this._setDefaultSidebar);
-    this.toggleHighlighter = this.toggleHighlighter.bind(this);
 
     this.sidebar.addTab("ruleview",
                         "chrome://browser/content/devtools/cssruleview.xhtml",
@@ -228,7 +294,7 @@ InspectorPanel.prototype = {
                         "chrome://browser/content/devtools/computedview.xhtml",
                         "computedview" == defaultTab);
 
-    if (Services.prefs.getBoolPref("devtools.fontinspector.enabled")) {
+    if (Services.prefs.getBoolPref("devtools.fontinspector.enabled") && !this.target.isRemote) {
       this.sidebar.addTab("fontinspector",
                           "chrome://browser/content/devtools/fontinspector/font-inspector.xhtml",
                           "fontinspector" == defaultTab);
@@ -239,48 +305,142 @@ InspectorPanel.prototype = {
                         "layoutview" == defaultTab);
 
     let ruleViewTab = this.sidebar.getTab("ruleview");
-    ruleViewTab.addEventListener("mouseover", this.toggleHighlighter, false);
-    ruleViewTab.addEventListener("mouseout", this.toggleHighlighter, false);
 
     this.sidebar.show();
   },
 
   /**
-   * Reset the inspector on navigate away.
+   * Reset the inspector on new root mutation.
    */
-  onNavigatedAway: function InspectorPanel_onNavigatedAway(event, payload) {
-    let newWindow = payload._navPayload || payload;
-    this.selection.setNode(null);
+  onNewRoot: function InspectorPanel_onNewRoot() {
+    this._defaultNode = null;
+    this.selection.setNodeFront(null);
     this._destroyMarkup();
     this.isDirty = false;
-    let self = this;
 
-    function onDOMReady() {
-      newWindow.removeEventListener("DOMContentLoaded", onDOMReady, true);
+    this._getDefaultNodeForSelection().then(defaultNode => {
+      this.selection.setNodeFront(defaultNode, "navigateaway");
 
-      if (self._destroyed) {
-        return;
-      }
+      this._initMarkup();
+      this.once("markuploaded", () => {
+        if (!this.markup) {
+          return;
+        }
+        this.markup.expandNode(this.selection.nodeFront);
+        this.setupSearchBox();
+        this.emit("new-root");
+      });
+    });
+  },
 
-      if (!self.selection.node) {
-        self.selection.setNode(newWindow.document.documentElement, "navigateaway");
-      }
-      self._initMarkup();
-      self.setupSearchBox();
-    }
+  _selectionCssSelector: null,
 
-    if (newWindow.document.readyState == "loading") {
-      newWindow.addEventListener("DOMContentLoaded", onDOMReady, true);
+  /**
+   * Set the currently selected node unique css selector.
+   * Will store the current target url along with it to allow pre-selection at
+   * reload
+   */
+  set selectionCssSelector(cssSelector) {
+    this._selectionCssSelector = {
+      selector: cssSelector,
+      url: this._target.url
+    };
+  },
+
+  /**
+   * Get the current selection unique css selector if any, that is, if a node
+   * is actually selected and that node has been selected while on the same url
+   */
+  get selectionCssSelector() {
+    if (this._selectionCssSelector &&
+        this._selectionCssSelector.url === this._target.url) {
+      return this._selectionCssSelector.selector;
     } else {
-      onDOMReady();
+      return null;
     }
   },
 
   /**
    * When a new node is selected.
    */
-  onNewSelection: function InspectorPanel_onNewSelection() {
+  onNewSelection: function InspectorPanel_onNewSelection(event, value, reason) {
+    if (reason === "selection-destroy") {
+      return;
+    }
+
     this.cancelLayoutChange();
+
+    // Wait for all the known tools to finish updating and then let the
+    // client know.
+    let selection = this.selection.nodeFront;
+
+    // On any new selection made by the user, store the unique css selector
+    // of the selected node so it can be restored after reload of the same page
+    if (reason !== "navigateaway" &&
+        this.selection.node &&
+        this.selection.isElementNode()) {
+      this.selectionCssSelector = CssLogic.findCssSelector(this.selection.node);
+    }
+
+    let selfUpdate = this.updating("inspector-panel");
+    Services.tm.mainThread.dispatch(() => {
+      try {
+        selfUpdate(selection);
+      } catch(ex) {
+        console.error(ex);
+      }
+    }, Ci.nsIThread.DISPATCH_NORMAL);
+  },
+
+  /**
+   * Delay the "inspector-updated" notification while a tool
+   * is updating itself.  Returns a function that must be
+   * invoked when the tool is done updating with the node
+   * that the tool is viewing.
+   */
+  updating: function(name) {
+    if (this._updateProgress && this._updateProgress.node != this.selection.nodeFront) {
+      this.cancelUpdate();
+    }
+
+    if (!this._updateProgress) {
+      // Start an update in progress.
+      var self = this;
+      this._updateProgress = {
+        node: this.selection.nodeFront,
+        outstanding: new Set(),
+        checkDone: function() {
+          if (this !== self._updateProgress) {
+            return;
+          }
+          if (this.node !== self.selection.nodeFront) {
+            self.cancelUpdate();
+            return;
+          }
+          if (this.outstanding.size !== 0) {
+            return;
+          }
+
+          self._updateProgress = null;
+          self.emit("inspector-updated", name);
+        },
+      };
+    }
+
+    let progress = this._updateProgress;
+    let done = function() {
+      progress.outstanding.delete(done);
+      progress.checkDone();
+    };
+    progress.outstanding.add(done);
+    return done;
+  },
+
+  /**
+   * Cancel notification of inspector updates.
+   */
+  cancelUpdate: function() {
+    this._updateProgress = null;
   },
 
   /**
@@ -295,23 +455,30 @@ InspectorPanel.prototype = {
   },
 
   /**
-   * When a node is deleted, select its parent node.
+   * When a node is deleted, select its parent node or the defaultNode if no
+   * parent is found (may happen when deleting an iframe inside which the
+   * node was selected).
    */
   onDetached: function InspectorPanel_onDetached(event, parentNode) {
     this.cancelLayoutChange();
     this.breadcrumbs.cutAfter(this.breadcrumbs.indexOf(parentNode));
-    this.selection.setNode(parentNode, "detached");
+    this.selection.setNodeFront(parentNode ? parentNode : this._defaultNode, "detached");
   },
 
   /**
    * Destroy the inspector.
    */
   destroy: function InspectorPanel__destroy() {
-    if (this._destroyed) {
-      return Promise.resolve(null);
+    if (this._panelDestroyer) {
+      return this._panelDestroyer;
     }
-    this._destroyed = true;
 
+    if (this.walker) {
+      this.walker.off("new-root", this.onNewRoot);
+      this.pageStyle = null;
+    }
+
+    this.cancelUpdate();
     this.cancelLayoutChange();
 
     if (this.browser) {
@@ -319,20 +486,12 @@ InspectorPanel.prototype = {
       this.browser = null;
     }
 
-    this.target.off("navigate", this.onNavigatedAway);
-
-    if (this.highlighter) {
-      this.highlighter.off("locked", this.onLockStateChanged);
-      this.highlighter.off("unlocked", this.onLockStateChanged);
-      this.highlighter.destroy();
-    }
+    this.target.off("will-navigate", this._onBeforeNavigate);
 
     this.target.off("thread-paused", this.updateDebuggerPausedWarning);
     this.target.off("thread-resumed", this.updateDebuggerPausedWarning);
     this._toolbox.off("select", this.updateDebuggerPausedWarning);
 
-    this._toolbox = null;
-
     this.sidebar.off("select", this._setDefaultSidebar);
     this.sidebar.destroy();
     this.sidebar = null;
@@ -341,12 +500,12 @@ InspectorPanel.prototype = {
     this.nodemenu.removeEventListener("popuphiding", this._resetNodeMenu, true);
     this.breadcrumbs.destroy();
     this.searchSuggestions.destroy();
-    this.selection.off("new-node", this.onNewSelection);
+    this.searchBox = null;
+    this.selection.off("new-node-front", this.onNewSelection);
     this.selection.off("before-new-node", this.onBeforeNewSelection);
-    this.selection.off("detached", this.onDetached);
-    this._destroyMarkup();
-    this._selection.destroy();
-    this._selection = null;
+    this.selection.off("before-new-node-front", this.onBeforeNewSelection);
+    this.selection.off("detached-front", this.onDetached);
+    this._panelDestroyer = this._destroyMarkup();
     this.panelWin.inspector = null;
     this.target = null;
     this.panelDoc = null;
@@ -355,9 +514,9 @@ InspectorPanel.prototype = {
     this.searchSuggestions = null;
     this.lastNodemenuItem = null;
     this.nodemenu = null;
-    this.highlighter = null;
+    this._toolbox = null;
 
-    return Promise.resolve(null);
+    return this._panelDestroyer;
   },
 
   /**
@@ -380,12 +539,14 @@ InspectorPanel.prototype = {
    * Disable the delete item if needed. Update the pseudo classes.
    */
   _setupNodeMenu: function InspectorPanel_setupNodeMenu() {
+    let isSelectionElement = this.selection.isElementNode();
+
     // Set the pseudo classes
     for (let name of ["hover", "active", "focus"]) {
       let menu = this.panelDoc.getElementById("node-menu-pseudo-" + name);
 
-      if (this.selection.isElementNode()) {
-        let checked = DOMUtils.hasPseudoClassLock(this.selection.node, ":" + name);
+      if (isSelectionElement) {
+        let checked = this.selection.nodeFront.hasPseudoClassLock(":" + name);
         menu.setAttribute("checked", checked);
         menu.removeAttribute("disabled");
       } else {
@@ -406,7 +567,7 @@ InspectorPanel.prototype = {
     let unique = this.panelDoc.getElementById("node-menu-copyuniqueselector");
     let copyInnerHTML = this.panelDoc.getElementById("node-menu-copyinner");
     let copyOuterHTML = this.panelDoc.getElementById("node-menu-copyouter");
-    if (this.selection.isElementNode()) {
+    if (isSelectionElement) {
       unique.removeAttribute("disabled");
       copyInnerHTML.removeAttribute("disabled");
       copyOuterHTML.removeAttribute("disabled");
@@ -415,6 +576,25 @@ InspectorPanel.prototype = {
       copyInnerHTML.setAttribute("disabled", "true");
       copyOuterHTML.setAttribute("disabled", "true");
     }
+
+    // Enable the "edit HTML" item if the selection is an element and the root
+    // actor has the appropriate trait (isOuterHTMLEditable)
+    let editHTML = this.panelDoc.getElementById("node-menu-edithtml");
+    if (this.isOuterHTMLEditable && isSelectionElement) {
+      editHTML.removeAttribute("disabled");
+    } else {
+      editHTML.setAttribute("disabled", "true");
+    }
+
+    // Enable the "copy image data-uri" item if the selection is previewable
+    // which essentially checks if it's an image or canvas tag
+    let copyImageData = this.panelDoc.getElementById("node-menu-copyimagedatauri");
+    let markupContainer = this.markup.getContainer(this.selection.nodeFront);
+    if (markupContainer && markupContainer.isPreviewable()) {
+      copyImageData.removeAttribute("disabled");
+    } else {
+      copyImageData.setAttribute("disabled", "true");
+    }
   },
 
   _resetNodeMenu: function InspectorPanel_resetNodeMenu() {
@@ -461,20 +641,28 @@ InspectorPanel.prototype = {
   },
 
   _destroyMarkup: function InspectorPanel__destroyMarkup() {
+    let destroyPromise;
+
     if (this._boundMarkupFrameLoad) {
       this._markupFrame.removeEventListener("load", this._boundMarkupFrameLoad, true);
-      delete this._boundMarkupFrameLoad;
+      this._boundMarkupFrameLoad = null;
     }
 
     if (this.markup) {
-      this.markup.destroy();
-      delete this.markup;
+      destroyPromise = this.markup.destroy();
+      this.markup = null;
+    } else {
+      destroyPromise = promise.resolve();
     }
 
     if (this._markupFrame) {
       this._markupFrame.parentNode.removeChild(this._markupFrame);
-      delete this._markupFrame;
+      this._markupFrame = null;
     }
+
+    this._markupBox = null;
+
+    return destroyPromise;
   },
 
   /**
@@ -482,46 +670,36 @@ InspectorPanel.prototype = {
    */
   togglePseudoClass: function InspectorPanel_togglePseudoClass(aPseudo) {
     if (this.selection.isElementNode()) {
-      if (DOMUtils.hasPseudoClassLock(this.selection.node, aPseudo)) {
-        this.breadcrumbs.nodeHierarchy.forEach(function(crumb) {
-          DOMUtils.removePseudoClassLock(crumb.node, aPseudo);
-        });
-      } else {
-        let hierarchical = aPseudo == ":hover" || aPseudo == ":active";
-        let node = this.selection.node;
-        do {
-          DOMUtils.addPseudoClassLock(node, aPseudo);
-          node = node.parentNode;
-        } while (hierarchical && node.parentNode)
+      let node = this.selection.nodeFront;
+      if (node.hasPseudoClassLock(aPseudo)) {
+        return this.walker.removePseudoClassLock(node, aPseudo, {parents: true});
       }
+
+      let hierarchical = aPseudo == ":hover" || aPseudo == ":active";
+      return this.walker.addPseudoClassLock(node, aPseudo, {parents: hierarchical});
     }
-    this.selection.emit("pseudoclass");
-    this.breadcrumbs.scroll();
   },
 
   /**
    * Clear any pseudo-class locks applied to the current hierarchy.
    */
   clearPseudoClasses: function InspectorPanel_clearPseudoClasses() {
-    this.breadcrumbs.nodeHierarchy.forEach(function(crumb) {
-      try {
-        DOMUtils.clearPseudoClassLocks(crumb.node);
-      } catch(e) {
-       // Ignore dead nodes after navigation.
-      }
-    });
+    if (!this.walker) {
+      return;
+    }
+    return this.walker.clearPseudoClassLocks().then(null, console.error);
   },
 
   /**
-   * Toggle the highlighter when ruleview is hovered.
+   * Edit the outerHTML of the selected Node.
    */
-  toggleHighlighter: function InspectorPanel_toggleHighlighter(event)
+  editHTML: function InspectorPanel_editHTML()
   {
-    if (event.type == "mouseover") {
-      this.highlighter.hide();
+    if (!this.selection.isNode()) {
+      return;
     }
-    else if (event.type == "mouseout") {
-      this.highlighter.show();
+    if (this.markup) {
+      this.markup.beginEditingOuterHTML(this.selection.nodeFront);
     }
   },
 
@@ -533,10 +711,7 @@ InspectorPanel.prototype = {
     if (!this.selection.isNode()) {
       return;
     }
-    let toCopy = this.selection.node.innerHTML;
-    if (toCopy) {
-      clipboardHelper.copyString(toCopy);
-    }
+    this._copyLongStr(this.walker.innerHTML(this.selection.nodeFront));
   },
 
   /**
@@ -547,12 +722,31 @@ InspectorPanel.prototype = {
     if (!this.selection.isNode()) {
       return;
     }
-    let toCopy = this.selection.node.outerHTML;
-    if (toCopy) {
-      clipboardHelper.copyString(toCopy);
+
+    this._copyLongStr(this.walker.outerHTML(this.selection.nodeFront));
+  },
+
+  /**
+   * Copy the data-uri for the currently selected image in the clipboard.
+   */
+  copyImageDataUri: function InspectorPanel_copyImageDataUri()
+  {
+    let container = this.markup.getContainer(this.selection.nodeFront);
+    if (container && container.isPreviewable()) {
+      container.copyImageDataUri();
     }
   },
 
+  _copyLongStr: function InspectorPanel_copyLongStr(promise)
+  {
+    return promise.then(longstr => {
+      return longstr.string().then(toCopy => {
+        longstr.release().then(null, console.error);
+        clipboardHelper.copyString(toCopy);
+      });
+    }).then(null, console.error);
+  },
+
   /**
    * Copy a unique selector of the selected Node to the clipboard.
    */
@@ -577,33 +771,41 @@ InspectorPanel.prototype = {
       return;
     }
 
-    let toDelete = this.selection.node;
-
-    let parent = this.selection.node.parentNode;
-
     // If the markup panel is active, use the markup panel to delete
     // the node, making this an undoable action.
     if (this.markup) {
-      this.markup.deleteNode(toDelete);
+      this.markup.deleteNode(this.selection.nodeFront);
     } else {
       // remove the node from content
-      parent.removeChild(toDelete);
+      this.walker.removeNode(this.selection.nodeFront);
     }
   },
 
+  /**
+  * Trigger a high-priority layout change for things that need to be
+  * updated immediately
+  */
+  immediateLayoutChange: function Inspector_immediateLayoutChange()
+  {
+    this.emit("layout-change");
+  },
+
   /**
    * Schedule a low-priority change event for things like paint
    * and resize.
    */
-  scheduleLayoutChange: function Inspector_scheduleLayoutChange()
+  scheduleLayoutChange: function Inspector_scheduleLayoutChange(event)
   {
-    if (this._timer) {
-      return null;
+    // Filter out non browser window resize events (i.e. triggered by iframes)
+    if (this.browser.contentWindow === event.target) {
+      if (this._timer) {
+        return null;
+      }
+      this._timer = this.panelWin.setTimeout(function() {
+        this.emit("layout-change");
+        this._timer = null;
+      }.bind(this), LAYOUT_CHANGE_TIMER);
     }
-    this._timer = this.panelWin.setTimeout(function() {
-      this.emit("layout-change");
-      this._timer = null;
-    }.bind(this), LAYOUT_CHANGE_TIMER);
   },
 
   /**
@@ -616,9 +818,8 @@ InspectorPanel.prototype = {
       this.panelWin.clearTimeout(this._timer);
       delete this._timer;
     }
-  },
-
-}
+  }
+};
 
 /////////////////////////////////////////////////////////////////////////
 //// Initializers
