@@ -5,12 +5,12 @@
 const { Cu } = require("chrome");
 module.exports = [];
 
-Cu.import("resource://gre/modules/devtools/gcli.jsm");
+var gcli = require('gcli/index');
 
 loader.lazyGetter(this, "gDevTools",
   () => Cu.import("resource:///modules/devtools/gDevTools.jsm", {}).gDevTools);
 
-var promise = require("sdk/core/promise");
+var { Promise: promise } = Cu.import("resource://gre/modules/Promise.jsm", {});
 
 /*
  * 'profiler' command. Doesn't do anything.
