@@ -2,6 +2,9 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
+let Ci = Components.interfaces;
+let Cc = Components.classes;
+
 dump("### SelectionHandler.js loaded\n");
 
 var SelectionHandler = {
@@ -25,6 +28,7 @@ var SelectionHandler = {
     addMessageListener("Browser:SelectionSwitchMode", this);
     addMessageListener("Browser:RepositionInfoRequest", this);
     addMessageListener("Browser:SelectionHandlerPing", this);
+    addMessageListener("Browser:ResetLastPos", this);
   },
 
   shutdown: function shutdown() {
@@ -44,6 +48,7 @@ var SelectionHandler = {
     removeMessageListener("Browser:SelectionSwitchMode", this);
     removeMessageListener("Browser:RepositionInfoRequest", this);
     removeMessageListener("Browser:SelectionHandlerPing", this);
+    removeMessageListener("Browser:ResetLastPos", this);
   },
 
   sendAsync: function sendAsync(aMsg, aJson) {
@@ -113,13 +118,24 @@ var SelectionHandler = {
       return;
     }
 
-    // Similar to _onSelectionStart - we need to create initial selection
-    // but without the initialization bits.
-    let framePoint = this._clientPointToFramePoint({ xPos: aX, yPos: aY });
-    if (!this._domWinUtils.selectAtPoint(framePoint.xPos, framePoint.yPos,
-                                         Ci.nsIDOMWindowUtils.SELECT_CHARACTER)) {
-      this._onFail("failed to set selection at point");
-      return;
+    // Only use selectAtPoint for editable content and avoid that for inputs,
+    // as we can expand caret to selection manually more precisely. We can use
+    // selectAtPoint for inputs too though, but only once bug 881938 is fully
+    // resolved.
+    if(Util.isEditableContent(this._targetElement)) {
+      // Similar to _onSelectionStart - we need to create initial selection
+      // but without the initialization bits.
+      let framePoint = this._clientPointToFramePoint({ xPos: aX, yPos: aY });
+      if (!this._domWinUtils.selectAtPoint(framePoint.xPos, framePoint.yPos,
+                                           Ci.nsIDOMWindowUtils.SELECT_CHARACTER)) {
+        this._onFail("failed to set selection at point");
+        return;
+      }
+    } else if (this._targetElement.selectionStart == 0 || aMarker == "end") {
+      // Expand caret forward or backward depending on direction
+      this._targetElement.selectionEnd++;
+    } else {
+      this._targetElement.selectionStart--;
     }
 
     // We bail if things get out of sync here implying we missed a message.
@@ -173,14 +189,7 @@ var SelectionHandler = {
       return;
     }
 
-    // Update selection in the doc
-    let pos = null;
-    if (aMsg.change == "start") {
-      pos = aMsg.start;
-    } else {
-      pos = aMsg.end;
-    }
-    this._handleSelectionPoint(aMsg.change, pos, false);
+    this._handleSelectionPoint(aMsg, false);
   },
 
   /*
@@ -197,15 +206,7 @@ var SelectionHandler = {
       return;
     }
 
-    // Update selection in the doc
-    let pos = null;
-    if (aMsg.change == "start") {
-      pos = aMsg.start;
-    } else {
-      pos = aMsg.end;
-    }
-
-    this._handleSelectionPoint(aMsg.change, pos, true);
+    this._handleSelectionPoint(aMsg, true);
     this._selectionMoveActive = false;
     
     // _handleSelectionPoint may set a scroll timer, so this must
@@ -295,12 +296,18 @@ var SelectionHandler = {
    * Called any time SelectionHelperUI would like us to
    * recalculate the selection bounds.
    */
-  _onSelectionUpdate: function _onSelectionUpdate() {
+  _onSelectionUpdate: function _onSelectionUpdate(aMsg) {
     if (!this._contentWindow) {
       this._onFail("_onSelectionUpdate was called without proper view set up");
       return;
     }
 
+    if (aMsg && aMsg.isInitiatedByAPZC) {
+      let {offset: offset} = Content.getCurrentWindowAndOffset(
+        this._targetCoordinates.x, this._targetCoordinates.y);
+      this._contentOffset = offset;
+    }
+
     // Update the position of our selection monocles
     this._updateSelectionUI("update", true, true);
   },
@@ -382,6 +389,7 @@ var SelectionHandler = {
     this._contentOffset = null;
     this._domWinUtils = null;
     this._targetIsEditable = false;
+    this._targetCoordinates = null;
     sendSyncMessage("Content:HandlerShutdown", {});
   },
 
@@ -395,7 +403,7 @@ var SelectionHandler = {
           contentWindow: contentWindow,
           offset: offset,
           utils: utils } =
-      this.getCurrentWindowAndOffset(aX, aY);
+      Content.getCurrentWindowAndOffset(aX, aY);
     if (!contentWindow) {
       return false;
     }
@@ -404,6 +412,11 @@ var SelectionHandler = {
     this._contentOffset = offset;
     this._domWinUtils = utils;
     this._targetIsEditable = Util.isEditable(this._targetElement);
+    this._targetCoordinates = {
+      x: aX,
+      y: aY
+    };
+
     return true;
   },
 
@@ -528,7 +541,7 @@ var SelectionHandler = {
         break;
 
       case "Browser:SelectionUpdate":
-        this._onSelectionUpdate();
+        this._onSelectionUpdate(json);
         break;
 
       case "Browser:RepositionInfoRequest":
@@ -543,6 +556,10 @@ var SelectionHandler = {
       case "Browser:SelectionHandlerPing":
         this._onPing(json.id);
         break;
+
+      case "Browser:ResetLastPos":
+        this.onClickCoords(json.xPos, json.yPos);
+        break;
     }
   },
 
@@ -550,61 +567,6 @@ var SelectionHandler = {
    * Utilities
    */
 
-  /*
-   * Retrieve the total offset from the window's origin to the sub frame
-   * element including frame and scroll offsets. The resulting offset is
-   * such that:
-   * sub frame coords + offset = root frame position
-   */
-  getCurrentWindowAndOffset: function(x, y) {
-    // If the element at the given point belongs to another document (such
-    // as an iframe's subdocument), the element in the calling document's
-    // DOM (e.g. the iframe) is returned.
-    let utils = Util.getWindowUtils(content);
-    let element = utils.elementFromPoint(x, y, true, false);
-    let offset = { x:0, y:0 };
-
-    while (element && (element instanceof HTMLIFrameElement ||
-                       element instanceof HTMLFrameElement)) {
-      // get the child frame position in client coordinates
-      let rect = element.getBoundingClientRect();
-
-      // calculate offsets for digging down into sub frames
-      // using elementFromPoint:
-
-      // Get the content scroll offset in the child frame
-      scrollOffset = ContentScroll.getScrollOffset(element.contentDocument.defaultView);
-      // subtract frame and scroll offset from our elementFromPoint coordinates
-      x -= rect.left + scrollOffset.x;
-      y -= rect.top + scrollOffset.y;
-
-      // calculate offsets we'll use to translate to client coords:
-
-      // add frame client offset to our total offset result
-      offset.x += rect.left;
-      offset.y += rect.top;
-
-      // get the frame's nsIDOMWindowUtils
-      utils = element.contentDocument
-                     .defaultView
-                     .QueryInterface(Ci.nsIInterfaceRequestor)
-                     .getInterface(Ci.nsIDOMWindowUtils);
-
-      // retrieve the target element in the sub frame at x, y
-      element = utils.elementFromPoint(x, y, true, false);
-    }
-
-    if (!element)
-      return {};
-
-    return {
-      element: element,
-      contentWindow: element.ownerDocument.defaultView,
-      offset: offset,
-      utils: utils
-    };
-  },
-
   _getDocShell: function _getDocShell(aWindow) {
     if (aWindow == null)
       return null;
@@ -645,6 +607,7 @@ var SelectionHandler = {
     }
   },
 };
+this.SelectionHandler = SelectionHandler;
 
 SelectionHandler.__proto__ = new SelectionPrototype();
 SelectionHandler.init();
