@@ -29,11 +29,11 @@ NS_IMPL_CI_INTERFACE_GETTER2(nsSystemPrincipal,
                              nsIPrincipal,
                              nsISerializable)
 
-NS_IMETHODIMP_(nsrefcnt) 
+NS_IMETHODIMP_(nsrefcnt)
 nsSystemPrincipal::AddRef()
 {
   NS_PRECONDITION(int32_t(refcount) >= 0, "illegal refcnt");
-  nsrefcnt count = PR_ATOMIC_INCREMENT(&refcount);
+  nsrefcnt count = ++refcount;
   NS_LOG_ADDREF(this, count, "nsSystemPrincipal", sizeof(*this));
   return count;
 }
@@ -42,7 +42,7 @@ NS_IMETHODIMP_(nsrefcnt)
 nsSystemPrincipal::Release()
 {
   NS_PRECONDITION(0 != refcount, "dup release");
-  nsrefcnt count = PR_ATOMIC_DECREMENT(&refcount);
+  nsrefcnt count = --refcount;
   NS_LOG_RELEASE(this, count, "nsSystemPrincipal");
   if (count == 0) {
     delete this;
@@ -79,7 +79,7 @@ nsSystemPrincipal::Equals(nsIPrincipal *other, bool *result)
 }
 
 NS_IMETHODIMP
-nsSystemPrincipal::EqualsIgnoringDomain(nsIPrincipal *other, bool *result)
+nsSystemPrincipal::EqualsConsideringDomain(nsIPrincipal *other, bool *result)
 {
     return Equals(other, result);
 }
@@ -92,7 +92,7 @@ nsSystemPrincipal::Subsumes(nsIPrincipal *other, bool *result)
 }
 
 NS_IMETHODIMP
-nsSystemPrincipal::SubsumesIgnoringDomain(nsIPrincipal *other, bool *result)
+nsSystemPrincipal::SubsumesConsideringDomain(nsIPrincipal *other, bool *result)
 {
     *result = true;
     return NS_OK;
@@ -152,19 +152,6 @@ nsSystemPrincipal::SetDomain(nsIURI* aDomain)
   return NS_OK;
 }
 
-NS_IMETHODIMP
-nsSystemPrincipal::GetSecurityPolicy(void** aSecurityPolicy)
-{
-    *aSecurityPolicy = nullptr;
-    return NS_OK;
-}
-
-NS_IMETHODIMP
-nsSystemPrincipal::SetSecurityPolicy(void* aSecurityPolicy)
-{
-    return NS_OK;
-}
-
 NS_IMETHODIMP
 nsSystemPrincipal::GetJarPrefix(nsACString& aJarPrefix)
 {
