@@ -63,7 +63,6 @@ included_inclnames_to_ignore = set([
     'double-conversion.h',      # strange MFBT case
     'javascript-trace.h',       # generated in $OBJDIR if HAVE_DTRACE is defined
     'jsautokw.h',               # generated in $OBJDIR
-    'jsautooplen.h',            # generated in $OBJDIR
     'jscustomallocator.h',      # provided by embedders;  allowed to be missing
     'js-config.h',              # generated in $OBJDIR
     'pratom.h',                 # NSPR
@@ -95,6 +94,7 @@ oddly_ordered_inclnames = set([
     'ctypes/typedefs.h',        # Included multiple times in the body of ctypes/CTypes.h
     'jsautokw.h',               # Included in the body of frontend/TokenStream.h
     'jswin.h',                  # Must be #included before <psapi.h>
+    'machine/endian.h',         # Must be included after <sys/types.h> on BSD
     'winbase.h',                # Must precede other system headers(?)
     'windef.h'                  # Must precede other system headers(?)
 ])
@@ -317,10 +317,6 @@ def is_module_header(enclosing_inclname, header_inclname):
     if m is not None and module.endswith('/' + m.group(1)):
         return True
 
-    # A weird public header case.
-    if module == 'jsmemorymetrics' and header_inclname == 'js/MemoryMetrics.h':
-        return True
-
     return False
 
 
