@@ -256,34 +256,11 @@ nsDOMFileBase::Slice(int64_t aStart, int64_t aEnd,
   
   // Create the new file
   *aBlob = CreateSlice((uint64_t)aStart, (uint64_t)(aEnd - aStart),
-                       aContentType).get();
+                       aContentType).take();
 
   return *aBlob ? NS_OK : NS_ERROR_UNEXPECTED;
 }
 
-NS_IMETHODIMP
-nsDOMFileBase::MozSlice(int64_t aStart, int64_t aEnd,
-                        const nsAString& aContentType, 
-                        JSContext* aCx,
-                        uint8_t optional_argc,
-                        nsIDOMBlob **aBlob)
-{
-  MOZ_ASSERT(NS_IsMainThread());
-
-  nsIScriptGlobalObject* sgo = nsJSUtils::GetDynamicScriptGlobal(aCx);
-  if (sgo) {
-    nsCOMPtr<nsPIDOMWindow> window = do_QueryInterface(sgo);
-    if (window) {
-      nsCOMPtr<nsIDocument> document = window->GetExtantDoc();
-      if (document) {
-        document->WarnOnceAbout(nsIDocument::eMozSlice);
-      }
-    }
-  }
-
-  return Slice(aStart, aEnd, aContentType, optional_argc, aBlob);
-}
-
 NS_IMETHODIMP
 nsDOMFileBase::GetInternalStream(nsIInputStream **aStream)
 {
@@ -443,6 +420,12 @@ nsDOMFileBase::SetMutable(bool aMutable)
   return rv;
 }
 
+NS_IMETHODIMP_(bool)
+nsDOMFileBase::IsMemoryFile(void)
+{
+  return false;
+}
+
 ////////////////////////////////////////////////////////////////////////////
 // nsDOMFile implementation
 
@@ -636,6 +619,12 @@ nsDOMMemoryFile::GetInternalStream(nsIInputStream **aStream)
   return DataOwnerAdapter::Create(mDataOwner, mStart, mLength, aStream);
 }
 
+NS_IMETHODIMP_(bool)
+nsDOMMemoryFile::IsMemoryFile(void)
+{
+  return true;
+}
+
 /* static */ StaticMutex
 nsDOMMemoryFile::DataOwner::sDataOwnerMutex;
 
