@@ -4,7 +4,7 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-#include "mozilla/Util.h"
+#include "mozilla/ArrayUtils.h"
 
 #include "nsTreeSanitizer.h"
 #include "nsCSSParser.h"
@@ -306,6 +306,7 @@ nsIAtom** const kElementsSVG[] = {
   &nsGkAtoms::feDiffuseLighting, // feDiffuseLighting
   &nsGkAtoms::feDisplacementMap, // feDisplacementMap
   &nsGkAtoms::feDistantLight, // feDistantLight
+  &nsGkAtoms::feDropShadow, // feDropShadow
   &nsGkAtoms::feFlood, // feFlood
   &nsGkAtoms::feFuncA, // feFuncA
   &nsGkAtoms::feFuncB, // feFuncB
@@ -1228,7 +1229,7 @@ nsTreeSanitizer::SanitizeAttributes(mozilla::dom::Element* aElement,
         // <link rel itemprop>
         continue;
       }
-      const PRUnichar* localStr = attrLocal->GetUTF16String();
+      const char16_t* localStr = attrLocal->GetUTF16String();
       // Allow underscore to cater to the MCE editor library.
       // Allow data-* on SVG and MathML, too, as a forward-compat measure.
       if (*localStr == '_' || (attrLocal->GetLength() > 5 && localStr[0] == 'd'
@@ -1359,8 +1360,7 @@ nsTreeSanitizer::Sanitize(nsIDocument* aDocument)
   // here that notifies / does not notify or that fires mutation events if
   // in tree.
 #ifdef DEBUG
-  nsCOMPtr<nsISupports> container = aDocument->GetContainer();
-  NS_PRECONDITION(!container, "The document is in a shell.");
+  NS_PRECONDITION(!aDocument->GetContainer(), "The document is in a shell.");
   nsRefPtr<mozilla::dom::Element> root = aDocument->GetRootElement();
   NS_PRECONDITION(root->IsHTML(nsGkAtoms::html), "Not HTML root.");
 #endif
