@@ -9,18 +9,23 @@
 #include "imgIRequest.h"
 #include "gfxASurface.h"
 #include "gfxPoint.h"
-#include "nsIDOMElement.h"
+#include "mozilla/dom/Element.h"
 #include "nsTHashtable.h"
-#include "nsHTMLCanvasElement.h"
+#include "mozilla/dom/HTMLCanvasElement.h"
 #include "nsContentUtils.h"
+#include "mozilla/Preferences.h"
+#include "mozilla/gfx/2D.h"
 
 namespace mozilla {
 
+using namespace dom;
+using namespace gfx;
+
 struct ImageCacheKey {
-  ImageCacheKey(nsIDOMElement* aImage, nsHTMLCanvasElement* aCanvas)
+  ImageCacheKey(Element* aImage, HTMLCanvasElement* aCanvas)
     : mImage(aImage), mCanvas(aCanvas) {}
-  nsIDOMElement* mImage;
-  nsHTMLCanvasElement* mCanvas;
+  Element* mImage;
+  HTMLCanvasElement* mCanvas;
 };
 
 struct ImageCacheEntryData {
@@ -29,24 +34,26 @@ struct ImageCacheEntryData {
     , mILC(aOther.mILC)
     , mCanvas(aOther.mCanvas)
     , mRequest(aOther.mRequest)
-    , mSurface(aOther.mSurface)
+    , mSourceSurface(aOther.mSourceSurface)
     , mSize(aOther.mSize)
   {}
   ImageCacheEntryData(const ImageCacheKey& aKey)
     : mImage(aKey.mImage)
-    , mILC(nsnull)
+    , mILC(nullptr)
     , mCanvas(aKey.mCanvas)
   {}
 
   nsExpirationState* GetExpirationState() { return &mState; }
 
+  size_t SizeInBytes() { return mSize.width * mSize.height * 4; }
+
   // Key
-  nsCOMPtr<nsIDOMElement> mImage;
+  nsRefPtr<Element> mImage;
   nsIImageLoadingContent* mILC;
-  nsRefPtr<nsHTMLCanvasElement> mCanvas;
+  nsRefPtr<HTMLCanvasElement> mCanvas;
   // Value
   nsCOMPtr<imgIRequest> mRequest;
-  nsRefPtr<gfxASurface> mSurface;
+  RefPtr<SourceSurface> mSourceSurface;
   gfxIntSize mSize;
   nsExpirationState mState;
 };
@@ -77,14 +84,21 @@ public:
   nsAutoPtr<ImageCacheEntryData> mData;
 };
 
+static bool sPrefsInitialized = false;
+static int32_t sCanvasImageCacheLimit = 0;
+
 class ImageCache MOZ_FINAL : public nsExpirationTracker<ImageCacheEntryData,4> {
 public:
   // We use 3 generations of 1 second each to get a 2-3 seconds timeout.
   enum { GENERATION_MS = 1000 };
   ImageCache()
     : nsExpirationTracker<ImageCacheEntryData,4>(GENERATION_MS)
+    , mTotal(0)
   {
-    mCache.Init();
+    if (!sPrefsInitialized) {
+      sPrefsInitialized = true;
+      Preferences::AddIntVarCache(&sCanvasImageCacheLimit, "canvas.image.cache.limit", 0);
+    }
   }
   ~ImageCache() {
     AgeAllGenerations();
@@ -92,15 +106,17 @@ public:
 
   virtual void NotifyExpired(ImageCacheEntryData* aObject)
   {
+    mTotal -= aObject->SizeInBytes();
     RemoveObject(aObject);
     // Deleting the entry will delete aObject since the entry owns aObject
     mCache.RemoveEntry(ImageCacheKey(aObject->mImage, aObject->mCanvas));
   }
 
   nsTHashtable<ImageCacheEntry> mCache;
+  size_t mTotal;
 };
 
-static ImageCache* gImageCache = nsnull;
+static ImageCache* gImageCache = nullptr;
 
 class CanvasImageCacheShutdownObserver MOZ_FINAL : public nsIObserver
 {
@@ -110,10 +126,10 @@ public:
 };
 
 void
-CanvasImageCache::NotifyDrawImage(nsIDOMElement* aImage,
-                                  nsHTMLCanvasElement* aCanvas,
+CanvasImageCache::NotifyDrawImage(Element* aImage,
+                                  HTMLCanvasElement* aCanvas,
                                   imgIRequest* aRequest,
-                                  gfxASurface* aSurface,
+                                  SourceSurface* aSource,
                                   const gfxIntSize& aSize)
 {
   if (!gImageCache) {
@@ -123,8 +139,9 @@ CanvasImageCache::NotifyDrawImage(nsIDOMElement* aImage,
 
   ImageCacheEntry* entry = gImageCache->mCache.PutEntry(ImageCacheKey(aImage, aCanvas));
   if (entry) {
-    if (entry->mData->mSurface) {
+    if (entry->mData->mSourceSurface) {
       // We are overwriting an existing entry.
+      gImageCache->mTotal -= entry->mData->SizeInBytes();
       gImageCache->RemoveObject(entry->mData);
     }
     gImageCache->AddObject(entry->mData);
@@ -135,32 +152,41 @@ CanvasImageCache::NotifyDrawImage(nsIDOMElement* aImage,
                       getter_AddRefs(entry->mData->mRequest));
     }
     entry->mData->mILC = ilc;
-    entry->mData->mSurface = aSurface;
+    entry->mData->mSourceSurface = aSource;
     entry->mData->mSize = aSize;
+
+    gImageCache->mTotal += entry->mData->SizeInBytes();
   }
+
+  if (!sCanvasImageCacheLimit)
+    return;
+
+  // Expire the image cache early if its larger than we want it to be.
+  while (gImageCache->mTotal > size_t(sCanvasImageCacheLimit))
+    gImageCache->AgeOneGeneration();
 }
 
-gfxASurface*
-CanvasImageCache::Lookup(nsIDOMElement* aImage,
-                         nsHTMLCanvasElement* aCanvas,
+SourceSurface*
+CanvasImageCache::Lookup(Element* aImage,
+                         HTMLCanvasElement* aCanvas,
                          gfxIntSize* aSize)
 {
   if (!gImageCache)
-    return nsnull;
+    return nullptr;
 
   ImageCacheEntry* entry = gImageCache->mCache.GetEntry(ImageCacheKey(aImage, aCanvas));
   if (!entry || !entry->mData->mILC)
-    return nsnull;
+    return nullptr;
 
   nsCOMPtr<imgIRequest> request;
   entry->mData->mILC->GetRequest(nsIImageLoadingContent::CURRENT_REQUEST, getter_AddRefs(request));
   if (request != entry->mData->mRequest)
-    return nsnull;
+    return nullptr;
 
   gImageCache->MarkUsed(entry->mData);
 
   *aSize = entry->mData->mSize;
-  return entry->mData->mSurface;
+  return entry->mData->mSourceSurface;
 }
 
 NS_IMPL_ISUPPORTS1(CanvasImageCacheShutdownObserver, nsIObserver)
@@ -168,11 +194,11 @@ NS_IMPL_ISUPPORTS1(CanvasImageCacheShutdownObserver, nsIObserver)
 NS_IMETHODIMP
 CanvasImageCacheShutdownObserver::Observe(nsISupports *aSubject,
                                           const char *aTopic,
-                                          const PRUnichar *aData)
+                                          const char16_t *aData)
 {
   if (strcmp(aTopic, NS_XPCOM_SHUTDOWN_OBSERVER_ID) == 0) {
     delete gImageCache;
-    gImageCache = nsnull;
+    gImageCache = nullptr;
 
     nsContentUtils::UnregisterShutdownObserver(this);
   }
