@@ -24,6 +24,9 @@ extern "C" {
 #include "VorbisUtils.h"
 #include "MediaMetadataManager.h"
 #include "nsISeekableStream.h"
+#include "gfx2DGlue.h"
+
+using namespace mozilla::gfx;
 
 namespace mozilla {
 
@@ -288,7 +291,7 @@ nsresult OggReader::ReadMetadata(MediaInfo* aInfo,
 
     nsIntSize frameSize(mTheoraState->mInfo.frame_width,
                         mTheoraState->mInfo.frame_height);
-    if (VideoInfo::ValidateVideoRegion(frameSize, picture, displaySize)) {
+    if (IsValidVideoRegion(frameSize, picture, displaySize)) {
       // Video track's frame sizes will not overflow. Activate the video track.
       mInfo.mVideo.mHasVideo = true;
       mInfo.mVideo.mDisplay = displaySize;
@@ -765,7 +768,7 @@ nsresult OggReader::DecodeTheora(ogg_packet* aPacket, int64_t aTimeThreshold)
                                      b,
                                      isKeyframe,
                                      aPacket->granulepos,
-                                     mPicture);
+                                     ToIntRect(mPicture));
     if (!v) {
       // There may be other reasons for this error, but for
       // simplicity just assume the worst case: out of memory.
@@ -1087,8 +1090,8 @@ nsresult OggReader::GetSeekRanges(nsTArray<SeekRange>& aRanges)
     if (startTime != -1 &&
         ((endTime = RangeEndTime(endOffset)) != -1))
     {
-      NS_ASSERTION(startTime < endTime,
-                   "Start time must be before end time");
+      NS_WARN_IF_FALSE(startTime < endTime,
+                       "Start time must be before end time");
       aRanges.AppendElement(SeekRange(startOffset,
                                       endOffset,
                                       startTime,
