@@ -15,26 +15,51 @@
 #include "AbstractMediaDecoder.h"
 #include "OmxDecoder.h"
 #include "MPAPI.h"
+#include "gfx2DGlue.h"
+
+#ifdef MOZ_AUDIO_OFFLOAD
+#include <stagefright/Utils.h>
+#include <cutils/properties.h>
+#include <stagefright/MetaData.h>
+#endif
 
 #define MAX_DROPPED_FRAMES 25
+// Try not to spend more than this much time in a single call to DecodeVideoFrame.
+#define MAX_VIDEO_DECODE_SECONDS 3.0
 
+using namespace mozilla::gfx;
 using namespace android;
 
 namespace mozilla {
 
+#ifdef PR_LOGGING
+extern PRLogModuleInfo* gMediaDecoderLog;
+#define DECODER_LOG(type, msg) PR_LOG(gMediaDecoderLog, type, msg)
+#else
+#define DECODER_LOG(type, msg)
+#endif
+
 MediaOmxReader::MediaOmxReader(AbstractMediaDecoder *aDecoder) :
   MediaDecoderReader(aDecoder),
   mHasVideo(false),
   mHasAudio(false),
   mVideoSeekTimeUs(-1),
   mAudioSeekTimeUs(-1),
-  mSkipCount(0)
+  mSkipCount(0),
+  mAudioChannelType(dom::AUDIO_CHANNEL_DEFAULT)
 {
+#ifdef PR_LOGGING
+  if (!gMediaDecoderLog) {
+    gMediaDecoderLog = PR_NewLogModule("MediaDecoder");
+  }
+#endif
 }
 
 MediaOmxReader::~MediaOmxReader()
 {
-  ResetDecode();
+  ReleaseMediaResources();
+  ReleaseDecoder();
+  mOmxDecoder.clear();
 }
 
 nsresult MediaOmxReader::Init(MediaDecoderReader* aCloneDonor)
@@ -42,18 +67,88 @@ nsresult MediaOmxReader::Init(MediaDecoderReader* aCloneDonor)
   return NS_OK;
 }
 
-nsresult MediaOmxReader::ReadMetadata(VideoInfo* aInfo,
+bool MediaOmxReader::IsWaitingMediaResources()
+{
+  if (!mOmxDecoder.get()) {
+    return false;
+  }
+  return mOmxDecoder->IsWaitingMediaResources();
+}
+
+bool MediaOmxReader::IsDormantNeeded()
+{
+  if (!mOmxDecoder.get()) {
+    return false;
+  }
+  return mOmxDecoder->IsDormantNeeded();
+}
+
+void MediaOmxReader::ReleaseMediaResources()
+{
+  ResetDecode();
+  // Before freeing a video codec, all video buffers needed to be released
+  // even from graphics pipeline.
+  VideoFrameContainer* container = mDecoder->GetVideoFrameContainer();
+  if (container) {
+    container->ClearCurrentFrame();
+  }
+  if (mOmxDecoder.get()) {
+    mOmxDecoder->ReleaseMediaResources();
+  }
+}
+
+void MediaOmxReader::ReleaseDecoder()
+{
+  if (mOmxDecoder.get()) {
+    mOmxDecoder->ReleaseDecoder();
+  }
+}
+
+nsresult MediaOmxReader::InitOmxDecoder()
+{
+  if (!mOmxDecoder.get()) {
+    //register sniffers, if they are not registered in this process.
+    DataSource::RegisterDefaultSniffers();
+    mDecoder->GetResource()->SetReadMode(MediaCacheStream::MODE_METADATA);
+
+    sp<DataSource> dataSource = new MediaStreamSource(mDecoder->GetResource(), mDecoder);
+    dataSource->initCheck();
+
+    mExtractor = MediaExtractor::Create(dataSource);
+    if (!mExtractor.get()) {
+      return NS_ERROR_FAILURE;
+    }
+    mOmxDecoder = new OmxDecoder(mDecoder->GetResource(), mDecoder);
+    if (!mOmxDecoder->Init(mExtractor)) {
+      return NS_ERROR_FAILURE;
+    }
+  }
+  return NS_OK;
+}
+
+nsresult MediaOmxReader::ReadMetadata(MediaInfo* aInfo,
                                       MetadataTags** aTags)
 {
   NS_ASSERTION(mDecoder->OnDecodeThread(), "Should be on decode thread.");
 
   *aTags = nullptr;
 
-  if (!mOmxDecoder.get()) {
-    mOmxDecoder = new OmxDecoder(mDecoder->GetResource(), mDecoder);
-    if (!mOmxDecoder->Init()) {
-      return NS_ERROR_FAILURE;
-    }
+  // Initialize the internal OMX Decoder.
+  nsresult rv = InitOmxDecoder();
+  if (NS_FAILED(rv)) {
+    return rv;
+  }
+
+  if (!mOmxDecoder->TryLoad()) {
+    return NS_ERROR_FAILURE;
+  }
+
+#ifdef MOZ_AUDIO_OFFLOAD
+  CheckAudioOffload();
+#endif
+
+  if (IsWaitingMediaResources()) {
+    return NS_OK;
   }
 
   // Set the total duration (the max of the audio and video track).
@@ -64,6 +159,9 @@ nsresult MediaOmxReader::ReadMetadata(VideoInfo* aInfo,
     mDecoder->SetMediaDuration(durationUs);
   }
 
+  // Check the MediaExtract flag if the source is seekable.
+  mDecoder->SetMediaSeekable(mExtractor->flags() & MediaExtractor::CAN_SEEK);
+
   if (mOmxDecoder->HasVideo()) {
     int32_t width, height;
     mOmxDecoder->GetVideoParameters(&width, &height);
@@ -73,13 +171,13 @@ nsresult MediaOmxReader::ReadMetadata(VideoInfo* aInfo,
     // that our video frame creation code doesn't overflow.
     nsIntSize displaySize(width, height);
     nsIntSize frameSize(width, height);
-    if (!VideoInfo::ValidateVideoRegion(frameSize, pictureRect, displaySize)) {
+    if (!IsValidVideoRegion(frameSize, pictureRect, displaySize)) {
       return NS_ERROR_FAILURE;
     }
 
     // Video track's frame sizes will not overflow. Activate the video track.
-    mHasVideo = mInfo.mHasVideo = true;
-    mInfo.mDisplay = displaySize;
+    mHasVideo = mInfo.mVideo.mHasVideo = true;
+    mInfo.mVideo.mDisplay = displaySize;
     mPicture = pictureRect;
     mInitialFrame = frameSize;
     VideoFrameContainer* container = mDecoder->GetVideoFrameContainer();
@@ -93,9 +191,9 @@ nsresult MediaOmxReader::ReadMetadata(VideoInfo* aInfo,
   if (mOmxDecoder->HasAudio()) {
     int32_t numChannels, sampleRate;
     mOmxDecoder->GetAudioParameters(&numChannels, &sampleRate);
-    mHasAudio = mInfo.mHasAudio = true;
-    mInfo.mAudioChannels = numChannels;
-    mInfo.mAudioRate = sampleRate;
+    mHasAudio = mInfo.mAudio.mHasAudio = true;
+    mInfo.mAudio.mChannels = numChannels;
+    mInfo.mAudio.mRate = sampleRate;
   }
 
  *aInfo = mInfo;
@@ -103,20 +201,6 @@ nsresult MediaOmxReader::ReadMetadata(VideoInfo* aInfo,
   return NS_OK;
 }
 
-// Resets all state related to decoding, emptying all buffers etc.
-nsresult MediaOmxReader::ResetDecode()
-{
-  MediaDecoderReader::ResetDecode();
-
-  VideoFrameContainer* container = mDecoder->GetVideoFrameContainer();
-  if (container) {
-    container->ClearCurrentFrame();
-  }
-
-  mOmxDecoder.clear();
-  return NS_OK;
-}
-
 bool MediaOmxReader::DecodeVideoFrame(bool &aKeyframeSkip,
                                       int64_t aTimeThreshold)
 {
@@ -130,28 +214,35 @@ bool MediaOmxReader::DecodeVideoFrame(bool &aKeyframeSkip,
     aTimeThreshold = mVideoSeekTimeUs;
   }
 
-  // Read next frame
-  while (true) {
+  TimeStamp start = TimeStamp::Now();
+
+  // Read next frame. Don't let this loop run for too long.
+  while ((TimeStamp::Now() - start) < TimeDuration::FromSeconds(MAX_VIDEO_DECODE_SECONDS)) {
     MPAPI::VideoFrame frame;
     frame.mGraphicBuffer = nullptr;
     frame.mShouldSkip = false;
     if (!mOmxDecoder->ReadVideo(&frame, aTimeThreshold, aKeyframeSkip, doSeek)) {
-      mVideoQueue.Finish();
       return false;
     }
+    doSeek = false;
+
+    // Ignore empty buffer which stagefright media read will sporadically return
+    if (frame.mSize == 0 && !frame.mGraphicBuffer) {
+      continue;
+    }
 
     parsed++;
     if (frame.mShouldSkip && mSkipCount < MAX_DROPPED_FRAMES) {
       mSkipCount++;
-      return true;
+      continue;
     }
 
     mSkipCount = 0;
 
     mVideoSeekTimeUs = -1;
-    doSeek = aKeyframeSkip = false;
+    aKeyframeSkip = false;
 
-    nsIntRect picture = mPicture;
+    IntRect picture = ToIntRect(mPicture);
     if (frame.Y.mWidth != mInitialFrame.width ||
         frame.Y.mHeight != mInitialFrame.height) {
 
@@ -192,21 +283,21 @@ bool MediaOmxReader::DecodeVideoFrame(bool &aKeyframeSkip,
       b.mPlanes[2].mOffset = frame.Cr.mOffset;
       b.mPlanes[2].mSkip = frame.Cr.mSkip;
 
-      v = VideoData::Create(mInfo,
+      v = VideoData::Create(mInfo.mVideo,
                             mDecoder->GetImageContainer(),
                             pos,
                             frame.mTimeUs,
-                            frame.mTimeUs+1, // We don't know the end time.
+                            1, // We don't know the duration.
                             b,
                             frame.mKeyFrame,
                             -1,
                             picture);
     } else {
-      v = VideoData::Create(mInfo,
+      v = VideoData::Create(mInfo.mVideo,
                             mDecoder->GetImageContainer(),
                             pos,
                             frame.mTimeUs,
-                            frame.mTimeUs+1, // We don't know the end time.
+                            1, // We don't know the duration.
                             frame.mGraphicBuffer,
                             frame.mKeyFrame,
                             -1,
@@ -229,6 +320,15 @@ bool MediaOmxReader::DecodeVideoFrame(bool &aKeyframeSkip,
   return true;
 }
 
+void MediaOmxReader::NotifyDataArrived(const char* aBuffer, uint32_t aLength, int64_t aOffset)
+{
+  android::OmxDecoder *omxDecoder = mOmxDecoder.get();
+
+  if (omxDecoder) {
+    omxDecoder->NotifyDataArrived(aBuffer, aLength, aOffset);
+  }
+}
+
 bool MediaOmxReader::DecodeAudioData()
 {
   NS_ASSERTION(mDecoder->OnDecodeThread(), "Should be on decode thread.");
@@ -237,42 +337,40 @@ bool MediaOmxReader::DecodeAudioData()
   int64_t pos = mDecoder->GetResource()->Tell();
 
   // Read next frame
-  MPAPI::AudioFrame frame;
-  if (!mOmxDecoder->ReadAudio(&frame, mAudioSeekTimeUs)) {
-    mAudioQueue.Finish();
+  MPAPI::AudioFrame source;
+  if (!mOmxDecoder->ReadAudio(&source, mAudioSeekTimeUs)) {
     return false;
   }
   mAudioSeekTimeUs = -1;
 
   // Ignore empty buffer which stagefright media read will sporadically return
-  if (frame.mSize == 0) {
+  if (source.mSize == 0) {
     return true;
   }
 
-  nsAutoArrayPtr<AudioDataValue> buffer(new AudioDataValue[frame.mSize/2] );
-  memcpy(buffer.get(), frame.mData, frame.mSize);
+  uint32_t frames = source.mSize / (source.mAudioChannels *
+                                    sizeof(AudioDataValue));
 
-  uint32_t frames = frame.mSize / (2 * frame.mAudioChannels);
-  CheckedInt64 duration = FramesToUsecs(frames, frame.mAudioSampleRate);
-  if (!duration.isValid()) {
-    return false;
-  }
-
-  mAudioQueue.Push(new AudioData(pos,
-                                 frame.mTimeUs,
-                                 duration.value(),
-                                 frames,
-                                 buffer.forget(),
-                                 frame.mAudioChannels));
-  return true;
+  typedef AudioCompactor::NativeCopy OmxCopy;
+  return mAudioCompactor.Push(pos,
+                              source.mTimeUs,
+                              source.mAudioSampleRate,
+                              frames,
+                              source.mAudioChannels,
+                              OmxCopy(static_cast<uint8_t *>(source.mData),
+                                      source.mSize,
+                                      source.mAudioChannels));
 }
 
 nsresult MediaOmxReader::Seek(int64_t aTarget, int64_t aStartTime, int64_t aEndTime, int64_t aCurrentTime)
 {
   NS_ASSERTION(mDecoder->OnDecodeThread(), "Should be on decode thread.");
 
-  mVideoQueue.Reset();
-  mAudioQueue.Reset();
+  ResetDecode();
+  VideoFrameContainer* container = mDecoder->GetVideoFrameContainer();
+  if (container && container->GetImageContainer()) {
+    container->GetImageContainer()->ClearAllImagesExceptFront();
+  }
 
   mAudioSeekTimeUs = mVideoSeekTimeUs = aTarget;
 
@@ -286,63 +384,56 @@ static uint64_t BytesToTime(int64_t offset, uint64_t length, uint64_t durationUs
   return uint64_t(double(durationUs) * perc);
 }
 
-nsresult MediaOmxReader::GetBuffered(mozilla::dom::TimeRanges* aBuffered, int64_t aStartTime)
+void MediaOmxReader::SetIdle() {
+  if (!mOmxDecoder.get()) {
+    return;
+  }
+  mOmxDecoder->Pause();
+}
+
+void MediaOmxReader::SetActive() {
+  if (!mOmxDecoder.get()) {
+    return;
+  }
+  DebugOnly<nsresult> result = mOmxDecoder->Play();
+  NS_ASSERTION(result == NS_OK, "OmxDecoder should be in play state to continue decoding");
+}
+
+#ifdef MOZ_AUDIO_OFFLOAD
+void MediaOmxReader::CheckAudioOffload()
 {
-  if (!mOmxDecoder.get())
-    return NS_OK;
+  NS_ASSERTION(mDecoder->OnDecodeThread(), "Should be on decode thread.");
 
-  MediaResource* stream = mOmxDecoder->GetResource();
-
-  int64_t durationUs = 0;
-  mOmxDecoder->GetDuration(&durationUs);
-
-  // Nothing to cache if the media takes 0us to play.
-  if (!durationUs)
-    return NS_OK;
-
-  // Special case completely cached files.  This also handles local files.
-  if (stream->IsDataCachedToEndOfResource(0)) {
-    aBuffered->Add(0, durationUs);
-    return NS_OK;
+  char offloadProp[128];
+  property_get("audio.offload.disable", offloadProp, "0");
+  bool offloadDisable =  atoi(offloadProp) != 0;
+  if (offloadDisable) {
+    return;
   }
 
-  int64_t totalBytes = stream->GetLength();
+  mAudioOffloadTrack = mOmxDecoder->GetAudioOffloadTrack();
+  sp<MetaData> meta = (mAudioOffloadTrack.get()) ?
+      mAudioOffloadTrack->getFormat() : nullptr;
 
-  // If we can't determine the total size, pretend that we have nothing
-  // buffered. This will put us in a state of eternally-low-on-undecoded-data
-  // which is not get, but about the best we can do.
-  if (totalBytes == -1)
-    return NS_OK;
+  // Supporting audio offload only when there is no video, no streaming
+  bool hasNoVideo = !mOmxDecoder->HasVideo();
+  bool isNotStreaming
+      = mDecoder->GetResource()->IsDataCachedToEndOfResource(0);
 
-  int64_t startOffset = stream->GetNextCachedData(0);
-  while (startOffset >= 0) {
-    int64_t endOffset = stream->GetCachedDataEnd(startOffset);
-    // Bytes [startOffset..endOffset] are cached.
-    NS_ASSERTION(startOffset >= 0, "Integer underflow in GetBuffered");
-    NS_ASSERTION(endOffset >= 0, "Integer underflow in GetBuffered");
+  // Not much benefit in trying to offload other channel types. Most of them
+  // aren't supported and also duration would be less than a minute
+  bool isTypeMusic = mAudioChannelType == dom::AUDIO_CHANNEL_CONTENT;
 
-    uint64_t startUs = BytesToTime(startOffset, totalBytes, durationUs);
-    uint64_t endUs = BytesToTime(endOffset, totalBytes, durationUs);
-    if (startUs != endUs) {
-      aBuffered->Add((double)startUs / USECS_PER_S, (double)endUs / USECS_PER_S);
-    }
-    startOffset = stream->GetNextCachedData(endOffset);
-  }
-  return NS_OK;
-}
+  DECODER_LOG(PR_LOG_DEBUG, ("%s meta %p, no video %d, no streaming %d,"
+      " channel type %d", __FUNCTION__, meta.get(), hasNoVideo,
+      isNotStreaming, mAudioChannelType));
 
-void MediaOmxReader::OnDecodeThreadFinish() {
-  if (mOmxDecoder.get()) {
-    mOmxDecoder->Pause();
-  }
-}
-
-void MediaOmxReader::OnDecodeThreadStart() {
-  if (mOmxDecoder.get()) {
-    DebugOnly<nsresult> result = mOmxDecoder->Play();
-    NS_ASSERTION(result == NS_OK, "OmxDecoder should be in play state to continue decoding");
+  if ((meta.get()) && hasNoVideo && isNotStreaming && isTypeMusic &&
+      canOffloadStream(meta, false, false, AUDIO_STREAM_MUSIC)) {
+    DECODER_LOG(PR_LOG_DEBUG, ("Can offload this audio stream"));
+    mDecoder->SetCanOffloadAudio(true);
   }
 }
+#endif
 
 } // namespace mozilla
-
