@@ -6,15 +6,13 @@
 
 #include "AudioBuffer.h"
 #include "mozilla/dom/AudioBufferBinding.h"
-#include "nsContentUtils.h"
-#include "AudioContext.h"
 #include "jsfriendapi.h"
 #include "mozilla/ErrorResult.h"
 #include "AudioSegment.h"
-#include "nsIScriptError.h"
-#include "nsPIDOMWindow.h"
 #include "AudioChannelFormat.h"
 #include "mozilla/PodOperations.h"
+#include "mozilla/CheckedInt.h"
+#include "AudioNodeEngine.h"
 
 namespace mozilla {
 namespace dom {
@@ -43,15 +41,15 @@ NS_IMPL_CYCLE_COLLECTION_TRACE_END
 NS_IMPL_CYCLE_COLLECTION_ROOT_NATIVE(AudioBuffer, AddRef)
 NS_IMPL_CYCLE_COLLECTION_UNROOT_NATIVE(AudioBuffer, Release)
 
-AudioBuffer::AudioBuffer(AudioContext* aContext, uint32_t aLength,
-                         float aSampleRate)
+AudioBuffer::AudioBuffer(AudioContext* aContext, uint32_t aNumberOfChannels,
+                         uint32_t aLength, float aSampleRate)
   : mContext(aContext),
     mLength(aLength),
     mSampleRate(aSampleRate)
 {
+  mJSChannels.SetCapacity(aNumberOfChannels);
   SetIsDOMBinding();
-
-  nsContentUtils::HoldJSObjects(this, NS_CYCLE_COLLECTION_PARTICIPANT(AudioBuffer));
+  mozilla::HoldJSObjects(this);
 }
 
 AudioBuffer::~AudioBuffer()
@@ -63,24 +61,39 @@ void
 AudioBuffer::ClearJSChannels()
 {
   mJSChannels.Clear();
-  nsContentUtils::DropJSObjects(this);
+  mozilla::DropJSObjects(this);
 }
 
-bool
-AudioBuffer::InitializeBuffers(uint32_t aNumberOfChannels, JSContext* aJSContext)
+/* static */ already_AddRefed<AudioBuffer>
+AudioBuffer::Create(AudioContext* aContext, uint32_t aNumberOfChannels,
+                    uint32_t aLength, float aSampleRate,
+                    JSContext* aJSContext, ErrorResult& aRv)
 {
-  if (!mJSChannels.SetCapacity(aNumberOfChannels)) {
-    return false;
-  }
-  for (uint32_t i = 0; i < aNumberOfChannels; ++i) {
-    JS::RootedObject array(aJSContext, JS_NewFloat32Array(aJSContext, mLength));
-    if (!array) {
-      return false;
-    }
-    mJSChannels.AppendElement(array.get());
+  // Note that a buffer with zero channels is permitted here for the sake of
+  // AudioProcessingEvent, where channel counts must match parameters passed
+  // to createScriptProcessor(), one of which may be zero.
+  if (aSampleRate < WebAudioUtils::MinSampleRate ||
+      aSampleRate > WebAudioUtils::MaxSampleRate ||
+      aNumberOfChannels > WebAudioUtils::MaxChannelCount ||
+      !aLength || aLength > INT32_MAX) {
+    aRv.Throw(NS_ERROR_DOM_INDEX_SIZE_ERR);
+    return nullptr;
   }
 
-  return true;
+  nsRefPtr<AudioBuffer> buffer =
+    new AudioBuffer(aContext, aNumberOfChannels, aLength, aSampleRate);
+
+  for (uint32_t i = 0; i < aNumberOfChannels; ++i) {
+    JS::Rooted<JSObject*> array(aJSContext,
+                                JS_NewFloat32Array(aJSContext, aLength));
+    if (!array) {
+      aRv.Throw(NS_ERROR_OUT_OF_MEMORY);
+      return nullptr;
+    }
+    buffer->mJSChannels.AppendElement(array.get());
+  }
+
+  return buffer.forget();
 }
 
 JSObject*
@@ -98,7 +111,8 @@ AudioBuffer::RestoreJSChannelData(JSContext* aJSContext)
       // The following code first zeroes the array and then copies our data
       // into it. We could avoid this with additional JS APIs to construct
       // an array (or ArrayBuffer) containing initial data.
-      JS::RootedObject array(aJSContext, JS_NewFloat32Array(aJSContext, mLength));
+      JS::Rooted<JSObject*> array(aJSContext,
+                                  JS_NewFloat32Array(aJSContext, mLength));
       if (!array) {
         return false;
       }
@@ -112,6 +126,64 @@ AudioBuffer::RestoreJSChannelData(JSContext* aJSContext)
   return true;
 }
 
+void
+AudioBuffer::CopyFromChannel(const Float32Array& aDestination, uint32_t aChannelNumber,
+                             uint32_t aStartInChannel, ErrorResult& aRv)
+{
+  aDestination.ComputeLengthAndData();
+
+  uint32_t length = aDestination.Length();
+  CheckedInt<uint32_t> end = aStartInChannel;
+  end += length;
+  if (aChannelNumber >= NumberOfChannels() ||
+      !end.isValid() || end.value() > mLength) {
+    aRv.Throw(NS_ERROR_DOM_INDEX_SIZE_ERR);
+    return;
+  }
+
+  if (!mSharedChannels && JS_GetTypedArrayLength(mJSChannels[aChannelNumber]) != mLength) {
+    // The array was probably neutered
+    aRv.Throw(NS_ERROR_DOM_INDEX_SIZE_ERR);
+    return;
+  }
+
+  const float* sourceData = mSharedChannels ?
+    mSharedChannels->GetData(aChannelNumber) :
+    JS_GetFloat32ArrayData(mJSChannels[aChannelNumber]);
+  PodMove(aDestination.Data(), sourceData + aStartInChannel, length);
+}
+
+void
+AudioBuffer::CopyToChannel(JSContext* aJSContext, const Float32Array& aSource,
+                           uint32_t aChannelNumber, uint32_t aStartInChannel,
+                           ErrorResult& aRv)
+{
+  aSource.ComputeLengthAndData();
+
+  uint32_t length = aSource.Length();
+  CheckedInt<uint32_t> end = aStartInChannel;
+  end += length;
+  if (aChannelNumber >= NumberOfChannels() ||
+      !end.isValid() || end.value() > mLength) {
+    aRv.Throw(NS_ERROR_DOM_INDEX_SIZE_ERR);
+    return;
+  }
+
+  if (!mSharedChannels && JS_GetTypedArrayLength(mJSChannels[aChannelNumber]) != mLength) {
+    // The array was probably neutered
+    aRv.Throw(NS_ERROR_DOM_INDEX_SIZE_ERR);
+    return;
+  }
+
+  if (!RestoreJSChannelData(aJSContext)) {
+    aRv.Throw(NS_ERROR_OUT_OF_MEMORY);
+    return;
+  }
+
+  PodMove(JS_GetFloat32ArrayData(mJSChannels[aChannelNumber]) + aStartInChannel,
+          aSource.Data(), length);
+}
+
 void
 AudioBuffer::SetRawChannelContents(JSContext* aJSContext, uint32_t aChannel,
                                    float* aContents)
@@ -143,7 +215,8 @@ StealJSArrayDataIntoThreadSharedFloatArrayBufferList(JSContext* aJSContext,
   nsRefPtr<ThreadSharedFloatArrayBufferList> result =
     new ThreadSharedFloatArrayBufferList(aJSArrays.Length());
   for (uint32_t i = 0; i < aJSArrays.Length(); ++i) {
-    JS::RootedObject arrayBuffer(aJSContext, JS_GetArrayBufferViewBuffer(aJSArrays[i]));
+    JS::Rooted<JSObject*> arrayBuffer(aJSContext,
+                                      JS_GetArrayBufferViewBuffer(aJSArrays[i]));
     void* dataToFree = nullptr;
     uint8_t* stolenData = nullptr;
     if (arrayBuffer &&
@@ -202,5 +275,16 @@ AudioBuffer::MixToMono(JSContext* aJSContext)
   delete[] downmixBuffer;
 }
 
+size_t
+AudioBuffer::SizeOfIncludingThis(MallocSizeOf aMallocSizeOf) const
+{
+  size_t amount = aMallocSizeOf(this);
+  amount += mJSChannels.SizeOfExcludingThis(aMallocSizeOf);
+  if (mSharedChannels) {
+    amount += mSharedChannels->SizeOfExcludingThis(aMallocSizeOf);
+  }
+  return amount;
+}
+
 }
 }
