@@ -8,6 +8,7 @@
 #include "AudioNodeEngine.h"
 #include "AudioNodeStream.h"
 #include "AudioDestinationNode.h"
+#include "PlayingRefChangeHandler.h"
 #include "WebAudioUtils.h"
 #include "blink/Biquad.h"
 #include "mozilla/Preferences.h"
@@ -136,20 +137,52 @@ public:
     }
   }
 
-  virtual void ProduceAudioBlock(AudioNodeStream* aStream,
-                                 const AudioChunk& aInput,
-                                 AudioChunk* aOutput,
-                                 bool* aFinished) MOZ_OVERRIDE
+  virtual void ProcessBlock(AudioNodeStream* aStream,
+                            const AudioChunk& aInput,
+                            AudioChunk* aOutput,
+                            bool* aFinished) MOZ_OVERRIDE
   {
+    float inputBuffer[WEBAUDIO_BLOCK_SIZE];
+
     if (aInput.IsNull()) {
-      aOutput->SetNull(WEBAUDIO_BLOCK_SIZE);
-      return;
+      bool hasTail = false;
+      for (uint32_t i = 0; i < mBiquads.Length(); ++i) {
+        if (mBiquads[i].hasTail()) {
+          hasTail = true;
+          break;
+        }
+      }
+      if (!hasTail) {
+        if (!mBiquads.IsEmpty()) {
+          mBiquads.Clear();
+
+          nsRefPtr<PlayingRefChangeHandler> refchanged =
+            new PlayingRefChangeHandler(aStream, PlayingRefChangeHandler::RELEASE);
+          aStream->Graph()->
+            DispatchToMainThreadAfterStreamStateUpdate(refchanged.forget());
+        }
+
+        aOutput->SetNull(WEBAUDIO_BLOCK_SIZE);
+        return;
+      }
+
+      PodArrayZero(inputBuffer);
+
+    } else if(mBiquads.Length() != aInput.mChannelData.Length()){
+      if (mBiquads.IsEmpty()) {
+        nsRefPtr<PlayingRefChangeHandler> refchanged =
+          new PlayingRefChangeHandler(aStream, PlayingRefChangeHandler::ADDREF);
+        aStream->Graph()->
+          DispatchToMainThreadAfterStreamStateUpdate(refchanged.forget());
+      } else { // Help people diagnose bug 924718
+        NS_WARNING("BiquadFilterNode channel count changes may produce audio glitches");
+      }
+
+      // Adjust the number of biquads based on the number of channels
+      mBiquads.SetLength(aInput.mChannelData.Length());
     }
 
-    // Adjust the number of biquads based on the number of channels
-    const uint32_t numberOfChannels = aInput.mChannelData.Length();
-    mBiquads.SetLength(numberOfChannels);
-
+    uint32_t numberOfChannels = mBiquads.Length();
     AllocateAudioBlock(numberOfChannels, aOutput);
 
     TrackTicks pos = aStream->GetCurrentPosition();
@@ -159,12 +192,16 @@ public:
     double gain = mGain.GetValueAtTime(pos);
     double detune = mDetune.GetValueAtTime(pos);
 
-    float inputBuffer[WEBAUDIO_BLOCK_SIZE];
     for (uint32_t i = 0; i < numberOfChannels; ++i) {
-      auto input = static_cast<const float*>(aInput.mChannelData[i]);
-      if (aInput.mVolume != 1.0) {
-        AudioBlockCopyChannelWithScale(input, aInput.mVolume, inputBuffer);
+      const float* input;
+      if (aInput.IsNull()) {
         input = inputBuffer;
+      } else {
+        input = static_cast<const float*>(aInput.mChannelData[i]);
+        if (aInput.mVolume != 1.0) {
+          AudioBlockCopyChannelWithScale(input, aInput.mVolume, inputBuffer);
+          input = inputBuffer;
+        }
       }
       SetParamsOnBiquad(mBiquads[i], aStream->SampleRate(), mType, freq, q, gain, detune);
 
@@ -259,6 +296,10 @@ BiquadFilterNode::GetFrequencyResponse(const Float32Array& aFrequencyHz,
                                        const Float32Array& aMagResponse,
                                        const Float32Array& aPhaseResponse)
 {
+  aFrequencyHz.ComputeLengthAndData();
+  aMagResponse.ComputeLengthAndData();
+  aPhaseResponse.ComputeLengthAndData();
+
   uint32_t length = std::min(std::min(aFrequencyHz.Length(), aMagResponse.Length()),
                              aPhaseResponse.Length());
   if (!length) {
