@@ -46,7 +46,7 @@ public:
     , mRatio(12.f)
     , mAttack(0.003f)
     , mRelease(0.25f)
-    , mCompressor(new DynamicsCompressor(IdealAudioRate(), 2))
+    , mCompressor(new DynamicsCompressor(mDestination->SampleRate(), 2))
   {
   }
 
@@ -62,7 +62,9 @@ public:
     ATTACK,
     RELEASE
   };
-  void SetTimelineParameter(uint32_t aIndex, const AudioParamTimeline& aValue) MOZ_OVERRIDE
+  void SetTimelineParameter(uint32_t aIndex,
+                            const AudioParamTimeline& aValue,
+                            TrackRate aSampleRate) MOZ_OVERRIDE
   {
     MOZ_ASSERT(mSource && mDestination);
     switch (aIndex) {
@@ -91,10 +93,10 @@ public:
     }
   }
 
-  virtual void ProduceAudioBlock(AudioNodeStream* aStream,
-                                 const AudioChunk& aInput,
-                                 AudioChunk* aOutput,
-                                 bool* aFinished) MOZ_OVERRIDE
+  virtual void ProcessBlock(AudioNodeStream* aStream,
+                            const AudioChunk& aInput,
+                            AudioChunk* aOutput,
+                            bool* aFinished) MOZ_OVERRIDE
   {
     if (aInput.IsNull()) {
       // Just output silence
@@ -105,7 +107,7 @@ public:
     const uint32_t channelCount = aInput.mChannelData.Length();
     if (mCompressor->numberOfChannels() != channelCount) {
       // Create a new compressor object with a new channel count
-      mCompressor = new WebCore::DynamicsCompressor(IdealAudioRate(),
+      mCompressor = new WebCore::DynamicsCompressor(aStream->SampleRate(),
                                                     aInput.mChannelData.Length());
     }
 
@@ -185,12 +187,18 @@ DynamicsCompressorNode::DynamicsCompressorNode(AudioContext* aContext)
               2,
               ChannelCountMode::Explicit,
               ChannelInterpretation::Speakers)
-  , mThreshold(new AudioParam(this, SendThresholdToStream, -24.f))
-  , mKnee(new AudioParam(this, SendKneeToStream, 30.f))
-  , mRatio(new AudioParam(this, SendRatioToStream, 12.f))
-  , mReduction(new AudioParam(this, Callback, 0.f))
-  , mAttack(new AudioParam(this, SendAttackToStream, 0.003f))
-  , mRelease(new AudioParam(this, SendReleaseToStream, 0.25f))
+  , mThreshold(new AudioParam(MOZ_THIS_IN_INITIALIZER_LIST(),
+                              SendThresholdToStream, -24.f))
+  , mKnee(new AudioParam(MOZ_THIS_IN_INITIALIZER_LIST(),
+                         SendKneeToStream, 30.f))
+  , mRatio(new AudioParam(MOZ_THIS_IN_INITIALIZER_LIST(),
+                          SendRatioToStream, 12.f))
+  , mReduction(new AudioParam(MOZ_THIS_IN_INITIALIZER_LIST(),
+                              Callback, 0.f))
+  , mAttack(new AudioParam(MOZ_THIS_IN_INITIALIZER_LIST(),
+                           SendAttackToStream, 0.003f))
+  , mRelease(new AudioParam(MOZ_THIS_IN_INITIALIZER_LIST(),
+                            SendReleaseToStream, 0.25f))
 {
   DynamicsCompressorNodeEngine* engine = new DynamicsCompressorNodeEngine(this, aContext->Destination());
   mStream = aContext->Graph()->CreateAudioNodeStream(engine, MediaStreamGraph::INTERNAL_STREAM);
