@@ -24,22 +24,7 @@ namespace dom {
 // buffers.
 static const float MAX_LATENCY_S = 0.5;
 
-NS_IMPL_CYCLE_COLLECTION_CLASS(ScriptProcessorNode)
-
-NS_IMPL_CYCLE_COLLECTION_UNLINK_BEGIN(ScriptProcessorNode)
-  if (tmp->Context()) {
-    tmp->Context()->UnregisterScriptProcessorNode(tmp);
-  }
-NS_IMPL_CYCLE_COLLECTION_UNLINK_END_INHERITED(AudioNode)
-
-NS_IMPL_CYCLE_COLLECTION_TRAVERSE_BEGIN_INHERITED(ScriptProcessorNode, AudioNode)
-NS_IMPL_CYCLE_COLLECTION_TRAVERSE_END
-
-NS_INTERFACE_MAP_BEGIN_CYCLE_COLLECTION_INHERITED(ScriptProcessorNode)
-NS_INTERFACE_MAP_END_INHERITING(AudioNode)
-
-NS_IMPL_ADDREF_INHERITED(ScriptProcessorNode, AudioNode)
-NS_IMPL_RELEASE_INHERITED(ScriptProcessorNode, AudioNode)
+NS_IMPL_ISUPPORTS_INHERITED0(ScriptProcessorNode, AudioNode)
 
 // This class manages a queue of output buffers shared between
 // the main thread and the Media Stream Graph thread.
@@ -82,6 +67,13 @@ private:
       return front;
     }
 
+    // Empties the buffer queue.
+    void Clear()
+    {
+      mMutex.AssertCurrentThreadOwns();
+      mBufferList.clear();
+    }
+
   private:
     typedef std::deque<AudioChunk> BufferList;
 
@@ -119,15 +111,24 @@ public:
       // interval will be very short. |latency - bufferDuration| will be
       // negative, effectively moving back mLatency to a smaller and smaller
       // value, until it crosses zero, at which point we stop dropping buffers
-      // and resume normal operation.
+      // and resume normal operation. This does not work if at the same time,
+      // the MSG thread was also slowed down, so if the latency on the MSG
+      // thread is normal, and we are still dropping buffers, and mLatency is
+      // still more than twice the duration of a buffer, we reset it and stop
+      // dropping buffers.
       float latency = (now - mLastEventTime).ToSeconds();
       float bufferDuration = aBufferSize / mSampleRate;
       mLatency += latency - bufferDuration;
       mLastEventTime = now;
-      if (mLatency > MAX_LATENCY_S || (mDroppingBuffers && mLatency > 0.0)) {
+      if (mLatency > MAX_LATENCY_S ||
+          (mDroppingBuffers && mLatency > 0.0 &&
+           fabs(latency - bufferDuration) < bufferDuration)) {
         mDroppingBuffers = true;
         return;
       } else {
+        if (mDroppingBuffers) {
+          mLatency = 0;
+        }
         mDroppingBuffers = false;
       }
     }
@@ -182,6 +183,18 @@ public:
     return mDelaySoFar == TRACK_TICKS_MAX ? 0 : mDelaySoFar;
   }
 
+  void Reset()
+  {
+    MOZ_ASSERT(!NS_IsMainThread());
+    mDelaySoFar = TRACK_TICKS_MAX;
+    mLatency = 0.0f;
+    {
+      MutexAutoLock lock(mOutputQueue.Lock());
+      mOutputQueue.Clear();
+    }
+    mLastEventTime = TimeStamp();
+  }
+
 private:
   OutputQueue mOutputQueue;
   // How much delay we've seen so far.  This measures the amount of delay
@@ -226,10 +239,10 @@ public:
     mSource = aSource;
   }
 
-  virtual void ProduceAudioBlock(AudioNodeStream* aStream,
-                                 const AudioChunk& aInput,
-                                 AudioChunk* aOutput,
-                                 bool* aFinished) MOZ_OVERRIDE
+  virtual void ProcessBlock(AudioNodeStream* aStream,
+                            const AudioChunk& aInput,
+                            AudioChunk* aOutput,
+                            bool* aFinished) MOZ_OVERRIDE
   {
     MutexAutoLock lock(NodeMutex());
 
@@ -239,6 +252,18 @@ public:
       return;
     }
 
+    // This node is not connected to anything. Per spec, we don't fire the
+    // onaudioprocess event. We also want to clear out the input and output
+    // buffer queue, and output a null buffer.
+    if (!(aStream->ConsumerCount() ||
+          aStream->AsProcessedStream()->InputPortCount())) {
+      aOutput->SetNull(WEBAUDIO_BLOCK_SIZE);
+      mSharedBuffers->Reset();
+      mSeenNonSilenceInput = false;
+      mInputWriteIndex = 0;
+      return;
+    }
+
     // First, record our input buffer
     for (uint32_t i = 0; i < mInputChannels.Length(); ++i) {
       if (aInput.IsNull()) {
@@ -289,10 +314,9 @@ private:
     // Add the delay caused by the main thread
     playbackTick += mSharedBuffers->DelaySoFar();
     // Compute the playback time in the coordinate system of the destination
+    // FIXME: bug 970773
     double playbackTime =
-      WebAudioUtils::StreamPositionToDestinationTime(playbackTick,
-                                                     mSource,
-                                                     mDestination);
+      mSource->DestinationTimeFromTicks(mDestination, playbackTick);
 
     class Command : public nsRunnable
     {
@@ -340,10 +364,12 @@ private:
           // Create the input buffer
           nsRefPtr<AudioBuffer> inputBuffer;
           if (!mNullInput) {
-            inputBuffer = new AudioBuffer(node->Context(),
-                                          node->BufferSize(),
-                                          node->Context()->SampleRate());
-            if (!inputBuffer->InitializeBuffers(mInputChannels.Length(), cx)) {
+            ErrorResult rv;
+            inputBuffer =
+              AudioBuffer::Create(node->Context(), mInputChannels.Length(),
+                                  node->BufferSize(),
+                                  node->Context()->SampleRate(), cx, rv);
+            if (rv.Failed()) {
               return NS_OK;
             }
             // Put the channel data inside it
@@ -363,10 +389,18 @@ private:
                            mPlaybackTime);
           node->DispatchTrustedEvent(event);
 
-          // Steal the output buffers
+          // Steal the output buffers if they have been set.  Don't create a
+          // buffer if it hasn't been used to return output;
+          // FinishProducingOutputBuffer() will optimize output = null.
+          // GetThreadSharedChannelsForRate() may also return null after OOM.
           nsRefPtr<ThreadSharedFloatArrayBufferList> output;
           if (event->HasOutputBuffer()) {
-            output = event->OutputBuffer()->GetThreadSharedChannelsForRate(cx);
+            ErrorResult rv;
+            AudioBuffer* buffer = event->GetOutputBuffer(rv);
+            // HasOutputBuffer() returning true means that GetOutputBuffer()
+            // will not fail.
+            MOZ_ASSERT(!rv.Failed());
+            output = buffer->GetThreadSharedChannelsForRate(cx);
           }
 
           // Append it to our output buffer queue
@@ -424,9 +458,6 @@ ScriptProcessorNode::ScriptProcessorNode(AudioContext* aContext,
 
 ScriptProcessorNode::~ScriptProcessorNode()
 {
-  if (Context()) {
-    Context()->UnregisterScriptProcessorNode(this);
-  }
 }
 
 JSObject*
