@@ -11,6 +11,8 @@
 #include "WebMBufferedParser.h"
 #include "mozilla/dom/TimeRanges.h"
 #include "VorbisUtils.h"
+#include "gfx2DGlue.h"
+
 #include <algorithm>
 
 #define VPX_DONT_DEFINE_STDINT_TYPES
@@ -30,6 +32,7 @@ public:
 
 namespace mozilla {
 
+using namespace gfx;
 using namespace layers;
 
 // Un-comment to enable logging of seek bisections.
@@ -329,7 +332,7 @@ nsresult WebMReader::ReadMetadata(MediaInfo* aInfo,
       // that our video frame creation code doesn't overflow.
       nsIntSize displaySize(params.display_width, params.display_height);
       nsIntSize frameSize(params.width, params.height);
-      if (!VideoInfo::ValidateVideoRegion(frameSize, pictureRect, displaySize)) {
+      if (!IsValidVideoRegion(frameSize, pictureRect, displaySize)) {
         // Video track's frame sizes will overflow. Ignore the video track.
         continue;
       }
@@ -945,7 +948,7 @@ bool WebMReader::DecodeVideoFrame(bool &aKeyframeSkip,
       b.mPlanes[2].mWidth = (img->d_w + 1) >> img->x_chroma_shift;
       b.mPlanes[2].mOffset = b.mPlanes[2].mSkip = 0;
   
-      nsIntRect picture = mPicture;
+      IntRect picture = ToIntRect(mPicture);
       if (img->d_w != static_cast<uint32_t>(mInitialFrame.width) ||
           img->d_h != static_cast<uint32_t>(mInitialFrame.height)) {
         // Frame size is different from what the container reports. This is legal
