@@ -4,27 +4,24 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-#pragma once
+#ifndef mozilla_dom_SpeechRecognitionAlternative_h
+#define mozilla_dom_SpeechRecognitionAlternative_h
 
 #include "nsCycleCollectionParticipant.h"
 #include "nsString.h"
 #include "nsWrapperCache.h"
 #include "nsAutoPtr.h"
+#include "js/TypeDecls.h"
 
 #include "mozilla/Attributes.h"
 
-#include "EnableWebSpeechRecognitionCheck.h"
-
-struct JSContext;
-
 namespace mozilla {
 namespace dom {
 
 class SpeechRecognition;
 
 class SpeechRecognitionAlternative MOZ_FINAL : public nsISupports,
-                                               public nsWrapperCache,
-                                               public EnableWebSpeechRecognitionCheck
+                                               public nsWrapperCache
 {
 public:
   SpeechRecognitionAlternative(SpeechRecognition* aParent);
@@ -35,7 +32,8 @@ public:
 
   nsISupports* GetParentObject() const;
 
-  virtual JSObject* WrapObject(JSContext* aCx, JSObject* aScope);
+  virtual JSObject* WrapObject(JSContext* aCx,
+                               JS::Handle<JSObject*> aScope) MOZ_OVERRIDE;
 
   void GetTranscript(nsString& aRetVal) const;
 
@@ -49,3 +47,5 @@ private:
 
 } // namespace dom
 } // namespace mozilla
+
+#endif
