@@ -8,6 +8,12 @@
 #include "nsSpeechTask.h"
 #include "SpeechSynthesis.h"
 
+// GetCurrentTime is defined in winbase.h as zero argument macro forwarding to
+// GetTickCount() and conflicts with nsSpeechTask::GetCurrentTime().
+#ifdef GetCurrentTime
+#undef GetCurrentTime
+#endif
+
 #undef LOG
 #ifdef PR_LOGGING
 extern PRLogModuleInfo* GetSpeechSynthLog();
@@ -60,6 +66,11 @@ public:
     }
   }
 
+  virtual void NotifyRemoved(MediaStreamGraph* aGraph)
+  {
+    mSpeechTask = nullptr;
+  }
+
 private:
   // Raw pointer; if we exist, the stream exists,
   // and 'mSpeechTask' exclusively owns it and therefor exists as well.
@@ -145,7 +156,7 @@ nsSpeechTask::Setup(nsISpeechTaskCallback* aCallback,
 }
 
 NS_IMETHODIMP
-nsSpeechTask::SendAudio(const JS::Value& aData, const JS::Value& aLandmarks,
+nsSpeechTask::SendAudio(JS::Handle<JS::Value> aData, JS::Handle<JS::Value> aLandmarks,
                         JSContext* aCx)
 {
   MOZ_ASSERT(XRE_GetProcessType() == GeckoProcessType_Default);
@@ -162,7 +173,7 @@ nsSpeechTask::SendAudio(const JS::Value& aData, const JS::Value& aLandmarks,
   JS::Rooted<JSObject*> darray(aCx, &aData.toObject());
   JSAutoCompartment ac(aCx, darray);
 
-  JS::Rooted<JSObject*> tsrc(aCx, NULL);
+  JS::Rooted<JSObject*> tsrc(aCx, nullptr);
 
   // Allow either Int16Array or plain JS Array
   if (JS_IsInt16Array(darray)) {
@@ -204,8 +215,7 @@ void
 nsSpeechTask::SendAudioImpl(int16_t* aData, uint32_t aDataLen)
 {
   if (aDataLen == 0) {
-    // XXX: We should end the track too, an undetermined bug does not allow that.
-    mStream->Finish();
+    mStream->EndAllTrackAndFinish();
     return;
   }
 
@@ -283,10 +293,15 @@ nsSpeechTask::DispatchEndImpl(float aElapsedTime, uint32_t aCharIndex)
     mSpeechSynthesis->OnEnd(this);
   }
 
-  utterance->mState = SpeechSynthesisUtterance::STATE_ENDED;
-  utterance->DispatchSpeechSynthesisEvent(NS_LITERAL_STRING("end"),
-                                          aCharIndex, aElapsedTime,
-                                          NS_LITERAL_STRING(""));
+  if (utterance->mState == SpeechSynthesisUtterance::STATE_PENDING) {
+    utterance->mState = SpeechSynthesisUtterance::STATE_NONE;
+  } else {
+    utterance->mState = SpeechSynthesisUtterance::STATE_ENDED;
+    utterance->DispatchSpeechSynthesisEvent(NS_LITERAL_STRING("end"),
+                                            aCharIndex, aElapsedTime,
+                                            EmptyString());
+  }
+
   return NS_OK;
 }
 
