@@ -3,11 +3,16 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
+#include "mozilla/ArrayUtils.h" // MOZ_ARRAY_LENGTH
+
 #include "SVGPathSegUtils.h"
+
+#include "gfx2DGlue.h"
 #include "nsSVGPathDataParser.h"
 #include "nsTextFormatter.h"
 
 using namespace mozilla;
+using namespace mozilla::gfx;
 
 static const float PATH_SEG_LENGTH_TOLERANCE = 0.0000001f;
 static const uint32_t MAX_RECURSION = 10;
@@ -17,19 +22,21 @@ static const uint32_t MAX_RECURSION = 10;
 SVGPathSegUtils::GetValueAsString(const float* aSeg, nsAString& aValue)
 {
   // Adding new seg type? Is the formatting below acceptable for the new types?
-  PR_STATIC_ASSERT(NS_SVG_PATH_SEG_LAST_VALID_TYPE ==
-                     PATHSEG_CURVETO_QUADRATIC_SMOOTH_REL);
-  PR_STATIC_ASSERT(NS_SVG_PATH_SEG_MAX_ARGS == 7);
+  static_assert(NS_SVG_PATH_SEG_LAST_VALID_TYPE ==
+                PATHSEG_CURVETO_QUADRATIC_SMOOTH_REL,
+                "Update GetValueAsString for the new value.");
+  static_assert(NS_SVG_PATH_SEG_MAX_ARGS == 7,
+                "Add another case to the switch below.");
 
   uint32_t type = DecodeType(aSeg[0]);
-  PRUnichar typeAsChar = GetPathSegTypeAsLetter(type);
+  char16_t typeAsChar = GetPathSegTypeAsLetter(type);
 
   // Special case arcs:
   if (IsArcType(type)) {
     bool largeArcFlag = aSeg[4] != 0.0f;
     bool sweepFlag = aSeg[5] != 0.0f;
     nsTextFormatter::ssprintf(aValue,
-                              NS_LITERAL_STRING("%c%g,%g %g %d,%d %g,%g").get(),
+                              MOZ_UTF16("%c%g,%g %g %d,%d %g,%g"),
                               typeAsChar, aSeg[1], aSeg[2], aSeg[3],
                               largeArcFlag, sweepFlag, aSeg[6], aSeg[7]);
   } else {
@@ -40,30 +47,30 @@ SVGPathSegUtils::GetValueAsString(const float* aSeg, nsAString& aValue)
       break;
 
     case 1:
-      nsTextFormatter::ssprintf(aValue, NS_LITERAL_STRING("%c%g").get(),
+      nsTextFormatter::ssprintf(aValue, MOZ_UTF16("%c%g"),
                                 typeAsChar, aSeg[1]);
       break;
 
     case 2:
-      nsTextFormatter::ssprintf(aValue, NS_LITERAL_STRING("%c%g,%g").get(),
+      nsTextFormatter::ssprintf(aValue, MOZ_UTF16("%c%g,%g"),
                                 typeAsChar, aSeg[1], aSeg[2]);
       break;
 
     case 4:
-      nsTextFormatter::ssprintf(aValue, NS_LITERAL_STRING("%c%g,%g %g,%g").get(),
+      nsTextFormatter::ssprintf(aValue, MOZ_UTF16("%c%g,%g %g,%g"),
                                 typeAsChar, aSeg[1], aSeg[2], aSeg[3], aSeg[4]);
       break;
 
     case 6:
       nsTextFormatter::ssprintf(aValue,
-                                NS_LITERAL_STRING("%c%g,%g %g,%g %g,%g").get(),
+                                MOZ_UTF16("%c%g,%g %g,%g %g,%g"),
                                 typeAsChar, aSeg[1], aSeg[2], aSeg[3], aSeg[4],
                                 aSeg[5], aSeg[6]);
       break;
 
     default:
       NS_ABORT_IF_FALSE(false, "Unknown segment type");
-      aValue = NS_LITERAL_STRING("<unknown-segment-type>").get();
+      aValue = MOZ_UTF16("<unknown-segment-type>");
       return;
     }
   }
@@ -73,21 +80,21 @@ SVGPathSegUtils::GetValueAsString(const float* aSeg, nsAString& aValue)
   // of the output string is one too long. We need to manually remove that '\0'
   // until nsTextFormatter is fixed.
   //
-  if (aValue[aValue.Length() - 1] == PRUnichar('\0')) {
+  if (aValue[aValue.Length() - 1] == char16_t('\0')) {
     aValue.SetLength(aValue.Length() - 1);
   }
 }
 
 
 static float
-CalcDistanceBetweenPoints(const gfxPoint& aP1, const gfxPoint& aP2)
+CalcDistanceBetweenPoints(const Point& aP1, const Point& aP2)
 {
   return NS_hypot(aP2.x - aP1.x, aP2.y - aP1.y);
 }
 
 
 static void
-SplitQuadraticBezier(const gfxPoint* aCurve, gfxPoint* aLeft, gfxPoint* aRight)
+SplitQuadraticBezier(const Point* aCurve, Point* aLeft, Point* aRight)
 {
   aLeft[0].x = aCurve[0].x;
   aLeft[0].y = aCurve[0].y;
@@ -102,9 +109,9 @@ SplitQuadraticBezier(const gfxPoint* aCurve, gfxPoint* aLeft, gfxPoint* aRight)
 }
 
 static void
-SplitCubicBezier(const gfxPoint* aCurve, gfxPoint* aLeft, gfxPoint* aRight)
+SplitCubicBezier(const Point* aCurve, Point* aLeft, Point* aRight)
 {
-  gfxPoint tmp;
+  Point tmp;
   tmp.x = (aCurve[1].x + aCurve[2].x) / 4;
   tmp.y = (aCurve[1].y + aCurve[2].y) / 4;
   aLeft[0].x = aCurve[0].x;
@@ -124,12 +131,12 @@ SplitCubicBezier(const gfxPoint* aCurve, gfxPoint* aLeft, gfxPoint* aRight)
 }
 
 static gfxFloat
-CalcBezLengthHelper(gfxPoint* aCurve, uint32_t aNumPts,
+CalcBezLengthHelper(const Point* aCurve, uint32_t aNumPts,
                     uint32_t aRecursionCount,
-                    void (*aSplit)(const gfxPoint*, gfxPoint*, gfxPoint*))
+                    void (*aSplit)(const Point*, Point*, Point*))
 {
-  gfxPoint left[4];
-  gfxPoint right[4];
+  Point left[4];
+  Point right[4];
   gfxFloat length = 0, dist;
   for (uint32_t i = 0; i < aNumPts - 1; i++) {
     length += CalcDistanceBetweenPoints(aCurve[i], aCurve[i+1]);
@@ -146,18 +153,18 @@ CalcBezLengthHelper(gfxPoint* aCurve, uint32_t aNumPts,
 }
 
 static inline gfxFloat
-CalcLengthOfCubicBezier(const gfxPoint& aPos, const gfxPoint &aCP1,
-                        const gfxPoint& aCP2, const gfxPoint &aTo)
+CalcLengthOfCubicBezier(const Point& aPos, const Point &aCP1,
+                        const Point& aCP2, const Point &aTo)
 {
-  gfxPoint curve[4] = { aPos, aCP1, aCP2, aTo };
+  Point curve[4] = { aPos, aCP1, aCP2, aTo };
   return CalcBezLengthHelper(curve, 4, 0, SplitCubicBezier);
 }
 
 static inline gfxFloat
-CalcLengthOfQuadraticBezier(const gfxPoint& aPos, const gfxPoint& aCP,
-                            const gfxPoint& aTo)
+CalcLengthOfQuadraticBezier(const Point& aPos, const Point& aCP,
+                            const Point& aTo)
 {
-  gfxPoint curve[3] = { aPos, aCP, aTo };
+  Point curve[3] = { aPos, aCP, aTo };
   return CalcBezLengthHelper(curve, 3, 0, SplitQuadraticBezier);
 }
 
@@ -175,7 +182,7 @@ TraverseClosePath(const float* aArgs, SVGPathTraversalState& aState)
 static void
 TraverseMovetoAbs(const float* aArgs, SVGPathTraversalState& aState)
 {
-  aState.start = aState.pos = gfxPoint(aArgs[0], aArgs[1]);
+  aState.start = aState.pos = Point(aArgs[0], aArgs[1]);
   if (aState.ShouldUpdateLengthAndControlPoints()) {
     // aState.length is unchanged, since move commands don't affect path length.
     aState.cp1 = aState.cp2 = aState.start;
@@ -185,7 +192,7 @@ TraverseMovetoAbs(const float* aArgs, SVGPathTraversalState& aState)
 static void
 TraverseMovetoRel(const float* aArgs, SVGPathTraversalState& aState)
 {
-  aState.start = aState.pos += gfxPoint(aArgs[0], aArgs[1]);
+  aState.start = aState.pos += Point(aArgs[0], aArgs[1]);
   if (aState.ShouldUpdateLengthAndControlPoints()) {
     // aState.length is unchanged, since move commands don't affect path length.
     aState.cp1 = aState.cp2 = aState.start;
@@ -195,7 +202,7 @@ TraverseMovetoRel(const float* aArgs, SVGPathTraversalState& aState)
 static void
 TraverseLinetoAbs(const float* aArgs, SVGPathTraversalState& aState)
 {
-  gfxPoint to(aArgs[0], aArgs[1]);
+  Point to(aArgs[0], aArgs[1]);
   if (aState.ShouldUpdateLengthAndControlPoints()) {
     aState.length += CalcDistanceBetweenPoints(aState.pos, to);
     aState.cp1 = aState.cp2 = to;
@@ -206,7 +213,7 @@ TraverseLinetoAbs(const float* aArgs, SVGPathTraversalState& aState)
 static void
 TraverseLinetoRel(const float* aArgs, SVGPathTraversalState& aState)
 {
-  gfxPoint to = aState.pos + gfxPoint(aArgs[0], aArgs[1]);
+  Point to = aState.pos + Point(aArgs[0], aArgs[1]);
   if (aState.ShouldUpdateLengthAndControlPoints()) {
     aState.length += CalcDistanceBetweenPoints(aState.pos, to);
     aState.cp1 = aState.cp2 = to;
@@ -217,7 +224,7 @@ TraverseLinetoRel(const float* aArgs, SVGPathTraversalState& aState)
 static void
 TraverseLinetoHorizontalAbs(const float* aArgs, SVGPathTraversalState& aState)
 {
-  gfxPoint to(aArgs[0], aState.pos.y);
+  Point to(aArgs[0], aState.pos.y);
   if (aState.ShouldUpdateLengthAndControlPoints()) {
     aState.length += fabs(to.x - aState.pos.x);
     aState.cp1 = aState.cp2 = to;
@@ -238,7 +245,7 @@ TraverseLinetoHorizontalRel(const float* aArgs, SVGPathTraversalState& aState)
 static void
 TraverseLinetoVerticalAbs(const float* aArgs, SVGPathTraversalState& aState)
 {
-  gfxPoint to(aState.pos.x, aArgs[0]);
+  Point to(aState.pos.x, aArgs[0]);
   if (aState.ShouldUpdateLengthAndControlPoints()) {
     aState.length += fabs(to.y - aState.pos.y);
     aState.cp1 = aState.cp2 = to;
@@ -259,10 +266,10 @@ TraverseLinetoVerticalRel(const float* aArgs, SVGPathTraversalState& aState)
 static void
 TraverseCurvetoCubicAbs(const float* aArgs, SVGPathTraversalState& aState)
 {
-  gfxPoint to(aArgs[4], aArgs[5]);
+  Point to(aArgs[4], aArgs[5]);
   if (aState.ShouldUpdateLengthAndControlPoints()) {
-    gfxPoint cp1(aArgs[0], aArgs[1]);
-    gfxPoint cp2(aArgs[2], aArgs[3]);
+    Point cp1(aArgs[0], aArgs[1]);
+    Point cp2(aArgs[2], aArgs[3]);
     aState.length += (float)CalcLengthOfCubicBezier(aState.pos, cp1, cp2, to);
     aState.cp2 = cp2;
     aState.cp1 = to;
@@ -273,10 +280,10 @@ TraverseCurvetoCubicAbs(const float* aArgs, SVGPathTraversalState& aState)
 static void
 TraverseCurvetoCubicSmoothAbs(const float* aArgs, SVGPathTraversalState& aState)
 {
-  gfxPoint to(aArgs[2], aArgs[3]);
+  Point to(aArgs[2], aArgs[3]);
   if (aState.ShouldUpdateLengthAndControlPoints()) {
-    gfxPoint cp1 = aState.pos - (aState.cp2 - aState.pos);
-    gfxPoint cp2(aArgs[0], aArgs[1]);
+    Point cp1 = aState.pos - (aState.cp2 - aState.pos);
+    Point cp2(aArgs[0], aArgs[1]);
     aState.length += (float)CalcLengthOfCubicBezier(aState.pos, cp1, cp2, to);
     aState.cp2 = cp2;
     aState.cp1 = to;
@@ -287,10 +294,10 @@ TraverseCurvetoCubicSmoothAbs(const float* aArgs, SVGPathTraversalState& aState)
 static void
 TraverseCurvetoCubicRel(const float* aArgs, SVGPathTraversalState& aState)
 {
-  gfxPoint to = aState.pos + gfxPoint(aArgs[4], aArgs[5]);
+  Point to = aState.pos + Point(aArgs[4], aArgs[5]);
   if (aState.ShouldUpdateLengthAndControlPoints()) {
-    gfxPoint cp1 = aState.pos + gfxPoint(aArgs[0], aArgs[1]);
-    gfxPoint cp2 = aState.pos + gfxPoint(aArgs[2], aArgs[3]);
+    Point cp1 = aState.pos + Point(aArgs[0], aArgs[1]);
+    Point cp2 = aState.pos + Point(aArgs[2], aArgs[3]);
     aState.length += (float)CalcLengthOfCubicBezier(aState.pos, cp1, cp2, to);
     aState.cp2 = cp2;
     aState.cp1 = to;
@@ -301,10 +308,10 @@ TraverseCurvetoCubicRel(const float* aArgs, SVGPathTraversalState& aState)
 static void
 TraverseCurvetoCubicSmoothRel(const float* aArgs, SVGPathTraversalState& aState)
 {
-  gfxPoint to = aState.pos + gfxPoint(aArgs[2], aArgs[3]);
+  Point to = aState.pos + Point(aArgs[2], aArgs[3]);
   if (aState.ShouldUpdateLengthAndControlPoints()) {
-    gfxPoint cp1 = aState.pos - (aState.cp2 - aState.pos);
-    gfxPoint cp2 = aState.pos + gfxPoint(aArgs[0], aArgs[1]);
+    Point cp1 = aState.pos - (aState.cp2 - aState.pos);
+    Point cp2 = aState.pos + Point(aArgs[0], aArgs[1]);
     aState.length += (float)CalcLengthOfCubicBezier(aState.pos, cp1, cp2, to);
     aState.cp2 = cp2;
     aState.cp1 = to;
@@ -315,9 +322,9 @@ TraverseCurvetoCubicSmoothRel(const float* aArgs, SVGPathTraversalState& aState)
 static void
 TraverseCurvetoQuadraticAbs(const float* aArgs, SVGPathTraversalState& aState)
 {
-  gfxPoint to(aArgs[2], aArgs[3]);
+  Point to(aArgs[2], aArgs[3]);
   if (aState.ShouldUpdateLengthAndControlPoints()) {
-    gfxPoint cp(aArgs[0], aArgs[1]);
+    Point cp(aArgs[0], aArgs[1]);
     aState.length += (float)CalcLengthOfQuadraticBezier(aState.pos, cp, to);
     aState.cp1 = cp;
     aState.cp2 = to;
@@ -329,9 +336,9 @@ static void
 TraverseCurvetoQuadraticSmoothAbs(const float* aArgs,
                                   SVGPathTraversalState& aState)
 {
-  gfxPoint to(aArgs[0], aArgs[1]);
+  Point to(aArgs[0], aArgs[1]);
   if (aState.ShouldUpdateLengthAndControlPoints()) {
-    gfxPoint cp = aState.pos - (aState.cp1 - aState.pos);
+    Point cp = aState.pos - (aState.cp1 - aState.pos);
     aState.length += (float)CalcLengthOfQuadraticBezier(aState.pos, cp, to);
     aState.cp1 = cp;
     aState.cp2 = to;
@@ -342,9 +349,9 @@ TraverseCurvetoQuadraticSmoothAbs(const float* aArgs,
 static void
 TraverseCurvetoQuadraticRel(const float* aArgs, SVGPathTraversalState& aState)
 {
-  gfxPoint to = aState.pos + gfxPoint(aArgs[2], aArgs[3]);
+  Point to = aState.pos + Point(aArgs[2], aArgs[3]);
   if (aState.ShouldUpdateLengthAndControlPoints()) {
-    gfxPoint cp = aState.pos + gfxPoint(aArgs[0], aArgs[1]);
+    Point cp = aState.pos + Point(aArgs[0], aArgs[1]);
     aState.length += (float)CalcLengthOfQuadraticBezier(aState.pos, cp, to);
     aState.cp1 = cp;
     aState.cp2 = to;
@@ -356,9 +363,9 @@ static void
 TraverseCurvetoQuadraticSmoothRel(const float* aArgs,
                                   SVGPathTraversalState& aState)
 {
-  gfxPoint to = aState.pos + gfxPoint(aArgs[0], aArgs[1]);
+  Point to = aState.pos + Point(aArgs[0], aArgs[1]);
   if (aState.ShouldUpdateLengthAndControlPoints()) {
-    gfxPoint cp = aState.pos - (aState.cp1 - aState.pos);
+    Point cp = aState.pos - (aState.cp1 - aState.pos);
     aState.length += (float)CalcLengthOfQuadraticBezier(aState.pos, cp, to);
     aState.cp1 = cp;
     aState.cp2 = to;
@@ -369,12 +376,11 @@ TraverseCurvetoQuadraticSmoothRel(const float* aArgs,
 static void
 TraverseArcAbs(const float* aArgs, SVGPathTraversalState& aState)
 {
-  gfxPoint to(aArgs[5], aArgs[6]);
+  Point to(aArgs[5], aArgs[6]);
   if (aState.ShouldUpdateLengthAndControlPoints()) {
     float dist = 0;
-    gfxPoint radii(aArgs[0], aArgs[1]);
-    gfxPoint bez[4] = { aState.pos, gfxPoint(0, 0),
-                        gfxPoint(0, 0), gfxPoint(0, 0) };
+    Point radii(aArgs[0], aArgs[1]);
+    Point bez[4] = { aState.pos, Point(0, 0), Point(0, 0), Point(0, 0) };
     nsSVGArcConverter converter(aState.pos, to, radii, aArgs[2],
                                 aArgs[3] != 0, aArgs[4] != 0);
     while (converter.GetNextSegment(&bez[1], &bez[2], &bez[3])) {
@@ -390,12 +396,11 @@ TraverseArcAbs(const float* aArgs, SVGPathTraversalState& aState)
 static void
 TraverseArcRel(const float* aArgs, SVGPathTraversalState& aState)
 {
-  gfxPoint to = aState.pos + gfxPoint(aArgs[5], aArgs[6]);
+  Point to = aState.pos + Point(aArgs[5], aArgs[6]);
   if (aState.ShouldUpdateLengthAndControlPoints()) {
     float dist = 0;
-    gfxPoint radii(aArgs[0], aArgs[1]);
-    gfxPoint bez[4] = { aState.pos, gfxPoint(0, 0),
-                        gfxPoint(0, 0), gfxPoint(0, 0) };
+    Point radii(aArgs[0], aArgs[1]);
+    Point bez[4] = { aState.pos, Point(0, 0), Point(0, 0), Point(0, 0) };
     nsSVGArcConverter converter(aState.pos, to, radii, aArgs[2],
                                 aArgs[3] != 0, aArgs[4] != 0);
     while (converter.GetNextSegment(&bez[1], &bez[2], &bez[3])) {
@@ -438,8 +443,9 @@ static TraverseFunc gTraverseFuncTable[NS_SVG_PATH_SEG_TYPE_COUNT] = {
 SVGPathSegUtils::TraversePathSegment(const float* aData,
                                      SVGPathTraversalState& aState)
 {
-  PR_STATIC_ASSERT(NS_ARRAY_LENGTH(gTraverseFuncTable) ==
-                     NS_SVG_PATH_SEG_TYPE_COUNT);
+  static_assert(MOZ_ARRAY_LENGTH(gTraverseFuncTable) ==
+                NS_SVG_PATH_SEG_TYPE_COUNT,
+                "gTraverseFuncTable is out of date");
   uint32_t type = DecodeType(aData[0]);
   gTraverseFuncTable[type](aData + 1, aState);
 }
