@@ -5,12 +5,14 @@
 
 #include "nsXMLElement.h"
 #include "mozilla/dom/ElementBinding.h"
+#include "mozilla/dom/ElementInlines.h"
 #include "nsContentUtils.h" // nsAutoScriptBlocker
 
 using namespace mozilla::dom;
 
 nsresult
-NS_NewXMLElement(nsIContent** aInstancePtrResult, already_AddRefed<nsINodeInfo> aNodeInfo)
+NS_NewXMLElement(Element** aInstancePtrResult,
+                 already_AddRefed<nsINodeInfo>&& aNodeInfo)
 {
   nsXMLElement* it = new nsXMLElement(aNodeInfo);
   NS_ADDREF(*aInstancePtrResult = it);
@@ -82,7 +84,7 @@ nsXMLElement::NodeInfoChanged(nsINodeInfo* aOldNodeInfo)
     const nsAttrValue* attrVal =
       mAttrsAndChildren.GetAttr(aOldNodeInfo->GetIDAttributeAtom());
     if (attrVal) {
-      doc->RemoveFromIdTable(this, attrVal->GetAtomValue());
+      RemoveFromIdTable(attrVal->GetAtomValue());
     }
   }
   
@@ -102,7 +104,7 @@ nsXMLElement::NodeInfoChanged(nsINodeInfo* aOldNodeInfo)
       NS_ASSERTION(attrVal->Type() == nsAttrValue::eAtom,
                    "Should be atom by now");
       if (doc) {
-        doc->AddToIdTable(this, attrVal->GetAtomValue());
+        AddToIdTable(attrVal->GetAtomValue());
       }
     }
   }
@@ -131,28 +133,3 @@ nsXMLElement::ParseAttribute(int32_t aNamespaceID,
 
   return false;
 }
-
-nsresult
-nsXMLElement::BindToTree(nsIDocument* aDocument, nsIContent* aParent,
-                         nsIContent* aBindingParent,
-                         bool aCompileEventHandlers)
-{
-  nsresult rv = Element::BindToTree(aDocument, aParent,
-                                    aBindingParent,
-                                    aCompileEventHandlers);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  if (aDocument && HasID() && !GetBindingParent()) {
-    aDocument->AddToIdTable(this, DoGetID());
-  }
-
-  return NS_OK;
-}
-
-void
-nsXMLElement::UnbindFromTree(bool aDeep, bool aNullParent)
-{
-  RemoveFromIdTable();
-
-  return Element::UnbindFromTree(aDeep, aNullParent);
-}
