@@ -79,6 +79,7 @@ Crypto::GetRandomValues(JSContext* aCx, const ArrayBufferView& aArray,
       return nullptr;
   }
 
+  aArray.ComputeLengthAndData();
   uint32_t dataLen = aArray.Length();
   if (dataLen == 0) {
     NS_WARNING("ArrayBufferView length is 0, cannot continue");
@@ -150,7 +151,7 @@ Crypto::GetVersion(nsString& aVersion)
 {
 }
 
-already_AddRefed<nsIDOMCRMFObject>
+mozilla::dom::CRMFObject*
 Crypto::GenerateCRMFRequest(JSContext* aContext,
                             const nsCString& aReqDN,
                             const nsCString& aRegToken,
