@@ -8,7 +8,6 @@
 #include "BatteryManager.h"
 #include "nsIDOMClassInfo.h"
 #include "Constants.h"
-#include "nsDOMEvent.h"
 #include "mozilla/Preferences.h"
 #include "nsDOMEventTargetHelper.h"
 #include "mozilla/dom/BatteryManagerBinding.h"
@@ -26,19 +25,17 @@ namespace mozilla {
 namespace dom {
 namespace battery {
 
-BatteryManager::BatteryManager()
-  : mLevel(kDefaultLevel)
+BatteryManager::BatteryManager(nsPIDOMWindow* aWindow)
+  : nsDOMEventTargetHelper(aWindow)
+  , mLevel(kDefaultLevel)
   , mCharging(kDefaultCharging)
   , mRemainingTime(kDefaultRemainingTime)
 {
-  SetIsDOMBinding();
 }
 
 void
-BatteryManager::Init(nsPIDOMWindow *aWindow)
+BatteryManager::Init()
 {
-  BindToOwner(aWindow);
-
   hal::RegisterBatteryObserver(this);
 
   hal::BatteryInformation batteryInfo;
@@ -54,7 +51,7 @@ BatteryManager::Shutdown()
 }
 
 JSObject*
-BatteryManager::WrapObject(JSContext* aCx, JSObject* aScope)
+BatteryManager::WrapObject(JSContext* aCx, JS::Handle<JSObject*> aScope)
 {
   return BatteryManagerBinding::Wrap(aCx, aScope, this);
 }
