@@ -5,20 +5,18 @@
  * You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "base/basictypes.h"
-#include "GeneratedEvents.h"
 #include "nsCxPusher.h"
 #include "nsDOMClassInfo.h"
-#include "nsIDOMBluetoothDeviceEvent.h"
-#include "nsIDOMBluetoothStatusChangedEvent.h"
 #include "nsTArrayHelpers.h"
 #include "DOMRequest.h"
 #include "nsThreadUtils.h"
 
 #include "mozilla/dom/bluetooth/BluetoothTypes.h"
 #include "mozilla/dom/BluetoothAdapterBinding.h"
+#include "mozilla/dom/BluetoothDeviceEvent.h"
+#include "mozilla/dom/BluetoothStatusChangedEvent.h"
 #include "mozilla/dom/ContentChild.h"
 #include "mozilla/LazyIdleThread.h"
-#include "mozilla/Util.h"
 
 #include "BluetoothAdapter.h"
 #include "BluetoothDevice.h"
@@ -82,7 +80,6 @@ public:
       v.get_ArrayOfBluetoothNamedValue();
 
     nsTArray<nsRefPtr<BluetoothDevice> > devices;
-    JSObject* JsDevices;
     for (uint32_t i = 0; i < values.Length(); i++) {
       const BluetoothValue properties = values[i].value();
       if (properties.type() != BluetoothValue::TArrayOfBluetoothNamedValue) {
@@ -106,6 +103,7 @@ public:
     }
 
     AutoPushJSContext cx(sc->GetNativeContext());
+    JSObject* JsDevices = nullptr;
     rv = nsTArrayToJSArray(cx, devices, &JsDevices);
     if (!JsDevices) {
       BT_WARNING("Cannot create JS array!");
@@ -248,6 +246,7 @@ BluetoothAdapter::SetPropertyByValue(const BluetoothNamedValue& aValue)
     nsresult rv;
     nsIScriptContext* sc = GetContextForEventHandlers(&rv);
     NS_ENSURE_SUCCESS_VOID(rv);
+    NS_ENSURE_TRUE_VOID(sc);
 
     AutoPushJSContext cx(sc->GetNativeContext());
     JS::Rooted<JSObject*> uuids(cx);
@@ -260,14 +259,10 @@ BluetoothAdapter::SetPropertyByValue(const BluetoothNamedValue& aValue)
   } else if (name.EqualsLiteral("Devices")) {
     mDeviceAddresses = value.get_ArrayOfnsString();
 
-    uint32_t length = mDeviceAddresses.Length();
-    for (int i = 0; i < length; i++) {
-      mDeviceAddresses[i] = GetAddressFromObjectPath(mDeviceAddresses[i]);
-    }
-
     nsresult rv;
     nsIScriptContext* sc = GetContextForEventHandlers(&rv);
     NS_ENSURE_SUCCESS_VOID(rv);
+    NS_ENSURE_TRUE_VOID(sc);
 
     AutoPushJSContext cx(sc->GetNativeContext());
     JS::Rooted<JSObject*> deviceAddresses(cx);
@@ -309,20 +304,23 @@ BluetoothAdapter::Notify(const BluetoothSignal& aData)
   BluetoothValue v = aData.value();
   if (aData.name().EqualsLiteral("DeviceFound")) {
     nsRefPtr<BluetoothDevice> device = BluetoothDevice::Create(GetOwner(), mPath, aData.value());
-    nsCOMPtr<nsIDOMEvent> event;
-    NS_NewDOMBluetoothDeviceEvent(getter_AddRefs(event), this, nullptr, nullptr);
 
-    nsCOMPtr<nsIDOMBluetoothDeviceEvent> e = do_QueryInterface(event);
-    e->InitBluetoothDeviceEvent(NS_LITERAL_STRING("devicefound"),
-                                false, false, device);
+    BluetoothDeviceEventInit init;
+    init.mBubbles = false;
+    init.mCancelable = false;
+    init.mDevice = device;
+    nsRefPtr<BluetoothDeviceEvent> event =
+      BluetoothDeviceEvent::Constructor(this, NS_LITERAL_STRING("devicefound"), init);
     DispatchTrustedEvent(event);
   } else if (aData.name().EqualsLiteral("PropertyChanged")) {
     MOZ_ASSERT(v.type() == BluetoothValue::TArrayOfBluetoothNamedValue);
+
     const InfallibleTArray<BluetoothNamedValue>& arr =
       v.get_ArrayOfBluetoothNamedValue();
 
-    MOZ_ASSERT(arr.Length() == 1);
-    SetPropertyByValue(arr[0]);
+    for (uint32_t i = 0, propCount = arr.Length(); i < propCount; ++i) {
+      SetPropertyByValue(arr[i]);
+    }
   } else if (aData.name().EqualsLiteral(PAIRED_STATUS_CHANGED_ID) ||
              aData.name().EqualsLiteral(HFP_STATUS_CHANGED_ID) ||
              aData.name().EqualsLiteral(SCO_STATUS_CHANGED_ID) ||
@@ -337,13 +335,13 @@ BluetoothAdapter::Notify(const BluetoothSignal& aData)
     nsString address = arr[0].value().get_nsString();
     bool status = arr[1].value().get_bool();
 
-    nsCOMPtr<nsIDOMEvent> event;
-    NS_NewDOMBluetoothStatusChangedEvent(
-      getter_AddRefs(event), this, nullptr, nullptr);
-
-    nsCOMPtr<nsIDOMBluetoothStatusChangedEvent> e = do_QueryInterface(event);
-    e->InitBluetoothStatusChangedEvent(aData.name(), false, false,
-                                       address, status);
+    BluetoothStatusChangedEventInit init;
+    init.mBubbles = false;
+    init.mCancelable = false;
+    init.mAddress = address;
+    init.mStatus = status;
+    nsRefPtr<BluetoothStatusChangedEvent> event =
+      BluetoothStatusChangedEvent::Constructor(this, aData.name(), init);
     DispatchTrustedEvent(event);
   } else if (aData.name().EqualsLiteral(REQUEST_MEDIA_PLAYSTATUS_ID)) {
     nsCOMPtr<nsIDOMEvent> event;
@@ -529,7 +527,7 @@ BluetoothAdapter::GetPairedDevices(ErrorResult& aRv)
 }
 
 already_AddRefed<DOMRequest>
-BluetoothAdapter::PairUnpair(bool aPair, BluetoothDevice& aDevice,
+BluetoothAdapter::PairUnpair(bool aPair, const nsAString& aDeviceAddress,
                              ErrorResult& aRv)
 {
   nsCOMPtr<nsPIDOMWindow> win = GetOwner();
@@ -542,9 +540,6 @@ BluetoothAdapter::PairUnpair(bool aPair, BluetoothDevice& aDevice,
   nsRefPtr<BluetoothVoidReplyRunnable> results =
     new BluetoothVoidReplyRunnable(request);
 
-  nsAutoString addr;
-  aDevice.GetAddress(addr);
-
   BluetoothService* bs = BluetoothService::Get();
   if (!bs) {
     aRv.Throw(NS_ERROR_FAILURE);
@@ -552,11 +547,11 @@ BluetoothAdapter::PairUnpair(bool aPair, BluetoothDevice& aDevice,
   }
   nsresult rv;
   if (aPair) {
-    rv = bs->CreatePairedDeviceInternal(addr,
+    rv = bs->CreatePairedDeviceInternal(aDeviceAddress,
                                         kCreatePairedDeviceTimeout,
                                         results);
   } else {
-    rv = bs->RemoveDeviceInternal(addr, results);
+    rv = bs->RemoveDeviceInternal(aDeviceAddress, results);
   }
   if (NS_FAILED(rv)) {
     BT_WARNING("Pair/Unpair failed!");
@@ -568,15 +563,15 @@ BluetoothAdapter::PairUnpair(bool aPair, BluetoothDevice& aDevice,
 }
 
 already_AddRefed<DOMRequest>
-BluetoothAdapter::Pair(BluetoothDevice& aDevice, ErrorResult& aRv)
+BluetoothAdapter::Pair(const nsAString& aDeviceAddress, ErrorResult& aRv)
 {
-  return PairUnpair(true, aDevice, aRv);
+  return PairUnpair(true, aDeviceAddress, aRv);
 }
 
 already_AddRefed<DOMRequest>
-BluetoothAdapter::Unpair(BluetoothDevice& aDevice, ErrorResult& aRv)
+BluetoothAdapter::Unpair(const nsAString& aDeviceAddress, ErrorResult& aRv)
 {
-  return PairUnpair(false, aDevice, aRv);
+  return PairUnpair(false, aDeviceAddress, aRv);
 }
 
 already_AddRefed<DOMRequest>
@@ -744,20 +739,30 @@ BluetoothAdapter::SendFile(const nsAString& aDeviceAddress,
   nsRefPtr<BluetoothVoidReplyRunnable> results =
     new BluetoothVoidReplyRunnable(request);
 
-  BlobChild* actor =
-    ContentChild::GetSingleton()->GetOrCreateActorForBlob(aBlob);
-  if (!actor) {
-    BT_WARNING("Can't create actor");
-    aRv.Throw(NS_ERROR_FAILURE);
-    return nullptr;
-  }
-
   BluetoothService* bs = BluetoothService::Get();
   if (!bs) {
     aRv.Throw(NS_ERROR_FAILURE);
     return nullptr;
   }
-  bs->SendFile(aDeviceAddress, nullptr, actor, results);
+
+  if (XRE_GetProcessType() == GeckoProcessType_Default) {
+    // In-process transfer
+    bs->SendFile(aDeviceAddress, aBlob, results);
+  } else {
+    ContentChild *cc = ContentChild::GetSingleton();
+    if (!cc) {
+      aRv.Throw(NS_ERROR_FAILURE);
+      return nullptr;
+    }
+
+    BlobChild* actor = cc->GetOrCreateActorForBlob(aBlob);
+    if (!actor) {
+      aRv.Throw(NS_ERROR_FAILURE);
+      return nullptr;
+    }
+
+    bs->SendFile(aDeviceAddress, nullptr, actor, results);
+  }
 
   return request.forget();
 }
@@ -881,6 +886,7 @@ BluetoothAdapter::IsScoConnected(ErrorResult& aRv)
 already_AddRefed<DOMRequest>
 BluetoothAdapter::AnswerWaitingCall(ErrorResult& aRv)
 {
+#ifdef MOZ_B2G_RIL
   nsCOMPtr<nsPIDOMWindow> win = GetOwner();
   if (!win) {
     aRv.Throw(NS_ERROR_FAILURE);
@@ -899,11 +905,16 @@ BluetoothAdapter::AnswerWaitingCall(ErrorResult& aRv)
   bs->AnswerWaitingCall(results);
 
   return request.forget();
+#else
+  aRv.Throw(NS_ERROR_NOT_IMPLEMENTED);
+  return nullptr;
+#endif // MOZ_B2G_RIL
 }
 
 already_AddRefed<DOMRequest>
 BluetoothAdapter::IgnoreWaitingCall(ErrorResult& aRv)
 {
+#ifdef MOZ_B2G_RIL
   nsCOMPtr<nsPIDOMWindow> win = GetOwner();
   if (!win) {
     aRv.Throw(NS_ERROR_FAILURE);
@@ -922,11 +933,16 @@ BluetoothAdapter::IgnoreWaitingCall(ErrorResult& aRv)
   bs->IgnoreWaitingCall(results);
 
   return request.forget();
+#else
+  aRv.Throw(NS_ERROR_NOT_IMPLEMENTED);
+  return nullptr;
+#endif // MOZ_B2G_RIL
 }
 
 already_AddRefed<DOMRequest>
 BluetoothAdapter::ToggleCalls(ErrorResult& aRv)
 {
+#ifdef MOZ_B2G_RIL
   nsCOMPtr<nsPIDOMWindow> win = GetOwner();
   if (!win) {
     aRv.Throw(NS_ERROR_FAILURE);
@@ -945,6 +961,10 @@ BluetoothAdapter::ToggleCalls(ErrorResult& aRv)
   bs->ToggleCalls(results);
 
   return request.forget();
+#else
+  aRv.Throw(NS_ERROR_NOT_IMPLEMENTED);
+  return nullptr;
+#endif // MOZ_B2G_RIL
 }
 
 already_AddRefed<DOMRequest>
