@@ -9,12 +9,17 @@
 dump("############################### browserElementPanning.js loaded\n");
 
 let { classes: Cc, interfaces: Ci, results: Cr, utils: Cu }  = Components;
-Cu.import("resource://gre/modules/XPCOMUtils.jsm");
 Cu.import("resource://gre/modules/Services.jsm");
 Cu.import("resource://gre/modules/Geometry.jsm");
 
 var global = this;
 
+const kObservedEvents = [
+  "BEC:ShownModalPrompt",
+  "Activity:Success",
+  "Activity:Error"
+];
+
 const ContentPanning = {
   // Are we listening to touch or mouse events?
   watchedEventsType: '',
@@ -45,27 +50,37 @@ const ContentPanning = {
       this.watchedEventsType = 'mouse';
     }
 
-    // If we are using an AsyncPanZoomController for the parent frame,
-    // it will handle subframe scrolling too. We don't need to listen for
-    // these events.
-    if (!this._asyncPanZoomForViewportFrame) {
-      let els = Cc["@mozilla.org/eventlistenerservice;1"]
-                  .getService(Ci.nsIEventListenerService);
+    let els = Cc["@mozilla.org/eventlistenerservice;1"]
+                .getService(Ci.nsIEventListenerService);
 
-      events.forEach(function(type) {
-        // Using the system group for mouse/touch events to avoid
-        // missing events if .stopPropagation() has been called.
-        els.addSystemEventListener(global, type,
-                                   this.handleEvent.bind(this),
-                                   /* useCapture = */ false);
-      }.bind(this));
-    }
+    events.forEach(function(type) {
+      // Using the system group for mouse/touch events to avoid
+      // missing events if .stopPropagation() has been called.
+      els.addSystemEventListener(global, type,
+                                 this.handleEvent.bind(this),
+                                 /* useCapture = */ false);
+    }.bind(this));
+
+    addEventListener("unload",
+		     this._unloadHandler.bind(this),
+		     /* useCapture = */ false,
+		     /* wantsUntrusted = */ false);
 
     addMessageListener("Viewport:Change", this._recvViewportChange.bind(this));
     addMessageListener("Gesture:DoubleTap", this._recvDoubleTap.bind(this));
+    addEventListener("visibilitychange", this._handleVisibilityChange.bind(this));
+    kObservedEvents.forEach((topic) => {
+      Services.obs.addObserver(this, topic, false);
+    });
   },
 
   handleEvent: function cp_handleEvent(evt) {
+    // Ignore events targeting a <iframe mozbrowser> since those will be
+    // handle by the BrowserElementPanning.js instance of it.
+    if (evt.target instanceof Ci.nsIMozBrowserFrame) {
+      return;
+    }
+
     if (evt.defaultPrevented || evt.multipleActionsPrevented) {
       // clean up panning state even if touchend/mouseup has been preventDefault.
       if(evt.type === 'touchend' || evt.type === 'mouseup') {
@@ -103,6 +118,10 @@ const ContentPanning = {
     }
   },
 
+  observe: function cp_observe(subject, topic, data) {
+    this._resetHover();
+  },
+
   position: new Point(0 , 0),
 
   findPrimaryPointer: function cp_findPrimaryPointer(touches) {
@@ -120,7 +139,9 @@ const ContentPanning = {
   onTouchStart: function cp_onTouchStart(evt) {
     let screenX, screenY;
     if (this.watchedEventsType == 'touch') {
-      if ('primaryPointerId' in this) {
+      if ('primaryPointerId' in this || evt.touches.length >= 2) {
+        this._resetActive();
+        this._finishPanning();
         return;
       }
 
@@ -140,23 +161,11 @@ const ContentPanning = {
     let oldTarget = this.target;
     [this.target, this.scrollCallback] = this.getPannable(this.pointerDownTarget);
 
-    // If we found a target, that means we have found a scrollable subframe. In
-    // this case, and if we are using async panning and zooming on the parent
-    // frame, inform the pan/zoom controller that it should not attempt to
-    // handle any touch events it gets until the next batch (meaning the next
-    // time we get a touch end).
-    if (this.target != null && this._asyncPanZoomForViewportFrame) {
-      this.detectingScrolling = true;
-      var os = Cc["@mozilla.org/observer-service;1"].getService(Ci.nsIObserverService);
-      os.notifyObservers(docShell, 'detect-scrollable-subframe', null);
-    }
-
-    // If we have a pointer down target and we're not async
-    // pan/zooming, we may need to fill in for EventStateManager in
-    // setting the active state on the target element.  Set a timer to
+    // If we have a pointer down target, we may need to fill in for EventStateManager
+    // in setting the active state on the target element.  Set a timer to
     // ensure the pointer-down target is active.  (If it's already
     // active, the timer is a no-op.)
-    if (this.pointerDownTarget !== null && !this.detectingScrolling) {
+    if (this.pointerDownTarget !== null) {
       // If there's no possibility this is a drag/pan, activate now.
       // Otherwise wait a little bit to see if the gesture isn't a
       // tap.
@@ -182,11 +191,13 @@ const ContentPanning = {
     }
 
     this.position.set(screenX, screenY);
+    KineticPanning.reset();
     KineticPanning.record(new Point(0, 0), evt.timeStamp);
 
     // We prevent start events to avoid sending a focus event at the end of this
     // touch series. See bug 889717.
-    if (this.panning || this.preventNextClick) {
+    if (docShell.asyncPanZoomEnabled === false &&
+        (this.panning || this.preventNextClick)) {
       evt.preventDefault();
     }
   },
@@ -225,10 +236,12 @@ const ContentPanning = {
         let view = target.ownerDocument ? target.ownerDocument.defaultView
                                         : target;
         view.addEventListener('click', this, true, true);
-      } else {
+      } else if (docShell.asyncPanZoomEnabled === false) {
         // We prevent end events to avoid sending a focus event. See bug 889717.
         evt.preventDefault();
       }
+    } else if (this.target && click && !this.panning) {
+      this.notify(this._activationTimer);
     }
 
     this._finishPanning();
@@ -237,12 +250,6 @@ const ContentPanning = {
     this.pointerDownTarget = null;
   },
 
-  // True when there's an async pan-zoom controll watching the
-  // outermost scrollable frame, and we're waiting to see whether
-  // we're going to take over from it and synchronously scroll an
-  // inner scrollable frame.
-  detectingScrolling: false,
-
   onTouchMove: function cp_onTouchMove(evt) {
     if (!this.dragging)
       return;
@@ -265,43 +272,31 @@ const ContentPanning = {
 
     KineticPanning.record(delta, evt.timeStamp);
 
+    let isPan = KineticPanning.isPan();
+
+    // If we've detected a pan gesture, cancel the active state of the
+    // current target.
+    if (!this.panning && isPan) {
+      this._resetActive();
+    }
+
     // There's no possibility of us panning anything.
     if (!this.scrollCallback) {
       return;
     }
 
-    let isPan = KineticPanning.isPan();
-    if (!isPan && this.detectingScrolling) {
-      // If panning distance is not large enough and we're waiting to
-      // see whether we should use the sync scroll fallback or not,
-      // don't attempt scrolling.
-      return;
+    // If the application is not managed by the AsyncPanZoomController, then
+    // scroll manually.
+    if (docShell.asyncPanZoomEnabled === false) {
+      this.scrollCallback(delta.scale(-1));
     }
 
-    let isScroll = this.scrollCallback(delta.scale(-1));
-
-    if (this.detectingScrolling) {
-      this.detectingScrolling = false;
-      // Stop async-pan-zooming if the user is panning the subframe.
-      if (isScroll) {
-        // We're going to drive synchronously scrolling an inner frame.
-        Services.obs.notifyObservers(docShell, 'cancel-default-pan-zoom', null);
-      } else {
-        // Let AsyncPanZoomController handle the scrolling gesture.
-        this.scrollCallback = null;
-        return;
-      }
-    }
-
-    // If we've detected a pan gesture, cancel the active state of the
-    // current target.
     if (!this.panning && isPan) {
       this.panning = true;
-      this._resetActive();
       this._activationTimer.cancel();
     }
 
-    if (this.panning) {
+    if (this.panning && docShell.asyncPanZoomEnabled === false) {
       // Only do this when we're actually executing a pan gesture.
       // Otherwise synthetic mouse events will be canceled.
       evt.stopPropagation();
@@ -367,13 +362,6 @@ const ContentPanning = {
       node = node.parentNode;
     }
 
-    if (ContentPanning._asyncPanZoomForViewportFrame &&
-        nodeContent === content) {
-        // The parent context is asynchronously panning and zooming our
-        // root scrollable frame, so don't use our synchronous fallback.
-        return null;
-    }
-
     if (nodeContent.scrollMaxX || nodeContent.scrollMaxY) {
       return nodeContent;
     }
@@ -428,6 +416,7 @@ const ContentPanning = {
 
     function scroll(delta) {
       current = root;
+      firstScroll = true;
       while (current) {
         if (doScroll(current, delta)) {
           firstScroll = false;
@@ -468,20 +457,24 @@ const ContentPanning = {
   },
 
   _resetActive: function cp_resetActive() {
-    let elt = this.target || this.pointerDownTarget;
+    let elt = this.pointerDownTarget || this.target;
     let root = elt.ownerDocument || elt.document;
     this._setActive(root.documentElement);
   },
 
+  _resetHover: function cp_resetHover() {
+    const kStateHover = 0x00000004;
+    try {
+      let element = content.document.createElement('foo');
+      this._domUtils.setContentState(element, kStateHover);
+    } catch(e) {}
+  },
+
   _setActive: function cp_setActive(elt) {
     const kStateActive = 0x00000001;
     this._domUtils.setContentState(elt, kStateActive);
   },
 
-  get _asyncPanZoomForViewportFrame() {
-    return docShell.asyncPanZoomEnabled;
-  },
-
   _recvViewportChange: function(data) {
     let metrics = data.json;
     this._viewport = new Rect(metrics.x, metrics.y,
@@ -553,11 +546,17 @@ const ContentPanning = {
         rect.y = cssTapY - (rect.h / 2);
       }
 
-      var os = Cc["@mozilla.org/observer-service;1"].getService(Ci.nsIObserverService);
-      os.notifyObservers(docShell, 'browser-zoom-to-rect', JSON.stringify(rect));
+      Services.obs.notifyObservers(docShell, 'browser-zoom-to-rect', JSON.stringify(rect));
     }
   },
 
+  _handleVisibilityChange: function(evt) {
+    if (!evt.target.hidden)
+      return;
+
+    this._resetHover();
+  },
+
   _shouldZoomToElement: function(aElement) {
     let win = aElement.ownerDocument.defaultView;
     if (win.getComputedStyle(aElement, null).display == "inline")
@@ -571,8 +570,7 @@ const ContentPanning = {
 
   _zoomOut: function() {
     let rect = new Rect(0, 0, 0, 0);
-    var os = Cc["@mozilla.org/observer-service;1"].getService(Ci.nsIObserverService);
-    os.notifyObservers(docShell, 'browser-zoom-to-rect', JSON.stringify(rect));
+    Services.obs.notifyObservers(docShell, 'browser-zoom-to-rect', JSON.stringify(rect));
   },
 
   _isRectZoomedIn: function(aRect, aViewport) {
@@ -591,20 +589,24 @@ const ContentPanning = {
   },
 
   _finishPanning: function() {
-    this._resetActive();
     this.dragging = false;
-    this.detectingScrolling = false;
     delete this.primaryPointerId;
     this._activationTimer.cancel();
 
-    if (this.panning) {
+    // If there is a scroll action but the application is not managed by
+    // the AsyncPanZoom controller, let's do a manual kinetic panning action.
+    if (this.panning && docShell.asyncPanZoomEnabled === false) {
       KineticPanning.start(this);
     }
+  },
+
+  _unloadHandler: function() {
+    kObservedEvents.forEach((topic) => {
+      Services.obs.removeObserver(this, topic);
+    });
   }
 };
 
-ContentPanning.init();
-
 // Min/max velocity of kinetic panning. This is in pixels/millisecond.
 const kMinVelocity = 0.2;
 const kMaxVelocity = 6;
@@ -684,16 +686,20 @@ const KineticPanning = {
   },
 
   stop: function kp_stop() {
+    this.reset();
+
     if (!this.target)
       return;
 
-    this.momentums = [];
-    this.distance.set(0, 0);
-
     this.target.onKineticEnd();
     this.target = null;
   },
 
+  reset: function kp_reset() {
+    this.momentums = [];
+    this.distance.set(0, 0);
+  },
+
   momentums: [],
   record: function kp_record(delta, timestamp) {
     this.momentums.push({ 'time': this._getTime(timestamp),
