@@ -8,14 +8,17 @@
 
 #include "nsContentUtils.h"
 #include "nsDOMClassInfoID.h"
+#include "nsNetUtil.h"
 
 #include "nsIDOMFile.h"
 #include "nsIFileStorage.h"
 
+#include "File.h"
 #include "FileRequest.h"
 #include "FileService.h"
 #include "LockedFile.h"
 #include "MetadataHelper.h"
+#include "mozilla/dom/FileHandleBinding.h"
 
 using namespace mozilla;
 using namespace mozilla::dom;
@@ -34,11 +37,12 @@ public:
     mFileHandle(aFileHandle)
   { }
 
-  nsresult
-  GetSuccessResult(JSContext* aCx, JS::Value* aVal);
+  virtual nsresult
+  GetSuccessResult(JSContext* aCx,
+                   JS::MutableHandle<JS::Value> aVal) MOZ_OVERRIDE;
 
-  void
-  ReleaseObjects()
+  virtual void
+  ReleaseObjects() MOZ_OVERRIDE
   {
     mFileHandle = nullptr;
 
@@ -55,51 +59,73 @@ NS_IMPL_CYCLE_COLLECTION_INHERITED_1(FileHandle, nsDOMEventTargetHelper,
                                      mFileStorage)
 
 NS_INTERFACE_MAP_BEGIN_CYCLE_COLLECTION_INHERITED(FileHandle)
-  NS_INTERFACE_MAP_ENTRY(nsIDOMFileHandle)
 NS_INTERFACE_MAP_END_INHERITING(nsDOMEventTargetHelper)
 
 NS_IMPL_ADDREF_INHERITED(FileHandle, nsDOMEventTargetHelper)
 NS_IMPL_RELEASE_INHERITED(FileHandle, nsDOMEventTargetHelper)
 
-NS_IMPL_EVENT_HANDLER(FileHandle, abort)
-NS_IMPL_EVENT_HANDLER(FileHandle, error)
-
-NS_IMETHODIMP
-FileHandle::GetDOMName(nsAString& aName)
+// static
+already_AddRefed<FileHandle>
+FileHandle::Create(nsPIDOMWindow* aWindow,
+                   nsIFileStorage* aFileStorage,
+                   nsIFile* aFile)
 {
-  aName = mName;
-  return NS_OK;
-}
+  MOZ_ASSERT(NS_IsMainThread(), "Wrong thread!");
 
-NS_IMETHODIMP
-FileHandle::GetDOMType(nsAString& aType)
-{
-  aType = mType;
-  return NS_OK;
-}
+  nsRefPtr<FileHandle> newFileHandle = new FileHandle(aWindow);
 
-NS_IMETHODIMP
-FileHandle::Open(const nsAString& aMode,
-                 uint8_t aOptionalArgCount,
-                 nsIDOMLockedFile** _retval)
-{
-  FileMode mode;
-  if (aOptionalArgCount) {
-    if (aMode.EqualsLiteral("readwrite")) {
-      mode = FileMode::Readwrite;
-    } else if (aMode.EqualsLiteral("readonly")) {
-      mode = FileMode::Readonly;
-    } else {
-      return NS_ERROR_TYPE_ERR;
-    }
-  } else {
-    mode = FileMode::Readonly;
+  newFileHandle->mFileStorage = aFileStorage;
+  nsresult rv = aFile->GetLeafName(newFileHandle->mName);
+  if (NS_WARN_IF(NS_FAILED(rv))) {
+    return nullptr;
   }
 
-  ErrorResult rv;
-  nsCOMPtr<nsIDOMLockedFile> lockedFile = Open(mode, rv);
-  lockedFile.forget(_retval);
-  return rv.ErrorCode();
+  newFileHandle->mFile = aFile;
+  newFileHandle->mFileName = newFileHandle->mName;
+
+  return newFileHandle.forget();
+}
+
+// virtual
+already_AddRefed<nsISupports>
+FileHandle::CreateStream(nsIFile* aFile, bool aReadOnly)
+{
+  nsresult rv;
+
+  if (aReadOnly) {
+    nsCOMPtr<nsIInputStream> stream;
+    rv = NS_NewLocalFileInputStream(getter_AddRefs(stream), aFile, -1, -1,
+                                    nsIFileInputStream::DEFER_OPEN);
+    if (NS_WARN_IF(NS_FAILED(rv))) {
+      return nullptr;
+    }
+    return stream.forget();
+  }
+
+  nsCOMPtr<nsIFileStream> stream;
+  rv = NS_NewLocalFileStream(getter_AddRefs(stream), aFile, -1, -1,
+                             nsIFileStream::DEFER_OPEN);
+  if (NS_WARN_IF(NS_FAILED(rv))) {
+    return nullptr;
+  }
+  return stream.forget();
+}
+
+// virtual
+already_AddRefed<nsIDOMFile>
+FileHandle::CreateFileObject(LockedFile* aLockedFile, uint32_t aFileSize)
+{
+  nsCOMPtr<nsIDOMFile> file =
+    new File(mName, mType, aFileSize, mFile, aLockedFile);
+
+  return file.forget();
+}
+
+// virtual
+JSObject*
+FileHandle::WrapObject(JSContext* aCx, JS::Handle<JSObject*> aScope)
+{
+  return FileHandleBinding::Wrap(aCx, aScope, this);
 }
 
 already_AddRefed<nsIDOMLockedFile>
@@ -112,15 +138,7 @@ FileHandle::Open(FileMode aMode, ErrorResult& aError)
     return nullptr;
   }
 
-  MOZ_STATIC_ASSERT(static_cast<uint32_t>(FileMode::Readonly) ==
-                    static_cast<uint32_t>(LockedFile::READ_ONLY),
-                    "Enum values should match.");
-  MOZ_STATIC_ASSERT(static_cast<uint32_t>(FileMode::Readwrite) ==
-                    static_cast<uint32_t>(LockedFile::READ_WRITE),
-                    "Enum values should match.");
-
-  nsRefPtr<LockedFile> lockedFile =
-    LockedFile::Create(this, LockedFile::Mode(static_cast<int>(aMode)));
+  nsRefPtr<LockedFile> lockedFile = LockedFile::Create(this, aMode);
   if (!lockedFile) {
     aError.Throw(NS_ERROR_DOM_FILEHANDLE_UNKNOWN_ERR);
     return nullptr;
@@ -129,15 +147,6 @@ FileHandle::Open(FileMode aMode, ErrorResult& aError)
   return lockedFile.forget();
 }
 
-NS_IMETHODIMP
-FileHandle::GetFile(nsIDOMDOMRequest** _retval)
-{
-  ErrorResult rv;
-  nsRefPtr<DOMRequest> request = GetFile(rv);
-  request.forget(_retval);
-  return rv.ErrorCode();
-}
-
 already_AddRefed<DOMRequest>
 FileHandle::GetFile(ErrorResult& aError)
 {
@@ -149,17 +158,16 @@ FileHandle::GetFile(ErrorResult& aError)
   }
 
   nsRefPtr<LockedFile> lockedFile =
-    LockedFile::Create(this, LockedFile::READ_ONLY, LockedFile::PARALLEL);
+    LockedFile::Create(this, FileMode::Readonly, LockedFile::PARALLEL);
   if (!lockedFile) {
     aError.Throw(NS_ERROR_DOM_FILEHANDLE_UNKNOWN_ERR);
     return nullptr;
   }
 
   nsRefPtr<FileRequest> request =
-    FileRequest::Create(GetOwner(), lockedFile, false);
+    FileRequest::Create(GetOwner(), lockedFile, /* aWrapAsDOMRequest */ true);
 
-  nsRefPtr<MetadataParameters> params = new MetadataParameters();
-  params->Init(true, false);
+  nsRefPtr<MetadataParameters> params = new MetadataParameters(true, false);
 
   nsRefPtr<GetFileHelper> helper =
     new GetFileHelper(lockedFile, request, params, this);
@@ -173,36 +181,17 @@ FileHandle::GetFile(ErrorResult& aError)
   return request.forget();
 }
 
-NS_IMETHODIMP_(int64_t)
-FileHandle::GetFileId()
-{
-  return -1;
-}
-
-NS_IMETHODIMP_(mozilla::dom::indexedDB::FileInfo*)
-FileHandle::GetFileInfo()
-{
-  return nullptr;
-}
-
 nsresult
-GetFileHelper::GetSuccessResult(JSContext* aCx, JS::Value* aVal)
+GetFileHelper::GetSuccessResult(JSContext* aCx,
+                                JS::MutableHandle<JS::Value> aVal)
 {
   nsCOMPtr<nsIDOMFile> domFile =
     mFileHandle->CreateFileObject(mLockedFile, mParams->Size());
 
-  JS::Rooted<JSObject*> global(aCx, JS_GetGlobalForScopeChain(aCx));
+  JS::Rooted<JSObject*> global(aCx, JS::CurrentGlobalOrNull(aCx));
   nsresult rv =
     nsContentUtils::WrapNative(aCx, global, domFile,
                                &NS_GET_IID(nsIDOMFile), aVal);
   NS_ENSURE_SUCCESS(rv, NS_ERROR_DOM_FILEHANDLE_UNKNOWN_ERR);
-
   return NS_OK;
 }
-
-/* virtual */
-JSObject*
-FileHandle::WrapObject(JSContext* aCx, JS::Handle<JSObject*> aScope)
-{
-  return FileHandleBinding::Wrap(aCx, aScope, this);
-}
