@@ -26,7 +26,7 @@ const uint32_t kThreadLimit = 20;
 const uint32_t kIdleThreadLimit = 5;
 const uint32_t kIdleThreadTimeoutMs = 30000;
 
-TransactionThreadPool* gInstance = nullptr;
+TransactionThreadPool* gThreadPool = nullptr;
 bool gShutdown = false;
 
 #ifdef MOZ_ENABLE_PROFILER_SPS
@@ -34,7 +34,7 @@ bool gShutdown = false;
 class TransactionThreadPoolListener : public nsIThreadPoolListener
 {
 public:
-  NS_DECL_ISUPPORTS
+  NS_DECL_THREADSAFE_ISUPPORTS
   NS_DECL_NSITHREADPOOLLISTENER
 
 private:
@@ -51,7 +51,7 @@ BEGIN_INDEXEDDB_NAMESPACE
 class FinishTransactionRunnable MOZ_FINAL : public nsIRunnable
 {
 public:
-  NS_DECL_ISUPPORTS
+  NS_DECL_THREADSAFE_ISUPPORTS
   NS_DECL_NSIRUNNABLE
 
   inline FinishTransactionRunnable(IDBTransaction* aTransaction,
@@ -67,37 +67,37 @@ END_INDEXEDDB_NAMESPACE
 TransactionThreadPool::TransactionThreadPool()
 {
   NS_ASSERTION(NS_IsMainThread(), "Wrong thread!");
-  NS_ASSERTION(!gInstance, "More than one instance!");
+  NS_ASSERTION(!gThreadPool, "More than one instance!");
 }
 
 TransactionThreadPool::~TransactionThreadPool()
 {
   NS_ASSERTION(NS_IsMainThread(), "Wrong thread!");
-  NS_ASSERTION(gInstance == this, "Different instances!");
-  gInstance = nullptr;
+  NS_ASSERTION(gThreadPool == this, "Different instances!");
+  gThreadPool = nullptr;
 }
 
 // static
 TransactionThreadPool*
 TransactionThreadPool::GetOrCreate()
 {
-  if (!gInstance && !gShutdown) {
+  if (!gThreadPool && !gShutdown) {
     NS_ASSERTION(NS_IsMainThread(), "Wrong thread!");
     nsAutoPtr<TransactionThreadPool> pool(new TransactionThreadPool());
 
     nsresult rv = pool->Init();
     NS_ENSURE_SUCCESS(rv, nullptr);
 
-    gInstance = pool.forget();
+    gThreadPool = pool.forget();
   }
-  return gInstance;
+  return gThreadPool;
 }
 
 // static
 TransactionThreadPool*
 TransactionThreadPool::Get()
 {
-  return gInstance;
+  return gThreadPool;
 }
 
 // static
@@ -108,12 +108,12 @@ TransactionThreadPool::Shutdown()
 
   gShutdown = true;
 
-  if (gInstance) {
-    if (NS_FAILED(gInstance->Cleanup())) {
+  if (gThreadPool) {
+    if (NS_FAILED(gThreadPool->Cleanup())) {
       NS_WARNING("Failed to shutdown thread pool!");
     }
-    delete gInstance;
-    gInstance = nullptr;
+    delete gThreadPool;
+    gThreadPool = nullptr;
   }
 }
 
@@ -122,8 +122,6 @@ TransactionThreadPool::Init()
 {
   NS_ASSERTION(NS_IsMainThread(), "Wrong thread!");
 
-  mTransactionsInProgress.Init();
-
   nsresult rv;
   mThreadPool = do_CreateInstance(NS_THREADPOOL_CONTRACTID, &rv);
   NS_ENSURE_SUCCESS(rv, rv);
@@ -214,7 +212,7 @@ TransactionThreadPool::FinishTransaction(IDBTransaction* aTransaction)
   // AddRef here because removing from the hash will call Release.
   nsRefPtr<IDBTransaction> transaction(aTransaction);
 
-  nsIAtom* databaseId = aTransaction->mDatabase->Id();
+  const nsACString& databaseId = aTransaction->mDatabase->Id();
 
   DatabaseTransactionInfo* dbTransactionInfo;
   if (!mTransactionsInProgress.Get(databaseId, &dbTransactionInfo)) {
@@ -288,7 +286,8 @@ TransactionThreadPool::GetQueueForTransaction(IDBTransaction* aTransaction)
   NS_ASSERTION(NS_IsMainThread(), "Wrong thread!");
   NS_ASSERTION(aTransaction, "Null pointer!");
 
-  nsIAtom* databaseId = aTransaction->mDatabase->Id();
+  const nsACString& databaseId = aTransaction->mDatabase->Id();
+
   const nsTArray<nsString>& objectStoreNames = aTransaction->mObjectStoreNames;
   const uint16_t mode = aTransaction->mMode;
 
@@ -448,7 +447,8 @@ TransactionThreadPool::AbortTransactionsForDatabase(IDBDatabase* aDatabase)
     // This can fail, for example if the transaction is in the process of
     // being comitted. That is expected and fine, so we ignore any returned
     // errors.
-    transactions[index]->Abort();
+    ErrorResult rv;
+    transactions[index]->Abort(rv);
   }
 }
 
@@ -510,8 +510,7 @@ TransactionThreadPool::MaybeFireCallback(DatabasesCompleteCallback aCallback)
       MOZ_CRASH();
     }
 
-    if (mTransactionsInProgress.Get(database->Id(),
-                                    nullptr)) {
+    if (mTransactionsInProgress.Get(database->Id(), nullptr)) {
       return false;
     }
   }
@@ -566,8 +565,7 @@ TransactionThreadPool::TransactionQueue::Finish(nsIRunnable* aFinishRunnable)
   mMonitor.Notify();
 }
 
-NS_IMPL_THREADSAFE_ISUPPORTS1(TransactionThreadPool::TransactionQueue,
-                              nsIRunnable)
+NS_IMPL_ISUPPORTS1(TransactionThreadPool::TransactionQueue, nsIRunnable)
 
 NS_IMETHODIMP
 TransactionThreadPool::TransactionQueue::Run()
@@ -639,7 +637,7 @@ FinishTransactionRunnable::FinishTransactionRunnable(
   mFinishRunnable.swap(aFinishRunnable);
 }
 
-NS_IMPL_THREADSAFE_ISUPPORTS1(FinishTransactionRunnable, nsIRunnable)
+NS_IMPL_ISUPPORTS1(FinishTransactionRunnable, nsIRunnable)
 
 NS_IMETHODIMP
 FinishTransactionRunnable::Run()
@@ -648,12 +646,12 @@ FinishTransactionRunnable::Run()
 
   PROFILER_MAIN_THREAD_LABEL("IndexedDB", "FinishTransactionRunnable::Run");
 
-  if (!gInstance) {
+  if (!gThreadPool) {
     NS_ERROR("Running after shutdown!");
     return NS_ERROR_FAILURE;
   }
 
-  gInstance->FinishTransaction(mTransaction);
+  gThreadPool->FinishTransaction(mTransaction);
 
   if (mFinishRunnable) {
     mFinishRunnable->Run();
@@ -665,14 +663,14 @@ FinishTransactionRunnable::Run()
 
 #ifdef MOZ_ENABLE_PROFILER_SPS
 
-NS_IMPL_THREADSAFE_ISUPPORTS1(TransactionThreadPoolListener,
-                              nsIThreadPoolListener)
+NS_IMPL_ISUPPORTS1(TransactionThreadPoolListener, nsIThreadPoolListener)
 
 NS_IMETHODIMP
 TransactionThreadPoolListener::OnThreadCreated()
 {
   MOZ_ASSERT(!NS_IsMainThread());
-  profiler_register_thread("IndexedDB Transaction");
+  char aLocal;
+  profiler_register_thread("IndexedDB Transaction", &aLocal);
   return NS_OK;
 }
 
