@@ -336,7 +336,9 @@ function isNetworkReady() {
                         .getService(SpecialPowers.Ci.nsINetworkInterfaceListService);
     var itfList = listService.getDataInterfaceList(
           SpecialPowers.Ci.nsINetworkInterfaceListService.LIST_NOT_INCLUDE_MMS_INTERFACES |
-          SpecialPowers.Ci.nsINetworkInterfaceListService.LIST_NOT_INCLUDE_SUPL_INTERFACES);
+          SpecialPowers.Ci.nsINetworkInterfaceListService.LIST_NOT_INCLUDE_SUPL_INTERFACES |
+          SpecialPowers.Ci.nsINetworkInterfaceListService.LIST_NOT_INCLUDE_IMS_INTERFACES |
+          SpecialPowers.Ci.nsINetworkInterfaceListService.LIST_NOT_INCLUDE_DUN_INTERFACES);
     var num = itfList.getNumberOfInterface();
     for (var i = 0; i < num; i++) {
       if (itfList.getInterface(i).ip) {
@@ -530,6 +532,11 @@ function PCT_createOffer(peer, onSuccess) {
   });
 };
 
+PeerConnectionTest.prototype.setIdentityProvider =
+function(peer, provider, protocol, identity) {
+  peer.setIdentityProvider(provider, protocol, identity);
+};
+
 /**
  * Sets the local description for the specified peer connection instance
  * and automatically handles the failure case.
@@ -765,7 +772,7 @@ DataChannelTest.prototype = Object.create(PeerConnectionTest.prototype, {
           });
         } else {
           check_next_test();
-	}
+        }
       });
     }
   },
@@ -1053,6 +1060,7 @@ DataChannelWrapper.prototype = {
 function PeerConnectionWrapper(label, configuration) {
   this.configuration = configuration;
   this.label = label;
+  this.whenCreated = Date.now();
 
   this.constraints = [ ];
   this.offerConstraints = {};
@@ -1101,10 +1109,16 @@ function PeerConnectionWrapper(label, configuration) {
    *        Event data which includes the stream to be added
    */
   this._pc.onaddstream = function (event) {
-    info(self + ": 'onaddstream' event fired for " + event.stream);
+    info(self + ": 'onaddstream' event fired for " + JSON.stringify(event.stream));
 
-    // TODO: Bug 834835 - Assume type is video until we get get{Audio,Video}Tracks.
-    self.attachMedia(event.stream, 'video', 'remote');
+    var type = '';
+    if (event.stream.getAudioTracks().length > 0) {
+      type = 'audio';
+    }
+    if (event.stream.getVideoTracks().length > 0) {
+      type += 'video';
+    }
+    self.attachMedia(event.stream, type, 'remote');
    };
 
   /**
@@ -1204,6 +1218,10 @@ PeerConnectionWrapper.prototype = {
     return this._pc.iceConnectionState;
   },
 
+  setIdentityProvider: function(provider, protocol, identity) {
+      this._pc.setIdentityProvider(provider, protocol, identity);
+  },
+
   /**
    * Callback when we get media from either side. Also an appropriate
    * HTML media element will be created.
@@ -1434,7 +1452,7 @@ PeerConnectionWrapper.prototype = {
   /**
    * Returns if the ICE the connection state is "connected".
    *
-   * @returns {boolean} True is the connection state is "connected", otherwise false.
+   * @returns {boolean} True if the connection state is "connected", otherwise false.
    */
   isIceConnected : function PCW_isIceConnected() {
     info("iceConnectionState: " + this.iceConnectionState);
@@ -1444,7 +1462,7 @@ PeerConnectionWrapper.prototype = {
   /**
    * Returns if the ICE the connection state is "checking".
    *
-   * @returns {boolean} True is the connection state is "checking", otherwise false.
+   * @returns {boolean} True if the connection state is "checking", otherwise false.
    */
   isIceChecking : function PCW_isIceChecking() {
     return this.iceConnectionState === "checking";
@@ -1453,12 +1471,23 @@ PeerConnectionWrapper.prototype = {
   /**
    * Returns if the ICE the connection state is "new".
    *
-   * @returns {boolean} True is the connection state is "new", otherwise false.
+   * @returns {boolean} True if the connection state is "new", otherwise false.
    */
   isIceNew : function PCW_isIceNew() {
     return this.iceConnectionState === "new";
   },
 
+  /**
+   * Checks if the ICE connection state still waits for a connection to get
+   * established.
+   *
+   * @returns {boolean} True if the connection state is "checking" or "new", 
+   *  otherwise false.
+   */
+  isIceConnectionPending : function PCW_isIceConnectionPending() {
+    return (this.isIceChecking() || this.isIceNew());
+  },
+
   /**
    * Registers a callback for the ICE connection state change and
    * reports success (=connected) or failure via the callbacks.
@@ -1479,7 +1508,7 @@ PeerConnectionWrapper.prototype = {
       if (self.isIceConnected()) {
         delete self.ice_connection_callbacks["waitForIceConnected"];
         mySuccess();
-      } else if (! (self.isIceChecking() || self.isIceNew())) {
+      } else if (! self.isIceConnectionPending()) {
         delete self.ice_connection_callbacks["waitForIceConnected"];
         myFailure();
       }
@@ -1560,16 +1589,81 @@ PeerConnectionWrapper.prototype = {
       return n;
     }
 
+    const isWinXP = navigator.userAgent.indexOf("Windows NT 5.1") != -1;
+
     // Use spec way of enumerating stats
     var counters = {};
     for (var key in stats) {
       if (stats.hasOwnProperty(key)) {
         var res = stats[key];
+        // validate stats
+        ok(res.id == key, "Coherent stats id");
+        var nowish = Date.now() + 1000;        // TODO: clock drift observed
+        var minimum = this.whenCreated - 1000; // on Windows XP (Bug 979649)
+        if (isWinXP) {
+          todo(false, "Can't reliably test rtcp timestamps on WinXP (Bug 979649)");
+        } else {
+          ok(res.timestamp >= minimum,
+             "Valid " + (res.isRemote? "rtcp" : "rtp") + " timestamp " +
+                 res.timestamp + " >= " + minimum + " (" +
+                 (res.timestamp - minimum) + " ms)");
+          ok(res.timestamp <= nowish,
+             "Valid " + (res.isRemote? "rtcp" : "rtp") + " timestamp " +
+                 res.timestamp + " <= " + nowish + " (" +
+                 (res.timestamp - nowish) + " ms)");
+        }
         if (!res.isRemote) {
           counters[res.type] = toNum(counters[res.type]) + 1;
+
+          switch (res.type) {
+            case "inboundrtp":
+            case "outboundrtp": {
+              // ssrc is a 32 bit number returned as a string by spec
+              ok(res.ssrc.length > 0, "Ssrc has length");
+              ok(res.ssrc.length < 11, "Ssrc not lengthy");
+              ok(!/[^0-9]/.test(res.ssrc), "Ssrc numeric");
+              ok(parseInt(res.ssrc) < Math.pow(2,32), "Ssrc within limits");
+
+              if (res.type == "outboundrtp") {
+                ok(res.packetsSent !== undefined, "Rtp packetsSent");
+                // minimum fragment is 8 (from RFC 791)
+                ok(res.bytesSent >= res.packetsSent * 8, "Rtp bytesSent");
+              } else {
+                ok(res.packetsReceived !== undefined, "Rtp packetsReceived");
+                ok(res.bytesReceived >= res.packetsReceived * 8, "Rtp bytesReceived");
+              }
+              if (res.remoteId) {
+                var rem = stats[res.remoteId];
+                ok(rem.isRemote, "Remote is rtcp");
+                ok(rem.remoteId == res.id, "Remote backlink match");
+                if(res.type == "outboundrtp") {
+                  ok(rem.type == "inboundrtp", "Rtcp is inbound");
+                  ok(rem.packetsReceived !== undefined, "Rtcp packetsReceived");
+                  ok(rem.packetsReceived <= res.packetsSent, "No more than sent");
+                  ok(rem.packetsLost !== undefined, "Rtcp packetsLost");
+                  ok(rem.bytesReceived >= rem.packetsReceived * 8, "Rtcp bytesReceived");
+                  ok(rem.bytesReceived <= res.bytesSent, "No more than sent bytes");
+                  ok(rem.jitter !== undefined, "Rtcp jitter");
+                  ok(rem.mozRtt !== undefined, "Rtcp rtt");
+                  ok(rem.mozRtt >= 0, "Rtcp rtt " + rem.mozRtt + " >= 0");
+                  ok(rem.mozRtt < 60000, "Rtcp rtt " + rem.mozRtt + " < 1 min");
+                } else {
+                  ok(rem.type == "outboundrtp", "Rtcp is outbound");
+                  ok(rem.packetsSent !== undefined, "Rtcp packetsSent");
+                  // We may have received more than outdated Rtcp packetsSent
+                  ok(rem.bytesSent >= rem.packetsSent * 8, "Rtcp bytesSent");
+                }
+                ok(rem.ssrc == res.ssrc, "Remote ssrc match");
+              } else {
+                info("No rtcp info received yet");
+              }
+            }
+            break;
+          }
         }
       }
     }
+
     // Use MapClass way of enumerating stats
     var counters2 = {};
     stats.forEach(function(res) {
