@@ -10,7 +10,6 @@
 #include "SmsMessage.h"
 #include "SmsFilter.h"
 #include "SmsSegmentInfo.h"
-#include "DictionaryHelpers.h"
 #include "nsJSUtils.h"
 #include "nsCxPusher.h"
 #include "mozilla/dom/MobileMessageManagerBinding.h"
@@ -120,7 +119,7 @@ SmsIPCService::SmsIPCService()
 NS_IMETHODIMP
 SmsIPCService::Observe(nsISupports* aSubject,
                        const char* aTopic,
-                       const PRUnichar* aData)
+                       const char16_t* aData)
 {
   if (!strcmp(aTopic, NS_PREFBRANCH_PREFCHANGE_TOPIC_ID)) {
     nsDependentString data(aData);
@@ -147,17 +146,6 @@ SmsIPCService::GetSmsDefaultServiceId(uint32_t* aServiceId)
   return NS_OK;
 }
 
-NS_IMETHODIMP
-SmsIPCService::HasSupport(bool* aHasSupport)
-{
-  PSmsChild* smsChild = GetSmsChild();
-  NS_ENSURE_TRUE(smsChild, NS_ERROR_FAILURE);
-
-  smsChild->SendHasSupport(aHasSupport);
-
-  return NS_OK;
-}
-
 NS_IMETHODIMP
 SmsIPCService::GetSegmentInfoForText(const nsAString& aText,
                                      nsIMobileMessageCallback* aRequest)
@@ -167,12 +155,21 @@ SmsIPCService::GetSegmentInfoForText(const nsAString& aText,
 }
 
 NS_IMETHODIMP
-SmsIPCService::Send(const nsAString& aNumber,
+SmsIPCService::GetSmscAddress(uint32_t aServiceId,
+                              nsIMobileMessageCallback* aRequest)
+{
+  return SendRequest(GetSmscAddressRequest(aServiceId), aRequest);
+}
+
+NS_IMETHODIMP
+SmsIPCService::Send(uint32_t aServiceId,
+                    const nsAString& aNumber,
                     const nsAString& aMessage,
                     const bool aSilent,
                     nsIMobileMessageCallback* aRequest)
 {
-  return SendRequest(SendMessageRequest(SendSmsMessageRequest(nsString(aNumber),
+  return SendRequest(SendMessageRequest(SendSmsMessageRequest(aServiceId,
+                                                              nsString(aNumber),
                                                               nsString(aMessage),
                                                               aSilent)),
                      aRequest);
@@ -241,9 +238,10 @@ SmsIPCService::CreateMessageCursor(nsIDOMMozSmsFilter* aFilter,
 NS_IMETHODIMP
 SmsIPCService::MarkMessageRead(int32_t aMessageId,
                                bool aValue,
+                               bool aSendReadReport,
                                nsIMobileMessageCallback* aRequest)
 {
-  return SendRequest(MarkMessageReadRequest(aMessageId, aValue), aRequest);
+  return SendRequest(MarkMessageReadRequest(aMessageId, aValue, aSendReadReport), aRequest);
 }
 
 NS_IMETHODIMP
@@ -255,7 +253,8 @@ SmsIPCService::CreateThreadCursor(nsIMobileMessageCursorCallback* aCursorCallbac
 }
 
 bool
-GetSendMmsMessageRequestFromParams(const JS::Value& aParam,
+GetSendMmsMessageRequestFromParams(uint32_t aServiceId,
+                                   const JS::Value& aParam,
                                    SendMmsMessageRequest& request) {
   if (aParam.isUndefined() || aParam.isNull() || !aParam.isObject()) {
     return false;
@@ -282,7 +281,7 @@ GetSendMmsMessageRequestFromParams(const JS::Value& aParam,
   }
 
   for (uint32_t i = 0; i < params.mAttachments.Value().Length(); i++) {
-    MmsAttachment& attachment = params.mAttachments.Value()[i];
+    mozilla::dom::MmsAttachment& attachment = params.mAttachments.Value()[i];
     MmsAttachmentData mmsAttachment;
     mmsAttachment.id().Assign(attachment.mId);
     mmsAttachment.location().Assign(attachment.mLocation);
@@ -296,6 +295,9 @@ GetSendMmsMessageRequestFromParams(const JS::Value& aParam,
   request.smil() = params.mSmil;
   request.subject() = params.mSubject;
 
+  // Set service ID.
+  request.serviceId() = aServiceId;
+
   return true;
 }
 
@@ -311,11 +313,12 @@ SmsIPCService::GetMmsDefaultServiceId(uint32_t* aServiceId)
 }
 
 NS_IMETHODIMP
-SmsIPCService::Send(const JS::Value& aParameters,
+SmsIPCService::Send(uint32_t aServiceId,
+                    JS::Handle<JS::Value> aParameters,
                     nsIMobileMessageCallback *aRequest)
 {
   SendMmsMessageRequest req;
-  if (!GetSendMmsMessageRequestFromParams(aParameters, req)) {
+  if (!GetSendMmsMessageRequestFromParams(aServiceId, aParameters, req)) {
     return NS_ERROR_INVALID_ARG;
   }
   return SendRequest(SendMessageRequest(req), aRequest);
@@ -326,3 +329,12 @@ SmsIPCService::Retrieve(int32_t aId, nsIMobileMessageCallback *aRequest)
 {
   return SendRequest(RetrieveMessageRequest(aId), aRequest);
 }
+
+NS_IMETHODIMP
+SmsIPCService::SendReadReport(const nsAString & messageID,
+                              const nsAString & toAddress,
+                              const nsAString & iccId)
+{
+  NS_ERROR("We should not be here!");
+  return NS_OK;
+}
