@@ -34,7 +34,8 @@ static JSObject*
 MmsAttachmentDataToJSObject(JSContext* aContext,
                             const MmsAttachmentData& aAttachment)
 {
-  JS::Rooted<JSObject*> obj(aContext, JS_NewObject(aContext, nullptr, nullptr, nullptr));
+  JS::Rooted<JSObject*> obj(aContext, JS_NewObject(aContext, nullptr, JS::NullPtr(),
+                                                   JS::NullPtr()));
   NS_ENSURE_TRUE(obj, nullptr);
 
   JSString* idStr = JS_NewUCStringCopyN(aContext,
@@ -77,7 +78,7 @@ GetParamsFromSendMmsMessageRequest(JSContext* aCx,
                                    const SendMmsMessageRequest& aRequest,
                                    JS::Value* aParam)
 {
-  JS::Rooted<JSObject*> paramsObj(aCx, JS_NewObject(aCx, nullptr, nullptr, nullptr));
+  JS::Rooted<JSObject*> paramsObj(aCx, JS_NewObject(aCx, nullptr, JS::NullPtr(), JS::NullPtr()));
   NS_ENSURE_TRUE(paramsObj, false);
 
   // smil
@@ -114,14 +115,12 @@ GetParamsFromSendMmsMessageRequest(JSContext* aCx,
 
   // attachments
   JS::Rooted<JSObject*> attachmentArray(aCx, JS_NewArrayObject(aCx,
-                                                               aRequest.attachments().Length(),
-                                                               nullptr));
+                                                               aRequest.attachments().Length()));
   for (uint32_t i = 0; i < aRequest.attachments().Length(); i++) {
     JS::Rooted<JSObject*> obj(aCx,
       MmsAttachmentDataToJSObject(aCx, aRequest.attachments().ElementAt(i)));
     NS_ENSURE_TRUE(obj, false);
-    JS::Rooted<JS::Value> val(aCx, JS::ObjectValue(*obj));
-    if (!JS_SetElement(aCx, attachmentArray, i, &val)) {
+    if (!JS_SetElement(aCx, attachmentArray, i, obj)) {
       return false;
     }
   }
@@ -153,6 +152,8 @@ SmsParent::SmsParent()
   obs->AddObserver(this, kSmsDeliverySuccessObserverTopic, false);
   obs->AddObserver(this, kSmsDeliveryErrorObserverTopic, false);
   obs->AddObserver(this, kSilentSmsReceivedObserverTopic, false);
+  obs->AddObserver(this, kSmsReadSuccessObserverTopic, false);
+  obs->AddObserver(this, kSmsReadErrorObserverTopic, false);
 }
 
 void
@@ -171,11 +172,13 @@ SmsParent::ActorDestroy(ActorDestroyReason why)
   obs->RemoveObserver(this, kSmsDeliverySuccessObserverTopic);
   obs->RemoveObserver(this, kSmsDeliveryErrorObserverTopic);
   obs->RemoveObserver(this, kSilentSmsReceivedObserverTopic);
+  obs->RemoveObserver(this, kSmsReadSuccessObserverTopic);
+  obs->RemoveObserver(this, kSmsReadErrorObserverTopic);
 }
 
 NS_IMETHODIMP
 SmsParent::Observe(nsISupports* aSubject, const char* aTopic,
-                   const PRUnichar* aData)
+                   const char16_t* aData)
 {
   if (!strcmp(aTopic, kSmsReceivedObserverTopic)) {
     MobileMessageData msgData;
@@ -272,6 +275,30 @@ SmsParent::Observe(nsISupports* aSubject, const char* aTopic,
     return NS_OK;
   }
 
+
+  if (!strcmp(aTopic, kSmsReadSuccessObserverTopic)) {
+    MobileMessageData msgData;
+    if (!GetMobileMessageDataFromMessage(aSubject, msgData)) {
+      NS_ERROR("Got a 'sms-read-success' topic without a valid message!");
+      return NS_OK;
+    }
+
+    unused << SendNotifyReadSuccessMessage(msgData);
+    return NS_OK;
+  }
+
+  if (!strcmp(aTopic, kSmsReadErrorObserverTopic)) {
+    MobileMessageData msgData;
+    if (!GetMobileMessageDataFromMessage(aSubject, msgData)) {
+      NS_ERROR("Got a 'sms-read-error' topic without a valid message!");
+      return NS_OK;
+    }
+
+    unused << SendNotifyReadErrorMessage(msgData);
+    return NS_OK;
+  }
+
+
   return NS_OK;
 }
 
@@ -300,18 +327,6 @@ SmsParent::GetMobileMessageDataFromMessage(nsISupports *aMsg,
   return false;
 }
 
-bool
-SmsParent::RecvHasSupport(bool* aHasSupport)
-{
-  *aHasSupport = false;
-
-  nsCOMPtr<nsISmsService> smsService = do_GetService(SMS_SERVICE_CONTRACTID);
-  NS_ENSURE_TRUE(smsService, true);
-
-  smsService->HasSupport(aHasSupport);
-  return true;
-}
-
 bool
 SmsParent::RecvAddSilentNumber(const nsString& aNumber)
 {
@@ -367,6 +382,8 @@ SmsParent::RecvPSmsRequestConstructor(PSmsRequestParent* aActor,
       return actor->DoRequest(aRequest.get_MarkMessageReadRequest());
     case IPCSmsRequest::TGetSegmentInfoForTextRequest:
       return actor->DoRequest(aRequest.get_GetSegmentInfoForTextRequest());
+    case IPCSmsRequest::TGetSmscAddressRequest:
+      return actor->DoRequest(aRequest.get_GetSmscAddressRequest());
     default:
       MOZ_CRASH("Unknown type!");
   }
@@ -451,8 +468,9 @@ SmsRequestParent::DoRequest(const SendMessageRequest& aRequest)
       nsCOMPtr<nsISmsService> smsService = do_GetService(SMS_SERVICE_CONTRACTID);
       NS_ENSURE_TRUE(smsService, true);
 
-      const SendSmsMessageRequest &data = aRequest.get_SendSmsMessageRequest();
-      smsService->Send(data.number(), data.message(), data.silent(), this);
+      const SendSmsMessageRequest &req = aRequest.get_SendSmsMessageRequest();
+      smsService->Send(req.serviceId(), req.number(), req.message(),
+                       req.silent(), this);
     }
     break;
   case SendMessageRequest::TSendMmsMessageRequest: {
@@ -461,14 +479,14 @@ SmsRequestParent::DoRequest(const SendMessageRequest& aRequest)
 
       AutoJSContext cx;
       JS::Rooted<JS::Value> params(cx);
-      if (!GetParamsFromSendMmsMessageRequest(
-              cx,
-              aRequest.get_SendMmsMessageRequest(),
-              params.address())) {
+      const SendMmsMessageRequest &req = aRequest.get_SendMmsMessageRequest();
+      if (!GetParamsFromSendMmsMessageRequest(cx,
+                                              req,
+                                              params.address())) {
         NS_WARNING("SmsRequestParent: Fail to build MMS params.");
         return true;
       }
-      mmsService->Send(params, this);
+      mmsService->Send(req.serviceId(), params, this);
     }
     break;
   default:
@@ -512,6 +530,23 @@ SmsRequestParent::DoRequest(const GetMessageRequest& aRequest)
   return true;
 }
 
+bool
+SmsRequestParent::DoRequest(const GetSmscAddressRequest& aRequest)
+{
+  nsresult rv = NS_ERROR_FAILURE;
+
+  nsCOMPtr<nsISmsService> smsService = do_GetService(SMS_SERVICE_CONTRACTID);
+  if (smsService) {
+    rv = smsService->GetSmscAddress(aRequest.serviceId(), this);
+  }
+
+  if (NS_FAILED(rv)) {
+    return NS_SUCCEEDED(NotifyGetSmscAddressFailed(nsIMobileMessageCallback::INTERNAL_ERROR));
+  }
+
+  return true;
+}
+
 bool
 SmsRequestParent::DoRequest(const DeleteMessageRequest& aRequest)
 {
@@ -541,7 +576,7 @@ SmsRequestParent::DoRequest(const MarkMessageReadRequest& aRequest)
     do_GetService(MOBILE_MESSAGE_DATABASE_SERVICE_CONTRACTID);
   if (dbService) {
     rv = dbService->MarkMessageRead(aRequest.messageId(), aRequest.value(),
-                                    this);
+                                    aRequest.sendReadReport(), this);
   }
 
   if (NS_FAILED(rv)) {
@@ -683,6 +718,18 @@ SmsRequestParent::NotifyGetSegmentInfoForTextFailed(int32_t aError)
   return SendReply(ReplyGetSegmentInfoForTextFail(aError));
 }
 
+NS_IMETHODIMP
+SmsRequestParent::NotifyGetSmscAddress(const nsAString& aSmscAddress)
+{
+  return SendReply(ReplyGetSmscAddress(nsString(aSmscAddress)));
+}
+
+NS_IMETHODIMP
+SmsRequestParent::NotifyGetSmscAddressFailed(int32_t aError)
+{
+  return SendReply(ReplyGetSmscAddressFail(aError));
+}
+
 /*******************************************************************************
  * MobileMessageCursorParent
  ******************************************************************************/
