@@ -3,6 +3,7 @@
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
+
 #include "nsCOMPtr.h"
 #include "nsAutoPtr.h"
 #include "jsapi.h"
@@ -20,7 +21,6 @@
 #include "nsIURI.h"
 #include "nsIScriptContext.h"
 #include "nsIScriptGlobalObject.h"
-#include "nsIScriptGlobalObjectOwner.h"
 #include "nsIPrincipal.h"
 #include "nsIScriptSecurityManager.h"
 #include "nsIInterfaceRequestor.h"
@@ -118,22 +118,19 @@ static
 nsIScriptGlobalObject* GetGlobalObject(nsIChannel* aChannel)
 {
     // Get the global object owner from the channel
-    nsCOMPtr<nsIScriptGlobalObjectOwner> globalOwner;
-    NS_QueryNotificationCallbacks(aChannel, globalOwner);
-    if (!globalOwner) {
-        NS_WARNING("Unable to get an nsIScriptGlobalObjectOwner from the "
-                   "channel!");
-    }
-    if (!globalOwner) {
+    nsCOMPtr<nsIDocShell> docShell;
+    NS_QueryNotificationCallbacks(aChannel, docShell);
+    if (!docShell) {
+        NS_WARNING("Unable to get a docShell from the channel!");
         return nullptr;
     }
 
-    // So far so good: get the script context from its owner.
-    nsIScriptGlobalObject* global = globalOwner->GetScriptGlobalObject();
+    // So far so good: get the script global from its docshell
+    nsIScriptGlobalObject* global = docShell->GetScriptGlobalObject();
 
     NS_ASSERTION(global,
                  "Unable to get an nsIScriptGlobalObject from the "
-                 "ScriptGlobalObjectOwner!");
+                 "docShell!");
     return global;
 }
 
@@ -182,7 +179,9 @@ nsresult nsJSThunk::EvaluateScript(nsIChannel *aChannel,
             csp->LogViolationDetails(nsIContentSecurityPolicy::VIOLATION_TYPE_INLINE_SCRIPT,
                                      NS_ConvertUTF8toUTF16(asciiSpec),
                                      NS_ConvertUTF8toUTF16(mURL),
-                                     0);
+                                     0,
+                                     EmptyString(),
+                                     EmptyString());
         }
 
         //return early if inline scripts are not allowed
@@ -242,6 +241,7 @@ nsresult nsJSThunk::EvaluateScript(nsIChannel *aChannel,
 
     AutoPushJSContext cx(scriptContext->GetNativeContext());
     JS::Rooted<JSObject*> globalJSObject(cx, innerGlobal->GetGlobalJSObject());
+    NS_ENSURE_TRUE(globalJSObject, NS_ERROR_UNEXPECTED);
 
     if (!useSandbox) {
         //-- Don't outside a sandbox unless the script principal subsumes the
@@ -266,13 +266,7 @@ nsresult nsJSThunk::EvaluateScript(nsIChannel *aChannel,
 
         // First check to make sure it's OK to evaluate this script to
         // start with.  For example, script could be disabled.
-        bool ok;
-        rv = securityManager->CanExecuteScripts(cx, principal, &ok);
-        if (NS_FAILED(rv)) {
-            return rv;
-        }
-
-        if (!ok) {
+        if (!securityManager->ScriptAllowed(globalJSObject)) {
             // Treat this as returning undefined from the script.  That's what
             // nsJSContext does.
             return NS_ERROR_DOM_RETVAL_UNDEFINED;
@@ -281,7 +275,8 @@ nsresult nsJSThunk::EvaluateScript(nsIChannel *aChannel,
         nsIXPConnect *xpc = nsContentUtils::XPConnect();
 
         nsCOMPtr<nsIXPConnectJSObjectHolder> sandbox;
-        rv = xpc->CreateSandbox(cx, principal, getter_AddRefs(sandbox));
+        // Important: Use a null principal here
+        rv = xpc->CreateSandbox(cx, nullptr, getter_AddRefs(sandbox));
         NS_ENSURE_SUCCESS(rv, rv);
 
         // The nsXPConnect sandbox API gives us a wrapper to the sandbox for
@@ -293,13 +288,13 @@ nsresult nsJSThunk::EvaluateScript(nsIChannel *aChannel,
         sandboxObj = js::UncheckedUnwrap(sandboxObj);
         JSAutoCompartment ac(cx, sandboxObj);
 
-        // Push our JSContext on the context stack so the JS_ValueToString call (and
+        // Push our JSContext on the context stack so the EvalInSandboxObject call (and
         // JS_ReportPendingException, if relevant) will use the principal of cx.
         nsCxPusher pusher;
         pusher.Push(cx);
         rv = xpc->EvalInSandboxObject(NS_ConvertUTF8toUTF16(script),
                                       /* filename = */ nullptr, cx,
-                                      sandboxObj, true, v.address());
+                                      sandboxObj, true, &v);
 
         // Propagate and report exceptions that happened in the
         // sandbox.
@@ -329,7 +324,7 @@ nsresult nsJSThunk::EvaluateScript(nsIChannel *aChannel,
 
     // If we took the sandbox path above, v might be in the sandbox
     // compartment.
-    if (!JS_WrapValue(cx, v.address())) {
+    if (!JS_WrapValue(cx, &v)) {
         return NS_ERROR_OUT_OF_MEMORY;
     }
 
@@ -1299,8 +1294,10 @@ nsJSURI::Read(nsIObjectInputStream* aStream)
     if (NS_FAILED(rv)) return rv;
 
     if (haveBase) {
-        rv = aStream->ReadObject(true, getter_AddRefs(mBaseURI));
+        nsCOMPtr<nsISupports> supports;
+        rv = aStream->ReadObject(true, getter_AddRefs(supports));
         if (NS_FAILED(rv)) return rv;
+        mBaseURI = do_QueryInterface(supports);
     }
 
     return NS_OK;
