@@ -4,9 +4,10 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "nsDOMOfflineResourceList.h"
+#include "nsIDOMEvent.h"
 #include "nsIScriptSecurityManager.h"
 #include "nsError.h"
-#include "nsDOMLists.h"
+#include "mozilla/dom/DOMStringList.h"
 #include "nsIPrefetchService.h"
 #include "nsCPrefetchService.h"
 #include "nsNetUtil.h"
@@ -14,7 +15,6 @@
 #include "nsICacheSession.h"
 #include "nsICacheService.h"
 #include "nsIOfflineCacheUpdate.h"
-#include "nsIDOMLoadStatus.h"
 #include "nsAutoPtr.h"
 #include "nsContentUtils.h"
 #include "nsEventDispatcher.h"
@@ -80,7 +80,8 @@ NS_IMPL_EVENT_HANDLER(nsDOMOfflineResourceList, obsolete)
 nsDOMOfflineResourceList::nsDOMOfflineResourceList(nsIURI *aManifestURI,
                                                    nsIURI *aDocumentURI,
                                                    nsPIDOMWindow *aWindow)
-  : mInitialized(false)
+  : nsDOMEventTargetHelper(aWindow)
+  , mInitialized(false)
   , mManifestURI(aManifestURI)
   , mDocumentURI(aDocumentURI)
   , mExposeCacheUpdateStatus(true)
@@ -88,8 +89,6 @@ nsDOMOfflineResourceList::nsDOMOfflineResourceList(nsIURI *aManifestURI,
   , mCachedKeys(nullptr)
   , mCachedKeysCount(0)
 {
-  BindToOwner(aWindow);
-  SetIsDOMBinding();
 }
 
 nsDOMOfflineResourceList::~nsDOMOfflineResourceList()
@@ -183,32 +182,35 @@ nsDOMOfflineResourceList::Disconnect()
 // nsDOMOfflineResourceList::nsIDOMOfflineResourceList
 //
 
-NS_IMETHODIMP
-nsDOMOfflineResourceList::GetMozItems(nsIDOMDOMStringList **aItems)
+already_AddRefed<DOMStringList>
+nsDOMOfflineResourceList::GetMozItems(ErrorResult& aRv)
 {
-  if (IS_CHILD_PROCESS()) 
-    return NS_ERROR_NOT_IMPLEMENTED;
+  if (IS_CHILD_PROCESS()) {
+    aRv.Throw(NS_ERROR_NOT_IMPLEMENTED);
+    return nullptr;
+  }
 
-  *aItems = nullptr;
-
-  nsRefPtr<nsDOMStringList> items = new nsDOMStringList();
+  nsRefPtr<DOMStringList> items = new DOMStringList();
 
   // If we are not associated with an application cache, return an
   // empty list.
   nsCOMPtr<nsIApplicationCache> appCache = GetDocumentAppCache();
   if (!appCache) {
-    NS_ADDREF(*aItems = items);
-    return NS_OK;
+    return items.forget();
   }
 
-  nsresult rv = Init();
-  NS_ENSURE_SUCCESS(rv, rv);
+  aRv = Init();
+  if (aRv.Failed()) {
+    return nullptr;
+  }
 
   uint32_t length;
   char **keys;
-  rv = appCache->GatherEntries(nsIApplicationCache::ITEM_DYNAMIC,
-                               &length, &keys);
-  NS_ENSURE_SUCCESS(rv, rv);
+  aRv = appCache->GatherEntries(nsIApplicationCache::ITEM_DYNAMIC,
+                                &length, &keys);
+  if (aRv.Failed()) {
+    return nullptr;
+  }
 
   for (uint32_t i = 0; i < length; i++) {
     items->Add(NS_ConvertUTF8toUTF16(keys[i]));
@@ -216,8 +218,16 @@ nsDOMOfflineResourceList::GetMozItems(nsIDOMDOMStringList **aItems)
 
   NS_FREE_XPCOM_ALLOCATED_POINTER_ARRAY(length, keys);
 
-  NS_ADDREF(*aItems = items);
-  return NS_OK;
+  return items.forget();
+}
+
+NS_IMETHODIMP
+nsDOMOfflineResourceList::GetMozItems(nsISupports** aItems)
+{
+  ErrorResult rv;
+  nsRefPtr<DOMStringList> items = GetMozItems(rv);
+  items.forget(aItems);
+  return rv.ErrorCode();
 }
 
 NS_IMETHODIMP
@@ -567,7 +577,7 @@ nsDOMOfflineResourceList::SendEvent(const nsAString &aEventName)
 NS_IMETHODIMP
 nsDOMOfflineResourceList::Observe(nsISupports *aSubject,
                                     const char *aTopic,
-                                    const PRUnichar *aData)
+                                    const char16_t *aData)
 {
   if (!strcmp(aTopic, "offline-cache-update-added")) {
     nsCOMPtr<nsIOfflineCacheUpdate> update = do_QueryInterface(aSubject);
@@ -803,9 +813,16 @@ nsDOMOfflineResourceList::CacheKeys()
   nsCOMPtr<nsIWebNavigation> webNav = do_GetInterface(window);
   nsCOMPtr<nsILoadContext> loadContext = do_QueryInterface(webNav);
 
+  uint32_t appId = 0;
+  bool inBrowser = false;
+  if (loadContext) {
+    loadContext->GetAppId(&appId);
+    loadContext->GetIsInBrowserElement(&inBrowser);
+  }
+
   nsAutoCString groupID;
-  mApplicationCacheService->BuildGroupID(
-      mManifestURI, loadContext, groupID);
+  mApplicationCacheService->BuildGroupIDForApp(
+      mManifestURI, appId, inBrowser, groupID);
 
   nsCOMPtr<nsIApplicationCache> appCache;
   mApplicationCacheService->GetActiveCache(groupID,
