@@ -14,11 +14,15 @@
  */
 
 #include <android/log.h>
+#include <cutils/properties.h>
 
 #include "AudioChannelService.h"
 #include "AudioManager.h"
 
 #include "nsIObserverService.h"
+#ifdef MOZ_B2G_RIL
+#include "nsIRadioInterfaceLayer.h"
+#endif
 #include "nsISettingsService.h"
 #include "nsPrintfCString.h"
 
@@ -27,8 +31,7 @@
 #include "base/message_loop.h"
 
 #include "BluetoothCommon.h"
-#include "BluetoothProfileManagerBase.h"
-#include "BluetoothHfpManager.h"
+#include "BluetoothHfpManagerBase.h"
 
 #include "nsJSUtils.h"
 #include "nsCxPusher.h"
@@ -44,10 +47,10 @@ using namespace mozilla::dom::bluetooth;
 
 #define LOG(args...)  __android_log_print(ANDROID_LOG_INFO, "AudioManager" , ## args)
 
-#define HEADPHONES_STATUS_HEADSET   NS_LITERAL_STRING("headset").get()
-#define HEADPHONES_STATUS_HEADPHONE NS_LITERAL_STRING("headphone").get()
-#define HEADPHONES_STATUS_OFF       NS_LITERAL_STRING("off").get()
-#define HEADPHONES_STATUS_UNKNOWN   NS_LITERAL_STRING("unknown").get()
+#define HEADPHONES_STATUS_HEADSET   MOZ_UTF16("headset")
+#define HEADPHONES_STATUS_HEADPHONE MOZ_UTF16("headphone")
+#define HEADPHONES_STATUS_OFF       MOZ_UTF16("off")
+#define HEADPHONES_STATUS_UNKNOWN   MOZ_UTF16("unknown")
 #define HEADPHONES_STATUS_CHANGED   "headphones-status-changed"
 #define MOZ_SETTINGS_CHANGE_ID      "mozsettings-changed"
 
@@ -118,7 +121,7 @@ public:
 
   AudioChannelVolInitCallback() {}
 
-  NS_IMETHOD Handle(const nsAString& aName, const JS::Value& aResult)
+  NS_IMETHOD Handle(const nsAString& aName, JS::Handle<JS::Value> aResult)
   {
     nsCOMPtr<nsIAudioManager> audioManager =
       do_GetService(NS_AUDIOMANAGER_CONTRACTID);
@@ -226,8 +229,8 @@ AudioManager::HandleBluetoothStatusChanged(nsISupports* aSubject,
 #ifdef MOZ_B2G_BT
   bool status;
   if (!strcmp(aTopic, BLUETOOTH_SCO_STATUS_CHANGED_ID)) {
-    BluetoothHfpManager* hfp =
-      static_cast<BluetoothHfpManager*>(aSubject);
+    BluetoothHfpManagerBase* hfp =
+      static_cast<BluetoothHfpManagerBase*>(aSubject);
     status = hfp->IsScoConnected();
   } else {
     BluetoothProfileManagerBase* profile =
@@ -277,7 +280,7 @@ AudioManager::HandleBluetoothStatusChanged(nsISupports* aSubject,
 nsresult
 AudioManager::Observe(nsISupports* aSubject,
                       const char* aTopic,
-                      const PRUnichar* aData)
+                      const char16_t* aData)
 {
   if ((strcmp(aTopic, BLUETOOTH_SCO_STATUS_CHANGED_ID) == 0) ||
       (strcmp(aTopic, BLUETOOTH_HFP_STATUS_CHANGED_ID) == 0) ||
@@ -310,7 +313,7 @@ AudioManager::Observe(nsISupports* aSubject,
       return NS_OK;
     }
 
-    JS::RootedString jsKey(cx, JS_ValueToString(cx, key));
+    JS::Rooted<JSString*> jsKey(cx, JS::ToString(cx, key));
     if (!jsKey) {
       return NS_OK;
     }
@@ -368,8 +371,12 @@ public:
   }
 };
 
-AudioManager::AudioManager() : mPhoneState(PHONE_STATE_CURRENT),
-                 mObserver(new HeadphoneSwitchObserver())
+AudioManager::AudioManager()
+  : mPhoneState(PHONE_STATE_CURRENT)
+  , mObserver(new HeadphoneSwitchObserver())
+#ifdef MOZ_B2G_RIL
+  , mMuteCallToRIL(false)
+#endif
 {
   RegisterSwitchObserver(SWITCH_HEADPHONES, mObserver);
 
@@ -390,7 +397,7 @@ AudioManager::AudioManager() : mPhoneState(PHONE_STATE_CURRENT),
     do_GetService("@mozilla.org/settingsService;1");
   NS_ENSURE_TRUE_VOID(settingsService);
   nsCOMPtr<nsISettingsServiceLock> lock;
-  nsresult rv = settingsService->CreateLock(getter_AddRefs(lock));
+  nsresult rv = settingsService->CreateLock(nullptr, getter_AddRefs(lock));
   NS_ENSURE_SUCCESS_VOID(rv);
   nsCOMPtr<nsISettingsServiceCallback> callback = new AudioChannelVolInitCallback();
   NS_ENSURE_TRUE_VOID(callback);
@@ -419,6 +426,14 @@ AudioManager::AudioManager() : mPhoneState(PHONE_STATE_CURRENT),
   if (NS_FAILED(obs->AddObserver(this, MOZ_SETTINGS_CHANGE_ID, false))) {
     NS_WARNING("Failed to add mozsettings-changed observer!");
   }
+
+#ifdef MOZ_B2G_RIL
+  char value[PROPERTY_VALUE_MAX];
+  property_get("ro.moz.mute.call.to_ril", value, "false");
+  if (!strcmp(value, "true")) {
+    mMuteCallToRIL = true;
+  }
+#endif
 }
 
 AudioManager::~AudioManager() {
@@ -443,6 +458,14 @@ AudioManager::~AudioManager() {
 NS_IMETHODIMP
 AudioManager::GetMicrophoneMuted(bool* aMicrophoneMuted)
 {
+#ifdef MOZ_B2G_RIL
+  if (mMuteCallToRIL) {
+    // Simply return cached mIsMicMuted if mute call go via RIL.
+    *aMicrophoneMuted = mIsMicMuted;
+    return NS_OK;
+  }
+#endif
+
   if (AudioSystem::isMicrophoneMuted(aMicrophoneMuted)) {
     return NS_ERROR_FAILURE;
   }
@@ -452,10 +475,19 @@ AudioManager::GetMicrophoneMuted(bool* aMicrophoneMuted)
 NS_IMETHODIMP
 AudioManager::SetMicrophoneMuted(bool aMicrophoneMuted)
 {
-  if (AudioSystem::muteMicrophone(aMicrophoneMuted)) {
-    return NS_ERROR_FAILURE;
+  if (!AudioSystem::muteMicrophone(aMicrophoneMuted)) {
+#ifdef MOZ_B2G_RIL
+    if (mMuteCallToRIL) {
+      // Extra mute request to RIL for specific platform.
+      nsCOMPtr<nsIRadioInterfaceLayer> ril = do_GetService("@mozilla.org/ril;1");
+      NS_ENSURE_TRUE(ril, NS_ERROR_FAILURE);
+      ril->SetMicrophoneMuted(aMicrophoneMuted);
+      mIsMicMuted = aMicrophoneMuted;
+    }
+#endif
+    return NS_OK;
   }
-  return NS_OK;
+  return NS_ERROR_FAILURE;
 }
 
 NS_IMETHODIMP
@@ -479,12 +511,6 @@ AudioManager::SetPhoneState(int32_t aState)
     obs->NotifyObservers(nullptr, "phone-state-changed", state.get());
   }
 
-  // follow the switch audio path logic for android, Bug 897364
-  int usage;
-  GetForceForUse(nsIAudioManager::USE_COMMUNICATION, &usage);
-  if (aState == PHONE_STATE_NORMAL && usage == nsIAudioManager::FORCE_BT_SCO) {
-    SetForceForUse(nsIAudioManager::USE_COMMUNICATION, nsIAudioManager::FORCE_NONE);
-  }
 #if ANDROID_VERSION < 17
   if (AudioSystem::setPhoneState(aState)) {
 #else
@@ -505,9 +531,9 @@ AudioManager::SetPhoneState(int32_t aState)
     MOZ_ASSERT(mPhoneAudioAgent);
     if (aState == PHONE_STATE_IN_CALL) {
       // Telephony doesn't be paused by any other channels.
-      mPhoneAudioAgent->Init(AUDIO_CHANNEL_TELEPHONY, nullptr);
+      mPhoneAudioAgent->Init(nullptr, AUDIO_CHANNEL_TELEPHONY, nullptr);
     } else {
-      mPhoneAudioAgent->Init(AUDIO_CHANNEL_RINGER, nullptr);
+      mPhoneAudioAgent->Init(nullptr, AUDIO_CHANNEL_RINGER, nullptr);
     }
 
     // Telephony can always play.
@@ -580,20 +606,23 @@ AudioManager::SetFmRadioAudioEnabled(bool aFmRadioAudioEnabled)
 
 NS_IMETHODIMP
 AudioManager::SetAudioChannelVolume(int32_t aChannel, int32_t aIndex) {
-  status_t status;
+  nsresult status;
 
   switch (aChannel) {
     case AUDIO_CHANNEL_CONTENT:
-      status = SetStreamVolumeIndex(AUDIO_STREAM_MUSIC, aIndex);
-      status += SetStreamVolumeIndex(AUDIO_STREAM_SYSTEM, aIndex);
       // sync FMRadio's volume with content channel.
       if (IsDeviceOn(AUDIO_DEVICE_OUT_FM)) {
-        status += SetStreamVolumeIndex(AUDIO_STREAM_FM, aIndex);
+        status = SetStreamVolumeIndex(AUDIO_STREAM_FM, aIndex);
+        NS_ENSURE_SUCCESS(status, status);
       }
+      status = SetStreamVolumeIndex(AUDIO_STREAM_MUSIC, aIndex);
+      NS_ENSURE_SUCCESS(status, status);
+      status = SetStreamVolumeIndex(AUDIO_STREAM_SYSTEM, aIndex);
       break;
     case AUDIO_CHANNEL_NOTIFICATION:
       status = SetStreamVolumeIndex(AUDIO_STREAM_NOTIFICATION, aIndex);
-      status += SetStreamVolumeIndex(AUDIO_STREAM_RING, aIndex);
+      NS_ENSURE_SUCCESS(status, status);
+      status = SetStreamVolumeIndex(AUDIO_STREAM_RING, aIndex);
       break;
     case AUDIO_CHANNEL_ALARM:
       status = SetStreamVolumeIndex(AUDIO_STREAM_ALARM, aIndex);
@@ -605,7 +634,7 @@ AudioManager::SetAudioChannelVolume(int32_t aChannel, int32_t aIndex) {
       return NS_ERROR_INVALID_ARG;
   }
 
-  return status ? NS_ERROR_FAILURE : NS_OK;
+  return status;
 }
 
 NS_IMETHODIMP
@@ -670,17 +699,19 @@ AudioManager::GetMaxAudioChannelVolume(int32_t aChannel, int32_t* aMaxIndex) {
    return NS_OK;
 }
 
-status_t
+nsresult
 AudioManager::SetStreamVolumeIndex(int32_t aStream, int32_t aIndex) {
   if (aIndex < 0 || aIndex > sMaxStreamVolumeTbl[aStream]) {
-    return BAD_VALUE;
+    return NS_ERROR_INVALID_ARG;
   }
 
   mCurrentStreamVolumeTbl[aStream] = aIndex;
+  status_t status;
 #if ANDROID_VERSION < 17
-  return AudioSystem::setStreamVolumeIndex(
-           static_cast<audio_stream_type_t>(aStream),
-           aIndex);
+   status = AudioSystem::setStreamVolumeIndex(
+              static_cast<audio_stream_type_t>(aStream),
+              aIndex);
+   return status ? NS_ERROR_FAILURE : NS_OK;
 #else
   int device = 0;
 
@@ -691,16 +722,21 @@ AudioManager::SetStreamVolumeIndex(int32_t aStream, int32_t aIndex) {
   }
 
   if (device != 0) {
-    return AudioSystem::setStreamVolumeIndex(
-             static_cast<audio_stream_type_t>(aStream),
-             aIndex,
-             device);
+    status = AudioSystem::setStreamVolumeIndex(
+               static_cast<audio_stream_type_t>(aStream),
+               aIndex,
+               device);
+    return status ? NS_ERROR_FAILURE : NS_OK;
   }
 
-  status_t status = AudioSystem::setStreamVolumeIndex(
-                      static_cast<audio_stream_type_t>(aStream),
-                      aIndex,
-                      AUDIO_DEVICE_OUT_SPEAKER);
+  status = AudioSystem::setStreamVolumeIndex(
+             static_cast<audio_stream_type_t>(aStream),
+             aIndex,
+             AUDIO_DEVICE_OUT_BLUETOOTH_A2DP);
+  status += AudioSystem::setStreamVolumeIndex(
+              static_cast<audio_stream_type_t>(aStream),
+              aIndex,
+              AUDIO_DEVICE_OUT_SPEAKER);
   status += AudioSystem::setStreamVolumeIndex(
               static_cast<audio_stream_type_t>(aStream),
               aIndex,
@@ -713,21 +749,21 @@ AudioManager::SetStreamVolumeIndex(int32_t aStream, int32_t aIndex) {
               static_cast<audio_stream_type_t>(aStream),
               aIndex,
               AUDIO_DEVICE_OUT_EARPIECE);
-  return status;
+  return status ? NS_ERROR_FAILURE : NS_OK;
 #endif
 }
 
-status_t
+nsresult
 AudioManager::GetStreamVolumeIndex(int32_t aStream, int32_t *aIndex) {
   if (!aIndex) {
-    return BAD_VALUE;
+    return NS_ERROR_INVALID_ARG;
   }
 
   if (aStream <= AUDIO_STREAM_DEFAULT || aStream >= AUDIO_STREAM_MAX) {
-    return BAD_VALUE;
+    return NS_ERROR_INVALID_ARG;
   }
 
   *aIndex = mCurrentStreamVolumeTbl[aStream];
 
-  return NO_ERROR;
+  return NS_OK;
 }
