@@ -4,25 +4,24 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "URL.h"
-#include "File.h"
-
-#include "nsTraceRefcnt.h"
-
-#include "WorkerPrivate.h"
-#include "nsThreadUtils.h"
-
-#include "nsPIDOMWindow.h"
-#include "nsGlobalWindow.h"
-#include "nsHostObjectProtocolHandler.h"
-#include "nsServiceManagerUtils.h"
 
 #include "nsIDocument.h"
 #include "nsIDOMFile.h"
-
-#include "DOMBindingInlines.h"
-#include "mozilla/dom/URL.h"
 #include "nsIIOService.h"
+#include "nsPIDOMWindow.h"
+
+#include "mozilla/dom/URL.h"
+#include "mozilla/dom/URLBinding.h"
+#include "mozilla/dom/URLSearchParams.h"
+#include "nsGlobalWindow.h"
+#include "nsHostObjectProtocolHandler.h"
 #include "nsNetCID.h"
+#include "nsServiceManagerUtils.h"
+#include "nsThreadUtils.h"
+
+#include "File.h"
+#include "WorkerPrivate.h"
+#include "WorkerRunnable.h"
 
 BEGIN_WORKERS_NAMESPACE
 using mozilla::dom::GlobalObject;
@@ -68,43 +67,7 @@ class URLRunnable : public nsRunnable
 {
 protected:
   WorkerPrivate* mWorkerPrivate;
-  uint32_t mSyncQueueKey;
-
-private:
-  class ResponseRunnable : public WorkerSyncRunnable
-  {
-    uint32_t mSyncQueueKey;
-
-  public:
-    ResponseRunnable(WorkerPrivate* aWorkerPrivate,
-                     uint32_t aSyncQueueKey)
-    : WorkerSyncRunnable(aWorkerPrivate, aSyncQueueKey, false),
-      mSyncQueueKey(aSyncQueueKey)
-    {
-      NS_ASSERTION(aWorkerPrivate, "Don't hand me a null WorkerPrivate!");
-    }
-
-    bool
-    WorkerRun(JSContext* aCx, WorkerPrivate* aWorkerPrivate)
-    {
-      aWorkerPrivate->StopSyncLoop(mSyncQueueKey, true);
-      return true;
-    }
-
-    bool
-    PreDispatch(JSContext* aCx, WorkerPrivate* aWorkerPrivate)
-    {
-      AssertIsOnMainThread();
-      return true;
-    }
-
-    void
-    PostDispatch(JSContext* aCx, WorkerPrivate* aWorkerPrivate,
-                 bool aDispatchResult)
-    {
-      AssertIsOnMainThread();
-    }
-  };
+  nsCOMPtr<nsIEventTarget> mSyncLoopTarget;
 
 protected:
   URLRunnable(WorkerPrivate* aWorkerPrivate)
@@ -118,15 +81,17 @@ public:
   Dispatch(JSContext* aCx)
   {
     mWorkerPrivate->AssertIsOnWorkerThread();
+
     AutoSyncLoopHolder syncLoop(mWorkerPrivate);
-    mSyncQueueKey = syncLoop.SyncQueueKey();
+
+    mSyncLoopTarget = syncLoop.EventTarget();
 
     if (NS_FAILED(NS_DispatchToMainThread(this, NS_DISPATCH_NORMAL))) {
       JS_ReportError(aCx, "Failed to dispatch to main thread!");
       return false;
     }
 
-    return syncLoop.RunAndForget(aCx);
+    return syncLoop.Run();
   }
 
 private:
@@ -136,8 +101,10 @@ private:
 
     MainThreadRun();
 
-    nsRefPtr<ResponseRunnable> response =
-      new ResponseRunnable(mWorkerPrivate, mSyncQueueKey);
+    nsRefPtr<MainThreadStopSyncLoopRunnable> response =
+      new MainThreadStopSyncLoopRunnable(mWorkerPrivate,
+                                         mSyncLoopTarget.forget(),
+                                         true);
     if (!response->Dispatch(nullptr)) {
       NS_WARNING("Failed to dispatch response!");
     }
@@ -345,10 +312,10 @@ public:
   }
 };
 
-class TeardownRunnable : public nsRunnable
+class TeardownURLRunnable : public nsRunnable
 {
 public:
-  TeardownRunnable(URLProxy* aURLProxy)
+  TeardownURLRunnable(URLProxy* aURLProxy)
     : mURLProxy(aURLProxy)
   {
   }
@@ -538,6 +505,17 @@ private:
   mozilla::ErrorResult& mRv;
 };
 
+NS_IMPL_CYCLE_COLLECTION_1(URL, mSearchParams)
+
+// The reason for using worker::URL is to have different refcnt logging than
+// for main thread URL.
+NS_IMPL_CYCLE_COLLECTING_ADDREF(workers::URL)
+NS_IMPL_CYCLE_COLLECTING_RELEASE(workers::URL)
+
+NS_INTERFACE_MAP_BEGIN_CYCLE_COLLECTION(URL)
+  NS_INTERFACE_MAP_ENTRY(nsISupports)
+NS_INTERFACE_MAP_END
+
 // static
 URL*
 URL::Constructor(const GlobalObject& aGlobal, const nsAString& aUrl,
@@ -559,14 +537,7 @@ URL::Constructor(const GlobalObject& aGlobal, const nsAString& aUrl,
     return nullptr;
   }
 
-  nsRefPtr<URL> url = new URL(workerPrivate, proxy);
-
-  if (!Wrap(aGlobal.GetContext(), aGlobal.Get(), url)) {
-    aRv.Throw(NS_ERROR_FAILURE);
-    return nullptr;
-  }
-
-  return url;
+  return new URL(workerPrivate, proxy);
 }
 
 // static
@@ -590,27 +561,23 @@ URL::Constructor(const GlobalObject& aGlobal, const nsAString& aUrl,
     return nullptr;
   }
 
-  nsRefPtr<URL> url = new URL(workerPrivate, proxy);
-
-  if (!Wrap(aGlobal.GetContext(), aGlobal.Get(), url)) {
-    aRv.Throw(NS_ERROR_FAILURE);
-    return nullptr;
-  }
-
-  return url;
+  return new URL(workerPrivate, proxy);
 }
 
 URL::URL(WorkerPrivate* aWorkerPrivate, URLProxy* aURLProxy)
-  : DOMBindingBase(aWorkerPrivate->GetJSContext())
-  , mWorkerPrivate(aWorkerPrivate)
+  : mWorkerPrivate(aWorkerPrivate)
   , mURLProxy(aURLProxy)
 {
+  MOZ_COUNT_CTOR(workers::URL);
 }
 
 URL::~URL()
 {
+  MOZ_COUNT_DTOR(workers::URL);
+
   if (mURLProxy) {
-    nsRefPtr<TeardownRunnable> runnable = new TeardownRunnable(mURLProxy);
+    nsRefPtr<TeardownURLRunnable> runnable =
+      new TeardownURLRunnable(mURLProxy);
     mURLProxy = nullptr;
 
     if (NS_FAILED(NS_DispatchToMainThread(runnable))) {
@@ -619,16 +586,10 @@ URL::~URL()
   }
 }
 
-void
-URL::_trace(JSTracer* aTrc)
+JSObject*
+URL::WrapObject(JSContext* aCx, JS::Handle<JSObject*> aScope)
 {
-  DOMBindingBase::_trace(aTrc);
-}
-
-void
-URL::_finalize(JSFreeOp* aFop)
-{
-  DOMBindingBase::_finalize(aFop);
+  return URLBinding_workers::Wrap(aCx, aScope, this);
 }
 
 void
@@ -653,6 +614,8 @@ URL::SetHref(const nsAString& aHref, ErrorResult& aRv)
   if (!runnable->Dispatch(mWorkerPrivate->GetJSContext())) {
     JS_ReportPendingException(mWorkerPrivate->GetJSContext());
   }
+
+  UpdateURLSearchParams();
 }
 
 void
@@ -856,6 +819,13 @@ URL::GetSearch(nsString& aSearch) const
 
 void
 URL::SetSearch(const nsAString& aSearch)
+{
+  SetSearchInternal(aSearch);
+  UpdateURLSearchParams();
+}
+
+void
+URL::SetSearchInternal(const nsAString& aSearch)
 {
   ErrorResult rv;
   nsRefPtr<SetterRunnable> runnable =
@@ -867,6 +837,32 @@ URL::SetSearch(const nsAString& aSearch)
   }
 }
 
+mozilla::dom::URLSearchParams*
+URL::GetSearchParams()
+{
+  CreateSearchParamsIfNeeded();
+  return mSearchParams;
+}
+
+void
+URL::SetSearchParams(URLSearchParams* aSearchParams)
+{
+  if (!aSearchParams) {
+    return;
+  }
+
+  if (mSearchParams) {
+    mSearchParams->RemoveObserver(this);
+  }
+
+  mSearchParams = aSearchParams;
+  mSearchParams->AddObserver(this);
+
+  nsString search;
+  mSearchParams->Serialize(search);
+  SetSearchInternal(search);
+}
+
 void
 URL::GetHash(nsString& aHash) const
 {
@@ -943,4 +939,34 @@ URL::RevokeObjectURL(const GlobalObject& aGlobal, const nsAString& aUrl)
   }
 }
 
+void
+URL::URLSearchParamsUpdated()
+{
+  MOZ_ASSERT(mSearchParams);
+
+  nsString search;
+  mSearchParams->Serialize(search);
+  SetSearchInternal(search);
+}
+
+void
+URL::UpdateURLSearchParams()
+{
+  if (mSearchParams) {
+    nsString search;
+    GetSearch(search);
+    mSearchParams->ParseInput(NS_ConvertUTF16toUTF8(Substring(search, 1)), this);
+  }
+}
+
+void
+URL::CreateSearchParamsIfNeeded()
+{
+  if (!mSearchParams) {
+    mSearchParams = new URLSearchParams();
+    mSearchParams->AddObserver(this);
+    UpdateURLSearchParams();
+  }
+}
+
 END_WORKERS_NAMESPACE
