@@ -119,7 +119,6 @@ public:
 
   // IURLSearchParamsObserver
   void URLSearchParamsUpdated() MOZ_OVERRIDE;
-  void URLSearchParamsNeedsUpdates() MOZ_OVERRIDE;
 
 private:
   URLProxy* GetURLProxy() const
@@ -131,6 +130,8 @@ private:
 
   void SetSearchInternal(const nsAString& aSearch);
 
+  void UpdateURLSearchParams();
+
   WorkerPrivate* mWorkerPrivate;
   nsRefPtr<URLProxy> mURLProxy;
   nsRefPtr<URLSearchParams> mSearchParams;
