@@ -301,11 +301,17 @@ public class GeckoSmsManager
    * defined in dom/mobilemessage/interfaces/nsISmsRequestManager.idl. They are owned
    * owned by the interface.
    */
-  public final static int kNoError       = 0;
-  public final static int kNoSignalError = 1;
-  public final static int kNotFoundError = 2;
-  public final static int kUnknownError  = 3;
-  public final static int kInternalError = 4;
+  public final static int kNoError               = 0;
+  public final static int kNoSignalError         = 1;
+  public final static int kNotFoundError         = 2;
+  public final static int kUnknownError          = 3;
+  public final static int kInternalError         = 4;
+  public final static int kNoSimCardError        = 5;
+  public final static int kRadioDisabledError    = 6;
+  public final static int kInvalidAddressError   = 7;
+  public final static int kFdnCheckError         = 8;
+  public final static int kNonActiveSimCardError = 9;
+  public final static int kStorageFullError      = 10;
 
   private final static int kMaxMessageSize    = 160;
 
@@ -423,7 +429,7 @@ public class GeckoSmsManager
                                  ? Envelope.SubParts.SENT_PART
                                  : Envelope.SubParts.DELIVERED_PART;
       envelope.decreaseRemainingParts(part);
- 
+
 
       if (getResultCode() != Activity.RESULT_OK) {
         switch (getResultCode()) {
