@@ -13,7 +13,7 @@
 
 using namespace mozilla;
 
-PLDHashTableOps nsCommandParams::sHashOps =
+const PLDHashTableOps nsCommandParams::sHashOps =
 {
     PL_DHashAllocTable,
     PL_DHashFreeTable,
@@ -42,9 +42,8 @@ nsCommandParams::~nsCommandParams()
 nsresult
 nsCommandParams::Init()
 {
-  if (!PL_DHashTableInit(&mValuesHash, &sHashOps, (void *)this, sizeof(HashEntry), 4))
-    return NS_ERROR_FAILURE;
-    
+  PL_DHashTableInit(&mValuesHash, &sHashOps, (void *)this, sizeof(HashEntry), 4);
+
   return NS_OK;
 }
 
