@@ -20,6 +20,7 @@
 
 #include "gfxPlatform.h"
 #include "gfx2DGlue.h"
+#include "gfxPrefs.h"
 
 #define DEBUG_GRALLOC
 #ifdef DEBUG_GRALLOC
@@ -34,9 +35,6 @@ using namespace gl;
 using namespace layers;
 using namespace android;
 
-static bool sForceReadPixelsToFence = false;
-
-
 SurfaceFactory_Gralloc::SurfaceFactory_Gralloc(GLContext* prodGL,
                                                const SurfaceCaps& caps,
                                                layers::ISurfaceAllocator* allocator)
@@ -58,14 +56,6 @@ SharedSurface_Gralloc::Create(GLContext* prodGL,
                               bool hasAlpha,
                               ISurfaceAllocator* allocator)
 {
-    static bool runOnce = true;
-    if (runOnce) {
-        sForceReadPixelsToFence = false;
-        mozilla::Preferences::AddBoolVarCache(&sForceReadPixelsToFence,
-                                              "gfx.gralloc.fence-with-readpixels");
-        runOnce = false;
-    }
-
     GLLibraryEGL* egl = &sEGLLibrary;
     MOZ_ASSERT(egl);
 
@@ -80,7 +70,7 @@ SharedSurface_Gralloc::Create(GLContext* prodGL,
     gfxImageFormat format
       = gfxPlatform::GetPlatform()->OptimalFormatForContent(type);
 
-    GrallocTextureClientOGL* grallocTC =
+    RefPtr<GrallocTextureClientOGL> grallocTC =
       new GrallocTextureClientOGL(
           allocator,
           gfx::ImageFormatToSurfaceFormat(format),
@@ -102,7 +92,6 @@ SharedSurface_Gralloc::Create(GLContext* prodGL,
                                        LOCAL_EGL_NATIVE_BUFFER_ANDROID,
                                        clientBuffer, attrs);
     if (!image) {
-        grallocTC->DropTextureData()->DeallocateSharedData(allocator);
         return nullptr;
     }
 
@@ -141,7 +130,7 @@ SharedSurface_Gralloc::~SharedSurface_Gralloc()
     DEBUG_PRINT("[SharedSurface_Gralloc %p] destroyed\n", this);
 
     mGL->MakeCurrent();
-    mGL->fDeleteTextures(1, (GLuint*)&mProdTex);
+    mGL->fDeleteTextures(1, &mProdTex);
 }
 
 void
@@ -150,7 +139,7 @@ SharedSurface_Gralloc::Fence()
     // We should be able to rely on genlock write locks/read locks.
     // But they're broken on some configs, and even a glFinish doesn't
     // work.  glReadPixels seems to, though.
-    if (sForceReadPixelsToFence) {
+    if (gfxPrefs::GrallocFenceWithReadPixels()) {
         mGL->MakeCurrent();
         // read a 1x1 pixel
         unsigned char pixels[4];
@@ -163,14 +152,3 @@ SharedSurface_Gralloc::WaitSync()
 {
     return true;
 }
-
-void
-SharedSurface_Gralloc::LockProdImpl()
-{
-}
-
-void
-SharedSurface_Gralloc::UnlockProdImpl()
-{
-}
-
