@@ -12,6 +12,7 @@
 #include "mozilla/layers/GrallocTextureClient.h"
 #include "gfx2DGlue.h"
 #include "gfxImageSurface.h"
+#include "YCbCrUtils.h"                 // for YCbCr conversions
 
 #include <OMX_IVCommon.h>
 #include <ColorConverter.h>
@@ -92,6 +93,9 @@ GrallocImage::SetData(const Data& aData)
                                        GraphicBuffer::USAGE_SW_WRITE_OFTEN |
                                        GraphicBuffer::USAGE_HW_TEXTURE,
                                        &desc);
+    if (desc.type() == SurfaceDescriptor::T__None) {
+      return;
+    }
     mBufferAllocated = true;
     mGraphicBufferLocked = new GraphicBufferLocked(desc);
   }
@@ -252,8 +256,7 @@ GrallocImage::DeprecatedGetAsSurface()
                             width, height);
 
     return imageSurface.forget();
-  }
-  else if (format == HAL_PIXEL_FORMAT_YCrCb_420_SP) {
+  } else if (format == HAL_PIXEL_FORMAT_YCrCb_420_SP) {
     uint32_t uvOffset = height * width;
     ConvertYVU420SPToRGB565(buffer, width,
                             buffer + uvOffset, width,
@@ -261,6 +264,13 @@ GrallocImage::DeprecatedGetAsSurface()
                             width, height);
 
     return imageSurface.forget();
+  } else if (format == HAL_PIXEL_FORMAT_YV12) {
+    gfx::ConvertYCbCrToRGB(mData,
+                           gfx::ImageFormatToSurfaceFormat(imageSurface->Format()),
+                           mSize,
+                           imageSurface->Data(),
+                           imageSurface->Stride());
+    return imageSurface.forget();
   }
 
   android::ColorConverter colorConverter((OMX_COLOR_FORMATTYPE)omxFormat,
@@ -344,7 +354,8 @@ GrallocImage::GetAsSourceSurface()
     surface->Unmap();
     return surface;
   }
-  else if (format == HAL_PIXEL_FORMAT_YCrCb_420_SP) {
+
+  if (format == HAL_PIXEL_FORMAT_YCrCb_420_SP) {
     uint32_t uvOffset = height * width;
     ConvertYVU420SPToRGB565(buffer, width,
                             buffer + uvOffset, width,
@@ -355,11 +366,22 @@ GrallocImage::GetAsSourceSurface()
     return surface;
   }
 
+  if (format == HAL_PIXEL_FORMAT_YV12) {
+    gfx::ConvertYCbCrToRGB(mData,
+                           surface->GetFormat(),
+                           mSize,
+                           surface->GetData(),
+                           surface->Stride());
+    surface->Unmap();
+    return surface;
+  }
+
   android::ColorConverter colorConverter((OMX_COLOR_FORMATTYPE)omxFormat,
                                          OMX_COLOR_Format16bitRGB565);
 
   if (!colorConverter.isValid()) {
     NS_WARNING("Invalid color conversion");
+    surface->Unmap();
     return nullptr;
   }
 
@@ -380,7 +402,7 @@ GrallocImage::GetAsSourceSurface()
 
 
 TextureClient*
-GrallocImage::GetTextureClient()
+GrallocImage::GetTextureClient(CompositableClient* aClient)
 {
   if (!mTextureClient) {
     const SurfaceDescriptor& sd = GetSurfaceDescriptor();
