@@ -11,8 +11,10 @@
 #include "gfxWindowsPlatform.h"
 #include "SurfaceStream.h"
 #include "SharedSurfaceANGLE.h"
+#include "SharedSurfaceGL.h"
 #include "gfxContext.h"
 #include "GLContext.h"
+#include "gfxPrefs.h"
 
 using namespace mozilla::gl;
 using namespace mozilla::gfx;
@@ -28,7 +30,6 @@ CanvasLayerD3D10::CanvasLayerD3D10(LayerManagerD3D10 *aManager)
   , mHasAlpha(true)
 {
     mImplData = static_cast<LayerD3D10*>(this);
-    mForceReadback = Preferences::GetBool("webgl.force-layers-readback", false);
 }
 
 CanvasLayerD3D10::~CanvasLayerD3D10()
@@ -40,13 +41,7 @@ CanvasLayerD3D10::Initialize(const Data& aData)
 {
   NS_ASSERTION(mSurface == nullptr, "BasicCanvasLayer::Initialize called twice!");
 
-  if (aData.mSurface) {
-    mSurface = aData.mSurface;
-    NS_ASSERTION(!aData.mGLContext && !aData.mDrawTarget,
-                 "CanvasLayer can't have both surface and WebGLContext/DrawTarget");
-    mNeedsYFlip = false;
-    mDataIsPremultiplied = true;
-  } else if (aData.mGLContext) {
+  if (aData.mGLContext) {
     mGLContext = aData.mGLContext;
     NS_ASSERTION(mGLContext->IsOffscreen(), "Canvas GLContext must be offscreen.");
     mDataIsPremultiplied = aData.mIsGLAlphaPremult;
@@ -58,7 +53,7 @@ CanvasLayerD3D10::Initialize(const Data& aData)
                                           screen->PreserveBuffer());
 
     SurfaceFactory_GL* factory = nullptr;
-    if (!mForceReadback) {
+    if (!gfxPrefs::WebGLForceLayersReadback()) {
       if (mGLContext->IsANGLE()) {
         factory = SurfaceFactory_ANGLEShareHandle::Create(mGLContext,
                                                           device(),
@@ -78,35 +73,22 @@ CanvasLayerD3D10::Initialize(const Data& aData)
     if (texture) {
       mTexture = static_cast<ID3D10Texture2D*>(texture);
 
-      NS_ASSERTION(!aData.mGLContext && !aData.mSurface,
-                   "CanvasLayer can't have both surface and WebGLContext/Surface");
+      NS_ASSERTION(!aData.mGLContext,
+                   "CanvasLayer can't have both DrawTarget and WebGLContext/Surface");
 
       mBounds.SetRect(0, 0, aData.mSize.width, aData.mSize.height);
       device()->CreateShaderResourceView(mTexture, nullptr, getter_AddRefs(mSRView));
       return;
-    } 
-    
+    }
+
     // XXX we should store mDrawTarget and use it directly in UpdateSurface,
     // bypassing Thebes
-    mSurface = gfxPlatform::GetPlatform()->GetThebesSurfaceForDrawTarget(mDrawTarget);
+    mSurface = mDrawTarget->Snapshot();
   } else {
     NS_ERROR("CanvasLayer created without mSurface, mDrawTarget or mGLContext?");
   }
 
   mBounds.SetRect(0, 0, aData.mSize.width, aData.mSize.height);
-
-  if (mSurface && mSurface->GetType() == gfxSurfaceType::D2D) {
-    void *data = mSurface->GetData(&gKeyD3D10Texture);
-    if (data) {
-      mTexture = static_cast<ID3D10Texture2D*>(data);
-      mIsD2DTexture = true;
-      device()->CreateShaderResourceView(mTexture, nullptr, getter_AddRefs(mSRView));
-      mHasAlpha =
-        mSurface->GetContentType() == gfxContentType::COLOR_ALPHA;
-      return;
-    }
-  }
-
   mIsD2DTexture = false;
 
   // Create a texture in case we need to readback.
@@ -129,23 +111,23 @@ CanvasLayerD3D10::UpdateSurface()
   if (!IsDirty())
     return;
   Painted();
-
   if (mDrawTarget) {
     mDrawTarget->Flush();
   } else if (mIsD2DTexture) {
-    mSurface->Flush();
+    return;
+  }
+  if (!mTexture) {
     return;
   }
 
   if (mGLContext) {
-    SharedSurface* surf = mGLContext->RequestFrame();
-    if (!surf)
-        return;
-
+    SharedSurface_GL* surf = mGLContext->RequestFrame();
+    if (!surf) {
+      return;
+    }
     switch (surf->Type()) {
       case SharedSurfaceType::EGLSurfaceANGLE: {
         SharedSurface_ANGLEShareHandle* shareSurf = SharedSurface_ANGLEShareHandle::Cast(surf);
-
         mSRView = shareSurf->GetSRV();
         return;
       }
@@ -186,12 +168,6 @@ CanvasLayerD3D10::UpdateSurface()
         MOZ_CRASH("Unhandled SharedSurfaceType.");
     }
   } else if (mSurface) {
-    RECT r;
-    r.left = 0;
-    r.top = 0;
-    r.right = mBounds.width;
-    r.bottom = mBounds.height;
-
     D3D10_MAPPED_TEXTURE2D map;
     HRESULT hr = mTexture->Map(0, D3D10_MAP_WRITE_DISCARD, 0, &map);
 
@@ -200,17 +176,13 @@ CanvasLayerD3D10::UpdateSurface()
       return;
     }
 
-    nsRefPtr<gfxImageSurface> dstSurface;
+    RefPtr<DrawTarget> destTarget =
+      Factory::CreateDrawTargetForD3D10Texture(mTexture,
+                                               SurfaceFormat::R8G8B8A8);
+    Rect r(Point(0, 0), ToRect(mBounds).Size());
+    destTarget->DrawSurface(mSurface, r, r, DrawSurfaceOptions(),
+                            DrawOptions(1.0F, CompositionOp::OP_SOURCE));
 
-    dstSurface = new gfxImageSurface((unsigned char*)map.pData,
-                                     gfxIntSize(mBounds.width, mBounds.height),
-                                     map.RowPitch,
-                                     gfxImageFormat::ARGB32);
-    nsRefPtr<gfxContext> ctx = new gfxContext(dstSurface);
-    ctx->SetOperator(gfxContext::OPERATOR_SOURCE);
-    ctx->SetSource(mSurface);
-    ctx->Paint();
-    
     mTexture->Unmap(0);
     mSRView = mUploadSRView;
   }
