@@ -20,12 +20,16 @@
 #include "nsTArray.h"
 #include "nsThreadUtils.h"
 #include "mozilla/dom/GamepadService.h"
+#include "mozilla/Mutex.h"
 #include "mozilla/Services.h"
 
 namespace {
 
 using mozilla::dom::GamepadService;
+using mozilla::Mutex;
+using mozilla::MutexAutoLock;
 
+const unsigned kMaxAxes = 32;
 const LONG kMaxAxisValue = 65535;
 const DWORD BUTTON_DOWN_MASK = 0x80;
 // Multiple devices-changed notifications can be sent when a device
@@ -34,6 +38,9 @@ const DWORD BUTTON_DOWN_MASK = 0x80;
 // device changes.
 const uint32_t kDevicesChangedStableDelay = 200;
 
+class WindowsGamepadService;
+WindowsGamepadService* gService = nullptr;
+
 typedef struct {
   float x,y;
 } HatState;
@@ -42,7 +49,7 @@ struct Gamepad {
   // From DirectInput, unique to this device+computer combination.
   GUID guidInstance;
   // The ID assigned by the base GamepadService
-  int id;
+  int globalID;
   // A somewhat unique string consisting of the USB vendor/product IDs,
   // and the controller name.
   char idstring[128];
@@ -50,6 +57,7 @@ struct Gamepad {
   int vendorID;
   int productID;
   // Information about the physical device.
+  DWORD axes[kMaxAxes];
   int numAxes;
   int numHats;
   int numButtons;
@@ -64,6 +72,9 @@ struct Gamepad {
   HatState hatState[4];
   // Used during rescan to find devices that were disconnected.
   bool present;
+  // Passed back from the main thread to indicate a device can
+  // now be removed.
+  bool remove;
 };
 
 // Given DWORD |hatPos| representing the position of the POV hat per:
@@ -122,104 +133,6 @@ HatPosToAxes(DWORD hatPos, HatState& axes) {
   }
 }
 
-// Used to post events from the background thread to the foreground thread.
-class GamepadEvent : public nsRunnable {
-public:
-  typedef enum {
-    Axis,
-    Button,
-    HatX,
-    HatY,
-    HatXY,
-    Unknown
-  } Type;
-
-  GamepadEvent(const Gamepad& gamepad,
-               Type type,
-               int which,
-               DWORD data) : mGamepad(gamepad),
-                             mType(type),
-                             mWhich(which),
-                             mData(data) {
-  }
-
-  NS_IMETHOD Run() {
-    nsRefPtr<GamepadService> gamepadsvc(GamepadService::GetService());
-
-    switch (mType) {
-    case Button:
-      gamepadsvc->NewButtonEvent(mGamepad.id, mWhich, mData & BUTTON_DOWN_MASK);
-      break;
-    case Axis: {
-      float adjustedData = ((float)mData * 2.0f) / (float)kMaxAxisValue - 1.0f;
-      gamepadsvc->NewAxisMoveEvent(mGamepad.id, mWhich, adjustedData);
-    }
-    case HatX:
-    case HatY:
-    case HatXY: {
-      // Synthesize 2 axes per POV hat for convenience.
-      HatState hatState;
-      HatPosToAxes(mData, hatState);
-      int xAxis = mGamepad.numAxes + 2 * mWhich;
-      int yAxis = mGamepad.numAxes + 2 * mWhich + 1;
-      //TODO: ostensibly we could not fire an event if one axis hasn't
-      // changed, but it's a pain to track that.
-      if (mType == HatX || mType == HatXY) {
-        gamepadsvc->NewAxisMoveEvent(mGamepad.id, xAxis, hatState.x);
-      }
-      if (mType == HatY || mType == HatXY) {
-        gamepadsvc->NewAxisMoveEvent(mGamepad.id, yAxis, hatState.y);
-      }
-      break;
-    }
-    case Unknown:
-      break;
-    }
-    return NS_OK;
-  }
-
-  const Gamepad& mGamepad;
-  // Type of event
-  Type mType;
-  // Which button/axis is involved
-  int mWhich;
-  // Data specific to event
-  DWORD mData;
-};
-
-class GamepadChangeEvent : public nsRunnable {
-public:
-  enum Type {
-    Added,
-    Removed
-  };
-  GamepadChangeEvent(Gamepad& gamepad,
-                     Type type) : mGamepad(gamepad),
-                                  mID(gamepad.id),
-                                  mType(type) {
-  }
-
-  NS_IMETHOD Run() {
-    nsRefPtr<GamepadService> gamepadsvc(GamepadService::GetService());
-    if (mType == Added) {
-      mGamepad.id = gamepadsvc->AddGamepad(mGamepad.idstring,
-                                           mGamepad.numButtons,
-                                           mGamepad.numAxes +
-                                           mGamepad.numHats*2);
-    } else {
-      gamepadsvc->RemoveGamepad(mID);
-    }
-    return NS_OK;
-  }
-
-private:
-  Gamepad& mGamepad;
-  uint32_t mID;
-  Type mType;
-};
-
-class WindowsGamepadService;
-
 class Observer : public nsIObserver {
 public:
   NS_DECL_ISUPPORTS
@@ -231,7 +144,6 @@ public:
     mTimer = do_CreateInstance("@mozilla.org/timer;1", &rv);
     nsCOMPtr<nsIObserverService> observerService =
       mozilla::services::GetObserverService();
-    observerService->AddObserver(this, "devices-changed", false);
     observerService->AddObserver(this,
                                  NS_XPCOM_WILL_SHUTDOWN_OBSERVER_ID,
                                  false);
@@ -244,7 +156,6 @@ public:
     if (mObserving) {
       nsCOMPtr<nsIObserverService> observerService =
         mozilla::services::GetObserverService();
-      observerService->RemoveObserver(this, "devices-changed");
       observerService->RemoveObserver(this, NS_XPCOM_WILL_SHUTDOWN_OBSERVER_ID);
       mObserving = false;
     }
@@ -254,6 +165,15 @@ public:
     Stop();
   }
 
+  void SetDeviceChangeTimer() {
+    // set stable timer, since we will get multiple devices-changed
+    // notifications at once
+    if (mTimer) {
+      mTimer->Cancel();
+      mTimer->Init(this, kDevicesChangedStableDelay, nsITimer::TYPE_ONE_SHOT);
+    }
+  }
+
 private:
   // Gamepad service owns us, we just hold a reference back to it.
   WindowsGamepadService& mSvc;
@@ -276,9 +196,15 @@ public:
     }
   }
 
-  void DevicesChanged();
+  enum DeviceChangeType {
+    DeviceChangeNotification,
+    DeviceChangeStable
+  };
+  void DevicesChanged(DeviceChangeType type);
   void Startup();
   void Shutdown();
+  void SetGamepadID(int localID, int globalID);
+  void RemoveGamepad(int localID);
 
 private:
   void ScanForDevices();
@@ -300,6 +226,8 @@ private:
 
   // List of connected devices.
   nsTArray<Gamepad> mGamepads;
+  // Used to lock mutation of mGamepads.
+  Mutex mMutex;
   // List of event handles used for signaling.
   nsTArray<HANDLE> mEvents;
 
@@ -308,10 +236,133 @@ private:
   nsRefPtr<Observer> mObserver;
 };
 
+// Used to post events from the background thread to the foreground thread.
+class GamepadEvent : public nsRunnable {
+public:
+  typedef enum {
+    Axis,
+    Button,
+    HatX,
+    HatY,
+    HatXY,
+    Unknown
+  } Type;
+
+  GamepadEvent(const Gamepad& gamepad,
+               Type type,
+               int which,
+               DWORD data) : mGlobalID(gamepad.globalID),
+                             mGamepadAxes(gamepad.numAxes),
+                             mType(type),
+                             mWhich(which),
+                             mData(data) {
+  }
+
+  NS_IMETHOD Run() {
+    nsRefPtr<GamepadService> gamepadsvc(GamepadService::GetService());
+    if (!gamepadsvc) {
+      return NS_OK;
+    }
+
+    switch (mType) {
+    case Button:
+      gamepadsvc->NewButtonEvent(mGlobalID, mWhich, mData & BUTTON_DOWN_MASK);
+      break;
+    case Axis: {
+      float adjustedData = ((float)mData * 2.0f) / (float)kMaxAxisValue - 1.0f;
+      gamepadsvc->NewAxisMoveEvent(mGlobalID, mWhich, adjustedData);
+    }
+    case HatX:
+    case HatY:
+    case HatXY: {
+      // Synthesize 2 axes per POV hat for convenience.
+      HatState hatState;
+      HatPosToAxes(mData, hatState);
+      int xAxis = mGamepadAxes + 2 * mWhich;
+      int yAxis = mGamepadAxes + 2 * mWhich + 1;
+      //TODO: ostensibly we could not fire an event if one axis hasn't
+      // changed, but it's a pain to track that.
+      if (mType == HatX || mType == HatXY) {
+        gamepadsvc->NewAxisMoveEvent(mGlobalID, xAxis, hatState.x);
+      }
+      if (mType == HatY || mType == HatXY) {
+        gamepadsvc->NewAxisMoveEvent(mGlobalID, yAxis, hatState.y);
+      }
+      break;
+    }
+    case Unknown:
+      break;
+    }
+    return NS_OK;
+  }
+
+  int mGlobalID;
+  int mGamepadAxes;
+  // Type of event
+  Type mType;
+  // Which button/axis is involved
+  int mWhich;
+  // Data specific to event
+  DWORD mData;
+};
+
+class GamepadChangeEvent : public nsRunnable {
+public:
+  enum Type {
+    Added,
+    Removed
+  };
+  GamepadChangeEvent(Gamepad& gamepad,
+                     int localID,
+                     Type type) : mLocalID(localID),
+                                  mName(gamepad.idstring),
+                                  mGlobalID(gamepad.globalID),
+                                  mGamepadButtons(gamepad.numButtons),
+                                  mGamepadAxes(gamepad.numAxes),
+                                  mGamepadHats(gamepad.numHats),
+                                  mType(type) {
+  }
+
+  NS_IMETHOD Run() {
+    nsRefPtr<GamepadService> gamepadsvc(GamepadService::GetService());
+    if (!gamepadsvc) {
+      return NS_OK;
+    }
+    if (mType == Added) {
+      int globalID = gamepadsvc->AddGamepad(mName.get(),
+                                            mozilla::dom::NoMapping,
+                                            mGamepadButtons,
+                                            mGamepadAxes +
+                                            mGamepadHats*2);
+      if (gService) {
+        gService->SetGamepadID(mLocalID, globalID);
+      }
+    } else {
+      gamepadsvc->RemoveGamepad(mGlobalID);
+      if (gService) {
+        gService->RemoveGamepad(mLocalID);
+      }
+    }
+    return NS_OK;
+  }
+
+private:
+  // ID in WindowsGamepadService::mGamepads
+  int mLocalID;
+  nsCString mName;
+  int mGamepadButtons;
+  int mGamepadAxes;
+  int mGamepadHats;
+  // ID from GamepadService
+  uint32_t mGlobalID;
+  Type mType;
+};
+
 WindowsGamepadService::WindowsGamepadService()
-  : mThreadExitEvent(CreateEvent(nullptr, FALSE, FALSE, nullptr)),
-    mThreadRescanEvent(CreateEvent(nullptr, FALSE, FALSE, nullptr)),
+  : mThreadExitEvent(CreateEventW(nullptr, FALSE, FALSE, nullptr)),
+    mThreadRescanEvent(CreateEventW(nullptr, FALSE, FALSE, nullptr)),
     mThread(nullptr),
+    mMutex("Windows Gamepad Service"),
     dinput(nullptr) {
   mObserver = new Observer(*this);
   // Initialize DirectInput
@@ -343,6 +394,16 @@ WindowsGamepadService::EnumObjectsCallback(LPCDIDEVICEOBJECTINSTANCE lpddoi,
   dp.lMin = 0;
   dp.lMax = kMaxAxisValue;
   gamepad->device->SetProperty(DIPROP_RANGE, &dp.diph);
+  // Find what the dwOfs of this object in the c_dfDIJoystick data format is.
+  for (DWORD i = 0; i < c_dfDIJoystick.dwNumObjs; i++) {
+    if (c_dfDIJoystick.rgodf[i].pguid &&
+        IsEqualGUID(*c_dfDIJoystick.rgodf[i].pguid, lpddoi->guidType) &&
+        gamepad->numAxes < kMaxAxes) {
+      gamepad->axes[gamepad->numAxes] = c_dfDIJoystick.rgodf[i].dwOfs;
+      gamepad->numAxes++;
+      break;
+    }
+  }
   return DIENUM_CONTINUE;
 }
 
@@ -353,16 +414,18 @@ WindowsGamepadService::EnumCallback(LPCDIDEVICEINSTANCE lpddi,
   WindowsGamepadService* self =
     reinterpret_cast<WindowsGamepadService*>(pvRef);
   // See if this device is already present in our list.
-  for (unsigned int i = 0; i < self->mGamepads.Length(); i++) {
-    if (memcmp(&lpddi->guidInstance, &self->mGamepads[i].guidInstance,
-               sizeof(GUID)) == 0) {
-      self->mGamepads[i].present = true;
-      return DIENUM_CONTINUE;
+  {
+    MutexAutoLock lock(self->mMutex);
+    for (unsigned int i = 0; i < self->mGamepads.Length(); i++) {
+      if (memcmp(&lpddi->guidInstance, &self->mGamepads[i].guidInstance,
+                 sizeof(GUID)) == 0) {
+        self->mGamepads[i].present = true;
+        return DIENUM_CONTINUE;
+      }
     }
   }
 
-  Gamepad gamepad;
-  memset(&gamepad, 0, sizeof(Gamepad));
+  Gamepad gamepad = {};
   if (self->dinput->CreateDevice(lpddi->guidInstance,
                                  getter_AddRefs(gamepad.device),
                                  nullptr)
@@ -389,13 +452,14 @@ WindowsGamepadService::EnumCallback(LPCDIDEVICEINSTANCE lpddi,
     DIDEVCAPS caps;
     caps.dwSize = sizeof(DIDEVCAPS);
     if (gamepad.device->GetCapabilities(&caps) == DI_OK) {
-      gamepad.numAxes = caps.dwAxes;
       gamepad.numHats = caps.dwPOVs;
       gamepad.numButtons = caps.dwButtons;
       //XXX: handle polled devices?
       // (caps.dwFlags & DIDC_POLLEDDATAFORMAT || caps.dwFlags & DIDC_POLLEDDEVICE)
     }
     // Set min/max range for all axes on the device.
+    // Axes will be gathered in EnumObjectsCallback.
+    gamepad.numAxes = 0;
     gamepad.device->EnumObjects(EnumObjectsCallback, &gamepad, DIDFT_AXIS);
     // Set up structure for setting buffer size for buffered data
     dp.diph.dwHeaderSize = sizeof(DIPROPHEADER);
@@ -404,16 +468,19 @@ WindowsGamepadService::EnumCallback(LPCDIDEVICEINSTANCE lpddi,
     dp.diph.dwHow = DIPH_DEVICE;
     dp.dwData = 64; // arbitrary
     // Create event so DInput can signal us when there's new data.
-    gamepad.event = CreateEvent(nullptr, FALSE, FALSE, nullptr);
+    gamepad.event = CreateEventW(nullptr, FALSE, FALSE, nullptr);
     // Set data format, event notification, and acquire device
     if (gamepad.device->SetDataFormat(&c_dfDIJoystick) == DI_OK &&
         gamepad.device->SetProperty(DIPROP_BUFFERSIZE, &dp.diph) == DI_OK &&
         gamepad.device->SetEventNotification(gamepad.event) == DI_OK &&
         gamepad.device->Acquire() == DI_OK) {
+      MutexAutoLock lock(self->mMutex);
       self->mGamepads.AppendElement(gamepad);
       // Inform the GamepadService
+      int localID = self->mGamepads.Length() - 1;
       nsRefPtr<GamepadChangeEvent> event =
-        new GamepadChangeEvent(self->mGamepads[self->mGamepads.Length() - 1],
+        new GamepadChangeEvent(self->mGamepads[localID],
+                               localID,
                                GamepadChangeEvent::Added);
       NS_DispatchToMainThread(event, NS_DISPATCH_NORMAL);
     }
@@ -429,8 +496,18 @@ WindowsGamepadService::EnumCallback(LPCDIDEVICEINSTANCE lpddi,
 
 void
 WindowsGamepadService::ScanForDevices() {
-  for (unsigned int i = 0; i < mGamepads.Length(); i++) {
-    mGamepads[i].present = false;
+  {
+    MutexAutoLock lock(mMutex);
+    for (int i = mGamepads.Length() - 1; i >= 0; i--) {
+      if (mGamepads[i].remove) {
+
+        // Main thread has already handled this, safe to remove.
+        CleanupGamepad(mGamepads[i]);
+        mGamepads.RemoveElementAt(i);
+      } else {
+        mGamepads[i].present = false;
+      }
+    }
   }
 
   dinput->EnumDevices(DI8DEVCLASS_GAMECTRL,
@@ -438,22 +515,25 @@ WindowsGamepadService::ScanForDevices() {
                       this,
                       DIEDFL_ATTACHEDONLY);
 
-  // Look for devices that were removed.
-  for (int i = mGamepads.Length() - 1; i >= 0; i--) {
-    if (!mGamepads[i].present) {
-      nsRefPtr<GamepadChangeEvent> event =
-        new GamepadChangeEvent(mGamepads[i],
-                               GamepadChangeEvent::Removed);
-      NS_DispatchToMainThread(event, NS_DISPATCH_NORMAL);
-      CleanupGamepad(mGamepads[i]);
-      mGamepads.RemoveElementAt(i);
+  // Look for devices that are no longer present and inform the main thread.
+  {
+    MutexAutoLock lock(mMutex);
+    for (int i = mGamepads.Length() - 1; i >= 0; i--) {
+      if (!mGamepads[i].present) {
+        nsRefPtr<GamepadChangeEvent> event =
+          new GamepadChangeEvent(mGamepads[i],
+                                 i,
+                                 GamepadChangeEvent::Removed);
+        NS_DispatchToMainThread(event, NS_DISPATCH_NORMAL);
+      }
+    }
+
+    mEvents.Clear();
+    for (unsigned int i = 0; i < mGamepads.Length(); i++) {
+      mEvents.AppendElement(mGamepads[i].event);
     }
   }
 
-  mEvents.Clear();
-  for (unsigned int i = 0; i < mGamepads.Length(); i++) {
-    mEvents.AppendElement(mGamepads[i].event);
-  }
 
   // These events must be the  last elements in the array, so that
   // the other elements match mGamepads in order.
@@ -486,66 +566,74 @@ WindowsGamepadService::DInputThread(LPVOID arg) {
       continue;
     }
 
-    if (i >= self->mGamepads.Length()) {
-      // Something would be terribly wrong here, possibly we got
-      // a WAIT_ABANDONED_x result.
-      continue;
-    }
+    {
+      MutexAutoLock lock(self->mMutex);
+      if (i >= self->mGamepads.Length()) {
+        // Something would be terribly wrong here, possibly we got
+        // a WAIT_ABANDONED_x result.
+        continue;
+      }
 
-    // first query for the number of items in the buffer
-    DWORD items = INFINITE;
-    nsRefPtr<IDirectInputDevice8> device = self->mGamepads[i].device;
-    if (device->GetDeviceData(sizeof(DIDEVICEOBJECTDATA),
-                              nullptr,
-                              &items,
-                              DIGDD_PEEK)== DI_OK) {
-      while (items > 0) {
-        // now read each buffered event
-        //TODO: read more than one event at a time
-        DIDEVICEOBJECTDATA data;
-        DWORD readCount = sizeof(data) / sizeof(DIDEVICEOBJECTDATA);
-        if (device->GetDeviceData(sizeof(DIDEVICEOBJECTDATA),
-                                  &data, &readCount, 0) == DI_OK) {
-          //TODO: data.dwTimeStamp
-          GamepadEvent::Type type = GamepadEvent::Unknown;
-          int which;
-          if (data.dwOfs >= DIJOFS_BUTTON0 && data.dwOfs < DIJOFS_BUTTON(32)) {
-            type = GamepadEvent::Button;
-            which = data.dwOfs - DIJOFS_BUTTON0;
-          }
-          else if(data.dwOfs >= DIJOFS_X  && data.dwOfs < DIJOFS_SLIDER(2)) {
-            // axis/slider
-            type = GamepadEvent::Axis;
-            which = (data.dwOfs - DIJOFS_X) / sizeof(LONG);
-          }
-          else if (data.dwOfs >= DIJOFS_POV(0) && data.dwOfs < DIJOFS_POV(4)) {
-            HatState hatState;
-            HatPosToAxes(data.dwData, hatState);
-            which = (data.dwOfs - DIJOFS_POV(0)) / sizeof(DWORD);
-            // Only send out axis move events for the axes that moved
-            // in this hat move.
-            if (hatState.x != self->mGamepads[i].hatState[which].x) {
-              type = GamepadEvent::HatX;
+      // first query for the number of items in the buffer
+      DWORD items = INFINITE;
+      nsRefPtr<IDirectInputDevice8> device = self->mGamepads[i].device;
+      if (device->GetDeviceData(sizeof(DIDEVICEOBJECTDATA),
+                                nullptr,
+                                &items,
+                                DIGDD_PEEK)== DI_OK) {
+        while (items > 0) {
+          // now read each buffered event
+          //TODO: read more than one event at a time
+          DIDEVICEOBJECTDATA data;
+          DWORD readCount = sizeof(data) / sizeof(DIDEVICEOBJECTDATA);
+          if (device->GetDeviceData(sizeof(DIDEVICEOBJECTDATA),
+                                    &data, &readCount, 0) == DI_OK) {
+            //TODO: data.dwTimeStamp
+            GamepadEvent::Type type = GamepadEvent::Unknown;
+            int which = -1;
+            if (data.dwOfs >= DIJOFS_BUTTON0 && data.dwOfs < DIJOFS_BUTTON(32)) {
+              type = GamepadEvent::Button;
+              which = data.dwOfs - DIJOFS_BUTTON0;
             }
-            if (hatState.y != self->mGamepads[i].hatState[which].y) {
-              if (type == GamepadEvent::HatX) {
-                type = GamepadEvent::HatXY;
-              }
-              else {
-                type = GamepadEvent::HatY;
+            else if(data.dwOfs >= DIJOFS_X  && data.dwOfs < DIJOFS_SLIDER(2)) {
+              // axis/slider
+              type = GamepadEvent::Axis;
+              for (int a = 0; a < self->mGamepads[i].numAxes; a++) {
+                if (self->mGamepads[i].axes[a] == data.dwOfs) {
+                  which = a;
+                  break;
+                }
               }
             }
-            self->mGamepads[i].hatState[which].x = hatState.x;
-            self->mGamepads[i].hatState[which].y = hatState.y;
-          }
+            else if (data.dwOfs >= DIJOFS_POV(0) && data.dwOfs < DIJOFS_POV(4)) {
+              HatState hatState;
+              HatPosToAxes(data.dwData, hatState);
+              which = (data.dwOfs - DIJOFS_POV(0)) / sizeof(DWORD);
+              // Only send out axis move events for the axes that moved
+              // in this hat move.
+              if (hatState.x != self->mGamepads[i].hatState[which].x) {
+                type = GamepadEvent::HatX;
+              }
+              if (hatState.y != self->mGamepads[i].hatState[which].y) {
+                if (type == GamepadEvent::HatX) {
+                  type = GamepadEvent::HatXY;
+                }
+                else {
+                  type = GamepadEvent::HatY;
+                }
+              }
+              self->mGamepads[i].hatState[which].x = hatState.x;
+              self->mGamepads[i].hatState[which].y = hatState.y;
+            }
 
-          if (type != GamepadEvent::Unknown) {
-            nsRefPtr<GamepadEvent> event =
-              new GamepadEvent(self->mGamepads[i], type, which, data.dwData);
-            NS_DispatchToMainThread(event, NS_DISPATCH_NORMAL);
+            if (type != GamepadEvent::Unknown && which != -1) {
+              nsRefPtr<GamepadEvent> event =
+                new GamepadEvent(self->mGamepads[i], type, which, data.dwData);
+              NS_DispatchToMainThread(event, NS_DISPATCH_NORMAL);
+            }
           }
+          items--;
         }
-        items--;
       }
     }
   }
@@ -572,6 +660,21 @@ WindowsGamepadService::Shutdown() {
   Cleanup();
 }
 
+// This method is called from the main thread.
+void
+WindowsGamepadService::SetGamepadID(int localID, int globalID) {
+  MutexAutoLock lock(mMutex);
+  mGamepads[localID].globalID = globalID;
+}
+
+// This method is called from the main thread.
+void WindowsGamepadService::RemoveGamepad(int localID) {
+  MutexAutoLock lock(mMutex);
+  mGamepads[localID].remove = true;
+  // Signal background thread to remove device.
+  DevicesChanged(DeviceChangeStable);
+}
+
 void
 WindowsGamepadService::Cleanup() {
   for (unsigned int i = 0; i < mGamepads.Length(); i++) {
@@ -588,36 +691,51 @@ WindowsGamepadService::CleanupGamepad(Gamepad& gamepad) {
 }
 
 void
-WindowsGamepadService::DevicesChanged() {
-  SetEvent(mThreadRescanEvent);
+WindowsGamepadService::DevicesChanged(DeviceChangeType type) {
+  if (type == DeviceChangeNotification) {
+    mObserver->SetDeviceChangeTimer();
+  } else if (type == DeviceChangeStable) {
+    SetEvent(mThreadRescanEvent);
+  }
 }
 
 NS_IMETHODIMP
 Observer::Observe(nsISupports* aSubject,
                   const char* aTopic,
-                  const PRUnichar* aData) {
+                  const char16_t* aData) {
   if (strcmp(aTopic, "timer-callback") == 0) {
-    mSvc.DevicesChanged();
+    mSvc.DevicesChanged(WindowsGamepadService::DeviceChangeStable);
   } else if (strcmp(aTopic, NS_XPCOM_WILL_SHUTDOWN_OBSERVER_ID) == 0) {
     Stop();
-  } else if (strcmp(aTopic, "devices-changed")) {
-    // set stable timer, since we will get multiple devices-changed
-    // notifications at once
-    if (mTimer) {
-      mTimer->Cancel();
-      mTimer->Init(this, kDevicesChangedStableDelay, nsITimer::TYPE_ONE_SHOT);
-    }
   }
   return NS_OK;
 }
 
+HWND sHWnd = nullptr;
+
+static
+LRESULT CALLBACK
+GamepadWindowProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam) {
+  const unsigned int DBT_DEVICEARRIVAL        = 0x8000;
+  const unsigned int DBT_DEVICEREMOVECOMPLETE = 0x8004;
+  const unsigned int DBT_DEVNODES_CHANGED     = 0x7;
+
+  if (msg == WM_DEVICECHANGE &&
+      (wParam == DBT_DEVICEARRIVAL ||
+       wParam == DBT_DEVICEREMOVECOMPLETE ||
+       wParam == DBT_DEVNODES_CHANGED)) {
+    if (gService) {
+      gService->DevicesChanged(WindowsGamepadService::DeviceChangeNotification);
+    }
+  }
+  return DefWindowProc(hwnd, msg, wParam, lParam);
+}
+
 } // namespace
 
 namespace mozilla {
 namespace hal_impl {
 
-WindowsGamepadService* gService = nullptr;
-
 void StartMonitoringGamepadStatus()
 {
   if (gService)
@@ -625,6 +743,23 @@ void StartMonitoringGamepadStatus()
 
   gService = new WindowsGamepadService();
   gService->Startup();
+
+  if (sHWnd == nullptr) {
+    WNDCLASSW wc;
+    HMODULE hSelf = GetModuleHandle(nullptr);
+
+    if (!GetClassInfoW(hSelf, L"MozillaGamepadClass", &wc)) {
+      ZeroMemory(&wc, sizeof(WNDCLASSW));
+      wc.hInstance = hSelf;
+      wc.lpfnWndProc = GamepadWindowProc;
+      wc.lpszClassName = L"MozillaGamepadClass";
+      RegisterClassW(&wc);
+    }
+
+    sHWnd = CreateWindowW(L"MozillaGamepadClass", L"Gamepad Watcher",
+                          0, 0, 0, 0, 0,
+                          nullptr, nullptr, hSelf, nullptr);
+  }
 }
 
 void StopMonitoringGamepadStatus()
@@ -632,6 +767,11 @@ void StopMonitoringGamepadStatus()
   if (!gService)
     return;
 
+  if (sHWnd) {
+    DestroyWindow(sHWnd);
+    sHWnd = nullptr;
+  }
+
   gService->Shutdown();
   delete gService;
   gService = nullptr;
