@@ -6,7 +6,6 @@
 #include "nsDateTimeFormatWin.h"
 #include "nsIServiceManager.h"
 #include "nsIComponentManager.h"
-#include "nsLocaleCID.h"
 #include "nsILocaleService.h"
 #include "nsWin32Locale.h"
 #include "nsUnicharUtils.h"
@@ -94,7 +93,7 @@ nsresult nsDateTimeFormatWin::FormatTMTime(nsILocale* locale,
   SYSTEMTIME system_time;
   DWORD dwFlags_Date = 0, dwFlags_Time = 0;
   int dateLen, timeLen;
-  PRUnichar dateBuffer[NSDATETIMEFORMAT_BUFFER_LEN], timeBuffer[NSDATETIMEFORMAT_BUFFER_LEN];
+  char16_t dateBuffer[NSDATETIMEFORMAT_BUFFER_LEN], timeBuffer[NSDATETIMEFORMAT_BUFFER_LEN];
 
   // set up locale data
   (void) Initialize(locale);
@@ -155,7 +154,7 @@ nsresult nsDateTimeFormatWin::FormatTMTime(nsILocale* locale,
                                  dateBuffer, NSDATETIMEFORMAT_BUFFER_LEN);
     }
     else {
-      dateLen = nsGetDateFormatW(dwFlags_Date, &system_time, NULL, 
+      dateLen = nsGetDateFormatW(dwFlags_Date, &system_time, nullptr,
                                  dateBuffer, NSDATETIMEFORMAT_BUFFER_LEN);
     }
     if (dateLen != 0) {
@@ -168,7 +167,7 @@ nsresult nsDateTimeFormatWin::FormatTMTime(nsILocale* locale,
     timeLen = 0;
   }
   else {
-    timeLen = nsGetTimeFormatW(dwFlags_Time, &system_time, NULL, 
+    timeLen = nsGetTimeFormatW(dwFlags_Time, &system_time, nullptr, 
                                timeBuffer, NSDATETIMEFORMAT_BUFFER_LEN);
     if (timeLen != 0) {
       timeLen--;  // Since the count includes the terminating null.
@@ -182,7 +181,7 @@ nsresult nsDateTimeFormatWin::FormatTMTime(nsILocale* locale,
   stringOut.Truncate();
   if (dateLen != 0 && timeLen != 0) {
     stringOut.Assign(dateBuffer, dateLen);
-    stringOut.Append((PRUnichar *)(L" "), 1);
+    stringOut.Append((char16_t *)(L" "), 1);
     stringOut.Append(timeBuffer, timeLen);
   }
   else if (dateLen != 0 && timeLen == 0) {
@@ -232,27 +231,23 @@ nsresult nsDateTimeFormatWin::FormatPRExplodedTime(nsILocale* locale,
 }
 
 int nsDateTimeFormatWin::nsGetTimeFormatW(DWORD dwFlags, const SYSTEMTIME *lpTime,
-                                          const char* format, PRUnichar *timeStr, int cchTime)
+                                          const char* format, char16_t *timeStr, int cchTime)
 {
   int len = 0;
   len = GetTimeFormatW(mLCID, dwFlags, lpTime, 
                        format ?
-                       const_cast<LPCWSTR>
-                                 (NS_ConvertASCIItoUTF16(format).get()) :
-                       NULL,
+                       NS_ConvertASCIItoUTF16(format).get() :
+                       nullptr,
                        (LPWSTR) timeStr, cchTime);
   return len;
 }
 
 int nsDateTimeFormatWin::nsGetDateFormatW(DWORD dwFlags, const SYSTEMTIME *lpDate,
-                                          const char* format, PRUnichar *dateStr, int cchDate)
+                                          const char* format, char16_t *dateStr, int cchDate)
 {
   int len = 0;
   len = GetDateFormatW(mLCID, dwFlags, lpDate, 
-                       format ?
-                       const_cast<LPCWSTR>
-                                 (NS_ConvertASCIItoUTF16(format).get()) :
-                       NULL,
+                       format ? NS_ConvertASCIItoUTF16(format).get() : nullptr,
                        (LPWSTR) dateStr, cchDate);
   return len;
 }
