@@ -48,8 +48,9 @@ SharedMemory::SharedMemory()
   : mAllocSize(0)
   , mMappedSize(0)
 {
-  static Atomic<uint32_t> registered;
-  if (registered.compareExchange(0, 1)) {
+  MOZ_COUNT_CTOR(SharedMemory);
+  static Atomic<bool> registered;
+  if (registered.compareExchange(false, true)) {
     RegisterStrongMemoryReporter(new ShmemReporter());
   }
 }
