@@ -5,11 +5,12 @@
 #include "TestShellParent.h"
 
 /* This must occur *after* TestShellParent.h to avoid typedefs conflicts. */
-#include "mozilla/Util.h"
+#include "mozilla/ArrayUtils.h"
 
 #include "mozilla/dom/ContentParent.h"
 
 #include "nsAutoPtr.h"
+#include "nsCxPusher.h"
 
 using namespace mozilla;
 using mozilla::ipc::TestShellParent;
@@ -35,47 +36,48 @@ TestShellParent::CommandDone(TestShellCommandParent* command,
                              const nsString& aResponse)
 {
   // XXX what should happen if the callback fails?
-  /*JSBool ok = */command->RunCallback(aResponse);
+  /*bool ok = */command->RunCallback(aResponse);
   command->ReleaseCallback();
 
   return true;
 }
 
-JSBool
+bool
 TestShellCommandParent::SetCallback(JSContext* aCx,
                                     JS::Value aCallback)
 {
   if (!mCallback.Hold(aCx)) {
-    return JS_FALSE;
+    return false;
   }
 
   mCallback = aCallback;
   mCx = aCx;
 
-  return JS_TRUE;
+  return true;
 }
 
-JSBool
+bool
 TestShellCommandParent::RunCallback(const nsString& aResponse)
 {
-  NS_ENSURE_TRUE(*mCallback.ToJSValPtr() != JSVAL_NULL && mCx, JS_FALSE);
+  NS_ENSURE_TRUE(*mCallback.ToJSValPtr() != JSVAL_NULL && mCx, false);
 
-  JSAutoRequest ar(mCx);
-  NS_ENSURE_TRUE(mCallback.ToJSObject(), JS_FALSE);
+  // We're pulling a cx off the heap, so make sure it's stack-top.
+  AutoCxPusher pusher(mCx);
+  NS_ENSURE_TRUE(mCallback.ToJSObject(), false);
   JSAutoCompartment ac(mCx, mCallback.ToJSObject());
   JS::Rooted<JSObject*> global(mCx, JS::CurrentGlobalOrNull(mCx));
 
   JSString* str = JS_NewUCStringCopyN(mCx, aResponse.get(), aResponse.Length());
-  NS_ENSURE_TRUE(str, JS_FALSE);
+  NS_ENSURE_TRUE(str, false);
 
   JS::Rooted<JS::Value> strVal(mCx, JS::StringValue(str));
 
   JS::Rooted<JS::Value> rval(mCx);
-  JSBool ok = JS_CallFunctionValue(mCx, global, mCallback, 1, strVal.address(),
-                                   rval.address());
-  NS_ENSURE_TRUE(ok, JS_FALSE);
+  JS::Rooted<JS::Value> callback(mCx, mCallback);
+  bool ok = JS_CallFunctionValue(mCx, global, callback, strVal, &rval);
+  NS_ENSURE_TRUE(ok, false);
 
-  return JS_TRUE;
+  return true;
 }
 
 void
