@@ -6,6 +6,7 @@
 
 #include "frontend/BytecodeCompiler.h"
 
+#include "jscntxt.h"
 #include "jsscript.h"
 
 #include "frontend/BytecodeEmitter.h"
@@ -18,7 +19,7 @@
 #include "jsobjinlines.h"
 #include "jsscriptinlines.h"
 
-#include "frontend/ParseMaps-inl.h"
+#include "frontend/Parser-inl.h"
 
 using namespace js;
 using namespace js::frontend;
@@ -32,17 +33,28 @@ CheckLength(ExclusiveContext *cx, size_t length)
     // is using size_t internally already.
     if (length > UINT32_MAX) {
         if (cx->isJSContext())
-            JS_ReportErrorNumber(cx->asJSContext(), js_GetErrorMessage, NULL, JSMSG_SOURCE_TOO_LONG);
+            JS_ReportErrorNumber(cx->asJSContext(), js_GetErrorMessage, nullptr,
+                                 JSMSG_SOURCE_TOO_LONG);
         return false;
     }
     return true;
 }
 
+static bool
+SetDisplayURL(ExclusiveContext *cx, TokenStream &tokenStream, ScriptSource *ss)
+{
+    if (tokenStream.hasDisplayURL()) {
+        if (!ss->setDisplayURL(cx, tokenStream.displayURL()))
+            return false;
+    }
+    return true;
+}
+
 static bool
 SetSourceMap(ExclusiveContext *cx, TokenStream &tokenStream, ScriptSource *ss)
 {
-    if (tokenStream.hasSourceMap()) {
-        if (!ss->setSourceMap(cx, tokenStream.releaseSourceMap()))
+    if (tokenStream.hasSourceMapURL()) {
+        if (!ss->setSourceMapURL(cx, tokenStream.sourceMapURL()))
             return false;
     }
     return true;
@@ -54,21 +66,23 @@ CheckArgumentsWithinEval(JSContext *cx, Parser<FullParseHandler> &parser, Handle
     if (fun->hasRest()) {
         // It's an error to use |arguments| in a function that has a rest
         // parameter.
-        parser.report(ParseError, false, NULL, JSMSG_ARGUMENTS_AND_REST);
+        parser.report(ParseError, false, nullptr, JSMSG_ARGUMENTS_AND_REST);
         return false;
     }
 
     // Force construction of arguments objects for functions that use
     // |arguments| within an eval.
-    RootedScript script(cx, fun->nonLazyScript());
+    RootedScript script(cx, fun->getOrCreateScript(cx));
+    if (!script)
+        return false;
     if (script->argumentsHasVarBinding()) {
         if (!JSScript::argumentsOptimizationFailed(cx, script))
             return false;
     }
 
     // It's an error to use |arguments| in a legacy generator expression.
-    if (script->isGeneratorExp && script->isLegacyGenerator) {
-        parser.report(ParseError, false, NULL, JSMSG_BAD_GENEXP_BODY, js_arguments_str);
+    if (script->isGeneratorExp() && script->isLegacyGenerator()) {
+        parser.report(ParseError, false, nullptr, JSMSG_BAD_GENEXP_BODY, js_arguments_str);
         return false;
     }
 
@@ -111,7 +125,9 @@ MaybeCheckEvalFreeVariables(ExclusiveContext *cxArg, HandleScript evalCaller, Ha
         RootedObject scope(cx, scopeChain);
         while (scope->is<ScopeObject>() || scope->is<DebugScopeObject>()) {
             if (scope->is<CallObject>() && !scope->as<CallObject>().isForEval()) {
-                RootedScript script(cx, scope->as<CallObject>().callee().nonLazyScript());
+                RootedScript script(cx, scope->as<CallObject>().callee().getOrCreateScript(cx));
+                if (!script)
+                    return false;
                 if (script->argumentsHasVarBinding()) {
                     if (!JSScript::argumentsOptimizationFailed(cx, script))
                         return false;
@@ -124,18 +140,18 @@ MaybeCheckEvalFreeVariables(ExclusiveContext *cxArg, HandleScript evalCaller, Ha
     return true;
 }
 
-inline bool
-CanLazilyParse(ExclusiveContext *cx, const CompileOptions &options)
+static inline bool
+CanLazilyParse(ExclusiveContext *cx, const ReadOnlyCompileOptions &options)
 {
     return options.canLazilyParse &&
         options.compileAndGo &&
         options.sourcePolicy == CompileOptions::SAVE_SOURCE &&
-        cx->isJSContext() &&
-        !cx->asJSContext()->compartment()->debugMode();
+        !(cx->compartment()->debugMode() &&
+          cx->compartment()->runtimeFromAnyThread()->debugHooks.newScriptHook);
 }
 
 void
-frontend::MaybeCallSourceHandler(JSContext *cx, const CompileOptions &options,
+frontend::MaybeCallSourceHandler(JSContext *cx, const ReadOnlyCompileOptions &options,
                                  const jschar *chars, size_t length)
 {
     JSSourceHandler listener = cx->runtime()->debugHooks.sourceHandler;
@@ -143,23 +159,44 @@ frontend::MaybeCallSourceHandler(JSContext *cx, const CompileOptions &options,
 
     if (listener) {
         void *listenerTSData;
-        listener(options.filename, options.lineno, chars, length,
+        listener(options.filename(), options.lineno, chars, length,
                  &listenerTSData, listenerData);
     }
 }
 
+ScriptSourceObject *
+frontend::CreateScriptSourceObject(ExclusiveContext *cx, const ReadOnlyCompileOptions &options)
+{
+    ScriptSource *ss = cx->new_<ScriptSource>();
+    if (!ss)
+        return nullptr;
+    ScriptSourceHolder ssHolder(ss);
+
+    if (!ss->initFromOptions(cx, options))
+        return nullptr;
+
+    return ScriptSourceObject::create(cx, ss, options);
+}
+
 JSScript *
 frontend::CompileScript(ExclusiveContext *cx, LifoAlloc *alloc, HandleObject scopeChain,
                         HandleScript evalCaller,
-                        const CompileOptions &options,
+                        const ReadOnlyCompileOptions &options,
                         const jschar *chars, size_t length,
-                        JSString *source_ /* = NULL */,
+                        JSString *source_ /* = nullptr */,
                         unsigned staticLevel /* = 0 */,
-                        SourceCompressionToken *extraSct /* = NULL */)
+                        SourceCompressionTask *extraSct /* = nullptr */)
 {
     RootedString source(cx, source_);
     SkipRoot skip(cx, &chars);
 
+#if JS_TRACE_LOGGING
+        js::AutoTraceLog logger(js::TraceLogging::defaultLogger(),
+                                js::TraceLogging::PARSER_COMPILE_SCRIPT_START,
+                                js::TraceLogging::PARSER_COMPILE_SCRIPT_STOP,
+                                options);
+#endif
+
     if (cx->isJSContext())
         MaybeCallSourceHandler(cx->asJSContext(), options, chars, length);
 
@@ -172,32 +209,22 @@ frontend::CompileScript(ExclusiveContext *cx, LifoAlloc *alloc, HandleObject sco
     JS_ASSERT_IF(staticLevel != 0, evalCaller);
 
     if (!CheckLength(cx, length))
-        return NULL;
+        return nullptr;
     JS_ASSERT_IF(staticLevel != 0, options.sourcePolicy != CompileOptions::LAZY_SOURCE);
-    ScriptSource *ss = cx->new_<ScriptSource>(options.originPrincipals());
-    if (!ss)
-        return NULL;
-    if (options.filename && !ss->setFilename(cx, options.filename))
-        return NULL;
 
-    JS::RootedScriptSource sourceObject(cx, ScriptSourceObject::create(cx, ss));
+    RootedScriptSource sourceObject(cx, CreateScriptSourceObject(cx, options));
     if (!sourceObject)
-        return NULL;
+        return nullptr;
 
-    // Saving source is not yet supported when parsing off thread.
-    JS_ASSERT_IF(!cx->isJSContext(), !extraSct && options.sourcePolicy == CompileOptions::NO_SOURCE);
+    ScriptSource *ss = sourceObject->source();
 
-    SourceCompressionToken *sct = extraSct;
-    Maybe<SourceCompressionToken> mysct;
-    if (cx->isJSContext() && !sct) {
-        mysct.construct(cx->asJSContext());
-        sct = mysct.addr();
-    }
+    SourceCompressionTask mysct(cx);
+    SourceCompressionTask *sct = extraSct ? extraSct : &mysct;
 
     switch (options.sourcePolicy) {
       case CompileOptions::SAVE_SOURCE:
-        if (!ss->setSourceCopy(cx->asJSContext(), chars, length, false, sct))
-            return NULL;
+        if (!ss->setSourceCopy(cx, chars, length, false, sct))
+            return nullptr;
         break;
       case CompileOptions::LAZY_SOURCE:
         ss->setSourceRetrievable();
@@ -211,57 +238,51 @@ frontend::CompileScript(ExclusiveContext *cx, LifoAlloc *alloc, HandleObject sco
     Maybe<Parser<SyntaxParseHandler> > syntaxParser;
     if (canLazilyParse) {
         syntaxParser.construct(cx, alloc, options, chars, length, /* foldConstants = */ false,
-                               (Parser<SyntaxParseHandler> *) NULL,
-                               (LazyScript *) NULL);
+                               (Parser<SyntaxParseHandler> *) nullptr,
+                               (LazyScript *) nullptr);
     }
 
     Parser<FullParseHandler> parser(cx, alloc, options, chars, length, /* foldConstants = */ true,
-                                    canLazilyParse ? &syntaxParser.ref() : NULL, NULL);
+                                    canLazilyParse ? &syntaxParser.ref() : nullptr, nullptr);
     parser.sct = sct;
     parser.ss = ss;
 
     Directives directives(options.strictOption);
     GlobalSharedContext globalsc(cx, scopeChain, directives, options.extraWarningsOption);
 
-    bool savedCallerFun =
-        options.compileAndGo &&
-        evalCaller &&
-        (evalCaller->function() || evalCaller->savedCallerFun);
+    bool savedCallerFun = options.compileAndGo &&
+                          evalCaller && evalCaller->functionOrCallerFunction();
     Rooted<JSScript*> script(cx, JSScript::Create(cx, NullPtr(), savedCallerFun,
                                                   options, staticLevel, sourceObject, 0, length));
     if (!script)
-        return NULL;
-
-    // Global/eval script bindings are always empty (all names are added to the
-    // scope dynamically via JSOP_DEFFUN/VAR).
-    InternalHandle<Bindings*> bindings(script, &script->bindings);
-    if (!Bindings::initWithTemporaryStorage(cx, bindings, 0, 0, NULL))
-        return NULL;
+        return nullptr;
 
     // We can specialize a bit for the given scope chain if that scope chain is the global object.
-    JSObject *globalScope = scopeChain && scopeChain == &scopeChain->global() ? (JSObject*) scopeChain : NULL;
+    JSObject *globalScope =
+        scopeChain && scopeChain == &scopeChain->global() ? (JSObject*) scopeChain : nullptr;
     JS_ASSERT_IF(globalScope, globalScope->isNative());
     JS_ASSERT_IF(globalScope, JSCLASS_HAS_GLOBAL_FLAG_AND_SLOTS(globalScope->getClass()));
 
     BytecodeEmitter::EmitterMode emitterMode =
         options.selfHostingMode ? BytecodeEmitter::SelfHosting : BytecodeEmitter::Normal;
-    BytecodeEmitter bce(/* parent = */ NULL, &parser, &globalsc, script, options.forEval, evalCaller,
-                        !!globalScope, options.lineno, emitterMode);
+    BytecodeEmitter bce(/* parent = */ nullptr, &parser, &globalsc, script, options.forEval,
+                        evalCaller, !!globalScope, options.lineno, emitterMode);
     if (!bce.init())
-        return NULL;
+        return nullptr;
 
     // Syntax parsing may cause us to restart processing of top level
     // statements in the script. Use Maybe<> so that the parse context can be
     // reset when this occurs.
     Maybe<ParseContext<FullParseHandler> > pc;
 
-    pc.construct(&parser, (GenericParseContext *) NULL, (ParseNode *) NULL, &globalsc,
-                 (Directives *) NULL, staticLevel, /* bodyid = */ 0);
-    if (!pc.ref().init())
-        return NULL;
+    pc.construct(&parser, (GenericParseContext *) nullptr, (ParseNode *) nullptr, &globalsc,
+                 (Directives *) nullptr, staticLevel, /* bodyid = */ 0,
+                 /* blockScopeDepth = */ 0);
+    if (!pc.ref().init(parser.tokenStream))
+        return nullptr;
 
     /* If this is a direct call to eval, inherit the caller's strictness.  */
-    if (evalCaller && evalCaller->strict)
+    if (evalCaller && evalCaller->strict())
         globalsc.strict = true;
 
     if (options.compileAndGo) {
@@ -270,10 +291,10 @@ frontend::CompileScript(ExclusiveContext *cx, LifoAlloc *alloc, HandleObject sco
              * Save eval program source in script->atoms[0] for the
              * eval cache (see EvalCacheLookup in jsobj.cpp).
              */
-            JSAtom *atom = AtomizeString<CanGC>(cx, source);
+            JSAtom *atom = AtomizeString(cx, source);
             jsatomid _;
             if (!atom || !bce.makeAtomIndex(atom, &_))
-                return NULL;
+                return nullptr;
         }
 
         if (evalCaller && evalCaller->functionOrCallerFunction()) {
@@ -284,9 +305,10 @@ frontend::CompileScript(ExclusiveContext *cx, LifoAlloc *alloc, HandleObject sco
              */
             JSFunction *fun = evalCaller->functionOrCallerFunction();
             Directives directives(/* strict = */ fun->strict());
-            ObjectBox *funbox = parser.newFunctionBox(/* fn = */ NULL, fun, pc.addr(), directives);
+            ObjectBox *funbox = parser.newFunctionBox(/* fn = */ nullptr, fun, pc.addr(),
+                                                      directives, fun->generatorKind());
             if (!funbox)
-                return NULL;
+                return nullptr;
             bce.objectList.add(funbox);
         }
     }
@@ -298,7 +320,7 @@ frontend::CompileScript(ExclusiveContext *cx, LifoAlloc *alloc, HandleObject sco
             if (tt == TOK_EOF)
                 break;
             JS_ASSERT(tt == TOK_ERROR);
-            return NULL;
+            return nullptr;
         }
 
         TokenStream::Position pos(parser.keepAtoms);
@@ -318,86 +340,120 @@ frontend::CompileScript(ExclusiveContext *cx, LifoAlloc *alloc, HandleObject sco
                 // Destroying the parse context will destroy its free
                 // variables, so check if any deoptimization is needed.
                 if (!MaybeCheckEvalFreeVariables(cx, evalCaller, scopeChain, parser, pc.ref()))
-                    return NULL;
+                    return nullptr;
 
                 pc.destroy();
-                pc.construct(&parser, (GenericParseContext *) NULL, (ParseNode *) NULL,
-                             &globalsc, (Directives *) NULL, staticLevel, /* bodyid = */ 0);
-                if (!pc.ref().init())
-                    return NULL;
+                pc.construct(&parser, (GenericParseContext *) nullptr, (ParseNode *) nullptr,
+                             &globalsc, (Directives *) nullptr, staticLevel, /* bodyid = */ 0,
+                             script->bindings.numBlockScoped());
+                if (!pc.ref().init(parser.tokenStream))
+                    return nullptr;
                 JS_ASSERT(parser.pc == pc.addr());
+
                 pn = parser.statement();
             }
             if (!pn) {
                 JS_ASSERT(!parser.hadAbortedSyntaxParse());
-                return NULL;
+                return nullptr;
             }
         }
 
+        // Accumulate the maximum block scope depth, so that EmitTree can assert
+        // when emitting JSOP_GETLOCAL that the local is indeed within the fixed
+        // part of the stack frame.
+        script->bindings.updateNumBlockScoped(pc.ref().blockScopeDepth);
+
         if (canHaveDirectives) {
-            if (!parser.maybeParseDirective(/* stmtList = */ NULL, pn, &canHaveDirectives))
-                return NULL;
+            if (!parser.maybeParseDirective(/* stmtList = */ nullptr, pn, &canHaveDirectives))
+                return nullptr;
         }
 
         if (!FoldConstants(cx, &pn, &parser))
-            return NULL;
+            return nullptr;
 
-        // Inferring names for functions in compiled scripts is currently only
-        // supported while on the main thread. See bug 895395.
-        if (cx->isJSContext() && !NameFunctions(cx->asJSContext(), pn))
-            return NULL;
+        if (!NameFunctions(cx, pn))
+            return nullptr;
 
         if (!EmitTree(cx, &bce, pn))
-            return NULL;
+            return nullptr;
 
         parser.handler.freeTree(pn);
     }
 
     if (!MaybeCheckEvalFreeVariables(cx, evalCaller, scopeChain, parser, pc.ref()))
-        return NULL;
+        return nullptr;
+
+    if (!SetDisplayURL(cx, parser.tokenStream, ss))
+        return nullptr;
 
     if (!SetSourceMap(cx, parser.tokenStream, ss))
-        return NULL;
+        return nullptr;
 
     /*
-     * Nowadays the threaded interpreter needs a stop instruction, so we
+     * Source map URLs passed as a compile option (usually via a HTTP source map
+     * header) override any source map urls passed as comment pragmas.
+     */
+    if (options.sourceMapURL()) {
+        if (!ss->setSourceMapURL(cx, options.sourceMapURL()))
+            return nullptr;
+    }
+
+    /*
+     * Nowadays the threaded interpreter needs a last return instruction, so we
      * do have to emit that here.
      */
-    if (Emit1(cx, &bce, JSOP_STOP) < 0)
-        return NULL;
+    if (Emit1(cx, &bce, JSOP_RETRVAL) < 0)
+        return nullptr;
+
+    // Global/eval script bindings are always empty (all names are added to the
+    // scope dynamically via JSOP_DEFFUN/VAR).  They may have block-scoped
+    // locals, however, which are allocated to the fixed part of the stack
+    // frame.
+    InternalHandle<Bindings*> bindings(script, &script->bindings);
+    if (!Bindings::initWithTemporaryStorage(cx, bindings, 0, 0, nullptr,
+                                            pc.ref().blockScopeDepth))
+        return nullptr;
 
     if (!JSScript::fullyInitFromEmitter(cx, script, &bce))
-        return NULL;
+        return nullptr;
 
     bce.tellDebuggerAboutCompiledScript(cx);
 
     if (sct && !extraSct && !sct->complete())
-        return NULL;
+        return nullptr;
 
     return script;
 }
 
 bool
-frontend::CompileLazyFunction(JSContext *cx, LazyScript *lazy, const jschar *chars, size_t length)
+frontend::CompileLazyFunction(JSContext *cx, Handle<LazyScript*> lazy, const jschar *chars, size_t length)
 {
-    JS_ASSERT(cx->compartment() == lazy->function()->compartment());
+    JS_ASSERT(cx->compartment() == lazy->functionNonDelazifying()->compartment());
 
     CompileOptions options(cx, lazy->version());
-    options.setPrincipals(cx->compartment()->principals)
-           .setOriginPrincipals(lazy->originPrincipals())
+    options.setOriginPrincipals(lazy->originPrincipals())
            .setFileAndLine(lazy->source()->filename(), lazy->lineno())
            .setColumn(lazy->column())
            .setCompileAndGo(true)
            .setNoScriptRval(false)
            .setSelfHostingMode(false);
 
+#if JS_TRACE_LOGGING
+        js::AutoTraceLog logger(js::TraceLogging::defaultLogger(),
+                                js::TraceLogging::PARSER_COMPILE_LAZY_START,
+                                js::TraceLogging::PARSER_COMPILE_LAZY_STOP,
+                                options);
+#endif
+
     Parser<FullParseHandler> parser(cx, &cx->tempLifoAlloc(), options, chars, length,
-                                    /* foldConstants = */ true, NULL, lazy);
+                                    /* foldConstants = */ true, nullptr, lazy);
 
     uint32_t staticLevel = lazy->staticLevel(cx);
 
-    Rooted<JSFunction*> fun(cx, lazy->function());
-    ParseNode *pn = parser.standaloneLazyFunction(fun, staticLevel, lazy->strict());
+    Rooted<JSFunction*> fun(cx, lazy->functionNonDelazifying());
+    JS_ASSERT(!lazy->isLegacyGenerator());
+    ParseNode *pn = parser.standaloneLazyFunction(fun, staticLevel, lazy->strict(),
+                                                  lazy->generatorKind());
     if (!pn)
         return false;
 
@@ -405,7 +461,7 @@ frontend::CompileLazyFunction(JSContext *cx, LazyScript *lazy, const jschar *cha
         return false;
 
     RootedObject enclosingScope(cx, lazy->enclosingScope());
-    JS::RootedScriptSource sourceObject(cx, lazy->sourceObject());
+    RootedScriptSource sourceObject(cx, lazy->sourceObject());
     JS_ASSERT(sourceObject);
 
     Rooted<JSScript*> script(cx, JSScript::Create(cx, enclosingScope, false,
@@ -417,25 +473,38 @@ frontend::CompileLazyFunction(JSContext *cx, LazyScript *lazy, const jschar *cha
     script->bindings = pn->pn_funbox->bindings;
 
     if (lazy->directlyInsideEval())
-        script->directlyInsideEval = true;
+        script->setDirectlyInsideEval();
     if (lazy->usesArgumentsAndApply())
-        script->usesArgumentsAndApply = true;
+        script->setUsesArgumentsAndApply();
+    if (lazy->hasBeenCloned())
+        script->setHasBeenCloned();
 
-    BytecodeEmitter bce(/* parent = */ NULL, &parser, pn->pn_funbox, script, options.forEval,
+    BytecodeEmitter bce(/* parent = */ nullptr, &parser, pn->pn_funbox, script, options.forEval,
                         /* evalCaller = */ NullPtr(), /* hasGlobalScope = */ true,
                         options.lineno, BytecodeEmitter::LazyFunction);
     if (!bce.init())
         return false;
 
+    if (lazy->treatAsRunOnce())
+        bce.lazyRunOnceLambda = true;
+
     return EmitFunctionScript(cx, &bce, pn->pn_body);
 }
 
 // Compile a JS function body, which might appear as the value of an event
 // handler attribute in an HTML <INPUT> tag, or in a Function() constructor.
-bool
-frontend::CompileFunctionBody(JSContext *cx, MutableHandleFunction fun, CompileOptions options,
-                              const AutoNameVector &formals, const jschar *chars, size_t length)
+static bool
+CompileFunctionBody(JSContext *cx, MutableHandleFunction fun, const ReadOnlyCompileOptions &options,
+                    const AutoNameVector &formals, const jschar *chars, size_t length,
+                    GeneratorKind generatorKind)
 {
+#if JS_TRACE_LOGGING
+        js::AutoTraceLog logger(js::TraceLogging::defaultLogger(),
+                                js::TraceLogging::PARSER_COMPILE_FUNCTION_START,
+                                js::TraceLogging::PARSER_COMPILE_FUNCTION_STOP,
+                                options);
+#endif
+
     // FIXME: make Function pass in two strings and parse them as arguments and
     // ProgramElements respectively.
     SkipRoot skip(cx, &chars);
@@ -444,15 +513,13 @@ frontend::CompileFunctionBody(JSContext *cx, MutableHandleFunction fun, CompileO
 
     if (!CheckLength(cx, length))
         return false;
-    ScriptSource *ss = cx->new_<ScriptSource>(options.originPrincipals());
-    if (!ss)
-        return false;
-    if (options.filename && !ss->setFilename(cx, options.filename))
-        return false;
-    JS::RootedScriptSource sourceObject(cx, ScriptSourceObject::create(cx, ss));
+
+    RootedScriptSource sourceObject(cx, CreateScriptSourceObject(cx, options));
     if (!sourceObject)
-        return false;
-    SourceCompressionToken sct(cx);
+        return nullptr;
+    ScriptSource *ss = sourceObject->source();
+
+    SourceCompressionTask sct(cx);
     JS_ASSERT(options.sourcePolicy != CompileOptions::LAZY_SOURCE);
     if (options.sourcePolicy == CompileOptions::SAVE_SOURCE) {
         if (!ss->setSourceCopy(cx, chars, length, true, &sct))
@@ -465,15 +532,15 @@ frontend::CompileFunctionBody(JSContext *cx, MutableHandleFunction fun, CompileO
     if (canLazilyParse) {
         syntaxParser.construct(cx, &cx->tempLifoAlloc(),
                                options, chars, length, /* foldConstants = */ false,
-                               (Parser<SyntaxParseHandler> *) NULL,
-                               (LazyScript *) NULL);
+                               (Parser<SyntaxParseHandler> *) nullptr,
+                               (LazyScript *) nullptr);
     }
 
     JS_ASSERT(!options.forEval);
 
     Parser<FullParseHandler> parser(cx, &cx->tempLifoAlloc(),
                                     options, chars, length, /* foldConstants = */ true,
-                                    canLazilyParse ? &syntaxParser.ref() : NULL, NULL);
+                                    canLazilyParse ? &syntaxParser.ref() : nullptr, nullptr);
     parser.sct = &sct;
     parser.ss = ss;
 
@@ -494,7 +561,7 @@ frontend::CompileFunctionBody(JSContext *cx, MutableHandleFunction fun, CompileO
     ParseNode *fn;
     while (true) {
         Directives newDirectives = directives;
-        fn = parser.standaloneFunctionBody(fun, formals, directives, &newDirectives);
+        fn = parser.standaloneFunctionBody(fun, formals, generatorKind, directives, &newDirectives);
         if (fn)
             break;
 
@@ -522,7 +589,7 @@ frontend::CompileFunctionBody(JSContext *cx, MutableHandleFunction fun, CompileO
     if (fn->pn_funbox->function()->isInterpreted()) {
         JS_ASSERT(fun == fn->pn_funbox->function());
 
-        Rooted<JSScript*> script(cx, JSScript::Create(cx, NullPtr(), false, options,
+        Rooted<JSScript*> script(cx, JSScript::Create(cx, js::NullPtr(), false, options,
                                                       /* staticLevel = */ 0, sourceObject,
                                                       /* sourceStart = */ 0, length));
         if (!script)
@@ -533,12 +600,12 @@ frontend::CompileFunctionBody(JSContext *cx, MutableHandleFunction fun, CompileO
         /*
          * The reason for checking fun->environment() below is that certain
          * consumers of JS::CompileFunction, namely
-         * nsEventListenerManager::CompileEventHandlerInternal, passes in a
-         * NULL environment. This compiled function is never used, but instead
-         * is cloned immediately onto the right scope chain.
+         * EventListenerManager::CompileEventHandlerInternal, passes in a
+         * nullptr environment. This compiled function is never used, but
+         * instead is cloned immediately onto the right scope chain.
          */
-        BytecodeEmitter funbce(/* parent = */ NULL, &parser, fn->pn_funbox, script,
-                               /* insideEval = */ false, /* evalCaller = */ NullPtr(),
+        BytecodeEmitter funbce(/* parent = */ nullptr, &parser, fn->pn_funbox, script,
+                               /* insideEval = */ false, /* evalCaller = */ js::NullPtr(),
                                fun->environment() && fun->environment()->is<GlobalObject>(),
                                options.lineno);
         if (!funbce.init())
@@ -551,6 +618,9 @@ frontend::CompileFunctionBody(JSContext *cx, MutableHandleFunction fun, CompileO
         JS_ASSERT(IsAsmJSModuleNative(fun->native()));
     }
 
+    if (!SetDisplayURL(cx, parser.tokenStream, ss))
+        return false;
+
     if (!SetSourceMap(cx, parser.tokenStream, ss))
         return false;
 
@@ -559,3 +629,19 @@ frontend::CompileFunctionBody(JSContext *cx, MutableHandleFunction fun, CompileO
 
     return true;
 }
+
+bool
+frontend::CompileFunctionBody(JSContext *cx, MutableHandleFunction fun,
+                              const ReadOnlyCompileOptions &options,
+                              const AutoNameVector &formals, const jschar *chars, size_t length)
+{
+    return CompileFunctionBody(cx, fun, options, formals, chars, length, NotGenerator);
+}
+
+bool
+frontend::CompileStarGeneratorBody(JSContext *cx, MutableHandleFunction fun,
+                                   const ReadOnlyCompileOptions &options, const AutoNameVector &formals,
+                                   const jschar *chars, size_t length)
+{
+    return CompileFunctionBody(cx, fun, options, formals, chars, length, StarGenerator);
+}
