@@ -14,8 +14,10 @@
 
 #include "gc/GCInternals.h"
 #include "gc/Zone.h"
+#include "js/GCAPI.h"
 #include "js/HashTable.h"
 
+#include "jscntxtinlines.h"
 #include "jsgcinlines.h"
 
 using namespace js;
@@ -34,7 +36,7 @@ CheckNonAddressThing(uintptr_t *w, Rooted<T> *rootp)
     return w >= (uintptr_t*)rootp->address() && w < (uintptr_t*)(rootp->address() + 1);
 }
 
-static JS_ALWAYS_INLINE bool
+static MOZ_ALWAYS_INLINE bool
 CheckStackRootThing(uintptr_t *w, Rooted<void *> *rootp, ThingRootKind kind)
 {
     if (kind == THING_ROOT_BINDINGS)
@@ -235,13 +237,13 @@ JS::CheckStackRoots(JSContext *cx)
 
     // Can switch to the atoms compartment during analysis.
     if (IsAtomsCompartment(cx->compartment())) {
-        for (CompartmentsIter c(rt); !c.done(); c.next()) {
+        for (CompartmentsIter c(rt, SkipAtoms); !c.done(); c.next()) {
             if (c.get()->activeAnalysis)
                 return;
         }
     }
 
-    AutoCopyFreeListToArenas copy(rt);
+    AutoCopyFreeListToArenas copy(rt, WithAtoms);
 
     ConservativeGCData *cgcd = &rt->conservativeGC;
     cgcd->recordStackTop();
@@ -364,7 +366,10 @@ typedef HashMap<void *, VerifyNode *, DefaultHasher<void *>, SystemAllocPolicy>
  * The nodemap field is a hashtable that maps from the address of the GC thing
  * to the VerifyNode that represents it.
  */
-struct VerifyPreTracer : JSTracer {
+struct VerifyPreTracer : JSTracer
+{
+    JS::AutoDisableGenerationalGC noggc;
+
     /* The gcNumber when the verification began. */
     uint64_t number;
 
@@ -378,8 +383,11 @@ struct VerifyPreTracer : JSTracer {
     char *term;
     NodeMap nodemap;
 
-    VerifyPreTracer() : root(nullptr) {}
-    ~VerifyPreTracer() { js_free(root); }
+    VerifyPreTracer(JSRuntime *rt) : noggc(rt), root(nullptr) {}
+
+    ~VerifyPreTracer() {
+        js_free(root);
+    }
 };
 
 /*
@@ -445,9 +453,18 @@ gc::StartVerifyPreBarriers(JSRuntime *rt)
     if (rt->gcVerifyPreData || rt->gcIncrementalState != NO_INCREMENTAL)
         return;
 
-    MinorGC(rt, JS::gcreason::API);
+    /*
+     * The post barrier verifier requires the storebuffer to be enabled, but the
+     * pre barrier verifier disables it as part of disabling GGC.  Don't allow
+     * starting the pre barrier verifier if the post barrier verifier is already
+     * running.
+     */
+    if (rt->gcVerifyPostData)
+        return;
 
-    AutoPrepareForTracing prep(rt);
+    MinorGC(rt, JS::gcreason::EVICT_NURSERY);
+
+    AutoPrepareForTracing prep(rt, WithAtoms);
 
     if (!IsIncrementalGCSafe(rt))
         return;
@@ -455,7 +472,7 @@ gc::StartVerifyPreBarriers(JSRuntime *rt)
     for (GCChunkSet::Range r(rt->gcChunkSet.all()); !r.empty(); r.popFront())
         r.front()->bitmap.clear();
 
-    VerifyPreTracer *trc = js_new<VerifyPreTracer>();
+    VerifyPreTracer *trc = js_new<VerifyPreTracer>(rt);
     if (!trc)
         return;
 
@@ -510,7 +527,7 @@ gc::StartVerifyPreBarriers(JSRuntime *rt)
     rt->gcMarker.start();
 
     rt->setNeedsBarrier(true);
-    for (ZonesIter zone(rt); !zone.done(); zone.next()) {
+    for (ZonesIter zone(rt, WithAtoms); !zone.done(); zone.next()) {
         PurgeJITCaches(zone);
         zone->setNeedsBarrier(true, Zone::UpdateIon);
         zone->allocator.arenas.purge();
@@ -564,6 +581,10 @@ AssertMarkedOrAllocated(const EdgeValue &edge)
     if (!edge.thing || IsMarkedOrAllocated(static_cast<Cell *>(edge.thing)))
         return;
 
+    // Permanent atoms aren't marked during graph traversal.
+    if (edge.kind == JSTRACE_STRING && static_cast<JSString *>(edge.thing)->isPermanentAtom())
+        return;
+
     char msgbuf[1024];
     const char *label = edge.label ? edge.label : "<unknown>";
 
@@ -575,7 +596,9 @@ AssertMarkedOrAllocated(const EdgeValue &edge)
 void
 gc::EndVerifyPreBarriers(JSRuntime *rt)
 {
-    AutoPrepareForTracing prep(rt);
+    JS_ASSERT(!JS::IsGenerationalGCEnabled(rt));
+
+    AutoPrepareForTracing prep(rt, SkipAtoms);
 
     VerifyPreTracer *trc = (VerifyPreTracer *)rt->gcVerifyPreData;
 
@@ -585,7 +608,7 @@ gc::EndVerifyPreBarriers(JSRuntime *rt)
     bool compartmentCreated = false;
 
     /* We need to disable barriers before tracing, which may invoke barriers. */
-    for (ZonesIter zone(rt); !zone.done(); zone.next()) {
+    for (ZonesIter zone(rt, WithAtoms); !zone.done(); zone.next()) {
         if (!zone->needsBarrier())
             compartmentCreated = true;
 
@@ -657,7 +680,7 @@ gc::StartVerifyPostBarriers(JSRuntime *rt)
         return;
     }
 
-    MinorGC(rt, JS::gcreason::API);
+    MinorGC(rt, JS::gcreason::EVICT_NURSERY);
 
     VerifyPostTracer *trc = js_new<VerifyPostTracer>();
     if (!trc)
@@ -740,7 +763,7 @@ js::gc::EndVerifyPostBarriers(JSRuntime *rt)
 {
 #ifdef JSGC_GENERATIONAL
     VerifyPostTracer::EdgeSet edges;
-    AutoPrepareForTracing prep(rt);
+    AutoPrepareForTracing prep(rt, SkipAtoms);
 
     VerifyPostTracer *trc = (VerifyPostTracer *)rt->gcVerifyPostData;
 
@@ -749,7 +772,7 @@ js::gc::EndVerifyPostBarriers(JSRuntime *rt)
     if (!edges.init())
         goto oom;
     trc->edges = &edges;
-    rt->gcStoreBuffer.mark(trc);
+    rt->gcStoreBuffer.markAll(trc);
 
     /* Walk the heap to find any edges not the the |edges| set. */
     JS_TracerInit(trc, rt, PostVerifierVisitEdge);
@@ -812,16 +835,18 @@ MaybeVerifyPreBarriers(JSRuntime *rt, bool always)
 
         EndVerifyPreBarriers(rt);
     }
+
     StartVerifyPreBarriers(rt);
 }
 
 static void
 MaybeVerifyPostBarriers(JSRuntime *rt, bool always)
 {
+#ifdef JSGC_GENERATIONAL
     if (rt->gcZeal() != ZealVerifierPostValue)
         return;
 
-    if (rt->mainThread.suppressGC)
+    if (rt->mainThread.suppressGC || !rt->gcStoreBuffer.isEnabled())
         return;
 
     if (VerifyPostTracer *trc = (VerifyPostTracer *)rt->gcVerifyPostData) {
@@ -831,6 +856,7 @@ MaybeVerifyPostBarriers(JSRuntime *rt, bool always)
         EndVerifyPostBarriers(rt);
     }
     StartVerifyPostBarriers(rt);
+#endif
 }
 
 void
