@@ -32,18 +32,20 @@
 using namespace js;
 using namespace jit;
 using namespace frontend;
+using mozilla::PodCopy;
 using mozilla::PodEqual;
 using mozilla::Compression::LZ4;
 
 void
 AsmJSModule::initHeap(Handle<ArrayBufferObject*> heap, JSContext *cx)
 {
-    JS_ASSERT(linked_);
+    JS_ASSERT(IsValidAsmJSHeapLength(heap->byteLength()));
+    JS_ASSERT(dynamicallyLinked_);
     JS_ASSERT(!maybeHeap_);
+
     maybeHeap_ = heap;
     heapDatum() = heap->dataPointer();
 
-    JS_ASSERT(IsValidAsmJSHeapLength(heap->byteLength()));
 #if defined(JS_CODEGEN_X86)
     uint8_t *heapOffset = heap->dataPointer();
     void *heapLength = (void*)heap->byteLength();
@@ -62,9 +64,6 @@ AsmJSModule::initHeap(Handle<ArrayBufferObject*> heap, JSContext *cx)
         jit::Assembler::updateBoundsCheck(heapLength,
                                           (jit::Instruction*)(heapAccesses_[i].offset() + code_));
     }
-    // We already know the exact extent of areas that need to be patched, just make sure we
-    // flush all of them at once.
-    jit::AutoFlushCache::updateTop(uintptr_t(code_), pod.codeBytes_);
 #endif
 }
 
@@ -208,6 +207,14 @@ RedirectCall(void *fun, ABIFunctionType type)
     return fun;
 }
 
+#ifdef DEBUG
+static void
+AssumeUnreachable()
+{
+    MOZ_CRASH("Reached unreachable code in asm.js");
+}
+#endif
+
 static void *
 AddressOf(AsmJSImmKind kind, ExclusiveContext *cx)
 {
@@ -219,7 +226,7 @@ AddressOf(AsmJSImmKind kind, ExclusiveContext *cx)
       case AsmJSImm_ReportOverRecursed:
         return RedirectCall(FuncCast<void (JSContext*)>(js_ReportOverRecursed), Args_General1);
       case AsmJSImm_HandleExecutionInterrupt:
-        return RedirectCall(FuncCast(js_HandleExecutionInterrupt), Args_General1);
+        return RedirectCall(FuncCast(js::HandleExecutionInterrupt), Args_General1);
       case AsmJSImm_InvokeFromAsmJS_Ignore:
         return RedirectCall(FuncCast(InvokeFromAsmJS_Ignore), Args_General4);
       case AsmJSImm_InvokeFromAsmJS_ToInt32:
@@ -272,6 +279,10 @@ AddressOf(AsmJSImmKind kind, ExclusiveContext *cx)
         return RedirectCall(FuncCast(ecmaPow), Args_Double_DoubleDouble);
       case AsmJSImm_ATan2D:
         return RedirectCall(FuncCast(ecmaAtan2), Args_Double_DoubleDouble);
+#ifdef DEBUG
+      case AsmJSImm_AssumeUnreachable:
+        return RedirectCall(FuncCast(AssumeUnreachable), Args_General0);
+#endif
       case AsmJSImm_Invalid:
         break;
     }
@@ -281,19 +292,54 @@ AddressOf(AsmJSImmKind kind, ExclusiveContext *cx)
 }
 
 void
-AsmJSModule::staticallyLink(const AsmJSStaticLinkData &linkData, ExclusiveContext *cx)
+AsmJSModule::restoreToInitialState(ArrayBufferObject *maybePrevBuffer, ExclusiveContext *cx)
 {
-    // Process AsmJSStaticLinkData:
+#ifdef DEBUG
+    // Put the absolute links back to -1 so patchDataWithValueCheck assertions
+    // in staticallyLink are valid.
+    for (size_t i = 0; i < staticLinkData_.absoluteLinks.length(); i++) {
+        AbsoluteLink link = staticLinkData_.absoluteLinks[i];
+        Assembler::patchDataWithValueCheck(code_ + link.patchAt.offset(),
+                                           PatchedImmPtr((void*)-1),
+                                           PatchedImmPtr(AddressOf(link.target, cx)));
+    }
+#endif
 
-    operationCallbackExit_ = code_ + linkData.operationCallbackExitOffset;
+    if (maybePrevBuffer) {
+#if defined(JS_CODEGEN_X86)
+        // Subtract out the base-pointer added by AsmJSModule::initHeap.
+        uint8_t *ptrBase = maybePrevBuffer->dataPointer();
+        for (unsigned i = 0; i < heapAccesses_.length(); i++) {
+            const jit::AsmJSHeapAccess &access = heapAccesses_[i];
+            void *addr = access.patchOffsetAt(code_);
+            uint8_t *ptr = reinterpret_cast<uint8_t*>(JSC::X86Assembler::getPointer(addr));
+            JS_ASSERT(ptr >= ptrBase);
+            JSC::X86Assembler::setPointer(addr, (void *)(ptr - ptrBase));
+        }
+#endif
+    }
+}
 
-    for (size_t i = 0; i < linkData.relativeLinks.length(); i++) {
-        AsmJSStaticLinkData::RelativeLink link = linkData.relativeLinks[i];
+void
+AsmJSModule::setAutoFlushICacheRange()
+{
+    AutoFlushICache::setRange(uintptr_t(code_), pod.codeBytes_);
+}
+
+void
+AsmJSModule::staticallyLink(ExclusiveContext *cx)
+{
+    // Process staticLinkData_
+
+    interruptExit_ = code_ + staticLinkData_.interruptExitOffset;
+
+    for (size_t i = 0; i < staticLinkData_.relativeLinks.length(); i++) {
+        RelativeLink link = staticLinkData_.relativeLinks[i];
         *(void **)(code_ + link.patchAtOffset) = code_ + link.targetOffset;
     }
 
-    for (size_t i = 0; i < linkData.absoluteLinks.length(); i++) {
-        AsmJSStaticLinkData::AbsoluteLink link = linkData.absoluteLinks[i];
+    for (size_t i = 0; i < staticLinkData_.absoluteLinks.length(); i++) {
+        AbsoluteLink link = staticLinkData_.absoluteLinks[i];
         Assembler::patchDataWithValueCheck(code_ + link.patchAt.offset(),
                                            PatchedImmPtr(AddressOf(link.target, cx)),
                                            PatchedImmPtr((void*)-1));
@@ -312,11 +358,12 @@ AsmJSModule::AsmJSModule(ScriptSource *scriptSource, uint32_t charsBegin)
     importArgumentName_(nullptr),
     bufferArgumentName_(nullptr),
     code_(nullptr),
-    operationCallbackExit_(nullptr),
-    linked_(false),
+    interruptExit_(nullptr),
+    dynamicallyLinked_(false),
     loadedFromCache_(false),
     charsBegin_(charsBegin),
-    scriptSource_(scriptSource)
+    scriptSource_(scriptSource),
+    codeIsProtected_(false)
 {
     mozilla::PodZero(&pod);
     scriptSource_->incref();
@@ -368,7 +415,8 @@ AsmJSModule::addSizeOfMisc(mozilla::MallocSizeOf mallocSizeOf, size_t *asmJSModu
                         profiledFunctions_.sizeOfExcludingThis(mallocSizeOf) +
                         perfProfiledBlocksFunctions_.sizeOfExcludingThis(mallocSizeOf) +
 #endif
-                        functionCounts_.sizeOfExcludingThis(mallocSizeOf);
+                        functionCounts_.sizeOfExcludingThis(mallocSizeOf) +
+                        staticLinkData_.sizeOfExcludingThis(mallocSizeOf);
 }
 
 static void
@@ -535,6 +583,20 @@ DeserializeVector(ExclusiveContext *cx, const uint8_t *cursor, js::Vector<T, 0,
     return cursor;
 }
 
+template <class T>
+bool
+CloneVector(ExclusiveContext *cx, const Vector<T, 0, SystemAllocPolicy> &in,
+            Vector<T, 0, SystemAllocPolicy> *out)
+{
+    if (!out->resize(in.length()))
+        return false;
+    for (size_t i = 0; i < in.length(); i++) {
+        if (!in[i].clone(cx, &(*out)[i]))
+            return false;
+    }
+    return true;
+}
+
 template <class T, class AllocPolicy, class ThisVector>
 size_t
 SerializedPodVectorSize(const mozilla::VectorBase<T, 0, AllocPolicy, ThisVector> &vec)
@@ -565,6 +627,17 @@ DeserializePodVector(ExclusiveContext *cx, const uint8_t *cursor,
     return cursor;
 }
 
+template <class T>
+bool
+ClonePodVector(ExclusiveContext *cx, const Vector<T, 0, SystemAllocPolicy> &in,
+               Vector<T, 0, SystemAllocPolicy> *out)
+{
+    if (!out->resize(in.length()))
+        return false;
+    PodCopy(out->begin(), in.begin(), in.length());
+    return true;
+}
+
 uint8_t *
 AsmJSModule::Global::serialize(uint8_t *cursor) const
 {
@@ -588,6 +661,13 @@ AsmJSModule::Global::deserialize(ExclusiveContext *cx, const uint8_t *cursor)
     return cursor;
 }
 
+bool
+AsmJSModule::Global::clone(ExclusiveContext *cx, Global *out) const
+{
+    *out = *this;
+    return true;
+}
+
 uint8_t *
 AsmJSModule::Exit::serialize(uint8_t *cursor) const
 {
@@ -608,6 +688,13 @@ AsmJSModule::Exit::deserialize(ExclusiveContext *cx, const uint8_t *cursor)
     return cursor;
 }
 
+bool
+AsmJSModule::Exit::clone(ExclusiveContext *cx, Exit *out) const
+{
+    *out = *this;
+    return true;
+}
+
 uint8_t *
 AsmJSModule::ExportedFunction::serialize(uint8_t *cursor) const
 {
@@ -638,6 +725,60 @@ AsmJSModule::ExportedFunction::deserialize(ExclusiveContext *cx, const uint8_t *
     return cursor;
 }
 
+bool
+AsmJSModule::ExportedFunction::clone(ExclusiveContext *cx, ExportedFunction *out) const
+{
+    out->name_ = name_;
+    out->maybeFieldName_ = maybeFieldName_;
+
+    if (!ClonePodVector(cx, argCoercions_, &out->argCoercions_))
+        return false;
+
+    out->pod = pod;
+    return true;
+}
+
+size_t
+AsmJSModule::StaticLinkData::serializedSize() const
+{
+    return sizeof(uint32_t) +
+           SerializedPodVectorSize(relativeLinks) +
+           SerializedPodVectorSize(absoluteLinks);
+}
+
+uint8_t *
+AsmJSModule::StaticLinkData::serialize(uint8_t *cursor) const
+{
+    cursor = WriteScalar<uint32_t>(cursor, interruptExitOffset);
+    cursor = SerializePodVector(cursor, relativeLinks);
+    cursor = SerializePodVector(cursor, absoluteLinks);
+    return cursor;
+}
+
+const uint8_t *
+AsmJSModule::StaticLinkData::deserialize(ExclusiveContext *cx, const uint8_t *cursor)
+{
+    (cursor = ReadScalar<uint32_t>(cursor, &interruptExitOffset)) &&
+    (cursor = DeserializePodVector(cx, cursor, &relativeLinks)) &&
+    (cursor = DeserializePodVector(cx, cursor, &absoluteLinks));
+    return cursor;
+}
+
+bool
+AsmJSModule::StaticLinkData::clone(ExclusiveContext *cx, StaticLinkData *out) const
+{
+    out->interruptExitOffset = interruptExitOffset;
+    return ClonePodVector(cx, relativeLinks, &out->relativeLinks) &&
+           ClonePodVector(cx, absoluteLinks, &out->absoluteLinks);
+}
+
+size_t
+AsmJSModule::StaticLinkData::sizeOfExcludingThis(mozilla::MallocSizeOf mallocSizeOf) const
+{
+    return relativeLinks.sizeOfExcludingThis(mallocSizeOf) +
+           absoluteLinks.sizeOfExcludingThis(mallocSizeOf);
+}
+
 size_t
 AsmJSModule::serializedSize() const
 {
@@ -649,7 +790,8 @@ AsmJSModule::serializedSize() const
            SerializedVectorSize(globals_) +
            SerializedVectorSize(exits_) +
            SerializedVectorSize(exports_) +
-           SerializedPodVectorSize(heapAccesses_);
+           SerializedPodVectorSize(heapAccesses_) +
+           staticLinkData_.serializedSize();
 }
 
 uint8_t *
@@ -664,6 +806,7 @@ AsmJSModule::serialize(uint8_t *cursor) const
     cursor = SerializeVector(cursor, exits_);
     cursor = SerializeVector(cursor, exports_);
     cursor = SerializePodVector(cursor, heapAccesses_);
+    cursor = staticLinkData_.serialize(cursor);
     return cursor;
 }
 
@@ -683,36 +826,135 @@ AsmJSModule::deserialize(ExclusiveContext *cx, const uint8_t *cursor)
     (cursor = DeserializeVector(cx, cursor, &globals_)) &&
     (cursor = DeserializeVector(cx, cursor, &exits_)) &&
     (cursor = DeserializeVector(cx, cursor, &exports_)) &&
-    (cursor = DeserializePodVector(cx, cursor, &heapAccesses_));
+    (cursor = DeserializePodVector(cx, cursor, &heapAccesses_)) &&
+    (cursor = staticLinkData_.deserialize(cx, cursor));
 
     loadedFromCache_ = true;
+
     return cursor;
 }
 
-size_t
-AsmJSStaticLinkData::serializedSize() const
+// When a module is cloned, we memcpy its executable code. If, right before or
+// during the clone, another thread calls AsmJSModule::protectCode() then the
+// executable code will become inaccessible. In theory, we could take away only
+// PROT_EXEC, but this seems to break emulators.
+class AutoUnprotectCodeForClone
 {
-    return sizeof(uint32_t) +
-           SerializedPodVectorSize(relativeLinks) +
-           SerializedPodVectorSize(absoluteLinks);
+    JSRuntime *rt_;
+    JSRuntime::AutoLockForInterrupt lock_;
+    const AsmJSModule &module_;
+    const bool protectedBefore_;
+
+  public:
+    AutoUnprotectCodeForClone(JSContext *cx, const AsmJSModule &module)
+      : rt_(cx->runtime()),
+        lock_(rt_),
+        module_(module),
+        protectedBefore_(module_.codeIsProtected(rt_))
+    {
+        if (protectedBefore_)
+            module_.unprotectCode(rt_);
+    }
+
+    ~AutoUnprotectCodeForClone()
+    {
+        if (protectedBefore_)
+            module_.protectCode(rt_);
+    }
+};
+
+bool
+AsmJSModule::clone(JSContext *cx, ScopedJSDeletePtr<AsmJSModule> *moduleOut) const
+{
+    AutoUnprotectCodeForClone cloneGuard(cx, *this);
+
+    *moduleOut = cx->new_<AsmJSModule>(scriptSource_, charsBegin_);
+    if (!*moduleOut)
+        return false;
+
+    AsmJSModule &out = **moduleOut;
+
+    // Mirror the order of serialize/deserialize in cloning:
+
+    out.pod = pod;
+
+    out.code_ = AllocateExecutableMemory(cx, pod.totalBytes_);
+    if (!out.code_)
+        return false;
+
+    memcpy(out.code_, code_, pod.codeBytes_);
+
+    out.globalArgumentName_ = globalArgumentName_;
+    out.importArgumentName_ = importArgumentName_;
+    out.bufferArgumentName_ = bufferArgumentName_;
+
+    if (!CloneVector(cx, globals_, &out.globals_) ||
+        !CloneVector(cx, exits_, &out.exits_) ||
+        !CloneVector(cx, exports_, &out.exports_) ||
+        !ClonePodVector(cx, heapAccesses_, &out.heapAccesses_) ||
+        !staticLinkData_.clone(cx, &out.staticLinkData_))
+    {
+        return false;
+    }
+
+    out.loadedFromCache_ = loadedFromCache_;
+
+    // We already know the exact extent of areas that need to be patched, just make sure we
+    // flush all of them at once.
+    out.setAutoFlushICacheRange();
+
+    out.restoreToInitialState(maybeHeap_, cx);
+    return true;
 }
 
-uint8_t *
-AsmJSStaticLinkData::serialize(uint8_t *cursor) const
+void
+AsmJSModule::protectCode(JSRuntime *rt) const
 {
-    cursor = WriteScalar<uint32_t>(cursor, operationCallbackExitOffset);
-    cursor = SerializePodVector(cursor, relativeLinks);
-    cursor = SerializePodVector(cursor, absoluteLinks);
-    return cursor;
+    JS_ASSERT(rt->currentThreadOwnsInterruptLock());
+
+    codeIsProtected_ = true;
+
+    if (!pod.functionBytes_)
+        return;
+
+    // Technically, we should be able to only take away the execute permissions,
+    // however this seems to break our emulators which don't always check
+    // execute permissions while executing code.
+#if defined(XP_WIN)
+    DWORD oldProtect;
+    if (!VirtualProtect(codeBase(), functionBytes(), PAGE_NOACCESS, &oldProtect))
+        MOZ_CRASH();
+#else  // assume Unix
+    if (mprotect(codeBase(), functionBytes(), PROT_NONE))
+        MOZ_CRASH();
+#endif
 }
 
-const uint8_t *
-AsmJSStaticLinkData::deserialize(ExclusiveContext *cx, const uint8_t *cursor)
+void
+AsmJSModule::unprotectCode(JSRuntime *rt) const
 {
-    (cursor = ReadScalar<uint32_t>(cursor, &operationCallbackExitOffset)) &&
-    (cursor = DeserializePodVector(cx, cursor, &relativeLinks)) &&
-    (cursor = DeserializePodVector(cx, cursor, &absoluteLinks));
-    return cursor;
+    JS_ASSERT(rt->currentThreadOwnsInterruptLock());
+
+    codeIsProtected_ = false;
+
+    if (!pod.functionBytes_)
+        return;
+
+#if defined(XP_WIN)
+    DWORD oldProtect;
+    if (!VirtualProtect(codeBase(), functionBytes(), PAGE_EXECUTE_READWRITE, &oldProtect))
+        MOZ_CRASH();
+#else  // assume Unix
+    if (mprotect(codeBase(), functionBytes(), PROT_READ | PROT_WRITE | PROT_EXEC))
+        MOZ_CRASH();
+#endif
+}
+
+bool
+AsmJSModule::codeIsProtected(JSRuntime *rt) const
+{
+    JS_ASSERT(rt->currentThreadOwnsInterruptLock());
+    return codeIsProtected_;
 }
 
 static bool
@@ -974,7 +1216,6 @@ struct ScopedCacheEntryOpenedForWrite
 bool
 js::StoreAsmJSModuleInCache(AsmJSParser &parser,
                             const AsmJSModule &module,
-                            const AsmJSStaticLinkData &linkData,
                             ExclusiveContext *cx)
 {
     MachineId machineId;
@@ -987,8 +1228,7 @@ js::StoreAsmJSModuleInCache(AsmJSParser &parser,
 
     size_t serializedSize = machineId.serializedSize() +
                             moduleChars.serializedSize() +
-                            module.serializedSize() +
-                            linkData.serializedSize();
+                            module.serializedSize();
 
     JS::OpenAsmJSCacheEntryForWriteOp open = cx->asmJSCacheOps().openEntryForWrite;
     if (!open)
@@ -996,16 +1236,18 @@ js::StoreAsmJSModuleInCache(AsmJSParser &parser,
 
     const jschar *begin = parser.tokenStream.rawBase() + ModuleChars::beginOffset(parser);
     const jschar *end = parser.tokenStream.rawBase() + ModuleChars::endOffset(parser);
+    bool installed = parser.options().installedFile;
 
     ScopedCacheEntryOpenedForWrite entry(cx, serializedSize);
-    if (!open(cx->global(), begin, end, entry.serializedSize, &entry.memory, &entry.handle))
+    if (!open(cx->global(), installed, begin, end, entry.serializedSize,
+              &entry.memory, &entry.handle)) {
         return false;
+    }
 
     uint8_t *cursor = entry.memory;
     cursor = machineId.serialize(cursor);
     cursor = moduleChars.serialize(cursor);
     cursor = module.serialize(cursor);
-    cursor = linkData.serialize(cursor);
 
     JS_ASSERT(cursor == entry.memory + serializedSize);
     return true;
@@ -1070,11 +1312,13 @@ js::LookupAsmJSModuleInCache(ExclusiveContext *cx,
     if (!module)
         return false;
     cursor = module->deserialize(cx, cursor);
-    if (!cursor)
-        return false;
 
-    AsmJSStaticLinkData linkData(cx);
-    cursor = linkData.deserialize(cx, cursor);
+    // No need to flush the instruction cache now, it will be flushed when dynamically linking.
+    AutoFlushICache afc("LookupAsmJSModuleInCache", /* inhibit= */ true);
+    // We already know the exact extent of areas that need to be patched, just make sure we
+    // flush all of them at once.
+    module->setAutoFlushICacheRange();
+
     if (!cursor)
         return false;
 
@@ -1083,7 +1327,7 @@ js::LookupAsmJSModuleInCache(ExclusiveContext *cx,
     if (!atEnd)
         return true;
 
-    module->staticallyLink(linkData, cx);
+    module->staticallyLink(cx);
 
     parser.tokenStream.advance(module->charsEnd());
 
