@@ -97,13 +97,15 @@ struct LinearTerm
 class LinearSum
 {
   public:
-    LinearSum()
-      : constant_(0)
+    LinearSum(TempAllocator &alloc)
+      : terms_(alloc),
+        constant_(0)
     {
     }
 
     LinearSum(const LinearSum &other)
-      : constant_(other.constant_)
+      : terms_(other.terms_.allocPolicy()),
+        constant_(other.constant_)
     {
         terms_.appendAll(other.terms_);
     }
@@ -119,6 +121,7 @@ class LinearSum
 
     void print(Sprinter &sp) const;
     void dump(FILE *) const;
+    void dump() const;
 
   private:
     Vector<LinearTerm, 2, IonAllocPolicy> terms_;
@@ -126,7 +129,7 @@ class LinearSum
 };
 
 bool
-AnalyzeNewScriptProperties(JSContext *cx, HandleFunction fun,
+AnalyzeNewScriptProperties(JSContext *cx, JSFunction *fun,
                            types::TypeObject *type, HandleObject baseobj,
                            Vector<types::TypeNewScript::Initializer> *initializerList);
 
