@@ -9,11 +9,12 @@
 
 #ifdef JS_ION
 
+#include "jsfun.h"
 #include "jsscript.h"
 #include "jstypes.h"
 
 #include "jit/IonCode.h"
-#include "jit/SnapshotReader.h"
+#include "jit/Snapshots.h"
 
 namespace js {
     class ActivationIterator;
@@ -66,6 +67,17 @@ enum FrameType
     IonFrame_Osr
 };
 
+enum ReadFrameArgsBehavior {
+    // Only read formals (i.e. [0 ... callee()->nargs]
+    ReadFrame_Formals,
+
+    // Only read overflown args (i.e. [callee()->nargs ... numActuals()]
+    ReadFrame_Overflown,
+
+    // Read all args (i.e. [0 ... numActuals()])
+    ReadFrame_Actuals
+};
+
 class IonCommonFrameLayout;
 class IonJSFrameLayout;
 class IonExitFrameLayout;
@@ -85,21 +97,24 @@ class IonFrameIterator
   private:
     mutable const SafepointIndex *cachedSafepointIndex_;
     const JitActivation *activation_;
+    ExecutionMode mode_;
 
     void dumpBaseline() const;
 
   public:
-    IonFrameIterator(uint8_t *top)
+    explicit IonFrameIterator(uint8_t *top, ExecutionMode mode)
       : current_(top),
         type_(IonFrame_Exit),
-        returnAddressToFp_(NULL),
+        returnAddressToFp_(nullptr),
         frameSize_(0),
-        cachedSafepointIndex_(NULL),
-        activation_(NULL)
+        cachedSafepointIndex_(nullptr),
+        activation_(nullptr),
+        mode_(mode)
     { }
 
-    IonFrameIterator(const ActivationIterator &activations);
-    IonFrameIterator(IonJSFrameLayout *fp);
+    explicit IonFrameIterator(JSContext *cx);
+    explicit IonFrameIterator(const ActivationIterator &activations);
+    explicit IonFrameIterator(IonJSFrameLayout *fp, ExecutionMode mode);
 
     // Current frame information.
     FrameType type() const {
@@ -151,7 +166,6 @@ class IonFrameIterator
         return type_ == IonFrame_Entry;
     }
     bool isFunctionFrame() const;
-    bool isParallelFunctionFrame() const;
 
     bool isConstructing() const;
 
@@ -203,15 +217,24 @@ class IonFrameIterator
     MachineState machineState() const;
 
     template <class Op>
-    void forEachCanonicalActualArg(Op op, unsigned start, unsigned count) const {
+    void unaliasedForEachActual(Op op, ReadFrameArgsBehavior behavior) const {
         JS_ASSERT(isBaselineJS());
 
         unsigned nactual = numActualArgs();
-        if (count == unsigned(-1))
-            count = nactual - start;
-
-        unsigned end = start + count;
-        JS_ASSERT(start <= end && end <= nactual);
+        unsigned start, end;
+        switch (behavior) {
+          case ReadFrame_Formals:
+            start = 0;
+            end = callee()->nargs();
+            break;
+          case ReadFrame_Overflown:
+            start = callee()->nargs();
+            end = nactual;
+            break;
+          case ReadFrame_Actuals:
+            start = 0;
+            end = nactual;
+        }
 
         Value *argv = actualArgs();
         for (unsigned i = start; i < end; i++)
@@ -235,13 +258,17 @@ class SnapshotIterator : public SnapshotReader
     IonScript *ionScript_;
 
   private:
-    bool hasLocation(const SnapshotReader::Location &loc);
-    uintptr_t fromLocation(const SnapshotReader::Location &loc);
-    static Value FromTypedPayload(JSValueType type, uintptr_t payload);
+    // Read a spilled register from the machine state.
+    bool hasRegister(const Location &loc);
+    uintptr_t fromRegister(const Location &loc);
 
-    Value slotValue(const Slot &slot);
-    bool slotReadable(const Slot &slot);
-    void warnUnreadableSlot();
+    // Read an uintptr_t from the stack.
+    bool hasStack(const Location &loc);
+    uintptr_t fromStack(const Location &loc);
+
+    Value allocationValue(const RValueAllocation &a);
+    bool allocationReadable(const RValueAllocation &a);
+    void warnUnreadableAllocation();
 
   public:
     SnapshotIterator(IonScript *ionScript, SnapshotOffset snapshotOffset,
@@ -250,27 +277,34 @@ class SnapshotIterator : public SnapshotReader
     SnapshotIterator(const IonBailoutIterator &iter);
     SnapshotIterator();
 
+    Value skip() {
+        readAllocation();
+        return UndefinedValue();
+    }
     Value read() {
-        return slotValue(readSlot());
+        return allocationValue(readAllocation());
     }
     Value maybeRead(bool silentFailure = false) {
-        Slot s = readSlot();
-        if (slotReadable(s))
-            return slotValue(s);
+        RValueAllocation a = readAllocation();
+        if (allocationReadable(a))
+            return allocationValue(a);
         if (!silentFailure)
-            warnUnreadableSlot();
+            warnUnreadableAllocation();
         return UndefinedValue();
     }
 
     template <class Op>
-    void readFrameArgs(Op &op, const Value *argv, Value *scopeChain, Value *thisv,
-                       unsigned start, unsigned formalEnd, unsigned iterEnd, JSScript *script)
+    void readFrameArgs(Op &op, Value *scopeChain, Value *thisv,
+                       unsigned start, unsigned end, JSScript *script)
     {
         if (scopeChain)
             *scopeChain = read();
         else
             skip();
 
+        // Skip slot for return value.
+        skip();
+
         // Skip slot for arguments object.
         if (script->argumentsHasVarBinding())
             skip();
@@ -281,33 +315,29 @@ class SnapshotIterator : public SnapshotReader
             skip();
 
         unsigned i = 0;
-        if (formalEnd < start)
+        if (end < start)
             i = start;
 
         for (; i < start; i++)
             skip();
-        for (; i < formalEnd && i < iterEnd; i++) {
+        for (; i < end; i++) {
             // We are not always able to read values from the snapshots, some values
             // such as non-gc things may still be live in registers and cause an
             // error while reading the machine state.
             Value v = maybeRead();
             op(v);
         }
-        if (iterEnd >= formalEnd) {
-            for (; i < iterEnd; i++)
-                op(argv[i]);
-        }
     }
 
-    Value maybeReadSlotByIndex(size_t index) {
+    Value maybeReadAllocByIndex(size_t index) {
         while (index--) {
-            JS_ASSERT(moreSlots());
+            JS_ASSERT(moreAllocations());
             skip();
         }
 
         Value s = maybeRead(true);
 
-        while (moreSlots())
+        while (moreAllocations())
             skip();
 
         return s;
@@ -328,6 +358,10 @@ class InlineFrameIteratorMaybeGC
     jsbytecode *pc_;
     uint32_t numActualArgs_;
 
+    struct Nop {
+        void operator()(const Value &v) { }
+    };
+
   private:
     void findNextFrame();
 
@@ -349,7 +383,7 @@ class InlineFrameIteratorMaybeGC
     InlineFrameIteratorMaybeGC(JSContext *cx, const IonBailoutIterator *iter);
 
     InlineFrameIteratorMaybeGC(JSContext *cx, const InlineFrameIteratorMaybeGC *iter)
-      : frame_(iter ? iter->frame_ : NULL),
+      : frame_(iter ? iter->frame_ : nullptr),
         framesRead_(0),
         callee_(cx),
         script_(cx)
@@ -386,50 +420,65 @@ class InlineFrameIteratorMaybeGC
         return frame_->numActualArgs();
     }
 
-    template <class Op>
-    void forEachCanonicalActualArg(JSContext *cx, Op op, unsigned start, unsigned count) const {
+    template <class ArgOp, class LocalOp>
+    void readFrameArgsAndLocals(JSContext *cx, ArgOp &argOp, LocalOp &localOp,
+                                Value *scopeChain, Value *thisv,
+                                ReadFrameArgsBehavior behavior) const
+    {
         unsigned nactual = numActualArgs();
-        if (count == unsigned(-1))
-            count = nactual - start;
+        unsigned nformal = callee()->nargs();
 
-        unsigned end = start + count;
-        unsigned nformal = callee()->nargs;
+        // Get the non overflown arguments, which are taken from the inlined
+        // frame, because it will have the updated value when JSOP_SETARG is
+        // done.
+        SnapshotIterator s(si_);
+        if (behavior != ReadFrame_Overflown)
+            s.readFrameArgs(argOp, scopeChain, thisv, 0, nformal, script());
 
-        JS_ASSERT(start <= end && end <= nactual);
+        if (behavior != ReadFrame_Formals) {
+            if (more()) {
+                // There is still a parent frame of this inlined frame.  All
+                // arguments (also the overflown) are the last pushed values
+                // in the parent frame.  To get the overflown arguments, we
+                // need to take them from there.
 
-        if (more()) {
-            // There is still a parent frame of this inlined frame.
-            // The not overflown arguments are taken from the inlined frame,
-            // because it will have the updated value when JSOP_SETARG is done.
-            // All arguments (also the overflown) are the last pushed values in the parent frame.
-            // To get the overflown arguments, we need to take them from there.
+                // The overflown arguments are not available in current frame.
+                // They are the last pushed arguments in the parent frame of
+                // this inlined frame.
+                InlineFrameIteratorMaybeGC it(cx, this);
+                ++it;
+                unsigned argsObjAdj = it.script()->argumentsHasVarBinding() ? 1 : 0;
+                SnapshotIterator parent_s(it.snapshotIterator());
 
-            // Get the non overflown arguments
-            unsigned formal_end = (end < nformal) ? end : nformal;
-            SnapshotIterator s(si_);
-            s.readFrameArgs(op, NULL, NULL, NULL, start, nformal, formal_end, script());
+                // Skip over all slots until we get to the last slots
+                // (= arguments slots of callee) the +3 is for [this], [returnvalue],
+                // [scopechain], and maybe +1 for [argsObj]
+                JS_ASSERT(parent_s.allocations() >= nactual + 3 + argsObjAdj);
+                unsigned skip = parent_s.allocations() - nactual - 3 - argsObjAdj;
+                for (unsigned j = 0; j < skip; j++)
+                    parent_s.skip();
 
-            // The overflown arguments are not available in current frame.
-            // They are the last pushed arguments in the parent frame of this inlined frame.
-            InlineFrameIteratorMaybeGC it(cx, this);
-            ++it;
-            unsigned argsObjAdj = it.script()->argumentsHasVarBinding() ? 1 : 0;
-            SnapshotIterator parent_s(it.snapshotIterator());
-
-            // Skip over all slots untill we get to the last slots (= arguments slots of callee)
-            // the +2 is for [this] and [scopechain], and maybe +1 for [argsObj]
-            JS_ASSERT(parent_s.slots() >= nactual + 2 + argsObjAdj);
-            unsigned skip = parent_s.slots() - nactual - 2 - argsObjAdj;
-            for (unsigned j = 0; j < skip; j++)
-                parent_s.skip();
-
-            // Get the overflown arguments
-            parent_s.readFrameArgs(op, NULL, NULL, NULL, nformal, nactual, end, it.script());
-        } else {
-            SnapshotIterator s(si_);
-            Value *argv = frame_->actualArgs();
-            s.readFrameArgs(op, argv, NULL, NULL, start, nformal, end, script());
+                // Get the overflown arguments
+                parent_s.readFrameArgs(argOp, nullptr, nullptr, nformal, nactual, it.script());
+            } else {
+                // There is no parent frame to this inlined frame, we can read
+                // from the frame's Value vector directly.
+                Value *argv = frame_->actualArgs();
+                for (unsigned i = nformal; i < nactual; i++)
+                    argOp(argv[i]);
+            }
         }
+
+        // At this point we've read all the formals in s, and can read the
+        // locals.
+        for (unsigned i = 0; i < script()->nfixed(); i++)
+            localOp(s.read());
+    }
+
+    template <class Op>
+    void unaliasedForEachActual(JSContext *cx, Op op, ReadFrameArgsBehavior behavior) const {
+        Nop nop;
+        readFrameArgsAndLocals(cx, op, nop, nullptr, nullptr, behavior);
     }
 
     JSScript *script() const {
@@ -449,10 +498,8 @@ class InlineFrameIteratorMaybeGC
 
         // scopeChain
         Value v = s.read();
-        if (v.isObject()) {
-            JS_ASSERT_IF(script()->hasAnalysis(), script()->analysis()->usesScopeChain());
+        if (v.isObject())
             return &v.toObject();
-        }
 
         return callee()->environment();
     }
@@ -464,6 +511,9 @@ class InlineFrameIteratorMaybeGC
         // scopeChain
         s.skip();
 
+        // return value
+        s.skip();
+
         // Arguments object.
         if (script()->argumentsHasVarBinding())
             s.skip();
