@@ -46,12 +46,13 @@ class MBasicBlock : public TempObject, public InlineListNode<MBasicBlock>
     MBasicBlock(MIRGraph &graph, CompileInfo &info, jsbytecode *pc, Kind kind);
     bool init();
     void copySlots(MBasicBlock *from);
-    bool inherit(BytecodeAnalysis *analysis, MBasicBlock *pred, uint32_t popped);
+    bool inherit(TempAllocator &alloc, BytecodeAnalysis *analysis, MBasicBlock *pred,
+                 uint32_t popped, unsigned stackPhiCount = 0);
     bool inheritResumePoint(MBasicBlock *pred);
     void assertUsesAreNotWithin(MUseIterator use, MUseIterator end);
 
-    // Does this block do something that forces it to terminate early?
-    bool earlyAbort_;
+    // This block cannot be reached by any means.
+    bool unreachable_;
 
     // Pushes a copy of a local variable or argument.
     void pushVariable(uint32_t slot);
@@ -75,7 +76,8 @@ class MBasicBlock : public TempObject, public InlineListNode<MBasicBlock>
                                            MBasicBlock *pred, jsbytecode *entryPc,
                                            MResumePoint *resumePoint);
     static MBasicBlock *NewPendingLoopHeader(MIRGraph &graph, CompileInfo &info,
-                                             MBasicBlock *pred, jsbytecode *entryPc);
+                                             MBasicBlock *pred, jsbytecode *entryPc,
+                                             unsigned loopStateSlots);
     static MBasicBlock *NewSplitEdge(MIRGraph &graph, CompileInfo &info, MBasicBlock *pred);
     static MBasicBlock *NewAbortPar(MIRGraph &graph, CompileInfo &info,
                                     MBasicBlock *pred, jsbytecode *entryPc,
@@ -88,14 +90,11 @@ class MBasicBlock : public TempObject, public InlineListNode<MBasicBlock>
     void setId(uint32_t id) {
         id_ = id;
     }
-    void setEarlyAbort() {
-        earlyAbort_ = true;
-    }
-    void clearEarlyAbort() {
-        earlyAbort_ = false;
-    }
-    bool earlyAbort() {
-        return earlyAbort_;
+
+    // Mark the current block and all dominated blocks as unreachable.
+    void setUnreachable();
+    bool unreachable() {
+        return unreachable_;
     }
     // Move the definition to the top of the stack.
     void pick(int32_t depth);
@@ -168,13 +167,13 @@ class MBasicBlock : public TempObject, public InlineListNode<MBasicBlock>
     // Adds a predecessor. Every predecessor must have the same exit stack
     // depth as the entry state to this block. Adding a predecessor
     // automatically creates phi nodes and rewrites uses as needed.
-    bool addPredecessor(MBasicBlock *pred);
-    bool addPredecessorPopN(MBasicBlock *pred, uint32_t popped);
+    bool addPredecessor(TempAllocator &alloc, MBasicBlock *pred);
+    bool addPredecessorPopN(TempAllocator &alloc, MBasicBlock *pred, uint32_t popped);
 
     // Stranger utilities used for inlining.
     bool addPredecessorWithoutPhis(MBasicBlock *pred);
     void inheritSlots(MBasicBlock *parent);
-    bool initEntrySlots();
+    bool initEntrySlots(TempAllocator &alloc);
 
     // Replaces an edge for a given block with a new block. This is
     // used for critical edge splitting and also for inserting
@@ -468,12 +467,13 @@ class MBasicBlock : public TempObject, public InlineListNode<MBasicBlock>
     }
 
     bool strict() const {
-        return info_.script()->strict;
+        return info_.script()->strict();
     }
 
     void dumpStack(FILE *fp);
 
     void dump(FILE *fp);
+    void dump();
 
     // Track bailouts by storing the current pc in MIR instruction added at this
     // cycle. This is also used for tracking calls when profiling.
@@ -532,13 +532,13 @@ typedef InlineListIterator<MBasicBlock> MBasicBlockIterator;
 typedef InlineListIterator<MBasicBlock> ReversePostorderIterator;
 typedef InlineListReverseIterator<MBasicBlock> PostorderIterator;
 
-typedef Vector<MBasicBlock *, 1, IonAllocPolicy> MIRGraphExits;
+typedef Vector<MBasicBlock *, 1, IonAllocPolicy> MIRGraphReturns;
 
 class MIRGraph
 {
     InlineList<MBasicBlock> blocks_;
     TempAllocator *alloc_;
-    MIRGraphExits *exitAccumulator_;
+    MIRGraphReturns *returnAccumulator_;
     uint32_t blockIdGen_;
     uint32_t idGen_;
     MBasicBlock *osrBlock_;
@@ -550,32 +550,36 @@ class MIRGraph
   public:
     MIRGraph(TempAllocator *alloc)
       : alloc_(alloc),
-        exitAccumulator_(nullptr),
+        returnAccumulator_(nullptr),
         blockIdGen_(0),
-        idGen_(0),
+        idGen_(1),
         osrBlock_(nullptr),
         osrStart_(nullptr),
         numBlocks_(0),
         hasTryBlock_(false)
     { }
 
+    TempAllocator &alloc() const {
+        return *alloc_;
+    }
+
     void addBlock(MBasicBlock *block);
     void insertBlockAfter(MBasicBlock *at, MBasicBlock *block);
 
     void unmarkBlocks();
 
-    void setExitAccumulator(MIRGraphExits *accum) {
-        exitAccumulator_ = accum;
+    void setReturnAccumulator(MIRGraphReturns *accum) {
+        returnAccumulator_ = accum;
     }
-    MIRGraphExits *exitAccumulator() const {
-        return exitAccumulator_;
+    MIRGraphReturns *returnAccumulator() const {
+        return returnAccumulator_;
     }
 
-    bool addExit(MBasicBlock *exitBlock) {
-        if (!exitAccumulator_)
+    bool addReturn(MBasicBlock *returnBlock) {
+        if (!returnAccumulator_)
             return true;
 
-        return exitAccumulator_->append(exitBlock);
+        return returnAccumulator_->append(returnBlock);
     }
 
     MBasicBlock *entryBlock() {
@@ -588,7 +592,9 @@ class MIRGraph
         numBlocks_ = 0;
     }
     void resetInstructionNumber() {
-        idGen_ = 0;
+        // This intentionally starts above 0. The id 0 is in places used to
+        // indicate a failure to perform an operation on an instruction.
+        idGen_ = 1;
     }
     MBasicBlockIterator begin() {
         return blocks_.begin();
@@ -628,12 +634,9 @@ class MIRGraph
         return blockIdGen_;
     }
     void allocDefinitionId(MDefinition *ins) {
-        // This intentionally starts above 0. The id 0 is in places used to
-        // indicate a failure to perform an operation on an instruction.
-        idGen_ += 2;
-        ins->setId(idGen_);
+        ins->setId(idGen_++);
     }
-    uint32_t getMaxInstructionId() {
+    uint32_t getNumInstructionIds() {
         return idGen_;
     }
     MResumePoint *entryResumePoint() {
@@ -668,12 +671,13 @@ class MIRGraph
     }
 
     // The per-thread context. So as not to modify the calling convention for
-    // parallel code, we obtain the current slice from thread-local storage.
-    // This helper method will lazilly insert an MForkJoinSlice instruction in
-    // the entry block and return the definition.
-    MDefinition *forkJoinSlice();
+    // parallel code, we obtain the current ForkJoinContext from thread-local
+    // storage.  This helper method will lazilly insert an MForkJoinContext
+    // instruction in the entry block and return the definition.
+    MDefinition *forkJoinContext();
 
     void dump(FILE *fp);
+    void dump();
 };
 
 class MDefinitionIterator
