@@ -18,9 +18,9 @@ using namespace jit;
 using mozilla::FloorLog2;
 
 bool
-SafepointWriter::init(uint32_t slotCount)
+SafepointWriter::init(TempAllocator &alloc, uint32_t slotCount)
 {
-    frameSlots_ = BitSet::New(slotCount);
+    frameSlots_ = BitSet::New(alloc, slotCount / sizeof(intptr_t));
     if (!frameSlots_)
         return false;
 
@@ -99,14 +99,17 @@ MapSlotsToBitset(BitSet *set, CompactBufferWriter &stream, uint32_t nslots, uint
     set->clear();
 
     for (uint32_t i = 0; i < nslots; i++) {
-        // Slots are represented at a distance from |fp|. Since the stack grows
-        // down, this means slots start at index 1, so we subtract 1 to pack
-        // the bitset.
-        set->insert(slots[i] - 1);
+        // Slots are represented at a distance from |fp|. We divide by the
+        // pointer size, since we only care about pointer-sized/aligned slots
+        // here. Since the stack grows down, this means slots start at index 1,
+        // so we subtract 1 to pack the bitset.
+        JS_ASSERT(slots[i] % sizeof(intptr_t) == 0);
+        JS_ASSERT(slots[i] / sizeof(intptr_t) > 0);
+        set->insert(slots[i] / sizeof(intptr_t) - 1);
     }
 
     size_t count = set->rawLength();
-    uint32_t *words = set->raw();
+    const uint32_t *words = set->raw();
     for (size_t i = 0; i < count; i++)
         stream.writeUnsigned(words[i]);
 }
@@ -334,7 +337,7 @@ SafepointWriter::endEntry()
 SafepointReader::SafepointReader(IonScript *script, const SafepointIndex *si)
   : stream_(script->safepoints() + si->safepointOffset(),
             script->safepoints() + script->safepointsSize()),
-    frameSlots_(script->frameSlots())
+    frameSlots_(script->frameSlots() / sizeof(intptr_t))
 {
     osiCallPointOffset_ = stream_.readUnsigned();
 
@@ -374,22 +377,21 @@ SafepointReader::InvalidationPatchPoint(IonScript *script, const SafepointIndex
 void
 SafepointReader::advanceFromGcRegs()
 {
-    currentSlotChunkNumber_ = 0;
-    currentSlotChunk_ = stream_.readUnsigned();
+    currentSlotChunk_ = 0;
+    nextSlotChunkNumber_ = 0;
 }
 
 bool
 SafepointReader::getSlotFromBitmap(uint32_t *slot)
 {
     while (currentSlotChunk_ == 0) {
-        currentSlotChunkNumber_++;
-
         // Are there any more chunks to read?
-        if (currentSlotChunkNumber_ == BitSet::RawLengthForBits(frameSlots_))
+        if (nextSlotChunkNumber_ == BitSet::RawLengthForBits(frameSlots_))
             return false;
 
         // Yes, read the next chunk.
         currentSlotChunk_ = stream_.readUnsigned();
+        nextSlotChunkNumber_++;
     }
 
     // The current chunk still has bits in it, so get the next bit, then mask
@@ -398,8 +400,9 @@ SafepointReader::getSlotFromBitmap(uint32_t *slot)
     currentSlotChunk_ &= ~(1 << bit);
 
     // Return the slot, taking care to add 1 back in since it was subtracted
-    // when added in the original bitset.
-    *slot = (currentSlotChunkNumber_ * sizeof(uint32_t) * 8) + bit + 1;
+    // when added in the original bitset, and re-scale it by the pointer size,
+    // reversing the transformation in MapSlotsToBitset.
+    *slot = (((nextSlotChunkNumber_ - 1) * BitSet::BitsPerWord) + bit + 1) * sizeof(intptr_t);
     return true;
 }
 
@@ -416,8 +419,8 @@ void
 SafepointReader::advanceFromGcSlots()
 {
     // No, reset the counter.
-    currentSlotChunkNumber_ = 0;
-    currentSlotChunk_ = stream_.readUnsigned();
+    currentSlotChunk_ = 0;
+    nextSlotChunkNumber_ = 0;
 }
 
 bool
@@ -453,7 +456,7 @@ PartFromStream(CompactBufferReader &stream, NunboxPartKind kind, uint32_t info)
         return LStackSlot(info);
 
     JS_ASSERT(kind == Part_Arg);
-    return LArgument(LAllocation::INT_ARGUMENT, info);
+    return LArgument(info);
 }
 
 bool
