@@ -7,9 +7,8 @@
 #ifndef jit_x64_Assembler_x64_h
 #define jit_x64_Assembler_x64_h
 
-#include "mozilla/Util.h"
+#include "mozilla/ArrayUtils.h"
 
-#include "jit/CompactBuffer.h"
 #include "jit/IonCode.h"
 #include "jit/shared/Assembler-shared.h"
 
@@ -76,6 +75,7 @@ static MOZ_CONSTEXPR_VAR Register HeapReg = r15;
 static MOZ_CONSTEXPR_VAR FloatRegister ReturnFloatReg = xmm0;
 static MOZ_CONSTEXPR_VAR FloatRegister ScratchFloatReg = xmm15;
 
+// Avoid rbp, which is the FramePointer, which is unavailable in some modes.
 static MOZ_CONSTEXPR_VAR Register ArgumentsRectifierReg = r8;
 static MOZ_CONSTEXPR_VAR Register CallTempReg0 = rax;
 static MOZ_CONSTEXPR_VAR Register CallTempReg1 = rdi;
@@ -83,7 +83,6 @@ static MOZ_CONSTEXPR_VAR Register CallTempReg2 = rbx;
 static MOZ_CONSTEXPR_VAR Register CallTempReg3 = rcx;
 static MOZ_CONSTEXPR_VAR Register CallTempReg4 = rsi;
 static MOZ_CONSTEXPR_VAR Register CallTempReg5 = rdx;
-static MOZ_CONSTEXPR_VAR Register CallTempReg6 = rbp;
 
 // Different argument registers for WIN64
 #if defined(_WIN64)
@@ -130,6 +129,13 @@ static MOZ_CONSTEXPR_VAR uint32_t NumFloatArgRegs = 8;
 static MOZ_CONSTEXPR_VAR FloatRegister FloatArgRegs[NumFloatArgRegs] = { xmm0, xmm1, xmm2, xmm3, xmm4, xmm5, xmm6, xmm7 };
 #endif
 
+// The convention used by the ForkJoinGetSlice stub. None of these can be rax
+// or rdx, which the stub also needs for cmpxchg and div, respectively.
+static MOZ_CONSTEXPR_VAR Register ForkJoinGetSliceReg_cx = rdi;
+static MOZ_CONSTEXPR_VAR Register ForkJoinGetSliceReg_temp0 = rbx;
+static MOZ_CONSTEXPR_VAR Register ForkJoinGetSliceReg_temp1 = rcx;
+static MOZ_CONSTEXPR_VAR Register ForkJoinGetSliceReg_output = rsi;
+
 class ABIArgGenerator
 {
 #if defined(XP_WIN)
@@ -168,95 +174,6 @@ static const uint32_t AlignmentMidPrologue = AlignmentAtPrologue;
 
 static const Scale ScalePointer = TimesEight;
 
-class Operand
-{
-  public:
-    enum Kind {
-        REG,
-        REG_DISP,
-        FPREG,
-        SCALE
-    };
-
-    Kind kind_ : 3;
-    int32_t base_ : 5;
-    Scale scale_ : 3;
-    int32_t index_ : 5;
-    int32_t disp_;
-
-  public:
-    explicit Operand(Register reg)
-      : kind_(REG),
-        base_(reg.code())
-    { }
-    explicit Operand(FloatRegister reg)
-      : kind_(FPREG),
-        base_(reg.code())
-    { }
-    explicit Operand(const Address &address)
-      : kind_(REG_DISP),
-        base_(address.base.code()),
-        disp_(address.offset)
-    { }
-    explicit Operand(const BaseIndex &address)
-      : kind_(SCALE),
-        base_(address.base.code()),
-        scale_(address.scale),
-        index_(address.index.code()),
-        disp_(address.offset)
-    { }
-    Operand(Register base, Register index, Scale scale, int32_t disp = 0)
-      : kind_(SCALE),
-        base_(base.code()),
-        scale_(scale),
-        index_(index.code()),
-        disp_(disp)
-    { }
-    Operand(Register reg, int32_t disp)
-      : kind_(REG_DISP),
-        base_(reg.code()),
-        disp_(disp)
-    { }
-
-    Address toAddress() const {
-        JS_ASSERT(kind() == REG_DISP);
-        return Address(Register::FromCode(base()), disp());
-    }
-
-    BaseIndex toBaseIndex() const {
-        JS_ASSERT(kind() == SCALE);
-        return BaseIndex(Register::FromCode(base()), Register::FromCode(index()), scale(), disp());
-    }
-
-    Kind kind() const {
-        return kind_;
-    }
-    Register::Code reg() const {
-        JS_ASSERT(kind() == REG);
-        return (Registers::Code)base_;
-    }
-    Registers::Code base() const {
-        JS_ASSERT(kind() == REG_DISP || kind() == SCALE);
-        return (Registers::Code)base_;
-    }
-    Registers::Code index() const {
-        JS_ASSERT(kind() == SCALE);
-        return (Registers::Code)index_;
-    }
-    Scale scale() const {
-        JS_ASSERT(kind() == SCALE);
-        return scale_;
-    }
-    FloatRegisters::Code fpu() const {
-        JS_ASSERT(kind() == FPREG);
-        return (FloatRegisters::Code)base_;
-    }
-    int32_t disp() const {
-        JS_ASSERT(kind() == REG_DISP || kind() == SCALE);
-        return disp_;
-    }
-};
-
 } // namespace jit
 } // namespace js
 
@@ -285,24 +202,25 @@ class Assembler : public AssemblerX86Shared
     // The start of the relocation table contains the offset from the code
     // buffer to the start of the extended jump table.
     //
-    // Each entry in this table is a jmp [rip], where the next eight bytes
-    // contain an immediate address. This comes out to 14 bytes, which we pad
-    // to 16.
+    // Each entry in this table is a jmp [rip], followed by a ud2 to hint to the
+    // hardware branch predictor that there is no fallthrough, followed by the
+    // eight bytes containing an immediate address. This comes out to 16 bytes.
     //    +1 byte for opcode
     //    +1 byte for mod r/m
-    //    +4 bytes for rip-relative offset (0)
+    //    +4 bytes for rip-relative offset (2)
+    //    +2 bytes for ud2 instruction
     //    +8 bytes for 64-bit address
     //
-    static const uint32_t SizeOfExtendedJump = 1 + 1 + 4 + 8;
+    static const uint32_t SizeOfExtendedJump = 1 + 1 + 4 + 2 + 8;
     static const uint32_t SizeOfJumpTableEntry = 16;
 
     uint32_t extendedJumpTable_;
 
-    static IonCode *CodeFromJump(IonCode *code, uint8_t *jump);
+    static JitCode *CodeFromJump(JitCode *code, uint8_t *jump);
 
   private:
     void writeRelocation(JmpSrc src, Relocation::Kind reloc);
-    void addPendingJump(JmpSrc src, void *target, Relocation::Kind reloc);
+    void addPendingJump(JmpSrc src, ImmPtr target, Relocation::Kind reloc);
 
   protected:
     size_t addPatchableJump(JmpSrc src, Relocation::Kind reloc);
@@ -313,7 +231,7 @@ class Assembler : public AssemblerX86Shared
     using AssemblerX86Shared::push;
     using AssemblerX86Shared::pop;
 
-    static uint8_t *PatchableJumpAddress(IonCode *code, size_t index);
+    static uint8_t *PatchableJumpAddress(JitCode *code, size_t index);
     static void PatchJumpEntry(uint8_t *entry, uint8_t *target);
 
     Assembler()
@@ -321,7 +239,7 @@ class Assembler : public AssemblerX86Shared
     {
     }
 
-    static void TraceJumpRelocations(JSTracer *trc, IonCode *code, CompactBufferReader &reader);
+    static void TraceJumpRelocations(JSTracer *trc, JitCode *code, CompactBufferReader &reader);
 
     // The buffer is about to be linked, make sure any constant pools or excess
     // bookkeeping has been flushed to the instruction stream.
@@ -347,9 +265,12 @@ class Assembler : public AssemblerX86Shared
             push(ScratchReg);
         }
     }
+    void push(const ImmPtr &imm) {
+        push(ImmWord(uintptr_t(imm.value)));
+    }
     void push(const FloatRegister &src) {
         subq(Imm32(sizeof(double)), StackPointer);
-        movsd(src, Operand(StackPointer, 0));
+        movsd(src, Address(StackPointer, 0));
     }
     CodeOffsetLabel pushWithPatch(const ImmWord &word) {
         CodeOffsetLabel label = movWithPatch(word, ScratchReg);
@@ -358,7 +279,7 @@ class Assembler : public AssemblerX86Shared
     }
 
     void pop(const FloatRegister &src) {
-        movsd(Operand(StackPointer, 0), src);
+        movsd(Address(StackPointer, 0), src);
         addq(Imm32(sizeof(double)), StackPointer);
     }
 
@@ -366,6 +287,9 @@ class Assembler : public AssemblerX86Shared
         masm.movq_i64r(word.value, dest.code());
         return masm.currentOffset();
     }
+    CodeOffsetLabel movWithPatch(const ImmPtr &imm, const Register &dest) {
+        return movWithPatch(ImmWord(uintptr_t(imm.value)), dest);
+    }
 
     // Load an ImmWord value into a register. Note that this instruction will
     // attempt to optimize its immediate field size. When a full 64-bit
@@ -385,6 +309,9 @@ class Assembler : public AssemblerX86Shared
             masm.movq_i64r(word.value, dest.code());
         }
     }
+    void movq(ImmPtr imm, const Register &dest) {
+        movq(ImmWord(uintptr_t(imm.value)), dest);
+    }
     void movq(ImmGCPtr ptr, const Register &dest) {
         masm.movq_i64r(ptr.value, dest.code());
         writeDataRelocation(ptr);
@@ -394,12 +321,15 @@ class Assembler : public AssemblerX86Shared
           case Operand::REG:
             masm.movq_rr(src.reg(), dest.code());
             break;
-          case Operand::REG_DISP:
+          case Operand::MEM_REG_DISP:
             masm.movq_mr(src.disp(), src.base(), dest.code());
             break;
-          case Operand::SCALE:
+          case Operand::MEM_SCALE:
             masm.movq_mr(src.disp(), src.base(), src.index(), src.scale(), dest.code());
             break;
+          case Operand::MEM_ADDRESS32:
+            masm.movq_mr(src.address(), dest.code());
+            break;
           default:
             MOZ_ASSUME_UNREACHABLE("unexpected operand kind");
         }
@@ -409,12 +339,15 @@ class Assembler : public AssemblerX86Shared
           case Operand::REG:
             masm.movq_rr(src.code(), dest.reg());
             break;
-          case Operand::REG_DISP:
+          case Operand::MEM_REG_DISP:
             masm.movq_rm(src.code(), dest.disp(), dest.base());
             break;
-          case Operand::SCALE:
+          case Operand::MEM_SCALE:
             masm.movq_rm(src.code(), dest.disp(), dest.base(), dest.index(), dest.scale());
             break;
+          case Operand::MEM_ADDRESS32:
+            masm.movq_rm(src.code(), dest.address());
+            break;
           default:
             MOZ_ASSUME_UNREACHABLE("unexpected operand kind");
         }
@@ -424,20 +357,23 @@ class Assembler : public AssemblerX86Shared
           case Operand::REG:
             masm.movl_i32r(imm32.value, dest.reg());
             break;
-          case Operand::REG_DISP:
+          case Operand::MEM_REG_DISP:
             masm.movq_i32m(imm32.value, dest.disp(), dest.base());
             break;
-          case Operand::SCALE:
+          case Operand::MEM_SCALE:
             masm.movq_i32m(imm32.value, dest.disp(), dest.base(), dest.index(), dest.scale());
             break;
+          case Operand::MEM_ADDRESS32:
+            masm.movq_i32m(imm32.value, dest.address());
+            break;
           default:
             MOZ_ASSUME_UNREACHABLE("unexpected operand kind");
         }
     }
-    void movqsd(const Register &src, const FloatRegister &dest) {
+    void movq(const Register &src, const FloatRegister &dest) {
         masm.movq_rr(src.code(), dest.code());
     }
-    void movqsd(const FloatRegister &src, const Register &dest) {
+    void movq(const FloatRegister &src, const Register &dest) {
         masm.movq_rr(src.code(), dest.code());
     }
     void movq(const Register &src, const Register &dest) {
@@ -463,9 +399,12 @@ class Assembler : public AssemblerX86Shared
           case Operand::REG:
             masm.addq_ir(imm.value, dest.reg());
             break;
-          case Operand::REG_DISP:
+          case Operand::MEM_REG_DISP:
             masm.addq_im(imm.value, dest.disp(), dest.base());
             break;
+          case Operand::MEM_ADDRESS32:
+            masm.addq_im(imm.value, dest.address());
+            break;
           default:
             MOZ_ASSUME_UNREACHABLE("unexpected operand kind");
         }
@@ -478,9 +417,12 @@ class Assembler : public AssemblerX86Shared
           case Operand::REG:
             masm.addq_rr(src.reg(), dest.code());
             break;
-          case Operand::REG_DISP:
+          case Operand::MEM_REG_DISP:
             masm.addq_mr(src.disp(), src.base(), dest.code());
             break;
+          case Operand::MEM_ADDRESS32:
+            masm.addq_mr(src.address(), dest.code());
+            break;
           default:
             MOZ_ASSUME_UNREACHABLE("unexpected operand kind");
         }
@@ -497,9 +439,12 @@ class Assembler : public AssemblerX86Shared
           case Operand::REG:
             masm.subq_rr(src.reg(), dest.code());
             break;
-          case Operand::REG_DISP:
+          case Operand::MEM_REG_DISP:
             masm.subq_mr(src.disp(), src.base(), dest.code());
             break;
+          case Operand::MEM_ADDRESS32:
+            masm.subq_mr(src.address(), dest.code());
+            break;
           default:
             MOZ_ASSUME_UNREACHABLE("unexpected operand kind");
         }
@@ -524,9 +469,12 @@ class Assembler : public AssemblerX86Shared
           case Operand::REG:
             masm.orq_rr(src.reg(), dest.code());
             break;
-          case Operand::REG_DISP:
+          case Operand::MEM_REG_DISP:
             masm.orq_mr(src.disp(), src.base(), dest.code());
             break;
+          case Operand::MEM_ADDRESS32:
+            masm.orq_mr(src.address(), dest.code());
+            break;
           default:
             MOZ_ASSUME_UNREACHABLE("unexpected operand kind");
         }
@@ -539,10 +487,23 @@ class Assembler : public AssemblerX86Shared
     }
 
     void mov(ImmWord word, const Register &dest) {
-        movq(word, dest);
+        // Use xor for setting registers to zero, as it is specially optimized
+        // for this purpose on modern hardware. Note that it does clobber FLAGS
+        // though. Use xorl instead of xorq since they are functionally
+        // equivalent (32-bit instructions zero-extend their results to 64 bits)
+        // and xorl has a smaller encoding.
+        if (word.value == 0)
+            xorl(dest, dest);
+        else
+            movq(word, dest);
     }
-    void mov(const Imm32 &imm32, const Register &dest) {
-        movl(imm32, dest);
+    void mov(ImmPtr imm, const Register &dest) {
+        movq(imm, dest);
+    }
+    void mov(AsmJSImmPtr imm, const Register &dest) {
+        masm.movq_i64r(-1, dest.code());
+        AsmJSAbsoluteLink link(masm.currentOffset(), imm.kind());
+        enoughMemory_ &= asmJSAbsoluteLinks_.append(link);
     }
     void mov(const Operand &src, const Register &dest) {
         movq(src, dest);
@@ -568,10 +529,10 @@ class Assembler : public AssemblerX86Shared
     }
     void lea(const Operand &src, const Register &dest) {
         switch (src.kind()) {
-          case Operand::REG_DISP:
+          case Operand::MEM_REG_DISP:
             masm.leaq_mr(src.disp(), src.base(), dest.code());
             break;
-          case Operand::SCALE:
+          case Operand::MEM_SCALE:
             masm.leaq_mr(src.disp(), src.base(), src.index(), src.scale(), dest.code());
             break;
           default:
@@ -606,9 +567,12 @@ class Assembler : public AssemblerX86Shared
           case Operand::REG:
             masm.cmpq_rr(rhs.code(), lhs.reg());
             break;
-          case Operand::REG_DISP:
+          case Operand::MEM_REG_DISP:
             masm.cmpq_rm(rhs.code(), lhs.disp(), lhs.base());
             break;
+          case Operand::MEM_ADDRESS32:
+            masm.cmpq_rm(rhs.code(), lhs.address());
+            break;
           default:
             MOZ_ASSUME_UNREACHABLE("unexpected operand kind");
         }
@@ -618,9 +582,12 @@ class Assembler : public AssemblerX86Shared
           case Operand::REG:
             masm.cmpq_ir(rhs.value, lhs.reg());
             break;
-          case Operand::REG_DISP:
+          case Operand::MEM_REG_DISP:
             masm.cmpq_im(rhs.value, lhs.disp(), lhs.base());
             break;
+          case Operand::MEM_ADDRESS32:
+            masm.cmpq_im(rhs.value, lhs.address());
+            break;
           default:
             MOZ_ASSUME_UNREACHABLE("unexpected operand kind");
         }
@@ -630,7 +597,7 @@ class Assembler : public AssemblerX86Shared
           case Operand::REG:
             masm.cmpq_rr(rhs.reg(), lhs.code());
             break;
-          case Operand::REG_DISP:
+          case Operand::MEM_REG_DISP:
             masm.cmpq_mr(rhs.disp(), rhs.base(), lhs.code());
             break;
           default:
@@ -655,7 +622,7 @@ class Assembler : public AssemblerX86Shared
           case Operand::REG:
             masm.testq_i32r(rhs.value, lhs.reg());
             break;
-          case Operand::REG_DISP:
+          case Operand::MEM_REG_DISP:
             masm.testq_i32m(rhs.value, lhs.disp(), lhs.base());
             break;
           default:
@@ -664,33 +631,33 @@ class Assembler : public AssemblerX86Shared
         }
     }
 
-    void jmp(void *target, Relocation::Kind reloc = Relocation::HARDCODED) {
+    void jmp(ImmPtr target, Relocation::Kind reloc = Relocation::HARDCODED) {
         JmpSrc src = masm.jmp();
         addPendingJump(src, target, reloc);
     }
-    void j(Condition cond, void *target,
+    void j(Condition cond, ImmPtr target,
            Relocation::Kind reloc = Relocation::HARDCODED) {
         JmpSrc src = masm.jCC(static_cast<JSC::X86Assembler::Condition>(cond));
         addPendingJump(src, target, reloc);
     }
 
-    void jmp(IonCode *target) {
-        jmp(target->raw(), Relocation::IONCODE);
+    void jmp(JitCode *target) {
+        jmp(ImmPtr(target->raw()), Relocation::JITCODE);
     }
-    void j(Condition cond, IonCode *target) {
-        j(cond, target->raw(), Relocation::IONCODE);
+    void j(Condition cond, JitCode *target) {
+        j(cond, ImmPtr(target->raw()), Relocation::JITCODE);
     }
-    void call(IonCode *target) {
+    void call(JitCode *target) {
         JmpSrc src = masm.call();
-        addPendingJump(src, target->raw(), Relocation::IONCODE);
+        addPendingJump(src, ImmPtr(target->raw()), Relocation::JITCODE);
     }
 
     // Emit a CALL or CMP (nop) instruction. ToggleCall can be used to patch
     // this instruction.
-    CodeOffsetLabel toggledCall(IonCode *target, bool enabled) {
+    CodeOffsetLabel toggledCall(JitCode *target, bool enabled) {
         CodeOffsetLabel offset(size());
         JmpSrc src = enabled ? masm.call() : masm.cmp_eax();
-        addPendingJump(src, target->raw(), Relocation::IONCODE);
+        addPendingJump(src, ImmPtr(target->raw()), Relocation::JITCODE);
         JS_ASSERT(size() - offset.offset() == ToggledCallSize());
         return offset;
     }
@@ -706,9 +673,15 @@ class Assembler : public AssemblerX86Shared
     void cvttsd2sq(const FloatRegister &src, const Register &dest) {
         masm.cvttsd2sq_rr(src.code(), dest.code());
     }
+    void cvttss2sq(const FloatRegister &src, const Register &dest) {
+        masm.cvttss2sq_rr(src.code(), dest.code());
+    }
     void cvtsq2sd(const Register &src, const FloatRegister &dest) {
         masm.cvtsq2sd_rr(src.code(), dest.code());
     }
+    void cvtsq2ss(const Register &src, const FloatRegister &dest) {
+        masm.cvtsq2ss_rr(src.code(), dest.code());
+    }
 };
 
 static inline void
