@@ -7,8 +7,9 @@
 
 #include "jsapi-tests/tests.h"
 
+#include "jsobjinlines.h"
+
 #include "vm/ArgumentsObject-inl.h"
-#include "vm/Stack-inl.h"
 
 using namespace js;
 
@@ -33,16 +34,6 @@ static const char STRICT_THREE[] =
 static const char * const CALL_CODES[] =
     { "f()", "f(0)", "f(0, 1)", "f(0, 1, 2)", "f(0, 1, 2, 3)", "f(0, 1, 2, 3, 4)" };
 
-static const size_t MAX_ELEMS = 6;
-
-static void
-ClearElements(Value elems[MAX_ELEMS])
-{
-    for (size_t i = 0; i < MAX_ELEMS - 1; i++)
-        elems[i] = NullValue();
-    elems[MAX_ELEMS - 1] = Int32Value(42);
-}
-
 BEGIN_TEST(testArgumentsObject)
 {
     return ExhaustiveTest<0>(NORMAL_ZERO) &&
@@ -83,6 +74,8 @@ BEGIN_TEST(testArgumentsObject)
            ExhaustiveTest<5>(STRICT_THREE);
 }
 
+static const size_t MAX_ELEMS = 6;
+
 template<size_t ArgCount> bool
 ExhaustiveTest(const char funcode[])
 {
@@ -92,12 +85,12 @@ ExhaustiveTest(const char funcode[])
     EVAL(CALL_CODES[ArgCount], v.address());
     Rooted<ArgumentsObject*> argsobj(cx, &JSVAL_TO_OBJECT(v)->as<ArgumentsObject>());
 
-    Value elems[MAX_ELEMS];
+    JS::AutoValueArray<MAX_ELEMS> elems(cx);
 
     for (size_t i = 0; i <= ArgCount; i++) {
         for (size_t j = 0; j <= ArgCount - i; j++) {
             ClearElements(elems);
-            CHECK(argsobj->maybeGetElements(i, j, elems));
+            CHECK(argsobj->maybeGetElements(i, j, elems.begin()));
             for (size_t k = 0; k < j; k++)
                 CHECK_SAME(elems[k], INT_TO_JSVAL(i + k));
             for (size_t k = j; k < MAX_ELEMS - 1; k++)
@@ -108,4 +101,13 @@ ExhaustiveTest(const char funcode[])
 
     return true;
 }
+
+template <size_t N>
+static void
+ClearElements(JS::AutoValueArray<N> &elems)
+{
+    for (size_t i = 0; i < elems.length() - 1; i++)
+        elems[i].setNull();
+    elems[elems.length() - 1].setInt32(42);
+}
 END_TEST(testArgumentsObject)
