@@ -6,12 +6,12 @@
 
 #include "jsapi-tests/tests.h"
 
-#define NUM_TEST_BUFFERS 2
-#define MAGIC_VALUE_1 3
-#define MAGIC_VALUE_2 17
-
 BEGIN_TEST(testArrayBuffer_bug720949_steal)
 {
+    static const unsigned NUM_TEST_BUFFERS  = 2;
+    static const unsigned MAGIC_VALUE_1 = 3;
+    static const unsigned MAGIC_VALUE_2 = 17;
+
     JS::RootedObject buf_len1(cx), buf_len200(cx);
     JS::RootedObject tarray_len1(cx), tarray_len200(cx);
 
@@ -23,8 +23,7 @@ BEGIN_TEST(testArrayBuffer_bug720949_steal)
     CHECK(buf_len1 = JS_NewArrayBuffer(cx, sizes[0]));
     CHECK(tarray_len1 = JS_NewInt32ArrayWithBuffer(cx, testBuf[0], 0, -1));
 
-    JS::RootedValue dummy(cx, INT_TO_JSVAL(MAGIC_VALUE_1));
-    JS_SetElement(cx, testArray[0], 0, &dummy);
+    JS_SetElement(cx, testArray[0], 0, MAGIC_VALUE_1);
 
     // Many-element ArrayBuffer (uses dynamic storage)
     CHECK(buf_len200 = JS_NewArrayBuffer(cx, 200 * sizeof(uint32_t)));
@@ -45,7 +44,7 @@ BEGIN_TEST(testArrayBuffer_bug720949_steal)
         CHECK_SAME(v, INT_TO_JSVAL(size));
 
         // Modifying the underlying data should update the value returned through the view
-        uint8_t *data = JS_GetArrayBufferData(obj);
+        uint8_t *data = JS_GetStableArrayBufferData(cx, obj);
         CHECK(data != nullptr);
         *reinterpret_cast<uint32_t*>(data) = MAGIC_VALUE_2;
         CHECK(JS_GetElement(cx, view, 0, &v));
@@ -75,13 +74,13 @@ BEGIN_TEST(testArrayBuffer_bug720949_steal)
         // Transfer to a new ArrayBuffer
         JS::RootedObject dst(cx, JS_NewArrayBufferWithContents(cx, contents));
         CHECK(JS_IsArrayBufferObject(dst));
-        data = JS_GetArrayBufferData(obj);
+        data = JS_GetStableArrayBufferData(cx, obj);
 
         JS::RootedObject dstview(cx, JS_NewInt32ArrayWithBuffer(cx, dst, 0, -1));
         CHECK(dstview != nullptr);
 
         CHECK_EQUAL(JS_GetArrayBufferByteLength(dst), size);
-        data = JS_GetArrayBufferData(dst);
+        data = JS_GetStableArrayBufferData(cx, dst);
         CHECK(data != nullptr);
         CHECK_EQUAL(*reinterpret_cast<uint32_t*>(data), MAGIC_VALUE_2);
         CHECK(JS_GetElement(cx, dstview, 0, &v));
@@ -92,12 +91,6 @@ BEGIN_TEST(testArrayBuffer_bug720949_steal)
 }
 END_TEST(testArrayBuffer_bug720949_steal)
 
-static void GC(JSContext *cx)
-{
-    JS_GC(JS_GetRuntime(cx));
-    JS_GC(JS_GetRuntime(cx)); // Trigger another to wait for background finalization to end
-}
-
 // Varying number of views of a buffer, to test the neutering weak pointers
 BEGIN_TEST(testArrayBuffer_bug720949_viewList)
 {
@@ -162,6 +155,12 @@ BEGIN_TEST(testArrayBuffer_bug720949_viewList)
     return true;
 }
 
+static void GC(JSContext *cx)
+{
+    JS_GC(JS_GetRuntime(cx));
+    JS_GC(JS_GetRuntime(cx)); // Trigger another to wait for background finalization to end
+}
+
 bool isNeutered(JS::HandleObject obj) {
     JS::RootedValue v(cx);
     return JS_GetProperty(cx, obj, "byteLength", &v) && v.toInt32() == 0;
