@@ -33,8 +33,10 @@ BEGIN_TEST(test_cloneScript)
     {
         JSAutoCompartment a(cx, A);
         JSFunction *fun;
-        CHECK(fun = JS_CompileFunction(cx, A, "f", 0, nullptr, source, strlen(source),
-                                       __FILE__, 1));
+        JS::CompileOptions options(cx);
+        options.setFileAndLine(__FILE__, 1);
+        CHECK(fun = JS_CompileFunction(cx, A, "f", 0, nullptr, source,
+                                       strlen(source), options));
         CHECK(obj = JS_GetFunctionObject(fun));
     }
 
@@ -48,7 +50,7 @@ BEGIN_TEST(test_cloneScript)
 }
 END_TEST(test_cloneScript)
 
-void
+static void
 DestroyPrincipals(JSPrincipals *principals)
 {
     delete principals;
@@ -104,9 +106,11 @@ BEGIN_TEST(test_cloneScriptWithPrincipals)
     // Compile in A
     {
         JSAutoCompartment a(cx, A);
-        JS::RootedFunction fun(cx, JS_CompileFunctionForPrincipals(cx, A, principalsA, "f",
-                                                               mozilla::ArrayLength(argnames), argnames,
-                                                               source, strlen(source), __FILE__, 1));
+        JS::CompileOptions options(cx);
+        options.setFileAndLine(__FILE__, 1);
+        JS::RootedFunction fun(cx, JS_CompileFunction(cx, A, "f",
+                mozilla::ArrayLength(argnames), argnames, source,
+                strlen(source), options));
         CHECK(fun);
 
         JSScript *script;
@@ -132,8 +136,8 @@ BEGIN_TEST(test_cloneScriptWithPrincipals)
         CHECK(JS_GetScriptPrincipals(script) == principalsB);
 
         JS::RootedValue v(cx);
-        JS::Value args[] = { JS::Int32Value(1) };
-        CHECK(JS_CallFunctionValue(cx, B, JS::ObjectValue(*cloned), 1, args, v.address()));
+        JS::RootedValue arg(cx, JS::Int32Value(1));
+        CHECK(JS_CallFunctionValue(cx, B, clonedValue, arg, &v));
         CHECK(v.isObject());
 
         JSObject *funobj = &v.toObject();
