@@ -2,20 +2,12 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-#include "jsdbgapi.h"
-
+#include "js/OldDebugAPI.h"
 #include "jsapi-tests/tests.h"
 
-JSPrincipals *sOriginPrincipalsInErrorReporter = NULL;
-
-static void
-ErrorReporter(JSContext *cx, const char *message, JSErrorReport *report)
-{
-    sOriginPrincipalsInErrorReporter = report->originPrincipals;
-}
-
-JSPrincipals prin1 = { 1 };
-JSPrincipals prin2 = { 1 };
+static JSPrincipals *sOriginPrincipalsInErrorReporter = nullptr;
+static TestJSPrincipals prin1(1);
+static TestJSPrincipals prin2(1);
 
 BEGIN_TEST(testOriginPrincipals)
 {
@@ -52,8 +44,14 @@ BEGIN_TEST(testOriginPrincipals)
     return true;
 }
 
+static void
+ErrorReporter(JSContext *cx, const char *message, JSErrorReport *report)
+{
+    sOriginPrincipalsInErrorReporter = report->originPrincipals;
+}
+
 bool
-eval(const char *asciiChars, JSPrincipals *principals, JSPrincipals *originPrincipals, jsval *rval)
+eval(const char *asciiChars, JSPrincipals *principals, JSPrincipals *originPrincipals, JS::MutableHandleValue rval)
 {
     size_t len = strlen(asciiChars);
     jschar *chars = new jschar[len+1];
@@ -65,11 +63,14 @@ eval(const char *asciiChars, JSPrincipals *principals, JSPrincipals *originPrinc
     CHECK(global);
     JSAutoCompartment ac(cx, global);
     CHECK(JS_InitStandardClasses(cx, global));
-    bool ok = JS_EvaluateUCScriptForPrincipalsVersionOrigin(cx, global,
-                                                            principals,
-                                                            originPrincipals,
-                                                            chars, len, "", 0, rval,
-                                                            JSVERSION_DEFAULT);
+
+
+    JS::CompileOptions options(cx);
+    options.setOriginPrincipals(originPrincipals)
+           .setFileAndLine("", 0);
+
+    bool ok = JS::Evaluate(cx, global, options, chars, len, rval.address());
+
     delete[] chars;
     return ok;
 }
@@ -86,7 +87,7 @@ bool
 testInner(const char *asciiChars, JSPrincipals *principal, JSPrincipals *originPrincipal)
 {
     JS::RootedValue rval(cx);
-    CHECK(eval(asciiChars, principal, originPrincipal, rval.address()));
+    CHECK(eval(asciiChars, principal, originPrincipal, &rval));
 
     JSScript *script = JS_GetFunctionScript(cx, &rval.toObject().as<JSFunction>());
     CHECK(JS_GetScriptPrincipals(script) == principal);
@@ -98,7 +99,7 @@ testInner(const char *asciiChars, JSPrincipals *principal, JSPrincipals *originP
 bool
 testError(const char *asciiChars)
 {
-    jsval rval;
+    JS::RootedValue rval(cx);
     CHECK(!eval(asciiChars, &prin1, &prin2 /* = originPrincipals */, &rval));
     CHECK(JS_ReportPendingException(cx));
     CHECK(sOriginPrincipalsInErrorReporter == &prin2);
