@@ -139,12 +139,21 @@ class JS_FRIEND_API(BaseProxyHandler)
      * The |act| parameter to enter() specifies the action being performed.
      * If |bp| is false, the trap suggests that the caller throw (though it
      * may still decide to squelch the error).
+     *
+     * We make these OR-able so that assertEnteredPolicy can pass a union of them.
+     * For example, get{,Own}PropertyDescriptor is invoked by both calls to ::get()
+     * and ::set() (since we need to look up the accessor), so its
+     * assertEnteredPolicy would pass GET | SET.
      */
-    enum Action {
-        GET,
-        SET,
-        CALL
+    typedef uint32_t Action;
+    enum {
+        NONE      = 0x00,
+        GET       = 0x01,
+        SET       = 0x02,
+        CALL      = 0x04,
+        ENUMERATE = 0x08
     };
+
     virtual bool enter(JSContext *cx, HandleObject wrapper, HandleId id, Action act,
                        bool *bp);
 
@@ -462,7 +471,7 @@ class JS_FRIEND_API(AutoEnterPolicy)
     {
         allow = handler->hasSecurityPolicy() ? handler->enter(cx, wrapper, id, act, &rv)
                                              : true;
-        recordEnter(cx, wrapper, id);
+        recordEnter(cx, wrapper, id, act);
         // We want to throw an exception if all of the following are true:
         // * The policy disallowed access.
         // * The policy set rv to false, indicating that we should throw.
@@ -481,6 +490,7 @@ class JS_FRIEND_API(AutoEnterPolicy)
     AutoEnterPolicy()
 #ifdef JS_DEBUG
         : context(nullptr)
+        , enteredAction(BaseProxyHandler::NONE)
 #endif
         {};
     void reportErrorIfExceptionIsNotPending(JSContext *cx, jsid id);
@@ -491,16 +501,18 @@ class JS_FRIEND_API(AutoEnterPolicy)
     JSContext *context;
     mozilla::Maybe<HandleObject> enteredProxy;
     mozilla::Maybe<HandleId> enteredId;
+    Action                   enteredAction;
+
     // NB: We explicitly don't track the entered action here, because sometimes
     // SET traps do an implicit GET during their implementation, leading to
     // spurious assertions.
     AutoEnterPolicy *prev;
-    void recordEnter(JSContext *cx, HandleObject proxy, HandleId id);
+    void recordEnter(JSContext *cx, HandleObject proxy, HandleId id, Action act);
     void recordLeave();
 
-    friend JS_FRIEND_API(void) assertEnteredPolicy(JSContext *cx, JSObject *proxy, jsid id);
+    friend JS_FRIEND_API(void) assertEnteredPolicy(JSContext *cx, JSObject *proxy, jsid id, Action act);
 #else
-    inline void recordEnter(JSContext *cx, JSObject *proxy, jsid id) {}
+    inline void recordEnter(JSContext *cx, JSObject *proxy, jsid id, Action act) {}
     inline void recordLeave() {}
 #endif
 
@@ -509,18 +521,32 @@ class JS_FRIEND_API(AutoEnterPolicy)
 #ifdef JS_DEBUG
 class JS_FRIEND_API(AutoWaivePolicy) : public AutoEnterPolicy {
 public:
-    AutoWaivePolicy(JSContext *cx, HandleObject proxy, HandleId id)
+    AutoWaivePolicy(JSContext *cx, HandleObject proxy, HandleId id,
+                    BaseProxyHandler::Action act)
     {
         allow = true;
-        recordEnter(cx, proxy, id);
+        recordEnter(cx, proxy, id, act);
     }
 };
 #else
 class JS_FRIEND_API(AutoWaivePolicy) {
-    public: AutoWaivePolicy(JSContext *cx, HandleObject proxy, HandleId id) {};
+  public:
+    AutoWaivePolicy(JSContext *cx, HandleObject proxy, HandleId id,
+                    BaseProxyHandler::Action act)
+    {}
 };
 #endif
 
+#ifdef JS_DEBUG
+extern JS_FRIEND_API(void)
+assertEnteredPolicy(JSContext *cx, JSObject *obj, jsid id,
+                    BaseProxyHandler::Action act);
+#else
+inline void assertEnteredPolicy(JSContext *cx, JSObject *obj, jsid id,
+                                BaseProxyHandler::Action act)
+{};
+#endif
+
 } /* namespace js */
 
 extern JS_FRIEND_API(JSObject *)
