@@ -31,63 +31,61 @@ namespace jit {
   class IonBuilder;
 }
 
-#ifdef JS_WORKER_THREADS
+#ifdef JS_THREADSAFE
 
-/* Per-runtime state for off thread work items. */
-class WorkerThreadState
+// Per-process state for off thread work items.
+class GlobalWorkerThreadState
 {
   public:
-    /* Available threads. */
+    // Number of CPUs to treat this machine as having when creating threads.
+    // May be accessed without locking.
+    size_t cpuCount;
+
+    // Number of threads to create. May be accessed without locking.
+    size_t threadCount;
+
+    typedef Vector<jit::IonBuilder*, 0, SystemAllocPolicy> IonBuilderVector;
+    typedef Vector<AsmJSParallelTask*, 0, SystemAllocPolicy> AsmJSParallelTaskVector;
+    typedef Vector<ParseTask*, 0, SystemAllocPolicy> ParseTaskVector;
+    typedef Vector<SourceCompressionTask*, 0, SystemAllocPolicy> SourceCompressionTaskVector;
+
+    // List of available threads, or null if the thread state has not been initialized.
     WorkerThread *threads;
-    size_t numThreads;
 
-    /*
-     * Whether all worker threads thread should pause their activity. This acts
-     * like the runtime's interrupt field and may be read without locking.
-     */
-    volatile size_t shouldPause;
+  private:
+    // The lists below are all protected by |lock|.
 
-    /* After shouldPause is set, the number of threads which are paused. */
-    uint32_t numPaused;
+    // Ion compilation worklist and finished jobs.
+    IonBuilderVector ionWorklist_, ionFinishedList_;
 
-    enum CondVar {
-        /* For notifying threads waiting for work that they may be able to make progress. */
-        CONSUMER,
+    // AsmJS worklist and finished jobs.
+    //
+    // Simultaneous AsmJS compilations all service the same AsmJS module.
+    // The main thread must pick up finished optimizations and perform codegen.
+    // |asmJSCompilationInProgress| is used to avoid triggering compilations
+    // for more than one module at a time.
+    AsmJSParallelTaskVector asmJSWorklist_, asmJSFinishedList_;
 
-        /* For notifying threads doing work that they may be able to make progress. */
-        PRODUCER
-    };
+  public:
+    // For now, only allow a single parallel asm.js compilation to happen at a
+    // time. This avoids race conditions on asmJSWorklist/asmJSFinishedList/etc.
+    mozilla::Atomic<bool> asmJSCompilationInProgress;
 
-    /* Shared worklist for Ion worker threads. */
-    Vector<jit::IonBuilder*, 0, SystemAllocPolicy> ionWorklist;
+  private:
+    // Script parsing/emitting worklist and finished jobs.
+    ParseTaskVector parseWorklist_, parseFinishedList_;
 
-    /* Worklist for AsmJS worker threads. */
-    Vector<AsmJSParallelTask*, 0, SystemAllocPolicy> asmJSWorklist;
+    // Parse tasks waiting for an atoms-zone GC to complete.
+    ParseTaskVector parseWaitingOnGC_;
 
-    /*
-     * Finished list for AsmJS worker threads.
-     * Simultaneous AsmJS compilations all service the same AsmJS module.
-     * The main thread must pick up finished optimizations and perform codegen.
-     */
-    Vector<AsmJSParallelTask*, 0, SystemAllocPolicy> asmJSFinishedList;
+    // Source compression worklist.
+    SourceCompressionTaskVector compressionWorklist_;
 
-    /*
-     * For now, only allow a single parallel asm.js compilation to happen at a
-     * time. This avoids race conditions on asmJSWorklist/asmJSFinishedList/etc.
-     */
-    mozilla::Atomic<uint32_t> asmJSCompilationInProgress;
+  public:
+    GlobalWorkerThreadState();
 
-    /* Shared worklist for parsing/emitting scripts on worker threads. */
-    Vector<ParseTask*, 0, SystemAllocPolicy> parseWorklist, parseFinishedList;
-
-    /* Worklist for source compression worker threads. */
-    Vector<SourceCompressionTask *, 0, SystemAllocPolicy> compressionWorklist;
-
-    WorkerThreadState() { mozilla::PodZero(this); }
-    ~WorkerThreadState();
-
-    bool init(JSRuntime *rt);
-    void cleanup(JSRuntime *rt);
+    bool ensureInitialized();
+    void finish();
 
     void lock();
     void unlock();
@@ -96,8 +94,61 @@ class WorkerThreadState
     bool isLocked();
 # endif
 
+    enum CondVar {
+        // For notifying threads waiting for work that they may be able to make progress.
+        CONSUMER,
+
+        // For notifying threads doing work that they may be able to make progress.
+        PRODUCER
+    };
+
     void wait(CondVar which, uint32_t timeoutMillis = 0);
     void notifyAll(CondVar which);
+    void notifyOne(CondVar which);
+
+    // Helper method for removing items from the vectors below while iterating over them.
+    template <typename T>
+    void remove(T &vector, size_t *index)
+    {
+        vector[(*index)--] = vector.back();
+        vector.popBack();
+    }
+
+    IonBuilderVector &ionWorklist() {
+        JS_ASSERT(isLocked());
+        return ionWorklist_;
+    }
+    IonBuilderVector &ionFinishedList() {
+        JS_ASSERT(isLocked());
+        return ionFinishedList_;
+    }
+
+    AsmJSParallelTaskVector &asmJSWorklist() {
+        JS_ASSERT(isLocked());
+        return asmJSWorklist_;
+    }
+    AsmJSParallelTaskVector &asmJSFinishedList() {
+        JS_ASSERT(isLocked());
+        return asmJSFinishedList_;
+    }
+
+    ParseTaskVector &parseWorklist() {
+        JS_ASSERT(isLocked());
+        return parseWorklist_;
+    }
+    ParseTaskVector &parseFinishedList() {
+        JS_ASSERT(isLocked());
+        return parseFinishedList_;
+    }
+    ParseTaskVector &parseWaitingOnGC() {
+        JS_ASSERT(isLocked());
+        return parseWaitingOnGC_;
+    }
+
+    SourceCompressionTaskVector &compressionWorklist() {
+        JS_ASSERT(isLocked());
+        return compressionWorklist_;
+    }
 
     bool canStartAsmJSCompile();
     bool canStartIonCompile();
@@ -161,11 +212,16 @@ class WorkerThreadState
     void *asmJSFailedFunction;
 };
 
+static inline GlobalWorkerThreadState &
+WorkerThreadState()
+{
+    extern GlobalWorkerThreadState gWorkerThreadState;
+    return gWorkerThreadState;
+}
+
 /* Individual helper thread, one allocated per core. */
 struct WorkerThread
 {
-    JSRuntime *runtime;
-
     mozilla::Maybe<PerThreadData> threadData;
     PRThread *thread;
 
@@ -188,40 +244,32 @@ struct WorkerThread
         return !ionBuilder && !asmData && !parseTask && !compressionTask;
     }
 
-    inline void maybePause();
-
-    void pause();
     void destroy();
 
-    void handleAsmJSWorkload(WorkerThreadState &state);
-    void handleIonWorkload(WorkerThreadState &state);
-    void handleParseWorkload(WorkerThreadState &state);
-    void handleCompressionWorkload(WorkerThreadState &state);
+    void handleAsmJSWorkload();
+    void handleIonWorkload();
+    void handleParseWorkload();
+    void handleCompressionWorkload();
 
     static void ThreadMain(void *arg);
     void threadLoop();
 };
 
-#endif /* JS_WORKER_THREADS */
-
-inline bool
-OffThreadIonCompilationEnabled(JSRuntime *rt)
-{
-#ifdef JS_WORKER_THREADS
-    return rt->useHelperThreads()
-        && rt->helperThreadCount() != 0
-        && rt->useHelperThreadsForIonCompilation();
-#else
-    return false;
-#endif
-}
+#endif /* JS_THREADSAFE */
 
 /* Methods for interacting with worker threads. */
 
-/* Initialize worker threads unless already initialized. */
+// Initialize worker threads unless already initialized.
 bool
 EnsureWorkerThreadsInitialized(ExclusiveContext *cx);
 
+// This allows the JS shell to override GetCPUCount() when passed the
+// --thread-count=N option.
+void
+SetFakeCPUCount(size_t count);
+
+#ifdef JS_ION
+
 /* Perform MIR optimization and LIR generation on a single function. */
 bool
 StartOffThreadAsmJSCompile(ExclusiveContext *cx, AsmJSParallelTask *asmData);
@@ -233,6 +281,8 @@ StartOffThreadAsmJSCompile(ExclusiveContext *cx, AsmJSParallelTask *asmData);
 bool
 StartOffThreadIonCompile(JSContext *cx, jit::IonBuilder *builder);
 
+#endif // JS_ION
+
 /*
  * Cancel a scheduled or in progress Ion compilation for script. If script is
  * nullptr, all compilations for the compartment are cancelled.
@@ -240,18 +290,25 @@ StartOffThreadIonCompile(JSContext *cx, jit::IonBuilder *builder);
 void
 CancelOffThreadIonCompile(JSCompartment *compartment, JSScript *script);
 
+/* Cancel all scheduled, in progress or finished parses for runtime. */
+void
+CancelOffThreadParses(JSRuntime *runtime);
+
 /*
  * Start a parse/emit cycle for a stream of source. The characters must stay
  * alive until the compilation finishes.
  */
 bool
-StartOffThreadParseScript(JSContext *cx, const CompileOptions &options,
+StartOffThreadParseScript(JSContext *cx, const ReadOnlyCompileOptions &options,
                           const jschar *chars, size_t length, HandleObject scopeChain,
                           JS::OffThreadCompileCallback callback, void *callbackData);
 
-/* Block until in progress and pending off thread parse jobs have finished. */
+/*
+ * Called at the end of GC to enqueue any Parse tasks that were waiting on an
+ * atoms-zone GC to finish.
+ */
 void
-WaitForOffThreadParsingToFinish(JSRuntime *rt);
+EnqueuePendingParseTasksAfterGC(JSRuntime *rt);
 
 /* Start a compression job for the specified token. */
 bool
@@ -261,25 +318,20 @@ class AutoLockWorkerThreadState
 {
     MOZ_DECL_USE_GUARD_OBJECT_NOTIFIER
 
-#ifdef JS_WORKER_THREADS
-    WorkerThreadState &state;
-
+#ifdef JS_THREADSAFE
   public:
-    AutoLockWorkerThreadState(WorkerThreadState &state
-                              MOZ_GUARD_OBJECT_NOTIFIER_PARAM)
-      : state(state)
+    AutoLockWorkerThreadState(MOZ_GUARD_OBJECT_NOTIFIER_ONLY_PARAM)
     {
         MOZ_GUARD_OBJECT_NOTIFIER_INIT;
-        state.lock();
+        WorkerThreadState().lock();
     }
 
     ~AutoLockWorkerThreadState() {
-        state.unlock();
+        WorkerThreadState().unlock();
     }
 #else
   public:
-    AutoLockWorkerThreadState(WorkerThreadState &state
-                              MOZ_GUARD_OBJECT_NOTIFIER_PARAM)
+    AutoLockWorkerThreadState(MOZ_GUARD_OBJECT_NOTIFIER_ONLY_PARAM)
     {
         MOZ_GUARD_OBJECT_NOTIFIER_INIT;
     }
@@ -288,88 +340,30 @@ class AutoLockWorkerThreadState
 
 class AutoUnlockWorkerThreadState
 {
-    JSRuntime *rt;
     MOZ_DECL_USE_GUARD_OBJECT_NOTIFIER
 
   public:
 
-    AutoUnlockWorkerThreadState(JSRuntime *rt
-                                MOZ_GUARD_OBJECT_NOTIFIER_PARAM)
-      : rt(rt)
+    AutoUnlockWorkerThreadState(MOZ_GUARD_OBJECT_NOTIFIER_ONLY_PARAM)
     {
         MOZ_GUARD_OBJECT_NOTIFIER_INIT;
-#ifdef JS_WORKER_THREADS
-        JS_ASSERT(rt->workerThreadState);
-        rt->workerThreadState->unlock();
-#else
-        (void)this->rt;
+#ifdef JS_THREADSAFE
+        WorkerThreadState().unlock();
 #endif
     }
 
     ~AutoUnlockWorkerThreadState()
     {
-#ifdef JS_WORKER_THREADS
-        rt->workerThreadState->lock();
+#ifdef JS_THREADSAFE
+        WorkerThreadState().lock();
 #endif
     }
 };
 
-#ifdef JS_WORKER_THREADS
-
-inline void
-WorkerThread::maybePause()
-{
-    if (runtime->workerThreadState->shouldPause) {
-        AutoLockWorkerThreadState lock(*runtime->workerThreadState);
-        pause();
-    }
-}
-
-#endif // JS_WORKER_THREADS
-
-/* Pause any threads that are running jobs off thread during GC activity. */
-class AutoPauseWorkersForTracing
-{
-#ifdef JS_WORKER_THREADS
-    JSRuntime *runtime;
-    bool needsUnpause;
-    mozilla::DebugOnly<bool> oldExclusiveThreadsPaused;
-#endif
-    MOZ_DECL_USE_GUARD_OBJECT_NOTIFIER
-
-  public:
-    AutoPauseWorkersForTracing(JSRuntime *rt MOZ_GUARD_OBJECT_NOTIFIER_PARAM);
-    ~AutoPauseWorkersForTracing();
-};
-
-/*
- * If the current thread is a worker thread, treat it as paused during this
- * class's lifetime. This should be used at any time the current thread is
- * waiting for a worker to complete.
- */
-class AutoPauseCurrentWorkerThread
-{
-#ifdef JS_WORKER_THREADS
-    ExclusiveContext *cx;
-#endif
-    MOZ_DECL_USE_GUARD_OBJECT_NOTIFIER
-
-  public:
-    AutoPauseCurrentWorkerThread(ExclusiveContext *cx MOZ_GUARD_OBJECT_NOTIFIER_PARAM);
-    ~AutoPauseCurrentWorkerThread();
-};
-
-/* Wait for any in progress off thread parses to halt. */
-void
-PauseOffThreadParsing();
-
-/* Resume any paused off thread parses. */
-void
-ResumeOffThreadParsing();
-
 #ifdef JS_ION
 struct AsmJSParallelTask
 {
+    JSRuntime *runtime;     // Associated runtime.
     LifoAlloc lifo;         // Provider of all heap memory used for compilation.
     void *func;             // Really, a ModuleCompiler::Func*
     jit::MIRGenerator *mir; // Passed from main thread to worker.
@@ -377,10 +371,11 @@ struct AsmJSParallelTask
     unsigned compileTime;
 
     AsmJSParallelTask(size_t defaultChunkSize)
-      : lifo(defaultChunkSize), func(nullptr), mir(nullptr), lir(nullptr), compileTime(0)
+      : runtime(nullptr), lifo(defaultChunkSize), func(nullptr), mir(nullptr), lir(nullptr), compileTime(0)
     { }
 
-    void init(void *func, jit::MIRGenerator *mir) {
+    void init(JSRuntime *rt, void *func, jit::MIRGenerator *mir) {
+        this->runtime = rt;
         this->func = func;
         this->mir = mir;
         this->lir = nullptr;
@@ -391,7 +386,7 @@ struct AsmJSParallelTask
 struct ParseTask
 {
     ExclusiveContext *cx;
-    CompileOptions options;
+    OwningCompileOptions options;
     const jschar *chars;
     size_t length;
     LifoAlloc alloc;
@@ -399,7 +394,18 @@ struct ParseTask
     // Rooted pointer to the scope in the target compartment which the
     // resulting script will be merged into. This is not safe to use off the
     // main thread.
-    JSObject *scopeChain;
+    PersistentRootedObject scopeChain;
+
+    // Rooted pointer to the global object used by 'cx'.
+    PersistentRootedObject exclusiveContextGlobal;
+
+    // Saved GC-managed CompileOptions fields that will populate slots in
+    // the ScriptSourceObject. We create the ScriptSourceObject in the
+    // compilation's temporary compartment, so storing these values there
+    // at that point would create cross-compartment references. Instead we
+    // hold them here, and install them after merging the compartments.
+    PersistentRootedObject optionsElement;
+    PersistentRootedScript optionsIntroductionScript;
 
     // Callback invoked off the main thread when the parse finishes.
     JS::OffThreadCompileCallback callback;
@@ -415,13 +421,28 @@ struct ParseTask
     Vector<frontend::CompileError *> errors;
     bool overRecursed;
 
-    ParseTask(ExclusiveContext *cx, const CompileOptions &options,
+    ParseTask(ExclusiveContext *cx, JSObject *exclusiveContextGlobal, JSContext *initCx,
               const jschar *chars, size_t length, JSObject *scopeChain,
               JS::OffThreadCompileCallback callback, void *callbackData);
+    bool init(JSContext *cx, const ReadOnlyCompileOptions &options);
+
+    void activate(JSRuntime *rt);
+    void finish();
+
+    bool runtimeMatches(JSRuntime *rt) {
+        return exclusiveContextGlobal->runtimeFromAnyThread() == rt;
+    }
 
     ~ParseTask();
 };
 
+#ifdef JS_THREADSAFE
+// Return whether, if a new parse task was started, it would need to wait for
+// an in-progress GC to complete before starting.
+extern bool
+OffThreadParsingMustWaitForGC(JSRuntime *rt);
+#endif
+
 // Compression tasks are allocated on the stack by their triggering thread,
 // which will block on the compression completing as the task goes out of scope
 // to ensure it completes at the required time.
@@ -429,7 +450,7 @@ struct SourceCompressionTask
 {
     friend class ScriptSource;
 
-#ifdef JS_WORKER_THREADS
+#ifdef JS_THREADSAFE
     // Thread performing the compression.
     WorkerThread *workerThread;
 #endif
@@ -444,17 +465,13 @@ struct SourceCompressionTask
 
     // Atomic flag to indicate to a worker thread that it should abort
     // compression on the source.
-#ifdef JS_THREADSAFE
-    mozilla::Atomic<int32_t, mozilla::Relaxed> abort_;
-#else
-    int32_t abort_;
-#endif
+    mozilla::Atomic<bool, mozilla::Relaxed> abort_;
 
   public:
     explicit SourceCompressionTask(ExclusiveContext *cx)
-      : cx(cx), ss(nullptr), chars(nullptr), oom(false), abort_(0)
+      : cx(cx), ss(nullptr), chars(nullptr), oom(false), abort_(false)
     {
-#ifdef JS_WORKER_THREADS
+#ifdef JS_THREADSAFE
         workerThread = nullptr;
 #endif
     }
@@ -464,15 +481,9 @@ struct SourceCompressionTask
         complete();
     }
 
-    void maybePause() { 
-#ifdef JS_WORKER_THREADS
-        workerThread->maybePause();
-#endif
-    }
-
-    bool compress();
+    bool work();
     bool complete();
-    void abort() { abort_ = 1; }
+    void abort() { abort_ = true; }
     bool active() const { return !!ss; }
     ScriptSource *source() { return ss; }
     const jschar *uncompressedChars() { return chars; }
