@@ -4,11 +4,17 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
+#ifndef mozJSComponentLoader_h
+#define mozJSComponentLoader_h
+
+#include "mozilla/MemoryReporting.h"
 #include "mozilla/ModuleLoader.h"
 #include "nsISupports.h"
 #include "nsIObserver.h"
+#include "nsIURI.h"
 #include "xpcIJSModuleLoader.h"
 #include "nsClassHashtable.h"
+#include "nsCxPusher.h"
 #include "nsDataHashtable.h"
 #include "jsapi.h"
 
@@ -51,6 +57,8 @@ class mozJSComponentLoader : public mozilla::ModuleLoader,
 
     void NoteSubScript(JS::HandleScript aScript, JS::HandleObject aThisObject);
 
+    size_t SizeOfIncludingThis(mozilla::MallocSizeOf aMallocSizeOf);
+
  protected:
     static mozJSComponentLoader* sSelf;
 
@@ -66,6 +74,7 @@ class mozJSComponentLoader : public mozilla::ModuleLoader,
     nsresult ObjectForLocation(nsIFile* aComponentFile,
                                nsIURI *aComponent,
                                JSObject **aObject,
+                               JSScript **aTableScript,
                                char **location,
                                bool aCatchException,
                                JS::MutableHandleValue aException);
@@ -95,6 +104,7 @@ class mozJSComponentLoader : public mozilla::ModuleLoader,
             unloadProc = nullptr;
 
             obj = nullptr;
+            thisObjectKey = nullptr;
             location = nullptr;
         }
 
@@ -106,31 +116,42 @@ class mozJSComponentLoader : public mozilla::ModuleLoader,
             getfactoryobj = nullptr;
 
             if (obj) {
-                JSAutoRequest ar(sSelf->mContext);
+                mozilla::AutoJSContext cx;
+                JSAutoCompartment ac(cx, obj);
 
-                JSAutoCompartment ac(sSelf->mContext, obj);
-
-                JS_SetAllNonReservedSlotsToUndefined(sSelf->mContext, obj);
-                JS_RemoveObjectRoot(sSelf->mContext, &obj);
+                JS_SetAllNonReservedSlotsToUndefined(cx, obj);
+                JS_RemoveObjectRoot(cx, &obj);
+                if (thisObjectKey)
+                    JS_RemoveScriptRoot(cx, &thisObjectKey);
             }
 
             if (location)
                 NS_Free(location);
 
             obj = nullptr;
+            thisObjectKey = nullptr;
             location = nullptr;
         }
 
+        size_t SizeOfIncludingThis(mozilla::MallocSizeOf aMallocSizeOf) const;
+
         static already_AddRefed<nsIFactory> GetFactory(const mozilla::Module& module,
                                                        const mozilla::Module::CIDEntry& entry);
 
         nsCOMPtr<xpcIJSGetFactory> getfactoryobj;
         JSObject            *obj;
+        JSScript            *thisObjectKey;
         char                *location;
     };
 
     friend class ModuleEntry;
 
+    static size_t DataEntrySizeOfExcludingThis(const nsACString& aKey, ModuleEntry* const& aData,
+                                               mozilla::MallocSizeOf aMallocSizeOf, void* arg);
+    static size_t ClassEntrySizeOfExcludingThis(const nsACString& aKey,
+                                                const nsAutoPtr<ModuleEntry>& aData,
+                                                mozilla::MallocSizeOf aMallocSizeOf, void* arg);
+
     // Modules are intentionally leaked, but still cleared.
     static PLDHashOperator ClearModules(const nsACString& key, ModuleEntry*& entry, void* cx);
     nsDataHashtable<nsCStringHashKey, ModuleEntry*> mModules;
@@ -142,3 +163,5 @@ class mozJSComponentLoader : public mozilla::ModuleLoader,
     bool mInitialized;
     bool mReuseLoaderGlobal;
 };
+
+#endif
