@@ -7,7 +7,7 @@
 
 /* Data conversion between native and JavaScript types. */
 
-#include "mozilla/Util.h"
+#include "mozilla/ArrayUtils.h"
 
 #include "xpcprivate.h"
 #include "nsIAtom.h"
@@ -194,18 +194,15 @@ XPCConvert::NativeData2JS(MutableHandleValue d, const void* s,
 
                 if (!p->IsVoid()) {
                     nsStringBuffer* buf;
-                    jsval str = XPCStringConvert::ReadableToJSVal(cx, *p, &buf);
-                    if (JSVAL_IS_NULL(str))
+                    if (!XPCStringConvert::ReadableToJSVal(cx, *p, &buf, d))
                         return false;
                     if (buf)
                         buf->AddRef();
-
-                    d.set(str);
                 }
 
                 // *d is defaulted to JSVAL_NULL so no need to set it
                 // again if p is a "void" string
-
+                MOZ_ASSERT_IF(p->IsVoid(), d.isNull());
                 break;
             }
 
@@ -260,9 +257,9 @@ XPCConvert::NativeData2JS(MutableHandleValue d, const void* s,
                 if (!len)
                     return false;
 
-                const size_t buffer_size = (len + 1) * sizeof(PRUnichar);
-                PRUnichar* buffer =
-                    static_cast<PRUnichar*>(JS_malloc(cx, buffer_size));
+                const size_t buffer_size = (len + 1) * sizeof(char16_t);
+                char16_t* buffer =
+                    static_cast<char16_t*>(JS_malloc(cx, buffer_size));
                 if (!buffer)
                     return false;
 
@@ -294,11 +291,6 @@ XPCConvert::NativeData2JS(MutableHandleValue d, const void* s,
                 if (!cString || cString->IsVoid())
                     break;
 
-                if (cString->IsEmpty()) {
-                    d.set(JS_GetEmptyStringValue(cx));
-                    break;
-                }
-
                 // c-strings (binary blobs) are deliberately not converted from
                 // UTF-8 to UTF-16. T_UTF8Sting is for UTF-8 encoded strings
                 // with automatic conversion.
@@ -382,8 +374,6 @@ XPCConvert::JSData2Native(void* d, HandleValue s,
 {
     NS_PRECONDITION(d, "bad param");
 
-    bool isDOMString = true;
-
     AutoJSContext cx;
     if (pErr)
         *pErr = NS_ERROR_XPC_BAD_CONVERT_JS;
@@ -413,7 +403,7 @@ XPCConvert::JSData2Native(void* d, HandleValue s,
         return ConvertToPrimitive(cx, s, static_cast<bool*>(d));
     case nsXPTType::T_CHAR   :
     {
-        JSString* str = JS_ValueToString(cx, s);
+        JSString* str = ToString(cx, s);
         if (!str) {
             return false;
         }
@@ -432,7 +422,7 @@ XPCConvert::JSData2Native(void* d, HandleValue s,
     case nsXPTType::T_WCHAR  :
     {
         JSString* str;
-        if (!(str = JS_ValueToString(cx, s))) {
+        if (!(str = ToString(cx, s))) {
             return false;
         }
         size_t length;
@@ -476,81 +466,70 @@ XPCConvert::JSData2Native(void* d, HandleValue s,
 
     case nsXPTType::T_ASTRING:
     {
-        isDOMString = false;
+        if (JSVAL_IS_VOID(s)) {
+            if (useAllocator)
+                *((const nsAString**)d) = &NullString();
+            else
+                (**((nsAString**)d)).SetIsVoid(true);
+            return true;
+        }
         // Fall through to T_DOMSTRING case.
     }
     case nsXPTType::T_DOMSTRING:
     {
-        static const PRUnichar EMPTY_STRING[] = { '\0' };
-        static const PRUnichar VOID_STRING[] = { 'u', 'n', 'd', 'e', 'f', 'i', 'n', 'e', 'd', '\0' };
-
-        const PRUnichar* chars = nullptr;
+        if (JSVAL_IS_NULL(s)) {
+            if (useAllocator)
+                *((const nsAString**)d) = &NullString();
+            else
+                (**((nsAString**)d)).SetIsVoid(true);
+            return true;
+        }
+        size_t length = 0;
+        const char16_t* chars = nullptr;
         JSString* str = nullptr;
-        bool isNewString = false;
-        uint32_t length = 0;
-
-        if (JSVAL_IS_VOID(s)) {
-            if (isDOMString) {
-                chars  = VOID_STRING;
-                length = ArrayLength(VOID_STRING) - 1;
-            } else {
-                chars = EMPTY_STRING;
-                length = 0;
-            }
-        } else if (!JSVAL_IS_NULL(s)) {
-            str = JS_ValueToString(cx, s);
+        if (!JSVAL_IS_VOID(s)) {
+            str = ToString(cx, s);
             if (!str)
                 return false;
 
-            length = (uint32_t) JS_GetStringLength(str);
-            if (length) {
-                chars = JS_GetStringCharsZ(cx, str);
-                if (!chars)
-                    return false;
-                if (STRING_TO_JSVAL(str) != s)
-                    isNewString = true;
-            } else {
-                str = nullptr;
-                chars = EMPTY_STRING;
+            chars = useAllocator ? JS_GetStringCharsZAndLength(cx, str, &length)
+                                 : JS_GetStringCharsAndLength(cx, str, &length);
+            if (!chars)
+                return false;
+
+            if (!length) {
+                if (useAllocator)
+                    *((const nsAString**)d) = &EmptyString();
+                else
+                    (**((nsAString**)d)).Truncate();
+                return true;
             }
         }
 
+        nsString* ws;
         if (useAllocator) {
-            // XXX extra string copy when isNewString
-            if (str && !isNewString) {
-                size_t strLength;
-                const jschar *strChars = JS_GetStringCharsZAndLength(cx, str, &strLength);
-                if (!strChars)
-                    return false;
-
-                XPCReadableJSStringWrapper *wrapper =
-                    nsXPConnect::GetRuntimeInstance()->NewStringWrapper(strChars, strLength);
-                if (!wrapper)
-                    return false;
-
-                *((const nsAString**)d) = wrapper;
-            } else if (JSVAL_IS_NULL(s)) {
-                XPCReadableJSStringWrapper *wrapper =
-                    new XPCReadableJSStringWrapper();
-                if (!wrapper)
-                    return false;
-
-                *((const nsAString**)d) = wrapper;
-            } else {
-                // use nsString to encourage sharing
-                const nsAString *rs = new nsString(chars, length);
-                if (!rs)
-                    return false;
-                *((const nsAString**)d) = rs;
-            }
+            ws = nsXPConnect::GetRuntimeInstance()->NewShortLivedString();
+            *((const nsString**)d) = ws;
         } else {
-            nsAString* ws = *((nsAString**)d);
+            ws = *((nsString**)d);
+        }
 
-            if (JSVAL_IS_NULL(s) || (!isDOMString && JSVAL_IS_VOID(s))) {
-                ws->Truncate();
-                ws->SetIsVoid(true);
-            } else
-                ws->Assign(chars, length);
+        if (!str) {
+            ws->AssignLiteral(MOZ_UTF16("undefined"));
+        } else if (XPCStringConvert::IsDOMString(str)) {
+            // The characters represent an existing nsStringBuffer that
+            // was shared by XPCStringConvert::ReadableToJSVal.
+            nsStringBuffer::FromData((void *)chars)->ToString(length, *ws);
+        } else if (XPCStringConvert::IsLiteral(str)) {
+            // The characters represent a literal char16_t string constant
+            // compiled into libxul, such as the string "undefined" above.
+            ws->AssignLiteral(chars, length);
+        } else if (useAllocator && STRING_TO_JSVAL(str) == s) {
+            // The JS string will exist over the function call.
+            // We don't need to copy the characters in this case.
+            ws->Rebind(chars, length);
+        } else {
+            ws->Assign(chars, length);
         }
         return true;
     }
@@ -562,7 +541,7 @@ XPCConvert::JSData2Native(void* d, HandleValue s,
             return true;
         }
 
-        JSString* str = JS_ValueToString(cx, s);
+        JSString* str = ToString(cx, s);
         if (!str) {
             return false;
         }
@@ -603,7 +582,7 @@ XPCConvert::JSData2Native(void* d, HandleValue s,
             return true;
         }
 
-        if (!(str = JS_ValueToString(cx, s))) {
+        if (!(str = ToString(cx, s))) {
             return false;
         }
         if (!(chars = JS_GetStringCharsZ(cx, str))) {
@@ -625,20 +604,14 @@ XPCConvert::JSData2Native(void* d, HandleValue s,
     case nsXPTType::T_UTF8STRING:
     {
         const jschar* chars;
-        uint32_t length;
+        size_t length;
         JSString* str;
 
         if (JSVAL_IS_NULL(s) || JSVAL_IS_VOID(s)) {
             if (useAllocator) {
-                nsACString *rs = new nsCString();
-                if (!rs)
-                    return false;
-
-                rs->SetIsVoid(true);
-                *((nsACString**)d) = rs;
+                *((const nsACString**)d) = &NullCString();
             } else {
                 nsCString* rs = *((nsCString**)d);
-                rs->Truncate();
                 rs->SetIsVoid(true);
             }
             return true;
@@ -646,12 +619,20 @@ XPCConvert::JSData2Native(void* d, HandleValue s,
 
         // The JS val is neither null nor void...
 
-        if (!(str = JS_ValueToString(cx, s))||
-            !(chars = JS_GetStringCharsZ(cx, str))) {
+        if (!(str = ToString(cx, s))||
+            !(chars = JS_GetStringCharsAndLength(cx, str, &length))) {
             return false;
         }
 
-        length = JS_GetStringLength(str);
+        if (!length) {
+            if (useAllocator) {
+                *((const nsACString**)d) = &EmptyCString();
+            } else {
+                nsCString* rs = *((nsCString**)d);
+                rs->Truncate();
+            }
+            return true;
+        }
 
         nsCString *rs;
         if (useAllocator) {
@@ -664,9 +645,7 @@ XPCConvert::JSData2Native(void* d, HandleValue s,
         } else {
             rs = *((nsCString**)d);
         }
-        const PRUnichar* start = (const PRUnichar*)chars;
-        const PRUnichar* end = start + length;
-        CopyUTF16toUTF8(nsDependentSubstring(start, end), *rs);
+        CopyUTF16toUTF8(Substring(chars, length), *rs);
         return true;
     }
 
@@ -689,7 +668,7 @@ XPCConvert::JSData2Native(void* d, HandleValue s,
         }
 
         // The JS val is neither null nor void...
-        JSString* str = JS_ValueToString(cx, s);
+        JSString* str = ToString(cx, s);
         if (!str) {
             return false;
         }
@@ -699,6 +678,16 @@ XPCConvert::JSData2Native(void* d, HandleValue s,
             return false;
         }
 
+        if (!length) {
+            if (useAllocator) {
+                *((const nsACString**)d) = &EmptyCString();
+            } else {
+                nsCString* rs = *((nsCString**)d);
+                rs->Truncate();
+            }
+            return true;
+        }
+
         nsACString *rs;
         if (useAllocator) {
             rs = new nsCString();
@@ -724,16 +713,17 @@ XPCConvert::JSData2Native(void* d, HandleValue s,
         MOZ_ASSERT(iid,"can't do interface conversions without iid");
 
         if (iid->Equals(NS_GET_IID(nsIVariant))) {
-            XPCVariant* variant = XPCVariant::newVariant(cx, s);
+            nsCOMPtr<nsIVariant> variant = XPCVariant::newVariant(cx, s);
             if (!variant)
                 return false;
-            *((nsISupports**)d) = static_cast<nsIVariant*>(variant);
+
+            variant.forget(static_cast<nsISupports**>(d));
             return true;
         } else if (iid->Equals(NS_GET_IID(nsIAtom)) &&
                    JSVAL_IS_STRING(s)) {
             // We're trying to pass a string as an nsIAtom.  Let's atomize!
             JSString* str = JSVAL_TO_STRING(s);
-            const PRUnichar* chars = JS_GetStringCharsZ(cx, str);
+            const char16_t* chars = JS_GetStringCharsZ(cx, str);
             if (!chars) {
                 if (pErr)
                     *pErr = NS_ERROR_XPC_BAD_CONVERT_JS_NULL_REF;
@@ -774,11 +764,11 @@ CreateHolderIfNeeded(HandleObject obj, MutableHandleValue d,
                      nsIXPConnectJSObjectHolder** dest)
 {
     if (dest) {
-        XPCJSObjectHolder* objHolder = XPCJSObjectHolder::newHolder(obj);
+        nsRefPtr<XPCJSObjectHolder> objHolder = XPCJSObjectHolder::newHolder(obj);
         if (!objHolder)
             return false;
 
-        NS_ADDREF(*dest = objHolder);
+        objHolder.forget(dest);
     }
 
     d.setObjectOrNull(obj);
@@ -921,7 +911,7 @@ XPCConvert::NativeInterface2JSObject(MutableHandleValue d,
     if (!allowNativeWrapper) {
         d.set(v);
         if (dest)
-            *dest = strongWrapper.forget().get();
+            strongWrapper.forget(dest);
         if (pErr)
             *pErr = NS_OK;
         return true;
@@ -938,14 +928,14 @@ XPCConvert::NativeInterface2JSObject(MutableHandleValue d,
     if (dest) {
         // The strongWrapper still holds the original flat object.
         if (flat == original) {
-            *dest = strongWrapper.forget().get();
+            strongWrapper.forget(dest);
         } else {
             nsRefPtr<XPCJSObjectHolder> objHolder =
                 XPCJSObjectHolder::newHolder(flat);
             if (!objHolder)
                 return false;
 
-            *dest = objHolder.forget().get();
+            objHolder.forget(dest);
         }
     }
 
@@ -998,6 +988,9 @@ XPCConvert::JSObject2NativeInterface(void** dest, HandleObject src,
         // involves fixing the contacts API and PeerConnection to stop using
         // COWs. This needs to happen, but for now just preserve the old
         // behavior.
+        //
+        // Note that there is an identical hack in getWrapper which should be
+        // removed if this one is.
         if (!inner && MOZ_UNLIKELY(xpc::WrapperFactory::IsCOW(src)))
             inner = js::UncheckedUnwrap(src);
         if (!inner) {
@@ -1070,7 +1063,6 @@ XPCConvert::ConstructException(nsresult rv, const char* message,
 
     static const char format[] = "\'%s\' when calling method: [%s::%s]";
     const char * msg = message;
-    char* sz = nullptr;
     nsXPIDLString xmsg;
     nsAutoCString sxmsg;
 
@@ -1084,19 +1076,18 @@ XPCConvert::ConstructException(nsresult rv, const char* message,
     if (!msg)
         if (!nsXPCException::NameAndFormatForNSResult(rv, nullptr, &msg) || ! msg)
             msg = "<error>";
-    if (ifaceName && methodName)
-        msg = sz = JS_smprintf(format, msg, ifaceName, methodName);
 
-    nsRefPtr<Exception> e = new Exception(msg, rv, nullptr, nullptr, data);
+    nsCString msgStr(msg);
+    if (ifaceName && methodName)
+        msgStr.AppendPrintf(format, msg, ifaceName, methodName);
+
+    nsRefPtr<Exception> e = new Exception(msgStr, rv, EmptyCString(), nullptr, data);
 
     if (cx && jsExceptionPtr) {
         e->StowJSVal(*jsExceptionPtr);
     }
 
     e.forget(exceptn);
-
-    if (sz)
-        JS_smprintf_free(sz);
     return NS_OK;
 }
 
@@ -1151,9 +1142,8 @@ XPCConvert::JSValToXPCException(MutableHandleValue s,
             nsCOMPtr<nsIException> iface = do_QueryInterface(supports);
             if (iface) {
                 // just pass through the exception (with extra ref and all)
-                nsIException* temp = iface;
-                NS_ADDREF(temp);
-                *exceptn = temp;
+                nsCOMPtr<nsIException> temp = iface;
+                temp.forget(exceptn);
                 return NS_OK;
             } else {
                 // it is a wrapped native, but not an exception!
@@ -1167,10 +1157,10 @@ XPCConvert::JSValToXPCException(MutableHandleValue s,
             // If it is an engine Error with an error report then let's
             // extract the report and build an xpcexception from that
             const JSErrorReport* report;
-            if (nullptr != (report = JS_ErrorFromException(cx, s))) {
+            if (nullptr != (report = JS_ErrorFromException(cx, obj))) {
                 JSAutoByteString message;
                 JSString* str;
-                if (nullptr != (str = JS_ValueToString(cx, s)))
+                if (nullptr != (str = ToString(cx, s)))
                     message.encodeLatin1(cx, str);
                 return JSErrorToXPCException(message.ptr(), ifaceName,
                                              methodName, report, exceptn);
@@ -1206,7 +1196,7 @@ XPCConvert::JSValToXPCException(MutableHandleValue s,
 
             // otherwise we'll just try to convert it to a string
 
-            JSString* str = JS_ValueToString(cx, s);
+            JSString* str = ToString(cx, s);
             if (!str)
                 return NS_ERROR_FAILURE;
 
@@ -1277,7 +1267,7 @@ XPCConvert::JSValToXPCException(MutableHandleValue s,
     // otherwise we'll just try to convert it to a string
     // Note: e.g., bools get converted to JSStrings by this code.
 
-    JSString* str = JS_ValueToString(cx, s);
+    JSString* str = ToString(cx, s);
     if (str) {
         JSAutoByteString strBytes(cx, str);
         if (!!strBytes) {
@@ -1305,15 +1295,15 @@ XPCConvert::JSErrorToXPCException(const char* message,
     if (report) {
         nsAutoString bestMessage;
         if (report && report->ucmessage) {
-            bestMessage = static_cast<const PRUnichar*>(report->ucmessage);
+            bestMessage = static_cast<const char16_t*>(report->ucmessage);
         } else if (message) {
             CopyASCIItoUTF16(message, bestMessage);
         } else {
             bestMessage.AssignLiteral("JavaScript Error");
         }
 
-        const PRUnichar* uclinebuf =
-            static_cast<const PRUnichar*>(report->uclinebuf);
+        const char16_t* uclinebuf =
+            static_cast<const char16_t*>(report->uclinebuf);
 
         data = new nsScriptError();
         data->InitWithWindowID(
@@ -1364,7 +1354,7 @@ XPCConvert::NativeArray2JS(MutableHandleValue d, const void** s,
 
     // XXX add support to indicate *which* array element was not convertable
 
-    RootedObject array(cx, JS_NewArrayObject(cx, count, nullptr));
+    RootedObject array(cx, JS_NewArrayObject(cx, count));
     if (!array)
         return false;
 
@@ -1378,7 +1368,7 @@ XPCConvert::NativeArray2JS(MutableHandleValue d, const void** s,
     PR_BEGIN_MACRO                                                                      \
         for (i = 0; i < count; i++) {                                                   \
             if (!NativeData2JS(&current, ((_t*)*s)+i, type, iid, pErr) ||               \
-                !JS_SetElement(cx, array, i, &current))                                 \
+                !JS_SetElement(cx, array, i, current))                                  \
                 goto failure;                                                           \
         }                                                                               \
     PR_END_MACRO
@@ -1798,7 +1788,7 @@ XPCConvert::JSStringWithSize2Native(void* d, HandleValue s,
                 return true;
             }
 
-            JSString* str = JS_ValueToString(cx, s);
+            JSString* str = ToString(cx, s);
             if (!str) {
                 return false;
             }
@@ -1853,7 +1843,7 @@ XPCConvert::JSStringWithSize2Native(void* d, HandleValue s,
                 return true;
             }
 
-            if (!(str = JS_ValueToString(cx, s))) {
+            if (!(str = ToString(cx, s))) {
                 return false;
             }
 
