@@ -43,7 +43,7 @@ nsLineBox::nsLineBox(nsIFrame* aFrame, int32_t aCount, bool aIsBlock)
   }
 #endif
 
-  static_assert(NS_STYLE_CLEAR_LAST_VALUE <= 15,
+  static_assert(NS_STYLE_CLEAR_MAX <= 15,
                 "FlagBits needs more bits to store the full range of "
                 "break type ('clear') values");
 #if NS_STYLE_CLEAR_NONE > 0
@@ -197,7 +197,7 @@ BreakTypeToString(uint8_t aBreakType)
   case NS_STYLE_CLEAR_NONE: return "nobr";
   case NS_STYLE_CLEAR_LEFT: return "leftbr";
   case NS_STYLE_CLEAR_RIGHT: return "rightbr";
-  case NS_STYLE_CLEAR_LEFT_AND_RIGHT: return "leftbr+rightbr";
+  case NS_STYLE_CLEAR_BOTH: return "leftbr+rightbr";
   case NS_STYLE_CLEAR_LINE: return "linebr";
   default:
     break;
