@@ -9,8 +9,8 @@
  * potentially re-creating) style contexts
  */
 
+#include "mozilla/ArrayUtils.h"
 #include "mozilla/MemoryReporting.h"
-#include "mozilla/Util.h"
 
 #include "nsStyleSet.h"
 #include "nsCSSStyleSheet.h"
@@ -79,7 +79,11 @@ nsInitialStyleRule::MapRuleInfoInto(nsRuleData* aRuleData)
               index == nsCSSProps::PropertyIndexInStruct(
                           eCSSProperty_script_size_multiplier) ||
               index == nsCSSProps::PropertyIndexInStruct(
-                          eCSSProperty_script_min_size)) {
+                          eCSSProperty_script_min_size) ||
+              index == nsCSSProps::PropertyIndexInStruct(
+                          eCSSProperty_math_variant) ||
+              index == nsCSSProps::PropertyIndexInStruct(
+                          eCSSProperty_math_display)) {
             continue;
           }
         }
@@ -835,12 +839,19 @@ nsStyleSet::GetContext(nsStyleContext* aParentContext,
                       result->GetStyleIfVisited()->RuleNode() ==
                         aVisitedRuleNode,
                       "unexpected visited rule node");
+    NS_ABORT_IF_FALSE(!aVisitedRuleNode ||
+                      oldAnimRule == GetAnimationRule(aVisitedRuleNode),
+                      "animation rule mismatch between rule nodes");
     if (oldAnimRule != animRule) {
       nsRuleNode *ruleNode =
         ReplaceAnimationRule(aRuleNode, oldAnimRule, animRule);
       nsRuleNode *visitedRuleNode = aVisitedRuleNode
         ? ReplaceAnimationRule(aVisitedRuleNode, oldAnimRule, animRule)
         : nullptr;
+      NS_ABORT_IF_FALSE(!visitedRuleNode ||
+                        GetAnimationRule(ruleNode) ==
+                          GetAnimationRule(visitedRuleNode),
+                        "animation rule mismatch between rule nodes");
       result = GetContext(aParentContext, ruleNode, visitedRuleNode,
                           aPseudoTag, aPseudoType, nullptr,
                           aFlags & ~eDoAnimation);
@@ -1150,12 +1161,21 @@ nsStyleSet::WalkRuleProcessors(nsIStyleRuleProcessor::EnumFunc aFunc,
   (*aFunc)(mRuleProcessors[eTransitionSheet], aData);
 }
 
+static void
+InitStyleScopes(TreeMatchContext& aTreeContext, Element* aElement)
+{
+  if (aElement->IsElementInStyleScope()) {
+    aTreeContext.InitStyleScopes(aElement->GetParentElement());
+  }
+}
+
 already_AddRefed<nsStyleContext>
 nsStyleSet::ResolveStyleFor(Element* aElement,
                             nsStyleContext* aParentContext)
 {
   TreeMatchContext treeContext(true, nsRuleWalker::eRelevantLinkUnvisited,
                                aElement->OwnerDoc());
+  InitStyleScopes(treeContext, aElement);
   return ResolveStyleFor(aElement, aParentContext, treeContext);
 }
 
@@ -1319,7 +1339,8 @@ nsStyleSet::WalkDisableTextZoomRule(Element* aElement, nsRuleWalker* aRuleWalker
 already_AddRefed<nsStyleContext>
 nsStyleSet::ResolvePseudoElementStyle(Element* aParentElement,
                                       nsCSSPseudoElements::Type aType,
-                                      nsStyleContext* aParentContext)
+                                      nsStyleContext* aParentContext,
+                                      Element* aPseudoElement)
 {
   NS_ENSURE_FALSE(mInShutdown, nullptr);
 
@@ -1330,8 +1351,10 @@ nsStyleSet::ResolvePseudoElementStyle(Element* aParentElement,
   nsRuleWalker ruleWalker(mRuleTree, mAuthorStyleDisabled);
   TreeMatchContext treeContext(true, nsRuleWalker::eRelevantLinkUnvisited,
                                aParentElement->OwnerDoc());
+  InitStyleScopes(treeContext, aParentElement);
   PseudoElementRuleProcessorData data(PresContext(), aParentElement,
-                                      &ruleWalker, aType, treeContext);
+                                      &ruleWalker, aType, treeContext,
+                                      aPseudoElement);
   WalkRestrictionRule(aType, &ruleWalker);
   FileRules(EnumRulesMatching<PseudoElementRuleProcessorData>, &data,
             aParentElement, &ruleWalker);
@@ -1373,6 +1396,7 @@ nsStyleSet::ProbePseudoElementStyle(Element* aParentElement,
 {
   TreeMatchContext treeContext(true, nsRuleWalker::eRelevantLinkUnvisited,
                                aParentElement->OwnerDoc());
+  InitStyleScopes(treeContext, aParentElement);
   return ProbePseudoElementStyle(aParentElement, aType, aParentContext,
                                  treeContext);
 }
@@ -1381,7 +1405,8 @@ already_AddRefed<nsStyleContext>
 nsStyleSet::ProbePseudoElementStyle(Element* aParentElement,
                                     nsCSSPseudoElements::Type aType,
                                     nsStyleContext* aParentContext,
-                                    TreeMatchContext& aTreeMatchContext)
+                                    TreeMatchContext& aTreeMatchContext,
+                                    Element* aPseudoElement)
 {
   NS_ENSURE_FALSE(mInShutdown, nullptr);
 
@@ -1393,7 +1418,8 @@ nsStyleSet::ProbePseudoElementStyle(Element* aParentElement,
   nsRuleWalker ruleWalker(mRuleTree, mAuthorStyleDisabled);
   aTreeMatchContext.ResetForUnvisitedMatching();
   PseudoElementRuleProcessorData data(PresContext(), aParentElement,
-                                      &ruleWalker, aType, aTreeMatchContext);
+                                      &ruleWalker, aType, aTreeMatchContext,
+                                      aPseudoElement);
   WalkRestrictionRule(aType, &ruleWalker);
   // not the root if there was a restriction rule
   nsRuleNode *adjustedRoot = ruleWalker.CurrentNode();
@@ -1511,6 +1537,7 @@ nsStyleSet::ResolveXULTreePseudoStyle(Element* aParentElement,
   nsRuleWalker ruleWalker(mRuleTree, mAuthorStyleDisabled);
   TreeMatchContext treeContext(true, nsRuleWalker::eRelevantLinkUnvisited,
                                aParentElement->OwnerDoc());
+  InitStyleScopes(treeContext, aParentElement);
   XULTreeRuleProcessorData data(PresContext(), aParentElement, &ruleWalker,
                                 aPseudoTag, aComparator, treeContext);
   FileRules(EnumRulesMatching<XULTreeRuleProcessorData>, &data, aParentElement,
@@ -1579,7 +1606,7 @@ nsStyleSet::AppendFontFeatureValuesRules(nsPresContext* aPresContext,
 {
   NS_ENSURE_FALSE(mInShutdown, false);
 
-  for (uint32_t i = 0; i < NS_ARRAY_LENGTH(gCSSSheetTypes); ++i) {
+  for (uint32_t i = 0; i < ArrayLength(gCSSSheetTypes); ++i) {
     nsCSSRuleProcessor *ruleProc = static_cast<nsCSSRuleProcessor*>
                                     (mRuleProcessors[gCSSSheetTypes[i]].get());
     if (ruleProc &&
@@ -1632,7 +1659,7 @@ nsStyleSet::AppendPageRules(nsPresContext* aPresContext,
 {
   NS_ENSURE_FALSE(mInShutdown, false);
 
-  for (uint32_t i = 0; i < NS_ARRAY_LENGTH(gCSSSheetTypes); ++i) {
+  for (uint32_t i = 0; i < ArrayLength(gCSSSheetTypes); ++i) {
     if (gCSSSheetTypes[i] == eScopedDocSheet)
       continue;
     nsCSSRuleProcessor* ruleProc = static_cast<nsCSSRuleProcessor*>
@@ -1721,15 +1748,30 @@ nsStyleSet::GCRuleTrees()
   }
 }
 
+/**
+ * Return an equivalent to aRuleNode with both animation and transition
+ * rules removed, and post a restyle if needed.
+ */
 static inline nsRuleNode*
 SkipAnimationRules(nsRuleNode* aRuleNode, Element* aElement, bool isPseudo)
 {
   nsRuleNode* ruleNode = aRuleNode;
-  while (!ruleNode->IsRoot() &&
-         (ruleNode->GetLevel() == nsStyleSet::eTransitionSheet ||
-          ruleNode->GetLevel() == nsStyleSet::eAnimationSheet)) {
+  // The transition rule must be at the top of the cascade.
+  if (!ruleNode->IsRoot() &&
+      ruleNode->GetLevel() == nsStyleSet::eTransitionSheet) {
     ruleNode = ruleNode->GetParent();
   }
+  NS_ABORT_IF_FALSE(ruleNode->IsRoot() ||
+                    ruleNode->GetLevel() != nsStyleSet::eTransitionSheet,
+                    "can't have more than one transition rule");
+
+  // Use our existing ReplaceAnimationRule function to replace the
+  // animation rule, if present.
+  nsIStyleRule* animationRule = GetAnimationRule(ruleNode);
+  if (animationRule) {
+    ruleNode = ReplaceAnimationRule(ruleNode, animationRule, nullptr);
+  }
+
   if (ruleNode != aRuleNode) {
     NS_ASSERTION(aElement, "How can we have transition rules but no element?");
     // Need to do an animation restyle, just like
@@ -1836,6 +1878,18 @@ struct MOZ_STACK_CLASS StatefulData : public StateRuleProcessorData {
   nsRestyleHint   mHint;
 };
 
+struct MOZ_STACK_CLASS StatefulPseudoElementData : public PseudoElementStateRuleProcessorData {
+  StatefulPseudoElementData(nsPresContext* aPresContext, Element* aElement,
+               nsEventStates aStateMask, nsCSSPseudoElements::Type aPseudoType,
+               TreeMatchContext& aTreeMatchContext, Element* aPseudoElement)
+    : PseudoElementStateRuleProcessorData(aPresContext, aElement, aStateMask,
+                                          aPseudoType, aTreeMatchContext,
+                                          aPseudoElement),
+      mHint(nsRestyleHint(0))
+  {}
+  nsRestyleHint   mHint;
+};
+
 static bool SheetHasDocumentStateStyle(nsIStyleRuleProcessor* aProcessor,
                                          void *aData)
 {
@@ -1858,6 +1912,7 @@ nsStyleSet::HasDocumentStateDependentStyle(nsPresContext* aPresContext,
 
   TreeMatchContext treeContext(false, nsRuleWalker::eLinksVisitedOrUnvisited,
                                aContent->OwnerDoc());
+  InitStyleScopes(treeContext, aContent->AsElement());
   StatefulData data(aPresContext, aContent->AsElement(), aStateMask,
                     treeContext);
   WalkRuleProcessors(SheetHasDocumentStateStyle, &data, true);
@@ -1873,6 +1928,15 @@ static bool SheetHasStatefulStyle(nsIStyleRuleProcessor* aProcessor,
   return true; // continue
 }
 
+static bool SheetHasStatefulPseudoElementStyle(nsIStyleRuleProcessor* aProcessor,
+                                               void *aData)
+{
+  StatefulPseudoElementData* data = (StatefulPseudoElementData*)aData;
+  nsRestyleHint hint = aProcessor->HasStateDependentStyle(data);
+  data->mHint = nsRestyleHint(data->mHint | hint);
+  return true; // continue
+}
+
 // Test if style is dependent on content state
 nsRestyleHint
 nsStyleSet::HasStateDependentStyle(nsPresContext*       aPresContext,
@@ -1881,11 +1945,28 @@ nsStyleSet::HasStateDependentStyle(nsPresContext*       aPresContext,
 {
   TreeMatchContext treeContext(false, nsRuleWalker::eLinksVisitedOrUnvisited,
                                aElement->OwnerDoc());
+  InitStyleScopes(treeContext, aElement);
   StatefulData data(aPresContext, aElement, aStateMask, treeContext);
   WalkRuleProcessors(SheetHasStatefulStyle, &data, false);
   return data.mHint;
 }
 
+nsRestyleHint
+nsStyleSet::HasStateDependentStyle(nsPresContext* aPresContext,
+                                   Element* aElement,
+                                   nsCSSPseudoElements::Type aPseudoType,
+                                   Element* aPseudoElement,
+                                   nsEventStates aStateMask)
+{
+  TreeMatchContext treeContext(false, nsRuleWalker::eLinksVisitedOrUnvisited,
+                               aElement->OwnerDoc());
+  InitStyleScopes(treeContext, aElement);
+  StatefulPseudoElementData data(aPresContext, aElement, aStateMask,
+                                 aPseudoType, treeContext, aPseudoElement);
+  WalkRuleProcessors(SheetHasStatefulPseudoElementStyle, &data, false);
+  return data.mHint;
+}
+
 struct MOZ_STACK_CLASS AttributeData : public AttributeRuleProcessorData {
   AttributeData(nsPresContext* aPresContext,
                 Element* aElement, nsIAtom* aAttribute, int32_t aModType,
@@ -1916,6 +1997,7 @@ nsStyleSet::HasAttributeDependentStyle(nsPresContext* aPresContext,
 {
   TreeMatchContext treeContext(false, nsRuleWalker::eLinksVisitedOrUnvisited,
                                aElement->OwnerDoc());
+  InitStyleScopes(treeContext, aElement);
   AttributeData data(aPresContext, aElement, aAttribute,
                      aModType, aAttrHasChanged, treeContext);
   WalkRuleProcessors(SheetHasAttributeStyle, &data, false);
@@ -1958,9 +2040,7 @@ nsStyleSet::EnsureUniqueInnerOnCSSSheets()
     nsCOMArray<nsIStyleSheet> &sheets = mSheets[gCSSSheetTypes[i]];
     for (uint32_t j = 0, j_end = sheets.Count(); j < j_end; ++j) {
       nsCSSStyleSheet *sheet = static_cast<nsCSSStyleSheet*>(sheets[j]);
-      if (!queue.AppendElement(sheet)) {
-        return nsCSSStyleSheet::eUniqueInner_CloneFailed;
-      }
+      queue.AppendElement(sheet);
     }
   }
 
@@ -1977,17 +2057,12 @@ nsStyleSet::EnsureUniqueInnerOnCSSSheets()
 
     nsCSSStyleSheet::EnsureUniqueInnerResult sheetRes =
       sheet->EnsureUniqueInner();
-    if (sheetRes == nsCSSStyleSheet::eUniqueInner_CloneFailed) {
-      return sheetRes;
-    }
     if (sheetRes == nsCSSStyleSheet::eUniqueInner_ClonedInner) {
       res = sheetRes;
     }
 
     // Enqueue all the sheet's children.
-    if (!sheet->AppendAllChildSheets(queue)) {
-      return nsCSSStyleSheet::eUniqueInner_CloneFailed;
-    }
+    sheet->AppendAllChildSheets(queue);
   }
   return res;
 }
