@@ -13,11 +13,13 @@ const XHTML_NS = "http://www.w3.org/1999/xhtml";
 
 const DEBUG_CONTRACTID = "@mozilla.org/xpcom/debug;1";
 const PRINTSETTINGS_CONTRACTID = "@mozilla.org/gfx/printsettings-service;1";
+const ENVIRONMENT_CONTRACTID = "@mozilla.org/process/environment;1";
 
 // "<!--CLEAR-->"
 const BLANK_URL_FOR_CLEARING = "data:text/html;charset=UTF-8,%3C%21%2D%2DCLEAR%2D%2D%3E";
 
 CU.import("resource://gre/modules/Timer.jsm");
+CU.import("resource://gre/modules/AsyncSpellCheckTestHelper.jsm");
 
 var gBrowserIsRemote;
 var gHaveCanvasSnapshot = false;
@@ -38,10 +40,13 @@ var gFailureReason;
 var gAssertionCount = 0;
 
 var gDebug;
+var gVerbose = false;
 
 var gCurrentTestStartTime;
 var gClearingForAssertionCheck = false;
 
+const TYPE_LOAD = 'load';  // test without a reference (just test that it does
+                           // not assert, crash, hang, or leak)
 const TYPE_SCRIPT = 'script'; // test contains individual test results
 
 function markupDocumentViewer() {
@@ -93,6 +98,8 @@ function OnInitialLoad()
 #endif
 
     gDebug = CC[DEBUG_CONTRACTID].getService(CI.nsIDebug2);
+    var env = CC[ENVIRONMENT_CONTRACTID].getService(CI.nsIEnvironment);
+    gVerbose = !!env.get("MOZ_REFTEST_VERBOSE");
 
     RegisterMessageListeners();
 
@@ -175,43 +182,96 @@ function setupPrintMode() {
    docShell.contentViewer.setPageMode(true, ps);
 }
 
-function setupDisplayport(contentRootElement) {
+function attrOrDefault(element, attr, def) {
+    return element.hasAttribute(attr) ? Number(element.getAttribute(attr)) : def;
+}
+
+function setupViewport(contentRootElement) {
     if (!contentRootElement) {
         return;
     }
 
-    function attrOrDefault(attr, def) {
-        return contentRootElement.hasAttribute(attr) ?
-            contentRootElement.getAttribute(attr) : def;
-    }
-
-    var vw = attrOrDefault("reftest-viewport-w", 0);
-    var vh = attrOrDefault("reftest-viewport-h", 0);
+    var vw = attrOrDefault(contentRootElement, "reftest-viewport-w", 0);
+    var vh = attrOrDefault(contentRootElement, "reftest-viewport-h", 0);
     if (vw !== 0 || vh !== 0) {
         LogInfo("Setting viewport to <w="+ vw +", h="+ vh +">");
         windowUtils().setCSSViewport(vw, vh);
     }
 
-    // XXX support displayPortX/Y when needed
-    var dpw = attrOrDefault("reftest-displayport-w", 0);
-    var dph = attrOrDefault("reftest-displayport-h", 0);
-    var dpx = attrOrDefault("reftest-displayport-x", 0);
-    var dpy = attrOrDefault("reftest-displayport-y", 0);
-    if (dpw !== 0 || dph !== 0) {
-        LogInfo("Setting displayport to <x="+ dpx +", y="+ dpy +", w="+ dpw +", h="+ dph +">");
-        windowUtils().setDisplayPortForElement(dpx, dpy, dpw, dph, content.document.documentElement);
-    }
-    var asyncScroll = attrOrDefault("reftest-async-scroll", false);
-    if (asyncScroll) {
-      SendEnableAsyncScroll();
-    }
-
     // XXX support resolution when needed
 
     // XXX support viewconfig when needed
 }
+ 
+function setupDisplayport(contentRootElement) {
+    if (!contentRootElement) {
+        return;
+    }
 
-function resetDisplayport() {
+    function setupDisplayportForElement(element) {
+        var dpw = attrOrDefault(element, "reftest-displayport-w", 0);
+        var dph = attrOrDefault(element, "reftest-displayport-h", 0);
+        var dpx = attrOrDefault(element, "reftest-displayport-x", 0);
+        var dpy = attrOrDefault(element, "reftest-displayport-y", 0);
+        if (dpw !== 0 || dph !== 0 || dpx != 0 || dpy != 0) {
+            LogInfo("Setting displayport to <x="+ dpx +", y="+ dpy +", w="+ dpw +", h="+ dph +">");
+            windowUtils().setDisplayPortForElement(dpx, dpy, dpw, dph, element, 1);
+        }
+    }
+
+    function setupDisplayportForElementSubtree(element) {
+        setupDisplayportForElement(element);
+        for (var c = element.firstElementChild; c; c = c.nextElementSibling) {
+            setupDisplayportForElementSubtree(c);
+        }
+    }
+
+    if (contentRootElement.hasAttribute("reftest-async-scroll")) {
+        SendEnableAsyncScroll();
+        setupDisplayportForElementSubtree(contentRootElement);
+    } else {
+        setupDisplayportForElement(contentRootElement);
+    }
+}
+
+function setupAsyncScrollOffsets(options) {
+    var currentDoc = content.document;
+    var contentRootElement = currentDoc ? currentDoc.documentElement : null;
+
+    if (!contentRootElement) {
+        return;
+    }
+
+    function setupAsyncScrollOffsetsForElement(element) {
+        var sx = attrOrDefault(element, "reftest-async-scroll-x", 0);
+        var sy = attrOrDefault(element, "reftest-async-scroll-y", 0);
+        if (sx != 0 || sy != 0) {
+            try {
+                // This might fail when called from RecordResult since layers
+                // may not have been constructed yet
+                windowUtils().setAsyncScrollOffset(element, sx, sy);
+            } catch (e) {
+                if (!options.allowFailure) {
+                    throw e;
+                }
+            }
+        }
+    }
+
+    function setupAsyncScrollOffsetsForElementSubtree(element) {
+        setupAsyncScrollOffsetsForElement(element);
+        for (var c = element.firstElementChild; c; c = c.nextElementSibling) {
+            setupAsyncScrollOffsetsForElementSubtree(c);
+        }
+    }
+
+    var asyncScroll = contentRootElement.hasAttribute("reftest-async-scroll");
+    if (asyncScroll) {
+        setupAsyncScrollOffsetsForElementSubtree(contentRootElement);
+    }
+}
+
+function resetDisplayportAndViewport() {
     // XXX currently the displayport configuration lives on the
     // presshell and so is "reset" on nav when we get a new presshell.
 }
@@ -234,6 +294,14 @@ function shouldWaitForReftestWaitRemoval(contentRootElement) {
                              .indexOf("reftest-wait") != -1;
 }
 
+function shouldSnapshotWholePage(contentRootElement) {
+    // use getAttribute because className works differently in HTML and SVG
+    return contentRootElement &&
+           contentRootElement.hasAttribute('class') &&
+           contentRootElement.getAttribute('class').split(/\s+/)
+                             .indexOf("reftest-snapshot-all") != -1;
+}
+
 function getNoPaintElements(contentRootElement) {
   return contentRootElement.getElementsByClassName('reftest-no-paint');
 }
@@ -245,12 +313,15 @@ const STATE_WAITING_TO_FIRE_INVALIDATE_EVENT = 0;
 // When reftest-wait has been removed from the root element, we can move to the
 // next state.
 const STATE_WAITING_FOR_REFTEST_WAIT_REMOVAL = 1;
+// When spell checking is done on all spell-checked elements, we can move to the
+// next state.
+const STATE_WAITING_FOR_SPELL_CHECKS = 2;
 // When all MozAfterPaint events and all explicit paint waits are flushed, we're
 // done and can move to the COMPLETED state.
-const STATE_WAITING_TO_FINISH = 2;
-const STATE_COMPLETED = 3;
+const STATE_WAITING_TO_FINISH = 3;
+const STATE_COMPLETED = 4;
 
-function WaitForTestEnd(contentRootElement, inPrintMode) {
+function WaitForTestEnd(contentRootElement, inPrintMode, spellCheckedElements) {
     var stopAfterPaintReceived = false;
     var currentDoc = content.document;
     var state = STATE_WAITING_TO_FIRE_INVALIDATE_EVENT;
@@ -296,7 +367,7 @@ function WaitForTestEnd(contentRootElement, inPrintMode) {
             return;
         }
 
-        SendUpdateCanvasForEvent(event);
+        SendUpdateCanvasForEvent(event, contentRootElement);
         // These events are fired immediately after a paint. Don't
         // confuse ourselves by firing synchronously if we triggered the
         // paint ourselves.
@@ -392,6 +463,20 @@ function WaitForTestEnd(contentRootElement, inPrintMode) {
                 LogInfo("MakeProgress: waiting for reftest-wait to be removed");
                 return;
             }
+
+            // Try next state
+            state = STATE_WAITING_FOR_SPELL_CHECKS;
+            MakeProgress();
+            return;
+
+        case STATE_WAITING_FOR_SPELL_CHECKS:
+            LogInfo("MakeProgress: STATE_WAITING_FOR_SPELL_CHECKS");
+            if (numPendingSpellChecks) {
+                gFailureReason = "timed out waiting for spell checks to end";
+                LogInfo("MakeProgress: waiting for spell checks to end");
+                return;
+            }
+
             state = STATE_WAITING_TO_FINISH;
             if (!inPrintMode && doPrintMode(contentRootElement)) {
                 LogInfo("MakeProgress: setting up print mode");
@@ -420,7 +505,7 @@ function WaitForTestEnd(contentRootElement, inPrintMode) {
               var elements = getNoPaintElements(contentRootElement);
               for (var i = 0; i < elements.length; ++i) {
                   if (windowUtils().checkAndClearPaintedState(elements[i])) {
-                      LogError("REFTEST TEST-UNEXPECTED-FAIL | element marked as reftest-no-paint got repainted!");
+                      SendFailedNoPaint();
                   }
               }
             }
@@ -444,6 +529,21 @@ function WaitForTestEnd(contentRootElement, inPrintMode) {
     gExplicitPendingPaintsCompleteHook = ExplicitPaintsCompleteListener;
     gTimeoutHook = RemoveListeners;
 
+    // Listen for spell checks on spell-checked elements.
+    var numPendingSpellChecks = spellCheckedElements.length;
+    function decNumPendingSpellChecks() {
+        --numPendingSpellChecks;
+        MakeProgress();
+    }
+    for (let editable of spellCheckedElements) {
+        try {
+            onSpellCheck(editable, decNumPendingSpellChecks);
+        } catch (err) {
+            // The element may not have an editor, so ignore it.
+            setTimeout(decNumPendingSpellChecks, 0);
+        }
+    }
+
     // Take a full snapshot now that all our listeners are set up. This
     // ensures it's impossible for us to miss updates between taking the snapshot
     // and adding our listeners.
@@ -470,9 +570,23 @@ function OnDocumentLoad(event)
         return;
     }
 
+    // Collect all editable, spell-checked elements.  It may be the case that
+    // not all the elements that match this selector will be spell checked: for
+    // example, a textarea without a spellcheck attribute may have a parent with
+    // spellcheck=false, or script may set spellcheck=false on an element whose
+    // markup sets it to true.  But that's OK since onSpellCheck detects the
+    // absence of spell checking, too.
+    var querySelector =
+        '*[class~="spell-checked"],' +
+        'textarea:not([spellcheck="false"]),' +
+        'input[spellcheck]:-moz-any([spellcheck=""],[spellcheck="true"]),' +
+        '*[contenteditable]:-moz-any([contenteditable=""],[contenteditable="true"])';
+    var spellCheckedElements = currentDoc.querySelectorAll(querySelector);
+
     var contentRootElement = currentDoc ? currentDoc.documentElement : null;
     currentDoc = null;
     setupZoom(contentRootElement);
+    setupViewport(contentRootElement);
     setupDisplayport(contentRootElement);
     var inPrintMode = false;
 
@@ -495,7 +609,7 @@ function OnDocumentLoad(event)
             !painted) {
             LogInfo("AfterOnLoadScripts belatedly entering WaitForTestEnd");
             // Go into reftest-wait mode belatedly.
-            WaitForTestEnd(contentRootElement, inPrintMode);
+            WaitForTestEnd(contentRootElement, inPrintMode, []);
         } else {
             CheckForProcessCrashExpectation();
             RecordResult();
@@ -503,12 +617,13 @@ function OnDocumentLoad(event)
     }
 
     if (shouldWaitForReftestWaitRemoval(contentRootElement) ||
-        shouldWaitForExplicitPaintWaiters()) {
+        shouldWaitForExplicitPaintWaiters() ||
+        spellCheckedElements.length) {
         // Go into reftest-wait mode immediately after painting has been
         // unsuppressed, after the onload event has finished dispatching.
         gFailureReason = "timed out waiting for test to complete (trying to get into WaitForTestEnd)";
         LogInfo("OnDocumentLoad triggering WaitForTestEnd");
-        setTimeout(function () { WaitForTestEnd(contentRootElement, inPrintMode); }, 0);
+        setTimeout(function () { WaitForTestEnd(contentRootElement, inPrintMode, spellCheckedElements); }, 0);
     } else {
         if (doPrintMode(contentRootElement)) {
             LogInfo("OnDocumentLoad setting up print mode");
@@ -587,6 +702,9 @@ function RecordResult()
         return;
     }
 
+    // Setup async scroll offsets now in case SynchronizeForSnapshot is not
+    // called (due to reftest-no-sync-layers being supplied).
+    setupAsyncScrollOffsets({allowFailure:true});
     SendTestDone(currentTestRunTime);
     FinishTestItem();
 }
@@ -626,18 +744,32 @@ function LoadURI(uri)
 
 function LogWarning(str)
 {
-    sendAsyncMessage("reftest:Log", { type: "warning", msg: str });
+    if (gVerbose) {
+        sendSyncMessage("reftest:Log", { type: "warning", msg: str });
+    } else {
+        sendAsyncMessage("reftest:Log", { type: "warning", msg: str });
+    }
 }
 
 function LogInfo(str)
 {
-    sendAsyncMessage("reftest:Log", { type: "info", msg: str });
+    if (gVerbose) {
+        sendSyncMessage("reftest:Log", { type: "info", msg: str });
+    } else {
+        sendAsyncMessage("reftest:Log", { type: "info", msg: str });
+    }
 }
 
 const SYNC_DEFAULT = 0x0;
 const SYNC_ALLOW_DISABLE = 0x1;
 function SynchronizeForSnapshot(flags)
 {
+    if (gCurrentTestType == TYPE_SCRIPT ||
+        gCurrentTestType == TYPE_LOAD) {
+        // Script tests or load-only tests do not need any snapshotting
+        return;
+    }
+
     if (flags & SYNC_ALLOW_DISABLE) {
         var docElt = content.document.documentElement;
         if (docElt && docElt.hasAttribute("reftest-no-sync-layers")) {
@@ -653,6 +785,10 @@ function SynchronizeForSnapshot(flags)
     var ctx = dummyCanvas.getContext("2d");
     var flags = ctx.DRAWWINDOW_DRAW_CARET | ctx.DRAWWINDOW_DRAW_VIEW | ctx.DRAWWINDOW_USE_WIDGET_LAYERS;
     ctx.drawWindow(content, 0, 0, 1, 1, "rgb(255,255,255)", flags);
+
+    // Setup async scroll offsets now, because any scrollable layers should
+    // have had their AsyncPanZoomControllers created.
+    setupAsyncScrollOffsets({allowFailure:false});
 }
 
 function RegisterMessageListeners()
@@ -694,7 +830,7 @@ function RecvLoadScriptTest(uri, timeout)
 function RecvResetRenderingState()
 {
     resetZoom();
-    resetDisplayport();
+    resetDisplayportAndViewport();
 }
 
 function SendAssertionCount(numAssertions)
@@ -717,6 +853,11 @@ function SendFailedLoad(why)
     sendAsyncMessage("reftest:FailedLoad", { why: why });
 }
 
+function SendFailedNoPaint()
+{
+    sendAsyncMessage("reftest:FailedNoPaint");
+}
+
 function SendEnableAsyncScroll()
 {
     sendAsyncMessage("reftest:EnableAsyncScroll");
@@ -768,12 +909,24 @@ function roundTo(x, fraction)
     return Math.round(x/fraction)*fraction;
 }
 
-function SendUpdateCanvasForEvent(event)
+function SendUpdateCanvasForEvent(event, contentRootElement)
 {
     var win = content;
     var scale = markupDocumentViewer().fullZoom;
 
     var rects = [ ];
+    if (shouldSnapshotWholePage) {
+      // See comments in SendInitCanvasWithSnapshot() re: the split
+      // logic here.
+      if (!gBrowserIsRemote) {
+          sendSyncMessage("reftest:UpdateWholeCanvasForInvalidation");
+      } else {
+          SynchronizeForSnapshot(SYNC_ALLOW_DISABLE);
+          sendAsyncMessage("reftest:UpdateWholeCanvasForInvalidation");
+      }
+      return;
+    }
+    
     var rectList = event.clientRects;
     LogInfo("SendUpdateCanvasForEvent with " + rectList.length + " rects");
     for (var i = 0; i < rectList.length; ++i) {
