@@ -6,12 +6,12 @@
 package org.mozilla.gecko;
 
 import org.mozilla.gecko.mozglue.GeckoLoader;
+import org.mozilla.gecko.mozglue.RobocopTarget;
 import org.mozilla.gecko.util.GeckoEventListener;
 import org.mozilla.gecko.util.ThreadUtils;
 
 import org.json.JSONObject;
 
-import android.content.Intent;
 import android.content.Context;
 import android.content.res.Configuration;
 import android.content.res.Resources;
@@ -28,26 +28,65 @@ import java.util.Locale;
 public class GeckoThread extends Thread implements GeckoEventListener {
     private static final String LOGTAG = "GeckoThread";
 
+    @RobocopTarget
     public enum LaunchState {
         Launching,
         WaitForDebugger,
         Launched,
         GeckoRunning,
-        GeckoExiting
+        GeckoExiting,
+        GeckoExited
     };
 
     private static LaunchState sLaunchState = LaunchState.Launching;
 
-    private Intent mIntent;
+    private static GeckoThread sGeckoThread;
+
+    private final String mArgs;
+    private final String mAction;
     private final String mUri;
 
-    GeckoThread(Intent intent, String uri) {
-        mIntent = intent;
+    public static boolean ensureInit() {
+        ThreadUtils.assertOnUiThread();
+        if (isCreated())
+            return false;
+        sGeckoThread = new GeckoThread(sArgs, sAction, sUri);
+        return true;
+    }
+
+    public static String sArgs;
+    public static String sAction;
+    public static String sUri;
+
+    public static void setArgs(String args) {
+        sArgs = args;
+    }
+
+    public static void setAction(String action) {
+        sAction = action;
+    }
+
+    public static void setUri(String uri) {
+        sUri = uri;
+    }
+
+    GeckoThread(String args, String action, String uri) {
+        mArgs = args;
+        mAction = action;
         mUri = uri;
         setName("Gecko");
         GeckoAppShell.getEventDispatcher().registerEventListener("Gecko:Ready", this);
     }
 
+    public static boolean isCreated() {
+        return sGeckoThread != null;
+    }
+
+    public static void createAndStart() {
+        if (ensureInit())
+            sGeckoThread.start();
+    }
+
     private String initGeckoEnvironment() {
         // At some point while loading the gecko libs our default locale gets set
         // so just save it to locale here and reset it as default after the join
@@ -72,7 +111,7 @@ public class GeckoThread extends Thread implements GeckoEventListener {
             Activity activity = (Activity)app;
             resourcePath = activity.getApplication().getPackageResourcePath();
             res = activity.getBaseContext().getResources();
-            GeckoLoader.setupGeckoEnvironment(activity, pluginDirs, GeckoProfile.get(app).getFilesDir().getPath());
+            GeckoLoader.setupGeckoEnvironment(activity, pluginDirs, app.getFilesDir().getPath());
         }
         GeckoLoader.loadSQLiteLibs(app, resourcePath);
         GeckoLoader.loadNSSLibs(app, resourcePath);
@@ -110,7 +149,7 @@ public class GeckoThread extends Thread implements GeckoEventListener {
                 if (args == null || !args.contains(BrowserApp.GUEST_BROWSING_ARG)) {
                     guest = " " + BrowserApp.GUEST_BROWSING_ARG;
                 }
-            } else if (!GeckoApp.sIsUsingCustomProfile) {
+            } else if (!GeckoProfile.sIsUsingCustomProfile) {
                 // If nothing was passed in in the intent, force Gecko to use the default profile for
                 // for this activity
                 profile = " -P " + GeckoAppShell.getGeckoInterface().getProfile().getName();
@@ -131,9 +170,8 @@ public class GeckoThread extends Thread implements GeckoEventListener {
 
         Log.w(LOGTAG, "zerdatime " + SystemClock.uptimeMillis() + " - runGecko");
 
-        String args = addCustomProfileArg(mIntent.getStringExtra("args"));
-        String type = getTypeFromAction(mIntent.getAction());
-        mIntent = null;
+        String args = addCustomProfileArg(mArgs);
+        String type = getTypeFromAction(mAction);
 
         // and then fire us up
         Log.i(LOGTAG, "RunGecko - args = " + args);
@@ -151,6 +189,7 @@ public class GeckoThread extends Thread implements GeckoEventListener {
         }
     }
 
+    @RobocopTarget
     public static boolean checkLaunchState(LaunchState checkState) {
         synchronized (sLock) {
             return sLaunchState == checkState;
