@@ -5,15 +5,15 @@
 
 package org.mozilla.gecko.gfx;
 
-import org.mozilla.gecko.BrowserApp;
 import org.mozilla.gecko.GeckoAppShell;
 import org.mozilla.gecko.GeckoEvent;
 import org.mozilla.gecko.Tab;
 import org.mozilla.gecko.Tabs;
 import org.mozilla.gecko.ZoomConstraints;
-import org.mozilla.gecko.util.EventDispatcher;
+import org.mozilla.gecko.mozglue.RobocopTarget;
+import org.mozilla.gecko.mozglue.generatorannotations.WrapElementForJNI;
+import org.mozilla.gecko.EventDispatcher;
 import org.mozilla.gecko.util.FloatUtils;
-import org.mozilla.gecko.util.ThreadUtils;
 
 import android.content.Context;
 import android.graphics.PointF;
@@ -55,7 +55,6 @@ public class GeckoLayerClient implements LayerView.Listener, PanZoomTarget
      */
     private ImmutableViewportMetrics mFrameMetrics;
 
-    /* Used by robocop for testing purposes */
     private DrawListener mDrawListener;
 
     /* Used as temporaries by syncViewportInfo */
@@ -122,9 +121,16 @@ public class GeckoLayerClient implements LayerView.Listener, PanZoomTarget
         DisplayMetrics displayMetrics = context.getResources().getDisplayMetrics();
         mViewportMetrics = new ImmutableViewportMetrics(displayMetrics)
                            .setViewportSize(view.getWidth(), view.getHeight());
-        mFrameMetrics = mViewportMetrics;
         mZoomConstraints = new ZoomConstraints(false);
 
+        Tab tab = Tabs.getInstance().getSelectedTab();
+        if (tab != null) {
+            mZoomConstraints = tab.getZoomConstraints();
+            mViewportMetrics = mViewportMetrics.setIsRTL(tab.getIsRTL());
+        }
+
+        mFrameMetrics = mViewportMetrics;
+
         mPanZoomController = PanZoomController.Factory.create(this, view, eventDispatcher);
         mMarginsAnimator = new LayerMarginsAnimator(this, view);
         mView = view;
@@ -149,13 +155,13 @@ public class GeckoLayerClient implements LayerView.Listener, PanZoomTarget
 
         // Gecko being ready is one of the two conditions (along with having an available
         // surface) that cause us to create the compositor. So here, now that we know gecko
-        // is ready, call createCompositor() to see if we can actually do the creation.
+        // is ready, call updateCompositor() to see if we can actually do the creation.
         // This needs to run on the UI thread so that the surface validity can't change on
         // us while we're in the middle of creating the compositor.
         mView.post(new Runnable() {
             @Override
             public void run() {
-                mView.getGLController().createCompositor();
+                mView.getGLController().updateCompositor();
             }
         });
     }
@@ -393,9 +399,6 @@ public class GeckoLayerClient implements LayerView.Listener, PanZoomTarget
                 if (!oldMetrics.fuzzyEquals(newMetrics)) {
                     abortPanZoomAnimation();
                 }
-                mPanZoomController.updateScrollOffset(
-                		messageMetrics.viewportRectLeft / messageMetrics.zoomFactor,
-                		messageMetrics.viewportRectTop / messageMetrics.zoomFactor);
                 break;
             case PAGE_SIZE:
                 // adjust the page dimensions to account for differences in zoom
@@ -422,7 +425,7 @@ public class GeckoLayerClient implements LayerView.Listener, PanZoomTarget
         return mDisplayPort;
     }
 
-    /* This is invoked by JNI on the gecko thread */
+    @WrapElementForJNI
     DisplayPortMetrics getDisplayPort(boolean pageSizeUpdate, boolean isBrowserContentDisplayed, int tabId, ImmutableViewportMetrics metrics) {
         Tabs tabs = Tabs.getInstance();
         if (isBrowserContentDisplayed && tabs.isSelectedTabId(tabId)) {
@@ -439,12 +442,12 @@ public class GeckoLayerClient implements LayerView.Listener, PanZoomTarget
         }
     }
 
-    /* This is invoked by JNI on the gecko thread */
+    @WrapElementForJNI
     void contentDocumentChanged() {
         mContentDocumentIsDisplayed = false;
     }
 
-    /* This is invoked by JNI on the gecko thread */
+    @WrapElementForJNI
     boolean isContentDocumentDisplayed() {
         return mContentDocumentIsDisplayed;
     }
@@ -454,6 +457,7 @@ public class GeckoLayerClient implements LayerView.Listener, PanZoomTarget
     // to abort the current update and continue with any subsequent ones. This
     // is useful for slow-to-render pages when the display-port starts lagging
     // behind enough that continuing to draw it is wasted effort.
+    @WrapElementForJNI(allowMultithread = true)
     public ProgressiveUpdateData progressiveUpdateCallback(boolean aHasPendingNewThebesContent,
                                                            float x, float y, float width, float height,
                                                            float resolution, boolean lowPrecision) {
@@ -479,7 +483,7 @@ public class GeckoLayerClient implements LayerView.Listener, PanZoomTarget
         // Always abort updates if the resolution has changed. There's no use
         // in drawing at the incorrect resolution.
         if (!FloatUtils.fuzzyEquals(resolution, viewportMetrics.zoomFactor)) {
-            Log.d(LOGTAG, "Aborting draw due to resolution change");
+            Log.d(LOGTAG, "Aborting draw due to resolution change: " + resolution + " != " + viewportMetrics.zoomFactor);
             mProgressiveUpdateData.abort = true;
             return mProgressiveUpdateData;
         }
@@ -561,13 +565,13 @@ public class GeckoLayerClient implements LayerView.Listener, PanZoomTarget
         }
     }
 
-    /** This function is invoked by Gecko via JNI; be careful when modifying signature.
-      * The compositor invokes this function just before compositing a frame where the document
+    /** The compositor invokes this function just before compositing a frame where the document
       * is different from the document composited on the last frame. In these cases, the viewport
       * information we have in Java is no longer valid and needs to be replaced with the new
       * viewport information provided. setPageRect will never be invoked on the same frame that
       * this function is invoked on; and this function will always be called prior to syncViewportInfo.
       */
+    @WrapElementForJNI(allowMultithread = true)
     public void setFirstPaintViewport(float offsetX, float offsetY, float zoom,
             float cssPageLeft, float cssPageTop, float cssPageRight, float cssPageBottom) {
         synchronized (getLock()) {
@@ -620,12 +624,12 @@ public class GeckoLayerClient implements LayerView.Listener, PanZoomTarget
         mContentDocumentIsDisplayed = true;
     }
 
-    /** This function is invoked by Gecko via JNI; be careful when modifying signature.
-      * The compositor invokes this function whenever it determines that the page rect
+    /** The compositor invokes this function whenever it determines that the page rect
       * has changed (based on the information it gets from layout). If setFirstPaintViewport
       * is invoked on a frame, then this function will not be. For any given frame, this
       * function will be invoked before syncViewportInfo.
       */
+    @WrapElementForJNI(allowMultithread = true)
     public void setPageRect(float cssPageLeft, float cssPageTop, float cssPageRight, float cssPageBottom) {
         synchronized (getLock()) {
             RectF cssPageRect = new RectF(cssPageLeft, cssPageTop, cssPageRight, cssPageBottom);
@@ -638,8 +642,7 @@ public class GeckoLayerClient implements LayerView.Listener, PanZoomTarget
         }
     }
 
-    /** This function is invoked by Gecko via JNI; be careful when modifying signature.
-      * The compositor invokes this function on every frame to figure out what part of the
+    /** The compositor invokes this function on every frame to figure out what part of the
       * page to display, and to inform Java of the current display port. Since it is called
       * on every frame, it needs to be ultra-fast.
       * It avoids taking any locks or allocating any objects. We keep around a
@@ -647,6 +650,7 @@ public class GeckoLayerClient implements LayerView.Listener, PanZoomTarget
       * everytime we're called. NOTE: we might be able to return a ImmutableViewportMetrics
       * which would avoid the copy into mCurrentViewTransform.
       */
+    @WrapElementForJNI(allowMultithread = true)
     public ViewTransform syncViewportInfo(int x, int y, int width, int height, float resolution, boolean layersUpdated) {
         // getViewportMetrics is thread safe so we don't need to synchronize.
         // We save the viewport metrics here, so we later use it later in
@@ -700,7 +704,7 @@ public class GeckoLayerClient implements LayerView.Listener, PanZoomTarget
         return mCurrentViewTransform;
     }
 
-    /* Invoked by JNI from the compositor thread */
+    @WrapElementForJNI(allowMultithread = true)
     public ViewTransform syncFrameMetrics(float offsetX, float offsetY, float zoom,
                 float cssPageLeft, float cssPageTop, float cssPageRight, float cssPageBottom,
                 boolean layersUpdated, int x, int y, int width, int height, float resolution,
@@ -714,7 +718,7 @@ public class GeckoLayerClient implements LayerView.Listener, PanZoomTarget
         return syncViewportInfo(x, y, width, height, resolution, layersUpdated);
     }
 
-    /** This function is invoked by Gecko via JNI; be careful when modifying signature. */
+    @WrapElementForJNI(allowMultithread = true)
     public LayerRenderer.Frame createFrame() {
         // Create the shaders and textures if necessary.
         if (!mLayerRendererInitialized) {
@@ -726,12 +730,12 @@ public class GeckoLayerClient implements LayerView.Listener, PanZoomTarget
         return mLayerRenderer.createFrame(mFrameMetrics);
     }
 
-    /** This function is invoked by Gecko via JNI; be careful when modifying signature. */
+    @WrapElementForJNI(allowMultithread = true)
     public void activateProgram() {
         mLayerRenderer.activateDefaultProgram();
     }
 
-    /** This function is invoked by Gecko via JNI; be careful when modifying signature. */
+    @WrapElementForJNI(allowMultithread = true)
     public void deactivateProgram() {
         mLayerRenderer.deactivateDefaultProgram();
     }
@@ -844,7 +848,6 @@ public class GeckoLayerClient implements LayerView.Listener, PanZoomTarget
         if (notifyGecko && mGeckoIsReady) {
             geometryChanged(null);
         }
-        setShadowVisibility();
     }
 
     /*
@@ -884,7 +887,7 @@ public class GeckoLayerClient implements LayerView.Listener, PanZoomTarget
      * You must hold the monitor while calling this.
      */
     @Override
-    public void onSubdocumentScrollBy(float dx, float dy) {
+    public void scrollMarginsBy(float dx, float dy) {
         ImmutableViewportMetrics newMarginsMetrics =
             mMarginsAnimator.scrollBy(mViewportMetrics, dx, dy);
         mViewportMetrics = mViewportMetrics.setMarginsFrom(newMarginsMetrics);
@@ -904,23 +907,6 @@ public class GeckoLayerClient implements LayerView.Listener, PanZoomTarget
         public void onPanZoomStopped();
     }
 
-    private void setShadowVisibility() {
-        try {
-            if (BrowserApp.mBrowserToolbar == null) // this will throw if we don't have BrowserApp
-                return;
-            ThreadUtils.postToUiThread(new Runnable() {
-                @Override
-                public void run() {
-                    if (BrowserApp.mBrowserToolbar == null) {
-                        return;
-                    }
-                    ImmutableViewportMetrics m = mViewportMetrics;
-                    BrowserApp.mBrowserToolbar.setShadowVisibility(m.viewportRectTop >= m.pageRectTop);
-                }
-            });
-        } catch (NoClassDefFoundError ex) {}
-    }
-
     /** Implementation of PanZoomTarget */
     @Override
     public void forceRedraw(DisplayPortMetrics displayPort) {
@@ -1000,11 +986,13 @@ public class GeckoLayerClient implements LayerView.Listener, PanZoomTarget
     }
 
     /** Used by robocop for testing purposes. Not for production use! */
+    @RobocopTarget
     public void setDrawListener(DrawListener listener) {
         mDrawListener = listener;
     }
 
     /** Used by robocop for testing purposes. Not for production use! */
+    @RobocopTarget
     public static interface DrawListener {
         public void drawFinished();
     }
