@@ -18,16 +18,20 @@ import java.util.Map.Entry;
 import java.util.Set;
 import java.util.concurrent.atomic.AtomicLong;
 
+import org.json.simple.JSONArray;
 import org.json.simple.parser.ParseException;
+import org.mozilla.gecko.background.common.log.Logger;
 import org.mozilla.gecko.sync.crypto.CryptoException;
 import org.mozilla.gecko.sync.crypto.KeyBundle;
+import org.mozilla.gecko.sync.delegates.BaseGlobalSessionCallback;
 import org.mozilla.gecko.sync.delegates.ClientsDataDelegate;
 import org.mozilla.gecko.sync.delegates.FreshStartDelegate;
-import org.mozilla.gecko.sync.delegates.GlobalSessionCallback;
-import org.mozilla.gecko.sync.delegates.InfoCollectionsDelegate;
+import org.mozilla.gecko.sync.delegates.JSONRecordFetchDelegate;
 import org.mozilla.gecko.sync.delegates.KeyUploadDelegate;
 import org.mozilla.gecko.sync.delegates.MetaGlobalDelegate;
+import org.mozilla.gecko.sync.delegates.NodeAssignmentCallback;
 import org.mozilla.gecko.sync.delegates.WipeServerDelegate;
+import org.mozilla.gecko.sync.net.AuthHeaderProvider;
 import org.mozilla.gecko.sync.net.BaseResource;
 import org.mozilla.gecko.sync.net.HttpResponseObserver;
 import org.mozilla.gecko.sync.net.SyncResponse;
@@ -57,10 +61,9 @@ import android.content.SharedPreferences;
 import android.os.Bundle;
 import ch.boye.httpclientandroidlib.HttpResponse;
 
-public class GlobalSession implements CredentialsSource, PrefsSource, HttpResponseObserver {
+public class GlobalSession implements PrefsSource, HttpResponseObserver {
   private static final String LOG_TAG = "GlobalSession";
 
-  public static final String API_VERSION   = "1.1";
   public static final long STORAGE_VERSION = 5;
 
   public SyncConfiguration config = null;
@@ -68,16 +71,18 @@ public class GlobalSession implements CredentialsSource, PrefsSource, HttpRespon
   protected Map<Stage, GlobalSyncStage> stages;
   public Stage currentState = Stage.idle;
 
-  public final GlobalSessionCallback callback;
-  private Context context;
-  private ClientsDataDelegate clientsDelegate;
+  public final BaseGlobalSessionCallback callback;
+  protected final Context context;
+  protected final ClientsDataDelegate clientsDelegate;
+  protected final NodeAssignmentCallback nodeAssignmentCallback;
 
   /**
    * Map from engine name to new settings for an updated meta/global record.
+   * Engines to remove will have <code>null</code> EngineSettings.
    */
   public final Map<String, EngineSettings> enginesToUpdate = new HashMap<String, EngineSettings>();
 
-  /*
+   /*
    * Key accessors.
    */
   public KeyBundle keyBundleForCollection(String collection) throws NoCollectionKeysSetException {
@@ -87,124 +92,86 @@ public class GlobalSession implements CredentialsSource, PrefsSource, HttpRespon
   /*
    * Config passthrough for convenience.
    */
-  @Override
-  public String credentials() {
-    return config.credentials();
+  public AuthHeaderProvider getAuthHeaderProvider() {
+    return config.getAuthHeaderProvider();
   }
 
   public URI wboURI(String collection, String id) throws URISyntaxException {
     return config.wboURI(collection, id);
   }
 
-  /*
-   * Validators.
-   */
-  private static boolean isInvalidString(String s) {
-    return s == null ||
-           s.trim().length() == 0;
-  }
-
-  private static boolean anyInvalidStrings(String s, String...strings) {
-    if (isInvalidString(s)) {
-      return true;
-    }
-    for (String str : strings) {
-      if (isInvalidString(str)) {
-        return true;
-      }
-    }
-    return false;
-  }
-
-  public GlobalSession(String userAPI,
-                       String serverURL,
-                       String username,
-                       String password,
-                       String prefsPath,
-                       KeyBundle syncKeyBundle,
-                       GlobalSessionCallback callback,
+  public GlobalSession(SyncConfiguration config,
+                       BaseGlobalSessionCallback callback,
                        Context context,
                        Bundle extras,
-                       ClientsDataDelegate clientsDelegate)
-                           throws SyncConfigurationException, IllegalArgumentException, IOException, ParseException, NonObjectJSONException {
+                       ClientsDataDelegate clientsDelegate, NodeAssignmentCallback nodeAssignmentCallback)
+    throws SyncConfigurationException, IllegalArgumentException, IOException, ParseException, NonObjectJSONException {
+
     if (callback == null) {
       throw new IllegalArgumentException("Must provide a callback to GlobalSession constructor.");
     }
 
-    if (anyInvalidStrings(username, password)) {
-      throw new SyncConfigurationException();
-    }
-
-    Logger.info(LOG_TAG, "GlobalSession initialized with bundle " + extras);
-    URI serverURI;
-    try {
-      serverURI = (serverURL == null) ? null : new URI(serverURL);
-    } catch (URISyntaxException e) {
-      throw new SyncConfigurationException();
-    }
-
-    if (syncKeyBundle == null ||
-        syncKeyBundle.getEncryptionKey() == null ||
-        syncKeyBundle.getHMACKey() == null) {
-      throw new SyncConfigurationException();
-    }
+    Logger.debug(LOG_TAG, "GlobalSession initialized with bundle " + extras);
 
     this.callback        = callback;
     this.context         = context;
     this.clientsDelegate = clientsDelegate;
+    this.nodeAssignmentCallback = nodeAssignmentCallback;
 
-    config = new SyncConfiguration(prefsPath, this);
-    config.userAPI       = userAPI;
-    config.serverURL     = serverURI;
-    config.username      = username;
-    config.password      = password;
-    config.syncKeyBundle = syncKeyBundle;
-
+    this.config = config;
     registerCommands();
     prepareStages();
 
+    Collection<String> knownStageNames = SyncConfiguration.validEngineNames();
+    config.stagesToSync = Utils.getStagesToSyncFromBundle(knownStageNames, extras);
+
     // TODO: data-driven plan for the sync, referring to prepareStages.
   }
 
-  protected void registerCommands() {
-    CommandProcessor processor = CommandProcessor.getProcessor();
+  /**
+   * Register commands this global session knows how to process.
+   * <p>
+   * Re-registering a command overwrites any existing registration.
+   */
+  protected static void registerCommands() {
+    final CommandProcessor processor = CommandProcessor.getProcessor();
 
-    processor.registerCommand("resetEngine", new CommandRunner() {
+    processor.registerCommand("resetEngine", new CommandRunner(1) {
       @Override
-      public void executeCommand(List<String> args) {
+      public void executeCommand(final GlobalSession session, List<String> args) {
         HashSet<String> names = new HashSet<String>();
         names.add(args.get(0));
-        resetStagesByName(names);
+        session.resetStagesByName(names);
       }
     });
 
-    processor.registerCommand("resetAll", new CommandRunner() {
+    processor.registerCommand("resetAll", new CommandRunner(0) {
       @Override
-      public void executeCommand(List<String> args) {
-        resetAllStages();
+      public void executeCommand(final GlobalSession session, List<String> args) {
+        session.resetAllStages();
       }
     });
 
-    processor.registerCommand("wipeEngine", new CommandRunner() {
+    processor.registerCommand("wipeEngine", new CommandRunner(1) {
       @Override
-      public void executeCommand(List<String> args) {
+      public void executeCommand(final GlobalSession session, List<String> args) {
         HashSet<String> names = new HashSet<String>();
         names.add(args.get(0));
-        wipeStagesByName(names);
+        session.wipeStagesByName(names);
       }
     });
 
-    processor.registerCommand("wipeAll", new CommandRunner() {
+    processor.registerCommand("wipeAll", new CommandRunner(0) {
       @Override
-      public void executeCommand(List<String> args) {
-        wipeAllStages();
+      public void executeCommand(final GlobalSession session, List<String> args) {
+        session.wipeAllStages();
       }
     });
 
-    processor.registerCommand("displayURI", new CommandRunner() {
+    processor.registerCommand("displayURI", new CommandRunner(3) {
       @Override
-      public void executeCommand(List<String> args) {
-        CommandProcessor.getProcessor().displayURI(args, getContext());
+      public void executeCommand(final GlobalSession session, List<String> args) {
+        CommandProcessor.displayURI(args, session.getContext());
       }
     });
   }
@@ -212,21 +179,21 @@ public class GlobalSession implements CredentialsSource, PrefsSource, HttpRespon
   protected void prepareStages() {
     HashMap<Stage, GlobalSyncStage> stages = new HashMap<Stage, GlobalSyncStage>();
 
-    stages.put(Stage.checkPreconditions,      new CheckPreconditionsStage(this));
-    stages.put(Stage.ensureClusterURL,        new EnsureClusterURLStage(this));
-    stages.put(Stage.fetchInfoCollections,    new FetchInfoCollectionsStage(this));
-    stages.put(Stage.fetchMetaGlobal,         new FetchMetaGlobalStage(this));
-    stages.put(Stage.ensureKeysStage,         new EnsureCrypto5KeysStage(this));
-    stages.put(Stage.syncClientsEngine,       new SyncClientsEngineStage(this));
+    stages.put(Stage.checkPreconditions,      new CheckPreconditionsStage());
+    stages.put(Stage.ensureClusterURL,        new EnsureClusterURLStage(nodeAssignmentCallback));
+    stages.put(Stage.fetchInfoCollections,    new FetchInfoCollectionsStage());
+    stages.put(Stage.fetchMetaGlobal,         new FetchMetaGlobalStage());
+    stages.put(Stage.ensureKeysStage,         new EnsureCrypto5KeysStage());
+    stages.put(Stage.syncClientsEngine,       new SyncClientsEngineStage());
 
-    stages.put(Stage.syncTabs,                new FennecTabsServerSyncStage(this));
-    stages.put(Stage.syncPasswords,           new PasswordsServerSyncStage(this));
-    stages.put(Stage.syncBookmarks,           new AndroidBrowserBookmarksServerSyncStage(this));
-    stages.put(Stage.syncHistory,             new AndroidBrowserHistoryServerSyncStage(this));
-    stages.put(Stage.syncFormHistory,         new FormHistoryServerSyncStage(this));
+    stages.put(Stage.syncTabs,                new FennecTabsServerSyncStage());
+    stages.put(Stage.syncPasswords,           new PasswordsServerSyncStage());
+    stages.put(Stage.syncBookmarks,           new AndroidBrowserBookmarksServerSyncStage());
+    stages.put(Stage.syncHistory,             new AndroidBrowserHistoryServerSyncStage());
+    stages.put(Stage.syncFormHistory,         new FormHistoryServerSyncStage());
 
-    stages.put(Stage.uploadMetaGlobal,        new UploadMetaGlobalStage(this));
-    stages.put(Stage.completed,               new CompletedStage(this));
+    stages.put(Stage.uploadMetaGlobal,        new UploadMetaGlobalStage());
+    stages.put(Stage.completed,               new CompletedStage());
 
     this.stages = Collections.unmodifiableMap(stages);
   }
@@ -304,7 +271,7 @@ public class GlobalSession implements CredentialsSource, PrefsSource, HttpRespon
     this.currentState = next;
     Logger.info(LOG_TAG, "Running next stage " + next + " (" + nextStage + ")...");
     try {
-      nextStage.execute();
+      nextStage.execute(this);
     } catch (Exception ex) {
       Logger.warn(LOG_TAG, "Caught exception " + ex + " running stage " + next);
       this.abort(ex, "Uncaught exception in stage.");
@@ -350,15 +317,23 @@ public class GlobalSession implements CredentialsSource, PrefsSource, HttpRespon
    */
   protected void restart() throws AlreadySyncingException {
     this.currentState = GlobalSyncStage.Stage.idle;
-    if (callback.shouldBackOff()) {
+    if (callback.shouldBackOffStorage()) {
       this.callback.handleAborted(this, "Told to back off.");
       return;
     }
     this.start();
   }
 
-  public void completeSync() {
+  /**
+   * We're finished (aborted or succeeded): release resources.
+   */
+  protected void cleanUp() {
     uninstallAsHttpResponseObserver();
+    this.stages = null;
+  }
+
+  public void completeSync() {
+    cleanUp();
     this.currentState = GlobalSyncStage.Stage.idle;
     this.callback.handleSuccess(this);
   }
@@ -370,29 +345,46 @@ public class GlobalSession implements CredentialsSource, PrefsSource, HttpRespon
    * @param engineName engine to update.
    * @param engineSettings new syncID and version.
    */
-  public void updateMetaGlobalWith(String engineName, EngineSettings engineSettings) {
+  public void recordForMetaGlobalUpdate(String engineName, EngineSettings engineSettings) {
     enginesToUpdate.put(engineName, engineSettings);
   }
 
+  /**
+   * Record that an updated meta/global record should be uploaded without the
+   * given engine name.
+   *
+   * @param engineName
+   *          engine to remove.
+   */
+  public void removeEngineFromMetaGlobal(String engineName) {
+    enginesToUpdate.put(engineName, null);
+  }
+
   public boolean hasUpdatedMetaGlobal() {
     if (enginesToUpdate.isEmpty()) {
       Logger.info(LOG_TAG, "Not uploading updated meta/global record since there are no engines requesting upload.");
       return false;
     }
 
-    if (Logger.logVerbose(LOG_TAG)) {
-      Logger.trace(LOG_TAG, "Uploading updated meta/global record since there are engines requesting upload: " +
-          Utils.toCommaSeparatedString(enginesToUpdate.keySet()));
+    if (Logger.shouldLogVerbose(LOG_TAG)) {
+      Logger.trace(LOG_TAG, "Uploading updated meta/global record since there are engine changes to meta/global.");
+      Logger.trace(LOG_TAG, "Engines requesting update [" + Utils.toCommaSeparatedString(enginesToUpdate.keySet()) + "]");
     }
 
     return true;
   }
 
   public void updateMetaGlobalInPlace() {
+    config.metaGlobal.declined = this.declinedEngineNames();
     ExtendedJSONObject engines = config.metaGlobal.getEngines();
     for (Entry<String, EngineSettings> pair : enginesToUpdate.entrySet()) {
-      engines.put(pair.getKey(), pair.getValue().toJSONObject());
+      if (pair.getValue() == null) {
+        engines.remove(pair.getKey());
+      } else {
+        engines.put(pair.getKey(), pair.getValue().toJSONObject());
+      }
     }
+
     enginesToUpdate.clear();
   }
 
@@ -414,6 +406,11 @@ public class GlobalSession implements CredentialsSource, PrefsSource, HttpRespon
           @Override
           public void handleSuccess(MetaGlobal global, SyncStorageResponse response) {
             Logger.info(LOG_TAG, "Successfully uploaded updated meta/global record.");
+            // Engine changes are stored as diffs, so update enabled engines in config to match uploaded meta/global.
+            config.enabledEngineNames = config.metaGlobal.getEnabledEngineNames();
+            // Clear userSelectedEngines because they are updated in config and meta/global.
+            config.userSelectedEngines = null;
+
             synchronized (monitor) {
               monitor.notify();
             }
@@ -461,7 +458,7 @@ public class GlobalSession implements CredentialsSource, PrefsSource, HttpRespon
 
   public void abort(Exception e, String reason) {
     Logger.warn(LOG_TAG, "Aborting sync: " + reason, e);
-    uninstallAsHttpResponseObserver();
+    cleanUp();
     long existingBackoff = largestBackoffObserved.get();
     if (existingBackoff > 0) {
       callback.requestBackoff(existingBackoff);
@@ -529,11 +526,9 @@ public class GlobalSession implements CredentialsSource, PrefsSource, HttpRespon
     }
   }
 
-  public void fetchInfoCollections(InfoCollectionsDelegate callback) throws URISyntaxException {
-    if (this.config.infoCollections == null) {
-      this.config.infoCollections = new InfoCollections(config.infoURL(), credentials());
-    }
-    this.config.infoCollections.fetch(callback);
+  public void fetchInfoCollections(JSONRecordFetchDelegate callback) throws URISyntaxException {
+    final JSONRecordFetcher fetcher = new JSONRecordFetcher(config.infoCollectionsURL(), getAuthHeaderProvider());
+    fetcher.fetch(callback);
   }
 
   /**
@@ -547,7 +542,6 @@ public class GlobalSession implements CredentialsSource, PrefsSource, HttpRespon
   public void uploadKeys(final CollectionKeys keys,
                          final KeyUploadDelegate keyUploadDelegate) {
     SyncStorageRecordRequest request;
-    final GlobalSession self = this;
     try {
       request = new SyncStorageRecordRequest(this.config.keysURI());
     } catch (URISyntaxException e) {
@@ -572,7 +566,7 @@ public class GlobalSession implements CredentialsSource, PrefsSource, HttpRespon
       @Override
       public void handleRequestFailure(SyncStorageResponse response) {
         Logger.debug(LOG_TAG, "Failed to upload keys.");
-        self.interpretHTTPFailure(response.httpResponse());
+        GlobalSession.this.interpretHTTPFailure(response.httpResponse());
         BaseResource.consumeEntity(response); // The exception thrown should not need the body of the response.
         keyUploadDelegate.onKeyUploadFailed(new HTTPFailureException(response));
       }
@@ -584,8 +578,8 @@ public class GlobalSession implements CredentialsSource, PrefsSource, HttpRespon
       }
 
       @Override
-      public String credentials() {
-        return self.credentials();
+      public AuthHeaderProvider getAuthHeaderProvider() {
+        return GlobalSession.this.getAuthHeaderProvider();
       }
     };
 
@@ -643,16 +637,54 @@ public class GlobalSession implements CredentialsSource, PrefsSource, HttpRespon
       config.purgeCryptoKeys();
       config.syncID = remoteSyncID;
     }
+    // Compare lastModified timestamps for remote/local engine selection times.
+    Logger.debug(LOG_TAG, "Comparing local engine selection timestamp [" + config.userSelectedEnginesTimestamp + "] to server meta/global timestamp [" + config.persistedMetaGlobal().lastModified() + "].");
+    if (config.userSelectedEnginesTimestamp < config.persistedMetaGlobal().lastModified()) {
+      // Remote has later meta/global timestamp. Don't upload engine changes.
+      config.userSelectedEngines = null;
+    }
     // Persist enabled engine names.
     config.enabledEngineNames = global.getEnabledEngineNames();
     if (config.enabledEngineNames == null) {
       Logger.warn(LOG_TAG, "meta/global reported no enabled engine names!");
     } else {
-      if (Logger.logVerbose(LOG_TAG)) {
+      if (Logger.shouldLogVerbose(LOG_TAG)) {
         Logger.trace(LOG_TAG, "Persisting enabled engine names '" +
             Utils.toCommaSeparatedString(config.enabledEngineNames) + "' from meta/global.");
       }
     }
+
+    // Persist declined.
+    // Our declined engines at any point are:
+    // Whatever they were remotely, plus whatever they were locally, less any
+    // engines that were just enabled locally or remotely.
+    // If remote just 'won', our recently enabled list just got cleared.
+    final HashSet<String> allDeclined = new HashSet<String>();
+
+    final Set<String> newRemoteDeclined = global.getDeclinedEngineNames();
+    final Set<String> oldLocalDeclined = config.declinedEngineNames;
+
+    allDeclined.addAll(newRemoteDeclined);
+    allDeclined.addAll(oldLocalDeclined);
+
+    if (config.userSelectedEngines != null) {
+      for (Entry<String, Boolean> selection : config.userSelectedEngines.entrySet()) {
+        if (selection.getValue()) {
+          allDeclined.remove(selection.getKey());
+        }
+      }
+    }
+
+    config.declinedEngineNames = allDeclined;
+    if (config.declinedEngineNames.isEmpty()) {
+      Logger.debug(LOG_TAG, "meta/global reported no declined engine names, and we have none declined locally.");
+    } else {
+      if (Logger.shouldLogVerbose(LOG_TAG)) {
+        Logger.trace(LOG_TAG, "Persisting declined engine names '" +
+            Utils.toCommaSeparatedString(config.declinedEngineNames) + "' from meta/global.");
+      }
+    }
+
     config.persistToPrefs();
     advance();
   }
@@ -705,7 +737,7 @@ public class GlobalSession implements CredentialsSource, PrefsSource, HttpRespon
 
     final MetaGlobal mg = session.generateNewMetaGlobal();
 
-    session.wipeServer(session, new WipeServerDelegate() {
+    session.wipeServer(session.getAuthHeaderProvider(), new WipeServerDelegate() {
 
       @Override
       public void onWiped(long timestamp) {
@@ -805,12 +837,12 @@ public class GlobalSession implements CredentialsSource, PrefsSource, HttpRespon
   // reset client to prompt reupload.
   // If sync ID mismatch: take that syncID and reset client.
 
-  protected void wipeServer(final CredentialsSource credentials, final WipeServerDelegate wipeDelegate) {
+  protected void wipeServer(final AuthHeaderProvider authHeaderProvider, final WipeServerDelegate wipeDelegate) {
     SyncStorageRequest request;
     final GlobalSession self = this;
 
     try {
-      request = new SyncStorageRequest(config.storageURL(false));
+      request = new SyncStorageRequest(config.storageURL());
     } catch (URISyntaxException ex) {
       Logger.warn(LOG_TAG, "Invalid URI in wipeServer.");
       wipeDelegate.onWipeFailed(ex);
@@ -846,8 +878,8 @@ public class GlobalSession implements CredentialsSource, PrefsSource, HttpRespon
       }
 
       @Override
-      public String credentials() {
-        return credentials.credentials();
+      public AuthHeaderProvider getAuthHeaderProvider() {
+        return GlobalSession.this.getAuthHeaderProvider();
       }
     };
     request.delete();
@@ -859,11 +891,11 @@ public class GlobalSession implements CredentialsSource, PrefsSource, HttpRespon
     this.wipeStagesByEnum(Stage.getNamedStages());
   }
 
-  public static void wipeStages(Collection<GlobalSyncStage> stages) {
+  public void wipeStages(Collection<GlobalSyncStage> stages) {
     for (GlobalSyncStage stage : stages) {
       try {
         Logger.info(LOG_TAG, "Wiping " + stage);
-        stage.wipeLocal();
+        stage.wipeLocal(this);
       } catch (Exception e) {
         Logger.error(LOG_TAG, "Ignoring wipe failure for stage " + stage, e);
       }
@@ -871,11 +903,11 @@ public class GlobalSession implements CredentialsSource, PrefsSource, HttpRespon
   }
 
   public void wipeStagesByEnum(Collection<Stage> stages) {
-    GlobalSession.wipeStages(this.getSyncStagesByEnum(stages));
+    wipeStages(this.getSyncStagesByEnum(stages));
   }
 
   public void wipeStagesByName(Collection<String> names) {
-    GlobalSession.wipeStages(this.getSyncStagesByName(names));
+    wipeStages(this.getSyncStagesByName(names));
   }
 
   public void resetAllStages() {
@@ -884,11 +916,11 @@ public class GlobalSession implements CredentialsSource, PrefsSource, HttpRespon
     this.resetStagesByEnum(Stage.getNamedStages());
   }
 
-  public static void resetStages(Collection<GlobalSyncStage> stages) {
+  public void resetStages(Collection<GlobalSyncStage> stages) {
     for (GlobalSyncStage stage : stages) {
       try {
         Logger.info(LOG_TAG, "Resetting " + stage);
-        stage.resetLocal();
+        stage.resetLocal(this);
       } catch (Exception e) {
         Logger.error(LOG_TAG, "Ignoring reset failure for stage " + stage, e);
       }
@@ -896,20 +928,32 @@ public class GlobalSession implements CredentialsSource, PrefsSource, HttpRespon
   }
 
   public void resetStagesByEnum(Collection<Stage> stages) {
-    GlobalSession.resetStages(this.getSyncStagesByEnum(stages));
+    resetStages(this.getSyncStagesByEnum(stages));
   }
 
   public void resetStagesByName(Collection<String> names) {
-    Collection<GlobalSyncStage> stages = new ArrayList<GlobalSyncStage>();
-    for (String name : names) {
-      try {
-        GlobalSyncStage stage = this.getSyncStageByName(name);
-        stages.add(stage);
-      } catch (NoSuchStageException e) {
-        Logger.warn(LOG_TAG, "Cannot reset stage " + name + ": no such stage.");
-      }
-    }
-    GlobalSession.resetStages(stages);
+    resetStages(this.getSyncStagesByName(names));
+  }
+
+  /**
+   * Engines to explicitly mark as declined in a fresh meta/global record.
+   * <p>
+   * Returns an empty array if the user hasn't elected to customize data types,
+   * or an array of engines that the user un-checked during customization.
+   * <p>
+   * Engines that Android Sync doesn't recognize are <b>not</b> included in
+   * the returned array.
+   *
+   * @return a new JSONArray of engine names.
+   */
+  @SuppressWarnings("unchecked")
+  protected JSONArray declinedEngineNames() {
+    final JSONArray declined = new JSONArray();
+    for (String engine : config.declinedEngineNames) {
+      declined.add(engine);
+    };
+
+    return declined;
   }
 
   /**
@@ -927,11 +971,30 @@ public class GlobalSession implements CredentialsSource, PrefsSource, HttpRespon
     if (config.enabledEngineNames != null) {
       return config.enabledEngineNames;
     }
-    Set<String> engineNames = new HashSet<String>();
-    for (Stage stage : Stage.getNamedStages()) {
-      engineNames.add(stage.getRepositoryName());
+
+    // These are the default set of engine names.
+    Set<String> validEngineNames = SyncConfiguration.validEngineNames();
+
+    // If the user hasn't set any selected engines, that's okay -- default to
+    // everything.
+    if (config.userSelectedEngines == null) {
+      return validEngineNames;
     }
-    return engineNames;
+
+    // userSelectedEngines has keys that are engine names, and boolean values
+    // corresponding to whether the user asked for the engine to sync or not. If
+    // an engine is not present, that means the user didn't change its sync
+    // setting. Since we default to everything on, that means the user didn't
+    // turn it off; therefore, it's included in the set of engines to sync.
+    Set<String> validAndSelectedEngineNames = new HashSet<String>();
+    for (String engineName : validEngineNames) {
+      if (config.userSelectedEngines.containsKey(engineName) &&
+          !config.userSelectedEngines.get(engineName)) {
+        continue;
+      }
+      validAndSelectedEngineNames.add(engineName);
+    }
+    return validAndSelectedEngineNames;
   }
 
   /**
@@ -939,6 +1002,7 @@ public class GlobalSession implements CredentialsSource, PrefsSource, HttpRespon
    * @return crypto/keys collection.
    * @throws CryptoException
    */
+  @SuppressWarnings("static-method")
   public CollectionKeys generateNewCryptoKeys() throws CryptoException {
     return CollectionKeys.generateCollectionKeys();
   }
@@ -950,7 +1014,6 @@ public class GlobalSession implements CredentialsSource, PrefsSource, HttpRespon
   public MetaGlobal generateNewMetaGlobal() {
     final String newSyncID   = Utils.generateGuid();
     final String metaURL     = this.config.metaURL();
-    final String credentials = this.credentials();
 
     ExtendedJSONObject engines = new ExtendedJSONObject();
     for (String engineName : enabledEngineNames()) {
@@ -970,11 +1033,15 @@ public class GlobalSession implements CredentialsSource, PrefsSource, HttpRespon
       engines.put(engineName, engineSettings.toJSONObject());
     }
 
-    MetaGlobal metaGlobal = new MetaGlobal(metaURL, credentials);
+    MetaGlobal metaGlobal = new MetaGlobal(metaURL, this.getAuthHeaderProvider());
     metaGlobal.setSyncID(newSyncID);
     metaGlobal.setStorageVersion(STORAGE_VERSION);
     metaGlobal.setEngines(engines);
 
+    // We assume that the config's declined engines have been updated
+    // according to the user's selections.
+    metaGlobal.setDeclinedEngineNames(this.declinedEngineNames());
+
     return metaGlobal;
   }
 
