@@ -31,6 +31,7 @@ let AboutReader = function(doc, win) {
   Services.obs.addObserver(this, "Reader:FaviconReturn", false);
   Services.obs.addObserver(this, "Reader:Add", false);
   Services.obs.addObserver(this, "Reader:Remove", false);
+  Services.obs.addObserver(this, "Reader:ListStatusReturn", false);
 
   this._article = null;
 
@@ -58,47 +59,66 @@ let AboutReader = function(doc, win) {
 
   this._setupAllDropdowns();
   this._setupButton("toggle-button", this._onReaderToggle.bind(this));
-  this._setupButton("list-button", this._onList.bind(this));
   this._setupButton("share-button", this._onShare.bind(this));
 
   let colorSchemeOptions = [
+    { name: gStrings.GetStringFromName("aboutReader.colorSchemeDark"),
+      value: "dark"},
     { name: gStrings.GetStringFromName("aboutReader.colorSchemeLight"),
       value: "light"},
-    { name: gStrings.GetStringFromName("aboutReader.colorSchemeDark"),
-      value: "dark"}
+    { name: gStrings.GetStringFromName("aboutReader.colorSchemeAuto"),
+      value: "auto"}
   ];
 
   let colorScheme = Services.prefs.getCharPref("reader.color_scheme");
-  this._setupSegmentedButton("color-scheme-buttons", colorSchemeOptions, colorScheme, this._setColorScheme.bind(this));
-  this._setColorScheme(colorScheme);
+  this._setupSegmentedButton("color-scheme-buttons", colorSchemeOptions, colorScheme, this._setColorSchemePref.bind(this));
+  this._setColorSchemePref(colorScheme);
 
+  let fontTypeSample = gStrings.GetStringFromName("aboutReader.fontTypeSample");
   let fontTypeOptions = [
-    { name: gStrings.GetStringFromName("aboutReader.fontTypeSansSerif"),
-      value: "sans-serif"},
-    { name: gStrings.GetStringFromName("aboutReader.fontTypeSerif"),
-      value: "serif"}
+    { name: fontTypeSample,
+      description: gStrings.GetStringFromName("aboutReader.fontTypeSerif"),
+      value: "serif",
+      linkClass: "serif" },
+    { name: fontTypeSample,
+      description: gStrings.GetStringFromName("aboutReader.fontTypeSansSerif"),
+      value: "sans-serif",
+      linkClass: "sans-serif"
+    },
   ];
 
   let fontType = Services.prefs.getCharPref("reader.font_type");
   this._setupSegmentedButton("font-type-buttons", fontTypeOptions, fontType, this._setFontType.bind(this));
   this._setFontType(fontType);
 
-  let fontTitle = gStrings.GetStringFromName("aboutReader.textTitle");
-  this._fontSize = 0;
-  this._setupStepControl("font-size-control", fontTitle,
-    this._FONT_SIZE_MIN, this._FONT_SIZE_MAX, this._FONT_SIZE_STEP, Services.prefs.getIntPref("reader.font_size"),
-    this._onFontSizeChange.bind(this));
+  let fontSizeSample = gStrings.GetStringFromName("aboutReader.fontSizeSample");
+  let fontSizeOptions = [
+    { name: fontSizeSample,
+      value: 1,
+      linkClass: "font-size1-sample" },
+    { name: fontSizeSample,
+      value: 2,
+      linkClass: "font-size2-sample" },
+    { name: fontSizeSample,
+      value: 3,
+      linkClass: "font-size3-sample" },
+    { name: fontSizeSample,
+      value: 4,
+      linkClass: "font-size4-sample" },
+    { name: fontSizeSample,
+      value: 5,
+      linkClass: "font-size5-sample" }
+  ];
 
-  let marginTitle = gStrings.GetStringFromName("aboutReader.marginTitle");
-  this._marginSize = 0;
-  this._setupStepControl("margin-size-control", marginTitle,
-    this._MARGIN_SIZE_MIN, this._MARGIN_SIZE_MAX, this._MARGIN_SIZE_STEP, Services.prefs.getIntPref("reader.margin_size"),
-    this._onMarginSizeChange.bind(this));
+  let fontSize = Services.prefs.getIntPref("reader.font_size");
+  this._setupSegmentedButton("font-size-buttons", fontSizeOptions, fontSize, this._setFontSize.bind(this));
+  this._setFontSize(fontSize);
 
   dump("Decoding query arguments");
   let queryArgs = this._decodeQueryString(win.location.href);
 
-  this._isReadingListItem = (queryArgs.readingList == "1");
+  // Track status of reader toolbar add/remove toggle button
+  this._isReadingListItem = -1;
   this._updateToggleButton();
 
   let url = queryArgs.url;
@@ -113,13 +133,6 @@ let AboutReader = function(doc, win) {
 }
 
 AboutReader.prototype = {
-  _FONT_SIZE_MIN: 1,
-  _FONT_SIZE_MAX: 7,
-  _FONT_SIZE_STEP: 1,
-  _MARGIN_SIZE_MIN: 5,
-  _MARGIN_SIZE_MAX: 25,
-  _MARGIN_SIZE_STEP: 5,
-
   _BLOCK_IMAGES_SELECTOR: ".content p > img:only-child, " +
                           ".content p > a:only-child > img:only-child, " +
                           ".content .wp-caption img, " +
@@ -173,8 +186,8 @@ AboutReader.prototype = {
       case "Reader:Add": {
         let args = JSON.parse(aData);
         if (args.url == this._article.url) {
-          if (!this._isReadingListItem) {
-            this._isReadingListItem = true;
+          if (this._isReadingListItem != 1) {
+            this._isReadingListItem = 1;
             this._updateToggleButton();
           }
         }
@@ -183,13 +196,30 @@ AboutReader.prototype = {
 
       case "Reader:Remove": {
         if (aData == this._article.url) {
-          if (this._isReadingListItem) {
-            this._isReadingListItem = false;
+          if (this._isReadingListItem != 0) {
+            this._isReadingListItem = 0;
             this._updateToggleButton();
           }
         }
         break;
       }
+
+      case "Reader:ListStatusReturn": {
+        let args = JSON.parse(aData);
+        if (args.url == this._article.url) {
+          if (this._isReadingListItem != args.inReadingList) {
+            let isInitialStateChange = (this._isReadingListItem == -1);
+            this._isReadingListItem = args.inReadingList;
+            this._updateToggleButton();
+
+            // Display the toolbar when all its initial component states are known
+            if (isInitialStateChange) {
+              this._setToolbarVisibility(true);
+            }
+          }
+        }
+        break;
+      }
     }
   },
 
@@ -207,8 +237,9 @@ AboutReader.prototype = {
         break;
       case "scroll":
         if (!this._scrolled) {
-          this._scrolled = true;
-          this._setToolbarVisibility(false);
+          let isScrollingUp = this._scrollOffset > aEvent.pageY;
+          this._setToolbarVisibility(isScrollingUp);
+          this._scrollOffset = aEvent.pageY;
         }
         break;
       case "popstate":
@@ -219,9 +250,14 @@ AboutReader.prototype = {
         this._updateImageMargins();
         break;
 
+      case "devicelight":
+        this._handleDeviceLight(aEvent.value);
+        break;
+
       case "unload":
         Services.obs.removeObserver(this, "Reader:Add");
         Services.obs.removeObserver(this, "Reader:Remove");
+        Services.obs.removeObserver(this, "Reader:ListStatusReturn");
         break;
     }
   },
@@ -229,21 +265,28 @@ AboutReader.prototype = {
   _updateToggleButton: function Reader_updateToggleButton() {
     let classes = this._doc.getElementById("toggle-button").classList;
 
-    if (this._isReadingListItem) {
+    if (this._isReadingListItem == 1) {
       classes.add("on");
     } else {
       classes.remove("on");
     }
   },
 
+  _requestReadingListStatus: function Reader_requestReadingListStatus() {
+    gChromeWin.sendMessageToJava({
+      type: "Reader:ListStatusRequest",
+      url: this._article.url
+    });
+  },
+
   _onReaderToggle: function Reader_onToggle() {
     if (!this._article)
       return;
 
-    this._isReadingListItem = !this._isReadingListItem;
+    this._isReadingListItem = (this._isReadingListItem == 1) ? 0 : 1;
     this._updateToggleButton();
 
-    if (this._isReadingListItem) {
+    if (this._isReadingListItem == 1) {
       gChromeWin.Reader.storeArticleInCache(this._article, function(success) {
         dump("Reader:Add (in reader) success=" + success);
 
@@ -258,29 +301,18 @@ AboutReader.prototype = {
           result: result,
           title: this._article.title,
           url: this._article.url,
+          length: this._article.length,
+          excerpt: this._article.excerpt
         });
       }.bind(this));
     } else {
-      gChromeWin.Reader.removeArticleFromCache(this._article.url , function(success) {
-        dump("Reader:Remove (in reader) success=" + success);
-
-        Services.obs.notifyObservers(null, "Reader:Remove", this._article.url);
-
-        gChromeWin.sendMessageToJava({
-          type: "Reader:Removed",
-          url: this._article.url
-        });
-      }.bind(this));
+      // In addition to removing the article from the cache (handled in
+      // browser.js), sending this message will cause the toggle button to be
+      // updated (handled in this file).
+      Services.obs.notifyObservers(null, "Reader:Remove", this._article.url);
     }
   },
 
-  _onList: function Reader_onList() {
-    if (!this._article)
-      return;
-
-    gChromeWin.sendMessageToJava({ type: "Reader:GoToReadingList" });
-  },
-
   _onShare: function Reader_onShare() {
     if (!this._article)
       return;
@@ -292,19 +324,7 @@ AboutReader.prototype = {
     });
   },
 
-  _onMarginSizeChange: function Reader_onMarginSizeChange(newMarginSize) {
-    let doc = this._doc;
-
-    this._marginSize = newMarginSize;
-    doc.body.style.marginLeft = this._marginSize + "%";
-    doc.body.style.marginRight = this._marginSize + "%";
-
-    this._updateImageMargins();
-
-    Services.prefs.setIntPref("reader.margin_size", this._marginSize);
-  },
-
-  _onFontSizeChange: function Reader_onFontSizeChange(newFontSize) {
+  _setFontSize: function Reader_setFontSize(newFontSize) {
     let bodyClasses = this._doc.body.classList;
 
     if (this._fontSize > 0)
@@ -316,6 +336,51 @@ AboutReader.prototype = {
     Services.prefs.setIntPref("reader.font_size", this._fontSize);
   },
 
+  _handleDeviceLight: function Reader_handleDeviceLight(newLux) {
+    // Desired size of the this._luxValues array.
+    let luxValuesSize = 10;
+    // Add new lux value at the front of the array.
+    this._luxValues.unshift(newLux);
+    // Add new lux value to this._totalLux for averaging later.
+    this._totalLux += newLux;
+
+    // Don't update when length of array is less than luxValuesSize except when it is 1.
+    if (this._luxValues.length < luxValuesSize) {
+      // Use the first lux value to set the color scheme until our array equals luxValuesSize.
+      if (this._luxValues.length == 1) {
+        this._updateColorScheme(newLux);
+      }
+      return;
+    }
+    // Holds the average of the lux values collected in this._luxValues.
+    let averageLuxValue = this._totalLux/luxValuesSize;
+
+    this._updateColorScheme(averageLuxValue);
+    // Pop the oldest value off the array.
+    let oldLux = this._luxValues.pop();
+    // Subtract oldLux since it has been discarded from the array.
+    this._totalLux -= oldLux;
+  },
+
+  _updateColorScheme: function Reader_updateColorScheme(luxValue) {
+    // Upper bound value for "dark" color scheme beyond which it changes to "light".
+    let upperBoundDark = 50;
+    // Lower bound value for "light" color scheme beyond which it changes to "dark".
+    let lowerBoundLight = 10;
+    // Threshold for color scheme change.
+    let colorChangeThreshold = 20;
+
+    // Ignore changes that are within a certain threshold of previous lux values.
+    if ((this._colorScheme === "dark" && luxValue < upperBoundDark) ||
+        (this._colorScheme === "light" && luxValue > lowerBoundLight))
+      return;
+
+    if (luxValue < colorChangeThreshold)
+      this._setColorScheme("dark");
+    else
+      this._setColorScheme("light");
+  },
+
   _setColorScheme: function Reader_setColorScheme(newColorScheme) {
     if (this._colorScheme === newColorScheme)
       return;
@@ -327,8 +392,23 @@ AboutReader.prototype = {
 
     this._colorScheme = newColorScheme;
     bodyClasses.add(this._colorScheme);
+  },
 
-    Services.prefs.setCharPref("reader.color_scheme", this._colorScheme);
+  // Pref values include "dark", "light", and "auto", which automatically switches
+  // between light and dark color schemes based on the ambient light level.
+  _setColorSchemePref: function Reader_setColorSchemePref(colorSchemePref) {
+    if (colorSchemePref === "auto") {
+      this._win.addEventListener("devicelight", this, false);
+      this._luxValues = [];
+      this._totalLux = 0;
+    } else {
+      this._win.removeEventListener("devicelight", this, false);
+      this._setColorScheme(colorSchemePref);
+      delete this._luxValues;
+      delete this._totalLux;
+    }
+
+    Services.prefs.setCharPref("reader.color_scheme", colorSchemePref);
   },
 
   _setFontType: function Reader_setFontType(newFontType) {
@@ -358,10 +438,15 @@ AboutReader.prototype = {
     if (!this._toolbarEnabled)
       return;
 
+    // Don't allow visible toolbar until banner state is known
+    if (this._isReadingListItem == -1)
+      return;
+
     if (this._getToolbarVisibility() === visible)
       return;
 
     this._toolbarElement.classList.toggle("toolbar-hidden");
+    this._setSystemUIVisibility(visible);
 
     if (!visible && !this._hasUsedToolbar) {
       this._hasUsedToolbar = Services.prefs.getBoolPref("reader.has_used_toolbar");
@@ -378,6 +463,13 @@ AboutReader.prototype = {
     this._setToolbarVisibility(!this._getToolbarVisibility());
   },
 
+  _setSystemUIVisibility: function Reader_setSystemUIVisibility(visible) {
+    gChromeWin.sendMessageToJava({
+      type: "SystemUI:Visibility",
+      visible: visible
+    });
+  },
+
   _loadFromURL: function Reader_loadFromURL(url) {
     this._showProgressDelayed();
 
@@ -483,15 +575,31 @@ AboutReader.prototype = {
     this._doc.title = error;
   },
 
+  // This function is the JS version of Java's StringUtils.stripCommonSubdomains.
+  _stripHost: function Reader_stripHost(host) {
+    if (!host)
+      return host;
+
+    let start = 0;
+
+    if (host.startsWith("www."))
+      start = 4;
+    else if (host.startsWith("m."))
+      start = 2;
+    else if (host.startsWith("mobile."))
+      start = 7;
+
+    return host.substring(start);
+  },
+
   _showContent: function Reader_showContent(article) {
     this._messageElement.style.display = "none";
 
     this._article = article;
 
+    this._domainElement.href = article.url;
     let articleUri = Services.io.newURI(article.url, null, null);
-    let domain = articleUri.host;
-
-    this._domainElement.innerHTML = domain;
+    this._domainElement.innerHTML = this._stripHost(articleUri.host);
 
     this._creditsElement.innerHTML = article.byline;
 
@@ -509,6 +617,7 @@ AboutReader.prototype = {
     this._maybeSetTextDirection(article);
 
     this._contentElement.style.display = "block";
+    this._requestReadingListStatus();
 
     this._toolbarEnabled = true;
     this._setToolbarVisibility(true);
@@ -550,69 +659,6 @@ AboutReader.prototype = {
     return result;
   },
 
-  _setupStepControl: function Reader_setupStepControl(id, name, min, max, step, initial, callback) {
-    let doc = this._doc;
-    let stepControl = doc.getElementById(id);
-
-    let title = this._doc.createElement("h1");
-    title.innerHTML = name;
-    stepControl.appendChild(title);
-
-    let plusButton = doc.createElement("div");
-    plusButton.className = "button plus-button";
-    stepControl.appendChild(plusButton);
-
-    let minusButton = doc.createElement("div");
-    minusButton.className = "button minus-button";
-    stepControl.appendChild(minusButton);
-
-    let updateControls = function() {
-      current = Math.max(min, Math.min(max, current));
-      if (current == min) {
-        minusButton.classList.add("disabled");
-      } else {
-        minusButton.classList.remove("disabled");
-      }
-      if (current == max) {
-        plusButton.classList.add("disabled");
-      } else {
-        plusButton.classList.remove("disabled");
-      }
-    }
-
-    let current = initial;
-    updateControls();
-
-    plusButton.addEventListener("click", function(aEvent) {
-      if (!aEvent.isTrusted)
-        return;
-
-      aEvent.stopPropagation();
-
-      if (current < max) {
-        current += step;
-        updateControls();
-        callback(current);
-      }
-    }.bind(this), true);
-
-    minusButton.addEventListener("click", function(aEvent) {
-      if (!aEvent.isTrusted)
-        return;
-
-      aEvent.stopPropagation();
-
-      if (current > min) {
-        current -= step;
-        updateControls();
-        callback(current);
-      }
-    }.bind(this), true);
-
-    // Always callback initial current setting
-    callback(current);
-  },
-
   _setupSegmentedButton: function Reader_setupSegmentedButton(id, options, initialValue, callback) {
     let doc = this._doc;
     let segmentedButton = doc.getElementById(id);
@@ -622,9 +668,18 @@ AboutReader.prototype = {
 
       let item = doc.createElement("li");
       let link = doc.createElement("a");
-      link.innerHTML = option.name;
+      link.textContent = option.name;
       item.appendChild(link);
 
+      if (option.linkClass !== undefined)
+        link.classList.add(option.linkClass);
+
+      if (option.description !== undefined) {
+        let description = doc.createElement("div");
+        description.textContent = option.description;
+        item.appendChild(description);
+      }
+
       link.style.MozUserSelect = 'none';
       segmentedButton.appendChild(item);
 
@@ -680,25 +735,26 @@ AboutReader.prototype = {
       dropdownPopup.appendChild(dropdownArrow);
 
       let updatePopupPosition = function() {
-          let popupWidth = dropdownPopup.offsetWidth + 30;
-          let arrowWidth = dropdownArrow.offsetWidth;
-          let toggleWidth = dropdownToggle.offsetWidth;
-          let toggleLeft = dropdownToggle.offsetLeft;
+        let popupWidth = dropdownPopup.offsetWidth + 30;
+        let arrowWidth = dropdownArrow.offsetWidth;
+        let toggleWidth = dropdownToggle.offsetWidth;
+        let toggleLeft = dropdownToggle.offsetLeft;
 
-          let popupShift = (toggleWidth - popupWidth) / 2;
-          let popupLeft = Math.max(0, Math.min(win.innerWidth - popupWidth, toggleLeft + popupShift));
-          dropdownPopup.style.left = popupLeft + "px";
+        let popupShift = (toggleWidth - popupWidth) / 2;
+        let popupLeft = Math.max(0, Math.min(win.innerWidth - popupWidth, toggleLeft + popupShift));
+        dropdownPopup.style.left = popupLeft + "px";
 
-          let arrowShift = (toggleWidth - arrowWidth) / 2;
-          let arrowLeft = toggleLeft - popupLeft + arrowShift;
-          dropdownArrow.style.left = arrowLeft + "px";
+        let arrowShift = (toggleWidth - arrowWidth) / 2;
+        let arrowLeft = toggleLeft - popupLeft + arrowShift;
+        dropdownArrow.style.left = arrowLeft + "px";
       };
 
       win.addEventListener("resize", function(aEvent) {
         if (!aEvent.isTrusted)
           return;
 
-        updatePopupPosition();
+        // Wait for reflow before calculating the new position of the popup.
+        setTimeout(updatePopupPosition, 0);
       }, true);
 
       dropdownToggle.addEventListener("click", function(aEvent) {
