@@ -29,8 +29,22 @@
 #include "nsICrashReporter.h"
 #endif
 
-// 1 MB should be enough for everyone.
-static const uint32_t MAX_PREF_LENGTH = 1 * 1024 * 1024;
+#include "nsIConsoleService.h"
+
+#ifdef DEBUG
+#define ENSURE_MAIN_PROCESS(message, pref) do {                                \
+  if (GetContentChild()) {                                                     \
+    nsPrintfCString msg("ENSURE_MAIN_PROCESS failed. %s %s", message, pref);   \
+    NS_ERROR(msg.get());                                                       \
+    return NS_ERROR_NOT_AVAILABLE;                                             \
+  }                                                                            \
+} while (0);
+#else
+#define ENSURE_MAIN_PROCESS(message, pref)                                     \
+  if (GetContentChild()) {                                                     \
+    return NS_ERROR_NOT_AVAILABLE;                                             \
+  }
+#endif
 
 // Definitions
 struct EnumerateData {
@@ -68,7 +82,6 @@ nsPrefBranch::nsPrefBranch(const char *aPrefRoot, bool aDefaultBranch)
   mPrefRootLength = mPrefRoot.Length();
   mIsDefault = aDefaultBranch;
   mFreeingObserverList = false;
-  mObservers.Init();
 
   nsCOMPtr<nsIObserverService> observerService =
     mozilla::services::GetObserverService();
@@ -95,8 +108,8 @@ nsPrefBranch::~nsPrefBranch()
  * nsISupports Implementation
  */
 
-NS_IMPL_THREADSAFE_ADDREF(nsPrefBranch)
-NS_IMPL_THREADSAFE_RELEASE(nsPrefBranch)
+NS_IMPL_ADDREF(nsPrefBranch)
+NS_IMPL_RELEASE(nsPrefBranch)
 
 NS_INTERFACE_MAP_BEGIN(nsPrefBranch)
   NS_INTERFACE_MAP_ENTRY_AMBIGUOUS(nsISupports, nsIPrefBranch)
@@ -137,11 +150,7 @@ NS_IMETHODIMP nsPrefBranch::GetBoolPref(const char *aPrefName, bool *_retval)
 
 NS_IMETHODIMP nsPrefBranch::SetBoolPref(const char *aPrefName, bool aValue)
 {
-  if (GetContentChild()) {
-    NS_ERROR("cannot set pref from content process");
-    return NS_ERROR_NOT_AVAILABLE;
-  }
-
+  ENSURE_MAIN_PROCESS("Cannot SetBoolPref from content process:", aPrefName);
   NS_ENSURE_ARG(aPrefName);
   const char *pref = getPrefName(aPrefName);
   return PREF_SetBoolPref(pref, aValue, mIsDefault);
@@ -169,11 +178,17 @@ NS_IMETHODIMP nsPrefBranch::GetCharPref(const char *aPrefName, char **_retval)
 
 NS_IMETHODIMP nsPrefBranch::SetCharPref(const char *aPrefName, const char *aValue)
 {
-  if (GetContentChild()) {
-    NS_ERROR("cannot set pref from content process");
-    return NS_ERROR_NOT_AVAILABLE;
+  nsresult rv = CheckSanityOfStringLength(aPrefName, aValue);
+  if (NS_FAILED(rv)) {
+    return rv;
   }
+  return SetCharPrefInternal(aPrefName, aValue);
+}
 
+nsresult nsPrefBranch::SetCharPrefInternal(const char *aPrefName, const char *aValue)
+
+{
+  ENSURE_MAIN_PROCESS("Cannot SetCharPref from content process:", aPrefName);
   NS_ENSURE_ARG(aPrefName);
   NS_ENSURE_ARG(aValue);
   const char *pref = getPrefName(aPrefName);
@@ -189,11 +204,7 @@ NS_IMETHODIMP nsPrefBranch::GetIntPref(const char *aPrefName, int32_t *_retval)
 
 NS_IMETHODIMP nsPrefBranch::SetIntPref(const char *aPrefName, int32_t aValue)
 {
-  if (GetContentChild()) {
-    NS_ERROR("cannot set pref from content process");
-    return NS_ERROR_NOT_AVAILABLE;
-  }
-
+  ENSURE_MAIN_PROCESS("Cannot SetIntPref from content process:", aPrefName);
   NS_ENSURE_ARG(aPrefName);
   const char *pref = getPrefName(aPrefName);
   return PREF_SetIntPref(pref, aValue, mIsDefault);
@@ -342,13 +353,43 @@ NS_IMETHODIMP nsPrefBranch::GetComplexValue(const char *aPrefName, const nsIID &
   return NS_NOINTERFACE;
 }
 
+nsresult nsPrefBranch::CheckSanityOfStringLength(const char* aPrefName, const char* aValue) {
+  if (!aValue) {
+    return NS_OK;
+  }
+  return CheckSanityOfStringLength(aPrefName, strlen(aValue));
+}
+
+nsresult nsPrefBranch::CheckSanityOfStringLength(const char* aPrefName, const nsAString& aValue) {
+  return CheckSanityOfStringLength(aPrefName, aValue.Length());
+}
+
+nsresult nsPrefBranch::CheckSanityOfStringLength(const char* aPrefName, const uint32_t aLength) {
+  if (aLength > MAX_PREF_LENGTH) {
+    return NS_ERROR_ILLEGAL_VALUE;
+  }
+  if (aLength <= MAX_ADVISABLE_PREF_LENGTH) {
+    return NS_OK;
+  }
+  nsresult rv;
+  nsCOMPtr<nsIConsoleService> console = do_GetService("@mozilla.org/consoleservice;1", &rv);
+  if (NS_FAILED(rv)) {
+    return rv;
+  }
+  nsAutoCString message(nsPrintfCString("Warning: attempting to write %d bytes to preference %s. This is bad for general performance and memory usage. Such an amount of data should rather be written to an external file.",
+                                        aLength,
+                                        aPrefName));
+  rv = console->LogStringMessage(NS_ConvertUTF8toUTF16(message).get());
+  if (NS_FAILED(rv)) {
+    return rv;
+  }
+  return NS_OK;
+}
+
+
 NS_IMETHODIMP nsPrefBranch::SetComplexValue(const char *aPrefName, const nsIID & aType, nsISupports *aValue)
 {
-  if (GetContentChild()) {
-    NS_ERROR("cannot set pref from content process");
-    return NS_ERROR_NOT_AVAILABLE;
-  }
-
+  ENSURE_MAIN_PROCESS("Cannot SetComplexValue from content process:", aPrefName);
   NS_ENSURE_ARG(aPrefName);
 
   nsresult   rv = NS_NOINTERFACE;
@@ -362,7 +403,7 @@ NS_IMETHODIMP nsPrefBranch::SetComplexValue(const char *aPrefName, const nsIID &
 
     rv = file->GetPersistentDescriptor(descriptorString);
     if (NS_SUCCEEDED(rv)) {
-      rv = SetCharPref(aPrefName, descriptorString.get());
+      rv = SetCharPrefInternal(aPrefName, descriptorString.get());
     }
     return rv;
   }
@@ -371,7 +412,7 @@ NS_IMETHODIMP nsPrefBranch::SetComplexValue(const char *aPrefName, const nsIID &
     nsCOMPtr<nsIRelativeFilePref> relFilePref = do_QueryInterface(aValue);
     if (!relFilePref)
       return NS_NOINTERFACE;
-    
+
     nsCOMPtr<nsIFile> file;
     relFilePref->GetFile(getter_AddRefs(file));
     if (!file)
@@ -391,13 +432,13 @@ NS_IMETHODIMP nsPrefBranch::SetComplexValue(const char *aPrefName, const nsIID &
     rv = file->GetRelativeDescriptor(relativeToFile, relDescriptor);
     if (NS_FAILED(rv))
       return rv;
-    
+
     nsAutoCString descriptorString;
     descriptorString.Append('[');
     descriptorString.Append(relativeToKey);
     descriptorString.Append(']');
     descriptorString.Append(relDescriptor);
-    return SetCharPref(aPrefName, descriptorString.get());
+    return SetCharPrefInternal(aPrefName, descriptorString.get());
   }
 
   if (aType.Equals(NS_GET_IID(nsISupportsString))) {
@@ -408,10 +449,12 @@ NS_IMETHODIMP nsPrefBranch::SetComplexValue(const char *aPrefName, const nsIID &
 
       rv = theString->GetData(wideString);
       if (NS_SUCCEEDED(rv)) {
-        if (wideString.Length() > MAX_PREF_LENGTH) {
-          return NS_ERROR_OUT_OF_MEMORY;
+        // Check sanity of string length before any lengthy conversion
+        rv = CheckSanityOfStringLength(aPrefName, wideString);
+        if (NS_FAILED(rv)) {
+          return rv;
         }
-        rv = SetCharPref(aPrefName, NS_ConvertUTF16toUTF8(wideString).get());
+        rv = SetCharPrefInternal(aPrefName, NS_ConvertUTF16toUTF8(wideString).get());
       }
     }
     return rv;
@@ -425,10 +468,12 @@ NS_IMETHODIMP nsPrefBranch::SetComplexValue(const char *aPrefName, const nsIID &
 
       rv = theString->GetData(getter_Copies(wideString));
       if (NS_SUCCEEDED(rv)) {
-        if (wideString.Length() > MAX_PREF_LENGTH) {
-          return NS_ERROR_OUT_OF_MEMORY;
+        // Check sanity of string length before any lengthy conversion
+        rv = CheckSanityOfStringLength(aPrefName, wideString);
+        if (NS_FAILED(rv)) {
+          return rv;
         }
-        rv = SetCharPref(aPrefName, NS_ConvertUTF16toUTF8(wideString).get());
+        rv = SetCharPrefInternal(aPrefName, NS_ConvertUTF16toUTF8(wideString).get());
       }
     }
     return rv;
@@ -440,11 +485,7 @@ NS_IMETHODIMP nsPrefBranch::SetComplexValue(const char *aPrefName, const nsIID &
 
 NS_IMETHODIMP nsPrefBranch::ClearUserPref(const char *aPrefName)
 {
-  if (GetContentChild()) {
-    NS_ERROR("cannot set pref from content process");
-    return NS_ERROR_NOT_AVAILABLE;
-  }
-
+  ENSURE_MAIN_PROCESS("Cannot ClearUserPref from content process:", aPrefName);
   NS_ENSURE_ARG(aPrefName);
   const char *pref = getPrefName(aPrefName);
   return PREF_ClearUserPref(pref);
@@ -461,11 +502,7 @@ NS_IMETHODIMP nsPrefBranch::PrefHasUserValue(const char *aPrefName, bool *_retva
 
 NS_IMETHODIMP nsPrefBranch::LockPref(const char *aPrefName)
 {
-  if (GetContentChild()) {
-    NS_ERROR("cannot lock pref from content process");
-    return NS_ERROR_NOT_AVAILABLE;
-  }
-
+  ENSURE_MAIN_PROCESS("Cannot LockPref from content process:", aPrefName);
   NS_ENSURE_ARG(aPrefName);
   const char *pref = getPrefName(aPrefName);
   return PREF_LockPref(pref, true);
@@ -473,11 +510,7 @@ NS_IMETHODIMP nsPrefBranch::LockPref(const char *aPrefName)
 
 NS_IMETHODIMP nsPrefBranch::PrefIsLocked(const char *aPrefName, bool *_retval)
 {
-  if (GetContentChild()) {
-    NS_ERROR("cannot check lock pref from content process");
-    return NS_ERROR_NOT_AVAILABLE;
-  }
-
+  ENSURE_MAIN_PROCESS("Cannot check PrefIsLocked from content process:", aPrefName);
   NS_ENSURE_ARG_POINTER(_retval);
   NS_ENSURE_ARG(aPrefName);
   const char *pref = getPrefName(aPrefName);
@@ -487,11 +520,7 @@ NS_IMETHODIMP nsPrefBranch::PrefIsLocked(const char *aPrefName, bool *_retval)
 
 NS_IMETHODIMP nsPrefBranch::UnlockPref(const char *aPrefName)
 {
-  if (GetContentChild()) {
-    NS_ERROR("cannot unlock pref from content process");
-    return NS_ERROR_NOT_AVAILABLE;
-  }
-
+  ENSURE_MAIN_PROCESS("Cannot UnlockPref from content process:", aPrefName);
   NS_ENSURE_ARG(aPrefName);
   const char *pref = getPrefName(aPrefName);
   return PREF_LockPref(pref, false);
@@ -505,11 +534,7 @@ NS_IMETHODIMP nsPrefBranch::ResetBranch(const char *aStartingAt)
 
 NS_IMETHODIMP nsPrefBranch::DeleteBranch(const char *aStartingAt)
 {
-  if (GetContentChild()) {
-    NS_ERROR("cannot set pref from content process");
-    return NS_ERROR_NOT_AVAILABLE;
-  }
-
+  ENSURE_MAIN_PROCESS("Cannot DeleteBranch from content process:", aStartingAt);
   NS_ENSURE_ARG(aStartingAt);
   const char *pref = getPrefName(aStartingAt);
   return PREF_DeleteBranch(pref);
@@ -641,7 +666,7 @@ NS_IMETHODIMP nsPrefBranch::RemoveObserver(const char *aDomain, nsIObserver *aOb
   return rv;
 }
 
-NS_IMETHODIMP nsPrefBranch::Observe(nsISupports *aSubject, const char *aTopic, const PRUnichar *someData)
+NS_IMETHODIMP nsPrefBranch::Observe(nsISupports *aSubject, const char *aTopic, const char16_t *someData)
 {
   // watch for xpcom shutdown and free our observers to eliminate any cyclic references
   if (!nsCRT::strcmp(aTopic, NS_XPCOM_SHUTDOWN_OBSERVER_ID)) {
@@ -651,7 +676,7 @@ NS_IMETHODIMP nsPrefBranch::Observe(nsISupports *aSubject, const char *aTopic, c
 }
 
 /* static */
-nsresult nsPrefBranch::NotifyObserver(const char *newpref, void *data)
+void nsPrefBranch::NotifyObserver(const char *newpref, void *data)
 {
   PrefCallback *pCallback = (PrefCallback *)data;
 
@@ -659,7 +684,7 @@ nsresult nsPrefBranch::NotifyObserver(const char *newpref, void *data)
   if (!observer) {
     // The observer has expired.  Let's remove this callback.
     pCallback->GetPrefBranch()->RemoveExpiredCallback(pCallback);
-    return NS_OK;
+    return;
   }
 
   // remove any root this string may contain so as to not confuse the observer
@@ -670,7 +695,6 @@ nsresult nsPrefBranch::NotifyObserver(const char *newpref, void *data)
   observer->Observe(static_cast<nsIPrefBranch *>(pCallback->GetPrefBranch()),
                     NS_PREFBRANCH_PREFCHANGE_TOPIC_ID,
                     NS_ConvertASCIItoUTF16(suffix).get());
-  return NS_OK;
 }
 
 PLDHashOperator
@@ -709,7 +733,7 @@ nsPrefBranch::RemoveExpiredCallback(PrefCallback *aCallback)
   mObservers.Remove(aCallback);
 }
 
-nsresult nsPrefBranch::GetDefaultFromPropertiesFile(const char *aPrefName, PRUnichar **return_buf)
+nsresult nsPrefBranch::GetDefaultFromPropertiesFile(const char *aPrefName, char16_t **return_buf)
 {
   nsresult rv;
 
@@ -781,8 +805,8 @@ nsPrefLocalizedString::~nsPrefLocalizedString()
  * nsISupports Implementation
  */
 
-NS_IMPL_THREADSAFE_ADDREF(nsPrefLocalizedString)
-NS_IMPL_THREADSAFE_RELEASE(nsPrefLocalizedString)
+NS_IMPL_ADDREF(nsPrefLocalizedString)
+NS_IMPL_RELEASE(nsPrefLocalizedString)
 
 NS_INTERFACE_MAP_BEGIN(nsPrefLocalizedString)
     NS_INTERFACE_MAP_ENTRY_AMBIGUOUS(nsISupports, nsIPrefLocalizedString)
@@ -799,7 +823,7 @@ nsresult nsPrefLocalizedString::Init()
 }
 
 NS_IMETHODIMP
-nsPrefLocalizedString::GetData(PRUnichar **_retval)
+nsPrefLocalizedString::GetData(char16_t **_retval)
 {
   nsAutoString data;
 
@@ -815,7 +839,7 @@ nsPrefLocalizedString::GetData(PRUnichar **_retval)
 }
 
 NS_IMETHODIMP
-nsPrefLocalizedString::SetData(const PRUnichar *aData)
+nsPrefLocalizedString::SetData(const char16_t *aData)
 {
   if (!aData)
     return SetData(EmptyString());
@@ -824,7 +848,7 @@ nsPrefLocalizedString::SetData(const PRUnichar *aData)
 
 NS_IMETHODIMP
 nsPrefLocalizedString::SetDataWithLength(uint32_t aLength,
-                                         const PRUnichar *aData)
+                                         const char16_t *aData)
 {
   if (!aData)
     return SetData(EmptyString());
@@ -835,7 +859,7 @@ nsPrefLocalizedString::SetDataWithLength(uint32_t aLength,
 // nsRelativeFilePref
 //----------------------------------------------------------------------------
 
-NS_IMPL_THREADSAFE_ISUPPORTS1(nsRelativeFilePref, nsIRelativeFilePref)
+NS_IMPL_ISUPPORTS1(nsRelativeFilePref, nsIRelativeFilePref)
 
 nsRelativeFilePref::nsRelativeFilePref()
 {
@@ -870,3 +894,5 @@ NS_IMETHODIMP nsRelativeFilePref::SetRelativeToKey(const nsACString& aRelativeTo
   mRelativeToKey.Assign(aRelativeToKey);
   return NS_OK;
 }
+
+#undef ENSURE_MAIN_PROCESS
\ No newline at end of file
