@@ -8,15 +8,13 @@
 #include "prefapi.h"
 #include "prefapi_private_data.h"
 #include "prefread.h"
+#include "MainThreadUtils.h"
 #include "nsReadableUtils.h"
 #include "nsCRT.h"
 
 #define PL_ARENA_CONST_ALIGN_MASK 3
 #include "plarena.h"
 
-#ifdef XP_OS2
-  #include <sys/types.h>
-#endif
 #ifdef _WIN32
   #include "windows.h"
 #endif /* _WIN32 */
@@ -33,11 +31,6 @@
 #include "nsPrintfCString.h"
 #include "prlink.h"
 
-#ifdef XP_OS2
-#define INCL_DOS
-#include <os2.h>
-#endif
-
 using namespace mozilla;
 
 static void
@@ -157,8 +150,8 @@ nsresult PREF_Init()
 {
     if (!gHashTable.ops) {
         if (!PL_DHashTableInit(&gHashTable, &pref_HashTableOps, nullptr,
-                               sizeof(PrefHashEntry),
-                               PREF_HASHTABLE_INITIAL_SIZE)) {
+                               sizeof(PrefHashEntry), PREF_HASHTABLE_INITIAL_SIZE,
+                               fallible_t())) {
             gHashTable.ops = nullptr;
             return NS_ERROR_OUT_OF_MEMORY;
         }
@@ -583,6 +576,10 @@ pref_DeleteItem(PLDHashTable *table, PLDHashEntryHdr *heh, uint32_t i, void *arg
 nsresult
 PREF_DeleteBranch(const char *branch_name)
 {
+#ifndef MOZ_B2G
+    MOZ_ASSERT(NS_IsMainThread());
+#endif
+
     int len = (int)strlen(branch_name);
 
     if (!gHashTable.ops)
@@ -650,6 +647,10 @@ pref_ClearUserPref(PLDHashTable *table, PLDHashEntryHdr *he, uint32_t,
 nsresult
 PREF_ClearAllUserPrefs()
 {
+#ifndef MOZ_B2G
+    MOZ_ASSERT(NS_IsMainThread());
+#endif
+
     if (!gHashTable.ops)
         return NS_ERROR_NOT_INITIALIZED;
 
@@ -724,6 +725,10 @@ static void pref_SetValue(PrefValue* oldValue, PrefValue newValue, PrefType type
 
 PrefHashEntry* pref_HashTableLookup(const void *key)
 {
+#ifndef MOZ_B2G
+    MOZ_ASSERT(NS_IsMainThread());
+#endif
+
     PrefHashEntry* result =
         static_cast<PrefHashEntry*>(PL_DHashTableOperate(&gHashTable, key, PL_DHASH_LOOKUP));
 
@@ -735,6 +740,10 @@ PrefHashEntry* pref_HashTableLookup(const void *key)
 
 nsresult pref_HashPref(const char *key, PrefValue value, PrefType type, uint32_t flags)
 {
+#ifndef MOZ_B2G
+    MOZ_ASSERT(NS_IsMainThread());
+#endif
+
     if (!gHashTable.ops)
         return NS_ERROR_OUT_OF_MEMORY;
 
