@@ -4,7 +4,6 @@
 
 pref("security.tls.version.min", 0);
 pref("security.tls.version.max", 3);
-pref("security.enable_md5_signatures", false);
 
 pref("security.ssl.allow_unrestricted_renego_everywhere__temporarily_available_pref", false);
 pref("security.ssl.renego_unrestricted_hosts", "");
@@ -12,9 +11,11 @@ pref("security.ssl.treat_unsafe_negotiation_as_broken", false);
 pref("security.ssl.require_safe_negotiation",  false);
 pref("security.ssl.warn_missing_rfc5746",  1);
 pref("security.ssl.enable_ocsp_stapling", true);
-pref("security.ssl.enable_false_start", false);
+pref("security.ssl.enable_false_start", true);
 pref("security.ssl.false_start.require-npn", true);
-pref("security.ssl.false_start.require-forward-secrecy", false);
+pref("security.ssl.false_start.require-forward-secrecy", true);
+pref("security.ssl.enable_npn", true);
+pref("security.ssl.enable_alpn", false);
 
 pref("security.ssl3.ecdhe_rsa_aes_128_gcm_sha256", true);
 pref("security.ssl3.ecdhe_ecdsa_aes_128_gcm_sha256", true);
