@@ -148,8 +148,7 @@ nsBaseChannel::ContinueRedirect()
 
   // close down this channel
   Cancel(NS_BINDING_REDIRECTED);
-  mListener = nullptr;
-  mListenerContext = nullptr;
+  ChannelDone();
 
   return NS_OK;
 }
@@ -157,7 +156,7 @@ nsBaseChannel::ContinueRedirect()
 bool
 nsBaseChannel::HasContentTypeHint() const
 {
-  NS_ASSERTION(!IsPending(), "HasContentTypeHint called too late");
+  NS_ASSERTION(!Pending(), "HasContentTypeHint called too late");
   return !mContentType.EqualsLiteral(UNKNOWN_CONTENT_TYPE);
 }
 
@@ -209,7 +208,7 @@ nsBaseChannel::BeginPumpingData()
       return rv;
   }
 
-  // By assigning mPump, we flag this channel as pending (see IsPending).  It's
+  // By assigning mPump, we flag this channel as pending (see Pending).  It's
   // important that the pending flag is set when we call into the stream (the
   // call to AsyncRead results in the stream's AsyncWait method being called)
   // and especially when we call into the loadgroup.  Our caller takes care to
@@ -254,8 +253,7 @@ nsBaseChannel::ContinueHandleAsyncRedirect(nsresult result)
     // Notify our consumer ourselves
     mListener->OnStartRequest(this, mListenerContext);
     mListener->OnStopRequest(this, mListenerContext, mStatus);
-    mListener = nullptr;
-    mListenerContext = nullptr;
+    ChannelDone();
   }
 
   if (mLoadGroup)
@@ -314,7 +312,7 @@ nsBaseChannel::GetName(nsACString &result)
 NS_IMETHODIMP
 nsBaseChannel::IsPending(bool *result)
 {
-  *result = IsPending();
+  *result = Pending();
   return NS_OK;
 }
 
@@ -598,8 +596,7 @@ nsBaseChannel::AsyncOpen(nsIStreamListener *listener, nsISupports *ctxt)
   rv = BeginPumpingData();
   if (NS_FAILED(rv)) {
     mPump = nullptr;
-    mListener = nullptr;
-    mListenerContext = nullptr;
+    ChannelDone();
     mCallbacks = nullptr;
     return rv;
   }
@@ -728,13 +725,12 @@ nsBaseChannel::OnStopRequest(nsIRequest *request, nsISupports *ctxt,
   if (NS_SUCCEEDED(mStatus))
     mStatus = status;
 
-  // Cause IsPending to return false.
+  // Cause Pending to return false.
   mPump = nullptr;
 
   if (mListener) // null in case of redirect
       mListener->OnStopRequest(this, mListenerContext, mStatus);
-  mListener = nullptr;
-  mListenerContext = nullptr;
+  ChannelDone();
 
   // No need to suspend pump in this scope since we will not be receiving
   // any more events from it.
