@@ -9,11 +9,8 @@
 
 #include "nsURLParsers.h"
 #include "nsURLHelper.h"
-#include "nsIURI.h"
-#include "prtypes.h"
 #include "nsString.h"
 #include "nsCRT.h"
-#include "netCore.h"
 
 using namespace mozilla;
 
@@ -150,12 +147,13 @@ nsBaseURLParser::ParseURL(const char *spec, int32_t specLen,
         // spec = <path-no-slashes-or-colon>
         //
         SET_RESULT(scheme, 0, -1);
-        if (authorityLen || pathLen)
+        if (authorityLen || pathLen) {
             ParseAfterScheme(spec, specLen,
                              authorityPos, authorityLen,
                              pathPos, pathLen);
             OFFSET_RESULT(authority, offset);
             OFFSET_RESULT(path, offset);
+        }
     }
     return NS_OK;
 }
@@ -370,7 +368,7 @@ nsNoAuthURLParser::ParseAfterScheme(const char *spec, int32_t specLen,
             const char *p = nullptr;
             if (specLen > 2) {
                 // looks like there is an authority section
-#if defined(XP_WIN) || defined(XP_OS2)
+#if defined(XP_WIN)
                 // if the authority looks like a drive number then we
                 // really want to treat it as part of the path
                 // [a-zA-Z][:|]{/\}
@@ -403,7 +401,7 @@ nsNoAuthURLParser::ParseAfterScheme(const char *spec, int32_t specLen,
     SET_RESULT(path, pos, specLen - pos);
 }
 
-#if defined(XP_WIN) || defined(XP_OS2)
+#if defined(XP_WIN)
 NS_IMETHODIMP
 nsNoAuthURLParser::ParseFilePath(const char *filepath, int32_t filepathLen,
                                  uint32_t *directoryPos, int32_t *directoryLen,
@@ -582,7 +580,7 @@ nsAuthURLParser::ParseServerInfo(const char *serverinfo, int32_t serverinfoLen,
 
                 nsresult err;
                 *port = buf.ToInteger(&err);
-                if (NS_FAILED(err) || *port <= 0)
+                if (NS_FAILED(err) || *port < 0)
                     return NS_ERROR_MALFORMED_URI;
             }
         }
