@@ -5,6 +5,7 @@
 #include "CacheLog.h"
 #include "CacheEntry.h"
 #include "CacheStorageService.h"
+#include "CacheObserver.h"
 
 #include "nsIInputStream.h"
 #include "nsIOutputStream.h"
@@ -14,6 +15,7 @@
 #include "nsICacheStorage.h"
 #include "nsISerializable.h"
 #include "nsIStreamTransportService.h"
+#include "nsISizeOf.h"
 
 #include "nsComponentManagerUtils.h"
 #include "nsServiceManagerUtils.h"
@@ -30,28 +32,116 @@ namespace net {
 
 static uint32_t const ENTRY_WANTED =
   nsICacheEntryOpenCallback::ENTRY_WANTED;
+static uint32_t const RECHECK_AFTER_WRITE_FINISHED =
+  nsICacheEntryOpenCallback::RECHECK_AFTER_WRITE_FINISHED;
 static uint32_t const ENTRY_NEEDS_REVALIDATION =
   nsICacheEntryOpenCallback::ENTRY_NEEDS_REVALIDATION;
 static uint32_t const ENTRY_NOT_WANTED =
   nsICacheEntryOpenCallback::ENTRY_NOT_WANTED;
 
-NS_IMPL_ISUPPORTS1(CacheEntry::Handle, nsICacheEntry)
+NS_IMPL_ISUPPORTS1(CacheEntryHandle, nsICacheEntry)
 
-// CacheEntry::Handle
+// CacheEntryHandle
 
-CacheEntry::Handle::Handle(CacheEntry* aEntry)
+CacheEntryHandle::CacheEntryHandle(CacheEntry* aEntry)
 : mEntry(aEntry)
 {
-  MOZ_COUNT_CTOR(CacheEntry::Handle);
+  MOZ_COUNT_CTOR(CacheEntryHandle);
 
-  LOG(("New CacheEntry::Handle %p for entry %p", this, aEntry));
+#ifdef DEBUG
+  if (!mEntry->mHandlersCount) {
+    // CacheEntry.mHandlersCount must go from zero to one only under
+    // the service lock. Can access CacheStorageService::Self() w/o a check
+    // since CacheEntry hrefs it.
+    CacheStorageService::Self()->Lock().AssertCurrentThreadOwns();
+  }
+#endif
+
+  ++mEntry->mHandlersCount;
+
+  LOG(("New CacheEntryHandle %p for entry %p", this, aEntry));
 }
 
-CacheEntry::Handle::~Handle()
+CacheEntryHandle::~CacheEntryHandle()
 {
-  mEntry->OnWriterClosed(this);
+  --mEntry->mHandlersCount;
+  mEntry->OnHandleClosed(this);
 
-  MOZ_COUNT_DTOR(CacheEntry::Handle);
+  MOZ_COUNT_DTOR(CacheEntryHandle);
+}
+
+// CacheEntry::Callback
+
+CacheEntry::Callback::Callback(CacheEntry* aEntry,
+                               nsICacheEntryOpenCallback *aCallback,
+                               bool aReadOnly, bool aCheckOnAnyThread)
+: mEntry(aEntry)
+, mCallback(aCallback)
+, mTargetThread(do_GetCurrentThread())
+, mReadOnly(aReadOnly)
+, mCheckOnAnyThread(aCheckOnAnyThread)
+, mRecheckAfterWrite(false)
+, mNotWanted(false)
+{
+  MOZ_COUNT_CTOR(CacheEntry::Callback);
+
+  // The counter may go from zero to non-null only under the service lock
+  // but here we expect it to be already positive.
+  MOZ_ASSERT(mEntry->mHandlersCount);
+  ++mEntry->mHandlersCount;
+}
+
+CacheEntry::Callback::Callback(CacheEntry::Callback const &aThat)
+: mEntry(aThat.mEntry)
+, mCallback(aThat.mCallback)
+, mTargetThread(aThat.mTargetThread)
+, mReadOnly(aThat.mReadOnly)
+, mCheckOnAnyThread(aThat.mCheckOnAnyThread)
+, mRecheckAfterWrite(aThat.mRecheckAfterWrite)
+, mNotWanted(aThat.mNotWanted)
+{
+  MOZ_COUNT_CTOR(CacheEntry::Callback);
+
+  // The counter may go from zero to non-null only under the service lock
+  // but here we expect it to be already positive.
+  MOZ_ASSERT(mEntry->mHandlersCount);
+  ++mEntry->mHandlersCount;
+}
+
+CacheEntry::Callback::~Callback()
+{
+  --mEntry->mHandlersCount;
+  MOZ_COUNT_DTOR(CacheEntry::Callback);
+}
+
+void CacheEntry::Callback::ExchangeEntry(CacheEntry* aEntry)
+{
+  if (mEntry == aEntry)
+    return;
+
+  // The counter may go from zero to non-null only under the service lock
+  // but here we expect it to be already positive.
+  MOZ_ASSERT(aEntry->mHandlersCount);
+  ++aEntry->mHandlersCount;
+  --mEntry->mHandlersCount;
+  mEntry = aEntry;
+}
+
+nsresult CacheEntry::Callback::OnCheckThread(bool *aOnCheckThread) const
+{
+  if (!mCheckOnAnyThread) {
+    // Check we are on the target
+    return mTargetThread->IsOnCurrentThread(aOnCheckThread);
+  }
+
+  // We can invoke check anywhere
+  *aOnCheckThread = true;
+  return NS_OK;
+}
+
+nsresult CacheEntry::Callback::OnAvailThread(bool *aOnAvailThread) const
+{
+  return mTargetThread->IsOnCurrentThread(aOnAvailThread);
 }
 
 // CacheEntry
@@ -76,13 +166,13 @@ CacheEntry::CacheEntry(const nsACString& aStorageID,
 , mIsDoomed(false)
 , mSecurityInfoLoaded(false)
 , mPreventCallbacks(false)
-, mHasMainThreadOnlyCallback(false)
 , mHasData(false)
 , mState(NOTLOADED)
 , mRegistration(NEVERREGISTERED)
 , mWriter(nullptr)
 , mPredictedDataSize(0)
 , mDataSize(0)
+, mReleaseThread(NS_GetCurrentThread())
 {
   MOZ_COUNT_CTOR(CacheEntry);
 
@@ -94,7 +184,7 @@ CacheEntry::CacheEntry(const nsACString& aStorageID,
 
 CacheEntry::~CacheEntry()
 {
-  ProxyReleaseMainThread(mURI);
+  ProxyRelease(mURI, mReleaseThread);
 
   LOG(("CacheEntry::~CacheEntry [this=%p]", this));
   MOZ_COUNT_DTOR(CacheEntry);
@@ -166,27 +256,24 @@ void CacheEntry::AsyncOpen(nsICacheEntryOpenCallback* aCallback, uint32_t aFlags
   bool readonly = aFlags & nsICacheStorage::OPEN_READONLY;
   bool truncate = aFlags & nsICacheStorage::OPEN_TRUNCATE;
   bool priority = aFlags & nsICacheStorage::OPEN_PRIORITY;
-
-  bool mainThreadOnly;
-  if (aCallback && NS_FAILED(aCallback->GetMainThreadOnly(&mainThreadOnly)))
-    mainThreadOnly = true; // rather play safe...
+  bool multithread = aFlags & nsICacheStorage::CHECK_MULTITHREADED;
 
   MOZ_ASSERT(!readonly || !truncate, "Bad flags combination");
   MOZ_ASSERT(!(truncate && mState > LOADING), "Must not call truncate on already loaded entry");
 
+  Callback callback(this, aCallback, readonly, multithread);
+
   mozilla::MutexAutoLock lock(mLock);
 
-  if (Load(truncate, priority) ||
-      PendingCallbacks() ||
-      !InvokeCallback(aCallback, readonly)) {
-    // Load in progress or callback bypassed...
-    if (mainThreadOnly) {
-      LOG(("  callback is main-thread only"));
-      mHasMainThreadOnlyCallback = true;
-    }
+  RememberCallback(callback);
 
-    RememberCallback(aCallback, readonly);
+  // Load() opens the lock
+  if (Load(truncate, priority)) {
+    // Loading is in progress...
+    return;
   }
+
+  InvokeCallbacks();
 }
 
 bool CacheEntry::Load(bool aTruncate, bool aPriority)
@@ -205,44 +292,47 @@ bool CacheEntry::Load(bool aTruncate, bool aPriority)
     return true;
   }
 
+  mState = LOADING;
+
   MOZ_ASSERT(!mFile);
 
   bool directLoad = aTruncate || !mUseDisk;
-  if (directLoad) {
-    // Just fake the load has already been done as "new".
-    mState = EMPTY;
+  if (directLoad)
     mFileStatus = NS_OK;
-  }
-  else {
-    mState = LOADING;
+  else
     mLoadStart = TimeStamp::Now();
-  }
 
   mFile = new CacheFile();
 
   BackgroundOp(Ops::REGISTER);
 
-  mozilla::MutexAutoUnlock unlock(mLock);
+  {
+    mozilla::MutexAutoUnlock unlock(mLock);
 
-  nsresult rv;
+    nsresult rv;
 
-  nsAutoCString fileKey;
-  rv = HashingKeyWithStorage(fileKey);
+    nsAutoCString fileKey;
+    rv = HashingKeyWithStorage(fileKey);
 
-  LOG(("  performing load, file=%p", mFile.get()));
-  if (NS_SUCCEEDED(rv)) {
-    rv = mFile->Init(fileKey,
-                     aTruncate,
-                     !mUseDisk,
-                     aPriority,
-                     false /* key is not a hash */,
-                     directLoad ? nullptr : this);
+    LOG(("  performing load, file=%p", mFile.get()));
+    if (NS_SUCCEEDED(rv)) {
+      rv = mFile->Init(fileKey,
+                       aTruncate,
+                       !mUseDisk,
+                       aPriority,
+                       directLoad ? nullptr : this);
+    }
+
+    if (NS_FAILED(rv)) {
+      mFileStatus = rv;
+      AsyncDoom(nullptr);
+      return false;
+    }
   }
 
-  if (NS_FAILED(rv)) {
-    mFileStatus = rv;
-    AsyncDoom(nullptr);
-    return false;
+  if (directLoad) {
+    // Just fake the load has already been done as "new".
+    mState = EMPTY;
   }
 
   return mState == LOADING;
@@ -282,9 +372,16 @@ NS_IMETHODIMP CacheEntry::OnFileReady(nsresult aResult, bool aIsNew)
 
   mFileStatus = aResult;
 
-  if (mState == READY)
+  if (mState == READY) {
     mHasData = true;
 
+    uint32_t frecency;
+    mFile->GetFrecency(&frecency);
+    // mFrecency is held in a double to increase computance precision.
+    // It is ok to persist frecency only as a uint32 with some math involved.
+    mFrecency = INT2FRECENCY(frecency);
+  }
+
   InvokeCallbacks();
   return NS_OK;
 }
@@ -300,7 +397,8 @@ NS_IMETHODIMP CacheEntry::OnFileDoomed(nsresult aResult)
   return NS_OK;
 }
 
-already_AddRefed<CacheEntry> CacheEntry::ReopenTruncated(nsICacheEntryOpenCallback* aCallback)
+already_AddRefed<CacheEntryHandle> CacheEntry::ReopenTruncated(bool aMemoryOnly,
+                                                               nsICacheEntryOpenCallback* aCallback)
 {
   LOG(("CacheEntry::ReopenTruncated [this=%p]", this));
 
@@ -309,6 +407,7 @@ already_AddRefed<CacheEntry> CacheEntry::ReopenTruncated(nsICacheEntryOpenCallba
   // Hold callbacks invocation, AddStorageEntry would invoke from doom prematurly
   mPreventCallbacks = true;
 
+  nsRefPtr<CacheEntryHandle> handle;
   nsRefPtr<CacheEntry> newEntry;
   {
     mozilla::MutexAutoUnlock unlock(mLock);
@@ -316,14 +415,15 @@ already_AddRefed<CacheEntry> CacheEntry::ReopenTruncated(nsICacheEntryOpenCallba
     // The following call dooms this entry (calls DoomAlreadyRemoved on us)
     nsresult rv = CacheStorageService::Self()->AddStorageEntry(
       GetStorageID(), GetURI(), GetEnhanceID(),
-      mUseDisk,
+      mUseDisk && !aMemoryOnly,
       true, // always create
       true, // truncate existing (this one)
-      getter_AddRefs(newEntry));
+      getter_AddRefs(handle));
 
     LOG(("  exchanged entry %p by entry %p, rv=0x%08x", this, newEntry.get(), rv));
 
     if (NS_SUCCEEDED(rv)) {
+      newEntry = handle->Entry();
       newEntry->AsyncOpen(aCallback, nsICacheStorage::OPEN_TRUNCATE);
     }
     else {
@@ -338,52 +438,42 @@ already_AddRefed<CacheEntry> CacheEntry::ReopenTruncated(nsICacheEntryOpenCallba
 
   newEntry->TransferCallbacks(*this);
   mCallbacks.Clear();
-  mReadOnlyCallbacks.Clear();
-  mHasMainThreadOnlyCallback = false;
 
-  return newEntry.forget();
+  return handle.forget();
 }
 
-void CacheEntry::TransferCallbacks(CacheEntry const& aFromEntry)
+void CacheEntry::TransferCallbacks(CacheEntry & aFromEntry)
 {
   mozilla::MutexAutoLock lock(mLock);
 
   LOG(("CacheEntry::TransferCallbacks [entry=%p, from=%p]",
     this, &aFromEntry));
 
-  mCallbacks.AppendObjects(aFromEntry.mCallbacks);
-  mReadOnlyCallbacks.AppendObjects(aFromEntry.mReadOnlyCallbacks);
-  if (aFromEntry.mHasMainThreadOnlyCallback)
-    mHasMainThreadOnlyCallback = true;
-
-  if (mCallbacks.Length() || mReadOnlyCallbacks.Length())
-    BackgroundOp(Ops::CALLBACKS, true);
-}
-
-void CacheEntry::RememberCallback(nsICacheEntryOpenCallback* aCallback,
-                                  bool aReadOnly)
-{
-  // AsyncOpen can be called w/o a callback reference (when this is a new/truncated entry)
-  if (!aCallback)
-    return;
-
-  LOG(("CacheEntry::RememberCallback [this=%p, cb=%p]", this, aCallback));
-
-  mLock.AssertCurrentThreadOwns();
-
-  if (!aReadOnly)
-    mCallbacks.AppendObject(aCallback);
+  if (!mCallbacks.Length())
+    mCallbacks.SwapElements(aFromEntry.mCallbacks);
   else
-    mReadOnlyCallbacks.AppendObject(aCallback);
+    mCallbacks.AppendElements(aFromEntry.mCallbacks);
+
+  uint32_t callbacksLength = mCallbacks.Length();
+  if (callbacksLength) {
+    // Carry the entry reference (unfortunatelly, needs to be done manually...)
+    for (uint32_t i = 0; i < callbacksLength; ++i)
+      mCallbacks[i].ExchangeEntry(this);
+
+    BackgroundOp(Ops::CALLBACKS, true);
+  }
 }
 
-bool CacheEntry::PendingCallbacks()
+void CacheEntry::RememberCallback(Callback const& aCallback)
 {
+  LOG(("CacheEntry::RememberCallback [this=%p, cb=%p]", this, aCallback.mCallback.get()));
+
   mLock.AssertCurrentThreadOwns();
-  return mCallbacks.Length() || mReadOnlyCallbacks.Length();
+
+  mCallbacks.AppendElement(aCallback);
 }
 
-void CacheEntry::InvokeCallbacksMainThread()
+void CacheEntry::InvokeCallbacksLock()
 {
   mozilla::MutexAutoLock lock(mLock);
   InvokeCallbacks();
@@ -391,80 +481,84 @@ void CacheEntry::InvokeCallbacksMainThread()
 
 void CacheEntry::InvokeCallbacks()
 {
-  LOG(("CacheEntry::InvokeCallbacks BEGIN [this=%p]", this));
-
   mLock.AssertCurrentThreadOwns();
 
-  do {
-    if (mPreventCallbacks) {
-      LOG(("CacheEntry::InvokeCallbacks END [this=%p] callbacks prevented!", this));
-      return;
-    }
+  LOG(("CacheEntry::InvokeCallbacks BEGIN [this=%p]", this));
 
-    if (!mCallbacks.Count()) {
-      LOG(("  no r/w callbacks"));
-      break;
-    }
-
-    if (mHasMainThreadOnlyCallback && !NS_IsMainThread()) {
-      nsRefPtr<nsRunnableMethod<CacheEntry> > event =
-        NS_NewRunnableMethod(this, &CacheEntry::InvokeCallbacksMainThread);
-      NS_DispatchToMainThread(event);
-      LOG(("CacheEntry::InvokeCallbacks END [this=%p] dispatching to maintread", this));
-      return;
-    }
-
-    nsCOMPtr<nsICacheEntryOpenCallback> callback = mCallbacks[0];
-    mCallbacks.RemoveElementAt(0);
-
-    if (!InvokeCallback(callback, false)) {
-      mCallbacks.InsertElementAt(0, callback);
-      LOG(("CacheEntry::InvokeCallbacks END [this=%p] callback bypassed", this));
-      return;
-    }
-  } while (true);
-
-  while (mReadOnlyCallbacks.Count()) {
-    if (mHasMainThreadOnlyCallback && !NS_IsMainThread()) {
-      nsRefPtr<nsRunnableMethod<CacheEntry> > event =
-        NS_NewRunnableMethod(this, &CacheEntry::InvokeCallbacksMainThread);
-      NS_DispatchToMainThread(event);
-      LOG(("CacheEntry::InvokeCallbacks END [this=%p] dispatching to maintread", this));
-      return;
-    }
-
-    nsCOMPtr<nsICacheEntryOpenCallback> callback = mReadOnlyCallbacks[0];
-    mReadOnlyCallbacks.RemoveElementAt(0);
-
-    if (!InvokeCallback(callback, true)) {
-      // Didn't trigger, so we must stop
-      mReadOnlyCallbacks.InsertElementAt(0, callback);
-      break;
-    }
-  }
-
-  if (!mCallbacks.Count() && !mReadOnlyCallbacks.Count())
-    mHasMainThreadOnlyCallback = false;
+  // Invoke first all r/w callbacks, then all r/o callbacks.
+  if (InvokeCallbacks(false))
+    InvokeCallbacks(true);
 
   LOG(("CacheEntry::InvokeCallbacks END [this=%p]", this));
 }
 
-bool CacheEntry::InvokeCallback(nsICacheEntryOpenCallback* aCallback,
-                                bool aReadOnly)
+bool CacheEntry::InvokeCallbacks(bool aReadOnly)
+{
+  mLock.AssertCurrentThreadOwns();
+
+  uint32_t i = 0;
+  while (i < mCallbacks.Length()) {
+    if (mPreventCallbacks) {
+      LOG(("  callbacks prevented!"));
+      return false;
+    }
+
+    if (!mIsDoomed && (mState == WRITING || mState == REVALIDATING)) {
+      LOG(("  entry is being written/revalidated"));
+      return false;
+    }
+
+    if (mCallbacks[i].mReadOnly != aReadOnly) {
+      // Callback is not r/w or r/o, go to another one in line
+      ++i;
+      continue;
+    }
+
+    bool onCheckThread;
+    nsresult rv = mCallbacks[i].OnCheckThread(&onCheckThread);
+
+    if (NS_SUCCEEDED(rv) && !onCheckThread) {
+      // Redispatch to the target thread
+      nsRefPtr<nsRunnableMethod<CacheEntry> > event =
+        NS_NewRunnableMethod(this, &CacheEntry::InvokeCallbacksLock);
+
+      rv = mCallbacks[i].mTargetThread->Dispatch(event, nsIEventTarget::DISPATCH_NORMAL);
+      if (NS_SUCCEEDED(rv)) {
+        LOG(("  re-dispatching to target thread"));
+        return false;
+      }
+    }
+
+    Callback callback = mCallbacks[i];
+    mCallbacks.RemoveElementAt(i);
+
+    if (NS_SUCCEEDED(rv) && !InvokeCallback(callback)) {
+      // Callback didn't fire, put it back and go to another one in line.
+      // Only reason InvokeCallback returns false is that onCacheEntryCheck
+      // returns RECHECK_AFTER_WRITE_FINISHED.  If we would stop the loop, other
+      // readers or potential writers would be unnecessarily kept from being
+      // invoked.
+      mCallbacks.InsertElementAt(i, callback);
+      ++i;
+    }
+  }
+
+  return true;
+}
+
+bool CacheEntry::InvokeCallback(Callback & aCallback)
 {
   LOG(("CacheEntry::InvokeCallback [this=%p, state=%s, cb=%p]",
-    this, StateString(mState), aCallback));
+    this, StateString(mState), aCallback.mCallback.get()));
 
   mLock.AssertCurrentThreadOwns();
 
-  bool notWanted = false;
-
+  // When this entry is doomed we want to notify the callback any time
   if (!mIsDoomed) {
     // When we are here, the entry must be loaded from disk
     MOZ_ASSERT(mState > LOADING);
 
-    if (mState == WRITING ||
-        mState == REVALIDATING) {
+    if (mState == WRITING || mState == REVALIDATING) {
       // Prevent invoking other callbacks since one of them is now writing
       // or revalidating this entry.  No consumers should get this entry
       // until metadata are filled with values downloaded from the server
@@ -473,20 +567,26 @@ bool CacheEntry::InvokeCallback(nsICacheEntryOpenCallback* aCallback,
       return false;
     }
 
-    if (!aReadOnly) {
-      if (mState == EMPTY) {
-        // Advance to writing state, we expect to invoke the callback and let
-        // it fill content of this entry.  Must set and check the state here
-        // to prevent more then one
-        mState = WRITING;
-        LOG(("  advancing to WRITING state"));
-      }
+    // mRecheckAfterWrite flag already set means the callback has already passed
+    // the onCacheEntryCheck call. Until the current write is not finished this
+    // callback will be bypassed.
+    if (!aCallback.mRecheckAfterWrite) {
 
-      if (!aCallback) {
-        // We can be given no callback only in case of recreate, it is ok
-        // to advance to WRITING state since the caller of recreate is expected
-        // to write this entry now.
-        return true;
+      if (!aCallback.mReadOnly) {
+        if (mState == EMPTY) {
+          // Advance to writing state, we expect to invoke the callback and let
+          // it fill content of this entry.  Must set and check the state here
+          // to prevent more then one
+          mState = WRITING;
+          LOG(("  advancing to WRITING state"));
+        }
+
+        if (!aCallback.mCallback) {
+          // We can be given no callback only in case of recreate, it is ok
+          // to advance to WRITING state since the caller of recreate is expected
+          // to write this entry now.
+          return true;
+        }
       }
 
       if (mState == READY) {
@@ -496,11 +596,12 @@ bool CacheEntry::InvokeCallback(nsICacheEntryOpenCallback* aCallback,
           // mayhemer: TODO check and solve any potential races of concurent OnCacheEntryCheck
           mozilla::MutexAutoUnlock unlock(mLock);
 
-          nsresult rv = aCallback->OnCacheEntryCheck(this, nullptr, &checkResult);
+          nsresult rv = aCallback.mCallback->OnCacheEntryCheck(
+            this, nullptr, &checkResult);
           LOG(("  OnCacheEntryCheck: rv=0x%08x, result=%d", rv, checkResult));
 
           if (NS_FAILED(rv))
-            checkResult = ENTRY_WANTED;
+            checkResult = ENTRY_NOT_WANTED;
         }
 
         switch (checkResult) {
@@ -510,6 +611,12 @@ bool CacheEntry::InvokeCallback(nsICacheEntryOpenCallback* aCallback,
           // Proceed to callback...
           break;
 
+        case RECHECK_AFTER_WRITE_FINISHED:
+          LOG(("  consumer will check on the entry again after write is done"));
+          // The consumer wants the entry to complete first.
+          aCallback.mRecheckAfterWrite = true;
+          break;
+
         case ENTRY_NEEDS_REVALIDATION:
           LOG(("  will be holding callbacks until entry is revalidated"));
           // State is READY now and from that state entry cannot transit to any other
@@ -521,46 +628,73 @@ bool CacheEntry::InvokeCallback(nsICacheEntryOpenCallback* aCallback,
         case ENTRY_NOT_WANTED:
           LOG(("  consumer not interested in the entry"));
           // Do not give this entry to the consumer, it is not interested in us.
-          notWanted = true;
+          aCallback.mNotWanted = true;
           break;
         }
       }
     }
   }
 
-  if (aCallback) {
+  if (aCallback.mCallback) {
+    if (!mIsDoomed && aCallback.mRecheckAfterWrite) {
+      // If we don't have data and the callback wants a complete entry,
+      // don't invoke now.
+      bool bypass = !mHasData;
+      if (!bypass) {
+        int64_t _unused;
+        bypass = !mFile->DataSize(&_unused);
+      }
+
+      if (bypass) {
+        LOG(("  bypassing, entry data still being written"));
+        return false;
+      }
+
+      // Entry is complete now, do the check+avail call again
+      aCallback.mRecheckAfterWrite = false;
+      return InvokeCallback(aCallback);
+    }
+
     mozilla::MutexAutoUnlock unlock(mLock);
-    InvokeAvailableCallback(aCallback, aReadOnly, notWanted);
+    InvokeAvailableCallback(aCallback);
   }
 
   return true;
 }
 
-void CacheEntry::InvokeAvailableCallback(nsICacheEntryOpenCallback* aCallback,
-                                         bool aReadOnly,
-                                         bool aNotWanted)
+void CacheEntry::InvokeAvailableCallback(Callback const & aCallback)
 {
   LOG(("CacheEntry::InvokeAvailableCallback [this=%p, state=%s, cb=%p, r/o=%d, n/w=%d]",
-    this, StateString(mState), aCallback, aReadOnly, aNotWanted));
+    this, StateString(mState), aCallback.mCallback.get(), aCallback.mReadOnly, aCallback.mNotWanted));
+
+  nsresult rv;
 
   uint32_t const state = mState;
 
   // When we are here, the entry must be loaded from disk
   MOZ_ASSERT(state > LOADING || mIsDoomed);
 
-  if (!NS_IsMainThread()) {
-    // Must happen on the main thread :(
-    nsRefPtr<AvailableCallbackRunnable> event =
-      new AvailableCallbackRunnable(this, aCallback, aReadOnly, aNotWanted);
-    NS_DispatchToMainThread(event);
+  bool onAvailThread;
+  rv = aCallback.OnAvailThread(&onAvailThread);
+  if (NS_FAILED(rv)) {
+    LOG(("  target thread dead?"));
     return;
   }
 
-  // This happens only on the main thread / :( /
+  if (!onAvailThread) {
+    // Dispatch to the right thread
+    nsRefPtr<AvailableCallbackRunnable> event =
+      new AvailableCallbackRunnable(this, aCallback);
 
-  if (mIsDoomed || aNotWanted) {
+    rv = aCallback.mTargetThread->Dispatch(event, nsIEventTarget::DISPATCH_NORMAL);
+    LOG(("  redispatched, (rv = 0x%08x)", rv));
+    return;
+  }
+
+  if (mIsDoomed || aCallback.mNotWanted) {
     LOG(("  doomed or not wanted, notifying OCEA with NS_ERROR_CACHE_KEY_NOT_FOUND"));
-    aCallback->OnCacheEntryAvailable(nullptr, false, nullptr, NS_ERROR_CACHE_KEY_NOT_FOUND);
+    aCallback.mCallback->OnCacheEntryAvailable(
+      nullptr, false, nullptr, NS_ERROR_CACHE_KEY_NOT_FOUND);
     return;
   }
 
@@ -571,48 +705,57 @@ void CacheEntry::InvokeAvailableCallback(nsICacheEntryOpenCallback* aCallback,
       BackgroundOp(Ops::FRECENCYUPDATE);
     }
 
-    aCallback->OnCacheEntryAvailable(this, false, nullptr, NS_OK);
+    nsRefPtr<CacheEntryHandle> handle = NewHandle();
+    aCallback.mCallback->OnCacheEntryAvailable(
+      handle, false, nullptr, NS_OK);
     return;
   }
 
-  if (aReadOnly) {
+  if (aCallback.mReadOnly) {
     LOG(("  r/o and not ready, notifying OCEA with NS_ERROR_CACHE_KEY_NOT_FOUND"));
-    aCallback->OnCacheEntryAvailable(nullptr, false, nullptr, NS_ERROR_CACHE_KEY_NOT_FOUND);
+    aCallback.mCallback->OnCacheEntryAvailable(
+      nullptr, false, nullptr, NS_ERROR_CACHE_KEY_NOT_FOUND);
     return;
   }
 
   // This is a new or potentially non-valid entry and needs to be fetched first.
-  // The Handle blocks other consumers until the channel
+  // The CacheEntryHandle blocks other consumers until the channel
   // either releases the entry or marks metadata as filled or whole entry valid,
   // i.e. until MetaDataReady() or SetValid() on the entry is called respectively.
 
   // Consumer will be responsible to fill or validate the entry metadata and data.
 
-  nsRefPtr<Handle> handle = NewWriteHandle();
-  nsresult rv = aCallback->OnCacheEntryAvailable(handle, state == WRITING, nullptr, NS_OK);
+  nsRefPtr<CacheEntryHandle> handle = NewWriteHandle();
+  rv = aCallback.mCallback->OnCacheEntryAvailable(
+    handle, state == WRITING, nullptr, NS_OK);
 
   if (NS_FAILED(rv)) {
     LOG(("  writing/revalidating failed (0x%08x)", rv));
 
     // Consumer given a new entry failed to take care of the entry.
-    OnWriterClosed(handle);
+    OnHandleClosed(handle);
     return;
   }
 
   LOG(("  writing/revalidating"));
 }
 
-CacheEntry::Handle* CacheEntry::NewWriteHandle()
+CacheEntryHandle* CacheEntry::NewHandle()
+{
+  return new CacheEntryHandle(this);
+}
+
+CacheEntryHandle* CacheEntry::NewWriteHandle()
 {
   mozilla::MutexAutoLock lock(mLock);
 
   BackgroundOp(Ops::FRECENCYUPDATE);
-  return (mWriter = new Handle(this));
+  return (mWriter = new CacheEntryHandle(this));
 }
 
-void CacheEntry::OnWriterClosed(Handle const* aHandle)
+void CacheEntry::OnHandleClosed(CacheEntryHandle const* aHandle)
 {
-  LOG(("CacheEntry::OnWriterClosed [this=%p, state=%s, handle=%p]", this, StateString(mState), aHandle));
+  LOG(("CacheEntry::OnHandleClosed [this=%p, state=%s, handle=%p]", this, StateString(mState), aHandle));
 
   nsCOMPtr<nsIOutputStream> outputStream;
 
@@ -620,7 +763,7 @@ void CacheEntry::OnWriterClosed(Handle const* aHandle)
     mozilla::MutexAutoLock lock(mLock);
 
     if (mWriter != aHandle) {
-      LOG(("  not the current writer"));
+      LOG(("  not the writer"));
       return;
     }
 
@@ -652,6 +795,15 @@ void CacheEntry::OnWriterClosed(Handle const* aHandle)
   }
 }
 
+void CacheEntry::OnOutputClosed()
+{
+  // Called when the file's output stream is closed.  Invoke any callbacks
+  // waiting for complete entry.
+
+  mozilla::MutexAutoLock lock(mLock);
+  InvokeCallbacks();
+}
+
 bool CacheEntry::UsingDisk() const
 {
   CacheStorageService::Self()->Lock().AssertCurrentThreadOwns();
@@ -676,6 +828,26 @@ bool CacheEntry::SetUsingDisk(bool aUsingDisk)
   return changed;
 }
 
+bool CacheEntry::IsReferenced() const
+{
+  CacheStorageService::Self()->Lock().AssertCurrentThreadOwns();
+
+  // Increasing this counter from 0 to non-null and this check both happen only
+  // under the service lock.
+  return mHandlersCount > 0;
+}
+
+bool CacheEntry::IsFileDoomed()
+{
+  mozilla::MutexAutoLock lock(mLock);
+
+  if (NS_SUCCEEDED(mFileStatus)) {
+    return mFile->IsDoomed();
+  }
+
+  return false;
+}
+
 uint32_t CacheEntry::GetMetadataMemoryConsumption()
 {
   NS_ENSURE_SUCCESS(mFileStatus, 0);
@@ -689,7 +861,7 @@ uint32_t CacheEntry::GetMetadataMemoryConsumption()
 
 // nsICacheEntry
 
-NS_IMETHODIMP CacheEntry::GetPersistToDisk(bool *aPersistToDisk)
+NS_IMETHODIMP CacheEntry::GetPersistent(bool *aPersistToDisk)
 {
   // No need to sync when only reading.
   // When consumer needs to be consistent with state of the memory storage entries
@@ -697,28 +869,6 @@ NS_IMETHODIMP CacheEntry::GetPersistToDisk(bool *aPersistToDisk)
   *aPersistToDisk = mUseDisk;
   return NS_OK;
 }
-NS_IMETHODIMP CacheEntry::SetPersistToDisk(bool aPersistToDisk)
-{
-  LOG(("CacheEntry::SetPersistToDisk [this=%p, persist=%d]", this, aPersistToDisk));
-
-  if (mState >= READY) {
-    LOG(("  failed, called after filling the entry"));
-    return NS_ERROR_NOT_AVAILABLE;
-  }
-
-  if (mUseDisk == aPersistToDisk)
-    return NS_OK;
-
-  mozilla::MutexAutoLock lock(CacheStorageService::Self()->Lock());
-
-  mUseDisk = aPersistToDisk;
-  CacheStorageService::Self()->RecordMemoryOnlyEntry(
-    this, !aPersistToDisk, false /* don't overwrite */);
-
-  // File persistence is setup just before we open output stream on it.
-
-  return NS_OK;
-}
 
 NS_IMETHODIMP CacheEntry::GetKey(nsACString & aKey)
 {
@@ -807,7 +957,7 @@ NS_IMETHODIMP CacheEntry::OpenOutputStream(int64_t offset, nsIOutputStream * *_r
 
   MOZ_ASSERT(mState > EMPTY);
 
-  if (mOutputStream) {
+  if (mOutputStream && !mIsDoomed) {
     LOG(("  giving phantom output stream"));
     mOutputStream.forget(_retval);
   }
@@ -841,9 +991,6 @@ nsresult CacheEntry::OpenOutputStreamInternal(int64_t offset, nsIOutputStream *
 
   MOZ_ASSERT(mState > LOADING);
 
-  if (!mFile)
-    return NS_ERROR_NOT_AVAILABLE;
-
   nsresult rv;
 
   // No need to sync on mUseDisk here, we don't need to be consistent
@@ -853,8 +1000,11 @@ nsresult CacheEntry::OpenOutputStreamInternal(int64_t offset, nsIOutputStream *
     NS_ENSURE_SUCCESS(rv, rv);
   }
 
+  nsRefPtr<CacheOutputCloseListener> listener =
+    new CacheOutputCloseListener(this);
+
   nsCOMPtr<nsIOutputStream> stream;
-  rv = mFile->OpenOutputStream(getter_AddRefs(stream));
+  rv = mFile->OpenOutputStream(listener, getter_AddRefs(stream));
   NS_ENSURE_SUCCESS(rv, rv);
 
   nsCOMPtr<nsISeekableStream> seekable =
@@ -879,6 +1029,14 @@ NS_IMETHODIMP CacheEntry::GetPredictedDataSize(int64_t *aPredictedDataSize)
 NS_IMETHODIMP CacheEntry::SetPredictedDataSize(int64_t aPredictedDataSize)
 {
   mPredictedDataSize = aPredictedDataSize;
+
+  if (CacheObserver::EntryIsTooBig(mPredictedDataSize, mUseDisk)) {
+    LOG(("CacheEntry::SetPredictedDataSize [this=%p] too big, dooming", this));
+    AsyncDoom(nullptr);
+
+    return NS_ERROR_FILE_TOO_BIG;
+  }
+
   return NS_OK;
 }
 
@@ -894,16 +1052,15 @@ NS_IMETHODIMP CacheEntry::GetSecurityInfo(nsISupports * *aSecurityInfo)
 
   NS_ENSURE_SUCCESS(mFileStatus, NS_ERROR_NOT_AVAILABLE);
 
-  char const* info;
+  nsXPIDLCString info;
   nsCOMPtr<nsISupports> secInfo;
   nsresult rv;
 
-  rv = mFile->GetElement("security-info", &info);
+  rv = mFile->GetElement("security-info", getter_Copies(info));
   NS_ENSURE_SUCCESS(rv, rv);
 
   if (info) {
-    rv = NS_DeserializeObject(nsDependentCString(info),
-                              getter_AddRefs(secInfo));
+    rv = NS_DeserializeObject(info, getter_AddRefs(secInfo));
     NS_ENSURE_SUCCESS(rv, rv);
   }
 
@@ -924,17 +1081,11 @@ NS_IMETHODIMP CacheEntry::SetSecurityInfo(nsISupports *aSecurityInfo)
 
   NS_ENSURE_SUCCESS(mFileStatus, mFileStatus);
 
-  nsRefPtr<CacheFile> file;
   {
     mozilla::MutexAutoLock lock(mLock);
 
     mSecurityInfo = aSecurityInfo;
     mSecurityInfoLoaded = true;
-
-    if (!mFile)
-      return NS_ERROR_NOT_AVAILABLE;
-
-    file = mFile;
   }
 
   nsCOMPtr<nsISerializable> serializable =
@@ -980,11 +1131,14 @@ NS_IMETHODIMP CacheEntry::AsyncDoom(nsICacheEntryDoomCallback *aCallback)
 
     mIsDoomed = true;
     mDoomCallback = aCallback;
-    BackgroundOp(Ops::DOOM);
   }
 
-  // Immediately remove the entry from the storage hash table
-  CacheStorageService::Self()->RemoveEntry(this);
+  // This immediately removes the entry from the master hashtable and also
+  // immediately dooms the file.  This way we make sure that any consumer
+  // after this point asking for the same entry won't get
+  //   a) this entry
+  //   b) a new entry with the same file
+  PurgeAndDoom();
 
   return NS_OK;
 }
@@ -993,15 +1147,7 @@ NS_IMETHODIMP CacheEntry::GetMetaDataElement(const char * aKey, char * *aRetval)
 {
   NS_ENSURE_SUCCESS(mFileStatus, NS_ERROR_NOT_AVAILABLE);
 
-  const char *value;
-  nsresult rv = mFile->GetElement(aKey, &value);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  if (!value)
-    return NS_ERROR_NOT_AVAILABLE;
-
-  *aRetval = NS_strdup(value);
-  return NS_OK;
+  return mFile->GetElement(aKey, aRetval);
 }
 
 NS_IMETHODIMP CacheEntry::SetMetaDataElement(const char * aKey, const char * aValue)
@@ -1054,15 +1200,15 @@ NS_IMETHODIMP CacheEntry::SetValid()
   return NS_OK;
 }
 
-NS_IMETHODIMP CacheEntry::Recreate(nsICacheEntry **_retval)
+NS_IMETHODIMP CacheEntry::Recreate(bool aMemoryOnly,
+                                   nsICacheEntry **_retval)
 {
   LOG(("CacheEntry::Recreate [this=%p, state=%s]", this, StateString(mState)));
 
   mozilla::MutexAutoLock lock(mLock);
 
-  nsRefPtr<CacheEntry> newEntry = ReopenTruncated(nullptr);
-  if (newEntry) {
-    nsRefPtr<Handle> handle = newEntry->NewWriteHandle();
+  nsRefPtr<CacheEntryHandle> handle = ReopenTruncated(aMemoryOnly, nullptr);
+  if (handle) {
     handle.forget(_retval);
     return NS_OK;
   }
@@ -1221,8 +1367,12 @@ bool CacheEntry::Purge(uint32_t aWhat)
   case PURGE_WHOLE_ONLY_DISK_BACKED:
   case PURGE_WHOLE:
     {
+      if (!CacheStorageService::Self()->RemoveEntry(this, true)) {
+        LOG(("  not purging, still referenced"));
+        return false;
+      }
+
       CacheStorageService::Self()->UnregisterEntry(this);
-      CacheStorageService::Self()->RemoveEntry(this);
 
       // Entry removed it self from control arrays, return true
       return true;
@@ -1247,8 +1397,6 @@ void CacheEntry::PurgeAndDoom()
 {
   LOG(("CacheEntry::PurgeAndDoom [this=%p]", this));
 
-  MOZ_ASSERT(CacheStorageService::IsOnManagementThread());
-
   CacheStorageService::Self()->RemoveEntry(this);
   DoomAlreadyRemoved();
 }
@@ -1257,36 +1405,45 @@ void CacheEntry::DoomAlreadyRemoved()
 {
   LOG(("CacheEntry::DoomAlreadyRemoved [this=%p]", this));
 
+  mozilla::MutexAutoLock lock(mLock);
+
   mIsDoomed = true;
 
-  if (!CacheStorageService::IsOnManagementThread()) {
-    mozilla::MutexAutoLock lock(mLock);
+  // This schedules dooming of the file, dooming is ensured to happen
+  // sooner than demand to open the same file made after this point
+  // so that we don't get this file for any newer opened entry(s).
+  DoomFile();
 
-    BackgroundOp(Ops::DOOM);
-    return;
-  }
+  // Must force post here since may be indirectly called from
+  // InvokeCallbacks of this entry and we don't want reentrancy here.
+  BackgroundOp(Ops::CALLBACKS, true);
+  // Process immediately when on the management thread.
+  BackgroundOp(Ops::UNREGISTER);
+}
 
-  CacheStorageService::Self()->UnregisterEntry(this);
-
-  {
-    mozilla::MutexAutoLock lock(mLock);
-
-    if (mCallbacks.Length() || mReadOnlyCallbacks.Length()) {
-      // Must force post here since may be indirectly called from
-      // InvokeCallbacks of this entry and we don't want reentrancy here.
-      BackgroundOp(Ops::CALLBACKS, true);
-    }
-  }
+void CacheEntry::DoomFile()
+{
+  nsresult rv = NS_ERROR_NOT_AVAILABLE;
 
   if (NS_SUCCEEDED(mFileStatus)) {
-    nsresult rv = mFile->Doom(mDoomCallback ? this : nullptr);
+    // Always calls the callback asynchronously.
+    rv = mFile->Doom(mDoomCallback ? this : nullptr);
     if (NS_SUCCEEDED(rv)) {
       LOG(("  file doomed"));
       return;
     }
+    
+    if (NS_ERROR_FILE_NOT_FOUND == rv) {
+      // File is set to be just memory-only, notify the callbacks
+      // and pretend dooming has succeeded.  From point of view of
+      // the entry it actually did - the data is gone and cannot be
+      // reused.
+      rv = NS_OK;
+    }
   }
 
-  OnFileDoomed(NS_OK);
+  // Always posts to the main thread.
+  OnFileDoomed(rv);
 }
 
 void CacheEntry::BackgroundOp(uint32_t aOperations, bool aForceAsync)
@@ -1310,8 +1467,8 @@ void CacheEntry::BackgroundOp(uint32_t aOperations, bool aForceAsync)
     #define M_LN2 0.69314718055994530942
     #endif
 
-    // Half-life is 90 days.
-    static double const half_life = 90.0 * (24 * 60 * 60);
+    // Half-life is dynamic, in seconds.
+     static double half_life = CacheObserver::HalfLifeSeconds();
     // Must convert from seconds to milliseconds since PR_Now() gives usecs.
     static double const decay = (M_LN2 / half_life) / static_cast<double>(PR_USEC_PER_SEC);
 
@@ -1326,6 +1483,12 @@ void CacheEntry::BackgroundOp(uint32_t aOperations, bool aForceAsync)
       mFrecency = log(exp(mFrecency - now_decay) + 1) + now_decay;
     }
     LOG(("CacheEntry FRECENCYUPDATE [this=%p, frecency=%1.10f]", this, mFrecency));
+
+    // Because CacheFile::Set*() are not thread-safe to use (uses WeakReference that
+    // is not thread-safe) we must post to the main thread...
+    nsRefPtr<nsRunnableMethod<CacheEntry> > event =
+      NS_NewRunnableMethod(this, &CacheEntry::StoreFrecency);
+    NS_DispatchToMainThread(event);
   }
 
   if (aOperations & Ops::REGISTER) {
@@ -1334,10 +1497,10 @@ void CacheEntry::BackgroundOp(uint32_t aOperations, bool aForceAsync)
     CacheStorageService::Self()->RegisterEntry(this);
   }
 
-  if (aOperations & Ops::DOOM) {
-    LOG(("CacheEntry DOOM [this=%p]", this));
+  if (aOperations & Ops::UNREGISTER) {
+    LOG(("CacheEntry UNREGISTER [this=%p]", this));
 
-    DoomAlreadyRemoved();
+    CacheStorageService::Self()->UnregisterEntry(this);
   }
 
   if (aOperations & Ops::CALLBACKS) {
@@ -1348,5 +1511,75 @@ void CacheEntry::BackgroundOp(uint32_t aOperations, bool aForceAsync)
   }
 }
 
+void CacheEntry::StoreFrecency()
+{
+  // No need for thread safety over mFrecency, it will be rewriten
+  // correctly on following invocation if broken by concurrency.
+  MOZ_ASSERT(NS_IsMainThread());
+  mFile->SetFrecency(FRECENCY2INT(mFrecency));
+}
+
+// CacheOutputCloseListener
+
+CacheOutputCloseListener::CacheOutputCloseListener(CacheEntry* aEntry)
+: mEntry(aEntry)
+{
+  MOZ_COUNT_CTOR(CacheOutputCloseListener);
+}
+
+CacheOutputCloseListener::~CacheOutputCloseListener()
+{
+  MOZ_COUNT_DTOR(CacheOutputCloseListener);
+}
+
+void CacheOutputCloseListener::OnOutputClosed()
+{
+  // We need this class and to redispatch since this callback is invoked
+  // under the file's lock and to do the job we need to enter the entry's
+  // lock too.  That would lead to potential deadlocks.
+  NS_DispatchToCurrentThread(this);
+}
+
+NS_IMETHODIMP CacheOutputCloseListener::Run()
+{
+  mEntry->OnOutputClosed();
+  return NS_OK;
+}
+
+// Memory reporting
+
+size_t CacheEntry::SizeOfExcludingThis(mozilla::MallocSizeOf mallocSizeOf) const
+{
+  size_t n = 0;
+  nsCOMPtr<nsISizeOf> sizeOf;
+
+  n += mCallbacks.SizeOfExcludingThis(mallocSizeOf);
+  if (mFile) {
+    n += mFile->SizeOfIncludingThis(mallocSizeOf);
+  }
+
+  sizeOf = do_QueryInterface(mURI);
+  if (sizeOf) {
+    n += sizeOf->SizeOfIncludingThis(mallocSizeOf);
+  }
+
+  n += mEnhanceID.SizeOfExcludingThisIfUnshared(mallocSizeOf);
+  n += mStorageID.SizeOfExcludingThisIfUnshared(mallocSizeOf);
+
+  // mDoomCallback is an arbitrary class that is probably reported elsewhere.
+  // mOutputStream is reported in mFile.
+  // mWriter is one of many handles we create, but (intentionally) not keep
+  // any reference to, so those unfortunatelly cannot be reported.  Handles are
+  // small, though.
+  // mSecurityInfo doesn't impl nsISizeOf.
+
+  return n;
+}
+
+size_t CacheEntry::SizeOfIncludingThis(mozilla::MallocSizeOf mallocSizeOf) const
+{
+  return mallocSizeOf(this) + SizeOfExcludingThis(mallocSizeOf);
+}
+
 } // net
 } // mozilla
