@@ -6,21 +6,35 @@
 #define CacheFileIOManager__h__
 
 #include "CacheIOThread.h"
-#include "CacheEntriesEnumerator.h"
 #include "nsIEventTarget.h"
 #include "nsITimer.h"
 #include "nsCOMPtr.h"
 #include "mozilla/SHA1.h"
+#include "mozilla/TimeStamp.h"
 #include "nsTArray.h"
 #include "nsString.h"
 #include "nsTHashtable.h"
 #include "prio.h"
 
+//#define DEBUG_HANDLES 1
+
 class nsIFile;
+class nsITimer;
+class nsIDirectoryEnumerator;
 
 namespace mozilla {
 namespace net {
 
+class CacheFile;
+#ifdef DEBUG_HANDLES
+class CacheFileHandlesEntry;
+#endif
+
+const char kEntriesDir[] = "entries";
+const char kDoomedDir[]  = "doomed";
+const char kTrashDir[]   = "trash";
+
+
 class CacheFileHandle : public nsISupports
 {
 public:
@@ -28,15 +42,23 @@ public:
   bool DispatchRelease();
 
   CacheFileHandle(const SHA1Sum::Hash *aHash, bool aPriority);
+  CacheFileHandle(const nsACString &aKey, bool aPriority);
   CacheFileHandle(const CacheFileHandle &aOther);
-  bool IsDoomed() { return mIsDoomed; }
-  const SHA1Sum::Hash *Hash() { return mHash; }
-  int64_t FileSize() { return mFileSize; }
-  bool IsPriority() { return mPriority; }
-  bool FileExists() { return mFileExists; }
-  bool IsClosed() { return mClosed; }
+  void Log();
+  bool IsDoomed() const { return mIsDoomed; }
+  const SHA1Sum::Hash *Hash() const { return mHash; }
+  int64_t FileSize() const { return mFileSize; }
+  uint32_t FileSizeInK() const;
+  bool IsPriority() const { return mPriority; }
+  bool FileExists() const { return mFileExists; }
+  bool IsClosed() const { return mClosed; }
+  bool IsSpecialFile() const { return !mHash; }
   nsCString & Key() { return mKey; }
 
+  // Memory reporting
+  size_t SizeOfExcludingThis(mozilla::MallocSizeOf mallocSizeOf) const;
+  size_t SizeOfIncludingThis(mozilla::MallocSizeOf mallocSizeOf) const;
+
 private:
   friend class CacheFileIOManager;
   friend class CacheFileHandles;
@@ -68,9 +90,18 @@ public:
   nsresult NewHandle(const SHA1Sum::Hash *aHash, bool aPriority, CacheFileHandle **_retval);
   void     RemoveHandle(CacheFileHandle *aHandlle);
   void     GetAllHandles(nsTArray<nsRefPtr<CacheFileHandle> > *_retval);
+  void     GetActiveHandles(nsTArray<nsRefPtr<CacheFileHandle> > *_retval);
   void     ClearAll();
   uint32_t HandleCount();
 
+#ifdef DEBUG_HANDLES
+  void     Log(CacheFileHandlesEntry *entry);
+#endif
+
+  // Memory reporting
+  size_t SizeOfExcludingThis(mozilla::MallocSizeOf mallocSizeOf) const;
+  size_t SizeOfIncludingThis(mozilla::MallocSizeOf mallocSizeOf) const;
+
   class HandleHashKey : public PLDHashEntryHdr
   {
   public:
@@ -110,8 +141,8 @@ public:
     already_AddRefed<CacheFileHandle> GetNewestHandle();
     void GetHandles(nsTArray<nsRefPtr<CacheFileHandle> > &aResult);
 
-    SHA1Sum::Hash *Hash() { return mHash; }
-    bool IsEmpty() { return mHandles.Length() == 0; }
+    SHA1Sum::Hash *Hash() const { return mHash; }
+    bool IsEmpty() const { return mHandles.Length() == 0; }
 
     enum { ALLOW_MEMMOVE = true };
 
@@ -119,6 +150,10 @@ public:
     void AssertHandlesState();
 #endif
 
+    // Memory reporting
+    size_t SizeOfExcludingThis(mozilla::MallocSizeOf mallocSizeOf) const;
+    size_t SizeOfIncludingThis(mozilla::MallocSizeOf mallocSizeOf) const;
+
   private:
     nsAutoArrayPtr<SHA1Sum::Hash> mHash;
     // Use weak pointers since the hash table access is on a single thread
@@ -160,6 +195,7 @@ public:
                         nsresult aResult) = 0;
   NS_IMETHOD OnFileDoomed(CacheFileHandle *aHandle, nsresult aResult) = 0;
   NS_IMETHOD OnEOFSet(CacheFileHandle *aHandle, nsresult aResult) = 0;
+  NS_IMETHOD OnFileRenamed(CacheFileHandle *aHandle, nsresult aResult) = 0;
 };
 
 NS_DEFINE_STATIC_IID_ACCESSOR(CacheFileIOListener, CACHEFILEIOLISTENER_IID)
@@ -172,11 +208,11 @@ public:
   NS_DECL_NSITIMERCALLBACK
 
   enum {
-    OPEN       = 0U,
-    CREATE     = 1U,
-    CREATE_NEW = 2U,
-    PRIORITY   = 4U,
-    NOHASH     = 8U
+    OPEN         = 0U,
+    CREATE       = 1U,
+    CREATE_NEW   = 2U,
+    PRIORITY     = 4U,
+    SPECIAL_FILE = 8U
   };
 
   CacheFileIOManager();
@@ -186,6 +222,7 @@ public:
   static nsresult OnProfile();
   static already_AddRefed<nsIEventTarget> IOTarget();
   static already_AddRefed<CacheIOThread> IOThread();
+  static bool IsOnIOThread();
   static bool IsOnIOThreadOrCeased();
   static bool IsShutdown();
 
@@ -215,14 +252,32 @@ public:
   static nsresult TruncateSeekSetEOF(CacheFileHandle *aHandle,
                                      int64_t aTruncatePos, int64_t aEOFPos,
                                      CacheFileIOListener *aCallback);
+  static nsresult RenameFile(CacheFileHandle *aHandle,
+                             const nsACString &aNewName,
+                             CacheFileIOListener *aCallback);
+  static nsresult EvictIfOverLimit();
+  static nsresult EvictAll();
+
+  static nsresult InitIndexEntry(CacheFileHandle *aHandle,
+                                 uint32_t         aAppId,
+                                 bool             aAnonymous,
+                                 bool             aInBrowser);
+  static nsresult UpdateIndexEntry(CacheFileHandle *aHandle,
+                                   const uint32_t  *aFrecency,
+                                   const uint32_t  *aExpirationTime);
+
+  static nsresult UpdateIndexEntry();
 
   enum EEnumerateMode {
     ENTRIES,
     DOOMED
   };
 
-  static nsresult EnumerateEntryFiles(EEnumerateMode aMode,
-                                      CacheEntriesEnumerator** aEnumerator);
+  static void GetCacheDirectory(nsIFile** result);
+
+  // Memory reporting
+  static size_t SizeOfExcludingThis(mozilla::MallocSizeOf mallocSizeOf);
+  static size_t SizeOfIncludingThis(mozilla::MallocSizeOf mallocSizeOf);
 
 private:
   friend class CacheFileHandle;
@@ -237,6 +292,8 @@ private:
   friend class DoomFileByKeyEvent;
   friend class ReleaseNSPRHandleEvent;
   friend class TruncateSeekSetEOFEvent;
+  friend class RenameFileEvent;
+  friend class CacheIndex;
   friend class MetadataWriteScheduleEvent;
 
   virtual ~CacheFileIOManager();
@@ -247,6 +304,9 @@ private:
   nsresult OpenFileInternal(const SHA1Sum::Hash *aHash,
                             uint32_t aFlags,
                             CacheFileHandle **_retval);
+  nsresult OpenSpecialFileInternal(const nsACString &aKey,
+                                   uint32_t aFlags,
+                                   CacheFileHandle **_retval);
   nsresult CloseHandleInternal(CacheFileHandle *aHandle);
   nsresult ReadInternal(CacheFileHandle *aHandle, int64_t aOffset,
                         char *aBuf, int32_t aCount);
@@ -257,12 +317,27 @@ private:
   nsresult ReleaseNSPRHandleInternal(CacheFileHandle *aHandle);
   nsresult TruncateSeekSetEOFInternal(CacheFileHandle *aHandle,
                                       int64_t aTruncatePos, int64_t aEOFPos);
+  nsresult RenameFileInternal(CacheFileHandle *aHandle,
+                              const nsACString &aNewName);
+  nsresult EvictIfOverLimitInternal();
+  nsresult OverLimitEvictionInternal();
+  nsresult EvictAllInternal();
+
+  nsresult TrashDirectory(nsIFile *aFile);
+  static void OnTrashTimer(nsITimer *aTimer, void *aClosure);
+  nsresult StartRemovingTrash();
+  nsresult RemoveTrashInternal();
+  nsresult FindTrashDirToRemove();
 
   nsresult CreateFile(CacheFileHandle *aHandle);
-  static void GetHashStr(const SHA1Sum::Hash *aHash, nsACString &_retval);
+  static void HashToStr(const SHA1Sum::Hash *aHash, nsACString &_retval);
+  static nsresult StrToHash(const nsACString &aHash, SHA1Sum::Hash *_retval);
   nsresult GetFile(const SHA1Sum::Hash *aHash, nsIFile **_retval);
+  nsresult GetSpecialFile(const nsACString &aKey, nsIFile **_retval);
   nsresult GetDoomedFile(nsIFile **_retval);
-  nsresult CheckAndCreateDir(nsIFile *aFile, const char *aDir);
+  nsresult IsEmptyDirectory(nsIFile *aFile, bool *_retval);
+  nsresult CheckAndCreateDir(nsIFile *aFile, const char *aDir,
+                             bool aEnsureEmptyDir);
   nsresult CreateCacheTree();
   nsresult OpenNSPRHandle(CacheFileHandle *aHandle, bool aCreate = false);
   void     NSPRHandleUsed(CacheFileHandle *aHandle);
@@ -271,15 +346,26 @@ private:
   nsresult UnscheduleMetadataWriteInternal(CacheFile * aFile);
   nsresult ShutdownMetadataWriteSchedulingInternal();
 
-  static CacheFileIOManager  *gInstance;
-  bool                        mShuttingDown;
-  nsRefPtr<CacheIOThread>     mIOThread;
-  nsCOMPtr<nsIFile>           mCacheDirectory;
-  bool                        mTreeCreated;
-  CacheFileHandles            mHandles;
-  nsTArray<CacheFileHandle *> mHandlesByLastUsed;
-  nsTArray<nsRefPtr<CacheFile> > mScheduledMetadataWrites;
-  nsCOMPtr<nsITimer>          mMetadataWritesTimer;
+  // Memory reporting (private part)
+  size_t SizeOfExcludingThisInternal(mozilla::MallocSizeOf mallocSizeOf) const;
+
+  static CacheFileIOManager           *gInstance;
+  TimeStamp                            mStartTime;
+  bool                                 mShuttingDown;
+  nsRefPtr<CacheIOThread>              mIOThread;
+  nsCOMPtr<nsIFile>                    mCacheDirectory;
+  bool                                 mTreeCreated;
+  CacheFileHandles                     mHandles;
+  nsTArray<CacheFileHandle *>          mHandlesByLastUsed;
+  nsTArray<nsRefPtr<CacheFileHandle> > mSpecialHandles;
+  nsTArray<nsRefPtr<CacheFile> >       mScheduledMetadataWrites;
+  nsCOMPtr<nsITimer>                   mMetadataWritesTimer;
+  bool                                 mOverLimitEvicting;
+  bool                                 mRemovingTrashDirs;
+  nsCOMPtr<nsITimer>                   mTrashTimer;
+  nsCOMPtr<nsIFile>                    mTrashDir;
+  nsCOMPtr<nsIDirectoryEnumerator>     mTrashDirEnumerator;
+  nsTArray<nsCString>                  mFailedTrashDirs;
 };
 
 } // net
