@@ -8,40 +8,31 @@
 #define nsHttpChannel_h__
 
 #include "HttpBaseChannel.h"
-
-#include "nsHttpTransaction.h"
-#include "nsInputStreamPump.h"
-#include "nsThreadUtils.h"
 #include "nsTArray.h"
-
-#include "nsIHttpEventSink.h"
 #include "nsICachingChannel.h"
-#include "nsICacheEntryDescriptor.h"
-#include "nsICacheListener.h"
+#include "nsICacheEntry.h"
+#include "nsICacheEntryOpenCallback.h"
+#include "nsIDNSListener.h"
 #include "nsIApplicationCacheChannel.h"
-#include "nsIPrompt.h"
-#include "nsIResumableChannel.h"
 #include "nsIProtocolProxyCallback.h"
-#include "nsICancelable.h"
 #include "nsIHttpAuthenticableChannel.h"
-#include "nsIHttpChannelAuthProvider.h"
 #include "nsIAsyncVerifyRedirectCallback.h"
 #include "nsITimedChannel.h"
-#include "nsIFile.h"
 #include "nsIThreadRetargetableRequest.h"
 #include "nsIThreadRetargetableStreamListener.h"
-#include "nsDNSPrefetch.h"
+#include "nsWeakReference.h"
 #include "TimingStruct.h"
 #include "AutoClose.h"
-#include "mozilla/Telemetry.h"
 
-class nsAHttpConnection;
 class nsIPrincipal;
+class nsDNSPrefetch;
+class nsICacheEntryDescriptor;
+class nsICancelable;
+class nsIHttpChannelAuthProvider;
+class nsInputStreamPump;
 
 namespace mozilla { namespace net {
 
-class HttpCacheQuery;
-
 //-----------------------------------------------------------------------------
 // nsHttpChannel
 //-----------------------------------------------------------------------------
@@ -50,7 +41,7 @@ class nsHttpChannel : public HttpBaseChannel
                     , public HttpAsyncAborter<nsHttpChannel>
                     , public nsIStreamListener
                     , public nsICachingChannel
-                    , public nsICacheListener
+                    , public nsICacheEntryOpenCallback
                     , public nsITransportEventSink
                     , public nsIProtocolProxyCallback
                     , public nsIHttpAuthenticableChannel
@@ -59,6 +50,8 @@ class nsHttpChannel : public HttpBaseChannel
                     , public nsITimedChannel
                     , public nsIThreadRetargetableRequest
                     , public nsIThreadRetargetableStreamListener
+                    , public nsIDNSListener
+                    , public nsSupportsWeakReference
 {
 public:
     NS_DECL_ISUPPORTS_INHERITED
@@ -67,7 +60,7 @@ public:
     NS_DECL_NSITHREADRETARGETABLESTREAMLISTENER
     NS_DECL_NSICACHEINFOCHANNEL
     NS_DECL_NSICACHINGCHANNEL
-    NS_DECL_NSICACHELISTENER
+    NS_DECL_NSICACHEENTRYOPENCALLBACK
     NS_DECL_NSITRANSPORTEVENTSINK
     NS_DECL_NSIPROTOCOLPROXYCALLBACK
     NS_DECL_NSIPROXIEDCHANNEL
@@ -76,6 +69,7 @@ public:
     NS_DECL_NSIASYNCVERIFYREDIRECTCALLBACK
     NS_DECL_NSITIMEDCHANNEL
     NS_DECL_NSITHREADRETARGETABLEREQUEST
+    NS_DECL_NSIDNSLISTENER
 
     // nsIHttpAuthenticableChannel. We can't use
     // NS_DECL_NSIHTTPAUTHENTICABLECHANNEL because it duplicates cancel() and
@@ -111,6 +105,7 @@ public:
     NS_IMETHOD Cancel(nsresult status);
     NS_IMETHOD Suspend();
     NS_IMETHOD Resume();
+    NS_IMETHOD IsPending(bool *aIsPending);
     // nsIChannel
     NS_IMETHOD GetSecurityInfo(nsISupports **aSecurityInfo);
     NS_IMETHOD AsyncOpen(nsIStreamListener *listener, nsISupports *aContext);
@@ -144,12 +139,12 @@ public: /* internal necko use only */
     // is gone.  Needed for e10s (see HttpChannelParent::RecvDocumentChannelCleanup)
     class OfflineCacheEntryAsForeignMarker {
         nsCOMPtr<nsIApplicationCache> mApplicationCache;
-        nsCString mCacheKey;
+        nsCOMPtr<nsIURI> mCacheURI;
     public:
         OfflineCacheEntryAsForeignMarker(nsIApplicationCache* appCache,
-                                         const nsCSubstring& key)
+                                         nsIURI* aURI)
              : mApplicationCache(appCache)
-             , mCacheKey(key)
+             , mCacheURI(aURI)
         {}
 
         nsresult MarkAsForeign();
@@ -157,6 +152,39 @@ public: /* internal necko use only */
 
     OfflineCacheEntryAsForeignMarker* GetOfflineCacheEntryAsForeignMarker();
 
+    // Helper to keep cache callbacks wait flags consistent
+    class AutoCacheWaitFlags
+    {
+    public:
+      AutoCacheWaitFlags(nsHttpChannel* channel)
+        : mChannel(channel)
+        , mKeep(0)
+      {
+        // Flags must be set before entering any AsyncOpenCacheEntry call.
+        mChannel->mCacheEntriesToWaitFor =
+          nsHttpChannel::WAIT_FOR_CACHE_ENTRY |
+          nsHttpChannel::WAIT_FOR_OFFLINE_CACHE_ENTRY;
+      }
+
+      void Keep(uint32_t flags)
+      {
+        // Called after successful call to appropriate AsyncOpenCacheEntry call.
+        mKeep |= flags;
+      }
+
+      ~AutoCacheWaitFlags()
+      {
+        // Keep only flags those are left to be wait for.
+        mChannel->mCacheEntriesToWaitFor &= mKeep;
+      }
+
+    private:
+      nsHttpChannel* mChannel;
+      uint32_t mKeep : 2;
+    };
+
+    void ForcePending(bool aForcePending);
+
 private:
     typedef nsresult (nsHttpChannel::*nsContinueRedirectionFunc)(nsresult result);
 
@@ -208,24 +236,24 @@ private:
 
     // cache specific methods
     nsresult OpenCacheEntry(bool usingSSL);
-    nsresult OnOfflineCacheEntryAvailable(nsICacheEntryDescriptor *aEntry,
-                                          nsCacheAccessMode aAccess,
+    nsresult OnOfflineCacheEntryAvailable(nsICacheEntry *aEntry,
+                                          bool aNew,
+                                          nsIApplicationCache* aAppCache,
                                           nsresult aResult);
-    nsresult OpenNormalCacheEntry(bool usingSSL);
-    nsresult OnNormalCacheEntryAvailable(nsICacheEntryDescriptor *aEntry,
-                                         nsCacheAccessMode aAccess,
+    nsresult OnNormalCacheEntryAvailable(nsICacheEntry *aEntry,
+                                         bool aNew,
                                          nsresult aResult);
     nsresult OpenOfflineCacheEntryForWriting();
-    nsresult OnOfflineCacheEntryForWritingAvailable(
-        nsICacheEntryDescriptor *aEntry,
-        nsCacheAccessMode aAccess,
-        nsresult aResult);
-    nsresult OnCacheEntryAvailableInternal(nsICacheEntryDescriptor *entry,
-                                           nsCacheAccessMode access,
-                                           nsresult status);
+    nsresult OnOfflineCacheEntryForWritingAvailable(nsICacheEntry *aEntry,
+                                                    nsIApplicationCache* aAppCache,
+                                                    nsresult aResult);
+    nsresult OnCacheEntryAvailableInternal(nsICacheEntry *entry,
+                                      bool aNew,
+                                      nsIApplicationCache* aAppCache,
+                                      nsresult status);
     nsresult GenerateCacheKey(uint32_t postID, nsACString &key);
     nsresult UpdateExpirationTime();
-    nsresult CheckCache();
+    nsresult CheckPartial(nsICacheEntry* aEntry, int64_t *aSize, int64_t *aContentLength);
     bool ShouldUpdateOfflineCacheEntry();
     nsresult ReadFromCache(bool alreadyMarkedValid);
     void     CloseCacheEntry(bool doomOnFailure);
@@ -233,14 +261,12 @@ private:
     nsresult InitCacheEntry();
     void     UpdateInhibitPersistentCachingFlag();
     nsresult InitOfflineCacheEntry();
-    nsresult AddCacheEntryHeaders(nsICacheEntryDescriptor *entry);
-    nsresult StoreAuthorizationMetaData(nsICacheEntryDescriptor *entry);
+    nsresult AddCacheEntryHeaders(nsICacheEntry *entry);
+    nsresult StoreAuthorizationMetaData(nsICacheEntry *entry);
     nsresult FinalizeCacheEntry();
-    nsresult InstallCacheListener(uint32_t offset = 0);
-    nsresult InstallOfflineCacheListener();
+    nsresult InstallCacheListener(int64_t offset = 0);
+    nsresult InstallOfflineCacheListener(int64_t offset = 0);
     void     MaybeInvalidateCacheEntryForSubsequentGet();
-    nsCacheStoragePolicy DetermineStoragePolicy();
-    nsresult DetermineCacheAccess(nsCacheAccessMode *_retval);
     void     AsyncOnExamineCachedResponse();
 
     // Handle the bogus Content-Encoding Apache sometimes sends
@@ -268,7 +294,7 @@ private:
     void InvalidateCacheEntryForLocation(const char *location);
     void AssembleCacheKey(const char *spec, uint32_t postID, nsACString &key);
     nsresult CreateNewURI(const char *loc, nsIURI **newURI);
-    void DoInvalidateCacheEntry(const nsCString &key);
+    void DoInvalidateCacheEntry(nsIURI* aURI);
 
     // Ref RFC2616 13.10: "invalidation... MUST only be performed if
     // the host part is the same as in the Request-URI"
@@ -290,8 +316,14 @@ private:
     // and ensure the transaction is updated to use it.
     void UpdateAggregateCallbacks();
 
-    // Disk cache is skipped for some requests when it is behaving slowly
-    bool ShouldSkipCache();
+    static bool HasQueryString(nsHttpAtom method, nsIURI * uri);
+    bool ResponseWouldVary(nsICacheEntry* entry) const;
+    bool MustValidateBasedOnQueryUrl() const;
+    bool IsResumable(int64_t partialLen, int64_t contentLength,
+                     bool ignoreMissingPartialLen = false) const;
+    nsresult MaybeSetupByteRangeRequest(int64_t partialLen, int64_t contentLength);
+    nsresult SetupByteRangeRequest(int64_t partialLen);
+    nsresult OpenCacheInputStream(nsICacheEntry* cacheEntry, bool startBuffering);
 
 private:
     nsCOMPtr<nsISupports>             mSecurityInfo;
@@ -303,24 +335,16 @@ private:
     uint64_t                          mLogicalOffset;
 
     // cache specific data
-    nsRefPtr<HttpCacheQuery>          mCacheQuery;
-    nsCOMPtr<nsICacheEntryDescriptor> mCacheEntry;
+    nsCOMPtr<nsICacheEntry>           mCacheEntry;
     // We must close mCacheInputStream explicitly to avoid leaks.
     AutoClose<nsIInputStream>         mCacheInputStream;
     nsRefPtr<nsInputStreamPump>       mCachePump;
     nsAutoPtr<nsHttpResponseHead>     mCachedResponseHead;
     nsCOMPtr<nsISupports>             mCachedSecurityInfo;
-    nsCacheAccessMode                 mCacheAccess;
-    mozilla::Telemetry::ID            mCacheEntryDeviceTelemetryID;
     uint32_t                          mPostID;
     uint32_t                          mRequestTime;
 
-    typedef nsresult (nsHttpChannel:: *nsOnCacheEntryAvailableCallback)(
-        nsICacheEntryDescriptor *, nsCacheAccessMode, nsresult);
-    nsOnCacheEntryAvailableCallback   mOnCacheEntryAvailableCallback;
-
-    nsCOMPtr<nsICacheEntryDescriptor> mOfflineCacheEntry;
-    nsCacheAccessMode                 mOfflineCacheAccess;
+    nsCOMPtr<nsICacheEntry> mOfflineCacheEntry;
     uint32_t                          mOfflineCacheLastModifiedTime;
     nsCOMPtr<nsIApplicationCache>     mApplicationCacheForWrite;
 
@@ -334,12 +358,14 @@ private:
 
     friend class AutoRedirectVetoNotifier;
     friend class HttpAsyncAborter<nsHttpChannel>;
-    friend class HttpCacheQuery;
 
     nsCOMPtr<nsIURI>                  mRedirectURI;
     nsCOMPtr<nsIChannel>              mRedirectChannel;
     uint32_t                          mRedirectType;
 
+    static const uint32_t WAIT_FOR_CACHE_ENTRY = 1;
+    static const uint32_t WAIT_FOR_OFFLINE_CACHE_ENTRY = 2;
+
     // state flags
     uint32_t                          mCachedContentIsValid     : 1;
     uint32_t                          mCachedContentIsPartial   : 1;
@@ -360,14 +386,28 @@ private:
     // True if mRequestTime has been set. In such a case it is safe to update
     // the cache entry's expiration time. Otherwise, it is not(see bug 567360).
     uint32_t                          mRequestTimeInitialized : 1;
+    uint32_t                          mCacheEntryIsReadOnly : 1;
+    uint32_t                          mCacheEntryIsWriteOnly : 1;
+    // see WAIT_FOR_* constants above
+    uint32_t                          mCacheEntriesToWaitFor : 2;
+    uint32_t                          mHasQueryString : 1;
+    // whether cache entry data write was in progress during cache entry check
+    // when true, after we finish read from cache we must check all data
+    // had been loaded from cache. If not, then an error has to be propagated
+    // to the consumer.
+    uint32_t                          mConcurentCacheAccess : 1;
+    // whether the request is setup be byte-range
+    uint32_t                          mIsPartialRequest : 1;
+    // true iff there is AutoRedirectVetoNotifier on the stack
+    uint32_t                          mHasAutoRedirectVetoNotifier : 1;
 
     nsTArray<nsContinueRedirectionFunc> mRedirectFuncStack;
 
     PRTime                            mChannelCreationTime;
-    mozilla::TimeStamp                mChannelCreationTimestamp;
-    mozilla::TimeStamp                mAsyncOpenTime;
-    mozilla::TimeStamp                mCacheReadStart;
-    mozilla::TimeStamp                mCacheReadEnd;
+    TimeStamp                         mChannelCreationTimestamp;
+    TimeStamp                         mAsyncOpenTime;
+    TimeStamp                         mCacheReadStart;
+    TimeStamp                         mCacheReadEnd;
     // copied from the transaction before we null out mTransaction
     // so that the timing can still be queried from OnStopRequest
     TimingStruct                      mTransactionTimings;
@@ -387,6 +427,7 @@ private: // cache telemetry
 private:
     nsIPrincipal *GetPrincipal();
     nsCOMPtr<nsIPrincipal> mPrincipal;
+    bool mForcePending;
 };
 
 } } // namespace mozilla::net
