@@ -30,6 +30,26 @@ from ..compilation.warnings import (
     WarningsDatabase,
 )
 
+from textwrap import TextWrapper
+
+INSTALL_TESTS_CLOBBER = ''.join([TextWrapper().fill(line) + '\n' for line in
+'''
+The build system was unable to install tests because the CLOBBER file has \
+been updated. This means if you edited any test files, your changes may not \
+be picked up until a full/clobber build is performed.
+
+The easiest and fastest way to perform a clobber build is to run:
+
+ $ mach clobber
+ $ mach build
+
+If you did not modify any test files, it is safe to ignore this message \
+and proceed with running tests. To do this run:
+
+ $ touch {clobber_file}
+'''.splitlines()])
+
+
 
 BuildOutputResult = namedtuple('BuildOutputResult',
     ('warning', 'state_changed', 'for_display'))
@@ -384,12 +404,17 @@ class BuildMonitor(MozbuildObject):
         if not record_usage:
             return
 
-        usage = self.record_resource_usage()
-        if not usage:
-            return
+        try:
+            usage = self.record_resource_usage()
+            if not usage:
+                return
 
-        with open(self._get_state_filename('build_resources.json'), 'w') as fh:
-            json.dump(usage, fh, indent=2)
+            with open(self._get_state_filename('build_resources.json'), 'w') as fh:
+                json.dump(usage, fh, indent=2)
+        except Exception as e:
+            self.log(logging.WARNING, 'build_resources_error',
+                {'msg': str(e)},
+                'Exception when writing resource usage file: {msg}')
 
     def _get_finder_cpu_usage(self):
         """Obtain the CPU usage of the Finder app on OS X.
@@ -448,6 +473,28 @@ class BuildMonitor(MozbuildObject):
 
         return finder_percent > 25, finder_percent
 
+    def have_excessive_swapping(self):
+        """Determine whether there was excessive swapping during the build.
+
+        Returns a tuple of (excessive, swap_in, swap_out). All values are None
+        if no swap information is available.
+        """
+        if not self.have_resource_usage:
+            return None, None, None
+
+        swap_in = sum(m.swap.sin for m in self.resources.measurements)
+        swap_out = sum(m.swap.sout for m in self.resources.measurements)
+
+        # The threshold of 1024 MB has been arbitrarily chosen.
+        #
+        # Choosing a proper value that is ideal for everyone is hard. We will
+        # likely iterate on the logic until people are generally satisfied.
+        # If a value is too low, the eventual warning produced does not carry
+        # much meaning. If the threshold is too high, people may not see the
+        # warning and the warning will thus be ineffective.
+        excessive = swap_in > 512 * 1048576 or swap_out > 512 * 1048576
+        return excessive, swap_in, swap_out
+
     @property
     def have_resource_usage(self):
         """Whether resource usage is available."""
@@ -475,6 +522,14 @@ class BuildMonitor(MozbuildObject):
         self._log_resource_usage('Overall system resources', 'resource_usage',
             self.end_time - self.start_time, cpu_percent, cpu_times, io)
 
+        excessive, sin, sout = self.have_excessive_swapping()
+        if excessive is not None and (sin or sout):
+            sin /= 1048576
+            sout /= 1048576
+            self.log(logging.WARNING, 'swap_activity',
+                {'sin': sin, 'sout': sout},
+                'Swap in/out (MB): {sin}/{sout}')
+
         o = dict(
             version=1,
             start=self.start_time,
@@ -540,6 +595,11 @@ class BuildDriver(MozbuildObject):
     def install_tests(self, remove=True):
         """Install test files (through manifest)."""
 
+        if self.is_clobber_needed():
+            print(INSTALL_TESTS_CLOBBER.format(
+                  clobber_file=os.path.join(self.topobjdir, 'CLOBBER')))
+            sys.exit(1)
+
         env = {}
         if not remove:
             env[b'NO_REMOVE'] = b'1'
