@@ -20,27 +20,32 @@ from __future__ import unicode_literals
 from collections import OrderedDict
 from mozbuild.util import (
     HierarchicalStringList,
+    HierarchicalStringListWithFlagsFactory,
     StrictOrderingOnAppendList,
+    StrictOrderingOnAppendListWithFlagsFactory,
 )
+from .sandbox import SandboxDerivedValue
+from types import StringTypes
+
+
+class FinalTargetValue(SandboxDerivedValue, unicode):
+    def __new__(cls, sandbox, value=""):
+        if not value:
+            value = 'dist/'
+            if sandbox['XPI_NAME']:
+                value += 'xpi-stage/' + sandbox['XPI_NAME']
+            else:
+                value += 'bin'
+            if sandbox['DIST_SUBDIR']:
+                value += '/' + sandbox['DIST_SUBDIR']
+        return unicode.__new__(cls, value)
 
 
-def compute_final_target(variables):
-    """Convert the default value for FINAL_TARGET"""
-    basedir = 'dist/'
-    if variables['XPI_NAME']:
-        basedir += 'xpi-stage/' + variables['XPI_NAME']
-    else:
-        basedir += 'bin'
-    if variables['DIST_SUBDIR']:
-        basedir += '/' + variables['DIST_SUBDIR']
-    return basedir
- 
- 
 # This defines the set of mutable global variables.
 #
 # Each variable is a tuple of:
 #
-#   (storage_type, input_types, default_value, docs, tier)
+#   (storage_type, input_types, docs, tier)
 #
 # Tier says for which specific tier the variable has an effect.
 # Valid tiers are:
@@ -55,7 +60,7 @@ def compute_final_target(variables):
 
 VARIABLES = {
     # Variables controlling reading of other frontend files.
-    'ANDROID_GENERATED_RESFILES': (StrictOrderingOnAppendList, list, [],
+    'ANDROID_GENERATED_RESFILES': (StrictOrderingOnAppendList, list,
         """Android resource files generated as part of the build.
 
         This variable contains a list of files that are expected to be
@@ -64,21 +69,67 @@ VARIABLES = {
         file.
         """, 'export'),
 
-    'ANDROID_RESFILES': (StrictOrderingOnAppendList, list, [],
-        """Android resource files.
+    'ANDROID_RES_DIRS': (list, list,
+        """Android resource directories.
 
-        This variable contains a list of files to package into a 'res'
+        This variable contains a list of directories, each relative to
+        the srcdir, containing static files to package into a 'res'
         directory and merge into an APK file.
         """, 'export'),
 
-    'SOURCES': (StrictOrderingOnAppendList, list, [],
+    'ANDROID_ECLIPSE_PROJECT_TARGETS': (dict, dict,
+        """Defines Android Eclipse project targets.
+
+        This variable should not be populated directly. Instead, it should
+        populated by calling add_android_eclipse{_library}_project().
+        """, 'export'),
+
+    'SOURCES': (StrictOrderingOnAppendListWithFlagsFactory({'no_pgo': bool, 'flags': list}), list,
         """Source code files.
 
         This variable contains a list of source code files to compile.
         Accepts assembler, C, C++, Objective C/C++.
         """, 'compile'),
 
-    'DEFINES': (OrderedDict, dict, OrderedDict(),
+    'GENERATED_SOURCES': (StrictOrderingOnAppendList, list,
+        """Generated source code files.
+
+        This variable contains a list of generated source code files to
+        compile. Accepts assembler, C, C++, Objective C/C++.
+        """, 'compile'),
+
+    'FILES_PER_UNIFIED_FILE': (int, int,
+        """The number of source files to compile into each unified source file.
+
+        """, 'None'),
+
+    'UNIFIED_SOURCES': (StrictOrderingOnAppendList, list,
+        """Source code files that can be compiled together.
+
+        This variable contains a list of source code files to compile,
+        that can be concatenated all together and built as a single source
+        file. This can help make the build faster and reduce the debug info
+        size.
+        """, 'compile'),
+
+    'GENERATED_UNIFIED_SOURCES': (StrictOrderingOnAppendList, list,
+        """Generated source code files that can be compiled together.
+
+        This variable contains a list of generated source code files to
+        compile, that can be concatenated all together, with UNIFIED_SOURCES,
+        and built as a single source file. This can help make the build faster
+        and reduce the debug info size.
+        """, 'compile'),
+
+    'GENERATED_FILES': (StrictOrderingOnAppendList, list,
+        """Generic generated files.
+
+        This variable contains a list of generate files for the build system
+        to generate at export time. The rules for those files still live in
+        Makefile.in.
+        """, 'export'),
+
+    'DEFINES': (OrderedDict, dict,
         """Dictionary of compiler defines to declare.
 
         These are passed in to the compiler as ``-Dkey='value'`` for string
@@ -106,7 +157,14 @@ VARIABLES = {
            })
         """, None),
 
-    'DIRS': (list, list, [],
+    'DELAYLOAD_DLLS': (list, list,
+        """Delay-loaded DLLs.
+
+        This variable contains a list of DLL files which the module being linked
+        should load lazily.  This only has an effect when building with MSVC.
+        """, 'binaries'),
+
+    'DIRS': (list, list,
         """Child directories to descend into looking for build frontend files.
 
         This works similarly to the ``DIRS`` variable in make files. Each str
@@ -120,18 +178,23 @@ VARIABLES = {
         delimiters.
         """, None),
 
-    'EXPORT_LIBRARY': (bool, bool, False,
+    'DISABLE_STL_WRAPPING': (bool, bool,
+        """Disable the wrappers for STL which allow it to work with C++ exceptions
+        disabled.
+        """, 'binaries'),
+
+    'EXPORT_LIBRARY': (bool, bool,
         """Install the library to the static libraries folder.
         """, None),
 
-    'EXTRA_COMPONENTS': (StrictOrderingOnAppendList, list, [],
+    'EXTRA_COMPONENTS': (StrictOrderingOnAppendList, list,
         """Additional component files to distribute.
 
        This variable contains a list of files to copy into
        ``$(FINAL_TARGET)/components/``.
         """, 'libs'),
 
-    'EXTRA_JS_MODULES': (StrictOrderingOnAppendList, list, [],
+    'EXTRA_JS_MODULES': (StrictOrderingOnAppendList, list,
         """Additional JavaScript files to distribute.
 
         This variable contains a list of files to copy into
@@ -139,7 +202,7 @@ VARIABLES = {
         to ``modules`` if left undefined.
         """, 'libs'),
 
-    'EXTRA_PP_JS_MODULES': (StrictOrderingOnAppendList, list, [],
+    'EXTRA_PP_JS_MODULES': (StrictOrderingOnAppendList, list,
         """Additional JavaScript files to distribute.
 
         This variable contains a list of files to copy into
@@ -147,56 +210,64 @@ VARIABLES = {
         ``JS_MODULES_PATH`` defaults to ``modules`` if left undefined.
         """, 'libs'),
 
-    'EXTRA_PP_COMPONENTS': (StrictOrderingOnAppendList, list, [],
+    'EXTRA_PP_COMPONENTS': (StrictOrderingOnAppendList, list,
         """Javascript XPCOM files.
 
        This variable contains a list of files to preprocess.  Generated
        files will be installed in the ``/components`` directory of the distribution.
         """, 'libs'),
 
-    'CPP_UNIT_TESTS': (StrictOrderingOnAppendList, list, [],
+    'FINAL_LIBRARY': (unicode, unicode,
+        """Library in which the objects of the current directory will be linked.
+
+        This variable contains the name of a library, defined elsewhere with
+        ``LIBRARY_NAME``, in which the objects of the current directory will be
+        linked.
+        """, 'binaries'),
+
+    'CPP_UNIT_TESTS': (StrictOrderingOnAppendList, list,
         """C++ source files for unit tests.
 
         This is a list of C++ unit test sources. Entries must be files that
         exist. These generally have ``.cpp`` extensions.
         """, 'binaries'),
 
-    'FAIL_ON_WARNINGS': (bool, bool, False,
+    'FAIL_ON_WARNINGS': (bool, bool,
         """Whether to treat warnings as errors.
         """, None),
 
-    'FORCE_SHARED_LIB': (bool, bool, False,
+    'FORCE_SHARED_LIB': (bool, bool,
         """Whether the library in this directory is a shared library.
         """, None),
 
-    'FORCE_STATIC_LIB': (bool, bool, False,
+    'FORCE_STATIC_LIB': (bool, bool,
         """Whether the library in this directory is a static library.
         """, None),
 
-    'GENERATED_INCLUDES' : (StrictOrderingOnAppendList, list, [],
+    'USE_STATIC_LIBS': (bool, bool,
+        """Whether the code in this directory is a built against the static
+        runtime library.
+
+        This variable only has an effect when building with MSVC.
+        """, None),
+
+    'GENERATED_INCLUDES' : (StrictOrderingOnAppendList, list,
         """Directories generated by the build system to be searched for include
         files by the compiler.
         """, None),
 
-    'GTEST_SOURCES': (StrictOrderingOnAppendList, list, [],
-        """Source code files for GTest unit tests.
-
-        This variable contains a list of GTEST unit test source files to
-        compile.
-        """, 'compile'),
-
-    'HOST_SOURCES': (StrictOrderingOnAppendList, list, [],
+    'HOST_SOURCES': (StrictOrderingOnAppendList, list,
         """Source code files to compile with the host compiler.
 
         This variable contains a list of source code files to compile.
         with the host compiler.
         """, 'compile'),
 
-    'IS_COMPONENT': (bool, bool, False,
+    'IS_COMPONENT': (bool, bool,
         """Whether the library contains a binary XPCOM component manifest.
         """, None),
 
-    'PARALLEL_DIRS': (list, list, [],
+    'PARALLEL_DIRS': (list, list,
         """A parallel version of ``DIRS``.
 
         Ideally this variable does not exist. It is provided so a transition
@@ -205,18 +276,18 @@ VARIABLES = {
         likely go away.
         """, None),
 
-    'HOST_LIBRARY_NAME': (unicode, unicode, "",
+    'HOST_LIBRARY_NAME': (unicode, unicode,
         """Name of target library generated when cross compiling.
         """, 'binaries'),
 
-    'JAVA_JAR_TARGETS': (dict, dict, {},
+    'JAVA_JAR_TARGETS': (dict, dict,
         """Defines Java JAR targets to be built.
 
         This variable should not be populated directly. Instead, it should
         populated by calling add_java_jar().
         """, 'binaries'),
 
-    'JS_MODULES_PATH': (unicode, unicode, "",
+    'JS_MODULES_PATH': (unicode, unicode,
         """Sub-directory of ``$(FINAL_TARGET)`` to install
         ``EXTRA_JS_MODULES``.
 
@@ -226,7 +297,7 @@ VARIABLES = {
         ``$(FINAL_TARGET)/modules``.
         """, None),
 
-    'LIBRARY_NAME': (unicode, unicode, "",
+    'LIBRARY_NAME': (unicode, unicode,
         """The name of the library generated for a directory.
 
         In ``example/components/moz.build``,::
@@ -237,54 +308,116 @@ VARIABLES = {
         ``example/components/xpcomsample.lib`` on Windows.
         """, 'binaries'),
 
-    'LIBS': (StrictOrderingOnAppendList, list, [],
+    'LIBS': (StrictOrderingOnAppendList, list,
         """Linker libraries and flags.
 
         A list of libraries and flags to include when linking.
         """, None),
 
-    'LIBXUL_LIBRARY': (bool, bool, False,
+    'LIBXUL_LIBRARY': (bool, bool,
         """Whether the library in this directory is linked into libxul.
 
-        Implies ``MOZILLA_INTERNAL_API`` and ``FORCE_STATIC_LIB``.
+        Implies ``FORCE_STATIC_LIB`` and the ``MOZILLA_INTERNAL_API``
+        preprocessor macro.
         """, None),
 
-    'LOCAL_INCLUDES': (StrictOrderingOnAppendList, list, [],
+    'LOCAL_INCLUDES': (StrictOrderingOnAppendList, list,
         """Additional directories to be searched for include files by the compiler.
         """, None),
 
-    'MSVC_ENABLE_PGO': (bool, bool, False,
-        """Whether profile-guided optimization is enabled in this directory.
+    'MSVC_ENABLE_PGO': (bool, bool,
+        """Whether profile-guided optimization is enabled for MSVC in this directory.
         """, None),
 
-    'OS_LIBS': (list, list, [],
+    'NO_PGO': (bool, bool,
+        """Whether profile-guided optimization is disable in this directory.
+        """, None),
+
+    'NO_VISIBILITY_FLAGS': (bool, bool,
+        """Build sources listed in this file without VISIBILITY_FLAGS.
+        """, None),
+
+    'OS_LIBS': (list, list,
         """System link libraries.
 
         This variable contains a list of system libaries to link against.
         """, None),
+    'RCFILE': (unicode, unicode,
+        """The program .rc file.
 
-    'SDK_LIBRARY': (StrictOrderingOnAppendList, list, [],
+        This variable can only be used on Windows.
+        """, None),
+
+    'RESFILE': (unicode, unicode,
+        """The program .res file.
+
+        This variable can only be used on Windows.
+        """, None),
+
+    'RCINCLUDE': (unicode, unicode,
+        """The resource script file to be included in the default .res file.
+
+        This variable can only be used on Windows.
+        """, None),
+
+    'DEFFILE': (unicode, unicode,
+        """The program .def (module definition) file.
+
+        This variable can only be used on Windows.
+        """, None),
+
+    'RESOURCE_FILES': (HierarchicalStringListWithFlagsFactory({'preprocess': bool}), list,
+        """List of resources to be exported, and in which subdirectories.
+
+        ``RESOURCE_FILES`` is used to list the resource files to be exported to
+        ``dist/bin/res``, but it can be used for other files as well. This variable
+        behaves as a list when appending filenames for resources in the top-level
+        directory. Files can also be appended to a field to indicate which
+        subdirectory they should be exported to. For example, to export
+        ``foo.res`` to the top-level directory, and ``bar.res`` to ``fonts/``,
+        append to ``RESOURCE_FILES`` like so::
+
+           RESOURCE_FILES += ['foo.res']
+           RESOURCE_FILES.fonts += ['bar.res']
+
+        Added files also have a 'preprocess' attribute, which will cause the
+        affected file to be run through the preprocessor, using any ``DEFINES``
+        set. It is used like this::
+
+           RESOURCE_FILES.fonts += ['baz.res.in']
+           RESOURCE_FILES.fonts['baz.res.in'].preprocess = True
+        """, None),
+
+    'SDK_LIBRARY': (StrictOrderingOnAppendList, list,
         """Elements of the distributed SDK.
 
         Files on this list will be copied into ``SDK_LIB_DIR``
         (``$DIST/sdk/lib``).
         """, None),
 
-    'SHARED_LIBRARY_LIBS': (StrictOrderingOnAppendList, list, [],
-        """Libraries linked into a shared library.
+    'SIMPLE_PROGRAMS': (StrictOrderingOnAppendList, list,
+        """Compile a list of executable names.
 
-        A list of static library paths which should be linked into the
-        current shared library.
-        """, None),
+        Each name in this variable corresponds to an executable built from the
+        corresponding source file with the same base name.
 
-    'SIMPLE_PROGRAMS': (StrictOrderingOnAppendList, list, [],
-        """Generate a list of binaries from source.
-
-        A list of sources, one per program, to compile & link with libs
-        into standalone programs.
+        If the configuration token ``BIN_SUFFIX`` is set, its value will be
+        automatically appended to each name. If a name already ends with
+        ``BIN_SUFFIX``, the name will remain unchanged.
         """, 'binaries'),
 
-    'TOOL_DIRS': (list, list, [],
+    'HOST_SIMPLE_PROGRAMS': (StrictOrderingOnAppendList, list,
+        """Compile a list of host executable names.
+
+        Each name in this variable corresponds to a hosst executable built
+        from the corresponding source file with the same base name.
+
+        If the configuration token ``HOST_BIN_SUFFIX`` is set, its value will
+        be automatically appended to each name. If a name already ends with
+        ``HOST_BIN_SUFFIX``, the name will remain unchanged.
+        """, 'binaries'),
+
+    'TOOL_DIRS': (list, list,
         """Like DIRS but for tools.
 
         Tools are for pieces of the build system that aren't required to
@@ -292,7 +425,7 @@ VARIABLES = {
         code and utilities.
         """, None),
 
-    'TEST_DIRS': (list, list, [],
+    'TEST_DIRS': (list, list,
         """Like DIRS but only for directories that contain test-only code.
 
         If tests are not enabled, this variable will be ignored.
@@ -301,12 +434,12 @@ VARIABLES = {
         complete.
         """, None),
 
-    'TEST_TOOL_DIRS': (list, list, [],
+    'TEST_TOOL_DIRS': (list, list,
         """TOOL_DIRS that is only executed if tests are enabled.
         """, None),
 
 
-    'TIERS': (OrderedDict, dict, OrderedDict(),
+    'TIERS': (OrderedDict, dict,
         """Defines directories constituting the tier traversal mechanism.
 
         The recursive make backend iteration is organized into tiers. There are
@@ -324,19 +457,7 @@ VARIABLES = {
         populated by calling add_tier_dir().
         """, None),
 
-    'EXTERNAL_MAKE_DIRS': (list, list, [],
-        """Directories that build with make but don't use moz.build files.
-
-        This is like ``DIRS`` except it implies that ``make`` is used to build the
-        directory and that the directory does not define itself with moz.build
-        files.
-        """, None),
-
-    'PARALLEL_EXTERNAL_MAKE_DIRS': (list, list, [],
-        """Parallel version of ``EXTERNAL_MAKE_DIRS``.
-        """, None),
-
-    'CONFIGURE_SUBST_FILES': (StrictOrderingOnAppendList, list, [],
+    'CONFIGURE_SUBST_FILES': (StrictOrderingOnAppendList, list,
         """Output files that will be generated using configure-like substitution.
 
         This is a substitute for ``AC_OUTPUT`` in autoconf. For each path in this
@@ -346,15 +467,15 @@ VARIABLES = {
         ``AC_SUBST`` variables declared during configure.
         """, None),
 
-    'MODULE': (unicode, unicode, "",
-        """Module name.
+    'CONFIGURE_DEFINE_FILES': (StrictOrderingOnAppendList, list,
+        """Output files generated from configure/config.status.
 
-        Historically, this variable was used to describe where to install header
-        files, but that feature is now handled by ``EXPORTS_NAMESPACES``.
-        ``MODULE`` will likely be removed in the future.
+        This is a substitute for ``AC_CONFIG_HEADER`` in autoconf. This is very
+        similar to ``CONFIGURE_SUBST_FILES`` except the generation logic takes
+        into account the values of ``AC_DEFINE`` instead of ``AC_SUBST``.
         """, None),
 
-    'EXPORTS': (HierarchicalStringList, list, HierarchicalStringList(),
+    'EXPORTS': (HierarchicalStringList, list,
         """List of files to be exported, and in which subdirectories.
 
         ``EXPORTS`` is generally used to list the include files to be exported to
@@ -369,7 +490,7 @@ VARIABLES = {
            EXPORTS.mozilla.dom += ['bar.h']
         """, None),
 
-    'PROGRAM' : (unicode, unicode, "",
+    'PROGRAM' : (unicode, unicode,
         """Compiled executable name.
 
         If the configuration token ``BIN_SUFFIX`` is set, its value will be
@@ -377,15 +498,31 @@ VARIABLES = {
         ``BIN_SUFFIX``, ``PROGRAM`` will remain unchanged.
         """, 'binaries'),
 
-    'NO_DIST_INSTALL': (bool, bool, False,
+    'HOST_PROGRAM' : (unicode, unicode,
+        """Compiled host executable name.
+
+        If the configuration token ``HOST_BIN_SUFFIX`` is set, its value will be
+        automatically appended to ``HOST_PROGRAM``. If ``HOST_PROGRAM`` already
+        ends with ``HOST_BIN_SUFFIX``, ``HOST_PROGRAM`` will remain unchanged.
+        """, 'binaries'),
+
+    'NO_DIST_INSTALL': (bool, bool,
         """Disable installing certain files into the distribution directory.
 
         If present, some files defined by other variables won't be
         distributed/shipped with the produced build.
         """, None),
 
+    'JAR_MANIFESTS': (StrictOrderingOnAppendList, list,
+        """JAR manifest files that should be processed as part of the build.
+
+        JAR manifests are files in the tree that define how to package files
+        into JARs and how chrome registration is performed. For more info,
+        see :ref:`jar_manifests`.
+        """, 'libs'),
+
     # IDL Generation.
-    'XPIDL_SOURCES': (StrictOrderingOnAppendList, list, [],
+    'XPIDL_SOURCES': (StrictOrderingOnAppendList, list,
         """XPCOM Interface Definition Files (xpidl).
 
         This is a list of files that define XPCOM interface definitions.
@@ -393,7 +530,7 @@ VARIABLES = {
         files.
         """, 'libs'),
 
-    'XPIDL_MODULE': (unicode, unicode, "",
+    'XPIDL_MODULE': (unicode, unicode,
         """XPCOM Interface Definition Module Name.
 
         This is the name of the ``.xpt`` file that is created by linking
@@ -401,93 +538,101 @@ VARIABLES = {
         as ``MODULE``.
         """, None),
 
-    'IPDL_SOURCES': (StrictOrderingOnAppendList, list, [],
+    'IPDL_SOURCES': (StrictOrderingOnAppendList, list,
         """IPDL source files.
 
         These are ``.ipdl`` files that will be parsed and converted to
         ``.cpp`` files.
         """, 'export'),
 
-    'WEBIDL_FILES': (StrictOrderingOnAppendList, list, [],
+    'WEBIDL_FILES': (StrictOrderingOnAppendList, list,
         """WebIDL source files.
 
         These will be parsed and converted to ``.cpp`` and ``.h`` files.
         """, 'export'),
 
-    'GENERATED_EVENTS_WEBIDL_FILES': (StrictOrderingOnAppendList, list, [],
+    'GENERATED_EVENTS_WEBIDL_FILES': (StrictOrderingOnAppendList, list,
         """WebIDL source files for generated events.
 
         These will be parsed and converted to ``.cpp`` and ``.h`` files.
         """, 'export'),
 
-    'TEST_WEBIDL_FILES': (StrictOrderingOnAppendList, list, [],
+    'TEST_WEBIDL_FILES': (StrictOrderingOnAppendList, list,
          """Test WebIDL source files.
 
          These will be parsed and converted to ``.cpp`` and ``.h`` files
          if tests are enabled.
          """, 'export'),
 
-    'GENERATED_WEBIDL_FILES': (StrictOrderingOnAppendList, list, [],
+    'GENERATED_WEBIDL_FILES': (StrictOrderingOnAppendList, list,
          """Generated WebIDL source files.
 
          These will be generated from some other files.
          """, 'export'),
 
-    'PREPROCESSED_TEST_WEBIDL_FILES': (StrictOrderingOnAppendList, list, [],
+    'PREPROCESSED_TEST_WEBIDL_FILES': (StrictOrderingOnAppendList, list,
          """Preprocessed test WebIDL source files.
 
          These will be preprocessed, then parsed and converted to .cpp
          and ``.h`` files if tests are enabled.
          """, 'export'),
 
-    'PREPROCESSED_WEBIDL_FILES': (StrictOrderingOnAppendList, list, [],
+    'PREPROCESSED_WEBIDL_FILES': (StrictOrderingOnAppendList, list,
          """Preprocessed WebIDL source files.
 
          These will be preprocessed before being parsed and converted.
          """, 'export'),
 
+    'WEBIDL_EXAMPLE_INTERFACES': (StrictOrderingOnAppendList, list,
+        """Names of example WebIDL interfaces to build as part of the build.
+
+        Names in this list correspond to WebIDL interface names defined in
+        WebIDL files included in the build from one of the \*WEBIDL_FILES
+        variables.
+        """, 'export'),
+
     # Test declaration.
-    'A11Y_MANIFESTS': (StrictOrderingOnAppendList, list, [],
+    'A11Y_MANIFESTS': (StrictOrderingOnAppendList, list,
         """List of manifest files defining a11y tests.
         """, None),
 
-    'BROWSER_CHROME_MANIFESTS': (StrictOrderingOnAppendList, list, [],
+    'BROWSER_CHROME_MANIFESTS': (StrictOrderingOnAppendList, list,
         """List of manifest files defining browser chrome tests.
         """, None),
 
-    'METRO_CHROME_MANIFESTS': (StrictOrderingOnAppendList, list, [],
+    'METRO_CHROME_MANIFESTS': (StrictOrderingOnAppendList, list,
         """List of manifest files defining metro browser chrome tests.
         """, None),
 
-    'MOCHITEST_CHROME_MANIFESTS': (StrictOrderingOnAppendList, list, [],
+    'MOCHITEST_CHROME_MANIFESTS': (StrictOrderingOnAppendList, list,
         """List of manifest files defining mochitest chrome tests.
         """, None),
 
-    'MOCHITEST_MANIFESTS': (StrictOrderingOnAppendList, list, [],
+    'MOCHITEST_MANIFESTS': (StrictOrderingOnAppendList, list,
         """List of manifest files defining mochitest tests.
         """, None),
 
-    'MOCHITEST_WEBAPPRT_CHROME_MANIFESTS': (StrictOrderingOnAppendList, list, [],
+    'MOCHITEST_WEBAPPRT_CHROME_MANIFESTS': (StrictOrderingOnAppendList, list,
         """List of manifest files defining webapprt mochitest chrome tests.
         """, None),
 
-    'WEBRTC_SIGNALLING_TEST_MANIFESTS': (StrictOrderingOnAppendList, list, [],
+    'WEBRTC_SIGNALLING_TEST_MANIFESTS': (StrictOrderingOnAppendList, list,
         """List of manifest files defining WebRTC signalling tests.
         """, None),
 
-    'XPCSHELL_TESTS_MANIFESTS': (StrictOrderingOnAppendList, list, [],
+    'XPCSHELL_TESTS_MANIFESTS': (StrictOrderingOnAppendList, list,
         """List of manifest files defining xpcshell tests.
         """, None),
 
     # The following variables are used to control the target of installed files.
-    'XPI_NAME': (unicode, unicode, "",
+    'XPI_NAME': (unicode, unicode,
         """The name of an extension XPI to generate.
 
         When this variable is present, the results of this directory will end up
         being packaged into an extension instead of the main dist/bin results.
         """, 'libs'),
 
-    'DIST_SUBDIR': (unicode, unicode, "",
+    'DIST_SUBDIR': (unicode, unicode,
         """The name of an alternate directory to install files to.
 
         When this variable is present, the results of this directory will end up
@@ -495,7 +640,7 @@ VARIABLES = {
         otherwise be placed.
         """, 'libs'),
 
-    'FINAL_TARGET': (unicode, unicode, compute_final_target,
+    'FINAL_TARGET': (FinalTargetValue, unicode,
         """The name of the directory to install targets to.
 
         The directory is relative to the top of the object directory. The
@@ -504,6 +649,115 @@ VARIABLES = {
         result is dist/xpi-stage/$(XPI_NAME). If DIST_SUBDIR is present, then
         the $(DIST_SUBDIR) directory of the otherwise default value is used.
         """, 'libs'),
+
+    'GYP_DIRS': (StrictOrderingOnAppendListWithFlagsFactory({
+            'variables': dict,
+            'input': unicode,
+            'sandbox_vars': dict,
+            'non_unified_sources': StrictOrderingOnAppendList,
+        }), list,
+        """Defines a list of object directories handled by gyp configurations.
+
+        Elements of this list give the relative object directory. For each
+        element of the list, GYP_DIRS may be accessed as a dictionary
+        (GYP_DIRS[foo]). The object this returns has attributes that need to be
+        set to further specify gyp processing:
+            - input, gives the path to the root gyp configuration file for that
+              object directory.
+            - variables, a dictionary containing variables and values to pass
+              to the gyp processor.
+            - sandbox_vars, a dictionary containing variables and values to
+              pass to the mozbuild processor on top of those derived from gyp
+              configuration.
+            - non_unified_sources, a list containing sources files, relative to
+              the current moz.build, that should be excluded from source file
+              unification.
+
+        Typical use looks like:
+            GYP_DIRS += ['foo', 'bar']
+            GYP_DIRS['foo'].input = 'foo/foo.gyp'
+            GYP_DIRS['foo'].variables = {
+                'foo': 'bar',
+                (...)
+            }
+            (...)
+        """, None),
+
+    'SPHINX_TREES': (dict, dict,
+        """Describes what the Sphinx documentation tree will look like.
+
+        Keys are relative directories inside the final Sphinx documentation
+        tree to install files into. Values are directories (relative to this
+        file) whose content to copy into the Sphinx documentation tree.
+        """, None),
+
+    'SPHINX_PYTHON_PACKAGE_DIRS': (StrictOrderingOnAppendList, list,
+        """Directories containing Python packages that Sphinx documents.
+        """, None),
+
+    'CFLAGS': (list, list,
+        """Flags passed to the C compiler for all of the C source files
+           declared in this directory.
+
+           Note that the ordering of flags matters here, these flags will be
+           added to the compiler's command line in the same order as they
+           appear in the moz.build file.
+        """, 'binaries'),
+
+    'CXXFLAGS': (list, list,
+        """Flags passed to the C++ compiler for all of the C++ source files
+           declared in this directory.
+
+           Note that the ordering of flags matters here; these flags will be
+           added to the compiler's command line in the same order as they
+           appear in the moz.build file.
+        """, 'binaries'),
+
+    'CMFLAGS': (list, list,
+        """Flags passed to the Objective-C compiler for all of the Objective-C
+           source files declared in this directory.
+
+           Note that the ordering of flags matters here; these flags will be
+           added to the compiler's command line in the same order as they
+           appear in the moz.build file.
+        """, 'binaries'),
+
+    'CMMFLAGS': (list, list,
+        """Flags passed to the Objective-C++ compiler for all of the
+           Objective-C++ source files declared in this directory.
+
+           Note that the ordering of flags matters here; these flags will be
+           added to the compiler's command line in the same order as they
+           appear in the moz.build file.
+        """, 'binaries'),
+
+    'LDFLAGS': (list, list,
+        """Flags passed to the linker when linking all of the libraries and
+           executables declared in this directory.
+
+           Note that the ordering of flags matters here; these flags will be
+           added to the linker's command line in the same order as they
+           appear in the moz.build file.
+        """, 'libs'),
+
+    'EXTRA_DSO_LDOPTS': (list, list,
+        """Flags passed to the linker when linking a shared library.
+
+           Note that the ordering of flags matter here, these flags will be
+           added to the linker's command line in the same order as they
+           appear in the moz.build file.
+        """, 'libs'),
+
+    'WIN32_EXE_LDFLAGS': (list, list,
+        """Flags passed to the linker when linking a Windows .exe executable
+           declared in this directory.
+
+           Note that the ordering of flags matter here, these flags will be
+           added to the linker's command line in the same order as they
+           appear in the moz.build file.
+
+           This variable only has an effect on Windows.
+        """, 'libs'),
 }
 
 # The set of functions exposed to the sandbox.
@@ -555,6 +809,33 @@ FUNCTIONS = {
         :py:class:`mozbuild.frontend.data.JavaJarData`.
         """),
 
+    'add_android_eclipse_project': ('_add_android_eclipse_project', (str, str),
+        """Declare an Android Eclipse project.
+
+        This is one of the supported ways to populate the
+        ANDROID_ECLIPSE_PROJECT_TARGETS variable.
+
+        The parameters are:
+        * name - project name.
+        * manifest - path to AndroidManifest.xml.
+
+        This returns a rich Android Eclipse project type, described at
+        :py:class:`mozbuild.frontend.data.AndroidEclipseProjectData`.
+        """),
+
+    'add_android_eclipse_library_project': ('_add_android_eclipse_library_project', (str,),
+        """Declare an Android Eclipse library project.
+
+        This is one of the supported ways to populate the
+        ANDROID_ECLIPSE_PROJECT_TARGETS variable.
+
+        The parameters are:
+        * name - project name.
+
+        This returns a rich Android Eclipse project type, described at
+        :py:class:`mozbuild.frontend.data.AndroidEclipseProjectData`.
+        """),
+
     'add_tier_dir': ('_add_tier_directory', (str, [str, list], bool, bool),
         """Register a directory for tier traversal.
 
