@@ -20,6 +20,7 @@ from mozbuild.frontend.data import (
     LocalInclude,
     Program,
     ReaderSummary,
+    Resources,
     SimpleProgram,
     TestManifest,
     VariablePassthru,
@@ -150,6 +151,7 @@ class TestEmitterBasic(unittest.TestCase):
             CMMSRCS=['fans.mm', 'tans.mm'],
             CSRCS=['fans.c', 'tans.c'],
             CPP_UNIT_TESTS=['foo.cpp'],
+            DISABLE_STL_WRAPPING=True,
             EXPORT_LIBRARY=True,
             EXTRA_COMPONENTS=['fans.js', 'tans.js'],
             EXTRA_PP_COMPONENTS=['fans.pp.js', 'tans.pp.js'],
@@ -169,6 +171,17 @@ class TestEmitterBasic(unittest.TestCase):
             SDK_LIBRARY=['fans.sdk', 'tans.sdk'],
             SSRCS=['bans.S', 'fans.S'],
             VISIBILITY_FLAGS='',
+            DELAYLOAD_LDFLAGS=['-DELAYLOAD:foo.dll', '-DELAYLOAD:bar.dll'],
+            USE_DELAYIMP=True,
+            RCFILE='foo.rc',
+            RESFILE='bar.res',
+            RCINCLUDE='bar.rc',
+            DEFFILE='baz.def',
+            USE_STATIC_LIBS=True,
+            MOZBUILD_CFLAGS=['-fno-exceptions', '-w'],
+            MOZBUILD_CXXFLAGS=['-fcxx-exceptions', '-include foo.h'],
+            MOZBUILD_LDFLAGS=['-framework Foo', '-x'],
+            WIN32_EXE_LDFLAGS=['-subsystem:console'],
         )
 
         variables = objs[0].variables
@@ -213,6 +226,63 @@ class TestEmitterBasic(unittest.TestCase):
         overwrite = exports._children['overwrite']
         self.assertEqual(overwrite.get_strings(), ['new.h'])
 
+    def test_resources(self):
+        reader = self.reader('resources')
+        objs = self.read_topsrcdir(reader)
+
+        expected_defines = reader.config.defines
+        expected_defines.update({
+            'FOO': True,
+            'BAR': 'BAZ',
+        })
+
+        self.assertEqual(len(objs), 2)
+        self.assertIsInstance(objs[0], Defines)
+        self.assertIsInstance(objs[1], Resources)
+
+        self.assertEqual(objs[1].defines, expected_defines)
+
+        resources = objs[1].resources
+        self.assertEqual(resources.get_strings(), ['foo.res', 'bar.res', 'baz.res',
+                                                   'foo_p.res.in', 'bar_p.res.in', 'baz_p.res.in'])
+        self.assertFalse(resources['foo.res'].preprocess)
+        self.assertFalse(resources['bar.res'].preprocess)
+        self.assertFalse(resources['baz.res'].preprocess)
+        self.assertTrue(resources['foo_p.res.in'].preprocess)
+        self.assertTrue(resources['bar_p.res.in'].preprocess)
+        self.assertTrue(resources['baz_p.res.in'].preprocess)
+
+        self.assertIn('mozilla', resources._children)
+        mozilla = resources._children['mozilla']
+        self.assertEqual(mozilla.get_strings(), ['mozilla1.res', 'mozilla2.res',
+                                                 'mozilla1_p.res.in', 'mozilla2_p.res.in'])
+        self.assertFalse(mozilla['mozilla1.res'].preprocess)
+        self.assertFalse(mozilla['mozilla2.res'].preprocess)
+        self.assertTrue(mozilla['mozilla1_p.res.in'].preprocess)
+        self.assertTrue(mozilla['mozilla2_p.res.in'].preprocess)
+
+        self.assertIn('dom', mozilla._children)
+        dom = mozilla._children['dom']
+        self.assertEqual(dom.get_strings(), ['dom1.res', 'dom2.res', 'dom3.res'])
+
+        self.assertIn('gfx', mozilla._children)
+        gfx = mozilla._children['gfx']
+        self.assertEqual(gfx.get_strings(), ['gfx.res'])
+
+        self.assertIn('vpx', resources._children)
+        vpx = resources._children['vpx']
+        self.assertEqual(vpx.get_strings(), ['mem.res', 'mem2.res'])
+
+        self.assertIn('nspr', resources._children)
+        nspr = resources._children['nspr']
+        self.assertIn('private', nspr._children)
+        private = nspr._children['private']
+        self.assertEqual(private.get_strings(), ['pprio.res', 'pprthred.res'])
+
+        self.assertIn('overwrite', resources._children)
+        overwrite = resources._children['overwrite']
+        self.assertEqual(overwrite.get_strings(), ['new.res'])
+
     def test_program(self):
         reader = self.reader('program')
         objs = self.read_topsrcdir(reader)
@@ -240,6 +310,80 @@ class TestEmitterBasic(unittest.TestCase):
         with self.assertRaisesRegexp(SandboxValidationError, 'Empty test manifest'):
             self.read_topsrcdir(reader)
 
+
+    def test_test_manifest_just_support_files(self):
+        """A test manifest with no tests but support-files is supported."""
+        reader = self.reader('test-manifest-just-support')
+
+        objs = self.read_topsrcdir(reader)
+        self.assertEqual(len(objs), 1)
+        o = objs[0]
+        self.assertEqual(len(o.installs), 2)
+        paths = sorted([k[len(o.directory)+1:] for k in o.installs.keys()])
+        self.assertEqual(paths, ["foo.txt", "just-support.ini"])
+
+    def test_test_manifest_support_files_disabled_test(self):
+        """A test manifest with just disabled tests and support-files is supported."""
+        reader = self.reader('test-manifest-support-disabled-tests')
+
+        objs = self.read_topsrcdir(reader)
+        self.assertEqual(len(objs), 1)
+        o = objs[0]
+        self.assertEqual(len(o.installs), 2)
+        paths = sorted([k[len(o.directory)+1:] for k in o.installs.keys()])
+        self.assertEqual(paths, ["foo.txt", "support-disabled-tests.ini"])
+
+    def test_test_manifest_absolute_support_files(self):
+        """Support files starting with '/' are placed relative to the install root"""
+        reader = self.reader('test-manifest-absolute-support')
+
+        objs = self.read_topsrcdir(reader)
+        self.assertEqual(len(objs), 1)
+        o = objs[0]
+        self.assertEqual(len(o.installs), 2)
+        expected = [
+            mozpath.normpath(mozpath.join(o.install_prefix, "../.well-known/foo.txt")),
+            mozpath.join(o.install_prefix, "absolute-support.ini"),
+        ]
+        paths = sorted([v[0] for v in o.installs.values()])
+        self.assertEqual(paths, expected)
+
+    def test_test_manifest_install_to_subdir(self):
+        """ """
+        reader = self.reader('test-manifest-install-subdir')
+
+        objs = self.read_topsrcdir(reader)
+        self.assertEqual(len(objs), 1)
+        o = objs[0]
+        self.assertEqual(len(o.installs), 3)
+        self.assertEqual(o.manifest_relpath, "subdir.ini")
+        self.assertEqual(o.manifest_obj_relpath, "subdir/subdir.ini")
+        expected = [
+            mozpath.normpath(mozpath.join(o.install_prefix, "subdir/subdir.ini")),
+            mozpath.normpath(mozpath.join(o.install_prefix, "subdir/support.txt")),
+            mozpath.normpath(mozpath.join(o.install_prefix, "subdir/test_foo.html")),
+        ]
+        paths = sorted([v[0] for v in o.installs.values()])
+        self.assertEqual(paths, expected)
+
+    def test_test_manifest_install_includes(self):
+        """Ensure that any [include:foo.ini] are copied to the objdir."""
+        reader = self.reader('test-manifest-install-includes')
+
+        objs = self.read_topsrcdir(reader)
+        self.assertEqual(len(objs), 1)
+        o = objs[0]
+        self.assertEqual(len(o.installs), 3)
+        self.assertEqual(o.manifest_relpath, "mochitest.ini")
+        self.assertEqual(o.manifest_obj_relpath, "subdir/mochitest.ini")
+        expected = [
+            mozpath.normpath(mozpath.join(o.install_prefix, "subdir/common.ini")),
+            mozpath.normpath(mozpath.join(o.install_prefix, "subdir/mochitest.ini")),
+            mozpath.normpath(mozpath.join(o.install_prefix, "subdir/test_foo.html")),
+        ]
+        paths = sorted([v[0] for v in o.installs.values()])
+        self.assertEqual(paths, expected)
+
     def test_test_manifest_keys_extracted(self):
         """Ensure all metadata from test manifests is extracted."""
         reader = self.reader('test-manifest-keys-extracted')
@@ -253,32 +397,32 @@ class TestEmitterBasic(unittest.TestCase):
             'a11y.ini': {
                 'flavor': 'a11y',
                 'installs': {
-                    'a11y.ini',
-                    'test_a11y.js',
+                    'a11y.ini': False,
+                    'test_a11y.js': True,
                 },
                 'pattern-installs': 1,
             },
             'browser.ini': {
                 'flavor': 'browser-chrome',
                 'installs': {
-                    'browser.ini',
-                    'test_browser.js',
-                    'support1',
-                    'support2',
+                    'browser.ini': False,
+                    'test_browser.js': True,
+                    'support1': False,
+                    'support2': False,
                 },
             },
             'metro.ini': {
                 'flavor': 'metro-chrome',
                 'installs': {
-                    'metro.ini',
-                    'test_metro.js',
+                    'metro.ini': False,
+                    'test_metro.js': True,
                 },
             },
             'mochitest.ini': {
                 'flavor': 'mochitest',
                 'installs': {
-                    'mochitest.ini',
-                    'test_mochitest.js',
+                    'mochitest.ini': False,
+                    'test_mochitest.js': True,
                 },
                 'external': {
                     'external1',
@@ -288,20 +432,20 @@ class TestEmitterBasic(unittest.TestCase):
             'chrome.ini': {
                 'flavor': 'chrome',
                 'installs': {
-                    'chrome.ini',
-                    'test_chrome.js',
+                    'chrome.ini': False,
+                    'test_chrome.js': True,
                 },
             },
             'xpcshell.ini': {
                 'flavor': 'xpcshell',
                 'dupe': True,
                 'installs': {
-                    'xpcshell.ini',
-                    'test_xpcshell.js',
-                    'head1',
-                    'head2',
-                    'tail1',
-                    'tail2',
+                    'xpcshell.ini': False,
+                    'test_xpcshell.js': True,
+                    'head1': False,
+                    'head2': False,
+                    'tail1': False,
+                    'tail2': False,
                 },
             },
         }
@@ -320,9 +464,10 @@ class TestEmitterBasic(unittest.TestCase):
             self.assertEqual(len(o.installs), len(m['installs']))
             for path in o.installs.keys():
                 self.assertTrue(path.startswith(o.directory))
-                path = path[len(o.directory)+1:]
+                relpath = path[len(o.directory)+1:]
 
-                self.assertIn(path, m['installs'])
+                self.assertIn(relpath, m['installs'])
+                self.assertEqual(o.installs[path][1], m['installs'][relpath])
 
             if 'pattern-installs' in m:
                 self.assertEqual(len(o.pattern_installs), m['pattern-installs'])
@@ -351,6 +496,30 @@ class TestEmitterBasic(unittest.TestCase):
         basenames = set(mozpath.basename(k) for k in o.installs.keys())
         self.assertEqual(basenames, {'mochitest.ini', 'test_active.html'})
 
+    def test_test_manifest_parent_support_files_dir(self):
+        """support-files referencing a file in a parent directory works."""
+        reader = self.reader('test-manifest-parent-support-files-dir')
+
+        objs = [o for o in self.read_topsrcdir(reader)
+                if isinstance(o, TestManifest)]
+
+        self.assertEqual(len(objs), 1)
+
+        o = objs[0]
+
+        expected = mozpath.join(o.srcdir, 'support-file.txt')
+        self.assertIn(expected, o.installs)
+        self.assertEqual(o.installs[expected],
+            ('testing/mochitest/tests/child/support-file.txt', False))
+
+    def test_test_manifest_missing_test_error(self):
+        """Missing test files should result in error."""
+        reader = self.reader('test-manifest-missing-test-file')
+
+        with self.assertRaisesRegexp(SandboxValidationError,
+            'lists test that does not exist: test_missing.html'):
+            self.read_topsrcdir(reader)
+
     def test_ipdl_sources(self):
         reader = self.reader('ipdl_sources')
         objs = self.read_topsrcdir(reader)
@@ -409,6 +578,7 @@ class TestEmitterBasic(unittest.TestCase):
             'BAZ': '"abcd"',
             'FOO': True,
             'VALUE': 'xyz',
+            'QUX': False,
         }
 
         self.assertEqual(defines, expected)
