@@ -5,6 +5,8 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "TransportSecurityInfo.h"
+
+#include "insanity/pkixtypes.h"
 #include "nsNSSComponent.h"
 #include "nsIWebProgressListener.h"
 #include "nsNSSCertificate.h"
@@ -15,9 +17,10 @@
 #include "nsIObjectInputStream.h"
 #include "nsIObjectOutputStream.h"
 #include "nsNSSCertHelper.h"
-#include "nsNSSCleaner.h"
 #include "nsIProgrammingLanguage.h"
 #include "nsIArray.h"
+#include "nsComponentManagerUtils.h"
+#include "nsServiceManagerUtils.h"
 #include "PSMRunnable.h"
 
 #include "secerr.h"
@@ -32,27 +35,16 @@
                        //we always write out to our own
                        //file.
 
-namespace {
-
-NSSCleanupAutoPtrClass(CERTCertificate, CERT_DestroyCertificate)
-
-static NS_DEFINE_CID(kNSSComponentCID, NS_NSSCOMPONENT_CID);
-
-} // unnamed namespace
-
 namespace mozilla { namespace psm {
 
 TransportSecurityInfo::TransportSecurityInfo()
   : mMutex("TransportSecurityInfo::mMutex"),
     mSecurityState(nsIWebProgressListener::STATE_IS_INSECURE),
-    mSubRequestsHighSecurity(0),
-    mSubRequestsLowSecurity(0),
     mSubRequestsBrokenSecurity(0),
     mSubRequestsNoSecurity(0),
     mErrorCode(0),
     mErrorMessageType(PlainErrorMessage),
-    mPort(0),
-    mIsCertIssuerBlacklisted(false)
+    mPort(0)
 {
 }
 
@@ -70,13 +62,13 @@ TransportSecurityInfo::virtualDestroyNSSReference()
 {
 }
 
-NS_IMPL_THREADSAFE_ISUPPORTS6(TransportSecurityInfo,
-                              nsITransportSecurityInfo,
-                              nsIInterfaceRequestor,
-                              nsISSLStatusProvider,
-                              nsIAssociatedContentSecurity,
-                              nsISerializable,
-                              nsIClassInfo)
+NS_IMPL_ISUPPORTS6(TransportSecurityInfo,
+                   nsITransportSecurityInfo,
+                   nsIInterfaceRequestor,
+                   nsISSLStatusProvider,
+                   nsIAssociatedContentSecurity,
+                   nsISerializable,
+                   nsIClassInfo)
 
 nsresult
 TransportSecurityInfo::SetHostName(const char* host)
@@ -139,40 +131,6 @@ TransportSecurityInfo::SetSecurityState(uint32_t aState)
   return NS_OK;
 }
 
-/* attribute unsigned long countSubRequestsHighSecurity; */
-NS_IMETHODIMP
-TransportSecurityInfo::GetCountSubRequestsHighSecurity(
-  int32_t *aSubRequestsHighSecurity)
-{
-  *aSubRequestsHighSecurity = mSubRequestsHighSecurity;
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-TransportSecurityInfo::SetCountSubRequestsHighSecurity(
-  int32_t aSubRequestsHighSecurity)
-{
-  mSubRequestsHighSecurity = aSubRequestsHighSecurity;
-  return NS_ERROR_NOT_IMPLEMENTED;
-}
-
-/* attribute unsigned long countSubRequestsLowSecurity; */
-NS_IMETHODIMP
-TransportSecurityInfo::GetCountSubRequestsLowSecurity(
-  int32_t *aSubRequestsLowSecurity)
-{
-  *aSubRequestsLowSecurity = mSubRequestsLowSecurity;
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-TransportSecurityInfo::SetCountSubRequestsLowSecurity(
-  int32_t aSubRequestsLowSecurity)
-{
-  mSubRequestsLowSecurity = aSubRequestsLowSecurity;
-  return NS_OK;
-}
-
 /* attribute unsigned long countSubRequestsBrokenSecurity; */
 NS_IMETHODIMP
 TransportSecurityInfo::GetCountSubRequestsBrokenSecurity(
@@ -214,26 +172,7 @@ TransportSecurityInfo::Flush()
 }
 
 NS_IMETHODIMP
-TransportSecurityInfo::GetShortSecurityDescription(PRUnichar** aText)
-{
-  if (mShortDesc.IsEmpty())
-    *aText = nullptr;
-  else {
-    *aText = ToNewUnicode(mShortDesc);
-    NS_ENSURE_TRUE(*aText, NS_ERROR_OUT_OF_MEMORY);
-  }
-  return NS_OK;
-}
-
-nsresult
-TransportSecurityInfo::SetShortSecurityDescription(const PRUnichar* aText)
-{
-  mShortDesc.Assign(aText);
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-TransportSecurityInfo::GetErrorMessage(PRUnichar** aText)
+TransportSecurityInfo::GetErrorMessage(char16_t** aText)
 {
   NS_ENSURE_ARG_POINTER(aText);
   *aText = nullptr;
@@ -253,7 +192,7 @@ TransportSecurityInfo::GetErrorMessage(PRUnichar** aText)
   }
 
   *aText = ToNewUnicode(mErrorMessageCached);
-  return *aText != nullptr ? NS_OK : NS_ERROR_OUT_OF_MEMORY;
+  return *aText ? NS_OK : NS_ERROR_OUT_OF_MEMORY;
 }
 
 void
@@ -338,9 +277,6 @@ TransportSecurityInfo::GetInterface(const nsIID & uuid, void * *result)
   nsresult rv;
   if (!mCallbacks) {
     nsCOMPtr<nsIInterfaceRequestor> ir = new PipUIContext();
-    if (!ir)
-      return NS_ERROR_OUT_OF_MEMORY;
-
     rv = ir->GetInterface(uuid, result);
   } else {
     rv = mCallbacks->GetInterface(uuid, result);
@@ -360,7 +296,7 @@ TransportSecurityInfo::Write(nsIObjectOutputStream* stream)
 
   MutexAutoLock lock(mMutex);
 
-  nsRefPtr<nsSSLStatus> status = mSSLStatus;
+  RefPtr<nsSSLStatus> status(mSSLStatus);
   nsCOMPtr<nsISerializable> certSerializable;
 
   // Write a redundant copy of the certificate for backward compatibility
@@ -399,7 +335,7 @@ TransportSecurityInfo::Write(nsIObjectOutputStream* stream)
   uint32_t version = 3;
   stream->Write32(version | 0xFFFF0000);
   stream->Write32(mSecurityState);
-  stream->WriteWStringZ(mShortDesc.get());
+  stream->WriteWStringZ(EmptyString().get()); 
 
   // XXX: uses nsNSSComponent string bundles off the main thread
   nsresult rv = formatErrorMessage(lock, 
@@ -411,8 +347,8 @@ TransportSecurityInfo::Write(nsIObjectOutputStream* stream)
   stream->WriteCompoundObject(NS_ISUPPORTS_CAST(nsISSLStatus*, status),
                               NS_GET_IID(nsISupports), true);
 
-  stream->Write32((uint32_t)mSubRequestsHighSecurity);
-  stream->Write32((uint32_t)mSubRequestsLowSecurity);
+  stream->Write32((uint32_t)0);
+  stream->Write32((uint32_t)0);
   stream->Write32((uint32_t)mSubRequestsBrokenSecurity);
   stream->Write32((uint32_t)mSubRequestsNoSecurity);
   return NS_OK;
@@ -500,7 +436,8 @@ TransportSecurityInfo::Read(nsIObjectInputStream* stream)
     mSecurityState = version;
     version = 1;
   }
-  stream->ReadString(mShortDesc);
+  nsAutoString dummyShortDesc;
+  stream->ReadString(dummyShortDesc);
   stream->ReadString(mErrorMessageCached);
   mErrorCode = 0;
 
@@ -514,14 +451,13 @@ TransportSecurityInfo::Read(nsIObjectInputStream* stream)
   }
 
   if (version >= 2) {
-    stream->Read32((uint32_t*)&mSubRequestsHighSecurity);
-    stream->Read32((uint32_t*)&mSubRequestsLowSecurity);
+    uint32_t dummySubRequests;
+    stream->Read32((uint32_t*)&dummySubRequests);
+    stream->Read32((uint32_t*)&dummySubRequests);
     stream->Read32((uint32_t*)&mSubRequestsBrokenSecurity);
     stream->Read32((uint32_t*)&mSubRequestsNoSecurity);
   }
   else {
-    mSubRequestsHighSecurity = 0;
-    mSubRequestsLowSecurity = 0;
     mSubRequestsBrokenSecurity = 0;
     mSubRequestsNoSecurity = 0;
   }
@@ -621,7 +557,9 @@ formatPlainErrorMessage(const nsXPIDLCString &host, int32_t port,
                         bool suppressPort443,
                         nsString &returnedMessage)
 {
-  const PRUnichar *params[1];
+  static NS_DEFINE_CID(kNSSComponentCID, NS_NSSCOMPONENT_CID);
+
+  const char16_t *params[1];
   nsresult rv;
 
   nsCOMPtr<nsINSSComponent> component = do_GetService(kNSSComponentCID, &rv);
@@ -696,9 +634,6 @@ AppendErrorTextUntrusted(PRErrorCode errTrust,
           errorID = "certErrorTrust_UnknownIssuer";
         break;
       }
-      case SEC_ERROR_INADEQUATE_KEY_USAGE:
-        // Should get an individual string in the future
-        // For now, use the same as CaInvalid
       case SEC_ERROR_CA_CERT_INVALID:
         errorID = "certErrorTrust_CaInvalid";
         break;
@@ -741,8 +676,8 @@ GetSubjectAltNames(CERTCertificate *nssCert,
   allNames.Truncate();
   nameCount = 0;
 
-  PRArenaPool *san_arena = nullptr;
-  SECItem altNameExtension = {siBuffer, NULL, 0 };
+  PLArenaPool *san_arena = nullptr;
+  SECItem altNameExtension = {siBuffer, nullptr, 0 };
   CERTGeneralName *sanNameList = nullptr;
 
   SECStatus rv = CERT_FindCertExtension(nssCert, SEC_OID_X509_SUBJECT_ALT_NAME,
@@ -767,7 +702,7 @@ GetSubjectAltNames(CERTCertificate *nssCert,
       case certDNSName:
         name.AssignASCII((char*)current->name.other.data, current->name.other.len);
         if (!allNames.IsEmpty()) {
-          allNames.Append(NS_LITERAL_STRING(" , "));
+          allNames.Append(NS_LITERAL_STRING(", "));
         }
         ++nameCount;
         allNames.Append(name);
@@ -792,7 +727,7 @@ GetSubjectAltNames(CERTCertificate *nssCert,
           }
           if (!name.IsEmpty()) {
             if (!allNames.IsEmpty()) {
-              allNames.Append(NS_LITERAL_STRING(" , "));
+              allNames.Append(NS_LITERAL_STRING(", "));
             }
             ++nameCount;
             allNames.Append(name);
@@ -817,11 +752,10 @@ AppendErrorTextMismatch(const nsString &host,
                         bool wantsHtml,
                         nsString &returnedMessage)
 {
-  const PRUnichar *params[1];
+  const char16_t *params[1];
   nsresult rv;
 
-  CERTCertificate *nssCert = NULL;
-  CERTCertificateCleaner nssCertCleaner(nssCert);
+  insanity::pkix::ScopedCERTCertificate nssCert;
 
   nsCOMPtr<nsIX509Cert2> cert2 = do_QueryInterface(ix509, &rv);
   if (cert2)
@@ -846,7 +780,7 @@ AppendErrorTextMismatch(const nsString &host,
   bool useSAN = false;
 
   if (nssCert)
-    useSAN = GetSubjectAltNames(nssCert, component, allNames, nameCount);
+    useSAN = GetSubjectAltNames(nssCert.get(), component, allNames, nameCount);
 
   if (!useSAN) {
     char *certName = nullptr;
@@ -875,7 +809,7 @@ AppendErrorTextMismatch(const nsString &host,
     }
   }
   else if (nameCount == 1) {
-    const PRUnichar *params[1];
+    const char16_t *params[1];
     params[0] = allNames.get();
     
     const char *stringID;
@@ -958,7 +892,7 @@ AppendErrorTextTime(nsIX509Cert* ix509,
   bool trueExpired_falseNotYetValid;
   GetDateBoundary(ix509, formattedDate, nowDate, trueExpired_falseNotYetValid);
 
-  const PRUnichar *params[2];
+  const char16_t *params[2];
   params[0] = formattedDate.get(); // might be empty, if helper function had a problem 
   params[1] = nowDate.get();
 
@@ -990,7 +924,7 @@ AppendErrorTextCode(PRErrorCode errorCodeToReport,
     ToLowerCase(error_id);
     NS_ConvertASCIItoUTF16 idU(error_id);
 
-    const PRUnichar *params[1];
+    const char16_t *params[1];
     params[0] = idU.get();
 
     nsString formattedString;
@@ -1023,7 +957,9 @@ formatOverridableCertErrorMessage(nsISSLStatus & sslStatus,
                                   bool wantsHtml,
                                   nsString & returnedMessage)
 {
-  const PRUnichar *params[1];
+  static NS_DEFINE_CID(kNSSComponentCID, NS_NSSCOMPONENT_CID);
+
+  const char16_t *params[1];
   nsresult rv;
   nsAutoString hostWithPort;
   nsAutoString hostWithoutPort;
@@ -1055,8 +991,8 @@ formatOverridableCertErrorMessage(nsISSLStatus & sslStatus,
 
   returnedMessage.Append(NS_LITERAL_STRING("\n\n"));
 
-  nsRefPtr<nsIX509Cert> ix509;
-  rv = sslStatus.GetServerCert(getter_AddRefs(ix509));
+  RefPtr<nsIX509Cert> ix509;
+  rv = sslStatus.GetServerCert(byRef(ix509));
   NS_ENSURE_SUCCESS(rv, rv);
 
   bool isUntrusted;
@@ -1092,9 +1028,9 @@ formatOverridableCertErrorMessage(nsISSLStatus & sslStatus,
 RememberCertErrorsTable::sInstance = nullptr;
 
 RememberCertErrorsTable::RememberCertErrorsTable()
-  : mMutex("RememberCertErrorsTable::mMutex")
+  : mErrorHosts(16)
+  , mMutex("RememberCertErrorsTable::mMutex")
 {
-  mErrorHosts.Init(16);
 }
 
 static nsresult
