@@ -70,6 +70,7 @@
 #include "certdb.h"
 #include "secmod.h"
 #include "ScopedNSSTypes.h"
+#include "insanity/pkixtypes.h"
 
 #include "ssl.h" // For SSL_ClearSessionCache
 
@@ -79,6 +80,9 @@
 #include <algorithm>
 #include "nsWrapperCacheInlines.h"
 #endif
+#ifndef MOZ_DISABLE_CRYPTOLEGACY
+#include "mozilla/dom/CRMFObjectBinding.h"
+#endif
 
 using namespace mozilla;
 using namespace mozilla::dom;
@@ -108,12 +112,6 @@ using namespace mozilla::dom;
 #define JS_ERR_BAD_CIPHER_ENABLE_FLAGS    -9
 #define JS_ERR_ADD_DUPLICATE_MOD          -10
 
-namespace {
-  
-NS_DEFINE_CID(kNSSComponentCID, NS_NSSCOMPONENT_CID);
-
-} // unnamed namespace
-
 #ifndef MOZ_DISABLE_CRYPTOLEGACY
 
 NSSCleanupAutoPtrClass_WithParam(PK11Context, PK11_DestroyContext, TrueParam, true)
@@ -219,16 +217,6 @@ NS_INTERFACE_MAP_END_INHERITING(mozilla::dom::Crypto)
 
 NS_IMPL_ADDREF_INHERITED(nsCrypto, mozilla::dom::Crypto)
 NS_IMPL_RELEASE_INHERITED(nsCrypto, mozilla::dom::Crypto)
- 
-// QueryInterface implementation for nsCRMFObject
-NS_INTERFACE_MAP_BEGIN(nsCRMFObject)
-  NS_INTERFACE_MAP_ENTRY(nsIDOMCRMFObject)
-  NS_INTERFACE_MAP_ENTRY(nsISupports)
-  NS_DOM_INTERFACE_MAP_ENTRY_CLASSINFO(CRMFObject)
-NS_INTERFACE_MAP_END
-
-NS_IMPL_ADDREF(nsCRMFObject)
-NS_IMPL_RELEASE(nsCRMFObject)
 
 // QueryInterface implementation for nsPkcs11
 #endif // MOZ_DISABLE_CRYPTOLEGACY
@@ -270,6 +258,8 @@ nsCrypto::Init(nsIDOMWindow* aWindow)
 void
 nsCrypto::SetEnableSmartCardEvents(bool aEnable, ErrorResult& aRv)
 {
+  NS_DEFINE_CID(kNSSComponentCID, NS_NSSCOMPONENT_CID);
+
   nsresult rv = NS_OK;
 
   // this has the side effect of starting the nssComponent (and initializing
@@ -553,11 +543,14 @@ nsConvertToActualKeyGenParams(uint32_t keyGenMech, char *params,
               next_input, name, name_len, value, value_len,
               next_input))
       {
-        if (PL_strncmp(name, "curve", std::min(name_len, 5)) == 0)
+        // use only the first specified curve
+        if (!curve && PL_strncmp(name, "curve", std::min(name_len, 5)) == 0)
         {
           curve = PL_strndup(value, value_len);
         }
-        else if (PL_strncmp(name, "popcert", std::min(name_len, 7)) == 0)
+        // use only the first specified popcert
+        else if (!keyPairInfo->ecPopCert &&
+                 PL_strncmp(name, "popcert", std::min(name_len, 7)) == 0)
         {
           char *certstr = PL_strndup(value, value_len);
           if (certstr) {
@@ -574,7 +567,7 @@ nsConvertToActualKeyGenParams(uint32_t keyGenMech, char *params,
     }
 
     // first try to use the params of the provided CA cert
-    if (keyPairInfo->ecPopPubKey)
+    if (keyPairInfo->ecPopPubKey && keyPairInfo->ecPopPubKey->keyType == ecKey)
     {
       returnParams = SECITEM_DupItem(&keyPairInfo->ecPopPubKey->u.ec.DEREncodedParams);
     }
@@ -954,7 +947,8 @@ cryptojs_ReadArgsAndGenerateKey(JSContext *cx,
   }
   keySize = JSVAL_TO_INT(argv[0]);
   if (!JSVAL_IS_NULL(argv[1])) {
-    jsString = JS_ValueToString(cx,argv[1]);
+    JS::Rooted<JS::Value> v(cx, argv[1]);
+    jsString = JS::ToString(cx, v);
     NS_ENSURE_TRUE(jsString, NS_ERROR_OUT_OF_MEMORY);
     argv[1] = STRING_TO_JSVAL(jsString);
     params.encodeLatin1(cx, jsString);
@@ -966,7 +960,8 @@ cryptojs_ReadArgsAndGenerateKey(JSContext *cx,
              "key generation type not specified");
     return NS_ERROR_FAILURE;
   }
-  jsString = JS_ValueToString(cx, argv[2]);
+  JS::Rooted<JS::Value> v(cx, argv[2]);
+  jsString = JS::ToString(cx, v);
   NS_ENSURE_TRUE(jsString, NS_ERROR_OUT_OF_MEMORY);
   argv[2] = STRING_TO_JSVAL(jsString);
   nsDependentJSString dependentKeyGenAlg;
@@ -1047,12 +1042,12 @@ nsSetEscrowAuthority(CRMFCertRequest *certReq, nsKeyPairInfo *keyInfo,
       CRMF_CertRequestIsControlPresent(certReq, crmfPKIArchiveOptionsControl)){
     return NS_ERROR_FAILURE;
   }
-  ScopedCERTCertificate cert(wrappingCert->GetCert());
+  insanity::pkix::ScopedCERTCertificate cert(wrappingCert->GetCert());
   if (!cert)
     return NS_ERROR_FAILURE;
 
   CRMFEncryptedKey *encrKey = 
-      CRMF_CreateEncryptedKeyWithEncryptedValue(keyInfo->privKey, cert);
+      CRMF_CreateEncryptedKeyWithEncryptedValue(keyInfo->privKey, cert.get());
   if (!encrKey)
     return NS_ERROR_FAILURE;
 
@@ -1849,7 +1844,7 @@ loser:
 static nsISupports *
 GetISupportsFromContext(JSContext *cx)
 {
-    if (JS_GetOptions(cx) & JSOPTION_PRIVATE_IS_NSISUPPORTS)
+    if (JS::ContextOptionsRef(cx).privateIsNSISupports())
         return static_cast<nsISupports *>(JS_GetContextPrivate(cx));
 
     return nullptr;
@@ -1857,7 +1852,7 @@ GetISupportsFromContext(JSContext *cx)
 
 //The top level method which is a member of nsIDOMCrypto
 //for generate a base64 encoded CRMF request.
-already_AddRefed<nsIDOMCRMFObject>
+CRMFObject*
 nsCrypto::GenerateCRMFRequest(JSContext* aContext,
                               const nsCString& aReqDN,
                               const nsCString& aRegToken,
@@ -1868,7 +1863,6 @@ nsCrypto::GenerateCRMFRequest(JSContext* aContext,
                               ErrorResult& aRv)
 {
   nsNSSShutDownPreventionLock locker;
-  nsCOMPtr<nsIDOMCRMFObject> crmf;
   nsresult nrv;
 
   uint32_t argc = aArgs.Length();
@@ -1923,7 +1917,9 @@ nsCrypto::GenerateCRMFRequest(JSContext* aContext,
     csp->LogViolationDetails(nsIContentSecurityPolicy::VIOLATION_TYPE_EVAL,
                              NS_ConvertASCIItoUTF16(fileName),
                              scriptSample,
-                             lineNum);
+                             lineNum,
+                             EmptyString(),
+                             EmptyString());
   }
 
   if (!evalAllowed) {
@@ -1946,15 +1942,15 @@ nsCrypto::GenerateCRMFRequest(JSContext* aContext,
       aRv.Throw(NS_ERROR_FAILURE);
       return nullptr;
     }
-    ScopedCERTCertificate cert(CERT_NewTempCertificate(CERT_GetDefaultCertDB(),
-                                                       &certDer, nullptr,
-                                                       false, true));
+    insanity::pkix::ScopedCERTCertificate cert(
+      CERT_NewTempCertificate(CERT_GetDefaultCertDB(),
+                              &certDer, nullptr, false, true));
     if (!cert) {
       aRv.Throw(NS_ERROR_FAILURE);
       return nullptr;
     }
 
-    escrowCert = nsNSSCertificate::Create(cert);
+    escrowCert = nsNSSCertificate::Create(cert.get());
     nssCert = escrowCert;
     if (!nssCert) {
       aRv.Throw(NS_ERROR_OUT_OF_MEMORY);
@@ -2026,9 +2022,8 @@ nsCrypto::GenerateCRMFRequest(JSContext* aContext,
     aRv.Throw(NS_ERROR_FAILURE);
     return nullptr;
   }
-  nsCRMFObject *newObject = new nsCRMFObject();
+  CRMFObject* newObject = new CRMFObject();
   newObject->SetCRMFRequest(encodedRequest);
-  crmf = newObject;
   nsFreeKeyPairInfo(keyids, numRequests);
 
   // Post an event on the UI queue so that the JS gets called after
@@ -2077,7 +2072,7 @@ nsCrypto::GenerateCRMFRequest(JSContext* aContext,
     delete cryptoRunnable;
   }
 
-  return crmf.forget();
+  return newObject;
 }
 
 // Reminder that we inherit the memory passed into us here.
@@ -2104,6 +2099,8 @@ nsP12Runnable::~nsP12Runnable()
 NS_IMETHODIMP
 nsP12Runnable::Run()
 {
+  NS_DEFINE_CID(kNSSComponentCID, NS_NSSCOMPONENT_CID);
+
   NS_ASSERTION(NS_IsMainThread(), "nsP12Runnable dispatched to the wrong thread");
 
   nsNSSShutDownPreventionLock locker;
@@ -2120,10 +2117,10 @@ nsP12Runnable::Run()
   //Build up the message that let's the user know we're trying to 
   //make PKCS12 backups of the new certs.
   nssComponent->GetPIPNSSBundleString("ForcedBackup1", final);
-  final.Append(NS_LITERAL_STRING("\n\n").get());
+  final.Append(MOZ_UTF16("\n\n"));
   nssComponent->GetPIPNSSBundleString("ForcedBackup2", temp);
   final.Append(temp.get());
-  final.Append(NS_LITERAL_STRING("\n\n").get());
+  final.Append(MOZ_UTF16("\n\n"));
 
   nssComponent->GetPIPNSSBundleString("ForcedBackup3", temp);
 
@@ -2210,11 +2207,8 @@ nsCryptoRunnable::Run()
   JSAutoCompartment ac(cx, m_args->m_scope);
 
   bool ok =
-    JS_EvaluateScriptForPrincipals(cx, m_args->m_scope,
-                                   nsJSPrincipals::get(m_args->m_principals),
-                                   m_args->m_jsCallback, 
-                                   strlen(m_args->m_jsCallback),
-                                   nullptr, 0, nullptr);
+    JS_EvaluateScript(cx, m_args->m_scope, m_args->m_jsCallback, 
+                      strlen(m_args->m_jsCallback), nullptr, 0, nullptr);
   return ok ? NS_OK : NS_ERROR_FAILURE;
 }
 
@@ -2226,12 +2220,13 @@ nsCertAlreadyExists(SECItem *derCert)
   CERTCertDBHandle *handle = CERT_GetDefaultCertDB();
   bool retVal = false;
 
-  ScopedCERTCertificate cert(CERT_FindCertByDERCert(handle, derCert));
+  insanity::pkix::ScopedCERTCertificate cert(
+    CERT_FindCertByDERCert(handle, derCert));
   if (cert) {
     if (cert->isperm && !cert->nickname && !cert->emailAddr) {
       //If the cert doesn't have a nickname or email addr, it is
       //bogus cruft, so delete it.
-      SEC_DeletePermCertificate(cert);
+      SEC_DeletePermCertificate(cert.get());
     } else if (cert->isperm) {
       retVal = true;
     }
@@ -2307,8 +2302,8 @@ nsCrypto::ImportUserCertificates(const nsAString& aNickname,
     //certificates.  Let's keep an array of them around which
     //we pass along to the nsP12Runnable to use.
     certArr = new nsIX509Cert*[numResponses];
-    // If this is NULL, chances are we're gonna fail really soon,
-    // but let's try to keep going just in case.
+    // If this is nullptr, chances are we're gonna fail really
+    // soon, but let's try to keep going just in case.
     if (!certArr)
       aDoForcedBackup = false;
 
@@ -2351,7 +2346,7 @@ nsCrypto::ImportUserCertificates(const nsAString& aNickname,
       localNick = currCert->nickname;
     }
     else if (!nickname || nickname[0] == '\0') {
-      nsNSSCertificateDB::get_default_nickname(currCert, ctx, localNick);
+      nsNSSCertificateDB::get_default_nickname(currCert, ctx, localNick, locker);
     } else {
       //This is the case where we're getting a brand new
       //cert that doesn't have the same subjectName as a cert
@@ -2387,9 +2382,10 @@ nsCrypto::ImportUserCertificates(const nsAString& aNickname,
 
   //Import the root chain into the cert db.
  {
-  ScopedCERTCertList caPubs(CMMF_CertRepContentGetCAPubs(certRepContent));
+  insanity::pkix::ScopedCERTCertList
+    caPubs(CMMF_CertRepContentGetCAPubs(certRepContent));
   if (caPubs) {
-    int32_t numCAs = nsCertListCount(caPubs);
+    int32_t numCAs = nsCertListCount(caPubs.get());
     
     NS_ASSERTION(numCAs > 0, "Invalid number of CA's");
     if (numCAs > 0) {
@@ -2407,7 +2403,7 @@ nsCrypto::ImportUserCertificates(const nsAString& aNickname,
            node = CERT_LIST_NEXT(node), i++) {
         derCerts[i] = node->cert->derCert;
       }
-      nsNSSCertificateDB::ImportValidCACerts(numCAs, derCerts, ctx);
+      nsNSSCertificateDB::ImportValidCACerts(numCAs, derCerts, ctx, locker);
       nsMemory::Free(derCerts);
     }
   }
@@ -2620,14 +2616,14 @@ nsCrypto::SignText(JSContext* aContext,
   NS_ASSERTION(nicknames->numnicknames == numberOfCerts,
                "nicknames->numnicknames != numberOfCerts");
 
-  nsAutoArrayPtr<PRUnichar*> certNicknameList(new PRUnichar*[nicknames->numnicknames * 2]);
+  nsAutoArrayPtr<char16_t*> certNicknameList(new char16_t*[nicknames->numnicknames * 2]);
   if (!certNicknameList) {
     aReturn.Append(internalError);
 
     return;
   }
 
-  PRUnichar** certDetailsList = certNicknameList.get() + nicknames->numnicknames;
+  char16_t** certDetailsList = certNicknameList.get() + nicknames->numnicknames;
 
   int32_t certsToUse;
   for (node = CERT_LIST_HEAD(certList), certsToUse = 0;
@@ -2668,8 +2664,8 @@ nsCrypto::SignText(JSContext* aContext,
     // of the selected cert.
     int32_t selectedIndex = -1;
     rv = fsd->ConfirmSignText(uiContext, utf16Host, aStringToSign,
-                              const_cast<const PRUnichar**>(certNicknameList.get()),
-                              const_cast<const PRUnichar**>(certDetailsList),
+                              const_cast<const char16_t**>(certNicknameList.get()),
+                              const_cast<const char16_t**>(certDetailsList),
                               certsToUse, &selectedIndex, password,
                               &canceled);
     if (NS_FAILED(rv) || canceled) {
@@ -2822,6 +2818,8 @@ nsCrypto::SignText(JSContext* aContext,
 void
 nsCrypto::Logout(ErrorResult& aRv)
 {
+  NS_DEFINE_CID(kNSSComponentCID, NS_NSSCOMPONENT_CID);
+
   nsresult rv;
   nsCOMPtr<nsINSSComponent> nssComponent(do_GetService(kNSSComponentCID, &rv));
   if (NS_FAILED(rv)) {
@@ -2847,29 +2845,31 @@ nsCrypto::DisableRightClick(ErrorResult& aRv)
   aRv.Throw(NS_ERROR_NOT_IMPLEMENTED);
 }
 
-nsCRMFObject::nsCRMFObject()
+CRMFObject::CRMFObject()
 {
+  MOZ_COUNT_CTOR(CRMFObject);
 }
 
-nsCRMFObject::~nsCRMFObject()
+CRMFObject::~CRMFObject()
 {
+  MOZ_COUNT_DTOR(CRMFObject);
 }
 
-nsresult
-nsCRMFObject::init()
+JSObject*
+CRMFObject::WrapObject(JSContext *aCx, JS::Handle<JSObject*> aScope,
+                       bool* aTookOwnership)
 {
-  return NS_OK;
+  return CRMFObjectBinding::Wrap(aCx, aScope, this, aTookOwnership);
 }
 
-NS_IMETHODIMP
-nsCRMFObject::GetRequest(nsAString& aRequest)
+void
+CRMFObject::GetRequest(nsAString& aRequest)
 {
   aRequest.Assign(mBase64Request);
-  return NS_OK;
 }
 
 nsresult
-nsCRMFObject::SetCRMFRequest(char *inRequest)
+CRMFObject::SetCRMFRequest(char *inRequest)
 {
   mBase64Request.AssignWithConversion(inRequest);  
   return NS_OK;
@@ -2889,6 +2889,8 @@ nsPkcs11::~nsPkcs11()
 NS_IMETHODIMP
 nsPkcs11::DeleteModule(const nsAString& aModuleName)
 {
+  NS_DEFINE_CID(kNSSComponentCID, NS_NSSCOMPONENT_CID);
+
   nsNSSShutDownPreventionLock locker;
   nsresult rv;
   nsString errorMessage;
@@ -2926,6 +2928,8 @@ nsPkcs11::AddModule(const nsAString& aModuleName,
                     int32_t aCryptoMechanismFlags, 
                     int32_t aCipherFlags)
 {
+  NS_DEFINE_CID(kNSSComponentCID, NS_NSSCOMPONENT_CID);
+
   nsNSSShutDownPreventionLock locker;
   nsresult rv;
   nsCOMPtr<nsINSSComponent> nssComponent(do_GetService(kNSSComponentCID, &rv));
