@@ -175,7 +175,7 @@ class DeviceManager(object):
         """
 
     @abstractmethod
-    def pushDir(self, localDirname, remoteDirname, retryLimit=1):
+    def pushDir(self, localDirname, remoteDirname, retryLimit=1, timeout=None):
         """
         Push local directory from host to remote directory on the device,
         """
@@ -282,6 +282,24 @@ class DeviceManager(object):
         Does a recursive delete of directory on the device: rm -Rf remoteDirname.
         """
 
+    @abstractmethod
+    def moveTree(self, source, destination):
+         """
+         Does a move of the file or directory on the device.
+
+        :param source: Path to the original file or directory
+        :param destination: Path to the destination file or directory
+         """
+
+    @abstractmethod
+    def copyTree(self, source, destination):
+         """
+         Does a copy of the file or directory on the device.
+
+        :param source: Path to the original file or directory
+        :param destination: Path to the destination file or directory
+         """
+
     @abstractmethod
     def chmodDir(self, remoteDirname, mask="777"):
         """
@@ -350,7 +368,7 @@ class DeviceManager(object):
         """
         Executes shell command on device and returns exit code.
 
-        :param cmd: Command string to execute
+        :param cmd: Commandline list to execute
         :param outputfile: File to store output
         :param env: Environment to pass to exec command
         :param cwd: Directory to execute command from
@@ -362,6 +380,7 @@ class DeviceManager(object):
         """
         Executes shell command on device and returns output as a string.
 
+        :param cmd: Commandline list to execute
         :param env: Environment to pass to exec command
         :param cwd: Directory to execute command from
         :param timeout: specified in seconds, defaults to 'default_timeout'
@@ -419,20 +438,25 @@ class DeviceManager(object):
 
 
     @abstractmethod
-    def killProcess(self, processName, forceKill=False):
+    def killProcess(self, processName, sig=None):
         """
-        Kills the process named processName. If forceKill is True, process is
-        killed regardless of state.
+        Kills the process named processName. If sig is not None, process is
+        killed with the specified signal.
+
+        :param processName: path or name of the process to kill
+        :param sig: signal to pass into the kill command (optional)
         """
 
     @abstractmethod
-    def reboot(self, ipAddr=None, port=30000):
+    def reboot(self, wait=False, ipAddr=None):
         """
         Reboots the device.
 
-        Some implementations may optionally support waiting for a TCP callback from
-        the device once it has restarted before returning, but this is not
-        guaranteed.
+        :param wait: block on device to come back up before returning
+        :param ipAddr: if specified, try to make the device connect to this
+                       specific IP address after rebooting (only works with
+                       SUT; if None, we try to determine a reasonable address
+                       ourselves)
         """
 
     @abstractmethod
@@ -463,21 +487,21 @@ class DeviceManager(object):
         """
 
     @abstractmethod
-    def updateApp(self, appBundlePath, processName=None, destPath=None, ipAddr=None, port=30000):
+    def updateApp(self, appBundlePath, processName=None, destPath=None,
+                  wait=False, ipAddr=None):
         """
-        Updates the application on the device.
+        Updates the application on the device and reboots.
 
         :param appBundlePath: path to the application bundle on the device
         :param processName: used to end the process if the applicaiton is
                             currently running (optional)
         :param destPath: Destination directory to where the application should
                          be installed (optional)
-        :param ipAddr: IP address to await a callback ping to let us know that
-                       the device has updated properly (defaults to current
-                       IP)
-        :param port: port to await a callback ping to let us know that the
-                     device has updated properly defaults to 30000, and counts
-                     up from there if it finds a conflict
+        :param wait: block on device to come back up before returning
+        :param ipAddr: if specified, try to make the device connect to this
+                       specific IP address after rebooting (only works with
+                       SUT; if None and wait is True, we try to determine a
+                       reasonable address ourselves)
         """
 
     @staticmethod
