@@ -48,11 +48,6 @@ XPCOMUtils.defineLazyGetter(this, "nsGzipConverter",
 let gMgr = Cc["@mozilla.org/memory-reporter-manager;1"]
              .getService(Ci.nsIMemoryReporterManager);
 
-// We need to know about "child-memory-reporter-update" events from child
-// processes.
-Services.obs.addObserver(updateAboutMemoryFromReporters,
-                         "child-memory-reporter-update", false);
-
 let gUnnamedProcessStr = "Main Process";
 
 let gIsDiff = false;
@@ -120,61 +115,6 @@ function debug(x)
 
 function onUnload()
 {
-  Services.obs.removeObserver(updateAboutMemoryFromReporters,
-                              "child-memory-reporter-update");
-}
-
-//---------------------------------------------------------------------------
-
-/**
- * Iterates over each reporter.
- *
- * @param aHandleReport
- *        The function that's called for each non-skipped report.
- */
-function processMemoryReporters(aHandleReport)
-{
-  let handleReport = function(aProcess, aUnsafePath, aKind, aUnits,
-                              aAmount, aDescription) {
-    aHandleReport(aProcess, aUnsafePath, aKind, aUnits, aAmount,
-                  aDescription, /* presence = */ undefined);
-  }
-
-  let e = gMgr.enumerateReporters();
-  while (e.hasMoreElements()) {
-    let mr = e.getNext().QueryInterface(Ci.nsIMemoryReporter);
-    mr.collectReports(handleReport, null);
-  }
-}
-
-/**
- * Iterates over each report.
- *
- * @param aReports
- *        Array of reports, read from a file or the clipboard.
- * @param aHandleReport
- *        The function that's called for each report.
- */
-function processMemoryReportsFromFile(aReports, aHandleReport)
-{
-  // Process each memory reporter with aHandleReport.
-
-  for (let i = 0; i < aReports.length; i++) {
-    let r = aReports[i];
-
-    // A hack: for a brief time (late in the FF26 and early in the FF27
-    // cycle) we were dumping memory report files that contained reports
-    // whose path began with "redundant/".  Such reports were ignored by
-    // about:memory.  These reports are no longer produced, but some older
-    // builds are still floating around and producing files that contain
-    // them, so we need to still handle them (i.e. ignore them).  This hack
-    // can be removed once FF26 and associated products (e.g. B2G 1.2) are
-    // no longer in common use.
-    if (!r.path.startsWith("redundant/")) {
-      aHandleReport(r.process, r.path, r.kind, r.units, r.amount,
-                    r.description, r._presence);
-    }
-  }
 }
 
 //---------------------------------------------------------------------------
@@ -343,6 +283,13 @@ function onLoad()
                  "process to reduce memory usage in other ways, e.g. by " +
                  "flushing various caches.";
 
+  const GCAndCCLogDesc = "Save garbage collection log and concise cycle " +
+                         "collection log.\n" +
+                         "WARNING: These logs may be large (>1GB).";
+  const GCAndCCAllLogDesc = "Save garbage collection log and verbose cycle " +
+                            "collection log.\n" +
+                            "WARNING: These logs may be large (>1GB).";
+
   let ops = appendElement(header, "div", "");
 
   let row1 = appendElement(ops, "div", "opsRow");
@@ -378,10 +325,19 @@ function onLoad()
   appendButton(row3, CCDesc, doCC,  "CC");
   appendButton(row3, MMDesc, doMMU, "Minimize memory usage");
 
+  let row4 = appendElement(ops, "div", "opsRow");
+
+  appendElementWithText(row4, "div", "opsRowLabel", "Save GC & CC logs");
+  appendButton(row4, GCAndCCLogDesc,
+               saveGCLogAndConciseCCLog, "Save concise", 'saveLogsConcise');
+  appendButton(row4, GCAndCCAllLogDesc,
+               saveGCLogAndVerboseCCLog, "Save verbose", 'saveLogsVerbose');
+
   // Generate the main div, where content ("section" divs) will go.  It's
   // hidden at first.
 
   gMain = appendElement(document.body, 'div', '');
+  gMain.id = 'mainDiv';
 
   // Generate the footer.  It's hidden at first.
 
@@ -419,50 +375,94 @@ function onLoad()
 
 function doGC()
 {
-  Cu.forceGC();
   Services.obs.notifyObservers(null, "child-gc-request", null);
+  Cu.forceGC();
   updateMainAndFooter("Garbage collection completed", HIDE_FOOTER);
 }
 
 function doCC()
 {
+  Services.obs.notifyObservers(null, "child-cc-request", null);
   window.QueryInterface(Ci.nsIInterfaceRequestor)
         .getInterface(Ci.nsIDOMWindowUtils)
         .cycleCollect();
-  Services.obs.notifyObservers(null, "child-cc-request", null);
   updateMainAndFooter("Cycle collection completed", HIDE_FOOTER);
 }
 
 function doMMU()
 {
+  Services.obs.notifyObservers(null, "child-mmu-request", null);
   gMgr.minimizeMemoryUsage(
     () => updateMainAndFooter("Memory minimization completed", HIDE_FOOTER));
 }
 
 function doMeasure()
 {
-  // Notify any children that they should measure memory consumption, then
-  // update the page.  If any reports come back from children,
-  // updateAboutMemoryFromReporters() will be called again and the page will
-  // regenerate.
-  Services.obs.notifyObservers(null, "child-memory-reporter-request", null);
   updateAboutMemoryFromReporters();
 }
 
+function saveGCLogAndConciseCCLog()
+{
+  dumpGCLogAndCCLog(false);
+}
+
+function saveGCLogAndVerboseCCLog()
+{
+  dumpGCLogAndCCLog(true);
+}
+
+function dumpGCLogAndCCLog(aVerbose)
+{
+  let gcLogPath = {};
+  let ccLogPath = {};
+
+  let dumper = Cc["@mozilla.org/memory-info-dumper;1"]
+                .getService(Ci.nsIMemoryInfoDumper);
+
+  updateMainAndFooter("Saving logs...", HIDE_FOOTER);
+
+  dumper.dumpGCAndCCLogsToFile("", aVerbose, /* dumpChildProcesses = */ false,
+                               gcLogPath, ccLogPath);
+
+  updateMainAndFooter("", HIDE_FOOTER);
+  let section = appendElement(gMain, 'div', "section");
+  appendElementWithText(section, 'div', "",
+                        "Saved GC log to " + gcLogPath.value);
+
+  let ccLogType = aVerbose ? "verbose" : "concise";
+  appendElementWithText(section, 'div', "",
+                        "Saved " + ccLogType + " CC log to " + ccLogPath.value);
+}
+
 /**
  * Top-level function that does the work of generating the page from the memory
  * reporters.
  */
 function updateAboutMemoryFromReporters()
 {
-  // First, clear the contents of main.  Necessary because
-  // updateAboutMemoryFromReporters() might be called more than once due to the
-  // "child-memory-reporter-update" observer.
-  updateMainAndFooter("", SHOW_FOOTER);
+  updateMainAndFooter("Measuring...", HIDE_FOOTER);
 
   try {
-    // Process the reports from the memory reporters.
-    appendAboutMemoryMain(processMemoryReporters, gMgr.hasMozMallocUsableSize);
+    let processLiveMemoryReports =
+        function(aHandleReport, aDisplayReports) {
+      let handleReport = function(aProcess, aUnsafePath, aKind, aUnits,
+                                  aAmount, aDescription) {
+        aHandleReport(aProcess, aUnsafePath, aKind, aUnits, aAmount,
+                      aDescription, /* presence = */ undefined);
+      }
+
+      let displayReportsAndFooter = function() {
+        updateMainAndFooter("", SHOW_FOOTER);
+        aDisplayReports();
+      }
+
+      gMgr.getReports(handleReport, null,
+                      displayReportsAndFooter, null);
+    }
+
+    // Process the reports from the live memory reporters.
+    appendAboutMemoryMain(processLiveMemoryReports,
+                          gMgr.hasMozMallocUsableSize);
 
   } catch (ex) {
     handleException(ex);
@@ -489,10 +489,29 @@ function updateAboutMemoryFromJSONObject(aObj)
                 "missing 'hasMozMallocUsableSize' property");
     assertInput(aObj.reports && aObj.reports instanceof Array,
                 "missing or non-array 'reports' property");
-    let process = function(aHandleReport) {
-      processMemoryReportsFromFile(aObj.reports, aHandleReport);
+
+    let processMemoryReportsFromFile =
+        function(aHandleReport, aDisplayReports) {
+      for (let i = 0; i < aObj.reports.length; i++) {
+        let r = aObj.reports[i];
+
+        // A hack: for a brief time (late in the FF26 and early in the FF27
+        // cycle) we were dumping memory report files that contained reports
+        // whose path began with "redundant/".  Such reports were ignored by
+        // about:memory.  These reports are no longer produced, but some older
+        // builds are still floating around and producing files that contain
+        // them, so we need to still handle them (i.e. ignore them).  This hack
+        // can be removed once FF26 and associated products (e.g. B2G 1.2) are
+        // no longer in common use.
+        if (!r.path.startsWith("redundant/")) {
+          aHandleReport(r.process, r.path, r.kind, r.units, r.amount,
+                        r.description, r._presence);
+        }
+      }
+      aDisplayReports();
     }
-    appendAboutMemoryMain(process, aObj.hasMozMallocUsableSize);
+    appendAboutMemoryMain(processMemoryReportsFromFile,
+                          aObj.hasMozMallocUsableSize);
   } catch (ex) {
     handleException(ex);
   }
@@ -847,71 +866,6 @@ function PColl()
  *        Boolean indicating if moz_malloc_usable_size works.
  */
 function appendAboutMemoryMain(aProcessReports, aHasMozMallocUsableSize)
-{
-  let pcollsByProcess = getPCollsByProcess(aProcessReports);
-
-  // Sort the processes.
-  let processes = Object.keys(pcollsByProcess);
-  processes.sort(function(aProcessA, aProcessB) {
-    assert(aProcessA != aProcessB,
-           "Elements of Object.keys() should be unique, but " +
-           "saw duplicate '" + aProcessA + "' elem.");
-
-    // Always put the main process first.
-    if (aProcessA == gUnnamedProcessStr) {
-      return -1;
-    }
-    if (aProcessB == gUnnamedProcessStr) {
-      return 1;
-    }
-
-    // Then sort by resident size.
-    let nodeA = pcollsByProcess[aProcessA]._degenerates['resident'];
-    let nodeB = pcollsByProcess[aProcessB]._degenerates['resident'];
-    let residentA = nodeA ? nodeA._amount : -1;
-    let residentB = nodeB ? nodeB._amount : -1;
-
-    if (residentA > residentB) {
-      return -1;
-    }
-    if (residentA < residentB) {
-      return 1;
-    }
-
-    // Then sort by process name.
-    if (aProcessA < aProcessB) {
-      return -1;
-    }
-    if (aProcessA > aProcessB) {
-      return 1;
-    }
-
-    return 0;
-  });
-
-  // Generate output for each process.
-  for (let i = 0; i < processes.length; i++) {
-    let process = processes[i];
-    let section = appendElement(gMain, 'div', 'section');
-
-    appendProcessAboutMemoryElements(section, i, process,
-                                     pcollsByProcess[process]._trees,
-                                     pcollsByProcess[process]._degenerates,
-                                     pcollsByProcess[process]._heapTotal,
-                                     aHasMozMallocUsableSize);
-  }
-}
-
-/**
- * This function reads all the memory reports, and puts that data in structures
- * that will be used to generate the page.
- *
- * @param aProcessReports
- *        Function that extracts the memory reports from the reporters or from
- *        file.
- * @return The table of PColls by process.
- */
-function getPCollsByProcess(aProcessReports)
 {
   let pcollsByProcess = {};
 
@@ -985,9 +939,61 @@ function getPCollsByProcess(aProcessReports)
     }
   }
 
-  aProcessReports(handleReport);
+  function displayReports()
+  {
+    // Sort the processes.
+    let processes = Object.keys(pcollsByProcess);
+    processes.sort(function(aProcessA, aProcessB) {
+      assert(aProcessA != aProcessB,
+             "Elements of Object.keys() should be unique, but " +
+             "saw duplicate '" + aProcessA + "' elem.");
 
-  return pcollsByProcess;
+      // Always put the main process first.
+      if (aProcessA == gUnnamedProcessStr) {
+        return -1;
+      }
+      if (aProcessB == gUnnamedProcessStr) {
+        return 1;
+      }
+
+      // Then sort by resident size.
+      let nodeA = pcollsByProcess[aProcessA]._degenerates['resident'];
+      let nodeB = pcollsByProcess[aProcessB]._degenerates['resident'];
+      let residentA = nodeA ? nodeA._amount : -1;
+      let residentB = nodeB ? nodeB._amount : -1;
+
+      if (residentA > residentB) {
+        return -1;
+      }
+      if (residentA < residentB) {
+        return 1;
+      }
+
+      // Then sort by process name.
+      if (aProcessA < aProcessB) {
+        return -1;
+      }
+      if (aProcessA > aProcessB) {
+        return 1;
+      }
+
+      return 0;
+    });
+
+    // Generate output for each process.
+    for (let i = 0; i < processes.length; i++) {
+      let process = processes[i];
+      let section = appendElement(gMain, 'div', 'section');
+
+      appendProcessAboutMemoryElements(section, i, process,
+                                       pcollsByProcess[process]._trees,
+                                       pcollsByProcess[process]._degenerates,
+                                       pcollsByProcess[process]._heapTotal,
+                                       aHasMozMallocUsableSize);
+    }
+  }
+
+  aProcessReports(handleReport, displayReports);
 }
 
 //---------------------------------------------------------------------------
@@ -1131,7 +1137,7 @@ function fillInTree(aRoot)
  * @param aHeapAllocatedNode
  *        The "heap-allocated" tree node.
  * @param aHeapTotal
- *        The sum of all explicit HEAP reporters for this process.
+ *        The sum of all explicit HEAP reports for this process.
  * @return A boolean indicating if "heap-allocated" is known for the process.
  */
 function addHeapUnclassifiedNode(aT, aHeapAllocatedNode, aHeapTotal)
@@ -1144,7 +1150,7 @@ function addHeapUnclassifiedNode(aT, aHeapAllocatedNode, aHeapTotal)
   let heapUnclassifiedT = new TreeNode("heap-unclassified", UNITS_BYTES);
   heapUnclassifiedT._amount = heapAllocatedBytes - aHeapTotal;
   heapUnclassifiedT._description =
-      "Memory not classified by a more specific reporter. This includes " +
+      "Memory not classified by a more specific report. This includes " +
       "slop bytes due to internal fragmentation in the heap allocator " +
       "(caused when the allocator rounds up request sizes).";
   aT._kids.push(heapUnclassifiedT);
@@ -1330,12 +1336,14 @@ function appendProcessAboutMemoryElements(aP, aN, aProcess, aTrees,
   let warningsDiv = appendElement(aP, "div", "accuracyWarning");
 
   // The explicit tree.
+  let hasExplicitTree;
   let hasKnownHeapAllocated;
   {
     let treeName = "explicit";
-    let pre = appendSectionHeader(aP, "Explicit Allocations");
     let t = aTrees[treeName];
     if (t) {
+      let pre = appendSectionHeader(aP, "Explicit Allocations");
+      hasExplicitTree = true;
       fillInTree(t);
       // Using the "heap-allocated" reporter here instead of
       // nsMemoryReporterManager.heapAllocated goes against the usual pattern.
@@ -1393,11 +1401,13 @@ function appendProcessAboutMemoryElements(aP, aN, aProcess, aTrees,
   }
   appendTextNode(aP, "\n");  // gives nice spacing when we copy and paste
 
-  // Add any warnings about inaccuracies due to platform limitations.
-  // These must be computed after generating all the text.  The newlines give
-  // nice spacing if we copy+paste into a text buffer.
-  appendWarningElements(warningsDiv, hasKnownHeapAllocated,
-                        aHasMozMallocUsableSize);
+  // Add any warnings about inaccuracies in the "explicit" tree due to platform
+  // limitations.  These must be computed after generating all the text.  The
+  // newlines give nice spacing if we copy+paste into a text buffer.
+  if (hasExplicitTree) {
+    appendWarningElements(warningsDiv, hasKnownHeapAllocated,
+                          aHasMozMallocUsableSize);
+  }
 
   appendElementWithText(aP, "h3", "", "End of " + aProcess);
   appendLink("end", "start", kUpwardsArrow);
@@ -1566,7 +1576,7 @@ function appendMrNameSpan(aP, aDescription, aUnsafeName, aIsInvalid, aNMerged,
     let noteSpan = appendElementWithText(aP, "span", "mrNote", noteText);
     noteSpan.title =
       "This value is the sum of " + aNMerged +
-      " memory reporters that all have the same path.";
+      " memory reports that all have the same path.";
   }
 
   if (aPresence) {
@@ -1834,24 +1844,39 @@ function appendSectionHeader(aP, aText)
 function saveReportsToFile()
 {
   let fp = Cc["@mozilla.org/filepicker;1"].createInstance(Ci.nsIFilePicker);
-  fp.init(window, "Save Memory Reports", Ci.nsIFilePicker.modeSave);
   fp.appendFilter("Zipped JSON files", "*.json.gz");
   fp.appendFilters(Ci.nsIFilePicker.filterAll);
   fp.filterIndex = 0;
   fp.addToRecentDocs = true;
   fp.defaultString = "memory-report.json.gz";
 
+  let fpFinish = function(file) {
+    let dumper = Cc["@mozilla.org/memory-info-dumper;1"]
+                   .getService(Ci.nsIMemoryInfoDumper);
+
+    let finishDumping = () => {
+      updateMainAndFooter("Saved reports to " + file.path, HIDE_FOOTER);
+    }
+
+    dumper.dumpMemoryReportsToNamedFile(file.path, finishDumping, null);
+  }
+
   let fpCallback = function(aResult) {
     if (aResult == Ci.nsIFilePicker.returnOK ||
         aResult == Ci.nsIFilePicker.returnReplace) {
-
-      let dumper = Cc["@mozilla.org/memory-info-dumper;1"]
-                     .getService(Ci.nsIMemoryInfoDumper);
-
-      dumper.dumpMemoryReportsToNamedFile(fp.file.path);
-
-      updateMainAndFooter("Saved reports to " + fp.file.path, HIDE_FOOTER);
+      fpFinish(fp.file);
     }
   };
+
+  try {
+    fp.init(window, "Save Memory Reports", Ci.nsIFilePicker.modeSave);
+  } catch(ex) {
+    // This will fail on Android, since there is no Save as file picker there.
+    // Just save to the default downloads dir if it does.
+    let file = Services.dirsvc.get("DfltDwnld", Ci.nsIFile);
+    file.append(fp.defaultString);
+    fpFinish(file);
+    return;
+  }
   fp.open(fpCallback);
 }
