@@ -15,7 +15,6 @@
 #include "nsICharsetConverterManager.h"
 #include "nsICollation.h"
 #include "nsCollationCID.h"
-#include "nsLocaleCID.h"
 #include "nsILocaleService.h"
 #include "nsIPrefService.h"
 #include "nsIPrefBranch.h"
@@ -133,7 +132,6 @@ private:
   static nsIRDFDataSource * mInner;
 
   bool mInitialized;
-  bool mBrowserMenuInitialized;
   bool mMailviewMenuInitialized;
   bool mComposerMenuInitialized;
   bool mMaileditMenuInitialized;
@@ -141,11 +139,6 @@ private:
   bool mAutoDetectInitialized;
   bool mOthersInitialized;
 
-  nsTArray<nsMenuEntry*> mBrowserMenu;
-  int32_t                mBrowserCacheStart;
-  int32_t                mBrowserCacheSize;
-  int32_t                mBrowserMenuRDFPosition;
-
   nsTArray<nsMenuEntry*> mMailviewMenu;
   int32_t                mMailviewCacheStart;
   int32_t                mMailviewCacheSize;
@@ -227,7 +220,7 @@ private:
   nsresult WriteCacheToPrefs(nsTArray<nsMenuEntry*> * aArray, int32_t aCacheStart,
     const char * aKey);
   nsresult UpdateCachePrefs(const char * aCacheKey, const char * aCacheSizeKey,
-    const char * aStaticKey, const PRUnichar * aCharset);
+    const char * aStaticKey, const char16_t * aCharset);
 
   nsresult ClearMenu(nsIRDFContainer * aContainer, nsTArray<nsMenuEntry*> * aArray);
   nsresult RemoveLastMenuItem(nsIRDFContainer * aContainer,
@@ -248,14 +241,12 @@ public:
   virtual ~nsCharsetMenu();
 
   nsresult Init();
-  nsresult InitBrowserMenu();
   nsresult InitMaileditMenu();
   nsresult InitMailviewMenu();
   nsresult InitComposerMenu();
   nsresult InitOthers();
   nsresult InitSecondaryTiers();
   nsresult InitAutodetMenu();
-  nsresult RefreshBrowserMenu();
   nsresult RefreshMailviewMenu();
   nsresult RefreshMaileditMenu();
   nsresult RefreshComposerMenu();
@@ -263,9 +254,9 @@ public:
   //--------------------------------------------------------------------------
   // Interface nsICurrentCharsetListener [declaration]
 
-  NS_IMETHOD SetCurrentCharset(const PRUnichar * aCharset);
-  NS_IMETHOD SetCurrentMailCharset(const PRUnichar * aCharset);
-  NS_IMETHOD SetCurrentComposerCharset(const PRUnichar * aCharset);
+  NS_IMETHOD SetCurrentCharset(const char16_t * aCharset);
+  NS_IMETHOD SetCurrentMailCharset(const char16_t * aCharset);
+  NS_IMETHOD SetCurrentComposerCharset(const char16_t * aCharset);
 
   //--------------------------------------------------------------------------
   // Interface nsIRDFDataSource [declaration]
@@ -377,7 +368,7 @@ private:
 
 NS_IMPL_ISUPPORTS1(nsCharsetMenuObserver, nsIObserver)
 
-NS_IMETHODIMP nsCharsetMenuObserver::Observe(nsISupports *aSubject, const char *aTopic, const PRUnichar *someData)
+NS_IMETHODIMP nsCharsetMenuObserver::Observe(nsISupports *aSubject, const char *aTopic, const char16_t *someData)
 {
   nsresult rv = NS_OK;
  
@@ -385,9 +376,6 @@ NS_IMETHODIMP nsCharsetMenuObserver::Observe(nsISupports *aSubject, const char *
   if (!nsCRT::strcmp(aTopic, "charsetmenu-selected")) {
     nsDependentString nodeName(someData);
     rv = mCharsetMenu->Init();
-    if (nodeName.EqualsLiteral("browser")) {
-      rv = mCharsetMenu->InitBrowserMenu();
-    }
     if (nodeName.EqualsLiteral("composer")) {
       rv = mCharsetMenu->InitComposerMenu();
     }
@@ -414,8 +402,6 @@ NS_IMETHODIMP nsCharsetMenuObserver::Observe(nsISupports *aSubject, const char *
 
     if (prefName.EqualsLiteral(kBrowserStaticPrefKey)) {
       // refresh menus which share this pref
-      rv = mCharsetMenu->RefreshBrowserMenu();
-      NS_ENSURE_SUCCESS(rv, rv);
       rv = mCharsetMenu->RefreshMailviewMenu();
       NS_ENSURE_SUCCESS(rv, rv);
       rv = mCharsetMenu->RefreshComposerMenu();
@@ -446,28 +432,27 @@ NS_INTERFACE_MAP_BEGIN_CYCLE_COLLECTION(nsCharsetMenu)
   NS_INTERFACE_MAP_ENTRY_AMBIGUOUS(nsISupports, nsIRDFDataSource)
 NS_INTERFACE_MAP_END
 
-nsIRDFDataSource * nsCharsetMenu::mInner = NULL;
-nsIRDFResource * nsCharsetMenu::kNC_BrowserAutodetMenuRoot = NULL;
-nsIRDFResource * nsCharsetMenu::kNC_BrowserCharsetMenuRoot = NULL;
-nsIRDFResource * nsCharsetMenu::kNC_BrowserMoreCharsetMenuRoot = NULL;
-nsIRDFResource * nsCharsetMenu::kNC_BrowserMore1CharsetMenuRoot = NULL;
-nsIRDFResource * nsCharsetMenu::kNC_BrowserMore2CharsetMenuRoot = NULL;
-nsIRDFResource * nsCharsetMenu::kNC_BrowserMore3CharsetMenuRoot = NULL;
-nsIRDFResource * nsCharsetMenu::kNC_BrowserMore4CharsetMenuRoot = NULL;
-nsIRDFResource * nsCharsetMenu::kNC_BrowserMore5CharsetMenuRoot = NULL;
-nsIRDFResource * nsCharsetMenu::kNC_MaileditCharsetMenuRoot = NULL;
-nsIRDFResource * nsCharsetMenu::kNC_MailviewCharsetMenuRoot = NULL;
-nsIRDFResource * nsCharsetMenu::kNC_ComposerCharsetMenuRoot = NULL;
-nsIRDFResource * nsCharsetMenu::kNC_DecodersRoot = NULL;
-nsIRDFResource * nsCharsetMenu::kNC_EncodersRoot = NULL;
-nsIRDFResource * nsCharsetMenu::kNC_Name = NULL;
-nsIRDFResource * nsCharsetMenu::kNC_CharsetDetector = NULL;
-nsIRDFResource * nsCharsetMenu::kNC_BookmarkSeparator = NULL;
-nsIRDFResource * nsCharsetMenu::kRDF_type = NULL;
+nsIRDFDataSource * nsCharsetMenu::mInner = nullptr;
+nsIRDFResource * nsCharsetMenu::kNC_BrowserAutodetMenuRoot = nullptr;
+nsIRDFResource * nsCharsetMenu::kNC_BrowserCharsetMenuRoot = nullptr;
+nsIRDFResource * nsCharsetMenu::kNC_BrowserMoreCharsetMenuRoot = nullptr;
+nsIRDFResource * nsCharsetMenu::kNC_BrowserMore1CharsetMenuRoot = nullptr;
+nsIRDFResource * nsCharsetMenu::kNC_BrowserMore2CharsetMenuRoot = nullptr;
+nsIRDFResource * nsCharsetMenu::kNC_BrowserMore3CharsetMenuRoot = nullptr;
+nsIRDFResource * nsCharsetMenu::kNC_BrowserMore4CharsetMenuRoot = nullptr;
+nsIRDFResource * nsCharsetMenu::kNC_BrowserMore5CharsetMenuRoot = nullptr;
+nsIRDFResource * nsCharsetMenu::kNC_MaileditCharsetMenuRoot = nullptr;
+nsIRDFResource * nsCharsetMenu::kNC_MailviewCharsetMenuRoot = nullptr;
+nsIRDFResource * nsCharsetMenu::kNC_ComposerCharsetMenuRoot = nullptr;
+nsIRDFResource * nsCharsetMenu::kNC_DecodersRoot = nullptr;
+nsIRDFResource * nsCharsetMenu::kNC_EncodersRoot = nullptr;
+nsIRDFResource * nsCharsetMenu::kNC_Name = nullptr;
+nsIRDFResource * nsCharsetMenu::kNC_CharsetDetector = nullptr;
+nsIRDFResource * nsCharsetMenu::kNC_BookmarkSeparator = nullptr;
+nsIRDFResource * nsCharsetMenu::kRDF_type = nullptr;
 
 nsCharsetMenu::nsCharsetMenu() 
 : mInitialized(false), 
-  mBrowserMenuInitialized(false),
   mMailviewMenuInitialized(false),
   mComposerMenuInitialized(false),
   mMaileditMenuInitialized(false),
@@ -517,53 +502,12 @@ nsCharsetMenu::~nsCharsetMenu()
 {
   Done();
 
-  FreeMenuItemArray(&mBrowserMenu);
   FreeMenuItemArray(&mMailviewMenu);
   FreeMenuItemArray(&mComposerMenu);
 
   FreeResources();
 }
 
-// XXX collapse these 2 in one
-
-nsresult nsCharsetMenu::RefreshBrowserMenu()
-{
-  nsresult res = NS_OK;
-
-  nsCOMPtr<nsIRDFContainer> container;
-  res = NewRDFContainer(mInner, kNC_BrowserCharsetMenuRoot, getter_AddRefs(container));
-  if (NS_FAILED(res)) return res;
-
-  // clean the menu
-  res = ClearMenu(container, &mBrowserMenu);
-  if (NS_FAILED(res)) return res;
-
-  // rebuild the menu
-  nsCOMPtr<nsIUTF8StringEnumerator> decoders;
-  res = mCCManager->GetDecoderList(getter_AddRefs(decoders));
-  if (NS_FAILED(res)) return res;
-
-  nsTArray<nsCString> decs;
-  SetArrayFromEnumerator(decoders, decs);
-  
-  res = AddFromPrefsToMenu(&mBrowserMenu, container, kBrowserStaticPrefKey, 
-                           decs, "charset.");
-  NS_ASSERTION(NS_SUCCEEDED(res), "error initializing static charset menu from prefs");
-
-  // mark the end of the static area, the rest is cache
-  mBrowserCacheStart = mBrowserMenu.Length();
-
-  // Remove "notForBrowser" entries before populating cache menu
-  res = RemoveFlaggedCharsets(decs, NS_LITERAL_STRING(".notForBrowser"));
-  NS_ASSERTION(NS_SUCCEEDED(res), "error removing flagged charsets");
-
-  res = InitCacheMenu(decs, kNC_BrowserCharsetMenuRoot, kBrowserCachePrefKey, 
-                      &mBrowserMenu);
-  NS_ASSERTION(NS_SUCCEEDED(res), "error initializing browser cache charset menu");
-
-  return res;
-}
-
 nsresult nsCharsetMenu::RefreshMailviewMenu()
 {
   nsresult res = NS_OK;
@@ -610,8 +554,9 @@ nsresult nsCharsetMenu::RefreshMaileditMenu()
   NS_ENSURE_SUCCESS(res, res);
 
   // clear the menu
-  nsCOMPtr<nsIRDFNode> node;
-  while (NS_SUCCEEDED(enumerator->GetNext(getter_AddRefs(node)))) {
+  nsCOMPtr<nsISupports> supports;
+  while (NS_SUCCEEDED(enumerator->GetNext(getter_AddRefs(supports)))) {
+    nsCOMPtr<nsIRDFNode> node = do_QueryInterface(supports);
 
     res = mInner->Unassert(kNC_MaileditCharsetMenuRoot, kNC_Name, node);
     NS_ENSURE_SUCCESS(res, res);
@@ -629,7 +574,7 @@ nsresult nsCharsetMenu::RefreshMaileditMenu()
   SetArrayFromEnumerator(encoders, encs);
   
   // add menu items from pref
-  res = AddFromPrefsToMenu(NULL, container, kMaileditPrefKey, encs, NULL);
+  res = AddFromPrefsToMenu(nullptr, container, kMaileditPrefKey, encs, nullptr);
   NS_ASSERTION(NS_SUCCEEDED(res), "error initializing mailedit charset menu from prefs");
 
   return res;
@@ -714,35 +659,35 @@ nsresult nsCharsetMenu::Init()
                              &kNC_BookmarkSeparator);
     mRDFService->GetResource(NS_LITERAL_CSTRING(kURINC_type), &kRDF_type);
 
-    nsIRDFContainerUtils * rdfUtil = NULL;
+    nsIRDFContainerUtils * rdfUtil = nullptr;
     res = CallGetService(kRDFContainerUtilsCID, &rdfUtil);
     if (NS_FAILED(res)) goto done;
 
-    res = rdfUtil->MakeSeq(mInner, kNC_BrowserAutodetMenuRoot, NULL);
+    res = rdfUtil->MakeSeq(mInner, kNC_BrowserAutodetMenuRoot, nullptr);
     if (NS_FAILED(res)) goto done;
-    res = rdfUtil->MakeSeq(mInner, kNC_BrowserCharsetMenuRoot, NULL);
+    res = rdfUtil->MakeSeq(mInner, kNC_BrowserCharsetMenuRoot, nullptr);
     if (NS_FAILED(res)) goto done;
-    res = rdfUtil->MakeSeq(mInner, kNC_BrowserMoreCharsetMenuRoot, NULL);
+    res = rdfUtil->MakeSeq(mInner, kNC_BrowserMoreCharsetMenuRoot, nullptr);
     if (NS_FAILED(res)) goto done;
-    res = rdfUtil->MakeSeq(mInner, kNC_BrowserMore1CharsetMenuRoot, NULL);
+    res = rdfUtil->MakeSeq(mInner, kNC_BrowserMore1CharsetMenuRoot, nullptr);
     if (NS_FAILED(res)) goto done;
-    res = rdfUtil->MakeSeq(mInner, kNC_BrowserMore2CharsetMenuRoot, NULL);
+    res = rdfUtil->MakeSeq(mInner, kNC_BrowserMore2CharsetMenuRoot, nullptr);
     if (NS_FAILED(res)) goto done;
-    res = rdfUtil->MakeSeq(mInner, kNC_BrowserMore3CharsetMenuRoot, NULL);
+    res = rdfUtil->MakeSeq(mInner, kNC_BrowserMore3CharsetMenuRoot, nullptr);
     if (NS_FAILED(res)) goto done;
-    res = rdfUtil->MakeSeq(mInner, kNC_BrowserMore4CharsetMenuRoot, NULL);
+    res = rdfUtil->MakeSeq(mInner, kNC_BrowserMore4CharsetMenuRoot, nullptr);
     if (NS_FAILED(res)) goto done;
-    res = rdfUtil->MakeSeq(mInner, kNC_BrowserMore5CharsetMenuRoot, NULL);
+    res = rdfUtil->MakeSeq(mInner, kNC_BrowserMore5CharsetMenuRoot, nullptr);
     if (NS_FAILED(res)) goto done;
-    res = rdfUtil->MakeSeq(mInner, kNC_MaileditCharsetMenuRoot, NULL);
+    res = rdfUtil->MakeSeq(mInner, kNC_MaileditCharsetMenuRoot, nullptr);
     if (NS_FAILED(res)) goto done;
-    res = rdfUtil->MakeSeq(mInner, kNC_MailviewCharsetMenuRoot, NULL);
+    res = rdfUtil->MakeSeq(mInner, kNC_MailviewCharsetMenuRoot, nullptr);
     if (NS_FAILED(res)) goto done;
-    res = rdfUtil->MakeSeq(mInner, kNC_ComposerCharsetMenuRoot, NULL);
+    res = rdfUtil->MakeSeq(mInner, kNC_ComposerCharsetMenuRoot, nullptr);
     if (NS_FAILED(res)) goto done;
-    res = rdfUtil->MakeSeq(mInner, kNC_DecodersRoot, NULL);
+    res = rdfUtil->MakeSeq(mInner, kNC_DecodersRoot, nullptr);
     if (NS_FAILED(res)) goto done;
-    res = rdfUtil->MakeSeq(mInner, kNC_EncodersRoot, NULL);
+    res = rdfUtil->MakeSeq(mInner, kNC_EncodersRoot, nullptr);
     if (NS_FAILED(res)) goto done;
 
   done:
@@ -793,52 +738,9 @@ nsresult nsCharsetMenu::FreeResources()
     /* nsIObserverService has to have released nsCharsetMenu already */
   }
 
-  mRDFService = NULL;
-  mCCManager  = NULL;
-  mPrefs      = NULL;
-
-  return res;
-}
-
-nsresult nsCharsetMenu::InitBrowserMenu() 
-{
-  nsresult res = NS_OK;
-
-  if (!mBrowserMenuInitialized)  {
-    nsCOMPtr<nsIRDFContainer> container;
-    res = NewRDFContainer(mInner, kNC_BrowserCharsetMenuRoot, getter_AddRefs(container));
-    if (NS_FAILED(res)) return res;
-
-    nsTArray<nsCString> browserDecoderList(mDecoderList);
-
-    res = InitStaticMenu(browserDecoderList, kNC_BrowserCharsetMenuRoot, 
-                         kBrowserStaticPrefKey, &mBrowserMenu);
-    NS_ASSERTION(NS_SUCCEEDED(res), "error initializing browser static charset menu");
-
-    // mark the end of the static area, the rest is cache
-    mBrowserCacheStart = mBrowserMenu.Length();
-    mPrefs->GetIntPref(kBrowserCacheSizePrefKey, &mBrowserCacheSize);
-
-    // compute the position of the menu in the RDF container
-    res = container->GetCount(&mBrowserMenuRDFPosition);
-    if (NS_FAILED(res)) return res;
-    // this "1" here is a correction necessary because the RDF container 
-    // elements are numbered from 1 (why god, WHY?!?!?!)
-    mBrowserMenuRDFPosition -= mBrowserCacheStart - 1;
-
-    // Remove "notForBrowser" entries before populating cache menu
-    res = RemoveFlaggedCharsets(browserDecoderList, NS_LITERAL_STRING(".notForBrowser"));
-    NS_ASSERTION(NS_SUCCEEDED(res), "error initializing static charset menu from prefs");
-
-    res = InitCacheMenu(browserDecoderList, kNC_BrowserCharsetMenuRoot, kBrowserCachePrefKey, 
-      &mBrowserMenu);
-    NS_ASSERTION(NS_SUCCEEDED(res), "error initializing browser cache charset menu");
-
-    // register prefs callback
-    mPrefs->AddObserver(kBrowserStaticPrefKey, mCharsetMenuObserver, false);
-  }
-
-  mBrowserMenuInitialized = NS_SUCCEEDED(res);
+  mRDFService = nullptr;
+  mCCManager  = nullptr;
+  mPrefs      = nullptr;
 
   return res;
 }
@@ -864,7 +766,8 @@ nsresult nsCharsetMenu::InitMaileditMenu()
     nsTArray<nsCString> maileditEncoderList;
     SetArrayFromEnumerator(encoders, maileditEncoderList);
   
-    res = AddFromPrefsToMenu(NULL, container, kMaileditPrefKey, maileditEncoderList, NULL);
+    res = AddFromPrefsToMenu(nullptr, container, kMaileditPrefKey,
+                             maileditEncoderList, nullptr);
     NS_ASSERTION(NS_SUCCEEDED(res), "error initializing mailedit charset menu from prefs");
 
     // register prefs callback
@@ -1098,7 +1001,7 @@ nsresult nsCharsetMenu::InitMoreMenu(nsTArray<nsCString>& aDecs,
   res = ReorderMenuItemArray(&moreMenu);
   if (NS_FAILED(res)) goto done;
 
-  res = AddMenuItemArrayToContainer(container, &moreMenu, NULL);
+  res = AddMenuItemArrayToContainer(container, &moreMenu, nullptr);
   if (NS_FAILED(res)) goto done;
 
 done:
@@ -1127,27 +1030,27 @@ nsresult nsCharsetMenu::InitMoreSubmenus(nsTArray<nsCString>& aDecs)
   res = NewRDFContainer(mInner, kNC_BrowserMore1CharsetMenuRoot, 
     getter_AddRefs(container1));
   if (NS_FAILED(res)) return res;
-  AddFromNolocPrefsToMenu(NULL, container1, key1, aDecs, NULL);
+  AddFromNolocPrefsToMenu(nullptr, container1, key1, aDecs, nullptr);
 
   res = NewRDFContainer(mInner, kNC_BrowserMore2CharsetMenuRoot, 
     getter_AddRefs(container2));
   if (NS_FAILED(res)) return res;
-  AddFromNolocPrefsToMenu(NULL, container2, key2, aDecs, NULL);
+  AddFromNolocPrefsToMenu(nullptr, container2, key2, aDecs, nullptr);
 
   res = NewRDFContainer(mInner, kNC_BrowserMore3CharsetMenuRoot, 
     getter_AddRefs(container3));
   if (NS_FAILED(res)) return res;
-  AddFromNolocPrefsToMenu(NULL, container3, key3, aDecs, NULL);
+  AddFromNolocPrefsToMenu(nullptr, container3, key3, aDecs, nullptr);
 
   res = NewRDFContainer(mInner, kNC_BrowserMore4CharsetMenuRoot, 
     getter_AddRefs(container4));
   if (NS_FAILED(res)) return res;
-  AddFromNolocPrefsToMenu(NULL, container4, key4, aDecs, NULL);
+  AddFromNolocPrefsToMenu(nullptr, container4, key4, aDecs, nullptr);
 
   res = NewRDFContainer(mInner, kNC_BrowserMore5CharsetMenuRoot, 
     getter_AddRefs(container5));
   if (NS_FAILED(res)) return res;
-  AddFromNolocPrefsToMenu(NULL, container5, key5, aDecs, NULL);
+  AddFromNolocPrefsToMenu(nullptr, container5, key5, aDecs, nullptr);
 
   return res;
 }
@@ -1158,12 +1061,12 @@ nsresult nsCharsetMenu::AddCharsetToItemArray(nsTArray<nsMenuEntry*> *aArray,
                                               int32_t aPlace)
 {
   nsresult res = NS_OK;
-  nsMenuEntry * item = NULL; 
+  nsMenuEntry * item = nullptr; 
 
-  if (aResult != NULL) *aResult = NULL;
+  if (aResult != nullptr) *aResult = nullptr;
   
   item = new nsMenuEntry();
-  if (item == NULL) {
+  if (item == nullptr) {
     res = NS_ERROR_OUT_OF_MEMORY;
     goto done;
   }
@@ -1174,7 +1077,7 @@ nsresult nsCharsetMenu::AddCharsetToItemArray(nsTArray<nsMenuEntry*> *aArray,
     item->mTitle.AssignWithConversion(aCharset.get());
   }
 
-  if (aArray != NULL) {
+  if (aArray != nullptr) {
     if (aPlace < 0) {
       aArray->AppendElement(item);
     } else {
@@ -1182,13 +1085,13 @@ nsresult nsCharsetMenu::AddCharsetToItemArray(nsTArray<nsMenuEntry*> *aArray,
     }
   }
 
-  if (aResult != NULL) *aResult = item;
+  if (aResult != nullptr) *aResult = item;
 
   // if we have made another reference to "item", do not delete it 
-  if ((aArray != NULL) || (aResult != NULL)) item = NULL; 
+  if ((aArray != nullptr) || (aResult != nullptr)) item = nullptr; 
 
 done:
-  if (item != NULL) delete item;
+  if (item != nullptr) delete item;
 
   return res;
 }
@@ -1202,7 +1105,7 @@ nsCharsetMenu::AddCharsetArrayToItemArray(nsTArray<nsMenuEntry*>& aArray,
   for (uint32_t i = 0; i < count; i++) {
 
     const nsCString& str = aCharsets[i];
-    nsresult res = AddCharsetToItemArray(&aArray, str, NULL, -1);
+    nsresult res = AddCharsetToItemArray(&aArray, str, nullptr, -1);
     
     if (NS_FAILED(res))
       return res;
@@ -1224,14 +1127,14 @@ nsresult nsCharsetMenu::AddMenuItemToContainer(
   nsCOMPtr<nsIRDFResource> node;
 
   nsAutoCString id;
-  if (aIDPrefix != NULL) id.Assign(aIDPrefix);
+  if (aIDPrefix != nullptr) id.Assign(aIDPrefix);
   id.Append(aItem->mCharset);
 
   // Make up a unique ID and create the RDF NODE
   res = mRDFService->GetResource(id, getter_AddRefs(node));
   if (NS_FAILED(res)) return res;
 
-  const PRUnichar * title = aItem->mTitle.get();
+  const char16_t * title = aItem->mTitle.get();
 
   // set node's title
   nsCOMPtr<nsIRDFLiteral> titleLiteral;
@@ -1246,7 +1149,7 @@ nsresult nsCharsetMenu::AddMenuItemToContainer(
     if (NS_FAILED(res)) return res;
   }
 
-  if (aType != NULL) {
+  if (aType != nullptr) {
     if (aPlace < -1) {
       res = Unassert(node, kRDF_type, aType);
       if (NS_FAILED(res)) return res;
@@ -1281,9 +1184,9 @@ nsresult nsCharsetMenu::AddMenuItemArrayToContainer(
 
   for (uint32_t i = 0; i < count; i++) {
     nsMenuEntry * item = aArray->ElementAt(i);
-    if (item == NULL) return NS_ERROR_UNEXPECTED;
+    if (item == nullptr) return NS_ERROR_UNEXPECTED;
 
-    res = AddMenuItemToContainer(aContainer, item, aType, NULL, -1);
+    res = AddMenuItemToContainer(aContainer, item, aType, nullptr, -1);
     if (NS_FAILED(res)) return res;
   }
 
@@ -1298,20 +1201,20 @@ nsresult nsCharsetMenu::AddCharsetToContainer(nsTArray<nsMenuEntry*> *aArray,
                                               int32_t aRDFPlace)
 {
   nsresult res = NS_OK;
-  nsMenuEntry * item = NULL; 
+  nsMenuEntry * item = nullptr; 
   
   res = AddCharsetToItemArray(aArray, aCharset, &item, aPlace);
   if (NS_FAILED(res)) goto done;
 
-  res = AddMenuItemToContainer(aContainer, item, NULL, aIDPrefix, 
+  res = AddMenuItemToContainer(aContainer, item, nullptr, aIDPrefix, 
     aPlace + aRDFPlace);
   if (NS_FAILED(res)) goto done;
 
   // if we have made another reference to "item", do not delete it 
-  if (aArray != NULL) item = NULL; 
+  if (aArray != nullptr) item = nullptr; 
 
 done:
-  if (item != NULL) delete item;
+  if (item != nullptr) delete item;
 
   return res;
 }
@@ -1350,11 +1253,11 @@ nsCharsetMenu::AddFromNolocPrefsToMenu(nsTArray<nsMenuEntry*> * aArray,
 {
   nsresult res = NS_OK;
 
-  char * value = NULL;
+  char * value = nullptr;
   res = mPrefs->GetCharPref(aKey, &value);
   if (NS_FAILED(res)) return res;
 
-  if (value != NULL) {
+  if (value != nullptr) {
     res = AddFromStringToMenu(value, aArray, aContainer, aDecs, aIDPrefix);
     nsMemory::Free(value);
   }
@@ -1414,7 +1317,7 @@ nsresult nsCharsetMenu::AddSeparatorToContainer(nsIRDFContainer * aContainer)
   item.mTitle.AssignWithConversion(str.get());
 
   return AddMenuItemToContainer(aContainer, &item, kNC_BookmarkSeparator, 
-    NULL, -1);
+    nullptr, -1);
 }
 
 nsresult
@@ -1428,7 +1331,7 @@ nsCharsetMenu::AddCharsetToCache(const nsAFlatCString& aCharset,
   int32_t i;
   nsresult res = NS_OK;
 
-  i = FindMenuItemInArray(aArray, aCharset, NULL);
+  i = FindMenuItemInArray(aArray, aCharset, nullptr);
   if (i >= 0) return res;
 
   nsCOMPtr<nsIRDFContainer> container;
@@ -1460,7 +1363,7 @@ nsresult nsCharsetMenu::WriteCacheToPrefs(nsTArray<nsMenuEntry*> * aArray,
 
   for (uint32_t i = aCacheStart; i < count; i++) {
     nsMenuEntry * item = aArray->ElementAt(i);
-    if (item != NULL) {    
+    if (item != nullptr) {    
       cache.Append(item->mCharset);
       if (i < count - 1) {
         cache.Append(sep);
@@ -1477,7 +1380,7 @@ nsresult nsCharsetMenu::WriteCacheToPrefs(nsTArray<nsMenuEntry*> * aArray,
 nsresult nsCharsetMenu::UpdateCachePrefs(const char * aCacheKey,
                                          const char * aCacheSizeKey,
                                          const char * aStaticKey,
-                                         const PRUnichar * aCharset)
+                                         const char16_t * aCharset)
 {
   nsresult rv = NS_OK;
   nsXPIDLCString cachePrefValue;
@@ -1517,8 +1420,8 @@ nsresult nsCharsetMenu::ClearMenu(nsIRDFContainer        * aContainer,
   uint32_t count = aArray->Length();
   for (uint32_t i = 0; i < count; i++) {
     nsMenuEntry * item = aArray->ElementAt(i);
-    if (item != NULL) {    
-      res = AddMenuItemToContainer(aContainer, item, NULL, "charset.", -2);
+    if (item != nullptr) {    
+      res = AddMenuItemToContainer(aContainer, item, nullptr, "charset.", -2);
       if (NS_FAILED(res)) return res;
     }
   }
@@ -1537,8 +1440,8 @@ nsresult nsCharsetMenu::RemoveLastMenuItem(nsIRDFContainer * aContainer,
   int32_t last = aArray->Length() - 1;
   if (last >= 0) {
     nsMenuEntry * item = aArray->ElementAt(last);
-    if (item != NULL) {    
-      res = AddMenuItemToContainer(aContainer, item, NULL, "charset.", -2);
+    if (item != nullptr) {    
+      res = AddMenuItemToContainer(aContainer, item, nullptr, "charset.", -2);
       if (NS_FAILED(res)) return res;
 
       aArray->RemoveElementAt(last);
@@ -1590,7 +1493,7 @@ void nsCharsetMenu::FreeMenuItemArray(nsTArray<nsMenuEntry*> * aArray)
   uint32_t count = aArray->Length();
   for (uint32_t i = 0; i < count; i++) {
     nsMenuEntry * item = aArray->ElementAt(i);
-    if (item != NULL) {
+    if (item != nullptr) {
       delete item;
     }
   }
@@ -1606,12 +1509,12 @@ int32_t nsCharsetMenu::FindMenuItemInArray(const nsTArray<nsMenuEntry*>* aArray,
   for (uint32_t i=0; i < count; i++) {
     nsMenuEntry * item = aArray->ElementAt(i);
     if (item->mCharset == aCharset) {
-      if (aResult != NULL) *aResult = item;
+      if (aResult != nullptr) *aResult = item;
       return i;
     }
   }
 
-  if (aResult != NULL) *aResult = NULL;
+  if (aResult != nullptr) *aResult = nullptr;
   return -1;
 }
 
@@ -1682,36 +1585,12 @@ nsresult nsCharsetMenu::GetCollation(nsICollation ** aCollation)
 //----------------------------------------------------------------------------
 // Interface nsICurrentCharsetListener [implementation]
 
-NS_IMETHODIMP nsCharsetMenu::SetCurrentCharset(const PRUnichar * aCharset)
+NS_IMETHODIMP nsCharsetMenu::SetCurrentCharset(const char16_t * aCharset)
 {
-  nsresult res = NS_OK;
-
-  if (mBrowserMenuInitialized) {
-    // Don't add item to the cache if it's marked "notForBrowser"
-    nsAutoString str;
-    res = mCCManager->GetCharsetData(NS_LossyConvertUTF16toASCII(aCharset).get(),
-                                     NS_LITERAL_STRING(".notForBrowser").get(), str);
-    if (NS_SUCCEEDED(res)) // succeeded means attribute exists
-      return res; // don't throw
-
-    res = AddCharsetToCache(NS_LossyConvertUTF16toASCII(aCharset),
-                            &mBrowserMenu, kNC_BrowserCharsetMenuRoot, 
-                            mBrowserCacheStart, mBrowserCacheSize,
-                            mBrowserMenuRDFPosition);
-    if (NS_FAILED(res)) {
-        return res;
-    }
-
-    res = WriteCacheToPrefs(&mBrowserMenu, mBrowserCacheStart, 
-      kBrowserCachePrefKey);
-  } else {
-    res = UpdateCachePrefs(kBrowserCachePrefKey, kBrowserCacheSizePrefKey, 
-                           kBrowserStaticPrefKey, aCharset);
-  }
-  return res;
+  return NS_OK;
 }
 
-NS_IMETHODIMP nsCharsetMenu::SetCurrentMailCharset(const PRUnichar * aCharset)
+NS_IMETHODIMP nsCharsetMenu::SetCurrentMailCharset(const char16_t * aCharset)
 {
   nsresult res = NS_OK;
 
@@ -1731,7 +1610,7 @@ NS_IMETHODIMP nsCharsetMenu::SetCurrentMailCharset(const PRUnichar * aCharset)
   return res;
 }
 
-NS_IMETHODIMP nsCharsetMenu::SetCurrentComposerCharset(const PRUnichar * aCharset)
+NS_IMETHODIMP nsCharsetMenu::SetCurrentComposerCharset(const char16_t * aCharset)
 {
   nsresult res = NS_OK;
 
