@@ -6,9 +6,8 @@
 
 #include "AsyncFaviconHelpers.h"
 
-#include "nsIContentSniffer.h"
 #include "nsICacheService.h"
-#include "nsICacheVisitor.h"
+#include "nsICacheEntry.h"
 #include "nsICachingChannel.h"
 #include "nsIAsyncVerifyRedirectCallback.h"
 
@@ -18,8 +17,9 @@
 #include "nsNetUtil.h"
 #include "nsPrintfCString.h"
 #include "nsStreamUtils.h"
-
-#define CONTENT_SNIFFING_SERVICES "content-sniffing-services"
+#include "nsIPrivateBrowsingChannel.h"
+#include "nsISupportsPriority.h"
+#include <algorithm>
 
 using namespace mozilla::places;
 using namespace mozilla::storage;
@@ -90,7 +90,7 @@ FetchPageInfo(nsRefPtr<Database>& aDB,
   bool isNull;
   rv = stmt->GetIsNull(1, &isNull);
   NS_ENSURE_SUCCESS(rv, rv);
-  // favicon_id can be NULL.
+  // favicon_id can be nullptr.
   if (!isNull) {
     rv = stmt->GetInt64(1, &_page.iconId);
     NS_ENSURE_SUCCESS(rv, rv);
@@ -210,42 +210,42 @@ FetchIconInfo(nsRefPtr<Database>& aDB,
   NS_ENSURE_STATE(stmt);
   mozStorageStatementScoper scoper(stmt);
 
-  nsresult rv = URIBinder::Bind(stmt, NS_LITERAL_CSTRING("icon_url"),
-                                _icon.spec);
-  NS_ENSURE_SUCCESS(rv, rv);
+  DebugOnly<nsresult> rv = URIBinder::Bind(stmt, NS_LITERAL_CSTRING("icon_url"),
+                                           _icon.spec);
+  MOZ_ASSERT(NS_SUCCEEDED(rv));
 
   bool hasResult;
   rv = stmt->ExecuteStep(&hasResult);
-  NS_ENSURE_SUCCESS(rv, rv);
+  MOZ_ASSERT(NS_SUCCEEDED(rv));
   if (!hasResult) {
     // The icon does not exist yet, bail out.
     return NS_OK;
   }
 
   rv = stmt->GetInt64(0, &_icon.id);
-  NS_ENSURE_SUCCESS(rv, rv);
+  MOZ_ASSERT(NS_SUCCEEDED(rv));
 
-  // Expiration can be NULL.
+  // Expiration can be nullptr.
   bool isNull;
   rv = stmt->GetIsNull(1, &isNull);
-  NS_ENSURE_SUCCESS(rv, rv);
+  MOZ_ASSERT(NS_SUCCEEDED(rv));
   if (!isNull) {
-    rv = stmt->GetInt64(1, &_icon.expiration);
-    NS_ENSURE_SUCCESS(rv, rv);
+    rv = stmt->GetInt64(1, reinterpret_cast<int64_t*>(&_icon.expiration));
+    MOZ_ASSERT(NS_SUCCEEDED(rv));
   }
 
-  // Data can be NULL.
+  // Data can be nullptr.
   rv = stmt->GetIsNull(2, &isNull);
-  NS_ENSURE_SUCCESS(rv, rv);
+  MOZ_ASSERT(NS_SUCCEEDED(rv));
   if (!isNull) {
-    PRUint8* data;
-    PRUint32 dataLen = 0;
+    uint8_t* data;
+    uint32_t dataLen = 0;
     rv = stmt->GetBlob(2, &dataLen, &data);
-    NS_ENSURE_SUCCESS(rv, rv);
+    MOZ_ASSERT(NS_SUCCEEDED(rv));
     _icon.data.Adopt(TO_CHARBUFFER(data), dataLen);
     // Read mime only if we have data.
     rv = stmt->GetUTF8String(3, _icon.mimeType);
-    NS_ENSURE_SUCCESS(rv, rv);
+    MOZ_ASSERT(NS_SUCCEEDED(rv));
   }
 
   return NS_OK;
@@ -284,55 +284,6 @@ FetchIconURL(nsRefPtr<Database>& aDB,
   return NS_OK;
 }
 
-/**
- * Tries to guess the mimeType from icon data.
- *
- * @param aRequest
- *        The network request object.
- * @param aData
- *        Data for this icon.
- * @param _mimeType
- *        The guessed mime-type or empty string if a valid one can't be found.
- */
-nsresult
-SniffMimeTypeForIconData(nsIRequest* aRequest,
-                         const nsCString& aData,
-                         nsCString& _mimeType)
-{
-  NS_PRECONDITION(NS_IsMainThread(),
-                  "This should be called on the main thread");
-
-  nsCOMPtr<nsICategoryManager> categoryManager =
-    do_GetService(NS_CATEGORYMANAGER_CONTRACTID);
-  NS_ENSURE_TRUE(categoryManager, NS_ERROR_OUT_OF_MEMORY);
-  nsCOMPtr<nsISimpleEnumerator> sniffers;
-  nsresult rv = categoryManager->EnumerateCategory(CONTENT_SNIFFING_SERVICES,
-                                                   getter_AddRefs(sniffers));
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  bool hasMore = false;
-  while (_mimeType.IsEmpty() &&
-         NS_SUCCEEDED(sniffers->HasMoreElements(&hasMore)) &&
-         hasMore) {
-    nsCOMPtr<nsISupports> snifferCIDSupports;
-    rv = sniffers->GetNext(getter_AddRefs(snifferCIDSupports));
-    NS_ENSURE_SUCCESS(rv, rv);
-    nsCOMPtr<nsISupportsCString> snifferCIDSupportsCString =
-      do_QueryInterface(snifferCIDSupports);
-    NS_ENSURE_STATE(snifferCIDSupports);
-    nsCAutoString snifferCID;
-    rv = snifferCIDSupportsCString->GetData(snifferCID);
-    NS_ENSURE_SUCCESS(rv, rv);
-    nsCOMPtr<nsIContentSniffer> sniffer = do_GetService(snifferCID.get());
-    NS_ENSURE_STATE(sniffer);
-
-     // Ignore errors: we'll try the next sniffer.
-    (void)sniffer->GetMIMETypeFromContent(aRequest, TO_INTBUFFER(aData),
-                                          aData.Length(), _mimeType);
-  }
-  return NS_OK;
-}
-
 /**
  * Tries to compute the expiration time for a icon from the channel.
  *
@@ -354,12 +305,12 @@ GetExpirationTimeFromChannel(nsIChannel* aChannel)
     nsCOMPtr<nsISupports> cacheToken;
     nsresult rv = cachingChannel->GetCacheToken(getter_AddRefs(cacheToken));
     if (NS_SUCCEEDED(rv)) {
-      nsCOMPtr<nsICacheEntryInfo> cacheEntry = do_QueryInterface(cacheToken);
-      PRUint32 seconds;
+      nsCOMPtr<nsICacheEntry> cacheEntry = do_QueryInterface(cacheToken);
+      uint32_t seconds;
       rv = cacheEntry->GetExpirationTime(&seconds);
       if (NS_SUCCEEDED(rv)) {
         // Set the expiration, but make sure we honor our cap.
-        expiration = PR_Now() + NS_MIN((PRTime)seconds * PR_USEC_PER_SEC,
+        expiration = PR_Now() + std::min((PRTime)seconds * PR_USEC_PER_SEC,
                                        MAX_FAVICON_EXPIRATION);
       }
     }
@@ -389,7 +340,7 @@ OptimizeIconSize(IconData& aIcon,
   // Even if the page provides a large image for the favicon (eg, a highres
   // image or a multiresolution .ico file), don't try to store more data than
   // needed.
-  nsCAutoString newData, newMimeType;
+  nsAutoCString newData, newMimeType;
   if (aIcon.data.Length() > MAX_ICON_FILESIZE(aFaviconSvc->GetOptimizedIconDimension())) {
     nsresult rv = aFaviconSvc->OptimizeFaviconImage(TO_INTBUFFER(aIcon.data),
                                                     aIcon.data.Length(),
@@ -435,6 +386,7 @@ nsresult
 AsyncFetchAndSetIconForPage::start(nsIURI* aFaviconURI,
                                    nsIURI* aPageURI,
                                    enum AsyncFaviconFetchMode aFetchMode,
+                                   uint32_t aFaviconLoadType,
                                    nsIFaviconDataCallback* aCallback)
 {
   NS_PRECONDITION(NS_IsMainThread(),
@@ -450,7 +402,7 @@ AsyncFetchAndSetIconForPage::start(nsIURI* aFaviconURI,
   NS_ENSURE_TRUE(navHistory, NS_ERROR_OUT_OF_MEMORY);
   rv = navHistory->CanAddURI(aPageURI, &canAddToHistory);
   NS_ENSURE_SUCCESS(rv, rv);
-  page.canAddToHistory = !!canAddToHistory;
+  page.canAddToHistory = !!canAddToHistory && aFaviconLoadType != nsIFaviconService::FAVICON_LOAD_PRIVATE;
 
   IconData icon;
 
@@ -481,7 +433,7 @@ AsyncFetchAndSetIconForPage::start(nsIURI* aFaviconURI,
   // The event will swap owning pointers, thus we need a new pointer.
   nsCOMPtr<nsIFaviconDataCallback> callback(aCallback);
   nsRefPtr<AsyncFetchAndSetIconForPage> event =
-    new AsyncFetchAndSetIconForPage(icon, page, callback);
+    new AsyncFetchAndSetIconForPage(icon, page, aFaviconLoadType, callback);
 
   // Get the target thread and start the work.
   nsRefPtr<Database> DB = Database::GetDatabase();
@@ -494,10 +446,12 @@ AsyncFetchAndSetIconForPage::start(nsIURI* aFaviconURI,
 AsyncFetchAndSetIconForPage::AsyncFetchAndSetIconForPage(
   IconData& aIcon
 , PageData& aPage
+, uint32_t aFaviconLoadType
 , nsCOMPtr<nsIFaviconDataCallback>& aCallback
 ) : AsyncFaviconHelperBase(aCallback)
   , mIcon(aIcon)
   , mPage(aPage)
+  , mFaviconLoadPrivate(aFaviconLoadType == nsIFaviconService::FAVICON_LOAD_PRIVATE)
 {
 }
 
@@ -533,7 +487,7 @@ AsyncFetchAndSetIconForPage::Run()
     // Fetch the icon from network.  When done this will associate the
     // icon to the page and notify.
     nsRefPtr<AsyncFetchAndSetIconFromNetwork> event =
-      new AsyncFetchAndSetIconFromNetwork(mIcon, mPage, mCallback);
+      new AsyncFetchAndSetIconFromNetwork(mIcon, mPage, mFaviconLoadPrivate, mCallback);
 
     // Start the work on the main thread.
     rv = NS_DispatchToMainThread(event);
@@ -557,21 +511,18 @@ NS_IMPL_ISUPPORTS_INHERITED3(
 AsyncFetchAndSetIconFromNetwork::AsyncFetchAndSetIconFromNetwork(
   IconData& aIcon
 , PageData& aPage
+, bool aFaviconLoadPrivate
 , nsCOMPtr<nsIFaviconDataCallback>& aCallback
 )
 : AsyncFaviconHelperBase(aCallback)
 , mIcon(aIcon)
 , mPage(aPage)
+, mFaviconLoadPrivate(aFaviconLoadPrivate)
 {
 }
 
 AsyncFetchAndSetIconFromNetwork::~AsyncFetchAndSetIconFromNetwork()
 {
-  nsCOMPtr<nsIThread> thread;
-  (void)NS_GetMainThread(getter_AddRefs(thread));
-  if (mChannel) {
-    (void)NS_ProxyRelease(thread, mChannel, true);
-  }
 }
 
 NS_IMETHODIMP
@@ -589,17 +540,26 @@ AsyncFetchAndSetIconFromNetwork::Run()
   nsCOMPtr<nsIURI> iconURI;
   nsresult rv = NS_NewURI(getter_AddRefs(iconURI), mIcon.spec);
   NS_ENSURE_SUCCESS(rv, rv);
-  rv = NS_NewChannel(getter_AddRefs(mChannel), iconURI);
+  nsCOMPtr<nsIChannel> channel;
+  rv = NS_NewChannel(getter_AddRefs(channel), iconURI);
   NS_ENSURE_SUCCESS(rv, rv);
   nsCOMPtr<nsIInterfaceRequestor> listenerRequestor =
     do_QueryInterface(reinterpret_cast<nsISupports*>(this));
   NS_ENSURE_STATE(listenerRequestor);
-  rv = mChannel->SetNotificationCallbacks(listenerRequestor);
-  NS_ENSURE_SUCCESS(rv, rv);
-  rv = mChannel->AsyncOpen(this, nsnull);
+  rv = channel->SetNotificationCallbacks(listenerRequestor);
   NS_ENSURE_SUCCESS(rv, rv);
+  nsCOMPtr<nsIPrivateBrowsingChannel> pbChannel = do_QueryInterface(channel);
+  if (pbChannel) {
+    rv = pbChannel->SetPrivate(mFaviconLoadPrivate);
+    NS_ENSURE_SUCCESS(rv, rv);
+  }
 
-  return NS_OK;
+  nsCOMPtr<nsISupportsPriority> priorityChannel = do_QueryInterface(channel);
+  if (priorityChannel) {
+    priorityChannel->AdjustPriority(nsISupportsPriority::PRIORITY_LOWEST);
+  }
+
+  return channel->AsyncOpen(this, nullptr);
 }
 
 NS_IMETHODIMP
@@ -613,10 +573,10 @@ NS_IMETHODIMP
 AsyncFetchAndSetIconFromNetwork::OnDataAvailable(nsIRequest* aRequest,
                                                  nsISupports* aContext,
                                                  nsIInputStream* aInputStream,
-                                                 PRUint32 aOffset,
-                                                 PRUint32 aCount)
+                                                 uint64_t aOffset,
+                                                 uint32_t aCount)
 {
-  nsCAutoString buffer;
+  nsAutoCString buffer;
   nsresult rv = NS_ConsumeStream(aInputStream, aCount, buffer);
   if (rv != NS_BASE_STREAM_WOULD_BLOCK && NS_FAILED(rv)) {
     return rv;
@@ -639,11 +599,10 @@ NS_IMETHODIMP
 AsyncFetchAndSetIconFromNetwork::AsyncOnChannelRedirect(
   nsIChannel* oldChannel
 , nsIChannel* newChannel
-, PRUint32 flags
+, uint32_t flags
 , nsIAsyncVerifyRedirectCallback *cb
 )
 {
-  mChannel = newChannel;
   (void)cb->OnRedirectVerifyCallback(NS_OK);
   return NS_OK;
 }
@@ -658,18 +617,21 @@ AsyncFetchAndSetIconFromNetwork::OnStopRequest(nsIRequest* aRequest,
   nsFaviconService* favicons = nsFaviconService::GetFaviconService();
   NS_ENSURE_STATE(favicons);
 
+  nsresult rv;
+
   // If fetching the icon failed, add it to the failed cache.
   if (NS_FAILED(aStatusCode) || mIcon.data.Length() == 0) {
     nsCOMPtr<nsIURI> iconURI;
-    nsresult rv = NS_NewURI(getter_AddRefs(iconURI), mIcon.spec);
+    rv = NS_NewURI(getter_AddRefs(iconURI), mIcon.spec);
     NS_ENSURE_SUCCESS(rv, rv);
     rv = favicons->AddFailedFavicon(iconURI);
     NS_ENSURE_SUCCESS(rv, rv);
     return NS_OK;
   }
 
-  nsresult rv = SniffMimeTypeForIconData(aRequest, mIcon.data, mIcon.mimeType);
-  NS_ENSURE_SUCCESS(rv, rv);
+  NS_SniffContent(NS_DATA_SNIFFER_CATEGORY, aRequest,
+                  TO_INTBUFFER(mIcon.data), mIcon.data.Length(),
+                  mIcon.mimeType);
 
   // If the icon does not have a valid MIME type, add it to the failed cache.
   if (mIcon.mimeType.IsEmpty()) {
@@ -681,7 +643,11 @@ AsyncFetchAndSetIconFromNetwork::OnStopRequest(nsIRequest* aRequest,
     return NS_OK;
   }
 
-  mIcon.expiration = GetExpirationTimeFromChannel(mChannel);
+  nsCOMPtr<nsIChannel> channel = do_QueryInterface(aRequest);
+  // aRequest should always QI to nsIChannel.
+  // See AsyncFetchAndSetIconFromNetwork::Run()
+  MOZ_ASSERT(channel);
+  mIcon.expiration = GetExpirationTimeFromChannel(channel);
 
   rv = OptimizeIconSize(mIcon, favicons);
   NS_ENSURE_SUCCESS(rv, rv);
@@ -812,7 +778,7 @@ AsyncGetFaviconURLForPage::start(nsIURI* aPageURI,
   NS_PRECONDITION(NS_IsMainThread(),
                   "This should be called on the main thread.");
 
-  nsCAutoString pageSpec;
+  nsAutoCString pageSpec;
   nsresult rv = aPageURI->GetSpec(pageSpec);
   NS_ENSURE_SUCCESS(rv, rv);
 
@@ -845,9 +811,9 @@ AsyncGetFaviconURLForPage::Run()
   NS_PRECONDITION(!NS_IsMainThread(),
                   "This should not be called on the main thread.");
 
-  nsCAutoString iconSpec;
+  nsAutoCString iconSpec;
   nsresult rv = FetchIconURL(mDB, mPageSpec, iconSpec);
-  MOZ_ASSERT(NS_SUCCEEDED(rv));
+  NS_ENSURE_SUCCESS(rv, rv);
 
   // Now notify our callback of the icon spec we retrieved, even if empty.
   IconData iconData;
@@ -877,7 +843,7 @@ AsyncGetFaviconDataForPage::start(nsIURI* aPageURI,
   NS_PRECONDITION(NS_IsMainThread(),
                   "This should be called on the main thread.");
 
-  nsCAutoString pageSpec;
+  nsAutoCString pageSpec;
   nsresult rv = aPageURI->GetSpec(pageSpec);
   NS_ENSURE_SUCCESS(rv, rv);
 
@@ -910,9 +876,9 @@ AsyncGetFaviconDataForPage::Run()
   NS_PRECONDITION(!NS_IsMainThread(),
                   "This should not be called on the main thread.");
 
-  nsCAutoString iconSpec;
+  nsAutoCString iconSpec;
   nsresult rv = FetchIconURL(mDB, mPageSpec, iconSpec);
-  MOZ_ASSERT(NS_SUCCEEDED(rv));
+  NS_ENSURE_SUCCESS(rv, rv);
 
   IconData iconData;
   iconData.spec.Assign(iconSpec);
@@ -924,7 +890,6 @@ AsyncGetFaviconDataForPage::Run()
     rv = FetchIconInfo(mDB, iconData);
     if (NS_FAILED(rv)) {
       iconData.spec.Truncate();
-      MOZ_NOT_REACHED("Fetching favicon information failed unexpectedly.");
     }
   }
 
