@@ -3,7 +3,7 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-#include "mozilla/Util.h"
+#include "mozilla/ArrayUtils.h"
 
 #include "nsAnnotationService.h"
 #include "nsNavHistory.h"
@@ -18,9 +18,12 @@
 #include "nsVariant.h"
 #include "mozilla/storage.h"
 
-#include "sampler.h"
+#include "GeckoProfiler.h"
+
+#include "nsNetCID.h"
 
 using namespace mozilla;
+using namespace mozilla::places;
 
 #define ENSURE_ANNO_TYPE(_type, _statement)                                    \
   PR_BEGIN_MACRO                                                               \
@@ -37,15 +40,71 @@ using namespace mozilla;
 const int32_t nsAnnotationService::kAnnoIndex_ID = 0;
 const int32_t nsAnnotationService::kAnnoIndex_PageOrItem = 1;
 const int32_t nsAnnotationService::kAnnoIndex_NameID = 2;
-const int32_t nsAnnotationService::kAnnoIndex_MimeType = 3;
-const int32_t nsAnnotationService::kAnnoIndex_Content = 4;
-const int32_t nsAnnotationService::kAnnoIndex_Flags = 5;
-const int32_t nsAnnotationService::kAnnoIndex_Expiration = 6;
-const int32_t nsAnnotationService::kAnnoIndex_Type = 7;
-const int32_t nsAnnotationService::kAnnoIndex_DateAdded = 8;
-const int32_t nsAnnotationService::kAnnoIndex_LastModified = 9;
+const int32_t nsAnnotationService::kAnnoIndex_Content = 3;
+const int32_t nsAnnotationService::kAnnoIndex_Flags = 4;
+const int32_t nsAnnotationService::kAnnoIndex_Expiration = 5;
+const int32_t nsAnnotationService::kAnnoIndex_Type = 6;
+const int32_t nsAnnotationService::kAnnoIndex_DateAdded = 7;
+const int32_t nsAnnotationService::kAnnoIndex_LastModified = 8;
 
-using namespace mozilla::places;
+namespace mozilla {
+namespace places {
+
+////////////////////////////////////////////////////////////////////////////////
+//// AnnotatedResult
+
+AnnotatedResult::AnnotatedResult(const nsCString& aGUID,
+                                 nsIURI* aURI,
+                                 int64_t aItemId,
+                                 const nsACString& aAnnotationName,
+                                 nsIVariant* aAnnotationValue)
+: mGUID(aGUID)
+, mURI(aURI)
+, mItemId(aItemId)
+, mAnnotationName(aAnnotationName)
+, mAnnotationValue(aAnnotationValue)
+{
+}
+
+NS_IMETHODIMP
+AnnotatedResult::GetGuid(nsACString& _guid)
+{
+  _guid = mGUID;
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+AnnotatedResult::GetUri(nsIURI** _uri)
+{
+  NS_IF_ADDREF(*_uri = mURI);
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+AnnotatedResult::GetItemId(int64_t* _itemId)
+{
+  *_itemId = mItemId;
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+AnnotatedResult::GetAnnotationName(nsACString& _annotationName)
+{
+  _annotationName = mAnnotationName;
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+AnnotatedResult::GetAnnotationValue(nsIVariant** _annotationValue)
+{
+  NS_IF_ADDREF(*_annotationValue = mAnnotationValue);
+  return NS_OK;
+}
+
+NS_IMPL_ISUPPORTS1(AnnotatedResult, mozIAnnotatedResult)
+
+} // namespace places
+} // namespace mozilla
 
 PLACES_FACTORY_SINGLETON_IMPLEMENTATION(nsAnnotationService, gAnnotationService)
 
@@ -106,8 +165,6 @@ nsAnnotationService::SetAnnotationStringInternal(nsIURI* aURI,
 
   rv = statement->BindStringByName(NS_LITERAL_CSTRING("content"), aValue);
   NS_ENSURE_SUCCESS(rv, rv);
-  rv = statement->BindNullByName(NS_LITERAL_CSTRING("mime_type"));
-  NS_ENSURE_SUCCESS(rv, rv);
 
   rv = statement->Execute();
   NS_ENSURE_SUCCESS(rv, rv);
@@ -202,7 +259,7 @@ nsAnnotationService::SetItemAnnotation(int64_t aItemId,
                                        int32_t aFlags,
                                        uint16_t aExpiration)
 {
-  SAMPLE_LABEL("AnnotationService", "SetItemAnnotation");
+  PROFILER_LABEL("AnnotationService", "SetItemAnnotation");
   NS_ENSURE_ARG_MIN(aItemId, 1);
   NS_ENSURE_ARG(aValue);
 
@@ -284,9 +341,6 @@ nsAnnotationService::SetPageAnnotationString(nsIURI* aURI,
 {
   NS_ENSURE_ARG(aURI);
 
-  if (InPrivateBrowsingMode())
-    return NS_OK;
-
   nsresult rv = SetAnnotationStringInternal(aURI, 0, aName, aValue,
                                             aFlags, aExpiration);
   NS_ENSURE_SUCCESS(rv, rv);
@@ -337,8 +391,6 @@ nsAnnotationService::SetAnnotationInt32Internal(nsIURI* aURI,
 
   rv = statement->BindInt32ByName(NS_LITERAL_CSTRING("content"), aValue);
   NS_ENSURE_SUCCESS(rv, rv);
-  rv = statement->BindNullByName(NS_LITERAL_CSTRING("mime_type"));
-  NS_ENSURE_SUCCESS(rv, rv);
 
   rv = statement->Execute();
   NS_ENSURE_SUCCESS(rv, rv);
@@ -359,9 +411,6 @@ nsAnnotationService::SetPageAnnotationInt32(nsIURI* aURI,
 {
   NS_ENSURE_ARG(aURI);
 
-  if (InPrivateBrowsingMode())
-    return NS_OK;
-
   nsresult rv = SetAnnotationInt32Internal(aURI, 0, aName, aValue,
                                            aFlags, aExpiration);
   NS_ENSURE_SUCCESS(rv, rv);
@@ -412,8 +461,6 @@ nsAnnotationService::SetAnnotationInt64Internal(nsIURI* aURI,
 
   rv = statement->BindInt64ByName(NS_LITERAL_CSTRING("content"), aValue);
   NS_ENSURE_SUCCESS(rv, rv);
-  rv = statement->BindNullByName(NS_LITERAL_CSTRING("mime_type"));
-  NS_ENSURE_SUCCESS(rv, rv);
 
   rv = statement->Execute();
   NS_ENSURE_SUCCESS(rv, rv);
@@ -434,9 +481,6 @@ nsAnnotationService::SetPageAnnotationInt64(nsIURI* aURI,
 {
   NS_ENSURE_ARG(aURI);
 
-  if (InPrivateBrowsingMode())
-    return NS_OK;
-
   nsresult rv = SetAnnotationInt64Internal(aURI, 0, aName, aValue,
                                            aFlags, aExpiration);
   NS_ENSURE_SUCCESS(rv, rv);
@@ -487,8 +531,6 @@ nsAnnotationService::SetAnnotationDoubleInternal(nsIURI* aURI,
 
   rv = statement->BindDoubleByName(NS_LITERAL_CSTRING("content"), aValue);
   NS_ENSURE_SUCCESS(rv, rv);
-  rv = statement->BindNullByName(NS_LITERAL_CSTRING("mime_type"));
-  NS_ENSURE_SUCCESS(rv, rv);
 
   rv = statement->Execute();
   NS_ENSURE_SUCCESS(rv, rv);
@@ -509,9 +551,6 @@ nsAnnotationService::SetPageAnnotationDouble(nsIURI* aURI,
 {
   NS_ENSURE_ARG(aURI);
 
-  if (InPrivateBrowsingMode())
-    return NS_OK;
-
   nsresult rv = SetAnnotationDoubleInternal(aURI, 0, aName, aValue,
                                             aFlags, aExpiration);
   NS_ENSURE_SUCCESS(rv, rv);
@@ -543,92 +582,6 @@ nsAnnotationService::SetItemAnnotationDouble(int64_t aItemId,
   return NS_OK;
 }
 
-
-nsresult
-nsAnnotationService::SetAnnotationBinaryInternal(nsIURI* aURI,
-                                                 int64_t aItemId,
-                                                 const nsACString& aName,
-                                                 const uint8_t* aData,
-                                                 uint32_t aDataLen,
-                                                 const nsACString& aMimeType,
-                                                 int32_t aFlags,
-                                                 uint16_t aExpiration)
-{
-  if (aMimeType.Length() == 0)
-    return NS_ERROR_INVALID_ARG;
-
-  mozStorageTransaction transaction(mDB->MainConn(), false);
-  nsCOMPtr<mozIStorageStatement> statement;
-  nsresult rv = StartSetAnnotation(aURI, aItemId, aName, aFlags, aExpiration,
-                                   nsIAnnotationService::TYPE_BINARY,
-                                   statement);
-  NS_ENSURE_SUCCESS(rv, rv);
-  mozStorageStatementScoper scoper(statement);
-
-  rv = statement->BindBlobByName(NS_LITERAL_CSTRING("content"), aData, aDataLen);
-  NS_ENSURE_SUCCESS(rv, rv);
-  rv = statement->BindUTF8StringByName(NS_LITERAL_CSTRING("mime_type"), aMimeType);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  rv = statement->Execute();
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  rv = transaction.Commit();
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  return NS_OK;
-}
-
-
-NS_IMETHODIMP
-nsAnnotationService::SetPageAnnotationBinary(nsIURI* aURI,
-                                             const nsACString& aName,
-                                             const uint8_t* aData,
-                                             uint32_t aDataLen,
-                                             const nsACString& aMimeType,
-                                             int32_t aFlags,
-                                             uint16_t aExpiration)
-{
-  NS_ENSURE_ARG(aURI);
-
-  if (InPrivateBrowsingMode())
-    return NS_OK;
-
-  nsresult rv = SetAnnotationBinaryInternal(aURI, 0, aName, aData, aDataLen,
-                                            aMimeType, aFlags, aExpiration);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  NOTIFY_ANNOS_OBSERVERS(OnPageAnnotationSet(aURI, aName));
-
-  return NS_OK;
-}
-
-
-NS_IMETHODIMP
-nsAnnotationService::SetItemAnnotationBinary(int64_t aItemId,
-                                             const nsACString& aName,
-                                             const uint8_t* aData,
-                                             uint32_t aDataLen,
-                                             const nsACString& aMimeType,
-                                             int32_t aFlags,
-                                             uint16_t aExpiration)
-{
-  NS_ENSURE_ARG_MIN(aItemId, 1);
-
-  if (aExpiration == EXPIRE_WITH_HISTORY)
-    return NS_ERROR_INVALID_ARG;
-
-  nsresult rv = SetAnnotationBinaryInternal(nullptr, aItemId, aName, aData,
-                                            aDataLen, aMimeType, aFlags,
-                                            aExpiration);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  NOTIFY_ANNOS_OBSERVERS(OnItemAnnotationSet(aItemId, aName));
-
-  return NS_OK;
-}
-
-
 NS_IMETHODIMP
 nsAnnotationService::GetPageAnnotationString(nsIURI* aURI,
                                              const nsACString& aName,
@@ -702,10 +655,6 @@ nsAnnotationService::GetPageAnnotation(nsIURI* aURI,
         rv = value->SetAsAString(valueString);
       break;
     }
-    case nsIAnnotationService::TYPE_BINARY: {
-      rv = NS_ERROR_INVALID_ARG;
-      break;
-    }
     default: {
       rv = NS_ERROR_UNEXPECTED;
       break;
@@ -750,10 +699,6 @@ nsAnnotationService::GetItemAnnotation(int64_t aItemId,
         rv = value->SetAsAString(valueString);
       break;
     }
-    case nsIAnnotationService::TYPE_BINARY: {
-      rv = NS_ERROR_INVALID_ARG;
-      break;
-    }
     default: {
       rv = NS_ERROR_UNEXPECTED;
       break;
@@ -933,66 +878,11 @@ nsAnnotationService::GetItemAnnotationDouble(int64_t aItemId,
 }
 
 
-NS_IMETHODIMP
-nsAnnotationService::GetPageAnnotationBinary(nsIURI* aURI,
-                                             const nsACString& aName,
-                                             uint8_t** _data,
-                                             uint32_t* _dataLen,
-                                             nsACString& _mimeType)
-{
-  NS_ENSURE_ARG(aURI);
-  NS_ENSURE_ARG_POINTER(_data);
-  NS_ENSURE_ARG_POINTER(_dataLen);
-
-  nsCOMPtr<mozIStorageStatement> statement;
-  nsresult rv = StartGetAnnotation(aURI, 0, aName, statement);
-  if (NS_FAILED(rv))
-    return rv;
-
-  mozStorageStatementScoper scoper(statement);
-  ENSURE_ANNO_TYPE(TYPE_BINARY, statement);
-  rv = statement->GetBlob(kAnnoIndex_Content, _dataLen, _data);
-  NS_ENSURE_SUCCESS(rv, rv);
-  rv = statement->GetUTF8String(kAnnoIndex_MimeType, _mimeType);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  return NS_OK;
-}
-
-
-NS_IMETHODIMP
-nsAnnotationService::GetItemAnnotationBinary(int64_t aItemId,
-                                             const nsACString& aName,
-                                             uint8_t** _data,
-                                             uint32_t* _dataLen,
-                                             nsACString& _mimeType)
-{
-  NS_ENSURE_ARG_MIN(aItemId, 1);
-  NS_ENSURE_ARG_POINTER(_data);
-  NS_ENSURE_ARG_POINTER(_dataLen);
-
-  nsCOMPtr<mozIStorageStatement> statement;
-  nsresult rv = StartGetAnnotation(nullptr, aItemId, aName, statement);
-  if (NS_FAILED(rv))
-    return rv;
-
-  mozStorageStatementScoper scoper(statement);
-  ENSURE_ANNO_TYPE(TYPE_BINARY, statement);
-  rv = statement->GetBlob(kAnnoIndex_Content, _dataLen, _data);
-  NS_ENSURE_SUCCESS(rv, rv);
-  rv = statement->GetUTF8String(kAnnoIndex_MimeType, _mimeType);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  return NS_OK;
-}
-
-
 NS_IMETHODIMP
 nsAnnotationService::GetPageAnnotationInfo(nsIURI* aURI,
                                            const nsACString& aName,
                                            int32_t* _flags,
                                            uint16_t* _expiration,
-                                           nsACString& _mimeType,
                                            uint16_t* _storageType)
 {
   NS_ENSURE_ARG(aURI);
@@ -1008,8 +898,6 @@ nsAnnotationService::GetPageAnnotationInfo(nsIURI* aURI,
   mozStorageStatementScoper scoper(statement);
   *_flags = statement->AsInt32(kAnnoIndex_Flags);
   *_expiration = (uint16_t)statement->AsInt32(kAnnoIndex_Expiration);
-  rv = statement->GetUTF8String(kAnnoIndex_MimeType, _mimeType);
-  NS_ENSURE_SUCCESS(rv, rv);
   int32_t type = (uint16_t)statement->AsInt32(kAnnoIndex_Type);
   if (type == 0) {
     // For annotations created before explicit typing,
@@ -1028,7 +916,6 @@ nsAnnotationService::GetItemAnnotationInfo(int64_t aItemId,
                                            const nsACString& aName,
                                            int32_t* _flags,
                                            uint16_t* _expiration,
-                                           nsACString& _mimeType,
                                            uint16_t* _storageType)
 {
   NS_ENSURE_ARG_MIN(aItemId, 1);
@@ -1044,8 +931,6 @@ nsAnnotationService::GetItemAnnotationInfo(int64_t aItemId,
   mozStorageStatementScoper scoper(statement);
   *_flags = statement->AsInt32(kAnnoIndex_Flags);
   *_expiration = (uint16_t)statement->AsInt32(kAnnoIndex_Expiration);
-  rv = statement->GetUTF8String(kAnnoIndex_MimeType, _mimeType);
-  NS_ENSURE_SUCCESS(rv, rv);
   int32_t type = (uint16_t)statement->AsInt32(kAnnoIndex_Type);
   if (type == 0) {
     // For annotations created before explicit typing,
@@ -1166,6 +1051,111 @@ nsAnnotationService::GetItemsWithAnnotation(const nsACString& aName,
 }
 
 
+NS_IMETHODIMP
+nsAnnotationService::GetAnnotationsWithName(const nsACString& aName,
+                                            uint32_t* _count,
+                                            mozIAnnotatedResult*** _annotations)
+{
+  NS_ENSURE_ARG(!aName.IsEmpty());
+  NS_ENSURE_ARG_POINTER(_annotations);
+
+  *_count = 0;
+  *_annotations = nullptr;
+  nsCOMArray<mozIAnnotatedResult> annotations;
+
+  nsCOMPtr<mozIStorageStatement> stmt = mDB->GetStatement(
+    "SELECT h.guid, h.url, -1, a.type, a.content "
+    "FROM moz_anno_attributes n "
+    "JOIN moz_annos a ON n.id = a.anno_attribute_id "
+    "JOIN moz_places h ON h.id = a.place_id "
+    "WHERE n.name = :anno_name "
+    "UNION ALL "
+    "SELECT b.guid, h.url, b.id, a.type, a.content "
+    "FROM moz_anno_attributes n "
+    "JOIN moz_items_annos a ON n.id = a.anno_attribute_id "
+    "JOIN moz_bookmarks b ON b.id = a.item_id "
+    "LEFT JOIN moz_places h ON h.id = b.fk "
+    "WHERE n.name = :anno_name "
+  );
+  NS_ENSURE_STATE(stmt);
+  mozStorageStatementScoper scoper(stmt);
+
+  nsresult rv = stmt->BindUTF8StringByName(NS_LITERAL_CSTRING("anno_name"),
+                                           aName);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  bool hasMore = false;
+  while (NS_SUCCEEDED(rv = stmt->ExecuteStep(&hasMore)) && hasMore) {
+    nsAutoCString guid;
+    rv = stmt->GetUTF8String(0, guid);
+    NS_ENSURE_SUCCESS(rv, rv);
+
+    nsCOMPtr<nsIURI> uri;
+    bool uriIsNull = false;
+    rv = stmt->GetIsNull(1, &uriIsNull);
+    NS_ENSURE_SUCCESS(rv, rv);
+    if (!uriIsNull) {
+      nsAutoCString url;
+      rv = stmt->GetUTF8String(1, url);
+      NS_ENSURE_SUCCESS(rv, rv);
+      rv = NS_NewURI(getter_AddRefs(uri), url);
+      NS_ENSURE_SUCCESS(rv, rv);
+    }
+
+    int64_t itemId = stmt->AsInt64(2);
+    int32_t type = stmt->AsInt32(3);
+
+    nsCOMPtr<nsIWritableVariant> variant = new nsVariant();
+    switch (type) {
+      case nsIAnnotationService::TYPE_INT32: {
+        rv = variant->SetAsInt32(stmt->AsInt32(4));
+        break;
+      }
+      case nsIAnnotationService::TYPE_INT64: {
+        rv = variant->SetAsInt64(stmt->AsInt64(4));
+        break;
+      }
+      case nsIAnnotationService::TYPE_DOUBLE: {
+        rv = variant->SetAsDouble(stmt->AsDouble(4));
+        break;
+      }
+      case nsIAnnotationService::TYPE_STRING: {
+        nsAutoString valueString;
+        rv = stmt->GetString(4, valueString);
+        NS_ENSURE_SUCCESS(rv, rv);
+
+        rv = variant->SetAsAString(valueString);
+        break;
+      }
+      default:
+        MOZ_ASSERT(false, "Unsupported annotation type");
+        // Move to the next result.
+        continue;
+    }
+    NS_ENSURE_SUCCESS(rv, rv);
+
+    nsCOMPtr<mozIAnnotatedResult> anno = new AnnotatedResult(guid, uri, itemId,
+                                                             aName, variant);
+    NS_ENSURE_TRUE(annotations.AppendObject(anno), NS_ERROR_OUT_OF_MEMORY);
+  }
+
+  // Convert to raw array.
+  if (annotations.Count() == 0)
+    return NS_OK;
+
+  *_annotations = static_cast<mozIAnnotatedResult**>
+    (nsMemory::Alloc(annotations.Count() * sizeof(mozIAnnotatedResult*)));
+  NS_ENSURE_TRUE(*_annotations, NS_ERROR_OUT_OF_MEMORY);
+
+  *_count = annotations.Count();
+  for (uint32_t i = 0; i < *_count; ++i) {
+    NS_ADDREF((*_annotations)[i] = annotations[i]);
+  }
+
+  return NS_OK;
+}
+
+
 nsresult
 nsAnnotationService::GetItemsWithAnnotationTArray(const nsACString& aName,
                                                   nsTArray<int64_t>* _results)
@@ -1498,9 +1488,6 @@ nsAnnotationService::CopyPageAnnotations(nsIURI* aSourceURI,
   NS_ENSURE_ARG(aSourceURI);
   NS_ENSURE_ARG(aDestURI);
 
-  if (InPrivateBrowsingMode())
-    return NS_OK;
-
   mozStorageTransaction transaction(mDB->MainConn(), false);
 
   nsCOMPtr<mozIStorageStatement> sourceStmt = mDB->GetStatement(
@@ -1523,10 +1510,10 @@ nsAnnotationService::CopyPageAnnotations(nsIURI* aSourceURI,
 
   nsCOMPtr<mozIStorageStatement> copyStmt = mDB->GetStatement(
     "INSERT INTO moz_annos "
-    "(place_id, anno_attribute_id, mime_type, content, flags, expiration, "
+    "(place_id, anno_attribute_id, content, flags, expiration, "
      "type, dateAdded, lastModified) "
     "SELECT (SELECT id FROM moz_places WHERE url = :page_url), "
-           "anno_attribute_id, mime_type, content, flags, expiration, type, "
+           "anno_attribute_id, content, flags, expiration, type, "
            ":date, :date "
     "FROM moz_annos "
     "WHERE place_id = :page_id "
@@ -1604,9 +1591,9 @@ nsAnnotationService::CopyItemAnnotations(int64_t aSourceItemId,
 
   nsCOMPtr<mozIStorageStatement> copyStmt = mDB->GetStatement(
       "INSERT OR REPLACE INTO moz_items_annos "
-      "(item_id, anno_attribute_id, mime_type, content, flags, expiration, "
+      "(item_id, anno_attribute_id, content, flags, expiration, "
        "type, dateAdded, lastModified) "
-      "SELECT :dest_item_id, anno_attribute_id, mime_type, content, flags, expiration, "
+      "SELECT :dest_item_id, anno_attribute_id, content, flags, expiration, "
              "type, :date, :date "
       "FROM moz_items_annos "
       "WHERE item_id = :source_item_id "
@@ -1677,33 +1664,6 @@ nsAnnotationService::RemoveObserver(nsIAnnotationObserver* aObserver)
   return NS_OK;
 }
 
-
-// nsAnnotationService::GetAnnotationURI
-//
-// XXX: does not support item-annotations
-
-NS_IMETHODIMP
-nsAnnotationService::GetAnnotationURI(nsIURI* aURI,
-                                      const nsACString& aName,
-                                      nsIURI** _result)
-{
-  if (aName.IsEmpty())
-    return NS_ERROR_INVALID_ARG;
-
-  nsAutoCString annoSpec;
-  nsresult rv = aURI->GetSpec(annoSpec);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  nsAutoCString spec;
-  spec.AssignLiteral("moz-anno:");
-  spec += aName;
-  spec += NS_LITERAL_CSTRING(":");
-  spec += annoSpec;
-
-  return NS_NewURI(_result, spec);
-}
-
-
 nsresult
 nsAnnotationService::HasAnnotationInternal(nsIURI* aURI,
                                            int64_t aItemId,
@@ -1783,7 +1743,7 @@ nsAnnotationService::StartGetAnnotation(nsIURI* aURI,
 
   if (isItemAnnotation) {
     aStatement = mDB->GetStatement(
-      "SELECT a.id, a.item_id, :anno_name, a.mime_type, a.content, a.flags, "
+      "SELECT a.id, a.item_id, :anno_name, a.content, a.flags, "
              "a.expiration, a.type "
       "FROM moz_anno_attributes n "
       "JOIN moz_items_annos a ON a.anno_attribute_id = n.id "
@@ -1793,7 +1753,7 @@ nsAnnotationService::StartGetAnnotation(nsIURI* aURI,
   }
   else {
     aStatement = mDB->GetStatement(
-      "SELECT a.id, a.place_id, :anno_name, a.mime_type, a.content, a.flags, "
+      "SELECT a.id, a.place_id, :anno_name, a.content, a.flags, "
              "a.expiration, a.type "
       "FROM moz_anno_attributes n "
       "JOIN moz_annos a ON n.id = a.anno_attribute_id "
@@ -1828,17 +1788,9 @@ nsAnnotationService::StartGetAnnotation(nsIURI* aURI,
 }
 
 
-bool
-nsAnnotationService::InPrivateBrowsingMode() const
-{
-  nsNavHistory* history = nsNavHistory::GetHistoryService();
-  return history && history->InPrivateBrowsingMode();
-}
-
-
 /**
  * This does most of the setup work needed to set an annotation, except for
- * binding the the actual value and MIME type and executing the statement.
+ * binding the the actual value and executing the statement.
  * It will either update an existing annotation or insert a new one.
  *
  * @note The aStatement RESULT IS NOT ADDREFED.  This is just one of the class
@@ -1931,18 +1883,18 @@ nsAnnotationService::StartSetAnnotation(nsIURI* aURI,
   if (isItemAnnotation) {
     aStatement = mDB->GetStatement(
       "INSERT OR REPLACE INTO moz_items_annos "
-        "(id, item_id, anno_attribute_id, mime_type, content, flags, "
+        "(id, item_id, anno_attribute_id, content, flags, "
          "expiration, type, dateAdded, lastModified) "
-      "VALUES (:id, :fk, :name_id, :mime_type, :content, :flags, "
+      "VALUES (:id, :fk, :name_id, :content, :flags, "
       ":expiration, :type, :date_added, :last_modified)"
     );
   }
   else {
     aStatement = mDB->GetStatement(
       "INSERT OR REPLACE INTO moz_annos "
-        "(id, place_id, anno_attribute_id, mime_type, content, flags, "
+        "(id, place_id, anno_attribute_id, content, flags, "
          "expiration, type, dateAdded, lastModified) "
-      "VALUES (:id, :fk, :name_id, :mime_type, :content, :flags, "
+      "VALUES (:id, :fk, :name_id, :content, :flags, "
       ":expiration, :type, :date_added, :last_modified)"
     );
   }
@@ -1967,7 +1919,7 @@ nsAnnotationService::StartSetAnnotation(nsIURI* aURI,
   NS_ENSURE_SUCCESS(rv, rv);
   rv = aStatement->BindInt64ByName(NS_LITERAL_CSTRING("name_id"), nameID);
   NS_ENSURE_SUCCESS(rv, rv);
-  // MimeType and Content will be bound by the caller.
+
   rv = aStatement->BindInt32ByName(NS_LITERAL_CSTRING("flags"), aFlags);
   NS_ENSURE_SUCCESS(rv, rv);
   rv = aStatement->BindInt32ByName(NS_LITERAL_CSTRING("expiration"), aExpiration);
@@ -1977,8 +1929,8 @@ nsAnnotationService::StartSetAnnotation(nsIURI* aURI,
   rv = aStatement->BindInt64ByName(NS_LITERAL_CSTRING("last_modified"), PR_Now());
   NS_ENSURE_SUCCESS(rv, rv);
 
-  // on success, leave the statement open, the caller will set the value
-  // and MIME type and execute the statement
+  // On success, leave the statement open, the caller will set the value
+  // and execute the statement.
   setAnnoScoper.Abandon();
 
   return NS_OK;
@@ -1990,7 +1942,7 @@ nsAnnotationService::StartSetAnnotation(nsIURI* aURI,
 NS_IMETHODIMP
 nsAnnotationService::Observe(nsISupports *aSubject,
                              const char *aTopic,
-                             const PRUnichar *aData)
+                             const char16_t *aData)
 {
   NS_ASSERTION(NS_IsMainThread(), "This can only be called on the main thread");
 
