@@ -10,17 +10,17 @@ const Cu = Components.utils;
 
 Cu.import("resource://gre/modules/Services.jsm");
 Cu.import("resource://gre/modules/TelemetryTimestamps.jsm");
+Cu.import("resource://gre/modules/TelemetryPing.jsm");
 
 const Telemetry = Services.telemetry;
 const bundle = Services.strings.createBundle(
   "chrome://global/locale/aboutTelemetry.properties");
 const brandBundle = Services.strings.createBundle(
   "chrome://branding/locale/brand.properties");
-const TelemetryPing = Cc["@mozilla.org/base/telemetry-ping;1"].
-  getService(Ci.nsITelemetryPing);
 
-// Maximum height of a histogram bar (in em)
+// Maximum height of a histogram bar (in em for html, in chars for text)
 const MAX_BAR_HEIGHT = 18;
+const MAX_BAR_CHARS = 25;
 const PREF_TELEMETRY_SERVER_OWNER = "toolkit.telemetry.server_owner";
 #ifdef MOZ_TELEMETRY_ON_BY_DEFAULT
 const PREF_TELEMETRY_ENABLED = "toolkit.telemetry.enabledPreRelease";
@@ -31,6 +31,15 @@ const PREF_DEBUG_SLOW_SQL = "toolkit.telemetry.debugSlowSql";
 const PREF_SYMBOL_SERVER_URI = "profiler.symbolicationUrl";
 const DEFAULT_SYMBOL_SERVER_URI = "http://symbolapi.mozilla.org";
 
+// ms idle before applying the filter (allow uninterrupted typing)
+const FILTER_IDLE_TIMEOUT = 500;
+
+#ifdef XP_WIN
+const EOL = "\r\n";
+#else
+const EOL = "\n";
+#endif
+
 // Cached value of document's RTL mode
 let documentRTLMode = "";
 
@@ -126,10 +135,11 @@ let SlowSQL = {
     let mainThreadCount = Object.keys(mainThread).length;
     let otherThreadCount = Object.keys(otherThreads).length;
     if (mainThreadCount == 0 && otherThreadCount == 0) {
-      showEmptySectionMessage("slow-sql-section");
       return;
     }
 
+    setHasData("slow-sql-section", true);
+
     if (debugSlowSql) {
       document.getElementById("sql-warning").classList.remove("hidden");
     }
@@ -275,10 +285,11 @@ let StackRenderer = {
     }
 
     if (aStacks.length == 0) {
-      showEmptySectionMessage(aPrefix + '-section');
       return;
     }
 
+    setHasData(aPrefix + '-section', true);
+
     this.renderMemoryMap(div, aMemoryMap);
 
     for (let i = 0; i < aStacks.length; ++i) {
@@ -367,7 +378,7 @@ function SymbolicationRequest_fetchSymbols() {
                  "version" : 3};
   let requestJSON = JSON.stringify(request);
 
-  this.symbolRequest = XMLHttpRequest();
+  this.symbolRequest = new XMLHttpRequest();
   this.symbolRequest.open("POST", symbolServerURI, true);
   this.symbolRequest.setRequestHeader("Content-type", "application/json");
   this.symbolRequest.setRequestHeader("Content-length",
@@ -399,6 +410,55 @@ let ChromeHangs = {
   }
 };
 
+let ThreadHangStats = {
+
+  /**
+   * Renders raw thread hang stats data
+   */
+  render: function() {
+    let div = document.getElementById("thread-hang-stats");
+    clearDivData(div);
+
+    let stats = Telemetry.threadHangStats;
+    stats.forEach((thread) => {
+      div.appendChild(this.renderThread(thread));
+    });
+    if (stats.length) {
+      setHasData("thread-hang-stats-section", true);
+    }
+  },
+
+  /**
+   * Creates and fills data corresponding to a thread
+   */
+  renderThread: function(aThread) {
+    let div = document.createElement("div");
+
+    let title = document.createElement("h2");
+    title.textContent = aThread.name;
+    div.appendChild(title);
+
+    // Don't localize the histogram name, because the
+    // name is also used as the div element's ID
+    Histogram.render(div, aThread.name + "-Activity",
+                     aThread.activity, {exponential: true});
+    aThread.hangs.forEach((hang, index) => {
+      let hangName = aThread.name + "-Hang-" + (index + 1);
+      let hangDiv = Histogram.render(
+        div, hangName, hang.histogram, {exponential: true});
+      let stackDiv = document.createElement("div");
+      hang.stack.forEach((frame) => {
+        stackDiv.appendChild(document.createTextNode(frame));
+        // Leave an extra <br> at the end of the stack listing
+        stackDiv.appendChild(document.createElement("br"));
+      });
+      // Insert stack after the histogram title
+      hangDiv.insertBefore(stackDiv, hangDiv.childNodes[1]);
+    });
+    return div;
+  },
+};
+
 let Histogram = {
 
   hgramSamplesCaption: bundle.GetStringFromName("histogramSamples"),
@@ -407,15 +467,20 @@ let Histogram = {
 
   hgramSumCaption: bundle.GetStringFromName("histogramSum"),
 
+  hgramCopyCaption: bundle.GetStringFromName("histogramCopy"),
+
   /**
    * Renders a single Telemetry histogram
    *
    * @param aParent Parent element
    * @param aName Histogram name
    * @param aHgram Histogram information
+   * @param aOptions Object with render options
+   *                 * exponential: bars follow logarithmic scale
    */
-  render: function Histogram_render(aParent, aName, aHgram) {
+  render: function Histogram_render(aParent, aName, aHgram, aOptions) {
     let hgram = this.unpack(aHgram);
+    let options = aOptions || {};
 
     let outerDiv = document.createElement("div");
     outerDiv.className = "histogram";
@@ -437,9 +502,22 @@ let Histogram = {
     if (isRTL())
       hgram.values.reverse();
 
-    this.renderValues(outerDiv, hgram.values, hgram.max);
+    let textData = this.renderValues(outerDiv, hgram.values, hgram.max,
+                                     hgram.sample_count, options);
+
+    // The 'Copy' button contains the textual data, copied to clipboard on click
+    let copyButton = document.createElement("button");
+    copyButton.className = "copy-node";
+    copyButton.appendChild(document.createTextNode(this.hgramCopyCaption));
+    copyButton.histogramText = aName + EOL + stats + EOL + EOL + textData;
+    copyButton.addEventListener("click", function(){
+      Cc["@mozilla.org/widget/clipboardhelper;1"].getService(Ci.nsIClipboardHelper)
+                                                 .copyString(this.histogramText);
+    });
+    outerDiv.appendChild(copyButton);
 
     aParent.appendChild(outerDiv);
+    return outerDiv;
   },
 
   /**
@@ -491,15 +569,44 @@ let Histogram = {
   },
 
   /**
-   * Create histogram bars
+   * Return a non-negative, logarithmic representation of a non-negative number.
+   * e.g. 0 => 0, 1 => 1, 10 => 2, 100 => 3
+   *
+   * @param aNumber Non-negative number
+   */
+  getLogValue: function(aNumber) {
+    return Math.max(0, Math.log10(aNumber) + 1);
+  },
+
+  /**
+   * Create histogram HTML bars, also returns a textual representation
+   * Both aMaxValue and aSumValues must be positive.
+   * Values are assumed to use 0 as baseline.
    *
    * @param aDiv Outer parent div
    * @param aValues Histogram values
-   * @param aMaxValue Largest histogram value in set
+   * @param aMaxValue Value of the longest bar (length, not label)
+   * @param aSumValues Sum of all bar values
+   * @param aOptions Object with render options (@see #render)
    */
-  renderValues: function Histogram_renderValues(aDiv, aValues, aMaxValue) {
+  renderValues: function Histogram_renderValues(aDiv, aValues, aMaxValue, aSumValues, aOptions) {
+    let text = "";
+    // If the last label is not the longest string, alignment will break a little
+    let labelPadTo = String(aValues[aValues.length -1][0]).length;
+    let maxBarValue = aOptions.exponential ? this.getLogValue(aMaxValue) : aMaxValue;
+
     for (let [label, value] of aValues) {
-      let belowEm = Math.round(MAX_BAR_HEIGHT * (value / aMaxValue) * 10) / 10;
+      let barValue = aOptions.exponential ? this.getLogValue(value) : value;
+
+      // Create a text representation: <right-aligned-label> |<bar-of-#><value>  <percentage>
+      text += EOL
+              + " ".repeat(Math.max(0, labelPadTo - String(label).length)) + label // Right-aligned label
+              + " |" + "#".repeat(Math.round(MAX_BAR_CHARS * barValue / maxBarValue)) // Bar
+              + "  " + value // Value
+              + "  " + Math.round(100 * value / aSumValues) + "%"; // Percentage
+
+      // Construct the HTML labels + bars
+      let belowEm = Math.round(MAX_BAR_HEIGHT * (barValue / maxBarValue) * 10) / 10;
       let aboveEm = MAX_BAR_HEIGHT - belowEm;
 
       let barDiv = document.createElement("div");
@@ -520,6 +627,74 @@ let Histogram = {
 
       aDiv.appendChild(barDiv);
     }
+
+    return text.substr(EOL.length); // Trim the EOL before the first line
+  },
+
+  /**
+   * Helper function for filtering histogram elements by their id
+   * Adds the "filter-blocked" class to histogram nodes whose IDs don't match the filter.
+   *
+   * @param aContainerNode Container node containing the histogram class nodes to filter
+   * @param aFilterText either text or /RegEx/. If text, case-insensitive and AND words
+   */
+  filterHistograms: function _filterHistograms(aContainerNode, aFilterText) {
+    let filter = aFilterText.toString();
+
+    // Pass if: all non-empty array items match (case-sensitive)
+    function isPassText(subject, filter) {
+      for (let item of filter) {
+        if (item.length && subject.indexOf(item) < 0) {
+          return false; // mismatch and not a spurious space
+        }
+      }
+      return true;
+    }
+
+    function isPassRegex(subject, filter) {
+      return filter.test(subject);
+    }
+
+    // Setup normalized filter string (trimmed, lower cased and split on spaces if not RegEx)
+    let isPassFunc; // filter function, set once, then applied to all elements
+    filter = filter.trim();
+    if (filter[0] != "/") { // Plain text: case insensitive, AND if multi-string
+      isPassFunc = isPassText;
+      filter = filter.toLowerCase().split(" ");
+    } else {
+      isPassFunc = isPassRegex;
+      var r = filter.match(/^\/(.*)\/(i?)$/);
+      try {
+        filter = RegExp(r[1], r[2]);
+      }
+      catch (e) { // Incomplete or bad RegExp - always no match
+        isPassFunc = function() {
+          return false;
+        };
+      }
+    }
+
+    let needLower = (isPassFunc === isPassText);
+
+    let histograms = aContainerNode.getElementsByClassName("histogram");
+    for (let hist of histograms) {
+      hist.classList[isPassFunc((needLower ? hist.id.toLowerCase() : hist.id), filter) ? "remove" : "add"]("filter-blocked");
+    }
+  },
+
+  /**
+   * Event handler for change at histograms filter input
+   *
+   * When invoked, 'this' is expected to be the filter HTML node.
+   */
+  histogramFilterChanged: function _histogramFilterChanged() {
+    if (this.idleTimeout) {
+      clearTimeout(this.idleTimeout);
+    }
+
+    this.idleTimeout = setTimeout( () => {
+      Histogram.filterHistograms(document.getElementById(this.getAttribute("target_id")), this.value);
+    }, FILTER_IDLE_TIMEOUT);
   }
 };
 
@@ -597,7 +772,8 @@ let KeyValueTable = {
    */
   renderBody: function KeyValueTable_renderBody(aTable, aMeasurements) {
     for (let [key, value] of Iterator(aMeasurements)) {
-      if (typeof value == "object") {
+      // use .valueOf() to unbox Number, String, etc. objects
+      if ((typeof value == "object") && (typeof value.valueOf() == "object")) {
         value = RenderObject(value);
       }
 
@@ -638,34 +814,14 @@ let AddonDetails = {
 };
 
 /**
- * Helper function for showing "No data collected" message for a section
+ * Helper function for showing either the toggle element or "No data collected" message for a section
  *
  * @param aSectionID ID of the section element that needs to be changed
+ * @param aHasData true (default) indicates that toggle should be displayed
  */
-function showEmptySectionMessage(aSectionID) {
+function setHasData(aSectionID, aHasData) {
   let sectionElement = document.getElementById(aSectionID);
-
-  // Hide toggle captions
-  let toggleElements = sectionElement.getElementsByClassName("toggle-caption");
-  toggleElements[0].classList.add("hidden");
-  toggleElements[1].classList.add("hidden");
-
-  // Show "No data collected" message
-  let messageElement = sectionElement.getElementsByClassName("empty-caption")[0];
-  messageElement.classList.remove("hidden");
-
-  // Don't allow section to be expanded by clicking on the header text
-  let sectionHeaders = sectionElement.getElementsByClassName("section-name");
-  for (let sectionHeader of sectionHeaders) {
-    sectionHeader.removeEventListener("click", toggleSection);
-    sectionHeader.style.cursor = "auto";
-  }
-
-  // Don't allow section to be expanded by clicking on the toggle text
-  let toggleLinks = sectionElement.getElementsByClassName("toggle-caption");
-  for (let toggleLink of toggleLinks) {
-    toggleLink.removeEventListener("click", toggleSection);
-  }
+  sectionElement.classList[aHasData ? "add" : "remove"]("has-data");
 }
 
 /**
@@ -674,12 +830,15 @@ function showEmptySectionMessage(aSectionID) {
  */
 function toggleSection(aEvent) {
   let parentElement = aEvent.target.parentElement;
-  let sectionDiv = parentElement.getElementsByTagName("div")[0];
-  sectionDiv.classList.toggle("hidden");
+  if (!parentElement.classList.contains("has-data")) {
+    return; // nothing to toggle
+  }
 
-  let toggleLinks = parentElement.getElementsByClassName("toggle-caption");
-  toggleLinks[0].classList.toggle("hidden");
-  toggleLinks[1].classList.toggle("hidden");
+  parentElement.classList.toggle("expanded");
+
+  // Store section opened/closed state in a hidden checkbox (which is then used on reload)
+  let statebox = parentElement.getElementsByClassName("statebox")[0];
+  statebox.checked = parentElement.classList.contains("expanded");
 }
 
 /**
@@ -753,13 +912,14 @@ function setupListeners() {
     sectionHeader.addEventListener("click", toggleSection, false);
   }
 
-  // Clicking on the "collapse"/"expand" text will also toggle section's state
+  // Clicking on the "toggle" text will also toggle section's state
   let toggleLinks = document.getElementsByClassName("toggle-caption");
   for (let toggleLink of toggleLinks) {
     toggleLink.addEventListener("click", toggleSection, false);
   }
 }
 
+
 function onLoad() {
   window.removeEventListener("load", onLoad);
 
@@ -775,6 +935,9 @@ function onLoad() {
   // Show chrome hang stacks
   ChromeHangs.render();
 
+  // Show thread hang stats
+  ThreadHangStats.render();
+
   // Show histogram data
   let histograms = Telemetry.histogramSnapshots;
   if (Object.keys(histograms).length) {
@@ -782,8 +945,14 @@ function onLoad() {
     for (let [name, hgram] of Iterator(histograms)) {
       Histogram.render(hgramDiv, name, hgram);
     }
-  } else {
-    showEmptySectionMessage("histograms-section");
+
+    let filterBox = document.getElementById("histograms-filter");
+    filterBox.addEventListener("input", Histogram.histogramFilterChanged, false);
+    if (filterBox.value.trim() != "") { // on load, no need to filter if empty
+      Histogram.filterHistograms(hgramDiv, filterBox.value);
+    }
+
+    setHasData("histograms-section", true);
   }
 
   // Show addon histogram data
@@ -797,13 +966,21 @@ function onLoad() {
     }
   }
 
-  if (!addonHistogramsRendered) {
-    showEmptySectionMessage("addon-histograms-section");
+  if (addonHistogramsRendered) {
+   setHasData("addon-histograms-section", true);
   }
 
   // Get the Telemetry Ping payload
   Telemetry.asyncFetchTelemetryData(displayPingData);
-};
+
+  // Restore sections states
+  let stateboxes = document.getElementsByClassName("statebox");
+  for (let box of stateboxes) {
+    if (box.checked) { // Was open. Will still display as empty if not has-data
+        box.parentElement.classList.add("expanded");
+    }
+  }
+}
 
 let LateWritesSingleton = {
   renderHeader: function LateWritesSingleton_renderHeader(aIndex) {
@@ -875,8 +1052,7 @@ function displayPingData() {
     let simpleSection = document.getElementById("simple-measurements");
     simpleSection.appendChild(KeyValueTable.render(simpleMeasurements,
                                                    keysHeader, valuesHeader));
-  } else {
-    showEmptySectionMessage("simple-measurements-section");
+    setHasData("simple-measurements-section", true);
   }
 
   LateWritesSingleton.renderLateWrites(ping.lateWrites);
@@ -886,15 +1062,13 @@ function displayPingData() {
     let infoSection = document.getElementById("system-info");
     infoSection.appendChild(KeyValueTable.render(ping.info,
                                                  keysHeader, valuesHeader));
-  } else {
-    showEmptySectionMessage("system-info-section");
+    setHasData("system-info-section", true);
   }
 
   let addonDetails = ping.addonDetails;
   if (Object.keys(addonDetails).length) {
     AddonDetails.render(addonDetails);
-  } else {
-    showEmptySectionMessage("addon-details-section");
+    setHasData("addon-details-section", true);
   }
 }
 
