@@ -4,13 +4,14 @@
 
 "use strict";
 
-let { components, Cc, Ci, Cu } = require('chrome');
+let { components, Cc, Ci, Cu } = require("chrome");
+let Services = require("Services");
 
 Cu.import("resource://gre/modules/XPCOMUtils.jsm");
-Cu.import("resource://gre/modules/Services.jsm");
 Cu.import("resource://gre/modules/NetUtil.jsm");
 Cu.import("resource://gre/modules/FileUtils.jsm");
 Cu.import("resource://gre/modules/devtools/SourceMap.jsm");
+Cu.import("resource://gre/modules/Task.jsm");
 
 const promise = require("sdk/core/promise");
 const events = require("sdk/event/core");
@@ -22,6 +23,7 @@ loader.lazyGetter(this, "CssLogic", () => require("devtools/styleinspector/css-l
 
 let TRANSITION_CLASS = "moz-styleeditor-transitioning";
 let TRANSITION_DURATION_MS = 500;
+let TRANSITION_BUFFER_MS = 1000;
 let TRANSITION_RULE = "\
 :root.moz-styleeditor-transitioning, :root.moz-styleeditor-transitioning * {\
 transition-duration: " + TRANSITION_DURATION_MS + "ms !important; \
@@ -94,18 +96,7 @@ let StyleSheetsActor = protocol.ActorClass({
     let window = this.window;
     var domReady = () => {
       window.removeEventListener("DOMContentLoaded", domReady, true);
-
-      let documents = [this.document];
-      let actors = [];
-      for (let doc of documents) {
-        let sheets = this._addStyleSheets(doc.styleSheets);
-        actors = actors.concat(sheets);
-        // Recursively handle style sheets of the documents in iframes.
-        for (let iframe of doc.getElementsByTagName("iframe")) {
-          documents.push(iframe.contentDocument);
-        }
-      }
-      deferred.resolve(actors);
+      this._addAllStyleSheets().then(deferred.resolve, Cu.reportError);
     };
 
     if (window.document.readyState === "loading") {
@@ -120,29 +111,55 @@ let StyleSheetsActor = protocol.ActorClass({
     response: { styleSheets: RetVal("array:stylesheet") }
   }),
 
+  /**
+   * Add all the stylesheets in this document and its subframes.
+   * Assumes the document is loaded.
+   *
+   * @return {Promise}
+   *         Promise that resolves with an array of StyleSheetActors
+   */
+  _addAllStyleSheets: function() {
+    return Task.spawn(function() {
+      let documents = [this.document];
+      let actors = [];
+
+      for (let doc of documents) {
+        let sheets = yield this._addStyleSheets(doc.styleSheets);
+        actors = actors.concat(sheets);
+
+        // Recursively handle style sheets of the documents in iframes.
+        for (let iframe of doc.getElementsByTagName("iframe")) {
+          documents.push(iframe.contentDocument);
+        }
+      }
+      throw new Task.Result(actors);
+    }.bind(this));
+  },
+
   /**
    * Add all the stylesheets to the map and create an actor for each one
-   * if not already created. Send event that there are new stylesheets.
+   * if not already created.
    *
    * @param {[DOMStyleSheet]} styleSheets
    *        Stylesheets to add
-   * @return {[object]}
-   *         Array of actors for each StyleSheetActor created
+   *
+   * @return {Promise}
+   *         Promise that resolves to an array of StyleSheetActors
    */
   _addStyleSheets: function(styleSheets)
   {
-    let sheets = [];
-    for (let i = 0; i < styleSheets.length; i++) {
-      let styleSheet = styleSheets[i];
-      sheets.push(styleSheet);
+    return Task.spawn(function() {
+      let actors = [];
+      for (let i = 0; i < styleSheets.length; i++) {
+        let actor = this._createStyleSheetActor(styleSheets[i]);
+        actors.push(actor);
 
-      // Get all sheets, including imported ones
-      let imports = this._getImported(styleSheet);
-      sheets = sheets.concat(imports);
-    }
-    let actors = sheets.map(this._createStyleSheetActor.bind(this));
-
-    return actors;
+        // Get all sheets, including imported ones
+        let imports = yield this._getImported(actor);
+        actors = actors.concat(imports);
+      }
+      throw new Task.Result(actors);
+    }.bind(this));
   },
 
   /**
@@ -150,31 +167,37 @@ let StyleSheetsActor = protocol.ActorClass({
    *
    * @param  {DOMStyleSheet} styleSheet
    *         Style sheet to search
-   * @return {array}
-   *         All the imported stylesheets
+   * @return {Promise}
+   *         A promise that resolves with an array of StyleSheetActors
    */
   _getImported: function(styleSheet) {
-   let imported = [];
+    return Task.spawn(function() {
+      let rules = yield styleSheet.getCSSRules();
+      let imported = [];
 
-   for (let i = 0; i < styleSheet.cssRules.length; i++) {
-      let rule = styleSheet.cssRules[i];
-      if (rule.type == Ci.nsIDOMCSSRule.IMPORT_RULE) {
-        // Associated styleSheet may be null if it has already been seen due to
-        // duplicate @imports for the same URL.
-        if (!rule.styleSheet) {
-          continue;
+      for (let i = 0; i < rules.length; i++) {
+        let rule = rules[i];
+        if (rule.type == Ci.nsIDOMCSSRule.IMPORT_RULE) {
+          // Associated styleSheet may be null if it has already been seen due
+          // to duplicate @imports for the same URL.
+          if (!rule.styleSheet) {
+            continue;
+          }
+          let actor = this._createStyleSheetActor(rule.styleSheet);
+          imported.push(actor);
+
+          // recurse imports in this stylesheet as well
+          let children = yield this._getImported(actor);
+          imported = imported.concat(children);
         }
-        imported.push(rule.styleSheet);
+        else if (rule.type != Ci.nsIDOMCSSRule.CHARSET_RULE) {
+          // @import rules must precede all others except @charset
+          break;
+        }
+      }
 
-        // recurse imports in this stylesheet as well
-        imported = imported.concat(this._getImported(rule.styleSheet));
-      }
-      else if (rule.type != Ci.nsIDOMCSSRule.CHARSET_RULE) {
-        // @import rules must precede all others except @charset
-        break;
-      }
-    }
-    return imported;
+      throw new Task.Result(imported);
+    }.bind(this));
   },
 
   /**
@@ -319,17 +342,50 @@ let StyleSheetActor = protocol.ActorClass({
     this._styleSheetIndex = -1;
 
     this._transitionRefCount = 0;
+  },
 
-    // if this sheet has an @import, then it's rules are loaded async
-    let ownerNode = this.rawSheet.ownerNode;
-    if (ownerNode) {
-      let onSheetLoaded = function(event) {
-        ownerNode.removeEventListener("load", onSheetLoaded, false);
-        this._notifyPropertyChanged("ruleCount");
-      }.bind(this);
-
-      ownerNode.addEventListener("load", onSheetLoaded, false);
+  /**
+   * Get the raw stylesheet's cssRules once the sheet has been loaded.
+   *
+   * @return {Promise}
+   *         Promise that resolves with a CSSRuleList
+   */
+  getCSSRules: function() {
+    let rules;
+    try {
+      rules = this.rawSheet.cssRules;
     }
+    catch (e) {
+      // sheet isn't loaded yet
+    }
+
+    if (rules) {
+      return promise.resolve(rules);
+    }
+
+    let ownerNode = this.rawSheet.ownerNode;
+    if (!ownerNode) {
+      return promise.resolve([]);
+    }
+
+    if (this._cssRules) {
+      return this._cssRules;
+    }
+
+    let deferred = promise.defer();
+
+    let onSheetLoaded = function(event) {
+      ownerNode.removeEventListener("load", onSheetLoaded, false);
+
+      deferred.resolve(this.rawSheet.cssRules);
+    }.bind(this);
+
+    ownerNode.addEventListener("load", onSheetLoaded, false);
+
+    // cache so we don't add many listeners if this is called multiple times.
+    this._cssRules = deferred.promise;
+
+    return this._cssRules;
   },
 
   /**
@@ -369,6 +425,9 @@ let StyleSheetActor = protocol.ActorClass({
     }
     catch(e) {
       // stylesheet had an @import rule that wasn't loaded yet
+      this.getCSSRules().then(() => {
+        this._notifyPropertyChanged("ruleCount");
+      });
     }
     return form;
   },
@@ -693,10 +752,10 @@ let StyleSheetActor = protocol.ActorClass({
 
     this._transitionRefCount++;
 
-    // Set up clean up and commit after transition duration (+10% buffer)
+    // Set up clean up and commit after transition duration (+buffer)
     // @see _onTransitionEnd
     this.window.setTimeout(this._onTransitionEnd.bind(this),
-                           Math.floor(TRANSITION_DURATION_MS * 1.1));
+                           TRANSITION_DURATION_MS + TRANSITION_BUFFER_MS);
   },
 
   /**
