@@ -16,13 +16,22 @@ let devtools = Cu.import("resource://gre/modules/devtools/Loader.jsm", {}).devto
 
 XPCOMUtils.defineLazyModuleGetter(this, "Services",
                                   "resource://gre/modules/Services.jsm");
-
-XPCOMUtils.defineLazyModuleGetter(this, "ConsoleAPIStorage",
-                                  "resource://gre/modules/ConsoleAPIStorage.jsm");
+XPCOMUtils.defineLazyGetter(this, "NetworkMonitor", () => {
+  return devtools.require("devtools/toolkit/webconsole/network-monitor")
+         .NetworkMonitor;
+});
+XPCOMUtils.defineLazyGetter(this, "NetworkMonitorChild", () => {
+  return devtools.require("devtools/toolkit/webconsole/network-monitor")
+         .NetworkMonitorChild;
+});
+XPCOMUtils.defineLazyGetter(this, "ConsoleProgressListener", () => {
+  return devtools.require("devtools/toolkit/webconsole/network-monitor")
+         .ConsoleProgressListener;
+});
 
 for (let name of ["WebConsoleUtils", "ConsoleServiceListener",
-                  "ConsoleAPIListener", "ConsoleProgressListener",
-                  "JSTermHelpers", "JSPropertyProvider", "NetworkMonitor"]) {
+                  "ConsoleAPIListener", "JSTermHelpers", "JSPropertyProvider",
+                  "ConsoleReflowListener"]) {
   Object.defineProperty(this, name, {
     get: function(prop) {
       if (prop == "WebConsoleUtils") {
@@ -58,20 +67,23 @@ function WebConsoleActor(aConnection, aParentActor)
 
   this.dbg = new Debugger();
 
-  this._protoChains = new Map();
-  this._dbgGlobals = new Map();
   this._netEvents = new Map();
-  this._getDebuggerGlobal(this.window);
+  this._gripDepth = 0;
 
+  this._onWillNavigate = this._onWillNavigate.bind(this);
   this._onObserverNotification = this._onObserverNotification.bind(this);
-  Services.obs.addObserver(this._onObserverNotification,
-                           "inner-window-destroyed", false);
   if (this.parentActor.isRootActor) {
     Services.obs.addObserver(this._onObserverNotification,
                              "last-pb-context-exited", false);
   }
+
+  this.traits = {
+    customNetworkRequest: !this._parentIsContentActor,
+  };
 }
 
+WebConsoleActor.l10n = new WebConsoleUtils.l10n("chrome://global/locale/console.properties");
+
 WebConsoleActor.prototype =
 {
   /**
@@ -81,6 +93,13 @@ WebConsoleActor.prototype =
    */
   dbg: null,
 
+  /**
+   * This is used by the ObjectActor to keep track of the depth of grip() calls.
+   * @private
+   * @type number
+   */
+  _gripDepth: null,
+
   /**
    * Actor pool for all of the actors we send to the client.
    * @private
@@ -96,14 +115,6 @@ WebConsoleActor.prototype =
    */
   _prefs: null,
 
-  /**
-   * Holds a map between inner window IDs and Debugger.Objects for the window
-   * objects.
-   * @private
-   * @type Map
-   */
-  _dbgGlobals: null,
-
   /**
    * Holds a map between nsIChannel objects and NetworkEventActors for requests
    * created with sendHTTPRequest.
@@ -113,16 +124,6 @@ WebConsoleActor.prototype =
    */
   _netEvents: null,
 
-  /**
-   * A cache of prototype chains for objects that have received a
-   * prototypeAndProperties request.
-   *
-   * @private
-   * @type Map
-   * @see dbg-script-actors.js, ThreadActor._protoChains
-   */
-  _protoChains: null,
-
   /**
    * The debugger server connection instance.
    * @type object
@@ -130,10 +131,127 @@ WebConsoleActor.prototype =
   conn: null,
 
   /**
-   * The content window we work with.
+   * List of supported features by the console actor.
+   * @type object
+   */
+  traits: null,
+
+  /**
+   * Boolean getter that tells if the parent actor is a ContentActor.
+   *
+   * @private
+   * @type boolean
+   */
+  get _parentIsContentActor() {
+    return "ContentActor" in DebuggerServer &&
+            this.parentActor instanceof DebuggerServer.ContentActor;
+  },
+
+  /**
+   * The window we work with.
    * @type nsIDOMWindow
    */
-  get window() this.parentActor.window,
+  get window() {
+    if (this.parentActor.isRootActor) {
+      return this._getWindowForBrowserConsole();
+    }
+    return this.parentActor.window;
+  },
+
+  /**
+   * Get a window to use for the browser console.
+   *
+   * @private
+   * @return nsIDOMWindow
+   *         The window to use, or null if no window could be found.
+   */
+  _getWindowForBrowserConsole: function WCA__getWindowForBrowserConsole()
+  {
+    // Check if our last used chrome window is still live.
+    let window = this._lastChromeWindow && this._lastChromeWindow.get();
+    // If not, look for a new one.
+    if (!window || window.closed) {
+      window = this.parentActor.window;
+      if (!window) {
+        // Try to find the Browser Console window to use instead.
+        window = Services.wm.getMostRecentWindow("devtools:webconsole");
+        // We prefer the normal chrome window over the console window,
+        // so we'll look for those windows in order to replace our reference.
+        let onChromeWindowOpened = () => {
+          // We'll look for this window when someone next requests window()
+          Services.obs.removeObserver(onChromeWindowOpened, "domwindowopened");
+          this._lastChromeWindow = null;
+        };
+        Services.obs.addObserver(onChromeWindowOpened, "domwindowopened", false);
+      }
+
+      this._handleNewWindow(window);
+    }
+
+    return window;
+  },
+
+  /**
+   * Store a newly found window on the actor to be used in the future.
+   *
+   * @private
+   * @param nsIDOMWindow window
+   *        The window to store on the actor (can be null).
+   */
+  _handleNewWindow: function WCA__handleNewWindow(window)
+  {
+    if (window) {
+      if (this._hadChromeWindow) {
+        let contextChangedMsg = WebConsoleActor.l10n.getStr("evaluationContextChanged");
+        Services.console.logStringMessage(contextChangedMsg);
+      }
+      this._lastChromeWindow = Cu.getWeakReference(window);
+      this._hadChromeWindow = true;
+    } else {
+      this._lastChromeWindow = null;
+    }
+  },
+
+  /**
+   * Whether we've been using a window before.
+   *
+   * @private
+   * @type boolean
+   */
+  _hadChromeWindow: false,
+
+  /**
+   * A weak reference to the last chrome window we used to work with.
+   *
+   * @private
+   * @type nsIWeakReference
+   */
+  _lastChromeWindow: null,
+
+  // The evalWindow is used at the scope for JS evaluation.
+  _evalWindow: null,
+  get evalWindow() {
+    return this._evalWindow || this.window;
+  },
+
+  set evalWindow(aWindow) {
+    this._evalWindow = aWindow;
+
+    if (!this._progressListenerActive && this.parentActor._progressListener) {
+      this.parentActor._progressListener.once("will-navigate", this._onWillNavigate);
+      this._progressListenerActive = true;
+    }
+  },
+
+  /**
+   * Flag used to track if we are listening for events from the progress
+   * listener of the tab actor. We use the progress listener to clear
+   * this.evalWindow on page navigation.
+   *
+   * @private
+   * @type boolean
+   */
+  _progressListenerActive: false,
 
   /**
    * The ConsoleServiceListener instance.
@@ -157,11 +275,16 @@ WebConsoleActor.prototype =
   consoleProgressListener: null,
 
   /**
-   * Getter for the NetworkMonitor.saveRequestAndResponseBodies preference.
-   * @type boolean
+   * The ConsoleReflowListener instance.
    */
-  get saveRequestAndResponseBodies()
-    this._prefs["NetworkMonitor.saveRequestAndResponseBodies"],
+  consoleReflowListener: null,
+
+  /**
+   * The JSTerm Helpers names cache.
+   * @private
+   * @type array
+   */
+  _jstermHelpersCache: null,
 
   actorPrefix: "console",
 
@@ -170,7 +293,17 @@ WebConsoleActor.prototype =
     return { actor: this.actorID };
   },
 
-  hasNativeConsoleAPI: BrowserTabActor.prototype.hasNativeConsoleAPI,
+  hasNativeConsoleAPI: function WCA_hasNativeConsoleAPI(aWindow) {
+    let isNative = false;
+    try {
+      // We are very explicitly examining the "console" property of
+      // the non-Xrayed object here.
+      let console = aWindow.wrappedJSObject.console;
+      isNative = console instanceof aWindow.Console;
+    }
+    catch (ex) { }
+    return isNative;
+  },
 
   _createValueGrip: ThreadActor.prototype.createValueGrip,
   _stringIsLong: ThreadActor.prototype._stringIsLong,
@@ -198,23 +331,52 @@ WebConsoleActor.prototype =
       this.consoleProgressListener.destroy();
       this.consoleProgressListener = null;
     }
+    if (this.consoleReflowListener) {
+      this.consoleReflowListener.destroy();
+      this.consoleReflowListener = null;
+    }
     this.conn.removeActorPool(this._actorPool);
-    Services.obs.removeObserver(this._onObserverNotification,
-                                "inner-window-destroyed");
     if (this.parentActor.isRootActor) {
       Services.obs.removeObserver(this._onObserverNotification,
                                   "last-pb-context-exited");
     }
     this._actorPool = null;
 
+    this._jstermHelpersCache = null;
+    this._evalWindow = null;
     this._netEvents.clear();
-    this._protoChains.clear();
-    this._dbgGlobals.clear();
     this.dbg.enabled = false;
     this.dbg = null;
     this.conn = null;
   },
 
+  /**
+   * Create and return an environment actor that corresponds to the provided
+   * Debugger.Environment. This is a straightforward clone of the ThreadActor's
+   * method except that it stores the environment actor in the web console
+   * actor's pool.
+   *
+   * @param Debugger.Environment aEnvironment
+   *        The lexical environment we want to extract.
+   * @return The EnvironmentActor for aEnvironment or undefined for host
+   *         functions or functions scoped to a non-debuggee global.
+   */
+  createEnvironmentActor: function WCA_createEnvironmentActor(aEnvironment) {
+    if (!aEnvironment) {
+      return undefined;
+    }
+
+    if (aEnvironment.actor) {
+      return aEnvironment.actor;
+    }
+
+    let actor = new EnvironmentActor(aEnvironment, this);
+    this._actorPool.addActor(actor);
+    aEnvironment.actor = actor;
+
+    return actor;
+  },
+
   /**
    * Create a grip for the given value.
    *
@@ -248,16 +410,7 @@ WebConsoleActor.prototype =
         // The above can throw an exception if aValue is not an actual object.
       }
     }
-    let dbgGlobal = null;
-    try {
-      dbgGlobal = this._getDebuggerGlobal(global);
-    }
-    catch (ex) {
-      // The above call can throw in addDebuggee() if the given global object
-      // is already in the stackframe of code that is executing now. Console.jsm
-      // and the Browser Console can cause this case.
-      dbgGlobal = this._getDebuggerGlobal(this.window);
-    }
+    let dbgGlobal = this.dbg.makeGlobalObjectReference(global);
     return dbgGlobal.makeDebuggeeValue(aValue);
   },
 
@@ -351,6 +504,14 @@ WebConsoleActor.prototype =
   {
     let startedListeners = [];
     let window = !this.parentActor.isRootActor ? this.window : null;
+    let appId = null;
+    let messageManager = null;
+
+    if (this._parentIsContentActor) {
+      // Filter network requests by appId on Firefox OS devices.
+      appId = this.parentActor.docShell.appId;
+      messageManager = this.parentActor._chromeGlobal;
+    }
 
     while (aRequest.listeners.length > 0) {
       let listener = aRequest.listeners.shift();
@@ -373,8 +534,13 @@ WebConsoleActor.prototype =
           break;
         case "NetworkActivity":
           if (!this.networkMonitor) {
-            this.networkMonitor =
-              new NetworkMonitor(window, this);
+            if (appId && messageManager) {
+              this.networkMonitor =
+                new NetworkMonitorChild(appId, messageManager, this);
+            }
+            else {
+              this.networkMonitor = new NetworkMonitor({ window: window }, this);
+            }
             this.networkMonitor.init();
           }
           startedListeners.push(listener);
@@ -388,11 +554,19 @@ WebConsoleActor.prototype =
                                                     MONITOR_FILE_ACTIVITY);
           startedListeners.push(listener);
           break;
+        case "ReflowActivity":
+          if (!this.consoleReflowListener) {
+            this.consoleReflowListener =
+              new ConsoleReflowListener(this.window, this);
+          }
+          startedListeners.push(listener);
+          break;
       }
     }
     return {
       startedListeners: startedListeners,
       nativeConsoleAPI: this.hasNativeConsoleAPI(this.window),
+      traits: this.traits,
     };
   },
 
@@ -447,6 +621,13 @@ WebConsoleActor.prototype =
           }
           stoppedListeners.push(listener);
           break;
+        case "ReflowActivity":
+          if (this.consoleReflowListener) {
+            this.consoleReflowListener.destroy();
+            this.consoleReflowListener = null;
+          }
+          stoppedListeners.push(listener);
+          break;
       }
     }
 
@@ -548,7 +729,7 @@ WebConsoleActor.prototype =
     let evalResult = evalInfo.result;
     let helperResult = evalInfo.helperResult;
 
-    let result, error, errorMessage;
+    let result, errorMessage, errorGrip = null;
     if (evalResult) {
       if ("return" in evalResult) {
         result = evalResult.return;
@@ -557,7 +738,8 @@ WebConsoleActor.prototype =
         result = evalResult.yield;
       }
       else if ("throw" in evalResult) {
-        error = evalResult.throw;
+        let error = evalResult.throw;
+        errorGrip = this.createValueGrip(error);
         let errorToString = evalInfo.window
                             .evalInGlobalWithBindings("ex + ''", {ex: error});
         if (errorToString && typeof errorToString.return == "string") {
@@ -571,7 +753,7 @@ WebConsoleActor.prototype =
       input: input,
       result: this.createValueGrip(result),
       timestamp: timestamp,
-      exception: error ? this.createValueGrip(error) : null,
+      exception: errorGrip,
       exceptionMessage: errorMessage,
       helperResult: helperResult,
     };
@@ -587,12 +769,29 @@ WebConsoleActor.prototype =
    */
   onAutocomplete: function WCA_onAutocomplete(aRequest)
   {
-    // TODO: Bug 842682 - use the debugger API for autocomplete in the Web
-    // Console, and provide suggestions from the selected debugger stack frame.
-    // Also, properly reuse _getJSTermHelpers instead of re-implementing it
-    // here.
-    let result = JSPropertyProvider(this.window, aRequest.text,
-                                    aRequest.cursor) || {};
+    let frameActorId = aRequest.frameActor;
+    let dbgObject = null;
+    let environment = null;
+
+    // This is the case of the paused debugger
+    if (frameActorId) {
+      let frameActor = this.conn.getActor(frameActorId);
+      if (frameActor) {
+        let frame = frameActor.frame;
+        environment = frame.environment;
+      }
+      else {
+        Cu.reportError("Web Console Actor: the frame actor was not found: " +
+                       frameActorId);
+      }
+    }
+    // This is the general case (non-paused debugger)
+    else {
+      dbgObject = this.dbg.makeGlobalObjectReference(this.evalWindow);
+    }
+
+    let result = JSPropertyProvider(dbgObject, environment, aRequest.text,
+                                    aRequest.cursor, frameActorId) || {};
     let matches = result.matches || [];
     let reqText = aRequest.text.substr(0, aRequest.cursor);
 
@@ -600,13 +799,14 @@ WebConsoleActor.prototype =
     // helper functions.
     let lastNonAlphaIsDot = /[.][a-zA-Z0-9$]*$/.test(reqText);
     if (!lastNonAlphaIsDot) {
-      let helpers = {
-        sandbox: Object.create(null)
-      };
-      JSTermHelpers(helpers);
-
-      let helperNames = Object.getOwnPropertyNames(helpers.sandbox);
-      matches = matches.concat(helperNames.filter(n => n.startsWith(result.matchProp)));
+      if (!this._jstermHelpersCache) {
+        let helpers = {
+          sandbox: Object.create(null)
+        };
+        JSTermHelpers(helpers);
+        this._jstermHelpersCache = Object.getOwnPropertyNames(helpers.sandbox);
+      }
+      matches = matches.concat(this._jstermHelpersCache.filter(n => n.startsWith(result.matchProp)));
     }
 
     return {
@@ -624,7 +824,10 @@ WebConsoleActor.prototype =
     // TODO: Bug 717611 - Web Console clear button does not clear cached errors
     let windowId = !this.parentActor.isRootActor ?
                    WebConsoleUtils.getInnerWindowId(this.window) : null;
+    let ConsoleAPIStorage = Cc["@mozilla.org/consoleAPI-storage;1"]
+                              .getService(Ci.nsIConsoleAPIStorage);
     ConsoleAPIStorage.clearEvents(windowId);
+
     if (this.parentActor.isRootActor) {
       Services.console.logStringMessage(null); // for the Error Console
       Services.console.reset();
@@ -659,6 +862,11 @@ WebConsoleActor.prototype =
   {
     for (let key in aRequest.preferences) {
       this._prefs[key] = aRequest.preferences[key];
+
+      if (key == "NetworkMonitor.saveRequestAndResponseBodies" &&
+          this.networkMonitor) {
+        this.networkMonitor.saveRequestAndResponseBodies = this._prefs[key];
+      }
     }
     return { updated: Object.keys(aRequest.preferences) };
   },
@@ -667,27 +875,6 @@ WebConsoleActor.prototype =
   // End of request handlers.
   //////////////////
 
-  /**
-   * Get the Debugger.Object for the given global object (usually a window
-   * object).
-   *
-   * @private
-   * @param object aGlobal
-   *        The global object for which you want a Debugger.Object.
-   * @return Debugger.Object
-   *         The Debugger.Object for the given global object.
-   */
-  _getDebuggerGlobal: function WCA__getDebuggerGlobal(aGlobal)
-  {
-    let windowId = WebConsoleUtils.getInnerWindowId(aGlobal);
-    if (!this._dbgGlobals.has(windowId)) {
-      let dbgGlobal = this.dbg.addDebuggee(aGlobal);
-      this.dbg.removeDebuggee(aGlobal);
-      this._dbgGlobals.set(windowId, dbgGlobal);
-    }
-    return this._dbgGlobals.get(windowId);
-  },
-
   /**
    * Create an object with the API we expose to the Web Console during
    * JavaScript evaluation.
@@ -705,12 +892,13 @@ WebConsoleActor.prototype =
   _getJSTermHelpers: function WCA__getJSTermHelpers(aDebuggerGlobal)
   {
     let helpers = {
-      window: this.window,
+      window: this.evalWindow,
       chromeWindow: this.chromeWindow.bind(this),
       makeDebuggeeValue: aDebuggerGlobal.makeDebuggeeValue.bind(aDebuggerGlobal),
       createValueGrip: this.createValueGrip.bind(this),
       sandbox: Object.create(null),
       helperResult: null,
+      consoleActor: this,
     };
     JSTermHelpers(helpers);
 
@@ -787,16 +975,6 @@ WebConsoleActor.prototype =
       aString = "help()";
     }
 
-    // Find the Debugger.Object of the given ObjectActor. This is used as
-    // a binding during eval: |_self|.
-    let bindSelf = null;
-    if (aOptions.bindObjectActor) {
-      let objActor = this.getActorByID(aOptions.bindObjectActor);
-      if (objActor) {
-        bindSelf = objActor.obj;
-      }
-    }
-
     // Find the Debugger.Frame of the given FrameActor.
     let frame = null, frameActor = null;
     if (aOptions.frameActor) {
@@ -810,36 +988,31 @@ WebConsoleActor.prototype =
       }
     }
 
-    // Determine which debugger to use, depending on the presence of the
-    // stackframe.
-    // This helps with avoid having bindings from a different Debugger. The
-    // Debugger.Frame comes from the jsdebugger's Debugger instance.
-    let dbg = this.dbg;
-    let dbgWindow = this._getDebuggerGlobal(this.window);
-    if (frame) {
-      dbg = frameActor.threadActor.dbg;
-      dbgWindow = dbg.addDebuggee(this.window);
-    }
+    // If we've been given a frame actor in whose scope we should evaluate the
+    // expression, be sure to use that frame's Debugger (that is, the JavaScript
+    // debugger's Debugger) for the whole operation, not the console's Debugger.
+    // (One Debugger will treat a different Debugger's Debugger.Object instances
+    // as ordinary objects, not as references to be followed, so mixing
+    // debuggers causes strange behaviors.)
+    let dbg = frame ? frameActor.threadActor.dbg : this.dbg;
+    let dbgWindow = dbg.makeGlobalObjectReference(this.evalWindow);
 
-    // If we have an object to bind to |_self| we need to determine the
-    // global of the given JavaScript object.
-    if (bindSelf) {
-      let jsObj = bindSelf.unsafeDereference();
-      let global = Cu.getGlobalForObject(jsObj);
-
-      // Get the Debugger.Object for the new global.
-      if (global != this.window) {
-        dbgWindow = dbg.addDebuggee(global);
-
-        // Remove the debuggee only if the Debugger instance belongs to the
-        // console actor, to avoid breaking the ThreadActor that owns the
-        // Debugger object.
-        if (dbg == this.dbg) {
-          dbg.removeDebuggee(global);
-        }
+    // If we have an object to bind to |_self|, create a Debugger.Object
+    // referring to that object, belonging to dbg.
+    let bindSelf = null;
+    let dbgWindow = dbg.makeGlobalObjectReference(this.evalWindow);
+    if (aOptions.bindObjectActor) {
+      let objActor = this.getActorByID(aOptions.bindObjectActor);
+      if (objActor) {
+        let jsObj = objActor.obj.unsafeDereference();
+        // If we use the makeDebuggeeValue method of jsObj's own global, then
+        // we'll get a D.O that sees jsObj as viewed from its own compartment -
+        // that is, without wrappers. The evalWithBindings call will then wrap
+        // jsObj appropriately for the evaluation compartment.
+        let global = Cu.getGlobalForObject(jsObj);
+        dbgWindow = dbg.makeGlobalObjectReference(global);
+        bindSelf = dbgWindow.makeDebuggeeValue(jsObj);
       }
-
-      bindSelf = dbgWindow.makeDebuggeeValue(jsObj);
     }
 
     // Get the JSTerm helpers for the given debugger window.
@@ -1004,6 +1177,8 @@ WebConsoleActor.prototype =
    *
    * @param object aEvent
    *        The initial network request event information.
+   * @param nsIHttpChannel aChannel
+   *        The network request nsIHttpChannel object.
    * @return object
    *         A new NetworkEventActor is returned. This is used for tracking the
    *         network request and response.
@@ -1028,8 +1203,10 @@ WebConsoleActor.prototype =
    * Get the NetworkEventActor for a nsIChannel, if it exists,
    * otherwise create a new one.
    *
-   * @param object aChannel
+   * @param nsIHttpChannel aChannel
    *        The channel for the network event.
+   * @return object
+   *         The NetworkEventActor for the given channel.
    */
   getNetworkEventActor: function WCA_getNetworkEventActor(aChannel) {
     let actor = this._netEvents.get(aChannel);
@@ -1093,6 +1270,29 @@ WebConsoleActor.prototype =
     this.conn.send(packet);
   },
 
+  /**
+   * Handler for reflow activity. This method forwards reflow events to the
+   * remote Web Console client.
+   *
+   * @see ConsoleReflowListener
+   * @param Object aReflowInfo
+   */
+  onReflowActivity: function WCA_onReflowActivity(aReflowInfo)
+  {
+    let packet = {
+      from: this.actorID,
+      type: "reflowActivity",
+      interruptible: aReflowInfo.interruptible,
+      start: aReflowInfo.start,
+      end: aReflowInfo.end,
+      sourceURL: aReflowInfo.sourceURL,
+      sourceLine: aReflowInfo.sourceLine,
+      functionName: aReflowInfo.functionName
+    };
+
+    this.conn.send(packet);
+  },
+
   //////////////////
   // End of event handlers for various listeners.
   //////////////////
@@ -1145,8 +1345,7 @@ WebConsoleActor.prototype =
   },
 
   /**
-   * Notification observer for the "inner-window-destroyed" topic. This function
-   * cleans up |this._dbgGlobals| when needed.
+   * Notification observer for the "last-pb-context-exited" topic.
    *
    * @private
    * @param object aSubject
@@ -1158,13 +1357,6 @@ WebConsoleActor.prototype =
   _onObserverNotification: function WCA__onObserverNotification(aSubject, aTopic)
   {
     switch (aTopic) {
-      case "inner-window-destroyed": {
-        let windowId = aSubject.QueryInterface(Ci.nsISupportsPRUint64).data;
-        if (this._dbgGlobals.has(windowId)) {
-          this._dbgGlobals.delete(windowId);
-        }
-        break;
-      }
       case "last-pb-context-exited":
         this.conn.send({
           from: this.actorID,
@@ -1172,7 +1364,17 @@ WebConsoleActor.prototype =
         });
         break;
     }
-  }
+  },
+
+  /**
+   * The "will-navigate" progress listener. This is used to clear the current
+   * eval scope.
+   */
+  _onWillNavigate: function WCA__onWillNavigate()
+  {
+    this._evalWindow = null;
+    this._progressListenerActive = false;
+  },
 };
 
 WebConsoleActor.prototype.requestTypes =
