@@ -77,11 +77,6 @@
 using mozilla::_ipdltest::IPDLUnitTestProcessChild;
 #endif  // ifdef MOZ_IPDL_TESTS
 
-#ifdef MOZ_NUWA_PROCESS
-#include "nsITimer.h"
-#define NUWA_PREPARATION_TIME 1000
-#endif
-
 using namespace mozilla;
 
 using mozilla::ipc::BrowserProcessSubThread;
@@ -104,14 +99,7 @@ using mozilla::startup::sChildProcessType;
 static NS_DEFINE_CID(kAppShellCID, NS_APPSHELL_CID);
 
 #ifdef XP_WIN
-static const PRUnichar kShellLibraryName[] =  L"shell32.dll";
-#endif
-
-#ifdef MOZ_NUWA_PROCESS
-extern "C" {
-void PrepareNuwaProcess() __attribute__((weak));
-void MakeNuwaProcess() __attribute__((weak));
-};
+static const wchar_t kShellLibraryName[] =  L"shell32.dll";
 #endif
 
 nsresult
@@ -277,16 +265,6 @@ SetTaskbarGroupId(const nsString& aId)
 }
 #endif
 
-#ifdef MOZ_NUWA_PROCESS
-void
-OnFinishNuwaPreparation(nsITimer *aTimer, void *aClosure)
-{
-    NS_ASSERTION(MakeNuwaProcess != nullptr,
-		 "MakeNuwaProcess() is not available!");
-    MakeNuwaProcess();
-}
-#endif
-
 nsresult
 XRE_InitChildProcess(int aArgc,
                      char* aArgv[],
@@ -416,7 +394,7 @@ XRE_InitChildProcess(int aArgc,
 
   if (PR_GetEnv("MOZ_DEBUG_CHILD_PROCESS")) {
 #ifdef OS_POSIX
-      printf("\n\nCHILDCHILDCHILDCHILD\n  debug me @%d\n\n", getpid());
+      printf("\n\nCHILDCHILDCHILDCHILD\n  debug me @ %d\n\n", getpid());
       sleep(30);
 #elif defined(OS_WIN)
       // Windows has a decent JIT debugging story, so NS_DebugBreak does the
@@ -534,19 +512,6 @@ XRE_InitChildProcess(int aArgc,
         return NS_ERROR_FAILURE;
       }
 
-#ifdef MOZ_NUWA_PROCESS
-      nsCOMPtr<nsITimer> timer;
-      if (aProcess == GeckoProcessType_Content &&
-          CommandLine::ForCurrentProcess()->HasSwitch(L"nuwa")) {
-        // Wait the Nuwa process for NUWA_PREPARATION_TIME ms.
-        timer = do_CreateInstance(NS_TIMER_CONTRACTID);
-        rv = timer->InitWithFuncCallback(OnFinishNuwaPreparation,
-                                         nullptr,
-                                         NUWA_PREPARATION_TIME,
-                                         nsITimer::TYPE_ONE_SHOT);
-      }
-#endif
-
       // Run the UI event loop on the main thread.
       uiMessageLoop.MessageLoop::Run();
 
@@ -753,7 +718,7 @@ ContentParent* gContentParent; //long-lived, manually refcounted
 TestShellParent* GetOrCreateTestShellParent()
 {
     if (!gContentParent) {
-        nsRefPtr<ContentParent> parent = ContentParent::GetNewOrUsed().get();
+        nsRefPtr<ContentParent> parent = ContentParent::GetNewOrUsed();
         parent.forget(&gContentParent);
     } else if (!gContentParent->IsAlive()) {
         return nullptr;
