@@ -9,8 +9,11 @@ import re
 import os
 
 
-BUGZILLA_FINGERPRINT = '47:13:a2:14:0c:46:45:53:12:0d:e5:36:16:a5:60:26:3e:da:3a:60'
-HG_FINGERPRINT = '10:78:e8:57:2d:95:de:7c:de:90:bd:22:e1:38:17:67:c5:a7:9c:14'
+HOST_FINGERPRINTS = {
+    'bitbucket.org': '67:b3:bf:9f:c5:38:0e:4c:dd:4e:8a:da:3d:11:1b:c2:a5:d1:6c:6b',
+    'bugzilla.mozilla.org': '47:13:a2:14:0c:46:45:53:12:0d:e5:36:16:a5:60:26:3e:da:3a:60',
+    'hg.mozilla.org': 'af:27:b9:34:47:4e:e5:98:01:f6:83:2b:51:c9:aa:d8:df:fb:1a:27',
+}
 
 
 class MercurialConfig(object):
@@ -67,9 +70,8 @@ class MercurialConfig(object):
         if 'hostfingerprints' not in self._c:
             self._c['hostfingerprints'] = {}
 
-        self._c['hostfingerprints']['bugzilla.mozilla.org'] = \
-            BUGZILLA_FINGERPRINT
-        self._c['hostfingerprints']['hg.mozilla.org'] = HG_FINGERPRINT
+        for k, v in HOST_FINGERPRINTS.items():
+            self._c['hostfingerprints'][k] = v
 
     def set_username(self, name, email):
         """Set the username to use for commits.
@@ -119,18 +121,18 @@ class MercurialConfig(object):
         d['showfunc'] = 1
         d['unified'] = 8
 
-    def autocommit_mq(self, value=True):
+    def have_mqext_autocommit_mq(self):
+        if 'mqext' not in self._c:
+            return False
+        v = self._c['mqext'].get('mqcommit')
+        return v == 'auto' or v == 'yes'
+
+    def ensure_mqext_autocommit_mq(self):
+        if self.have_mqext_autocommit_mq():
+            return
         if 'mqext' not in self._c:
             self._c['mqext'] = {}
-
-        if value:
-            self._c['mqext']['mqcommit'] = 'auto'
-        else:
-            try:
-                del self._c['mqext']['mqcommit']
-            except KeyError:
-                pass
-
+        self._c['mqext']['mqcommit'] = 'auto'
 
     def have_qnew_currentuser_default(self):
         if 'defaults' not in self._c:
@@ -152,8 +154,6 @@ class MercurialConfig(object):
 
         d = self._c['defaults']
         if 'qnew' not in d:
-          d['qnew'] = '-U'
+            d['qnew'] = '-U'
         else:
-          d['qnew'] = '-U ' + d['qnew']
-
-
+            d['qnew'] = '-U ' + d['qnew']
