@@ -32,16 +32,13 @@
 #include "nsToolkit.h"
 #include "TextInputHandler.h"
 #include "mozilla/HangMonitor.h"
-#include "sampler.h"
+#include "GeckoProfiler.h"
+#include "pratom.h"
 
 #include "npapi.h"
 
 using namespace mozilla::widget;
 
-// defined in nsChildView.mm
-extern nsIRollupListener * gRollupListener;
-extern nsIWidget         * gRollupWidget;
-
 // defined in nsCocoaWindow.mm
 extern int32_t             gXULModalLevel;
 
@@ -316,7 +313,7 @@ nsAppShell::Init()
 
   rv = nsBaseAppShell::Init();
 
-#ifndef NP_NO_CARBON
+#ifndef __LP64__
   TextInputHandler::InstallPluginKeyEventsHandler();
 #endif
 
@@ -332,17 +329,6 @@ nsAppShell::Init()
       nsToolkit::SwizzleMethods([NSApplication class], @selector(terminate:),
                                 @selector(nsAppShell_NSApplication_terminate:));
     }
-    if (!nsCocoaFeatures::OnSnowLeopardOrLater()) {
-      dlopen("/System/Library/Frameworks/Carbon.framework/Frameworks/Print.framework/Versions/Current/Plugins/PrintCocoaUI.bundle/Contents/MacOS/PrintCocoaUI",
-             RTLD_LAZY);
-      Class PDEPluginCallbackClass = ::NSClassFromString(@"PDEPluginCallback");
-      nsresult rv1 = nsToolkit::SwizzleMethods(PDEPluginCallbackClass, @selector(initWithPrintWindowController:),
-                                               @selector(nsAppShell_PDEPluginCallback_initWithPrintWindowController:));
-      if (NS_SUCCEEDED(rv1)) {
-        nsToolkit::SwizzleMethods(PDEPluginCallbackClass, @selector(dealloc),
-                                  @selector(nsAppShell_PDEPluginCallback_dealloc));
-      }
-    }
     gAppShellMethodsSwizzled = true;
   }
 
@@ -367,7 +353,7 @@ void
 nsAppShell::ProcessGeckoEvents(void* aInfo)
 {
   NS_OBJC_BEGIN_TRY_ABORT_BLOCK;
-  SAMPLE_LABEL("Events", "ProcessGeckoEvents");
+  PROFILER_LABEL("Events", "ProcessGeckoEvents");
   nsAppShell* self = static_cast<nsAppShell*> (aInfo);
 
   if (self->mRunningEventLoop) {
@@ -631,7 +617,11 @@ nsAppShell::ProcessNextNativeEvent(bool aMayWait)
     // need to use nextEventMatchingMask and sendEvent -- otherwise (in
     // Minefield) the modal window (or non-main event loop) won't receive key
     // events or most mouse events.
-    if ([NSApp _isRunningModal] || !InGeckoMainEventLoop()) {
+    //
+    // Add aMayWait to minimize the number of calls to -[NSApp sendEvent:]
+    // made from nsAppShell::ProcessNextNativeEvent() (and indirectly from
+    // nsBaseAppShell::OnProcessNextEvent()), to work around bug 959281.
+    if ([NSApp _isRunningModal] || (aMayWait && !InGeckoMainEventLoop())) {
       if ((nextEvent = [NSApp nextEventMatchingMask:NSAnyEventMask
                                           untilDate:waitUntil
                                              inMode:currentMode
@@ -749,7 +739,7 @@ NS_IMETHODIMP
 nsAppShell::Run(void)
 {
   NS_ASSERTION(!mStarted, "nsAppShell::Run() called multiple times");
-  if (mStarted)
+  if (mStarted || mTerminated)
     return NS_OK;
 
   mStarted = true;
@@ -778,7 +768,7 @@ nsAppShell::Exit(void)
   delete gCocoaAppModalWindowList;
   gCocoaAppModalWindowList = NULL;
 
-#ifndef NP_NO_CARBON
+#ifndef __LP64__
   TextInputHandler::RemovePluginKeyEventsHandler();
 #endif
 
@@ -854,7 +844,8 @@ nsAppShell::OnProcessNextEvent(nsIThreadInternal *aThread, bool aMayWait,
 // public
 NS_IMETHODIMP
 nsAppShell::AfterProcessNextEvent(nsIThreadInternal *aThread,
-                                  uint32_t aRecursionDepth)
+                                  uint32_t aRecursionDepth,
+                                  bool aEventWasProcessed)
 {
   NS_OBJC_BEGIN_TRY_ABORT_BLOCK_NSRESULT;
 
@@ -870,7 +861,8 @@ nsAppShell::AfterProcessNextEvent(nsIThreadInternal *aThread,
   ::CFArrayRemoveValueAtIndex(mAutoreleasePools, count - 1);
   [pool release];
 
-  return nsBaseAppShell::AfterProcessNextEvent(aThread, aRecursionDepth);
+  return nsBaseAppShell::AfterProcessNextEvent(aThread, aRecursionDepth,
+                                               aEventWasProcessed);
 
   NS_OBJC_END_TRY_ABORT_BLOCK_NSRESULT;
 }
@@ -963,8 +955,10 @@ nsAppShell::AfterProcessNextEvent(nsIThreadInternal *aThread,
 
   NSString *sender = [aNotification object];
   if (!sender || ![sender isEqualToString:@"org.mozilla.gecko.PopupWindow"]) {
-    if (gRollupListener && gRollupWidget)
-      gRollupListener->Rollup(0);
+    nsIRollupListener* rollupListener = nsBaseWidget::GetActiveRollupListener();
+    nsCOMPtr<nsIWidget> rollupWidget = rollupListener->GetRollupWidget();
+    if (rollupWidget)
+      rollupListener->Rollup(0, nullptr, nullptr);
   }
 
   NS_OBJC_END_TRY_ABORT_BLOCK;
@@ -1033,40 +1027,3 @@ nsAppShell::AfterProcessNextEvent(nsIThreadInternal *aThread,
 }
 
 @end
-
-@interface NSObject (PDEPluginCallbackMethodSwizzling)
-- (id)nsAppShell_PDEPluginCallback_initWithPrintWindowController:(id)controller;
-- (void)nsAppShell_PDEPluginCallback_dealloc;
-@end
-
-@implementation NSObject (PDEPluginCallbackMethodSwizzling)
-
-// On Leopard, the PDEPluginCallback class in Apple's PrintCocoaUI module
-// fails to retain and release its PMPrintWindowController object.  This
-// causes the PMPrintWindowController to sometimes be deleted prematurely,
-// leading to crashes on attempts to access it.  One example is bug 396680,
-// caused by attempting to call a deleted PMPrintWindowController object's
-// printSettings method.  We work around the problem by hooking the
-// appropriate methods and retaining and releasing the object ourselves.
-// PrintCocoaUI.bundle is a "plugin" of the Carbon framework's Print
-// framework.
-
-- (id)nsAppShell_PDEPluginCallback_initWithPrintWindowController:(id)controller
-{
-  return [self nsAppShell_PDEPluginCallback_initWithPrintWindowController:[controller retain]];
-}
-
-- (void)nsAppShell_PDEPluginCallback_dealloc
-{
-  // Since the PDEPluginCallback class is undocumented (and the OS header
-  // files have no definition for it), we need to use low-level methods to
-  // access its _printWindowController variable.  (object_getInstanceVariable()
-  // is also available in Objective-C 2.0, so this code is 64-bit safe.)
-  id _printWindowController = nil;
-  object_getInstanceVariable(self, "_printWindowController",
-                             (void **) &_printWindowController);
-  [_printWindowController release];
-  [self nsAppShell_PDEPluginCallback_dealloc];
-}
-
-@end
