@@ -216,7 +216,7 @@ GonkDisplayJB::SwapBuffers(EGLDisplay dpy, EGLSurface sur)
     mList->outbufAcquireFenceFd = -1;
 #endif
     eglSwapBuffers(dpy, sur);
-    return Post(mFBSurface->lastHandle, mFBSurface->lastFenceFD);
+    return Post(mFBSurface->lastHandle, mFBSurface->GetPrevFBAcquireFd());
 }
 
 bool
@@ -249,10 +249,21 @@ GonkDisplayJB::Post(buffer_handle_t buf, int fence)
     mList->hwLayers[1].handle = buf;
     mList->hwLayers[1].transform = 0;
     mList->hwLayers[1].blending = HWC_BLENDING_PREMULT;
+#if ANDROID_VERSION >= 19
+    if (mHwc->common.version >= HWC_DEVICE_API_VERSION_1_3) {
+        mList->hwLayers[1].sourceCropf.left = 0;
+        mList->hwLayers[1].sourceCropf.top = 0;
+        mList->hwLayers[1].sourceCropf.right = mWidth;
+        mList->hwLayers[1].sourceCropf.bottom = mHeight;
+    } else {
+        mList->hwLayers[1].sourceCrop = r;
+    }
+#else
     mList->hwLayers[1].sourceCrop = r;
+#endif
     mList->hwLayers[1].displayFrame = r;
     mList->hwLayers[1].visibleRegionScreen.numRects = 1;
-    mList->hwLayers[1].visibleRegionScreen.rects = &mList->hwLayers[1].sourceCrop;
+    mList->hwLayers[1].visibleRegionScreen.rects = &mList->hwLayers[1].displayFrame;
     mList->hwLayers[1].acquireFenceFd = fence;
     mList->hwLayers[1].releaseFenceFd = -1;
 #if ANDROID_VERSION == 18
@@ -293,6 +304,12 @@ GonkDisplayJB::SetFBReleaseFd(int fd)
     mFBSurface->setReleaseFenceFd(fd);
 }
 
+int
+GonkDisplayJB::GetPrevFBAcquireFd()
+{
+    return mFBSurface->GetPrevFBAcquireFd();
+}
+
 __attribute__ ((visibility ("default")))
 GonkDisplay*
 GetGonkDisplay()
