@@ -17,6 +17,11 @@
 #include "nsLookAndFeel.h"
 #include "nsStyleConsts.h"
 #include "gfxFont.h"
+#include "gfxFontConstants.h"
+#include "mozilla/gfx/2D.h"
+#include "cutils/properties.h"
+
+static const char16_t UNICODE_BULLET = 0x2022;
 
 nsLookAndFeel::nsLookAndFeel()
     : nsXPLookAndFeel()
@@ -82,7 +87,7 @@ nsLookAndFeel::NativeGetColor(ColorID aID, nscolor &aColor)
         // not used?
         aColor = BASE_NORMAL_COLOR;
         break;
-    case eColorID_TextForeground: 
+    case eColorID_TextForeground:
         // not used?
         aColor = TEXT_NORMAL_COLOR;
         break;
@@ -243,7 +248,7 @@ nsLookAndFeel::NativeGetColor(ColorID aID, nscolor &aColor)
         break;
     case eColorID__moz_dragtargetzone:
         aColor = BG_SELECTED_COLOR;
-        break; 
+        break;
     case eColorID__moz_buttondefault:
         // default button border color
         aColor = NS_RGB(0,0,0);
@@ -350,12 +355,22 @@ nsLookAndFeel::GetIntImpl(IntID aID, int32_t &aResult)
             break;
 
         case eIntID_WindowsDefaultTheme:
-        case eIntID_MaemoClassic:
         case eIntID_WindowsThemeIdentifier:
+        case eIntID_OperatingSystemVersionIdentifier:
             aResult = 0;
             rv = NS_ERROR_NOT_IMPLEMENTED;
             break;
 
+        case eIntID_IMERawInputUnderlineStyle:
+        case eIntID_IMEConvertedTextUnderlineStyle:
+            aResult = NS_STYLE_TEXT_DECORATION_STYLE_SOLID;
+            break;
+
+        case eIntID_IMESelectedRawTextUnderlineStyle:
+        case eIntID_IMESelectedConvertedTextUnderline:
+            aResult = NS_STYLE_TEXT_DECORATION_STYLE_NONE;
+            break;
+
         case eIntID_SpellCheckerUnderlineStyle:
             aResult = NS_STYLE_TEXT_DECORATION_STYLE_WAVY;
             break;
@@ -364,6 +379,13 @@ nsLookAndFeel::GetIntImpl(IntID aID, int32_t &aResult)
             aResult = 0;
             break;
 
+        case eIntID_PhysicalHomeButton: {
+            char propValue[PROPERTY_VALUE_MAX];
+            property_get("ro.moz.has_home_button", propValue, "1");
+            aResult = atoi(propValue);
+            break;
+        }
+
         default:
             aResult = 0;
             rv = NS_ERROR_FAILURE;
@@ -372,13 +394,35 @@ nsLookAndFeel::GetIntImpl(IntID aID, int32_t &aResult)
     return rv;
 }
 
+nsresult
+nsLookAndFeel::GetFloatImpl(FloatID aID, float &aResult)
+{
+  nsresult res = nsXPLookAndFeel::GetFloatImpl(aID, aResult);
+  if (NS_SUCCEEDED(res))
+    return res;
+  res = NS_OK;
+
+  switch (aID) {
+    case eFloatID_IMEUnderlineRelativeSize:
+        aResult = 1.0f;
+        break;
+    case eFloatID_SpellCheckerUnderlineRelativeSize:
+        aResult = 1.0f;
+        break;
+    default:
+        aResult = -1.0;
+        res = NS_ERROR_FAILURE;
+    }
+  return res;
+}
+
 /*virtual*/
 bool
 nsLookAndFeel::GetFontImpl(FontID aID, nsString& aFontName,
                            gfxFontStyle& aFontStyle,
                            float aDevPixPerCSSPixel)
 {
-    aFontName.AssignLiteral("\"Droid Sans\"");
+    aFontName.AssignLiteral("\"Fira Sans OT\"");
     aFontStyle.style = NS_FONT_STYLE_NORMAL;
     aFontStyle.weight = NS_FONT_WEIGHT_NORMAL;
     aFontStyle.stretch = NS_FONT_STRETCH_NORMAL;
@@ -400,3 +444,10 @@ nsLookAndFeel::GetPasswordMaskDelayImpl()
     // Same value on Android framework
     return 1500;
 }
+
+/* virtual */
+char16_t
+nsLookAndFeel::GetPasswordCharacterImpl()
+{
+    return UNICODE_BULLET;
+}
