@@ -15,26 +15,21 @@
 
 #include "mozilla/DebugOnly.h"
 
-#include <EGL/egl.h>
-#include <EGL/eglext.h>
-
 #include <fcntl.h>
 
 #include "android/log.h"
-#include "ui/FramebufferNativeWindow.h"
 
 #include "mozilla/dom/TabParent.h"
 #include "mozilla/Hal.h"
 #include "mozilla/Preferences.h"
 #include "mozilla/FileUtils.h"
-#include "BootAnimation.h"
+#include "mozilla/ClearOnShutdown.h"
 #include "Framebuffer.h"
 #include "gfxContext.h"
 #include "gfxPlatform.h"
 #include "gfxUtils.h"
 #include "GLContextProvider.h"
-#include "HwcComposer2D.h"
-#include "LayerManagerOGL.h"
+#include "GLContext.h"
 #include "nsAutoPtr.h"
 #include "nsAppShell.h"
 #include "nsIdleService.h"
@@ -45,6 +40,14 @@
 #include "cutils/properties.h"
 #include "ClientLayerManager.h"
 #include "BasicLayers.h"
+#include "libdisplay/GonkDisplay.h"
+#include "pixelflinger/format.h"
+#include "mozilla/BasicEvents.h"
+#include "mozilla/layers/APZCTreeManager.h"
+#include "mozilla/layers/CompositorParent.h"
+#include "ParentProcessController.h"
+#include "nsThreadUtils.h"
+#include "HwcComposer2D.h"
 
 #define LOG(args...)  __android_log_print(ANDROID_LOG_INFO, "Gonk" , ## args)
 #define LOGW(args...) __android_log_print(ANDROID_LOG_WARN, "Gonk", ## args)
@@ -63,18 +66,15 @@ nsIntRect gScreenBounds;
 static uint32_t sScreenRotation;
 static uint32_t sPhysicalScreenRotation;
 static nsIntRect sVirtualBounds;
-static gfxMatrix sRotationMatrix;
 
 static nsRefPtr<GLContext> sGLContext;
 static nsTArray<nsWindow *> sTopWindows;
-static nsWindow *gWindowToRedraw = nullptr;
 static nsWindow *gFocusedWindow = nullptr;
 static bool sFramebufferOpen;
 static bool sUsingOMTC;
 static bool sUsingHwc;
 static bool sScreenInitialized;
 static nsRefPtr<gfxASurface> sOMTCSurface;
-static pthread_t sFramebufferWatchThread;
 
 namespace {
 
@@ -99,6 +99,15 @@ public:
             }
         }
 
+        // Notify observers that the screen state has just changed.
+        nsCOMPtr<nsIObserverService> observerService = mozilla::services::GetObserverService();
+        if (observerService) {
+          observerService->NotifyObservers(
+            nullptr, "screen-state-changed",
+            mIsOn ? MOZ_UTF16("on") : MOZ_UTF16("off")
+          );
+        }
+
         return NS_OK;
     }
 
@@ -106,30 +115,13 @@ private:
     bool mIsOn;
 };
 
-static const char* kSleepFile = "/sys/power/wait_for_fb_sleep";
-static const char* kWakeFile = "/sys/power/wait_for_fb_wake";
+static StaticRefPtr<ScreenOnOffEvent> sScreenOnEvent;
+static StaticRefPtr<ScreenOnOffEvent> sScreenOffEvent;
 
-static void *frameBufferWatcher(void *) {
-
-    char buf;
-    bool ret;
-
-    nsRefPtr<ScreenOnOffEvent> mScreenOnEvent = new ScreenOnOffEvent(true);
-    nsRefPtr<ScreenOnOffEvent> mScreenOffEvent = new ScreenOnOffEvent(false);
-
-    while (true) {
-        // Cannot use epoll here because kSleepFile and kWakeFile are
-        // always ready to read and blocking.
-        ret = ReadSysFile(kSleepFile, &buf, sizeof(buf));
-        NS_WARN_IF_FALSE(ret, "WAIT_FOR_FB_SLEEP failed");
-        NS_DispatchToMainThread(mScreenOffEvent);
-
-        ret = ReadSysFile(kWakeFile, &buf, sizeof(buf));
-        NS_WARN_IF_FALSE(ret, "WAIT_FOR_FB_WAKE failed");
-        NS_DispatchToMainThread(mScreenOnEvent);
-    }
-
-    return NULL;
+static void
+displayEnabledCallback(bool enabled)
+{
+    NS_DispatchToMainThread(enabled ? sScreenOnEvent : sScreenOffEvent);
 }
 
 } // anonymous namespace
@@ -137,11 +129,11 @@ static void *frameBufferWatcher(void *) {
 nsWindow::nsWindow()
 {
     if (!sScreenInitialized) {
-        // Watching screen on/off state by using a pthread
-        // which implicitly calls exit() when the main thread ends
-        if (pthread_create(&sFramebufferWatchThread, NULL, frameBufferWatcher, NULL)) {
-            NS_RUNTIMEABORT("Failed to create framebufferWatcherThread, aborting...");
-        }
+        sScreenOnEvent = new ScreenOnOffEvent(true);
+        ClearOnShutdown(&sScreenOnEvent);
+        sScreenOffEvent = new ScreenOnOffEvent(false);
+        ClearOnShutdown(&sScreenOffEvent);
+        GetGonkDisplay()->OnEnabled(displayEnabledCallback);
 
         nsIntSize screenSize;
         bool gotFB = Framebuffer::GetSize(&screenSize);
@@ -154,20 +146,6 @@ nsWindow::nsWindow()
         property_get("ro.sf.hwrotation", propValue, "0");
         sPhysicalScreenRotation = atoi(propValue) / 90;
 
-        // Unlike nsScreenGonk::SetRotation(), only support 0 and 180 as there
-        // are no known screens that are mounted at 90 or 270 at the moment.
-        switch (sPhysicalScreenRotation) {
-        case nsIScreen::ROTATION_0_DEG:
-            break;
-        case nsIScreen::ROTATION_180_DEG:
-            sRotationMatrix.Translate(gfxPoint(gScreenBounds.width,
-                                               gScreenBounds.height));
-            sRotationMatrix.Rotate(M_PI);
-            break;
-        default:
-            MOZ_NOT_REACHED("Unknown rotation");
-            break;
-        }
         sVirtualBounds = gScreenBounds;
 
         sScreenInitialized = true;
@@ -184,12 +162,13 @@ nsWindow::nsWindow()
         sUsingOMTC = ShouldUseOffMainThreadCompositing();
 
         property_get("ro.display.colorfill", propValue, "0");
-        sUsingHwc = Preferences::GetBool("layers.composer2d.enabled",
-                                         atoi(propValue) == 1);
+
+        //Update sUsingHwc whenever layers.composer2d.enabled changes
+        Preferences::AddBoolVarCache(&sUsingHwc, "layers.composer2d.enabled");
 
         if (sUsingOMTC) {
           sOMTCSurface = new gfxImageSurface(gfxIntSize(1, 1),
-                                             gfxASurface::ImageFormatRGB24);
+                                             gfxImageFormat::RGB24);
         }
     }
 }
@@ -206,32 +185,27 @@ nsWindow::DoDraw(void)
         return;
     }
 
-    if (!gWindowToRedraw) {
+    if (sTopWindows.IsEmpty()) {
         LOG("  no window to draw, bailing");
         return;
     }
 
-    nsIntRegion region = gWindowToRedraw->mDirtyRegion;
-    gWindowToRedraw->mDirtyRegion.SetEmpty();
+    nsWindow *targetWindow = (nsWindow *)sTopWindows[0];
+    while (targetWindow->GetLastChild())
+        targetWindow = (nsWindow *)targetWindow->GetLastChild();
 
-    nsIWidgetListener* listener = gWindowToRedraw->GetWidgetListener();
+    nsIntRegion region = sTopWindows[0]->mDirtyRegion;
+    sTopWindows[0]->mDirtyRegion.SetEmpty();
+
+    nsIWidgetListener* listener = targetWindow->GetWidgetListener();
     if (listener) {
-        listener->WillPaintWindow(gWindowToRedraw);
+        listener->WillPaintWindow(targetWindow);
     }
 
-    LayerManager* lm = gWindowToRedraw->GetLayerManager();
-    if (mozilla::layers::LAYERS_OPENGL == lm->GetBackendType()) {
-        LayerManagerOGL* oglm = static_cast<LayerManagerOGL*>(lm);
-        oglm->SetClippingRegion(region);
-        oglm->SetWorldTransform(sRotationMatrix);
-
-        listener = gWindowToRedraw->GetWidgetListener();
-        if (listener) {
-            listener->PaintWindow(gWindowToRedraw, region, 0);
-        }
-    } else if (mozilla::layers::LAYERS_CLIENT == lm->GetBackendType()) {
+    LayerManager* lm = targetWindow->GetLayerManager();
+    if (mozilla::layers::LayersBackend::LAYERS_CLIENT == lm->GetBackendType()) {
       // No need to do anything, the compositor will handle drawing
-    } else if (mozilla::layers::LAYERS_BASIC == lm->GetBackendType()) {
+    } else if (mozilla::layers::LayersBackend::LAYERS_BASIC == lm->GetBackendType()) {
         MOZ_ASSERT(sFramebufferOpen || sUsingOMTC);
         nsRefPtr<gfxASurface> targetSurface;
 
@@ -247,12 +221,12 @@ nsWindow::DoDraw(void)
 
             // No double-buffering needed.
             AutoLayerManagerSetup setupLayerManager(
-                gWindowToRedraw, ctx, mozilla::layers::BUFFER_NONE,
+                targetWindow, ctx, mozilla::layers::BufferMode::BUFFER_NONE,
                 ScreenRotation(EffectiveScreenRotation()));
 
-            listener = gWindowToRedraw->GetWidgetListener();
+            listener = targetWindow->GetWidgetListener();
             if (listener) {
-                listener->PaintWindow(gWindowToRedraw, region, 0);
+                listener->PaintWindow(targetWindow, region);
             }
         }
 
@@ -264,14 +238,14 @@ nsWindow::DoDraw(void)
         NS_RUNTIMEABORT("Unexpected layer manager type");
     }
 
-    listener = gWindowToRedraw->GetWidgetListener();
+    listener = targetWindow->GetWidgetListener();
     if (listener) {
         listener->DidPaintWindow();
     }
 }
 
 nsEventStatus
-nsWindow::DispatchInputEvent(nsGUIEvent &aEvent, bool* aWasCaptured)
+nsWindow::DispatchInputEvent(WidgetGUIEvent& aEvent, bool* aWasCaptured)
 {
     if (aWasCaptured) {
         *aWasCaptured = false;
@@ -311,11 +285,10 @@ nsWindow::Create(nsIWidget *aParent,
 
     mBounds = aRect;
 
-    nsWindow *parent = (nsWindow *)aNativeParent;
-    mParent = parent;
+    mParent = (nsWindow *)aParent;
     mVisible = false;
 
-    if (!aNativeParent) {
+    if (!aParent) {
         mBounds = sVirtualBounds;
     }
 
@@ -331,11 +304,11 @@ nsWindow::Create(nsIWidget *aParent,
 NS_IMETHODIMP
 nsWindow::Destroy(void)
 {
+    mOnDestroyCalled = true;
     sTopWindows.RemoveElement(this);
-    if (this == gWindowToRedraw)
-        gWindowToRedraw = nullptr;
     if (this == gFocusedWindow)
         gFocusedWindow = nullptr;
+    nsBaseWidget::OnDestroy();
     return NS_OK;
 }
 
@@ -409,8 +382,8 @@ nsWindow::Resize(double aX,
     if (mWidgetListener)
         mWidgetListener->WindowResized(this, mBounds.width, mBounds.height);
 
-    if (aRepaint && gWindowToRedraw)
-        gWindowToRedraw->Invalidate(sVirtualBounds);
+    if (aRepaint)
+        Invalidate(sVirtualBounds);
 
     return NS_OK;
 }
@@ -446,14 +419,13 @@ nsWindow::ConfigureChildren(const nsTArray<nsIWidget::Configuration>&)
 NS_IMETHODIMP
 nsWindow::Invalidate(const nsIntRect &aRect)
 {
-    nsWindow *parent = mParent;
-    while (parent && parent != sTopWindows[0])
-        parent = parent->mParent;
-    if (parent != sTopWindows[0])
+    nsWindow *top = mParent;
+    while (top && top->mParent)
+        top = top->mParent;
+    if (top != sTopWindows[0] && this != sTopWindows[0])
         return NS_OK;
 
     mDirtyRegion.Or(mDirtyRegion, aRect);
-    gWindowToRedraw = this;
     gDrawRequest = true;
     mozilla::NotifyEvent();
     return NS_OK;
@@ -480,15 +452,13 @@ nsWindow::GetNativeData(uint32_t aDataType)
 {
     switch (aDataType) {
     case NS_NATIVE_WINDOW:
-        return NativeWindow();
-    case NS_NATIVE_WIDGET:
-        return this;
+        return GetGonkDisplay()->GetNativeWindow();
     }
     return nullptr;
 }
 
 NS_IMETHODIMP
-nsWindow::DispatchEvent(nsGUIEvent *aEvent, nsEventStatus &aStatus)
+nsWindow::DispatchEvent(WidgetGUIEvent* aEvent, nsEventStatus& aStatus)
 {
     if (mWidgetListener)
       aStatus = mWidgetListener->HandleEvent(aEvent, mUseAttachedEvents);
@@ -541,18 +511,24 @@ nsWindow::MakeFullScreen(bool aFullScreen)
 float
 nsWindow::GetDPI()
 {
-    return NativeWindow()->xdpi;
+    return GetGonkDisplay()->xdpi;
 }
 
 double
 nsWindow::GetDefaultScaleInternal()
 {
-    double rawscale = GetDPI() / 192.0;
-    if (rawscale < 1.25)
-        return 1;
-    else if (rawscale < 1.75)
-        return 1.5;
-    return 2;
+    float dpi = GetDPI();
+    // The mean pixel density for mdpi devices is 160dpi, 240dpi for hdpi,
+    // and 320dpi for xhdpi, respectively.
+    // We'll take the mid-value between these three numbers as the boundary.
+    if (dpi < 200.0) {
+        return 1.0; // mdpi devices.
+    }
+    if (dpi < 300.0) {
+        return 1.5; // hdpi devices.
+    }
+    // xhdpi devices and beyond.
+    return floor(dpi / 150.0 + 0.5);
 }
 
 LayerManager *
@@ -566,22 +542,20 @@ nsWindow::GetLayerManager(PLayerTransactionChild* aShadowManager,
     if (mLayerManager) {
         // This layer manager might be used for painting outside of DoDraw(), so we need
         // to set the correct rotation on it.
-        if (mLayerManager->GetBackendType() == LAYERS_BASIC) {
+        if (mLayerManager->GetBackendType() == LayersBackend::LAYERS_BASIC) {
             BasicLayerManager* manager =
                 static_cast<BasicLayerManager*>(mLayerManager.get());
-            manager->SetDefaultTargetConfiguration(mozilla::layers::BUFFER_NONE,
+            manager->SetDefaultTargetConfiguration(mozilla::layers::BufferMode::BUFFER_NONE,
                                                    ScreenRotation(EffectiveScreenRotation()));
-        } else if (mLayerManager->GetBackendType() == LAYERS_CLIENT) {
+        } else if (mLayerManager->GetBackendType() == LayersBackend::LAYERS_CLIENT) {
             ClientLayerManager* manager =
                 static_cast<ClientLayerManager*>(mLayerManager.get());
-            manager->SetDefaultTargetConfiguration(mozilla::layers::BUFFER_NONE,
+            manager->SetDefaultTargetConfiguration(mozilla::layers::BufferMode::BUFFER_NONE,
                                                    ScreenRotation(EffectiveScreenRotation()));
         }
         return mLayerManager;
     }
 
-    StopBootAnimation();
-
     // Set mUseLayersAcceleration here to make it consistent with
     // nsBaseWidget::GetLayerManager
     mUseLayersAcceleration = ComputeShouldAccelerate(mUseLayersAcceleration);
@@ -594,6 +568,11 @@ nsWindow::GetLayerManager(PLayerTransactionChild* aShadowManager,
 
     if (sUsingOMTC) {
         CreateCompositor();
+        if (mCompositorParent) {
+            uint64_t rootLayerTreeId = mCompositorParent->RootLayerTreeId();
+            CompositorParent::SetControllerForLayerTree(rootLayerTreeId, new ParentProcessController());
+            CompositorParent::GetAPZCTreeManager(rootLayerTreeId)->SetDPI(GetDPI());
+        }
         if (mLayerManager)
             return mLayerManager;
     }
@@ -605,18 +584,6 @@ nsWindow::GetLayerManager(PLayerTransactionChild* aShadowManager,
         }
 
         MOZ_ASSERT(fbBounds.value == gScreenBounds);
-        if (sGLContext) {
-            nsRefPtr<LayerManagerOGL> layerManager = new LayerManagerOGL(this);
-
-            if (layerManager->Initialize(sGLContext)) {
-                mLayerManager = layerManager;
-                return mLayerManager;
-            } else {
-                LOGW("Could not create OGL LayerManager");
-            }
-        } else {
-            LOGW("GL context was not created");
-        }
     }
 
     // Fall back to software rendering.
@@ -643,7 +610,7 @@ nsWindow::GetThebesSurface()
 
     // XXX this really wants to return already_AddRefed, but this only really gets used
     // on direct assignment to a gfxASurface
-    return new gfxImageSurface(gfxIntSize(5,5), gfxImageSurface::ImageFormatRGB24);
+    return new gfxImageSurface(gfxIntSize(5,5), gfxImageFormat::RGB24);
 }
 
 void
@@ -678,7 +645,7 @@ uint32_t
 nsWindow::GetGLFrameBufferFormat()
 {
     if (mLayerManager &&
-        mLayerManager->GetBackendType() == mozilla::layers::LAYERS_OPENGL) {
+        mLayerManager->GetBackendType() == mozilla::layers::LayersBackend::LAYERS_OPENGL) {
         // We directly map the hardware fb on Gonk.  The hardware fb
         // has RGB format.
         return LOCAL_GL_RGB;
@@ -707,9 +674,11 @@ nsWindow::GetComposer2D()
     if (!sUsingHwc) {
         return nullptr;
     }
+
     if (HwcComposer2D* hwc = HwcComposer2D::GetInstance()) {
         return hwc->Initialized() ? hwc : nullptr;
     }
+
     return nullptr;
 }
 
@@ -746,7 +715,7 @@ nsScreenGonk::GetAvailRect(int32_t *outLeft,  int32_t *outTop,
 static uint32_t
 ColorDepth()
 {
-    switch (NativeWindow()->getDevice()->format) {
+    switch (GetGonkDisplay()->surfaceformat) {
     case GGL_PIXEL_FORMAT_RGB_565:
         return 16;
     case GGL_PIXEL_FORMAT_RGBA_8888:
@@ -787,9 +756,6 @@ nsScreenGonk::SetRotation(uint32_t aRotation)
         return NS_OK;
 
     sScreenRotation = aRotation;
-    sRotationMatrix =
-        ComputeTransformForRotation(gScreenBounds,
-                                    ScreenRotation(EffectiveScreenRotation()));
     uint32_t rotation = EffectiveScreenRotation();
     if (rotation == nsIScreen::ROTATION_90_DEG ||
         rotation == nsIScreen::ROTATION_270_DEG) {
@@ -799,12 +765,12 @@ nsScreenGonk::SetRotation(uint32_t aRotation)
         sVirtualBounds = gScreenBounds;
     }
 
+    nsAppShell::NotifyScreenRotation();
+
     for (unsigned int i = 0; i < sTopWindows.Length(); i++)
         sTopWindows[i]->Resize(sVirtualBounds.width,
                                sVirtualBounds.height,
-                               !i);
-
-    nsAppShell::NotifyScreenRotation();
+                               true);
 
     return NS_OK;
 }
@@ -831,8 +797,7 @@ ComputeOrientation(uint32_t aRotation, const nsIntSize& aScreenSize)
         return (naturallyPortrait ? eScreenOrientation_LandscapeSecondary : 
                 eScreenOrientation_PortraitSecondary);
     default:
-        MOZ_NOT_REACHED("Gonk screen must always have a known rotation");
-        return eScreenOrientation_None;
+        MOZ_CRASH("Gonk screen must always have a known rotation");
     }
 }
 
