@@ -13,10 +13,8 @@
 #include <gtk/gtk.h>
 #include <gdk/gdk.h>
 
-#ifdef MOZ_PANGO
 #include <pango/pango.h>
 #include <pango/pango-fontmap.h>
-#endif
 
 #include <fontconfig/fontconfig.h>
 #include "gfxPlatformGtk.h"
@@ -663,6 +661,9 @@ nsLookAndFeel::GetIntImpl(IntID aID, int32_t &aResult)
     case eIntID_SwipeAnimationEnabled:
         aResult = 0;
         break;
+    case eIntID_ColorPickerAvailable:
+        aResult = 1;
+        break;
     default:
         aResult = 0;
         res     = NS_ERROR_FAILURE;
@@ -697,7 +698,6 @@ nsLookAndFeel::GetFloatImpl(FloatID aID, float &aResult)
     return res;
 }
 
-#ifdef MOZ_PANGO
 static void
 GetSystemFontInfo(GtkWidget *aWidget,
                   nsString *aFontName,
@@ -801,24 +801,6 @@ GetSystemFontInfo(LookAndFeel::FontID aID,
     }
 }
 
-#else // not MOZ_PANGO
-
-static void
-GetSystemFontInfo(LookAndFeel::FontID /*unused */,
-                  nsString *aFontName,
-                  gfxFontStyle *aFontStyle)
-{
-    /* FIXME: DFB FT2 Hardcoding the system font info for now. */
-    aFontStyle->style      = NS_FONT_STYLE_NORMAL;
-    aFontStyle->weight     = NS_FONT_WEIGHT_NORMAL;
-    aFontStyle->size       = 40/3;
-    aFontStyle->stretch    = NS_FONT_STRETCH_NORMAL;
-    aFontStyle->systemFont = true;
-    aFontName->AssignLiteral("\"Sans\"");
-}
-
-#endif // not MOZ_PANGO
-
 bool
 nsLookAndFeel::GetFontImpl(FontID aID, nsString& aFontName,
                            gfxFontStyle& aFontStyle,
@@ -964,6 +946,11 @@ nsLookAndFeel::Init()
     GdkRGBA color;
     GtkStyleContext *style;
 
+    // Gtk manages a screen's CSS in the settings object so we
+    // ask Gtk to create it explicitly. Otherwise we may end up 
+    // with wrong color theme, see Bug 972382
+    (void)gtk_settings_get_for_screen(gdk_screen_get_default());
+
     GtkWidgetPath *path = gtk_widget_path_new();
     gtk_widget_path_append_type(path, GTK_TYPE_WINDOW);
 
@@ -1210,7 +1197,7 @@ nsLookAndFeel::Init()
     // invisible character styles
     guint value;
     g_object_get (entry, "invisible-char", &value, nullptr);
-    sInvisibleCharacter = PRUnichar(value);
+    sInvisibleCharacter = char16_t(value);
 
     // caret styles
     gtk_widget_style_get(entry,
@@ -1221,7 +1208,7 @@ nsLookAndFeel::Init()
 }
 
 // virtual
-PRUnichar
+char16_t
 nsLookAndFeel::GetPasswordCharacterImpl()
 {
     return sInvisibleCharacter;
