@@ -6,9 +6,9 @@
 
 /* API for getting a stack trace of the C/C++ stack on the current thread */
 
-#include "mozilla/Util.h"
+#include "mozilla/Assertions.h"
+#include "mozilla/IntegerPrintfMacros.h"
 #include "mozilla/StackWalk.h"
-#include "nsDebug.h"
 #include "nsStackWalkPrivate.h"
 
 #include "nsStackWalk.h"
@@ -16,13 +16,19 @@
 using namespace mozilla;
 
 // The presence of this address is the stack must stop the stack walk. If
-// there is no such address, the structure will be {NULL, true}.
+// there is no such address, the structure will be {nullptr, true}.
 struct CriticalAddress {
   void* mAddr;
   bool mInit;
 };
 static CriticalAddress gCriticalAddress;
 
+// for _Unwind_Backtrace from libcxxrt or libunwind
+// cxxabi.h from libcxxrt implicitly includes unwind.h first
+#if defined(HAVE__UNWIND_BACKTRACE) && !defined(_GNU_SOURCE)
+#define _GNU_SOURCE
+#endif
+
 #if defined(HAVE_DLOPEN) || defined(XP_MACOSX)
 #include <dlfcn.h>
 #endif
@@ -42,7 +48,6 @@ static CriticalAddress gCriticalAddress;
 
 #if NSSTACKWALK_SUPPORTS_MACOSX
 #include <pthread.h>
-#include <errno.h>
 #include <CoreServices/CoreServices.h>
 
 typedef void
@@ -61,13 +66,13 @@ stack_callback(void *pc, void *sp, void *closure)
   // correct one is the first that we find on our way up, so the
   // following check for gCriticalAddress.mAddr is critical.
   if (gCriticalAddress.mAddr || dladdr(pc, &info) == 0  ||
-      info.dli_sname == NULL || strcmp(info.dli_sname, name) != 0)
+      info.dli_sname == nullptr || strcmp(info.dli_sname, name) != 0)
     return;
   gCriticalAddress.mAddr = pc;
 }
 
+#ifdef DEBUG
 #define MAC_OS_X_VERSION_10_7_HEX 0x00001070
-#define MAC_OS_X_VERSION_10_6_HEX 0x00001060
 
 static int32_t OSXVersion()
 {
@@ -83,11 +88,7 @@ static bool OnLionOrLater()
 {
   return (OSXVersion() >= MAC_OS_X_VERSION_10_7_HEX);
 }
-
-static bool OnSnowLeopardOrLater()
-{
-  return (OSXVersion() >= MAC_OS_X_VERSION_10_6_HEX);
-}
+#endif
 
 static void
 my_malloc_logger(uint32_t type, uintptr_t arg1, uintptr_t arg2, uintptr_t arg3,
@@ -100,9 +101,9 @@ my_malloc_logger(uint32_t type, uintptr_t arg1, uintptr_t arg2, uintptr_t arg3,
 
   // On Leopard dladdr returns the wrong value for "new_sem_from_pool". The
   // stack shows up as having two pthread_cond_wait$UNIX2003 frames.
-  const char *name = OnSnowLeopardOrLater() ? "new_sem_from_pool" :
-    "pthread_cond_wait$UNIX2003";
-  NS_StackWalk(stack_callback, 0, const_cast<char*>(name), 0);
+  const char *name = "new_sem_from_pool";
+  NS_StackWalk(stack_callback, /* skipFrames */ 0, /* maxFrames */ 0,
+               const_cast<char*>(name), 0, nullptr);
 }
 
 // This is called from NS_LogInit() and from the stack walking functions, but
@@ -145,7 +146,7 @@ StackWalkInitCriticalAddress()
   // On Lion, malloc is no longer called from pthread_cond_*wait*. This prevents
   // us from finding the address, but that is fine, since with no call to malloc
   // there is no critical address.
-  MOZ_ASSERT(OnLionOrLater() || gCriticalAddress.mAddr != NULL);
+  MOZ_ASSERT(OnLionOrLater() || gCriticalAddress.mAddr != nullptr);
   MOZ_ASSERT(r == ETIMEDOUT);
   r = pthread_mutex_unlock(&mutex);
   MOZ_ASSERT(r == 0);
@@ -182,6 +183,7 @@ StackWalkInitCriticalAddress()
 #include <stdio.h>
 #include <malloc.h>
 #include "plstr.h"
+#include "mozilla/ArrayUtils.h"
 
 #include "nspr.h"
 #include <imagehlp.h>
@@ -204,7 +206,7 @@ extern HANDLE hStackWalkMutex;
 
 bool EnsureSymInitialized();
 
-bool EnsureImageHlpInitialized();
+bool EnsureWalkThreadReady();
 
 struct WalkStackData {
   uint32_t skipFrames;
@@ -216,9 +218,11 @@ struct WalkStackData {
   void **pcs;
   uint32_t pc_size;
   uint32_t pc_count;
+  uint32_t pc_max;
   void **sps;
   uint32_t sp_size;
   uint32_t sp_count;
+  void *platformData;
 };
 
 void PrintError(char *prefix, WalkStackData* data);
@@ -232,19 +236,18 @@ CRITICAL_SECTION gDbgHelpCS;
 }
 
 // Routine to print an error message to standard error.
-// Will also call callback with error, if data supplied.
-void PrintError(char *prefix)
+void PrintError(const char *prefix)
 {
     LPVOID lpMsgBuf;
     DWORD lastErr = GetLastError();
     FormatMessageA(
       FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS,
-      NULL,
+      nullptr,
       lastErr,
       MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
       (LPSTR) &lpMsgBuf,
       0,
-      NULL
+      nullptr
     );
     fprintf(stderr, "### ERROR: %s: %s",
                     prefix, lpMsgBuf ? lpMsgBuf : "(null)\n");
@@ -253,38 +256,59 @@ void PrintError(char *prefix)
 }
 
 bool
-EnsureImageHlpInitialized()
+EnsureWalkThreadReady()
 {
-    static bool gInitialized = false;
+    static bool walkThreadReady = false;
+    static HANDLE stackWalkThread = nullptr;
+    static HANDLE readyEvent = nullptr;
 
-    if (gInitialized)
-        return gInitialized;
+    if (walkThreadReady)
+        return walkThreadReady;
 
-    // Hope that our first call doesn't happen during static
-    // initialization.  If it does, this CreateThread call won't
-    // actually start the thread until after the static initialization
-    // is done, which means we'll deadlock while waiting for it to
-    // process a stack.
-    HANDLE readyEvent = ::CreateEvent(NULL, FALSE /* auto-reset*/,
-                            FALSE /* initially non-signaled */, NULL);
-    unsigned int threadID;
-    HANDLE hStackWalkThread = (HANDLE)
-      _beginthreadex(NULL, 0, WalkStackThread, (void*)readyEvent,
-                     0, &threadID);
-    gStackWalkThread = threadID;
-    if (hStackWalkThread == NULL) {
-        PrintError("CreateThread");
+    if (stackWalkThread == nullptr) {
+        readyEvent = ::CreateEvent(nullptr, FALSE /* auto-reset*/,
+                                   FALSE /* initially non-signaled */,
+                                   nullptr);
+        if (readyEvent == nullptr) {
+            PrintError("CreateEvent");
+            return false;
+        }
+
+        unsigned int threadID;
+        stackWalkThread = (HANDLE)
+            _beginthreadex(nullptr, 0, WalkStackThread, (void*)readyEvent,
+                           0, &threadID);
+        if (stackWalkThread == nullptr) {
+            PrintError("CreateThread");
+            ::CloseHandle(readyEvent);
+            readyEvent = nullptr;
+            return false;
+        }
+        gStackWalkThread = threadID;
+        ::CloseHandle(stackWalkThread);
+    }
+
+    MOZ_ASSERT((stackWalkThread != nullptr && readyEvent != nullptr) ||
+               (stackWalkThread == nullptr && readyEvent == nullptr));
+
+    // The thread was created. Try to wait an arbitrary amount of time (1 second
+    // should be enough) for its event loop to start before posting events to it.
+    DWORD waitRet = ::WaitForSingleObject(readyEvent, 1000);
+    if (waitRet == WAIT_TIMEOUT) {
+        // We get a timeout if we're called during static initialization because
+        // the thread will only start executing after we return so it couldn't
+        // have signalled the event. If that is the case, give up for now and
+        // try again next time we're called.
         return false;
     }
-    ::CloseHandle(hStackWalkThread);
-
-    // Wait for the thread's event loop to start before posting events to it.
-    ::WaitForSingleObject(readyEvent, INFINITE);
     ::CloseHandle(readyEvent);
+    stackWalkThread = nullptr;
+    readyEvent = nullptr;
+
 
     ::InitializeCriticalSection(&gDbgHelpCS);
 
-    return gInitialized = true;
+    return walkThreadReady = true;
 }
 
 void
@@ -304,13 +328,17 @@ WalkStackMain64(struct WalkStackData* data)
     BOOL ok;
 
     // Get a context for the specified thread.
-    memset(&context, 0, sizeof(CONTEXT));
-    context.ContextFlags = CONTEXT_FULL;
-    if (!GetThreadContext(myThread, &context)) {
-        if (data->walkCallingThread) {
-            PrintError("GetThreadContext");
+    if (!data->platformData) {
+        memset(&context, 0, sizeof(CONTEXT));
+        context.ContextFlags = CONTEXT_FULL;
+        if (!GetThreadContext(myThread, &context)) {
+            if (data->walkCallingThread) {
+                PrintError("GetThreadContext");
+            }
+            return;
         }
-        return;
+    } else {
+        context = *static_cast<CONTEXT*>(data->platformData);
     }
 
     // Setup initial stack frame to walk from
@@ -354,7 +382,7 @@ WalkStackMain64(struct WalkStackData* data)
           myThread,
           &frame64,
           &context,
-          NULL,
+          nullptr,
           SymFunctionTableAccess64, // function table access routine
           SymGetModuleBase64,       // module base routine
           0
@@ -388,6 +416,9 @@ WalkStackMain64(struct WalkStackData* data)
             data->sps[data->sp_count] = (void*)spaddr;
         ++data->sp_count;
 
+        if (data->pc_max != 0 && data->pc_count == data->pc_max)
+            break;
+
         if (frame64.AddrReturn.Offset == 0)
             break;
     }
@@ -403,7 +434,7 @@ WalkStackThread(void* aData)
 
     // Call PeekMessage to force creation of a message queue so that
     // other threads can safely post events to us.
-    ::PeekMessage(&msg, NULL, WM_USER, WM_USER, PM_NOREMOVE);
+    ::PeekMessage(&msg, nullptr, WM_USER, WM_USER, PM_NOREMOVE);
 
     // and tell the thread that created us that we're ready.
     HANDLE readyEvent = (HANDLE)aData;
@@ -457,16 +488,17 @@ WalkStackThread(void* aData)
 
 EXPORT_XPCOM_API(nsresult)
 NS_StackWalk(NS_WalkStackCallback aCallback, uint32_t aSkipFrames,
-             void *aClosure, uintptr_t aThread)
+             uint32_t aMaxFrames, void *aClosure, uintptr_t aThread,
+             void *aPlatformData)
 {
     StackWalkInitCriticalAddress();
-    static HANDLE myProcess = NULL;
+    static HANDLE myProcess = nullptr;
     HANDLE myThread;
     DWORD walkerReturn;
     struct WalkStackData data;
 
-    if (!EnsureImageHlpInitialized())
-        return NS_OK;
+    if (!EnsureWalkThreadReady())
+        return NS_ERROR_FAILURE;
 
     HANDLE targetThread = ::GetCurrentThread();
     data.walkCallingThread = true;
@@ -512,10 +544,12 @@ NS_StackWalk(NS_WalkStackCallback aCallback, uint32_t aSkipFrames,
     data.pcs = local_pcs;
     data.pc_count = 0;
     data.pc_size = ArrayLength(local_pcs);
+    data.pc_max = aMaxFrames;
     void *local_sps[1024];
     data.sps = local_sps;
     data.sp_count = 0;
-    data.sp_size = ArrayLength(local_pcs);
+    data.sp_size = ArrayLength(local_sps);
+    data.platformData = aPlatformData;
 
     if (aThread) {
         // If we're walking the stack of another thread, we don't need to
@@ -532,10 +566,10 @@ NS_StackWalk(NS_WalkStackCallback aCallback, uint32_t aSkipFrames,
             WalkStackMain64(&data);
         }
     } else {
-        data.eventStart = ::CreateEvent(NULL, FALSE /* auto-reset*/,
-                              FALSE /* initially non-signaled */, NULL);
-        data.eventEnd = ::CreateEvent(NULL, FALSE /* auto-reset*/,
-                            FALSE /* initially non-signaled */, NULL);
+        data.eventStart = ::CreateEvent(nullptr, FALSE /* auto-reset*/,
+                              FALSE /* initially non-signaled */, nullptr);
+        data.eventEnd = ::CreateEvent(nullptr, FALSE /* auto-reset*/,
+                            FALSE /* initially non-signaled */, nullptr);
 
         ::PostThreadMessage(gStackWalkThread, WM_USER, 0, (LPARAM)&data);
 
@@ -566,7 +600,7 @@ NS_StackWalk(NS_WalkStackCallback aCallback, uint32_t aSkipFrames,
     for (uint32_t i = 0; i < data.pc_count; ++i)
         (*aCallback)(data.pcs[i], data.sps[i], aClosure);
 
-    return NS_OK;
+    return data.pc_count == 0 ? NS_ERROR_FAILURE : NS_OK;
 }
 
 
@@ -593,7 +627,9 @@ static BOOL CALLBACK callbackEspecial64(
        ? (addr >= aModuleBase && addr <= (aModuleBase + aModuleSize))
        : (addr <= aModuleBase && addr >= (aModuleBase - aModuleSize))
         ) {
-        retval = SymLoadModule64(GetCurrentProcess(), NULL, (PSTR)aModuleName, NULL, aModuleBase, aModuleSize);
+        retval = !!SymLoadModule64(GetCurrentProcess(), nullptr,
+                                   (PSTR)aModuleName, nullptr,
+                                   aModuleBase, aModuleSize);
         if (!retval)
             PrintError("SymLoadModule64");
     }
@@ -693,11 +729,11 @@ EnsureSymInitialized()
     if (gInitialized)
         return gInitialized;
 
-    if (!EnsureImageHlpInitialized())
+    if (!EnsureWalkThreadReady())
         return false;
 
     SymSetOptions(SYMOPT_LOAD_LINES | SYMOPT_UNDNAME);
-    retStat = SymInitialize(GetCurrentProcess(), NULL, TRUE);
+    retStat = SymInitialize(GetCurrentProcess(), nullptr, TRUE);
     if (!retStat)
         PrintError("SymInitialize");
 
@@ -762,7 +798,7 @@ NS_DescribeCodeAddress(void *aPC, nsCodeAddressDetails *aDetails)
     if (ok) {
         PL_strncpyz(aDetails->function, pSymbol->Name,
                     sizeof(aDetails->function));
-        aDetails->foffset = displacement;
+        aDetails->foffset = static_cast<ptrdiff_t>(displacement);
     }
 
     LeaveCriticalSection(&gDbgHelpCS); // release our lock
@@ -773,11 +809,16 @@ EXPORT_XPCOM_API(nsresult)
 NS_FormatCodeAddressDetails(void *aPC, const nsCodeAddressDetails *aDetails,
                             char *aBuffer, uint32_t aBufferSize)
 {
-    if (aDetails->function[0])
-        _snprintf(aBuffer, aBufferSize, "%s!%s+0x%016lX",
-                  aDetails->library, aDetails->function, aDetails->foffset);
-    else
-        _snprintf(aBuffer, aBufferSize, "0x%016lX", aPC);
+    if (aDetails->function[0]) {
+        _snprintf(aBuffer, aBufferSize, "%s+0x%08lX [%s +0x%016lX]",
+                  aDetails->function, aDetails->foffset,
+                  aDetails->library, aDetails->loffset);
+    } else if (aDetails->library[0]) {
+        _snprintf(aBuffer, aBufferSize, "UNKNOWN [%s +0x%016lX]",
+                  aDetails->library, aDetails->loffset);
+    } else {
+        _snprintf(aBuffer, aBufferSize, "UNKNOWN 0x%016lX", aPC);
+    }
 
     aBuffer[aBufferSize - 1] = '\0';
 
@@ -801,7 +842,6 @@ NS_FormatCodeAddressDetails(void *aPC, const nsCodeAddressDetails *aDetails,
 
 #include <stdlib.h>
 #include <string.h>
-#include <math.h>
 #include "nscore.h"
 #include <stdio.h>
 #include "plstr.h"
@@ -818,7 +858,6 @@ NS_FormatCodeAddressDetails(void *aPC, const nsCodeAddressDetails *aDetails,
 // Yes, this is a gcc only hack
 #if defined(MOZ_DEMANGLE_SYMBOLS)
 #include <cxxabi.h>
-#include <stdlib.h> // for free()
 #endif // MOZ_DEMANGLE_SYMBOLS
 
 void DemangleSymbol(const char * aSymbol, 
@@ -892,6 +931,8 @@ struct bucket {
 struct my_user_args {
     NS_WalkStackCallback callback;
     uint32_t skipFrames;
+    uint32_t maxFrames;
+    uint32_t numFrames;
     void *closure;
 };
 
@@ -910,7 +951,7 @@ myinit()
         const char *libdem = "libdemangle.so.1";
 
         /* load libdemangle if we can and need to (only try this once) */
-        if ((handle = dlopen(libdem, RTLD_LAZY)) != NULL) {
+        if ((handle = dlopen(libdem, RTLD_LAZY)) != nullptr) {
             demf = (demf_t *)dlsym(handle,
                            "cplus_demangle"); /*lint !e611 */
                 /*
@@ -925,7 +966,7 @@ myinit()
 
 
 static int
-load_address(void * pc, void * arg )
+load_address(void * pc, void * arg)
 {
     static struct bucket table[2048];
     static mutex_t lock;
@@ -943,15 +984,19 @@ load_address(void * pc, void * arg )
         ptr = ptr->next;
     }
 
+    int stop = 0;
     if (ptr->next) {
         mutex_unlock(&lock);
     } else {
         (args->callback)(pc, args->closure);
+        args->numFrames++;
+        if (args->maxFrames != 0 && args->numFrames == args->maxFrames)
+          stop = 1;   // causes us to stop getting frames
 
         ptr->next = newbucket(pc);
         mutex_unlock(&lock);
     }
-    return 0;
+    return stop;
 }
 
 
@@ -962,7 +1007,7 @@ newbucket(void * pc)
     static int index; /* protected by lock in caller */
                      
     ptr->index = index++;
-    ptr->next = NULL;
+    ptr->next = nullptr;
     ptr->pc = pc;    
     return (ptr);    
 }
@@ -993,7 +1038,7 @@ cswalkstack(struct frame *fp, int (*operate_func)(void *, void *, void *),
 
     while (fp != 0 && fp->fr_savpc != 0) {
 
-        if (operate_func((void *)fp->fr_savpc, NULL, usrarg) != 0)
+        if (operate_func((void *)fp->fr_savpc, nullptr, usrarg) != 0)
             break;
         /*
          * watch out - libthread stacks look funny at the top
@@ -1014,9 +1059,11 @@ cs_operate(int (*operate_func)(void *, void *, void *), void * usrarg)
 
 EXPORT_XPCOM_API(nsresult)
 NS_StackWalk(NS_WalkStackCallback aCallback, uint32_t aSkipFrames,
-             void *aClosure, uintptr_t aThread)
+             uint32_t aMaxFrames, void *aClosure, uintptr_t aThread,
+             void *aPlatformData)
 {
     MOZ_ASSERT(!aThread);
+    MOZ_ASSERT(!aPlatformData);
     struct my_user_args args;
 
     StackWalkInitCriticalAddress();
@@ -1026,9 +1073,11 @@ NS_StackWalk(NS_WalkStackCallback aCallback, uint32_t aSkipFrames,
 
     args.callback = aCallback;
     args.skipFrames = aSkipFrames; /* XXX Not handled! */
+    args.maxFrames = aMaxFrames;
+    args.numFrames = 0;
     args.closure = aClosure;
     cs_operate(load_address, &args);
-    return NS_OK;
+    return args.numFrames == 0 ? NS_ERROR_FAILURE : NS_OK;
 }
 
 EXPORT_XPCOM_API(nsresult)
@@ -1093,11 +1142,13 @@ extern void *__libc_stack_end; // from ld-linux.so
 namespace mozilla {
 nsresult
 FramePointerStackWalk(NS_WalkStackCallback aCallback, uint32_t aSkipFrames,
-                      void *aClosure, void **bp, void *aStackEnd)
+                      uint32_t aMaxFrames, void *aClosure, void **bp,
+                      void *aStackEnd)
 {
   // Stack walking code courtesy Kipp's "leaky".
 
-  int skip = aSkipFrames;
+  int32_t skip = aSkipFrames;
+  uint32_t numFrames = 0;
   while (1) {
     void **next = (void**)*bp;
     // bp may not be a frame pointer on i386 if code was compiled with
@@ -1129,10 +1180,13 @@ FramePointerStackWalk(NS_WalkStackCallback aCallback, uint32_t aSkipFrames,
       // but this should be sufficient for our use the SP
       // to order elements on the stack.
       (*aCallback)(pc, bp, aClosure);
+      numFrames++;
+      if (aMaxFrames != 0 && numFrames == aMaxFrames)
+        break;
     }
     bp = next;
   }
-  return NS_OK;
+  return numFrames == 0 ? NS_ERROR_FAILURE : NS_OK;
 }
 
 }
@@ -1142,9 +1196,11 @@ FramePointerStackWalk(NS_WalkStackCallback aCallback, uint32_t aSkipFrames,
 
 EXPORT_XPCOM_API(nsresult)
 NS_StackWalk(NS_WalkStackCallback aCallback, uint32_t aSkipFrames,
-             void *aClosure, uintptr_t aThread)
+             uint32_t aMaxFrames, void *aClosure, uintptr_t aThread,
+             void *aPlatformData)
 {
   MOZ_ASSERT(!aThread);
+  MOZ_ASSERT(!aPlatformData);
   StackWalkInitCriticalAddress();
 
   // Get the frame pointer
@@ -1164,7 +1220,7 @@ NS_StackWalk(NS_WalkStackCallback aCallback, uint32_t aSkipFrames,
 #else
   stackEnd = reinterpret_cast<void*>(-1);
 #endif
-  return FramePointerStackWalk(aCallback, aSkipFrames,
+  return FramePointerStackWalk(aCallback, aSkipFrames, aMaxFrames,
                                aClosure, bp, stackEnd);
 
 }
@@ -1172,14 +1228,14 @@ NS_StackWalk(NS_WalkStackCallback aCallback, uint32_t aSkipFrames,
 #elif defined(HAVE__UNWIND_BACKTRACE)
 
 // libgcc_s.so symbols _Unwind_Backtrace@@GCC_3.3 and _Unwind_GetIP@@GCC_3.0
-#ifndef _GNU_SOURCE
-#define _GNU_SOURCE
-#endif
 #include <unwind.h>
 
 struct unwind_info {
     NS_WalkStackCallback callback;
     int skip;
+    int maxFrames;
+    int numFrames;
+    bool isCriticalAbort;
     void *closure;
 };
 
@@ -1191,40 +1247,52 @@ unwind_callback (struct _Unwind_Context *context, void *closure)
     // TODO Use something like '_Unwind_GetGR()' to get the stack pointer.
     if (IsCriticalAddress(pc)) {
         printf("Aborting stack trace, PC is critical\n");
-        /* We just want to stop the walk, so any error code will do.
-           Using _URC_NORMAL_STOP would probably be the most accurate,
-           but it is not defined on Android for ARM. */
+        info->isCriticalAbort = true;
+        // We just want to stop the walk, so any error code will do.  Using
+        // _URC_NORMAL_STOP would probably be the most accurate, but it is not
+        // defined on Android for ARM.
         return _URC_FOREIGN_EXCEPTION_CAUGHT;
     }
-    if (--info->skip < 0)
-        (*info->callback)(pc, NULL, info->closure);
+    if (--info->skip < 0) {
+        (*info->callback)(pc, nullptr, info->closure);
+        info->numFrames++;
+        if (info->maxFrames != 0 && info->numFrames == info->maxFrames) {
+            // Again, any error code that stops the walk will do.
+            return _URC_FOREIGN_EXCEPTION_CAUGHT;
+        }
+    }
     return _URC_NO_REASON;
 }
 
 EXPORT_XPCOM_API(nsresult)
 NS_StackWalk(NS_WalkStackCallback aCallback, uint32_t aSkipFrames,
-             void *aClosure, uintptr_t aThread)
+             uint32_t aMaxFrames, void *aClosure, uintptr_t aThread,
+             void *aPlatformData)
 {
     MOZ_ASSERT(!aThread);
+    MOZ_ASSERT(!aPlatformData);
     StackWalkInitCriticalAddress();
     unwind_info info;
     info.callback = aCallback;
     info.skip = aSkipFrames + 1;
+    info.maxFrames = aMaxFrames;
+    info.numFrames = 0;
+    info.isCriticalAbort = false;
     info.closure = aClosure;
 
-    _Unwind_Reason_Code t = _Unwind_Backtrace(unwind_callback, &info);
-#if defined(ANDROID) && defined(__arm__)
-    // Ignore the _Unwind_Reason_Code on Android + ARM, because bionic's
-    // _Unwind_Backtrace usually (always?) returns _URC_FAILURE.  See
-    // https://bugzilla.mozilla.org/show_bug.cgi?id=717853#c110.
-    //
-    // (Ideally, the #if above would be specifically for bionic, not for
-    // Android + ARM, but we don't have a define specifically for bionic.)
-#else
-    if (t != _URC_END_OF_STACK)
+    (void)_Unwind_Backtrace(unwind_callback, &info);
+
+    // We ignore the return value from _Unwind_Backtrace and instead determine
+    // the outcome from |info|.  There are two main reasons for this:
+    // - On ARM/Android bionic's _Unwind_Backtrace usually (always?) returns
+    //   _URC_FAILURE.  See
+    //   https://bugzilla.mozilla.org/show_bug.cgi?id=717853#c110.
+    // - If aMaxFrames != 0, we want to stop early, and the only way to do that
+    //   is to make unwind_callback return something other than _URC_NO_REASON,
+    //   which causes _Unwind_Backtrace to return a non-success code.
+    if (info.isCriticalAbort)
         return NS_ERROR_UNEXPECTED;
-#endif
-    return NS_OK;
+    return info.numFrames == 0 ? NS_ERROR_FAILURE : NS_OK;
 }
 
 #endif
@@ -1275,10 +1343,11 @@ NS_FormatCodeAddressDetails(void *aPC, const nsCodeAddressDetails *aDetails,
   if (!aDetails->library[0]) {
     snprintf(aBuffer, aBufferSize, "UNKNOWN %p\n", aPC);
   } else if (!aDetails->function[0]) {
-    snprintf(aBuffer, aBufferSize, "UNKNOWN [%s +0x%08lX]\n",
+    snprintf(aBuffer, aBufferSize, "UNKNOWN [%s +0x%08" PRIXPTR "]\n",
                                    aDetails->library, aDetails->loffset);
   } else {
-    snprintf(aBuffer, aBufferSize, "%s+0x%08lX [%s +0x%08lX]\n",
+    snprintf(aBuffer, aBufferSize, "%s+0x%08" PRIXPTR
+                                   " [%s +0x%08" PRIXPTR "]\n",
                                    aDetails->function, aDetails->foffset,
                                    aDetails->library, aDetails->loffset);
   }
@@ -1291,9 +1360,11 @@ NS_FormatCodeAddressDetails(void *aPC, const nsCodeAddressDetails *aDetails,
 
 EXPORT_XPCOM_API(nsresult)
 NS_StackWalk(NS_WalkStackCallback aCallback, uint32_t aSkipFrames,
-             void *aClosure, uintptr_t aThread)
+             uint32_t aMaxFrames, void *aClosure, uintptr_t aThread,
+             void *aPlatformData)
 {
     MOZ_ASSERT(!aThread);
+    MOZ_ASSERT(!aPlatformData);
     return NS_ERROR_NOT_IMPLEMENTED;
 }
 
