@@ -40,7 +40,6 @@
 #include "mozilla/MemoryReporting.h"
 #include "mozilla/XPTInterfaceInfoManager.h"
 #include "nsIConsoleService.h"
-#include "nsIMemoryReporter.h"
 #include "nsIObserverService.h"
 #include "nsISimpleEnumerator.h"
 #include "nsIStringEnumerator.h"
@@ -65,9 +64,12 @@
 #include "nsManifestLineReader.h"
 #include "mozilla/GenericFactory.h"
 #include "nsSupportsPrimitives.h"
+#include "nsArray.h"
+#include "nsIMutableArray.h"
 #include "nsArrayEnumerator.h"
 #include "nsStringEnumerator.h"
 #include "mozilla/FileUtils.h"
+#include "nsNetUtil.h"
 
 #include <new>     // for placement new
 
@@ -94,43 +96,15 @@ extern mozilla::Module const *const *const kPStaticModules[];
 #define BIG_REGISTRY_BUFLEN   (512*1024)
 
 // Common Key Names
-const char classIDKeyName[]="classID";
-const char classesKeyName[]="contractID";
-const char componentsKeyName[]="components";
 const char xpcomComponentsKeyName[]="software/mozilla/XPCOM/components";
 const char xpcomKeyName[]="software/mozilla/XPCOM";
 
 // Common Value Names
-const char classIDValueName[]="ClassID";
-const char classNameValueName[]="ClassName";
-const char componentCountValueName[]="ComponentsCount";
-const char componentTypeValueName[]="ComponentType";
-const char contractIDValueName[]="ContractID";
 const char fileSizeValueName[]="FileSize";
-const char inprocServerValueName[]="InprocServer";
 const char lastModValueName[]="LastModTimeStamp";
 const char nativeComponentType[]="application/x-mozilla-native";
 const char staticComponentType[]="application/x-mozilla-static";
-const char jarComponentType[]="application/x-mozilla-jarjs";
-const char versionValueName[]="VersionString";
 
-const static char XPCOM_ABSCOMPONENT_PREFIX[] = "abs:";
-const static char XPCOM_RELCOMPONENT_PREFIX[] = "rel:";
-const static char XPCOM_GRECOMPONENT_PREFIX[] = "gre:";
-
-static const char gIDFormat[] =
-  "{%08x-%04x-%04x-%02x%02x-%02x%02x%02x%02x%02x%02x}";
-
-
-#define NS_EMPTY_IID                                 \
-{                                                    \
-    0x00000000,                                      \
-    0x0000,                                          \
-    0x0000,                                          \
-    {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00} \
-}
-
-NS_DEFINE_CID(kEmptyCID, NS_EMPTY_IID);
 NS_DEFINE_CID(kCategoryManagerCID, NS_CATEGORYMANAGER_CID);
 
 #define UID_STRING_LENGTH 39
@@ -280,24 +254,6 @@ CloneAndAppend(nsIFile* aBase, const nsACString& append)
 // nsComponentManagerImpl
 ////////////////////////////////////////////////////////////////////////////////
 
-class XPCOMComponentManagerReporter MOZ_FINAL : public MemoryUniReporter
-{
-public:
-    XPCOMComponentManagerReporter()
-      : MemoryUniReporter("explicit/xpcom/component-manager",
-                           KIND_HEAP, UNITS_BYTES,
-                           "Memory used for the XPCOM component manager.")
-    {}
-private:
-    int64_t Amount() MOZ_OVERRIDE
-    {
-        return nsComponentManagerImpl::gComponentManager
-             ? nsComponentManagerImpl::gComponentManager->SizeOfIncludingThis(
-                 MallocSizeOf)
-             : 0;
-    }
-};
-
 nsresult
 nsComponentManagerImpl::Create(nsISupports* aOuter, REFNSIID aIID, void** aResult)
 {
@@ -411,8 +367,7 @@ nsresult nsComponentManagerImpl::Init()
 
     nsCategoryManager::GetSingleton()->SuppressNotifications(false);
 
-    mReporter = new XPCOMComponentManagerReporter();
-    NS_RegisterMemoryReporter(mReporter);
+    RegisterWeakMemoryReporter(this);
 
     // Unfortunately, we can't register the nsCategoryManager memory reporter
     // in its constructor (which is triggered by the GetSingleton() call
@@ -474,6 +429,24 @@ nsComponentManagerImpl::RegisterModule(const mozilla::Module* aModule,
     }
 }
 
+static bool
+ProcessSelectorMatches(Module::ProcessSelector selector)
+{
+    if (selector == Module::ANY_PROCESS) {
+        return true;
+    }
+
+    GeckoProcessType type = XRE_GetProcessType();
+    switch (selector) {
+      case Module::MAIN_PROCESS_ONLY:
+        return type == GeckoProcessType_Default;
+      case Module::CONTENT_PROCESS_ONLY:
+        return type == GeckoProcessType_Content;
+      default:
+        MOZ_CRASH("invalid process selector");
+    }
+}
+
 void
 nsComponentManagerImpl::RegisterCIDEntryLocked(
     const mozilla::Module::CIDEntry* aEntry,
@@ -481,6 +454,10 @@ nsComponentManagerImpl::RegisterCIDEntryLocked(
 {
     mLock.AssertCurrentThreadOwns();
 
+    if (!ProcessSelectorMatches(aEntry->processSelector)) {
+        return;
+    }
+
     nsFactoryEntry* f = mFactories.Get(*aEntry->cid);
     if (f) {
         NS_WARNING("Re-registering a CID?");
@@ -511,6 +488,10 @@ nsComponentManagerImpl::RegisterContractIDLocked(
 {
     mLock.AssertCurrentThreadOwns();
 
+    if (!ProcessSelectorMatches(aEntry->processSelector)) {
+        return;
+    }
+
     nsFactoryEntry* f = mFactories.Get(*aEntry->cid);
     if (!f) {
         NS_ERROR("No CID found when attempting to map contract ID");
@@ -797,8 +778,7 @@ nsresult nsComponentManagerImpl::Shutdown(void)
     // Shutdown the component manager
     PR_LOG(nsComponentManagerLog, PR_LOG_DEBUG, ("nsComponentManager: Beginning Shutdown."));
 
-    NS_UnregisterMemoryReporter(mReporter);
-    mReporter = nullptr;
+    UnregisterWeakMemoryReporter(this);
 
     // Release all cached factories
     mContractIDs.Clear();
@@ -833,13 +813,14 @@ nsComponentManagerImpl::~nsComponentManagerImpl()
     PR_LOG(nsComponentManagerLog, PR_LOG_DEBUG, ("nsComponentManager: Destroyed."));
 }
 
-NS_IMPL_ISUPPORTS5(nsComponentManagerImpl,
-                   nsIComponentManager,
-                   nsIServiceManager,
-                   nsIComponentRegistrar,
-                   nsISupportsWeakReference,
-                   nsIInterfaceRequestor)
-
+NS_IMPL_ISUPPORTS6(
+    nsComponentManagerImpl,
+    nsIComponentManager,
+    nsIServiceManager,
+    nsIComponentRegistrar,
+    nsISupportsWeakReference,
+    nsIInterfaceRequestor,
+    nsIMemoryReporter)
 
 nsresult
 nsComponentManagerImpl::GetInterface(const nsIID & uuid, void **result)
@@ -929,8 +910,9 @@ nsComponentManagerImpl::GetClassObjectByContractID(const char *contractID,
                                                    const nsIID &aIID,
                                                    void **aResult)
 {
-    NS_ENSURE_ARG_POINTER(aResult);
-    NS_ENSURE_ARG_POINTER(contractID);
+    if (NS_WARN_IF(!aResult) ||
+        NS_WARN_IF(!contractID))
+        return NS_ERROR_INVALID_ARG;
 
     nsresult rv;
 
@@ -1049,7 +1031,8 @@ nsComponentManagerImpl::CreateInstanceByContractID(const char *aContractID,
                                                    const nsIID &aIID,
                                                    void **aResult)
 {
-    NS_ENSURE_ARG_POINTER(aContractID);
+    if (NS_WARN_IF(!aContractID))
+        return NS_ERROR_INVALID_ARG;
 
     // test this first, since there's no point in creating a component during
     // shutdown -- whether it's available or not would depend on the order it
@@ -1604,7 +1587,9 @@ NS_IMETHODIMP
 nsComponentManagerImpl::IsContractIDRegistered(const char *aClass,
                                                bool *_retval)
 {
-    NS_ENSURE_ARG_POINTER(aClass);
+    if (NS_WARN_IF(!aClass))
+        return NS_ERROR_INVALID_ARG;
+
     nsFactoryEntry *entry = GetFactoryEntry(aClass, strlen(aClass));
 
     if (entry)
@@ -1700,8 +1685,20 @@ SizeOfContractIDsEntryExcludingThis(nsCStringHashKey::KeyType aKey,
     return aKey.SizeOfExcludingThisMustBeUnshared(aMallocSizeOf);
 }
 
+MOZ_DEFINE_MALLOC_SIZE_OF(ComponentManagerMallocSizeOf)
+
+NS_IMETHODIMP
+nsComponentManagerImpl::CollectReports(nsIHandleReportCallback* aHandleReport,
+                                       nsISupports* aData)
+{
+    return MOZ_COLLECT_REPORT(
+        "explicit/xpcom/component-manager", KIND_HEAP, UNITS_BYTES,
+        SizeOfIncludingThis(ComponentManagerMallocSizeOf),
+        "Memory used for the XPCOM component manager.");
+}
+
 size_t
-nsComponentManagerImpl::SizeOfIncludingThis(MallocSizeOf aMallocSizeOf)
+nsComponentManagerImpl::SizeOfIncludingThis(mozilla::MallocSizeOf aMallocSizeOf)
 {
     size_t n = aMallocSizeOf(this);
     n += mLoaderMap.SizeOfExcludingThis(nullptr, aMallocSizeOf);
@@ -1914,6 +1911,32 @@ nsComponentManagerImpl::RemoveBootstrappedManifestLocation(nsIFile* aLocation)
   return rv;
 }
 
+NS_IMETHODIMP
+nsComponentManagerImpl::GetManifestLocations(nsIArray **aLocations)
+{
+  NS_ENSURE_ARG_POINTER(aLocations);
+  *aLocations = nullptr;
+
+  if (!sModuleLocations)
+    return NS_ERROR_NOT_INITIALIZED;
+
+  nsCOMPtr<nsIMutableArray> locations = nsArray::Create();
+  nsresult rv;
+  for (uint32_t i = 0; i < sModuleLocations->Length(); ++i) {
+    ComponentLocation& l = sModuleLocations->ElementAt(i);
+    FileLocation loc = l.location;
+    nsCString uriString;
+    loc.GetURIString(uriString);
+    nsCOMPtr<nsIURI> uri;
+    rv = NS_NewURI(getter_AddRefs(uri), uriString);
+    if (NS_SUCCEEDED(rv))
+      locations->AppendElement(uri, false);
+  }
+
+  locations.forget(aLocations);
+  return NS_OK;
+}
+
 EXPORT_XPCOM_API(nsresult)
 XRE_AddManifestLocation(NSLocationType aType, nsIFile* aLocation)
 {
