@@ -296,6 +296,7 @@ template <class T, uint32_t K> class nsExpirationTracker {
         }
       }
       void Destroy() {
+        mOwner = nullptr;
         nsCOMPtr<nsIObserverService> obs = mozilla::services::GetObserverService();
         if (obs)
           obs->RemoveObserver(this, "memory-pressure");
@@ -332,9 +333,9 @@ template<class T, uint32_t K>
 NS_IMETHODIMP
 nsExpirationTracker<T, K>::ExpirationTrackerObserver::Observe(nsISupports     *aSubject,
                                                               const char      *aTopic,
-                                                              const PRUnichar *aData)
+                                                              const char16_t *aData)
 {
-  if (!strcmp(aTopic, "memory-pressure"))
+  if (!strcmp(aTopic, "memory-pressure") && mOwner)
     mOwner->AgeAllGenerations();
   return NS_OK;
 }
