@@ -8,6 +8,7 @@
 #include "nsCOMArray.h"
 #include "nsUnicharInputStream.h"
 #include "nsPrintfCString.h"
+#include "nsAutoPtr.h"
 
 #define PL_ARENA_CONST_ALIGN_MASK 3
 #include "nsPersistentProperties.h"
@@ -338,9 +339,9 @@ NS_METHOD nsPropertiesParser::SegmentWriter(nsIUnicharInputStream* aStream,
                                             uint32_t aCount,
                                             uint32_t *aWriteCount)
 {
-  nsPropertiesParser *parser = 
+  nsPropertiesParser *parser =
     static_cast<nsPropertiesParser *>(aClosure);
-  
+
   parser->ParseBuffer(aFromSegment, aCount);
 
   *aWriteCount = aCount;
@@ -446,7 +447,10 @@ nsPersistentProperties::nsPersistentProperties()
 : mIn(nullptr)
 {
   mSubclass = static_cast<nsIPersistentProperties*>(this);
-  mTable.ops = nullptr;
+
+  PL_DHashTableInit(&mTable, &property_HashTableOps, nullptr,
+                    sizeof(PropertyTableEntry), 20);
+
   PL_INIT_ARENA_POOL(&mArena, "PersistentPropertyArena", 2048);
 }
 
@@ -457,33 +461,13 @@ nsPersistentProperties::~nsPersistentProperties()
     PL_DHashTableFinish(&mTable);
 }
 
-nsresult
-nsPersistentProperties::Init()
-{
-  if (!PL_DHashTableInit(&mTable, &property_HashTableOps, nullptr,
-                         sizeof(PropertyTableEntry), 20)) {
-    mTable.ops = nullptr;
-    return NS_ERROR_OUT_OF_MEMORY;
-  }
-  return NS_OK;
-}
-
 nsresult
 nsPersistentProperties::Create(nsISupports *aOuter, REFNSIID aIID, void **aResult)
 {
   if (aOuter)
     return NS_ERROR_NO_AGGREGATION;
-  nsPersistentProperties* props = new nsPersistentProperties();
-  if (props == nullptr)
-    return NS_ERROR_OUT_OF_MEMORY;
-
-  NS_ADDREF(props);
-  nsresult rv = props->Init();
-  if (NS_SUCCEEDED(rv))
-    rv = props->QueryInterface(aIID, aResult);
-
-  NS_RELEASE(props);
-  return rv;
+  nsRefPtr<nsPersistentProperties> props = new nsPersistentProperties();
+  return props->QueryInterface(aIID, aResult);
 }
 
 NS_IMPL_ISUPPORTS2(nsPersistentProperties, nsIPersistentProperties, nsIProperties)
@@ -513,7 +497,7 @@ nsPersistentProperties::Load(nsIInputStream *aIn)
   // We may have an unprocessed value at this point
   // if the last line did not have a proper line ending.
   if (parser.GetState() == eParserState_Value) {
-    nsAutoString oldValue;  
+    nsAutoString oldValue;
     parser.FinishValueState(oldValue);
   }
 
@@ -662,13 +646,8 @@ nsPropertyElement::Create(nsISupports *aOuter, REFNSIID aIID, void **aResult)
 {
   if (aOuter)
     return NS_ERROR_NO_AGGREGATION;
-  nsPropertyElement* propElem = new nsPropertyElement();
-  if (propElem == nullptr)
-    return NS_ERROR_OUT_OF_MEMORY;
-  NS_ADDREF(propElem);
-  nsresult rv = propElem->QueryInterface(aIID, aResult);
-  NS_RELEASE(propElem);
-  return rv;
+  nsRefPtr<nsPropertyElement> propElem = new nsPropertyElement();
+  return propElem->QueryInterface(aIID, aResult);
 }
 
 NS_IMPL_ISUPPORTS1(nsPropertyElement, nsIPropertyElement)
@@ -702,4 +681,3 @@ nsPropertyElement::SetValue(const nsAString& aValue)
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-
