@@ -4,24 +4,21 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "nsSupportsPrimitives.h"
-#include "nsCRT.h"
 #include "nsMemory.h"
 #include "prprf.h"
-#include "nsIInterfaceInfoManager.h"
-#include "nsDependentString.h"
-#include "nsReadableUtils.h"
-#include "nsPromiseFlatString.h"
+
+using mozilla::fallible_t;
 
 /***************************************************************************/
 
 NS_IMPL_ISUPPORTS2(nsSupportsIDImpl, nsISupportsID, nsISupportsPrimitive)
 
 nsSupportsIDImpl::nsSupportsIDImpl()
-    : mData(nsnull)
+    : mData(nullptr)
 {
 }
 
-NS_IMETHODIMP nsSupportsIDImpl::GetType(PRUint16 *aType)
+NS_IMETHODIMP nsSupportsIDImpl::GetType(uint16_t *aType)
 {
     NS_ASSERTION(aType, "Bad pointer");
     *aType = TYPE_ID;
@@ -37,7 +34,7 @@ NS_IMETHODIMP nsSupportsIDImpl::GetData(nsID **aData)
         *aData = (nsID*) nsMemory::Clone(mData, sizeof(nsID));
         return *aData ? NS_OK : NS_ERROR_OUT_OF_MEMORY;
     }
-    *aData = nsnull;
+    *aData = nullptr;
     return NS_OK;
 }
 
@@ -48,7 +45,7 @@ NS_IMETHODIMP nsSupportsIDImpl::SetData(const nsID *aData)
     if(aData)
         mData = (nsID*) nsMemory::Clone(aData, sizeof(nsID));
     else
-        mData = nsnull;
+        mData = nullptr;
     return NS_OK;
 }
 
@@ -77,7 +74,7 @@ NS_IMETHODIMP nsSupportsIDImpl::ToString(char **_retval)
 NS_IMPL_ISUPPORTS2(nsSupportsCStringImpl, nsISupportsCString,
                    nsISupportsPrimitive)
 
-NS_IMETHODIMP nsSupportsCStringImpl::GetType(PRUint16 *aType)
+NS_IMETHODIMP nsSupportsCStringImpl::GetType(uint16_t *aType)
 {
     NS_ASSERTION(aType, "Bad pointer");
 
@@ -103,7 +100,9 @@ NS_IMETHODIMP nsSupportsCStringImpl::ToString(char **_retval)
 
 NS_IMETHODIMP nsSupportsCStringImpl::SetData(const nsACString& aData)
 {
-    mData = aData;
+    bool ok = mData.Assign(aData, fallible_t());
+    if (!ok)
+        return NS_ERROR_OUT_OF_MEMORY;
     return NS_OK;
 }
 
@@ -114,7 +113,7 @@ NS_IMETHODIMP nsSupportsCStringImpl::SetData(const nsACString& aData)
 NS_IMPL_ISUPPORTS2(nsSupportsStringImpl, nsISupportsString,
                    nsISupportsPrimitive)
 
-NS_IMETHODIMP nsSupportsStringImpl::GetType(PRUint16 *aType)
+NS_IMETHODIMP nsSupportsStringImpl::GetType(uint16_t *aType)
 {
     NS_ASSERTION(aType, "Bad pointer");
 
@@ -128,7 +127,7 @@ NS_IMETHODIMP nsSupportsStringImpl::GetData(nsAString& aData)
     return NS_OK;
 }
 
-NS_IMETHODIMP nsSupportsStringImpl::ToString(PRUnichar **_retval)
+NS_IMETHODIMP nsSupportsStringImpl::ToString(char16_t **_retval)
 {
     *_retval = ToNewUnicode(mData);
     
@@ -140,21 +139,23 @@ NS_IMETHODIMP nsSupportsStringImpl::ToString(PRUnichar **_retval)
 
 NS_IMETHODIMP nsSupportsStringImpl::SetData(const nsAString& aData)
 {
-    mData = aData;
+    bool ok = mData.Assign(aData, fallible_t());
+    if (!ok)
+        return NS_ERROR_OUT_OF_MEMORY;
     return NS_OK;
 }
 
 /***************************************************************************/
 
-NS_IMPL_THREADSAFE_ISUPPORTS2(nsSupportsPRBoolImpl, nsISupportsPRBool,
-                              nsISupportsPrimitive)
+NS_IMPL_ISUPPORTS2(nsSupportsPRBoolImpl, nsISupportsPRBool,
+                   nsISupportsPrimitive)
 
 nsSupportsPRBoolImpl::nsSupportsPRBoolImpl()
     : mData(false)
 {
 }
 
-NS_IMETHODIMP nsSupportsPRBoolImpl::GetType(PRUint16 *aType)
+NS_IMETHODIMP nsSupportsPRBoolImpl::GetType(uint16_t *aType)
 {
     NS_ASSERTION(aType, "Bad pointer");
     *aType = TYPE_PRBOOL;
@@ -195,7 +196,7 @@ nsSupportsPRUint8Impl::nsSupportsPRUint8Impl()
 {
 }
 
-NS_IMETHODIMP nsSupportsPRUint8Impl::GetType(PRUint16 *aType)
+NS_IMETHODIMP nsSupportsPRUint8Impl::GetType(uint16_t *aType)
 {
     NS_ASSERTION(aType, "Bad pointer");
     *aType = TYPE_PRUINT8;
@@ -203,14 +204,14 @@ NS_IMETHODIMP nsSupportsPRUint8Impl::GetType(PRUint16 *aType)
     return NS_OK;
 }
 
-NS_IMETHODIMP nsSupportsPRUint8Impl::GetData(PRUint8 *aData)
+NS_IMETHODIMP nsSupportsPRUint8Impl::GetData(uint8_t *aData)
 {
     NS_ASSERTION(aData, "Bad pointer");
     *aData = mData;
     return NS_OK;
 }
 
-NS_IMETHODIMP nsSupportsPRUint8Impl::SetData(PRUint8 aData)
+NS_IMETHODIMP nsSupportsPRUint8Impl::SetData(uint8_t aData)
 {
     mData = aData;
     return NS_OK;
@@ -222,7 +223,7 @@ NS_IMETHODIMP nsSupportsPRUint8Impl::ToString(char **_retval)
     static const int size = 8;
     char buf[size];
 
-    PR_snprintf(buf, size, "%u", (PRUint16) mData);
+    PR_snprintf(buf, size, "%u", (uint16_t) mData);
 
     char* result = (char*) nsMemory::Clone(buf,
                                 (strlen(buf)+1)*sizeof(char));
@@ -240,7 +241,7 @@ nsSupportsPRUint16Impl::nsSupportsPRUint16Impl()
 {
 }
 
-NS_IMETHODIMP nsSupportsPRUint16Impl::GetType(PRUint16 *aType)
+NS_IMETHODIMP nsSupportsPRUint16Impl::GetType(uint16_t *aType)
 {
     NS_ASSERTION(aType, "Bad pointer");
     *aType = TYPE_PRUINT16;
@@ -248,14 +249,14 @@ NS_IMETHODIMP nsSupportsPRUint16Impl::GetType(PRUint16 *aType)
     return NS_OK;
 }
 
-NS_IMETHODIMP nsSupportsPRUint16Impl::GetData(PRUint16 *aData)
+NS_IMETHODIMP nsSupportsPRUint16Impl::GetData(uint16_t *aData)
 {
     NS_ASSERTION(aData, "Bad pointer");
     *aData = mData;
     return NS_OK;
 }
 
-NS_IMETHODIMP nsSupportsPRUint16Impl::SetData(PRUint16 aData)
+NS_IMETHODIMP nsSupportsPRUint16Impl::SetData(uint16_t aData)
 {
     mData = aData;
     return NS_OK;
@@ -285,7 +286,7 @@ nsSupportsPRUint32Impl::nsSupportsPRUint32Impl()
 {
 }
 
-NS_IMETHODIMP nsSupportsPRUint32Impl::GetType(PRUint16 *aType)
+NS_IMETHODIMP nsSupportsPRUint32Impl::GetType(uint16_t *aType)
 {
     NS_ASSERTION(aType, "Bad pointer");
     *aType = TYPE_PRUINT32;
@@ -293,14 +294,14 @@ NS_IMETHODIMP nsSupportsPRUint32Impl::GetType(PRUint16 *aType)
     return NS_OK;
 }
 
-NS_IMETHODIMP nsSupportsPRUint32Impl::GetData(PRUint32 *aData)
+NS_IMETHODIMP nsSupportsPRUint32Impl::GetData(uint32_t *aData)
 {
     NS_ASSERTION(aData, "Bad pointer");
     *aData = mData;
     return NS_OK;
 }
 
-NS_IMETHODIMP nsSupportsPRUint32Impl::SetData(PRUint32 aData)
+NS_IMETHODIMP nsSupportsPRUint32Impl::SetData(uint32_t aData)
 {
     mData = aData;
     return NS_OK;
@@ -326,11 +327,11 @@ NS_IMPL_ISUPPORTS2(nsSupportsPRUint64Impl, nsISupportsPRUint64,
                    nsISupportsPrimitive)
 
 nsSupportsPRUint64Impl::nsSupportsPRUint64Impl()
-    : mData(LL_ZERO)
+    : mData(0)
 {
 }
 
-NS_IMETHODIMP nsSupportsPRUint64Impl::GetType(PRUint16 *aType)
+NS_IMETHODIMP nsSupportsPRUint64Impl::GetType(uint16_t *aType)
 {
     NS_ASSERTION(aType, "Bad pointer");
     *aType = TYPE_PRUINT64;
@@ -338,14 +339,14 @@ NS_IMETHODIMP nsSupportsPRUint64Impl::GetType(PRUint16 *aType)
     return NS_OK;
 }
 
-NS_IMETHODIMP nsSupportsPRUint64Impl::GetData(PRUint64 *aData)
+NS_IMETHODIMP nsSupportsPRUint64Impl::GetData(uint64_t *aData)
 {
     NS_ASSERTION(aData, "Bad pointer");
     *aData = mData;
     return NS_OK;
 }
 
-NS_IMETHODIMP nsSupportsPRUint64Impl::SetData(PRUint64 aData)
+NS_IMETHODIMP nsSupportsPRUint64Impl::SetData(uint64_t aData)
 {
     mData = aData;
     return NS_OK;
@@ -371,11 +372,11 @@ NS_IMPL_ISUPPORTS2(nsSupportsPRTimeImpl, nsISupportsPRTime,
                    nsISupportsPrimitive)
 
 nsSupportsPRTimeImpl::nsSupportsPRTimeImpl()
-    : mData(LL_ZERO)
+    : mData(0)
 {
 }
 
-NS_IMETHODIMP nsSupportsPRTimeImpl::GetType(PRUint16 *aType)
+NS_IMETHODIMP nsSupportsPRTimeImpl::GetType(uint16_t *aType)
 {
     NS_ASSERTION(aType, "Bad pointer");
     *aType = TYPE_PRTIME;
@@ -420,7 +421,7 @@ nsSupportsCharImpl::nsSupportsCharImpl()
 {
 }
 
-NS_IMETHODIMP nsSupportsCharImpl::GetType(PRUint16 *aType)
+NS_IMETHODIMP nsSupportsCharImpl::GetType(uint16_t *aType)
 {
     NS_ASSERTION(aType, "Bad pointer");
     *aType = TYPE_CHAR;
@@ -446,7 +447,7 @@ NS_IMETHODIMP nsSupportsCharImpl::ToString(char **_retval)
     char* result;
     NS_ASSERTION(_retval, "Bad pointer");
 
-    if(nsnull != (result = (char*) nsMemory::Alloc(2*sizeof(char))))
+    if(nullptr != (result = (char*) nsMemory::Alloc(2*sizeof(char))))
     {
         result[0] = mData;
         result[1] = '\0';
@@ -465,7 +466,7 @@ nsSupportsPRInt16Impl::nsSupportsPRInt16Impl()
 {
 }
 
-NS_IMETHODIMP nsSupportsPRInt16Impl::GetType(PRUint16 *aType)
+NS_IMETHODIMP nsSupportsPRInt16Impl::GetType(uint16_t *aType)
 {
     NS_ASSERTION(aType, "Bad pointer");
     *aType = TYPE_PRINT16;
@@ -473,14 +474,14 @@ NS_IMETHODIMP nsSupportsPRInt16Impl::GetType(PRUint16 *aType)
     return NS_OK;
 }
 
-NS_IMETHODIMP nsSupportsPRInt16Impl::GetData(PRInt16 *aData)
+NS_IMETHODIMP nsSupportsPRInt16Impl::GetData(int16_t *aData)
 {
     NS_ASSERTION(aData, "Bad pointer");
     *aData = mData;
     return NS_OK;
 }
 
-NS_IMETHODIMP nsSupportsPRInt16Impl::SetData(PRInt16 aData)
+NS_IMETHODIMP nsSupportsPRInt16Impl::SetData(int16_t aData)
 {
     mData = aData;
     return NS_OK;
@@ -510,7 +511,7 @@ nsSupportsPRInt32Impl::nsSupportsPRInt32Impl()
 {
 }
 
-NS_IMETHODIMP nsSupportsPRInt32Impl::GetType(PRUint16 *aType)
+NS_IMETHODIMP nsSupportsPRInt32Impl::GetType(uint16_t *aType)
 {
     NS_ASSERTION(aType, "Bad pointer");
     *aType = TYPE_PRINT32;
@@ -518,14 +519,14 @@ NS_IMETHODIMP nsSupportsPRInt32Impl::GetType(PRUint16 *aType)
     return NS_OK;
 }
 
-NS_IMETHODIMP nsSupportsPRInt32Impl::GetData(PRInt32 *aData)
+NS_IMETHODIMP nsSupportsPRInt32Impl::GetData(int32_t *aData)
 {
     NS_ASSERTION(aData, "Bad pointer");
     *aData = mData;
     return NS_OK;
 }
 
-NS_IMETHODIMP nsSupportsPRInt32Impl::SetData(PRInt32 aData)
+NS_IMETHODIMP nsSupportsPRInt32Impl::SetData(int32_t aData)
 {
     mData = aData;
     return NS_OK;
@@ -551,11 +552,11 @@ NS_IMPL_ISUPPORTS2(nsSupportsPRInt64Impl, nsISupportsPRInt64,
                    nsISupportsPrimitive)
 
 nsSupportsPRInt64Impl::nsSupportsPRInt64Impl()
-    : mData(LL_ZERO)
+    : mData(0)
 {
 }
 
-NS_IMETHODIMP nsSupportsPRInt64Impl::GetType(PRUint16 *aType)
+NS_IMETHODIMP nsSupportsPRInt64Impl::GetType(uint16_t *aType)
 {
     NS_ASSERTION(aType, "Bad pointer");
     *aType = TYPE_PRINT64;
@@ -563,14 +564,14 @@ NS_IMETHODIMP nsSupportsPRInt64Impl::GetType(PRUint16 *aType)
     return NS_OK;
 }
 
-NS_IMETHODIMP nsSupportsPRInt64Impl::GetData(PRInt64 *aData)
+NS_IMETHODIMP nsSupportsPRInt64Impl::GetData(int64_t *aData)
 {
     NS_ASSERTION(aData, "Bad pointer");
     *aData = mData;
     return NS_OK;
 }
 
-NS_IMETHODIMP nsSupportsPRInt64Impl::SetData(PRInt64 aData)
+NS_IMETHODIMP nsSupportsPRInt64Impl::SetData(int64_t aData)
 {
     mData = aData;
     return NS_OK;
@@ -600,7 +601,7 @@ nsSupportsFloatImpl::nsSupportsFloatImpl()
 {
 }
 
-NS_IMETHODIMP nsSupportsFloatImpl::GetType(PRUint16 *aType)
+NS_IMETHODIMP nsSupportsFloatImpl::GetType(uint16_t *aType)
 {
     NS_ASSERTION(aType, "Bad pointer");
     *aType = TYPE_FLOAT;
@@ -645,7 +646,7 @@ nsSupportsDoubleImpl::nsSupportsDoubleImpl()
 {
 }
 
-NS_IMETHODIMP nsSupportsDoubleImpl::GetType(PRUint16 *aType)
+NS_IMETHODIMP nsSupportsDoubleImpl::GetType(uint16_t *aType)
 {
     NS_ASSERTION(aType, "Bad pointer");
     *aType = TYPE_DOUBLE;
@@ -683,15 +684,15 @@ NS_IMETHODIMP nsSupportsDoubleImpl::ToString(char **_retval)
 /***************************************************************************/
 
 
-NS_IMPL_THREADSAFE_ISUPPORTS2(nsSupportsVoidImpl, nsISupportsVoid,
-                              nsISupportsPrimitive)
+NS_IMPL_ISUPPORTS2(nsSupportsVoidImpl, nsISupportsVoid,
+                   nsISupportsPrimitive)
 
 nsSupportsVoidImpl::nsSupportsVoidImpl()
-    : mData(nsnull)
+    : mData(nullptr)
 {
 }
 
-NS_IMETHODIMP nsSupportsVoidImpl::GetType(PRUint16 *aType)
+NS_IMETHODIMP nsSupportsVoidImpl::GetType(uint16_t *aType)
 {
     NS_ASSERTION(aType, "Bad pointer");
     *aType = TYPE_VOID;
@@ -725,12 +726,12 @@ NS_IMETHODIMP nsSupportsVoidImpl::ToString(char **_retval)
 /***************************************************************************/
 
 
-NS_IMPL_THREADSAFE_ISUPPORTS2(nsSupportsInterfacePointerImpl,
-                              nsISupportsInterfacePointer,
-                              nsISupportsPrimitive)
+NS_IMPL_ISUPPORTS2(nsSupportsInterfacePointerImpl,
+                   nsISupportsInterfacePointer,
+                   nsISupportsPrimitive)
 
 nsSupportsInterfacePointerImpl::nsSupportsInterfacePointerImpl()
-    : mIID(nsnull)
+    : mIID(nullptr)
 {
 }
 
@@ -741,7 +742,7 @@ nsSupportsInterfacePointerImpl::~nsSupportsInterfacePointerImpl()
     }
 }
 
-NS_IMETHODIMP nsSupportsInterfacePointerImpl::GetType(PRUint16 *aType)
+NS_IMETHODIMP nsSupportsInterfacePointerImpl::GetType(uint16_t *aType)
 {
     NS_ASSERTION(aType, "Bad pointer");
     *aType = TYPE_INTERFACE_POINTER;
@@ -775,7 +776,7 @@ NS_IMETHODIMP nsSupportsInterfacePointerImpl::GetDataIID(nsID **aIID)
         *aIID = (nsID*) nsMemory::Clone(mIID, sizeof(nsID));
         return *aIID ? NS_OK : NS_ERROR_OUT_OF_MEMORY;
     }
-    *aIID = nsnull;
+    *aIID = nullptr;
     return NS_OK;
 }
 
@@ -786,7 +787,7 @@ NS_IMETHODIMP nsSupportsInterfacePointerImpl::SetDataIID(const nsID *aIID)
     if(aIID)
         mIID = (nsID*) nsMemory::Clone(aIID, sizeof(nsID));
     else
-        mIID = nsnull;
+        mIID = nullptr;
 
     return NS_OK;
 }
@@ -813,9 +814,10 @@ nsSupportsDependentCString::nsSupportsDependentCString(const char* aStr)
 { }
 
 NS_IMETHODIMP
-nsSupportsDependentCString::GetType(PRUint16 *aType)
+nsSupportsDependentCString::GetType(uint16_t *aType)
 {
-    NS_ENSURE_ARG_POINTER(aType);
+    if (NS_WARN_IF(!aType))
+        return NS_ERROR_INVALID_ARG;
 
     *aType = TYPE_CSTRING;
     return NS_OK;
@@ -831,7 +833,8 @@ nsSupportsDependentCString::GetData(nsACString& aData)
 NS_IMETHODIMP
 nsSupportsDependentCString::ToString(char **_retval)
 {
-    NS_ENSURE_ARG_POINTER(_retval);
+    if (NS_WARN_IF(!_retval))
+        return NS_ERROR_INVALID_ARG;
 
     *_retval = ToNewCString(mData);
     if (!*_retval)
