@@ -20,6 +20,10 @@
 */
 
 #include "mozilla/Attributes.h"
+#include "mozilla/TypeTraits.h"
+#include "mozilla/Assertions.h"
+#include "mozilla/NullPtr.h"
+#include "mozilla/Move.h"
 
   // Wrapping includes can speed up compiles (see "Large Scale C++ Software Design")
 #ifndef nsDebug_h___
@@ -37,6 +41,8 @@
   // for |NS_COM_GLUE|
 #endif
 
+#include "nsCycleCollectionNoteChild.h"
+
 
 /*
   WARNING:
@@ -64,17 +70,7 @@
 //#define NSCAP_FEATURE_TEST_NONNULL_QUERY_SUCCEEDS
 #endif
 
-  /*
-    |...TEST_DONTQUERY_CASES| and |...DEBUG_PTR_TYPES| introduce some code that is 
-    problematic on a select few of our platforms, e.g., QNX.  Therefore, I'm providing
-    a mechanism by which these features can be explicitly disabled from the command-line.
-  */
-
-#ifdef NSCAP_DISABLE_TEST_DONTQUERY_CASES
-  #undef NSCAP_FEATURE_TEST_DONTQUERY_CASES
-#endif
-
-#if __GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 3)
+#ifdef __GNUC__
   // Our use of nsCOMPtr_base::mRawPtr violates the C++ standard's aliasing
   // rules. Mark it with the may_alias attribute so that gcc 3.3 and higher
   // don't reorder instructions based on aliasing assumptions for
@@ -94,7 +90,7 @@
     The following three macros (|NSCAP_ADDREF|, |NSCAP_RELEASE|, and |NSCAP_LOG_ASSIGNMENT|)
       allow external clients the ability to add logging or other interesting debug facilities.
       In fact, if you want |nsCOMPtr| to participate in the standard logging facility, you
-      provide (e.g., in "nsTraceRefcnt.h") suitable definitions
+      provide (e.g., in "nsISupportsImpl.h") suitable definitions
 
         #define NSCAP_ADDREF(this, ptr)         NS_ADDREF(ptr)
         #define NSCAP_RELEASE(this, ptr)        NS_RELEASE(ptr)
@@ -123,6 +119,12 @@
   #define NSCAP_LOG_RELEASE(this, ptr)
 #endif
 
+namespace mozilla {
+
+struct unused_t;
+
+} // namespace mozilla
+
 template <class T>
 struct already_AddRefed
     /*
@@ -141,13 +143,56 @@ struct already_AddRefed
       |nsCOMPtr_helper|.
     */
   {
-    already_AddRefed( T* aRawPtr )
-        : mRawPtr(aRawPtr)
-      {
-        // nothing else to do here
-      }
+    /*
+     * Prohibit all one-argument overloads but already_AddRefed(T*) and
+     * already_AddRefed(decltype(nullptr)), and funnel the nullptr case through
+     * the T* constructor.
+     */
+    template<typename N>
+    already_AddRefed(N,
+                     typename mozilla::EnableIf<mozilla::IsNullPointer<N>::value,
+                                                int>::Type dummy = 0)
+      : mRawPtr(nullptr)
+    {
+      // nothing else to do here
+    }
 
-    T* get() const { return mRawPtr; }
+    already_AddRefed( T* aRawPtr )
+      : mRawPtr(aRawPtr)
+    {
+      // nothing else to do here
+    }
+
+    // Disallowed.  Use move semantics instead.
+    already_AddRefed(const already_AddRefed<T>& aOther) MOZ_DELETE;
+
+    already_AddRefed(already_AddRefed<T>&& aOther)
+      : mRawPtr(aOther.take())
+    {
+      // nothing else to do here
+    }
+
+    ~already_AddRefed()
+    {
+      MOZ_ASSERT(!mRawPtr);
+    }
+
+    // Specialize the unused operator<< for already_AddRefed, to allow
+    // nsCOMPtr<nsIFoo> foo;
+    // unused << foo.forget();
+    friend void operator<<(const mozilla::unused_t& unused,
+                           const already_AddRefed<T>& rhs)
+    {
+      auto mutableAlreadyAddRefed = const_cast<already_AddRefed<T>*>(&rhs);
+      unused << mutableAlreadyAddRefed->take();
+    }
+
+    MOZ_WARN_UNUSED_RESULT T* take()
+    {
+      T* rawPtr = mRawPtr;
+      mRawPtr = nullptr;
+      return rawPtr;
+    }
 
     /**
      * This helper is useful in cases like
@@ -168,36 +213,43 @@ struct already_AddRefed
     operator already_AddRefed<U>()
     {
       U* tmp = mRawPtr;
-      mRawPtr = NULL;
-      return tmp;
+      mRawPtr = nullptr;
+      return already_AddRefed<U>(tmp);
     }
 
+    /**
+     * This helper provides a static_cast replacement for already_AddRefed, so
+     * if you have
+     *
+     *   already_AddRefed<Parent> F();
+     *
+     * you can write
+     *
+     *   already_AddRefed<Child>
+     *   G()
+     *   {
+     *     return F().downcast<Child>();
+     *   }
+     *
+     * instead of
+     *
+     *     return dont_AddRef(static_cast<Child*>(F().get()));
+     */
+    template<class U>
+    already_AddRefed<U> downcast()
+    {
+      U* tmp = static_cast<U*>(mRawPtr);
+      mRawPtr = nullptr;
+      return already_AddRefed<U>(tmp);
+    }
+
+  private:
     T* mRawPtr;
   };
 
 template <class T>
 inline
-const already_AddRefed<T>
-getter_AddRefs( T* aRawPtr )
-    /*
-      ...makes typing easier, because it deduces the template type, e.g., 
-      you write |dont_AddRef(fooP)| instead of |already_AddRefed<IFoo>(fooP)|.
-    */
-  {
-    return already_AddRefed<T>(aRawPtr);
-  }
-
-template <class T>
-inline
-const already_AddRefed<T>
-getter_AddRefs( const already_AddRefed<T> aAlreadyAddRefedPtr )
-  {
-    return aAlreadyAddRefedPtr;
-  }
-
-template <class T>
-inline
-const already_AddRefed<T>
+already_AddRefed<T>
 dont_AddRef( T* aRawPtr )
   {
     return already_AddRefed<T>(aRawPtr);
@@ -205,10 +257,10 @@ dont_AddRef( T* aRawPtr )
 
 template <class T>
 inline
-const already_AddRefed<T>
-dont_AddRef( const already_AddRefed<T> aAlreadyAddRefedPtr )
+already_AddRefed<T>&&
+dont_AddRef( already_AddRefed<T>&& aAlreadyAddRefedPtr )
   {
-    return aAlreadyAddRefedPtr;
+    return mozilla::Move(aAlreadyAddRefedPtr);
   }
 
 
@@ -243,7 +295,7 @@ class nsCOMPtr_helper
 
 class
   NS_COM_GLUE
-  NS_STACK_CLASS
+  MOZ_STACK_CLASS
 nsQueryInterface MOZ_FINAL
   {
     public:
@@ -538,11 +590,43 @@ class nsCOMPtr MOZ_FINAL
           NSCAP_ASSERT_NO_QUERY_NEEDED();
         }
 
-      nsCOMPtr( const already_AddRefed<T>& aSmartPtr )
-            : NSCAP_CTOR_BASE(aSmartPtr.mRawPtr)
-          // construct from |dont_AddRef(expr)|
+      nsCOMPtr( already_AddRefed<T>& aSmartPtr )
+            : NSCAP_CTOR_BASE(aSmartPtr.take())
+          // construct from |already_AddRefed|
         {
-          NSCAP_LOG_ASSIGNMENT(this, aSmartPtr.mRawPtr);
+          NSCAP_LOG_ASSIGNMENT(this, mRawPtr);
+          NSCAP_ASSERT_NO_QUERY_NEEDED();
+        }
+
+      nsCOMPtr( already_AddRefed<T>&& aSmartPtr )
+            : NSCAP_CTOR_BASE(aSmartPtr.take())
+          // construct from |otherComPtr.forget()|
+        {
+          NSCAP_LOG_ASSIGNMENT(this, mRawPtr);
+          NSCAP_ASSERT_NO_QUERY_NEEDED();
+        }
+
+      template<typename U>
+      nsCOMPtr( already_AddRefed<U>& aSmartPtr )
+            : NSCAP_CTOR_BASE(static_cast<T*>(aSmartPtr.take()))
+          // construct from |already_AddRefed|
+        {
+          // But make sure that U actually inherits from T
+          static_assert(mozilla::IsBaseOf<T, U>::value,
+                        "U is not a subclass of T");
+          NSCAP_LOG_ASSIGNMENT(this, static_cast<T*>(mRawPtr));
+          NSCAP_ASSERT_NO_QUERY_NEEDED();
+        }
+
+      template<typename U>
+      nsCOMPtr( already_AddRefed<U>&& aSmartPtr )
+            : NSCAP_CTOR_BASE(static_cast<T*>(aSmartPtr.take()))
+          // construct from |otherComPtr.forget()|
+        {
+          // But make sure that U actually inherits from T
+          static_assert(mozilla::IsBaseOf<T, U>::value,
+                        "U is not a subclass of T");
+          NSCAP_LOG_ASSIGNMENT(this, static_cast<T*>(mRawPtr));
           NSCAP_ASSERT_NO_QUERY_NEEDED();
         }
 
@@ -624,11 +708,28 @@ class nsCOMPtr MOZ_FINAL
           return *this;
         }
 
+      template<typename U>
       nsCOMPtr<T>&
-      operator=( const already_AddRefed<T>& rhs )
-          // assign from |dont_AddRef(expr)|
+      operator=( already_AddRefed<U>& rhs )
+          // assign from |already_AddRefed|
         {
-          assign_assuming_AddRef(rhs.mRawPtr);
+          // Make sure that U actually inherits from T
+          static_assert(mozilla::IsBaseOf<T, U>::value,
+                        "U is not a subclass of T");
+          assign_assuming_AddRef(static_cast<T*>(rhs.take()));
+          NSCAP_ASSERT_NO_QUERY_NEEDED();
+          return *this;
+        }
+
+      template<typename U>
+      nsCOMPtr<T>&
+      operator=( already_AddRefed<U>&& rhs )
+          // assign from |otherComPtr.forget()|
+        {
+          // Make sure that U actually inherits from T
+          static_assert(mozilla::IsBaseOf<T, U>::value,
+                        "U is not a subclass of T");
+          assign_assuming_AddRef(static_cast<T*>(rhs.take()));
           NSCAP_ASSERT_NO_QUERY_NEEDED();
           return *this;
         }
@@ -735,7 +836,7 @@ class nsCOMPtr MOZ_FINAL
         {
           T* temp = 0;
           swap(temp);
-          return temp;
+          return already_AddRefed<T>(temp);
         }
 
       template <typename I>
@@ -863,11 +964,18 @@ class nsCOMPtr<nsISupports>
           NSCAP_LOG_ASSIGNMENT(this, aRawPtr);
         }
 
-      nsCOMPtr( const already_AddRefed<nsISupports>& aSmartPtr )
-            : nsCOMPtr_base(aSmartPtr.mRawPtr)
-          // construct from |dont_AddRef(expr)|
+      nsCOMPtr( already_AddRefed<nsISupports>& aSmartPtr )
+            : nsCOMPtr_base(aSmartPtr.take())
+          // construct from |already_AddRefed|
         {
-          NSCAP_LOG_ASSIGNMENT(this, aSmartPtr.mRawPtr);
+          NSCAP_LOG_ASSIGNMENT(this, mRawPtr);
+        }
+
+      nsCOMPtr( already_AddRefed<nsISupports>&& aSmartPtr )
+            : nsCOMPtr_base(aSmartPtr.take())
+          // construct from |otherComPtr.forget()|
+        {
+          NSCAP_LOG_ASSIGNMENT(this, mRawPtr);
         }
 
       nsCOMPtr( const nsQueryInterface qi )
@@ -947,10 +1055,18 @@ class nsCOMPtr<nsISupports>
         }
 
       nsCOMPtr<nsISupports>&
-      operator=( const already_AddRefed<nsISupports>& rhs )
-          // assign from |dont_AddRef(expr)|
+      operator=( already_AddRefed<nsISupports>& rhs )
+          // assign from |already_AddRefed|
         {
-          assign_assuming_AddRef(rhs.mRawPtr);
+          assign_assuming_AddRef(rhs.take());
+          return *this;
+        }
+
+      nsCOMPtr<nsISupports>&
+      operator=( already_AddRefed<nsISupports>&& rhs )
+          // assign from |otherComPtr.forget()|
+        {
+          assign_assuming_AddRef(rhs.take());
           return *this;
         }
 
@@ -1042,7 +1158,7 @@ class nsCOMPtr<nsISupports>
         {
           nsISupports* temp = 0;
           swap(temp);
-          return temp;
+          return already_AddRefed<nsISupports>(temp);
         }
 
       void
@@ -1126,6 +1242,23 @@ class nsCOMPtr<nsISupports>
         }
   };
 
+template <typename T>
+inline void
+ImplCycleCollectionUnlink(nsCOMPtr<T>& aField)
+{
+  aField = nullptr;
+}
+
+template <typename T>
+inline void
+ImplCycleCollectionTraverse(nsCycleCollectionTraversalCallback& aCallback,
+                            nsCOMPtr<T>& aField,
+                            const char* aName,
+                            uint32_t aFlags = 0)
+{
+  CycleCollectionNoteChild(aCallback, aField.get(), aName, aFlags);
+}
+
 #ifndef NSCAP_FEATURE_USE_BASE
 template <class T>
 void
@@ -1278,12 +1411,6 @@ class nsGetterAddRefs
         {
           return reinterpret_cast<void**>(mTargetSmartPtr.StartAssignment());
         }
-
-      operator nsISupports**()
-        {
-          return reinterpret_cast<nsISupports**>(mTargetSmartPtr.StartAssignment());
-        }
-
       operator T**()
         {
           return mTargetSmartPtr.StartAssignment();
