@@ -1,45 +1,16 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /* vim:set ts=2 sw=2 sts=2 et cindent: */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is Mozilla.
- *
- * The Initial Developer of the Original Code is IBM Corporation.
- * Portions created by IBM Corporation are Copyright (C) 2003
- * IBM Corporation. All Rights Reserved.
- *
- * Contributor(s):
- *   Darin Fisher <darin@meer.net>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
-#include "prdtoa.h"
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
+#include "mozilla/MemoryReporting.h"
+#include "double-conversion.h"
+
+using double_conversion::DoubleToStringConverter;
 
 #ifdef XPCOM_STRING_CONSTRUCTOR_OUT_OF_LINE
 nsTSubstring_CharT::nsTSubstring_CharT( char_type *data, size_type length,
-                                        PRUint32 flags)
+                                        uint32_t flags)
   : mData(data),
     mLength(length),
     mFlags(flags)
@@ -70,11 +41,11 @@ AsFixedString( const nsTSubstring_CharT* s )
    * returns the old data and old flags members if mData is newly allocated.
    * the old data must be released by the caller.
    */
-PRBool
-nsTSubstring_CharT::MutatePrep( size_type capacity, char_type** oldData, PRUint32* oldFlags )
+bool
+nsTSubstring_CharT::MutatePrep( size_type capacity, char_type** oldData, uint32_t* oldFlags )
   {
     // initialize to no old data
-    *oldData = nsnull;
+    *oldData = nullptr;
     *oldFlags = 0;
 
     size_type curCapacity = Capacity();
@@ -89,7 +60,7 @@ nsTSubstring_CharT::MutatePrep( size_type capacity, char_type** oldData, PRUint3
       // Also assert for |capacity| equal to |size_type(-1)|, since we used to
       // use that value to flag immutability.
       NS_ASSERTION(capacity != size_type(-1), "Bogus capacity");
-      return PR_FALSE;
+      return false;
     }
 
     // |curCapacity == 0| means that the buffer is immutable or 0-sized, so we
@@ -100,16 +71,16 @@ nsTSubstring_CharT::MutatePrep( size_type capacity, char_type** oldData, PRUint3
       {
         if (capacity <= curCapacity) {
           mFlags &= ~F_VOIDED;  // mutation clears voided flag
-          return PR_TRUE;
+          return true;
         }
 
         // Use doubling algorithm when forced to increase available capacity.
         size_type temp = curCapacity;
         while (temp < capacity)
           temp <<= 1;
-        NS_ASSERTION(NS_MIN(temp, kMaxCapacity) >= capacity,
+        NS_ASSERTION(XPCOM_MIN(temp, kMaxCapacity) >= capacity,
                      "should have hit the early return at the top");
-        capacity = NS_MIN(temp, kMaxCapacity);
+        capacity = XPCOM_MIN(temp, kMaxCapacity);
       }
 
     //
@@ -134,17 +105,17 @@ nsTSubstring_CharT::MutatePrep( size_type capacity, char_type** oldData, PRUint3
           {
             nsStringBuffer *newHdr = nsStringBuffer::Realloc(hdr, storageSize);
             if (!newHdr)
-              return PR_FALSE; // out-of-memory (original header left intact)
+              return false; // out-of-memory (original header left intact)
 
             hdr = newHdr;
             mData = (char_type*) hdr->Data();
             mFlags &= ~F_VOIDED;  // mutation clears voided flag
-            return PR_TRUE;
+            return true;
           }
       }
 
     char_type* newData;
-    PRUint32 newDataFlags;
+    uint32_t newDataFlags;
 
       // if we have a fixed buffer of sufficient size, then use it.  this helps
       // avoid heap allocations.
@@ -159,9 +130,10 @@ nsTSubstring_CharT::MutatePrep( size_type capacity, char_type** oldData, PRUint3
         // make use of our F_OWNED or F_FIXED buffers because they are not
         // large enough.
 
-        nsStringBuffer* newHdr = nsStringBuffer::Alloc(storageSize);
+        nsStringBuffer* newHdr =
+          nsStringBuffer::Alloc(storageSize).take();
         if (!newHdr)
-          return PR_FALSE; // we are still in a consistent state
+          return false; // we are still in a consistent state
 
         newData = (char_type*) newHdr->Data();
         newDataFlags = F_TERMINATED | F_SHARED;
@@ -179,7 +151,7 @@ nsTSubstring_CharT::MutatePrep( size_type capacity, char_type** oldData, PRUint3
     // though we are not necessarily terminated at the moment, now is probably
     // still the best time to set F_TERMINATED.
 
-    return PR_TRUE;
+    return true;
   }
 
 void
@@ -189,14 +161,14 @@ nsTSubstring_CharT::Finalize()
     // mData, mLength, and mFlags are purposefully left dangling
   }
 
-PRBool
+bool
 nsTSubstring_CharT::ReplacePrepInternal(index_type cutStart, size_type cutLen,
                                         size_type fragLen, size_type newLen)
   {
     char_type* oldData;
-    PRUint32 oldFlags;
+    uint32_t oldFlags;
     if (!MutatePrep(newLen, &oldData, &oldFlags))
-      return PR_FALSE; // out-of-memory
+      return false; // out-of-memory
 
     if (oldData)
       {
@@ -214,7 +186,7 @@ nsTSubstring_CharT::ReplacePrepInternal(index_type cutStart, size_type cutLen,
             // copy suffix from old string to new offset
             size_type from = cutStart + cutLen;
             size_type fromLen = mLength - from;
-            PRUint32 to = cutStart + fragLen;
+            uint32_t to = cutStart + fragLen;
             char_traits::copy(mData + to, oldData + from, fromLen);
           }
 
@@ -228,9 +200,9 @@ nsTSubstring_CharT::ReplacePrepInternal(index_type cutStart, size_type cutLen,
         // to make room for the requested hole.
         if (fragLen != cutLen && cutStart + cutLen < mLength)
           {
-            PRUint32 from = cutStart + cutLen;
-            PRUint32 fromLen = mLength - from;
-            PRUint32 to = cutStart + fragLen;
+            uint32_t from = cutStart + cutLen;
+            uint32_t fromLen = mLength - from;
+            uint32_t to = cutStart + fragLen;
             char_traits::move(mData + to, mData + from, fromLen);
           }
       }
@@ -240,7 +212,7 @@ nsTSubstring_CharT::ReplacePrepInternal(index_type cutStart, size_type cutLen,
     mData[newLen] = char_type(0);
     mLength = newLen;
 
-    return PR_TRUE;
+    return true;
   }
 
 nsTSubstring_CharT::size_type
@@ -279,26 +251,19 @@ nsTSubstring_CharT::Capacity() const
     return capacity;
   }
 
-PRBool
+bool
 nsTSubstring_CharT::EnsureMutable( size_type newLen )
   {
     if (newLen == size_type(-1) || newLen == mLength)
       {
         if (mFlags & (F_FIXED | F_OWNED))
-          return PR_TRUE;
+          return true;
         if ((mFlags & F_SHARED) && !nsStringBuffer::FromData(mData)->IsReadonly())
-          return PR_TRUE;
+          return true;
 
-        // promote to a shared string buffer
-        char_type* prevData = mData;
-        Assign(mData, mLength);
-        return mData != prevData;
-      }
-    else
-      {
-        SetLength(newLen);
-        return mLength == newLen;
+        newLen = mLength;
       }
+    return SetLength(newLen, fallible_t());
   }
 
 // ---------------------------------------------------------------------------
@@ -307,19 +272,43 @@ nsTSubstring_CharT::EnsureMutable( size_type newLen )
 void
 nsTSubstring_CharT::Assign( char_type c )
   {
-    if (ReplacePrep(0, mLength, 1))
-      *mData = c;
+    if (!ReplacePrep(0, mLength, 1))
+      NS_ABORT_OOM(mLength);
+
+    *mData = c;
   }
 
+bool
+nsTSubstring_CharT::Assign( char_type c, const fallible_t& )
+  {
+    if (!ReplacePrep(0, mLength, 1))
+      return false;
+
+    *mData = c;
+    return true;
+  }
+
+void
+nsTSubstring_CharT::Assign( const char_type* data )
+  {
+    if (!Assign(data, size_type(-1), fallible_t()))
+      NS_ABORT_OOM(char_traits::length(data));
+  }
 
 void
 nsTSubstring_CharT::Assign( const char_type* data, size_type length )
   {
-      // unfortunately, some callers pass null :-(
-    if (!data)
+    if (!Assign(data, length, fallible_t()))
+      NS_ABORT_OOM(length);
+  }
+
+bool
+nsTSubstring_CharT::Assign( const char_type* data, size_type length, const fallible_t& )
+  {
+    if (!data || length == 0)
       {
         Truncate();
-        return;
+        return true;
       }
 
     if (length == size_type(-1))
@@ -327,54 +316,75 @@ nsTSubstring_CharT::Assign( const char_type* data, size_type length )
 
     if (IsDependentOn(data, data + length))
       {
-        // take advantage of sharing here...
-        Assign(string_type(data, length));
-        return;
+        return Assign(string_type(data, length), fallible_t());
       }
 
-    if (ReplacePrep(0, mLength, length))
-      char_traits::copy(mData, data, length);
+    if (!ReplacePrep(0, mLength, length))
+      return false;
+
+    char_traits::copy(mData, data, length);
+    return true;
   }
 
 void
 nsTSubstring_CharT::AssignASCII( const char* data, size_type length )
+  {
+    if (!AssignASCII(data, length, fallible_t()))
+      NS_ABORT_OOM(length);
+  }
+
+bool
+nsTSubstring_CharT::AssignASCII( const char* data, size_type length, const fallible_t& )
   {
     // A Unicode string can't depend on an ASCII string buffer,
     // so this dependence check only applies to CStrings.
 #ifdef CharT_is_char
     if (IsDependentOn(data, data + length))
       {
-        // take advantage of sharing here...
-        Assign(string_type(data, length));
-        return;
+        return Assign(string_type(data, length), fallible_t());
       }
 #endif
 
-    if (ReplacePrep(0, mLength, length))
-      char_traits::copyASCII(mData, data, length);
+    if (!ReplacePrep(0, mLength, length))
+      return false;
+
+    char_traits::copyASCII(mData, data, length);
+    return true;
   }
 
 void
-nsTSubstring_CharT::AssignASCII( const char* data )
+nsTSubstring_CharT::AssignLiteral( const char_type* data, size_type length )
   {
-    AssignASCII(data, strlen(data));
+    ::ReleaseData(mData, mFlags);
+    mData = const_cast<char_type*>(data);
+    mLength = length;
+    SetDataFlags(F_TERMINATED | F_LITERAL);
   }
 
 void
 nsTSubstring_CharT::Assign( const self_type& str )
+{
+  if (!Assign(str, fallible_t()))
+    NS_ABORT_OOM(str.Length());
+}
+
+bool
+nsTSubstring_CharT::Assign( const self_type& str, const fallible_t& )
   {
     // |str| could be sharable.  we need to check its flags to know how to
     // deal with it.
 
     if (&str == this)
-      return;
+      return true;
 
     if (!str.mLength)
       {
         Truncate();
         mFlags |= str.mFlags & F_VOIDED;
+        return true;
       }
-    else if (str.mFlags & F_SHARED)
+
+    if (str.mFlags & F_SHARED)
       {
         // nice! we can avoid a string copy :-)
 
@@ -389,37 +399,51 @@ nsTSubstring_CharT::Assign( const self_type& str )
 
         // get an owning reference to the mData
         nsStringBuffer::FromData(mData)->AddRef();
+        return true;
       }
-    else
+    else if (str.mFlags & F_LITERAL)
       {
-        // else, treat this like an ordinary assignment.
-        Assign(str.Data(), str.Length());
+        NS_ABORT_IF_FALSE(str.mFlags & F_TERMINATED, "Unterminated literal");
+
+        AssignLiteral(str.mData, str.mLength);
+        return true;
       }
+
+    // else, treat this like an ordinary assignment.
+    return Assign(str.Data(), str.Length(), fallible_t());
   }
 
 void
 nsTSubstring_CharT::Assign( const substring_tuple_type& tuple )
+  {
+    if (!Assign(tuple, fallible_t()))
+      NS_ABORT_OOM(tuple.Length());
+  }
+
+bool
+nsTSubstring_CharT::Assign( const substring_tuple_type& tuple, const fallible_t& )
   {
     if (tuple.IsDependentOn(mData, mData + mLength))
       {
         // take advantage of sharing here...
-        Assign(string_type(tuple));
-        return;
+        return Assign(string_type(tuple), fallible_t());
       }
 
     size_type length = tuple.Length();
 
     // don't use ReplacePrep here because it changes the length
     char_type* oldData;
-    PRUint32 oldFlags;
-    if (MutatePrep(length, &oldData, &oldFlags)) {
-      if (oldData)
-        ::ReleaseData(oldData, oldFlags);
+    uint32_t oldFlags;
+    if (!MutatePrep(length, &oldData, &oldFlags))
+      return false;
 
-      tuple.WriteTo(mData, length);
-      mData[length] = 0;
-      mLength = length;
-    }
+    if (oldData)
+      ::ReleaseData(oldData, oldFlags);
+
+    tuple.WriteTo(mData, length);
+    mData[length] = 0;
+    mLength = length;
+    return true;
   }
 
 void
@@ -445,7 +469,7 @@ nsTSubstring_CharT::Adopt( char_type* data, size_type length )
       }
     else
       {
-        SetIsVoid(PR_TRUE);
+        SetIsVoid(true);
       }
   }
 
@@ -454,7 +478,7 @@ nsTSubstring_CharT::Adopt( char_type* data, size_type length )
 void
 nsTSubstring_CharT::Replace( index_type cutStart, size_type cutLength, char_type c )
   {
-    cutStart = NS_MIN(cutStart, Length());
+    cutStart = XPCOM_MIN(cutStart, Length());
 
     if (ReplacePrep(cutStart, cutLength, 1))
       mData[cutStart] = c;
@@ -482,7 +506,7 @@ nsTSubstring_CharT::Replace( index_type cutStart, size_type cutLength, const cha
           }
       }
 
-    cutStart = NS_MIN(cutStart, Length());
+    cutStart = XPCOM_MIN(cutStart, Length());
 
     if (ReplacePrep(cutStart, cutLength, length) && length > 0)
       char_traits::copy(mData + cutStart, data, length);
@@ -505,7 +529,7 @@ nsTSubstring_CharT::ReplaceASCII( index_type cutStart, size_type cutLength, cons
       }
 #endif
 
-    cutStart = NS_MIN(cutStart, Length());
+    cutStart = XPCOM_MIN(cutStart, Length());
 
     if (ReplacePrep(cutStart, cutLength, length) && length > 0)
       char_traits::copyASCII(mData + cutStart, data, length);
@@ -523,14 +547,32 @@ nsTSubstring_CharT::Replace( index_type cutStart, size_type cutLength, const sub
 
     size_type length = tuple.Length();
 
-    cutStart = NS_MIN(cutStart, Length());
+    cutStart = XPCOM_MIN(cutStart, Length());
 
     if (ReplacePrep(cutStart, cutLength, length) && length > 0)
       tuple.WriteTo(mData + cutStart, length);
   }
 
-PRBool
+void
+nsTSubstring_CharT::ReplaceLiteral( index_type cutStart, size_type cutLength, const char_type* data, size_type length )
+  {
+    cutStart = XPCOM_MIN(cutStart, Length());
+
+    if (!cutStart && cutLength == Length())
+      AssignLiteral(data, length);
+    else if (ReplacePrep(cutStart, cutLength, length) && length > 0)
+      char_traits::copy(mData + cutStart, data, length);
+  }
+
+void
 nsTSubstring_CharT::SetCapacity( size_type capacity )
+  {
+    if (!SetCapacity(capacity, fallible_t()))
+      NS_ABORT_OOM(capacity);
+  }
+
+bool
+nsTSubstring_CharT::SetCapacity( size_type capacity, const fallible_t& )
   {
     // capacity does not include room for the terminating null char
 
@@ -541,47 +583,56 @@ nsTSubstring_CharT::SetCapacity( size_type capacity )
         mData = char_traits::sEmptyBuffer;
         mLength = 0;
         SetDataFlags(F_TERMINATED);
+        return true;
       }
-    else
+
+    char_type* oldData;
+    uint32_t oldFlags;
+    if (!MutatePrep(capacity, &oldData, &oldFlags))
+      return false; // out-of-memory
+
+    // compute new string length
+    size_type newLen = XPCOM_MIN(mLength, capacity);
+
+    if (oldData)
       {
-        char_type* oldData;
-        PRUint32 oldFlags;
-        if (!MutatePrep(capacity, &oldData, &oldFlags))
-          return PR_FALSE; // out-of-memory
+        // preserve old data
+        if (mLength > 0)
+          char_traits::copy(mData, oldData, newLen);
 
-        // compute new string length
-        size_type newLen = NS_MIN(mLength, capacity);
-
-        if (oldData)
-          {
-            // preserve old data
-            if (mLength > 0)
-              char_traits::copy(mData, oldData, newLen);
-
-            ::ReleaseData(oldData, oldFlags);
-          }
-
-        // adjust mLength if our buffer shrunk down in size
-        if (newLen < mLength)
-          mLength = newLen;
-
-        // always null-terminate here, even if the buffer got longer.  this is
-        // for backwards compat with the old string implementation.
-        mData[capacity] = char_type(0);
+        ::ReleaseData(oldData, oldFlags);
       }
 
-    return PR_TRUE;
+    // adjust mLength if our buffer shrunk down in size
+    if (newLen < mLength)
+      mLength = newLen;
+
+    // always null-terminate here, even if the buffer got longer.  this is
+    // for backwards compat with the old string implementation.
+    mData[capacity] = char_type(0);
+
+    return true;
   }
 
 void
 nsTSubstring_CharT::SetLength( size_type length )
   {
-    if (SetCapacity(length))
-      mLength = length;
+    SetCapacity(length);
+    mLength = length;
+  }
+
+bool
+nsTSubstring_CharT::SetLength( size_type length, const fallible_t& )
+  {
+    if (!SetCapacity(length, fallible_t()))
+      return false;
+
+    mLength = length;
+    return true;
   }
 
 void
-nsTSubstring_CharT::SetIsVoid( PRBool val )
+nsTSubstring_CharT::SetIsVoid( bool val )
   {
     if (val)
       {
@@ -594,19 +645,19 @@ nsTSubstring_CharT::SetIsVoid( PRBool val )
       }
   }
 
-PRBool
+bool
 nsTSubstring_CharT::Equals( const self_type& str ) const
   {
     return mLength == str.mLength && char_traits::compare(mData, str.mData, mLength) == 0;
   }
 
-PRBool
+bool
 nsTSubstring_CharT::Equals( const self_type& str, const comparator_type& comp ) const
   {
     return mLength == str.mLength && comp(mData, str.mData, mLength, str.mLength) == 0;
   }
 
-PRBool
+bool
 nsTSubstring_CharT::Equals( const char_type* data ) const
   {
     // unfortunately, some callers pass null :-(
@@ -621,7 +672,7 @@ nsTSubstring_CharT::Equals( const char_type* data ) const
     return mLength == length && char_traits::compare(mData, data, mLength) == 0;
   }
 
-PRBool
+bool
 nsTSubstring_CharT::Equals( const char_type* data, const comparator_type& comp ) const
   {
     // unfortunately, some callers pass null :-(
@@ -636,25 +687,25 @@ nsTSubstring_CharT::Equals( const char_type* data, const comparator_type& comp )
     return mLength == length && comp(mData, data, mLength, length) == 0;
   }
 
-PRBool
+bool
 nsTSubstring_CharT::EqualsASCII( const char* data, size_type len ) const
   {
     return mLength == len && char_traits::compareASCII(mData, data, len) == 0;
   }
 
-PRBool
+bool
 nsTSubstring_CharT::EqualsASCII( const char* data ) const
   {
     return char_traits::compareASCIINullTerminated(mData, mLength, data) == 0;
   }
 
-PRBool
+bool
 nsTSubstring_CharT::LowerCaseEqualsASCII( const char* data, size_type len ) const
   {
     return mLength == len && char_traits::compareLowerCaseToASCII(mData, data, len) == 0;
   }
 
-PRBool
+bool
 nsTSubstring_CharT::LowerCaseEqualsASCII( const char* data ) const
   {
     return char_traits::compareLowerCaseToASCIINullTerminated(mData, mLength, data) == 0;
@@ -669,7 +720,7 @@ nsTSubstring_CharT::CountChar( char_type c ) const
     return NS_COUNT(start, end, c);
   }
 
-PRInt32
+int32_t
 nsTSubstring_CharT::FindChar( char_type c, index_type offset ) const
   {
     if (offset < mLength)
@@ -682,12 +733,13 @@ nsTSubstring_CharT::FindChar( char_type c, index_type offset ) const
   }
 
 void
-nsTSubstring_CharT::StripChar( char_type aChar, PRInt32 aOffset )
+nsTSubstring_CharT::StripChar( char_type aChar, int32_t aOffset )
   {
-    if (mLength == 0 || aOffset >= PRInt32(mLength))
+    if (mLength == 0 || aOffset >= int32_t(mLength))
       return;
 
-    EnsureMutable(); // XXX do this lazily?
+    if (!EnsureMutable()) // XXX do this lazily?
+      NS_ABORT_OOM(mLength);
 
     // XXX(darin): this code should defer writing until necessary.
 
@@ -706,12 +758,13 @@ nsTSubstring_CharT::StripChar( char_type aChar, PRInt32 aOffset )
   }
 
 void
-nsTSubstring_CharT::StripChars( const char_type* aChars, PRUint32 aOffset )
+nsTSubstring_CharT::StripChars( const char_type* aChars, uint32_t aOffset )
   {
-    if (aOffset >= PRUint32(mLength))
+    if (aOffset >= uint32_t(mLength))
       return;
 
-    EnsureMutable(); // XXX do this lazily?
+    if (!EnsureMutable()) // XXX do this lazily?
+      NS_ABORT_OOM(mLength);
 
     // XXX(darin): this code should defer writing until necessary.
 
@@ -735,131 +788,201 @@ nsTSubstring_CharT::StripChars( const char_type* aChars, PRUint32 aOffset )
     mLength = to - mData;
   }
 
-void nsTSubstring_CharT::AppendPrintf31( const char* format, ...)
+int
+nsTSubstring_CharT::AppendFunc(void* arg, const char* s, uint32_t len)
   {
-    char buf[32];
-    va_list ap;
-    va_start(ap, format);
-    PRUint32 len = PR_vsnprintf(buf, sizeof(buf), format, ap);
-    AppendASCII(buf, len);
-    va_end(ap);
+    self_type* self = static_cast<self_type*>(arg);
+
+    // NSPR sends us the final null terminator even though we don't want it
+    if (len && s[len - 1] == '\0') {
+      --len;
+    }
+
+    self->AppendASCII(s, len);
+
+    return len;
   }
 
 void nsTSubstring_CharT::AppendPrintf( const char* format, ...)
   {
-    char *buf;
     va_list ap;
     va_start(ap, format);
-    buf = PR_vsmprintf(format, ap);
-    AppendASCII(buf);
-    PR_smprintf_free(buf);
+    uint32_t r = PR_vsxprintf(AppendFunc, this, format, ap);
+    if (r == (uint32_t) -1)
+      NS_RUNTIMEABORT("Allocation or other failure in PR_vsxprintf");
     va_end(ap);
   }
 
+void nsTSubstring_CharT::AppendPrintf( const char* format, va_list ap )
+  {
+    uint32_t r = PR_vsxprintf(AppendFunc, this, format, ap);
+    if (r == (uint32_t) -1)
+      NS_RUNTIMEABORT("Allocation or other failure in PR_vsxprintf");
+  }
 
-/* hack to make sure we define Modified_cnvtf only once */
+/* hack to make sure we define FormatWithoutTrailingZeros only once */
 #ifdef CharT_is_PRUnichar
-/**
- * This is a copy of |PR_cnvtf| with a bug fixed.  (The second argument
- * of PR_dtoa is 2 rather than 1.)
- *
- * XXX(darin): if this is the right thing, then why wasn't it fixed in NSPR?!?
- */
-static void 
-Modified_cnvtf(char *buf, int bufsz, int prcsn, double fval)
+// Returns the length of the formatted aDouble in buf.
+static int
+FormatWithoutTrailingZeros(char (& buf)[40], double aDouble,
+                           int precision)
 {
-  PRIntn decpt, sign, numdigits;
-  char *num, *nump;
-  char *bufp = buf;
-  char *endnum;
+  static const DoubleToStringConverter converter(DoubleToStringConverter::UNIQUE_ZERO |
+                                                 DoubleToStringConverter::EMIT_POSITIVE_EXPONENT_SIGN,
+                                                 "Infinity",
+                                                 "NaN",
+                                                 'e',
+                                                 -6, 21,
+                                                 6, 1);
+  double_conversion::StringBuilder builder(buf, sizeof(buf));
+  bool exponential_notation = false;
+  converter.ToPrecision(aDouble, precision, &exponential_notation, &builder);
+  int length = builder.position();
+  char* formattedDouble = builder.Finalize();
 
-  /* If anything fails, we store an empty string in 'buf' */
-  num = (char*)malloc(bufsz);
-  if (num == NULL) {
-    buf[0] = '\0';
-    return;
+  // If we have a shorter string than precision, it means we have a special
+  // value (NaN or Infinity).  All other numbers will be formatted with at
+  // least precision digits.
+  if (length <= precision) {
+    return length;
   }
-  if (PR_dtoa(fval, 2, prcsn, &decpt, &sign, &endnum, num, bufsz)
-      == PR_FAILURE) {
-    buf[0] = '\0';
-    goto done;
-  }
-  numdigits = endnum - num;
-  nump = num;
-
-  /*
-   * The NSPR code had a fancy way of checking that we weren't dealing
-   * with -0.0 or -NaN, but I'll just use < instead.
-   * XXX Should we check !isnan(fval) as well?  Is it portable?  We
-   * probably don't need to bother since NAN isn't portable.
-   */
-  if (sign && fval < 0.0f) {
-    *bufp++ = '-';
+    
+  char* end = formattedDouble + length;
+  char* decimalPoint = strchr(buf, '.');
+  // No trailing zeros to remove.
+  if (decimalPoint == nullptr) {
+    return length;
   }
 
-  if (decpt == 9999) {
-    while ((*bufp++ = *nump++) != 0) {} /* nothing to execute */
-    goto done;
-  }
-
-  if (decpt > (prcsn+1) || decpt < -(prcsn-1) || decpt < -5) {
-    *bufp++ = *nump++;
-    if (numdigits != 1) {
-      *bufp++ = '.';
-    }
-
-    while (*nump != '\0') {
-      *bufp++ = *nump++;
-    }
-    *bufp++ = 'e';
-    PR_snprintf(bufp, bufsz - (bufp - buf), "%+d", decpt-1);
-  }
-  else if (decpt >= 0) {
-    if (decpt == 0) {
-      *bufp++ = '0';
-    }
-    else {
-      while (decpt--) {
-        if (*nump != '\0') {
-          *bufp++ = *nump++;
-        }
-        else {
-          *bufp++ = '0';
-        }
+  if (MOZ_UNLIKELY(exponential_notation)) {
+    // We need to check for cases like 1.00000e-10 (yes, this is
+    // disgusting).
+    char* exponent = end - 1;
+    for ( ; ; --exponent) {
+      if (*exponent == 'e') {
+        break;
       }
     }
-    if (*nump != '\0') {
-      *bufp++ = '.';
-      while (*nump != '\0') {
-        *bufp++ = *nump++;
+    char* zerosBeforeExponent = exponent - 1;
+    for ( ; zerosBeforeExponent != decimalPoint; --zerosBeforeExponent) {
+      if (*zerosBeforeExponent != '0') {
+        break;
       }
     }
-    *bufp++ = '\0';
-  }
-  else if (decpt < 0) {
-    *bufp++ = '0';
-    *bufp++ = '.';
-    while (decpt++) {
-      *bufp++ = '0';
+    if (zerosBeforeExponent == decimalPoint) {
+      --zerosBeforeExponent;
     }
+    // Slide the exponent to the left over the trailing zeros.  Don't
+    // worry about copying the trailing NUL character.
+    size_t exponentSize = end - exponent;
+    memmove(zerosBeforeExponent + 1, exponent, exponentSize);
+    length -= exponent - (zerosBeforeExponent + 1);
+  } else {
+    char* trailingZeros = end - 1;
+    for ( ; trailingZeros != decimalPoint; --trailingZeros) {
+      if (*trailingZeros != '0') {
+        break;
+      }
+    }
+    if (trailingZeros == decimalPoint) {
+      --trailingZeros;
+    }
+    length -= end - (trailingZeros + 1);
+  }
 
-    while (*nump != '\0') {
-      *bufp++ = *nump++;
-    }
-    *bufp++ = '\0';
-  }
-done:
-  free(num);
+  return length;
 }
 #endif /* CharT_is_PRUnichar */
 
 void
-nsTSubstring_CharT::DoAppendFloat( double aFloat, int digits )
+nsTSubstring_CharT::AppendFloat( float aFloat )
 {
   char buf[40];
-  // Use Modified_cnvtf, which is locale-insensitive, instead of the
-  // locale-sensitive PR_snprintf or sprintf(3)
-  Modified_cnvtf(buf, sizeof(buf), digits, aFloat);
-  AppendASCII(buf);
+  int length = FormatWithoutTrailingZeros(buf, aFloat, 6);
+  AppendASCII(buf, length);
+}
+
+void
+nsTSubstring_CharT::AppendFloat( double aFloat )
+{
+  char buf[40];
+  int length = FormatWithoutTrailingZeros(buf, aFloat, 15);
+  AppendASCII(buf, length);
+}
+
+size_t
+nsTSubstring_CharT::SizeOfExcludingThisMustBeUnshared(
+    mozilla::MallocSizeOf mallocSizeOf) const
+{
+  if (mFlags & F_SHARED) {
+    return nsStringBuffer::FromData(mData)->
+             SizeOfIncludingThisMustBeUnshared(mallocSizeOf);
+  }
+  if (mFlags & F_OWNED) {
+    return mallocSizeOf(mData);
+  }
+
+  // If we reach here, exactly one of the following must be true:
+  // - F_VOIDED is set, and mData points to sEmptyBuffer;
+  // - F_FIXED is set, and mData points to a buffer within a string
+  //   object (e.g. nsAutoString);
+  // - None of F_SHARED, F_OWNED, F_FIXED is set, and mData points to a buffer
+  //   owned by something else.
+  //
+  // In all three cases, we don't measure it.
+  return 0;
+}
+
+size_t
+nsTSubstring_CharT::SizeOfExcludingThisIfUnshared(
+    mozilla::MallocSizeOf mallocSizeOf) const
+{
+  // This is identical to SizeOfExcludingThisMustBeUnshared except for the
+  // F_SHARED case.
+  if (mFlags & F_SHARED) {
+    return nsStringBuffer::FromData(mData)->
+             SizeOfIncludingThisIfUnshared(mallocSizeOf);
+  }
+  if (mFlags & F_OWNED) {
+    return mallocSizeOf(mData);
+  }
+  return 0;
+}
+
+size_t
+nsTSubstring_CharT::SizeOfExcludingThisEvenIfShared(
+    mozilla::MallocSizeOf mallocSizeOf) const
+{
+  // This is identical to SizeOfExcludingThisMustBeUnshared except for the
+  // F_SHARED case.
+  if (mFlags & F_SHARED) {
+    return nsStringBuffer::FromData(mData)->
+             SizeOfIncludingThisEvenIfShared(mallocSizeOf);
+  }
+  if (mFlags & F_OWNED) {
+    return mallocSizeOf(mData);
+  }
+  return 0;
+}
+
+size_t
+nsTSubstring_CharT::SizeOfIncludingThisMustBeUnshared(
+    mozilla::MallocSizeOf mallocSizeOf) const
+{
+  return mallocSizeOf(this) + SizeOfExcludingThisMustBeUnshared(mallocSizeOf);
+}
+
+size_t
+nsTSubstring_CharT::SizeOfIncludingThisIfUnshared(
+    mozilla::MallocSizeOf mallocSizeOf) const
+{
+  return mallocSizeOf(this) + SizeOfExcludingThisIfUnshared(mallocSizeOf);
+}
+
+size_t
+nsTSubstring_CharT::SizeOfIncludingThisEvenIfShared(
+    mozilla::MallocSizeOf mallocSizeOf) const
+{
+  return mallocSizeOf(this) + SizeOfExcludingThisEvenIfShared(mallocSizeOf);
 }
 
