@@ -25,6 +25,9 @@ GetThreadPoolLog()
   return sLog;
 }
 #endif
+#ifdef LOG
+#undef LOG
+#endif
 #define LOG(args) PR_LOG(GetThreadPoolLog(), PR_LOG_DEBUG, args)
 
 // DESIGN:
@@ -37,27 +40,9 @@ GetThreadPoolLog()
 #define DEFAULT_IDLE_THREAD_LIMIT 1
 #define DEFAULT_IDLE_THREAD_TIMEOUT PR_SecondsToInterval(60)
 
-class ShutdownHelper MOZ_FINAL : public nsRunnable
-{
-public:
-  NS_DECL_NSIRUNNABLE
-
-  ShutdownHelper(nsCOMArray<nsIThread>& aThreads,
-                 already_AddRefed<nsIThreadPoolListener> aListener)
-  : mListener(aListener)
-  {
-    MOZ_ASSERT(!aThreads.IsEmpty());
-    mThreads.SwapElements(aThreads);
-  }
-
-private:
-  nsCOMArray<nsIThread> mThreads;
-  nsCOMPtr<nsIThreadPoolListener> mListener;
-};
-
 NS_IMPL_ADDREF(nsThreadPool)
 NS_IMPL_RELEASE(nsThreadPool)
-NS_IMPL_CLASSINFO(nsThreadPool, NULL, nsIClassInfo::THREADSAFE,
+NS_IMPL_CLASSINFO(nsThreadPool, nullptr, nsIClassInfo::THREADSAFE,
                   NS_THREADPOOL_CID)
 NS_IMPL_QUERY_INTERFACE3_CI(nsThreadPool, nsIThreadPool, nsIEventTarget,
                             nsIRunnable)
@@ -68,42 +53,25 @@ nsThreadPool::nsThreadPool()
   , mIdleThreadLimit(DEFAULT_IDLE_THREAD_LIMIT)
   , mIdleThreadTimeout(DEFAULT_IDLE_THREAD_TIMEOUT)
   , mIdleCount(0)
+  , mStackSize(nsIThreadManager::DEFAULT_STACK_SIZE)
   , mShutdown(false)
 {
 }
 
 nsThreadPool::~nsThreadPool()
 {
-  // Calling Shutdown() directly is not safe since it will spin the event loop
-  // (perhaps during a GC). Instead we try to delay-shutdown each thread that is
-  // still alive.
-  nsCOMArray<nsIThread> threads;
-  nsCOMPtr<nsIThreadPoolListener> listener;
-  {
-    ReentrantMonitorAutoEnter mon(mEvents.GetReentrantMonitor());
-    if (!mShutdown) {
-      NS_WARNING("nsThreadPool destroyed before Shutdown() was called!");
-
-      mThreads.SwapElements(threads);
-      mListener.swap(listener);
-    }
-  }
-
-  if (!threads.IsEmpty()) {
-    nsRefPtr<ShutdownHelper> helper =
-      new ShutdownHelper(threads, listener.forget());
-    if (NS_FAILED(NS_DispatchToMainThread(helper, NS_DISPATCH_NORMAL))) {
-      NS_WARNING("Unable to shut down threads in this thread pool!");
-    }
-  }
+  // Threads keep a reference to the nsThreadPool until they return from Run()
+  // after removing themselves from mThreads.
+  MOZ_ASSERT(mThreads.IsEmpty());
 }
 
 nsresult
 nsThreadPool::PutEvent(nsIRunnable *event)
 {
   // Avoid spawning a new thread while holding the event queue lock...
- 
+
   bool spawnThread = false;
+  uint32_t stackSize = 0;
   {
     ReentrantMonitorAutoEnter mon(mEvents.GetReentrantMonitor());
 
@@ -116,6 +84,7 @@ nsThreadPool::PutEvent(nsIRunnable *event)
       spawnThread = true;
 
     mEvents.PutEvent(event);
+    stackSize = mStackSize;
   }
 
   LOG(("THRD-P(%p) put [spawn=%d]\n", this, spawnThread));
@@ -124,9 +93,10 @@ nsThreadPool::PutEvent(nsIRunnable *event)
 
   nsCOMPtr<nsIThread> thread;
   nsThreadManager::get()->NewThread(0,
-                                    nsIThreadManager::DEFAULT_STACK_SIZE,
+                                    stackSize,
                                     getter_AddRefs(thread));
-  NS_ENSURE_STATE(thread);
+  if (NS_WARN_IF(!thread))
+    return NS_ERROR_UNEXPECTED;
 
   bool killThread = false;
   {
@@ -259,12 +229,14 @@ nsThreadPool::Dispatch(nsIRunnable *event, uint32_t flags)
 {
   LOG(("THRD-P(%p) dispatch [%p %x]\n", this, event, flags));
 
-  NS_ENSURE_STATE(!mShutdown);
+  if (NS_WARN_IF(mShutdown))
+    return NS_ERROR_NOT_AVAILABLE;
 
   if (flags & DISPATCH_SYNC) {
     nsCOMPtr<nsIThread> thread;
     nsThreadManager::get()->GetCurrentThread(getter_AddRefs(thread));
-    NS_ENSURE_STATE(thread);
+    if (NS_WARN_IF(!thread))
+      return NS_ERROR_NOT_AVAILABLE;
 
     nsRefPtr<nsThreadSyncDispatch> wrapper =
         new nsThreadSyncDispatch(thread, event);
@@ -282,10 +254,14 @@ nsThreadPool::Dispatch(nsIRunnable *event, uint32_t flags)
 NS_IMETHODIMP
 nsThreadPool::IsOnCurrentThread(bool *result)
 {
-  // No one should be calling this method.  If this assertion gets hit, then we
-  // need to think carefully about what this method should be returning.
-  NS_NOTREACHED("implement me");
-
+  ReentrantMonitorAutoEnter mon(mEvents.GetReentrantMonitor());
+  nsIThread* thread = NS_GetCurrentThread();
+  for (uint32_t i = 0; i < static_cast<uint32_t>(mThreads.Count()); ++i) {
+    if (mThreads[i] == thread) {
+      *result = true;
+      return NS_OK;
+    }
+  }
   *result = false;
   return NS_OK;
 }
@@ -382,6 +358,22 @@ nsThreadPool::SetIdleThreadTimeout(uint32_t value)
   return NS_OK;
 }
 
+NS_IMETHODIMP
+nsThreadPool::GetThreadStackSize(uint32_t* value)
+{
+  ReentrantMonitorAutoEnter mon(mEvents.GetReentrantMonitor());
+  *value = mStackSize;
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsThreadPool::SetThreadStackSize(uint32_t value)
+{
+  ReentrantMonitorAutoEnter mon(mEvents.GetReentrantMonitor());
+  mStackSize = value;
+  return NS_OK;
+}
+
 NS_IMETHODIMP
 nsThreadPool::GetListener(nsIThreadPoolListener** aListener)
 {
@@ -413,17 +405,3 @@ nsThreadPool::SetName(const nsACString& aName)
   mName = aName;
   return NS_OK;
 }
-
-NS_IMETHODIMP
-ShutdownHelper::Run()
-{
-  MOZ_ASSERT(!mThreads.IsEmpty());
-
-  for (int32_t i = 0; i < mThreads.Count(); ++i)
-    mThreads[i]->Shutdown();
-
-  mThreads.Clear();
-
-  mListener = nullptr;
-  return NS_OK;
-}
